(function initHammerheadClient () {
    (function () {

	var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

	function createCommonjsModule(fn) {
	  var module = { exports: {} };
		return fn(module, module.exports), module.exports;
	}

	var PENDING = 'pending';
	var SETTLED = 'settled';
	var FULFILLED = 'fulfilled';
	var REJECTED = 'rejected';
	var NOOP = function () {};
	var isNode = typeof commonjsGlobal !== 'undefined' && typeof commonjsGlobal.process !== 'undefined' && typeof commonjsGlobal.process.emit === 'function';

	var asyncSetTimer = typeof setImmediate === 'undefined' ? setTimeout : setImmediate;
	var asyncQueue = [];
	var asyncTimer;

	function asyncFlush() {
		// run promise callbacks
		for (var i = 0; i < asyncQueue.length; i++) {
			asyncQueue[i][0](asyncQueue[i][1]);
		}

		// reset async asyncQueue
		asyncQueue = [];
		asyncTimer = false;
	}

	function asyncCall(callback, arg) {
		asyncQueue.push([callback, arg]);

		if (!asyncTimer) {
			asyncTimer = true;
			asyncSetTimer(asyncFlush, 0);
		}
	}

	function invokeResolver(resolver, promise) {
		function resolvePromise(value) {
			resolve(promise, value);
		}

		function rejectPromise(reason) {
			reject(promise, reason);
		}

		try {
			resolver(resolvePromise, rejectPromise);
		} catch (e) {
			rejectPromise(e);
		}
	}

	function invokeCallback(subscriber) {
		var owner = subscriber.owner;
		var settled = owner._state;
		var value = owner._data;
		var callback = subscriber[settled];
		var promise = subscriber.then;

		if (typeof callback === 'function') {
			settled = FULFILLED;
			try {
				value = callback(value);
			} catch (e) {
				reject(promise, e);
			}
		}

		if (!handleThenable(promise, value)) {
			if (settled === FULFILLED) {
				resolve(promise, value);
			}

			if (settled === REJECTED) {
				reject(promise, value);
			}
		}
	}

	function handleThenable(promise, value) {
		var resolved;

		try {
			if (promise === value) {
				throw new TypeError('A promises callback cannot return that same promise.');
			}

			if (value && (typeof value === 'function' || typeof value === 'object')) {
				// then should be retrieved only once
				var then = value.then;

				if (typeof then === 'function') {
					then.call(value, function (val) {
						if (!resolved) {
							resolved = true;

							if (value === val) {
								fulfill(promise, val);
							} else {
								resolve(promise, val);
							}
						}
					}, function (reason) {
						if (!resolved) {
							resolved = true;

							reject(promise, reason);
						}
					});

					return true;
				}
			}
		} catch (e) {
			if (!resolved) {
				reject(promise, e);
			}

			return true;
		}

		return false;
	}

	function resolve(promise, value) {
		if (promise === value || !handleThenable(promise, value)) {
			fulfill(promise, value);
		}
	}

	function fulfill(promise, value) {
		if (promise._state === PENDING) {
			promise._state = SETTLED;
			promise._data = value;

			asyncCall(publishFulfillment, promise);
		}
	}

	function reject(promise, reason) {
		if (promise._state === PENDING) {
			promise._state = SETTLED;
			promise._data = reason;

			asyncCall(publishRejection, promise);
		}
	}

	function publish(promise) {
		promise._then = promise._then.forEach(invokeCallback);
	}

	function publishFulfillment(promise) {
		promise._state = FULFILLED;
		publish(promise);
	}

	function publishRejection(promise) {
		promise._state = REJECTED;
		publish(promise);
		if (!promise._handled && isNode) {
			commonjsGlobal.process.emit('unhandledRejection', promise._data, promise);
		}
	}

	function notifyRejectionHandled(promise) {
		commonjsGlobal.process.emit('rejectionHandled', promise);
	}

	/**
	 * @class
	 */
	function Promise$1(resolver) {
		if (typeof resolver !== 'function') {
			throw new TypeError('Promise resolver ' + resolver + ' is not a function');
		}

		if (this instanceof Promise$1 === false) {
			throw new TypeError('Failed to construct \'Promise\': Please use the \'new\' operator, this object constructor cannot be called as a function.');
		}

		this._then = [];

		invokeResolver(resolver, this);
	}

	Promise$1.prototype = {
		constructor: Promise$1,

		_state: PENDING,
		_then: null,
		_data: undefined,
		_handled: false,

		then: function (onFulfillment, onRejection) {
			var subscriber = {
				owner: this,
				then: new this.constructor(NOOP),
				fulfilled: onFulfillment,
				rejected: onRejection
			};

			if ((onRejection || onFulfillment) && !this._handled) {
				this._handled = true;
				if (this._state === REJECTED && isNode) {
					asyncCall(notifyRejectionHandled, this);
				}
			}

			if (this._state === FULFILLED || this._state === REJECTED) {
				// already resolved, call callback async
				asyncCall(invokeCallback, subscriber);
			} else {
				// subscribe
				this._then.push(subscriber);
			}

			return subscriber.then;
		},

		catch: function (onRejection) {
			return this.then(null, onRejection);
		}
	};

	Promise$1.all = function (promises) {
		if (!Array.isArray(promises)) {
			throw new TypeError('You must pass an array to Promise.all().');
		}

		return new Promise$1(function (resolve, reject) {
			var results = [];
			var remaining = 0;

			function resolver(index) {
				remaining++;
				return function (value) {
					results[index] = value;
					if (!--remaining) {
						resolve(results);
					}
				};
			}

			for (var i = 0, promise; i < promises.length; i++) {
				promise = promises[i];

				if (promise && typeof promise.then === 'function') {
					promise.then(resolver(i), reject);
				} else {
					results[i] = promise;
				}
			}

			if (!remaining) {
				resolve(results);
			}
		});
	};

	Promise$1.race = function (promises) {
		if (!Array.isArray(promises)) {
			throw new TypeError('You must pass an array to Promise.race().');
		}

		return new Promise$1(function (resolve, reject) {
			for (var i = 0, promise; i < promises.length; i++) {
				promise = promises[i];

				if (promise && typeof promise.then === 'function') {
					promise.then(resolve, reject);
				} else {
					resolve(promise);
				}
			}
		});
	};

	Promise$1.resolve = function (value) {
		if (value && typeof value === 'object' && value.constructor === Promise$1) {
			return value;
		}

		return new Promise$1(function (resolve) {
			resolve(value);
		});
	};

	Promise$1.reject = function (reason) {
		return new Promise$1(function (resolve, reject) {
			reject(reason);
		});
	};

	var pinkie = Promise$1;

	/*! *****************************************************************************
	Copyright (c) Microsoft Corporation.

	Permission to use, copy, modify, and/or distribute this software for any
	purpose with or without fee is hereby granted.

	THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
	REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
	INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
	LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
	OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
	PERFORMANCE OF THIS SOFTWARE.
	***************************************************************************** */
	/* global Reflect, Promise */

	var extendStatics = function(d, b) {
	    extendStatics = Object.setPrototypeOf ||
	        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
	        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
	    return extendStatics(d, b);
	};

	function __extends(d, b) {
	    extendStatics(d, b);
	    function __() { this.constructor = d; }
	    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
	}

	function __spreadArrays() {
	    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
	    for (var r = Array(s), k = 0, i = 0; i < il; i++)
	        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
	            r[k] = a[j];
	    return r;
	}

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	var INTERNAL_PROPS = {
	    processDomMethodName: 'hammerhead|process-dom-method',
	    processedContext: 'hammerhead|processed-context',
	    documentWasCleaned: 'hammerhead|document-was-cleaned',
	    documentCharset: 'hammerhead|document-charset',
	    iframeNativeMethods: 'hammerhead|iframe-native-methods',
	    hammerhead: '%hammerhead%',
	    selection: 'hammerhead|selection',
	    shadowUIElement: 'hammerhead|shadow-ui-element',
	    forceProxySrcForImage: 'hammerhead|image|force-proxy-src-flag',
	    skipNextLoadEventForImage: 'hammerhead|image|skip-next-load-event-flag',
	    cachedImage: 'hammerhead|image|cached-image',
	    sandboxIsReattached: 'hammerhead|sandbox-is-reattached',
	    nativeStrRepresentation: 'hammerhead|native-string-representation'
	};

	var isInWorker = typeof window === 'undefined' && typeof self === 'object';
	var global$1 = (isInWorker ? self : window);
	var globalContextInfo = {
	    isInWorker: isInWorker,
	    global: global$1,
	    isServiceWorker: isInWorker && !global$1.XMLHttpRequest
	};

	function replaceNativeAccessor(descriptor, accessorName, newAccessor) {
	    if (newAccessor && descriptor[accessorName]) {
	        var stringifiedNativeAccessor_1 = descriptor[accessorName].toString();
	        newAccessor.toString = function () { return stringifiedNativeAccessor_1; };
	    }
	    descriptor[accessorName] = newAccessor;
	}
	function createOverriddenDescriptor(obj, prop, _a) {
	    var getter = _a.getter, setter = _a.setter, value = _a.value;
	    var descriptor = nativeMethods.objectGetOwnPropertyDescriptor(obj, prop);
	    if ((getter || setter) && value)
	        throw new Error('Cannot both specify accessors and a value or writable attribute.');
	    if (value) {
	        if (!nativeMethods.objectHasOwnProperty.call(descriptor, 'writable')) {
	            descriptor.writable = !!descriptor.set;
	            delete descriptor.get;
	            delete descriptor.set;
	        }
	        descriptor.value = value; // eslint-disable-line no-restricted-properties
	    }
	    else {
	        if (nativeMethods.objectHasOwnProperty.call(descriptor, 'writable')) {
	            delete descriptor.value; // eslint-disable-line no-restricted-properties
	            delete descriptor.writable;
	        }
	        if (getter !== null)
	            replaceNativeAccessor(descriptor, 'get', getter);
	        if (setter !== null)
	            replaceNativeAccessor(descriptor, 'set', setter);
	    }
	    return descriptor;
	}
	function overrideDescriptor(obj, prop, propertyAccessors) {
	    var descriptor = createOverriddenDescriptor(obj, prop, propertyAccessors);
	    nativeMethods.objectDefineProperty(obj, prop, descriptor);
	}
	function overrideFunctionName(fn, name) {
	    var nameDescriptor = nativeMethods.objectGetOwnPropertyDescriptor(fn, 'name');
	    if (!nameDescriptor)
	        return;
	    nameDescriptor.value = name; // eslint-disable-line no-restricted-properties
	    nativeMethods.objectDefineProperty(fn, 'name', nameDescriptor);
	}
	function overrideToString(nativeFnWrapper, nativeFn) {
	    nativeMethods.objectDefineProperty(nativeFnWrapper, INTERNAL_PROPS.nativeStrRepresentation, {
	        value: nativeMethods.Function.prototype.toString.call(nativeFn),
	        configurable: true
	    });
	}
	// TODO: this function should not be used outside this file
	// for now it's used to flag cases in which we assign our wrapper to a native function when it is missing
	function overrideStringRepresentation(nativeFnWrapper, nativeFn) {
	    overrideFunctionName(nativeFnWrapper, nativeFn.name);
	    overrideToString(nativeFnWrapper, nativeFn);
	}
	function isNativeFunction(fn) {
	    return !nativeMethods.objectHasOwnProperty.call(fn, INTERNAL_PROPS.nativeStrRepresentation);
	}
	function overrideFunction(obj, fnName, wrapper) {
	    var fn = obj[fnName];
	    if (isNativeFunction(fn)) {
	        overrideStringRepresentation(wrapper, fn);
	        obj[fnName] = wrapper;
	    }
	}
	function overrideConstructor(obj, fnName, wrapper, overrideProtoConstructor) {
	    if (overrideProtoConstructor === void 0) { overrideProtoConstructor = false; }
	    var nativePrototype = obj[fnName]['prototype'];
	    overrideFunction(obj, fnName, wrapper);
	    // NOTE: restore native prototype (to make `instanceof` work as expected)
	    obj[fnName]['prototype'] = nativePrototype;
	    // NOTE: we need to override the `constructor` property of a prototype
	    // because sometimes native constructor can be retrieved from it
	    if (overrideProtoConstructor)
	        obj[fnName]['prototype']['constructor'] = wrapper;
	}

	var overridingUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		createOverriddenDescriptor: createOverriddenDescriptor,
		overrideDescriptor: overrideDescriptor,
		overrideStringRepresentation: overrideStringRepresentation,
		isNativeFunction: isNativeFunction,
		overrideFunction: overrideFunction,
		overrideConstructor: overrideConstructor
	});

	/*global Document, Window */
	var NATIVE_CODE_RE = /\[native code]/;
	var NativeMethods = /** @class */ (function () {
	    function NativeMethods(doc, win) {
	        win = win || globalContextInfo.global;
	        this.refreshWindowMeths(win, globalContextInfo.isInWorker);
	        if (globalContextInfo.isInWorker)
	            return;
	        this.refreshDocumentMeths(doc, win);
	        this.refreshElementMeths(doc, win);
	    }
	    NativeMethods._getDocumentPropOwnerName = function (docPrototype, propName) {
	        return docPrototype.hasOwnProperty(propName) ? 'Document' : 'HTMLDocument';
	    };
	    NativeMethods.prototype.getStoragesPropsOwner = function (win) {
	        return this.isStoragePropsLocatedInProto ? win.Window.prototype : win;
	    };
	    NativeMethods.prototype.refreshDocumentMeths = function (doc, win) {
	        doc = doc || document;
	        win = win || window;
	        var docPrototype = win.Document.prototype;
	        // Dom
	        this.createDocumentFragment = docPrototype.createDocumentFragment;
	        this.createElement = docPrototype.createElement;
	        this.createElementNS = docPrototype.createElementNS;
	        this.createTextNode = docPrototype.createTextNode;
	        this.documentOpenPropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'open');
	        this.documentClosePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'close');
	        this.documentWritePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'write');
	        this.documentWriteLnPropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'writeln');
	        this.documentOpen = win[this.documentOpenPropOwnerName].prototype.open;
	        this.documentClose = win[this.documentClosePropOwnerName].prototype.close;
	        this.documentWrite = win[this.documentWritePropOwnerName].prototype.write;
	        this.documentWriteLn = win[this.documentWriteLnPropOwnerName].prototype.writeln;
	        this.elementFromPoint = docPrototype.elementFromPoint;
	        this.caretRangeFromPoint = docPrototype.caretRangeFromPoint;
	        this.caretPositionFromPoint = docPrototype.caretPositionFromPoint;
	        this.getElementById = docPrototype.getElementById;
	        this.getElementsByClassName = docPrototype.getElementsByClassName;
	        this.getElementsByName = docPrototype.getElementsByName;
	        this.getElementsByTagName = docPrototype.getElementsByTagName;
	        this.querySelector = docPrototype.querySelector;
	        this.querySelectorAll = docPrototype.querySelectorAll;
	        this.createHTMLDocument = win.DOMImplementation.prototype.createHTMLDocument;
	        // @ts-ignore
	        if (doc.registerElement) {
	            // @ts-ignore
	            this.registerElement = docPrototype.registerElement;
	        }
	        // Event
	        // NOTE: IE11 has no EventTarget so we should save "Event" methods separately
	        if (!win.EventTarget) {
	            this.documentAddEventListener = docPrototype.addEventListener;
	            this.documentRemoveEventListener = docPrototype.removeEventListener;
	        }
	        this.documentCreateEvent = docPrototype.createEvent;
	        // @ts-ignore Deprecated
	        this.documentCreateTouch = docPrototype.createTouch;
	        // @ts-ignore Deprecated
	        this.documentCreateTouchList = docPrototype.createTouchList;
	        // getters/setters
	        this.documentCookiePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'cookie');
	        var documentCookieDescriptor = win.Object.getOwnPropertyDescriptor(win[this.documentCookiePropOwnerName].prototype, 'cookie');
	        // TODO: remove this condition after the GH-1649 fix
	        if (!this.isNativeCode(documentCookieDescriptor.get) ||
	            !this.isNativeCode(documentCookieDescriptor.get.toString)) {
	            try {
	                var parentNativeMethods = win.parent['%hammerhead%'].nativeMethods;
	                documentCookieDescriptor.get = parentNativeMethods.documentCookieGetter;
	                documentCookieDescriptor.set = parentNativeMethods.documentCookieSetter;
	            }
	            catch (_a) { } // eslint-disable-line no-empty
	        }
	        this.documentReferrerGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'referrer').get;
	        this.documentStyleSheetsGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'styleSheets').get;
	        this.documentActiveElementGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'activeElement').get;
	        this.documentCookieGetter = documentCookieDescriptor.get;
	        this.documentCookieSetter = documentCookieDescriptor.set;
	        var documentDocumentURIDescriptor = win.Object.getOwnPropertyDescriptor(docPrototype, 'documentURI');
	        if (documentDocumentURIDescriptor)
	            this.documentDocumentURIGetter = documentDocumentURIDescriptor.get;
	        var documentTitleDescriptor = win.Object.getOwnPropertyDescriptor(docPrototype, 'title');
	        this.documentTitleGetter = documentTitleDescriptor.get;
	        this.documentTitleSetter = documentTitleDescriptor.set;
	    };
	    NativeMethods.prototype.refreshElementMeths = function (doc, win) {
	        var _this = this;
	        win = win || window;
	        var createElement = (function (tagName) { return _this.createElement.call(doc || document, tagName); });
	        var nativeElement = createElement('div');
	        var createTextNode = function (data) { return _this.createTextNode.call(doc || document, data); };
	        var textNode = createTextNode('text');
	        // Dom
	        this.appendChild = win.Node.prototype.appendChild;
	        this.append = win.Element.prototype.append;
	        this.prepend = win.Element.prototype.prepend;
	        this.after = win.Element.prototype.after;
	        this.attachShadow = win.Element.prototype.attachShadow;
	        this.replaceChild = nativeElement.replaceChild;
	        this.cloneNode = nativeElement.cloneNode;
	        this.elementGetElementsByClassName = nativeElement.getElementsByClassName;
	        this.elementGetElementsByTagName = nativeElement.getElementsByTagName;
	        this.elementQuerySelector = nativeElement.querySelector;
	        this.elementQuerySelectorAll = nativeElement.querySelectorAll;
	        this.getAttribute = nativeElement.getAttribute;
	        this.getAttributeNS = nativeElement.getAttributeNS;
	        this.insertBefore = nativeElement.insertBefore;
	        this.insertCell = createElement('tr').insertCell;
	        this.insertTableRow = createElement('table').insertRow;
	        this.insertTBodyRow = createElement('tbody').insertRow;
	        this.removeAttribute = nativeElement.removeAttribute;
	        this.removeAttributeNS = nativeElement.removeAttributeNS;
	        this.removeChild = win.Node.prototype.removeChild;
	        this.remove = win.Element.prototype.remove;
	        this.elementReplaceWith = win.Element.prototype.replaceWith;
	        this.setAttribute = nativeElement.setAttribute;
	        this.setAttributeNS = nativeElement.setAttributeNS;
	        this.hasAttribute = nativeElement.hasAttribute;
	        this.hasAttributeNS = nativeElement.hasAttributeNS;
	        this.hasAttributes = nativeElement.hasAttributes;
	        this.anchorToString = win.HTMLAnchorElement.prototype.toString;
	        this.matches = nativeElement.matches || nativeElement.msMatchesSelector;
	        this.closest = nativeElement.closest;
	        // NOTE: The 'insertAdjacent...' methods is located in HTMLElement prototype in IE11 only
	        this.insertAdjacentMethodsOwner = win.Element.prototype.hasOwnProperty('insertAdjacentElement')
	            ? win.Element.prototype
	            : win.HTMLElement.prototype;
	        this.insertAdjacentElement = this.insertAdjacentMethodsOwner.insertAdjacentElement;
	        this.insertAdjacentHTML = this.insertAdjacentMethodsOwner.insertAdjacentHTML;
	        this.insertAdjacentText = this.insertAdjacentMethodsOwner.insertAdjacentText;
	        // Text node
	        this.appendData = textNode.appendData;
	        // TODO: remove this condition after the GH-1649 fix
	        if (!this.isNativeCode(this.elementGetElementsByTagName)) {
	            try {
	                var parentNativeMethods = win.parent['%hammerhead%'].nativeMethods;
	                this.elementGetElementsByTagName = parentNativeMethods.elementGetElementsByTagName;
	            }
	            // eslint-disable-next-line no-empty
	            catch (e) {
	            }
	        }
	        // Event
	        if (win.EventTarget) {
	            this.addEventListener = win.EventTarget.prototype.addEventListener;
	            this.removeEventListener = win.EventTarget.prototype.removeEventListener;
	            this.dispatchEvent = win.EventTarget.prototype.dispatchEvent;
	        }
	        // NOTE: IE11 has no EventTarget
	        else {
	            this.addEventListener = nativeElement.addEventListener;
	            this.removeEventListener = nativeElement.removeEventListener;
	            this.dispatchEvent = nativeElement.dispatchEvent;
	        }
	        this.blur = nativeElement.blur;
	        this.click = nativeElement.click;
	        this.focus = nativeElement.focus;
	        // @ts-ignore
	        this.select = window.TextRange ? createElement('body').createTextRange().select : null;
	        this.setSelectionRange = createElement('input').setSelectionRange;
	        this.textAreaSetSelectionRange = createElement('textarea').setSelectionRange;
	        this.svgFocus = win.SVGElement ? win.SVGElement.prototype.focus : this.focus;
	        this.svgBlur = win.SVGElement ? win.SVGElement.prototype.blur : this.blur;
	        // Style
	        // NOTE: The 'style' descriptor is located in the Element.prototype in the Safari on IOS
	        this.htmlElementStylePropOwnerName = win.Element.prototype.hasOwnProperty('style') ? 'Element' : 'HTMLElement';
	        var htmlElementStyleDescriptor = win.Object.getOwnPropertyDescriptor(win[this.htmlElementStylePropOwnerName].prototype, 'style');
	        this.htmlElementStyleGetter = htmlElementStyleDescriptor.get;
	        // NOTE: IE does not allow to set a style property
	        if (htmlElementStyleDescriptor.set)
	            this.htmlElementStyleSetter = htmlElementStyleDescriptor.set;
	        var styleCssTextDescriptor = win.Object.getOwnPropertyDescriptor(win.CSSStyleDeclaration.prototype, 'cssText');
	        this.styleCssTextGetter = styleCssTextDescriptor.get;
	        this.styleCssTextSetter = styleCssTextDescriptor.set;
	    };
	    NativeMethods.prototype._refreshGettersAndSetters = function (win, isInWorker) {
	        if (isInWorker === void 0) { isInWorker = false; }
	        win = win || window;
	        var winProto = win.constructor.prototype;
	        // NOTE: Event properties is located in window prototype only in IE11
	        this.isEventPropsLocatedInProto = winProto.hasOwnProperty('onerror');
	        var eventPropsOwner = this.isEventPropsLocatedInProto ? winProto : win;
	        var winOnBeforeUnloadDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onbeforeunload');
	        var winOnUnloadDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onunload');
	        var winOnPageHideDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onpagehide');
	        var winOnMessageDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onmessage');
	        var winOnErrorDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onerror');
	        var winOnHashChangeDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onhashchange');
	        this.winOnBeforeUnloadSetter = winOnBeforeUnloadDescriptor && winOnBeforeUnloadDescriptor.set;
	        this.winOnUnloadSetter = winOnUnloadDescriptor && winOnUnloadDescriptor.set;
	        this.winOnPageHideSetter = winOnPageHideDescriptor && winOnPageHideDescriptor.set;
	        this.winOnMessageSetter = winOnMessageDescriptor && winOnMessageDescriptor.set;
	        this.winOnErrorSetter = winOnErrorDescriptor && winOnErrorDescriptor.set;
	        this.winOnHashChangeSetter = winOnHashChangeDescriptor && winOnHashChangeDescriptor.set;
	        var winOnUnhandledRejectionDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onunhandledrejection');
	        if (winOnUnhandledRejectionDescriptor)
	            this.winOnUnhandledRejectionSetter = winOnUnhandledRejectionDescriptor.set;
	        // Getters
	        if (win.WebSocket) {
	            var urlPropDescriptor = win.Object.getOwnPropertyDescriptor(win.WebSocket.prototype, 'url');
	            if (urlPropDescriptor && urlPropDescriptor.get && urlPropDescriptor.configurable)
	                this.webSocketUrlGetter = urlPropDescriptor.get;
	        }
	        this.messageEventOriginGetter = win.Object.getOwnPropertyDescriptor(win.MessageEvent.prototype, 'origin').get;
	        // NOTE: At present we proxy only the PerformanceNavigationTiming.
	        // Another types of the PerformanceEntry will be fixed later
	        // https://developer.mozilla.org/en-US/docs/Web/API/PerformanceEntry
	        if (win.PerformanceNavigationTiming)
	            this.performanceEntryNameGetter = win.Object.getOwnPropertyDescriptor(win.PerformanceEntry.prototype, 'name').get;
	        var dataPropDescriptor = win.Object.getOwnPropertyDescriptor(win.MessageEvent.prototype, 'data');
	        // NOTE: This condition is used for the Android 6.0 browser
	        if (dataPropDescriptor)
	            this.messageEventDataGetter = dataPropDescriptor.get;
	        if (win.fetch) {
	            this.responseStatusGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'status').get;
	            this.responseTypeGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'type').get;
	            this.responseUrlGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'url').get;
	            this.requestUrlGetter = win.Object.getOwnPropertyDescriptor(win.Request.prototype, 'url').get;
	            this.requestReferrerGetter = win.Object.getOwnPropertyDescriptor(win.Request.prototype, 'referrer').get;
	        }
	        if (win.XMLHttpRequest) {
	            var xhrResponseURLDescriptor = win.Object.getOwnPropertyDescriptor(win.XMLHttpRequest.prototype, 'responseURL');
	            // NOTE: IE doesn't support the 'responseURL' property
	            if (xhrResponseURLDescriptor)
	                this.xhrResponseURLGetter = xhrResponseURLDescriptor.get;
	        }
	        // eslint-disable-next-line no-restricted-properties
	        if (win.Window) {
	            // NOTE: The 'localStorage' and 'sessionStorage' properties is located in window prototype only in IE11
	            this.isStoragePropsLocatedInProto = win.Window.prototype.hasOwnProperty('localStorage');
	            var storagesPropsOwner = this.getStoragesPropsOwner(win);
	            this.winLocalStorageGetter = win.Object.getOwnPropertyDescriptor(storagesPropsOwner, 'localStorage').get;
	            this.winSessionStorageGetter = win.Object.getOwnPropertyDescriptor(storagesPropsOwner, 'sessionStorage').get;
	        }
	        if (isInWorker)
	            return;
	        this.storageGetItem = win.Storage.prototype.getItem;
	        this.storageSetItem = win.Storage.prototype.setItem;
	        this.storageRemoveItem = win.Storage.prototype.removeItem;
	        this.storageClear = win.Storage.prototype.clear;
	        this.storageKey = win.Storage.prototype.key;
	        this.storageLengthGetter = win.Object.getOwnPropertyDescriptor(win.Storage.prototype, 'length');
	        var objectDataDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLObjectElement.prototype, 'data');
	        var inputTypeDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'type');
	        var inputValueDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'value');
	        var inputDisabledDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'disabled');
	        var inputRequiredDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'required');
	        var textAreaValueDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLTextAreaElement.prototype, 'value');
	        var imageSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLImageElement.prototype, 'src');
	        var scriptSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'src');
	        var scriptIntegrityDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'integrity');
	        var embedSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLEmbedElement.prototype, 'src');
	        var sourceSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLSourceElement.prototype, 'src');
	        var mediaSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLMediaElement.prototype, 'src');
	        var inputSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'src');
	        var frameSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFrameElement.prototype, 'src');
	        var iframeSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'src');
	        var anchorHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'href');
	        var linkHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'href');
	        var linkIntegrityDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'integrity');
	        var linkRelDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'rel');
	        var linkAsDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'as');
	        var areaHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAreaElement.prototype, 'href');
	        var baseHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLBaseElement.prototype, 'href');
	        var anchorHostDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'host');
	        var anchorHostnameDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'hostname');
	        var anchorPathnameDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'pathname');
	        var anchorPortDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'port');
	        var anchorProtocolDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'protocol');
	        var anchorSearchDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'search');
	        var anchorTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'target');
	        var formTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFormElement.prototype, 'target');
	        var areaTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAreaElement.prototype, 'target');
	        var baseTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLBaseElement.prototype, 'target');
	        var inputFormTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'formTarget');
	        var buttonFormTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLButtonElement.prototype, 'formTarget');
	        var svgImageHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGImageElement.prototype, 'href');
	        var svgAnimStrAnimValDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGAnimatedString.prototype, 'animVal');
	        var svgAnimStrBaseValDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGAnimatedString.prototype, 'baseVal');
	        var inputAutocompleteDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'autocomplete');
	        var formActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFormElement.prototype, 'action');
	        var inputFormActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'formAction');
	        var buttonFormActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLButtonElement.prototype, 'formAction');
	        var nodeTextContentDescriptor = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'textContent');
	        var htmlElementInnerTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLElement.prototype, 'innerText');
	        var scriptTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'text');
	        var anchorTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'text');
	        var titleElementTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLTitleElement.prototype, 'text');
	        var iframeSandboxDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'sandbox');
	        var windowOriginDescriptor = win.Object.getOwnPropertyDescriptor(win, 'origin');
	        if (windowOriginDescriptor) {
	            this.windowOriginGetter = windowOriginDescriptor.get;
	            this.windowOriginSetter = windowOriginDescriptor.set;
	        }
	        // NOTE: We need 'disabled' property only for Chrome.
	        // In Chrome it's located in HTMLInputElement.prototype
	        // But in IE11 it's located in HTMLElement.prototype
	        // So we need the null check
	        if (inputDisabledDescriptor) {
	            this.inputDisabledSetter = inputDisabledDescriptor.set;
	            this.inputDisabledGetter = inputDisabledDescriptor.get;
	        }
	        // NOTE: Html properties is located in HTMLElement prototype in IE11 only
	        this.elementHTMLPropOwnerName = win.Element.prototype.hasOwnProperty('innerHTML') ? 'Element' : 'HTMLElement';
	        var elementInnerHTMLDescriptor = win.Object.getOwnPropertyDescriptor(win[this.elementHTMLPropOwnerName].prototype, 'innerHTML');
	        var elementOuterHTMLDescriptor = win.Object.getOwnPropertyDescriptor(win[this.elementHTMLPropOwnerName].prototype, 'outerHTML');
	        // Setters
	        this.objectDataSetter = objectDataDescriptor.set;
	        this.inputTypeSetter = inputTypeDescriptor.set;
	        this.inputValueSetter = inputValueDescriptor.set;
	        this.inputRequiredSetter = inputRequiredDescriptor.set;
	        this.textAreaValueSetter = textAreaValueDescriptor.set;
	        this.imageSrcSetter = imageSrcDescriptor.set;
	        this.scriptSrcSetter = scriptSrcDescriptor.set;
	        this.embedSrcSetter = embedSrcDescriptor.set;
	        this.sourceSrcSetter = sourceSrcDescriptor.set;
	        this.mediaSrcSetter = mediaSrcDescriptor.set;
	        this.inputSrcSetter = inputSrcDescriptor.set;
	        this.frameSrcSetter = frameSrcDescriptor.set;
	        this.iframeSrcSetter = iframeSrcDescriptor.set;
	        this.anchorHrefSetter = anchorHrefDescriptor.set;
	        this.linkHrefSetter = linkHrefDescriptor.set;
	        this.linkRelSetter = linkRelDescriptor.set;
	        this.linkAsSetter = linkAsDescriptor && linkAsDescriptor.set;
	        this.areaHrefSetter = areaHrefDescriptor.set;
	        this.baseHrefSetter = baseHrefDescriptor.set;
	        this.anchorHostSetter = anchorHostDescriptor.set;
	        this.anchorHostnameSetter = anchorHostnameDescriptor.set;
	        this.anchorPathnameSetter = anchorPathnameDescriptor.set;
	        this.anchorPortSetter = anchorPortDescriptor.set;
	        this.anchorProtocolSetter = anchorProtocolDescriptor.set;
	        this.anchorSearchSetter = anchorSearchDescriptor.set;
	        this.anchorTargetSetter = anchorTargetDescriptor.set;
	        this.formTargetSetter = formTargetDescriptor.set;
	        this.areaTargetSetter = areaTargetDescriptor.set;
	        this.baseTargetSetter = baseTargetDescriptor.set;
	        this.inputFormTargetSetter = inputFormTargetDescriptor.set;
	        this.buttonFormTargetSetter = buttonFormTargetDescriptor.set;
	        this.svgAnimStrBaseValSetter = svgAnimStrBaseValDescriptor.set;
	        this.inputAutocompleteSetter = inputAutocompleteDescriptor.set;
	        this.formActionSetter = formActionDescriptor.set;
	        this.inputFormActionSetter = inputFormActionDescriptor.set;
	        this.buttonFormActionSetter = buttonFormActionDescriptor.set;
	        this.iframeSandboxSetter = iframeSandboxDescriptor.set;
	        this.htmlElementOnloadSetter = win.Object.getOwnPropertyDescriptor(win.HTMLElement.prototype, 'onload').set;
	        this.nodeTextContentSetter = nodeTextContentDescriptor.set;
	        this.htmlElementInnerTextSetter = htmlElementInnerTextDescriptor.set;
	        this.scriptTextSetter = scriptTextDescriptor.set;
	        this.anchorTextSetter = anchorTextDescriptor.set;
	        this.elementInnerHTMLSetter = elementInnerHTMLDescriptor.set;
	        this.elementOuterHTMLSetter = elementOuterHTMLDescriptor.set;
	        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
	        if (scriptIntegrityDescriptor && linkIntegrityDescriptor) {
	            this.scriptIntegritySetter = scriptIntegrityDescriptor.set;
	            this.linkIntegritySetter = linkIntegrityDescriptor.set;
	        }
	        this.titleElementTextSetter = titleElementTextDescriptor.set;
	        // NOTE: the classList property is located in HTMLElement prototype in IE11
	        this.elementClassListPropOwnerName = win.Element.prototype.hasOwnProperty('classList') ? 'Element' : 'HTMLElement';
	        this.elementClassListGetter = win.Object.getOwnPropertyDescriptor(win[this.elementClassListPropOwnerName].prototype, 'classList').get;
	        this.htmlCollectionLengthGetter = win.Object.getOwnPropertyDescriptor(win.HTMLCollection.prototype, 'length').get;
	        this.nodeListLengthGetter = win.Object.getOwnPropertyDescriptor(win.NodeList.prototype, 'length').get;
	        this.elementChildElementCountGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'childElementCount').get;
	        this.inputFilesGetter = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'files').get;
	        this.styleSheetHrefGetter = win.Object.getOwnPropertyDescriptor(win.StyleSheet.prototype, 'href').get;
	        this.objectDataGetter = objectDataDescriptor.get;
	        this.inputTypeGetter = inputTypeDescriptor.get;
	        this.inputValueGetter = inputValueDescriptor.get;
	        this.inputRequiredGetter = inputRequiredDescriptor.get;
	        this.textAreaValueGetter = textAreaValueDescriptor.get;
	        this.imageSrcGetter = imageSrcDescriptor.get;
	        this.scriptSrcGetter = scriptSrcDescriptor.get;
	        this.embedSrcGetter = embedSrcDescriptor.get;
	        this.sourceSrcGetter = sourceSrcDescriptor.get;
	        this.mediaSrcGetter = mediaSrcDescriptor.get;
	        this.inputSrcGetter = inputSrcDescriptor.get;
	        this.frameSrcGetter = frameSrcDescriptor.get;
	        this.iframeSrcGetter = iframeSrcDescriptor.get;
	        this.anchorHrefGetter = anchorHrefDescriptor.get;
	        this.linkHrefGetter = linkHrefDescriptor.get;
	        this.linkRelGetter = linkRelDescriptor.get;
	        this.areaHrefGetter = areaHrefDescriptor.get;
	        this.baseHrefGetter = baseHrefDescriptor.get;
	        this.anchorHostGetter = anchorHostDescriptor.get;
	        this.anchorHostnameGetter = anchorHostnameDescriptor.get;
	        this.anchorPathnameGetter = anchorPathnameDescriptor.get;
	        this.anchorPortGetter = anchorPortDescriptor.get;
	        this.anchorProtocolGetter = anchorProtocolDescriptor.get;
	        this.anchorSearchGetter = anchorSearchDescriptor.get;
	        this.anchorTargetGetter = anchorTargetDescriptor.get;
	        this.formTargetGetter = formTargetDescriptor.get;
	        this.areaTargetGetter = areaTargetDescriptor.get;
	        this.baseTargetGetter = baseTargetDescriptor.get;
	        this.inputFormTargetGetter = inputFormTargetDescriptor.get;
	        this.buttonFormTargetGetter = buttonFormTargetDescriptor.get;
	        this.svgImageHrefGetter = svgImageHrefDescriptor.get;
	        this.svgAnimStrAnimValGetter = svgAnimStrAnimValDescriptor.get;
	        this.svgAnimStrBaseValGetter = svgAnimStrBaseValDescriptor.get;
	        this.inputAutocompleteGetter = inputAutocompleteDescriptor.get;
	        this.formActionGetter = formActionDescriptor.get;
	        this.inputFormActionGetter = inputFormActionDescriptor.get;
	        this.buttonFormActionGetter = buttonFormActionDescriptor.get;
	        this.iframeSandboxGetter = iframeSandboxDescriptor.get;
	        this.contentWindowGetter = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'contentWindow').get;
	        this.contentDocumentGetter = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'contentDocument').get;
	        this.frameContentWindowGetter = win.Object.getOwnPropertyDescriptor(win.HTMLFrameElement.prototype, 'contentWindow').get;
	        this.nodeTextContentGetter = nodeTextContentDescriptor.get;
	        this.htmlElementInnerTextGetter = htmlElementInnerTextDescriptor.get;
	        this.scriptTextGetter = scriptTextDescriptor.get;
	        this.anchorTextGetter = anchorTextDescriptor.get;
	        this.elementInnerHTMLGetter = elementInnerHTMLDescriptor.get;
	        this.elementOuterHTMLGetter = elementOuterHTMLDescriptor.get;
	        this.nodeFirstChildGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'firstChild').get;
	        this.nodeLastChildGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'lastChild').get;
	        this.nodeNextSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'nextSibling').get;
	        this.nodePrevSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'previousSibling').get;
	        this.nodeParentNodeGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'parentNode').get;
	        this.nodeChildNodesGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'childNodes').get;
	        this.elementFirstElementChildGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'firstElementChild').get;
	        this.elementLastElementChildGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'lastElementChild').get;
	        this.elementNextElementSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'nextElementSibling').get;
	        this.elementPrevElementSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'previousElementSibling').get;
	        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
	        if (scriptIntegrityDescriptor && linkIntegrityDescriptor) {
	            this.scriptIntegrityGetter = scriptIntegrityDescriptor.get;
	            this.linkIntegrityGetter = linkIntegrityDescriptor.get;
	        }
	        // NOTE: In the Internet Explorer 11 the children property is located in HTMLElement.
	        var childrenPropOwner = win.Element.prototype.hasOwnProperty('children')
	            ? win.Element.prototype
	            : win.HTMLElement.prototype;
	        this.elementChildrenGetter = win.Object.getOwnPropertyDescriptor(childrenPropOwner, 'children').get;
	        var anchorOriginDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'origin');
	        // NOTE: IE and Edge don't support origin property
	        if (anchorOriginDescriptor)
	            this.anchorOriginGetter = anchorOriginDescriptor.get;
	        var iframeSrcdocDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'srcdoc');
	        // NOTE: IE11 doesn't support the 'srcdoc' property
	        if (iframeSrcdocDescriptor) {
	            this.iframeSrcdocGetter = iframeSrcdocDescriptor.get;
	            this.iframeSrcdocSetter = iframeSrcdocDescriptor.set;
	        }
	        var cssStyleSheetHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.CSSStyleSheet.prototype, 'href');
	        // NOTE: IE11 doesn't support the 'href' property
	        if (cssStyleSheetHrefDescriptor)
	            this.cssStyleSheetHrefGetter = cssStyleSheetHrefDescriptor.get;
	        var nodeBaseURIDescriptor = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'baseURI');
	        // NOTE: IE11 doesn't support the 'baseURI' property
	        if (nodeBaseURIDescriptor)
	            this.nodeBaseURIGetter = nodeBaseURIDescriptor.get;
	        // NOTE: The 'attributes' property is located in Node prototype in IE11 only
	        this.elementAttributesPropOwnerName = win.Element.prototype.hasOwnProperty('attributes') ? 'Element' : 'Node';
	        this.elementAttributesGetter = win.Object.getOwnPropertyDescriptor(win[this.elementAttributesPropOwnerName].prototype, 'attributes').get;
	        var htmlManifestDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLHtmlElement.prototype, 'manifest');
	        // NOTE: Only the Safari browser supports the 'manifest' property
	        if (htmlManifestDescriptor) {
	            this.htmlManifestGetter = htmlManifestDescriptor.get;
	            this.htmlManifestSetter = htmlManifestDescriptor.set;
	        }
	        this.titleElementTextGetter = titleElementTextDescriptor.get;
	        // MutationRecord
	        this.mutationRecordNextSiblingGetter = win.Object.getOwnPropertyDescriptor(win.MutationRecord.prototype, 'nextSibling').get;
	        this.mutationRecordPrevSiblingGetter = win.Object.getOwnPropertyDescriptor(win.MutationRecord.prototype, 'previousSibling').get;
	    };
	    NativeMethods.prototype.refreshWindowMeths = function (win, isInWorker) {
	        if (isInWorker === void 0) { isInWorker = false; }
	        win = win || window;
	        var winProto = win.constructor.prototype;
	        // Dom
	        this.eval = win.eval;
	        this.formSubmit = win.HTMLFormElement && win.HTMLFormElement.prototype.submit;
	        this.documentFragmentQuerySelector = win.DocumentFragment && win.DocumentFragment.prototype.querySelector;
	        this.documentFragmentQuerySelectorAll = win.DocumentFragment && win.DocumentFragment.prototype.querySelectorAll;
	        this.preventDefault = win.Event.prototype.preventDefault;
	        this.historyPushState = win.history && win.history.pushState;
	        this.historyReplaceState = win.history && win.history.replaceState;
	        this.postMessage = win.postMessage || winProto.postMessage;
	        this.windowOpen = win.open || winProto.open;
	        this.setTimeout = win.setTimeout || winProto.setTimeout;
	        this.setInterval = win.setInterval || winProto.setInterval;
	        this.clearTimeout = win.clearTimeout || winProto.clearTimeout;
	        this.clearInterval = win.clearInterval || winProto.clearInterval;
	        this.registerProtocolHandler = win.navigator.registerProtocolHandler;
	        this.sendBeacon = win.navigator.sendBeacon;
	        if (win.XMLHttpRequest) {
	            // NOTE: IE11 has no EventTarget so we should save "Event" methods separately
	            var xhrEventProto = (win.EventTarget || win.XMLHttpRequest).prototype;
	            this.xhrAbort = win.XMLHttpRequest.prototype.abort;
	            this.xhrOpen = win.XMLHttpRequest.prototype.open;
	            this.xhrSend = win.XMLHttpRequest.prototype.send;
	            this.xhrAddEventListener = xhrEventProto.addEventListener;
	            this.xhrRemoveEventListener = xhrEventProto.removeEventListener;
	            this.xhrDispatchEvent = xhrEventProto.dispatchEvent;
	            this.xhrGetResponseHeader = win.XMLHttpRequest.prototype.getResponseHeader;
	            this.xhrGetAllResponseHeaders = win.XMLHttpRequest.prototype.getAllResponseHeaders;
	            this.xhrSetRequestHeader = win.XMLHttpRequest.prototype.setRequestHeader;
	            this.xhrOverrideMimeType = win.XMLHttpRequest.prototype.overrideMimeType;
	        }
	        try {
	            this.registerServiceWorker = win.navigator.serviceWorker.register;
	            this.getRegistrationServiceWorker = win.navigator.serviceWorker.getRegistration;
	        }
	        catch (e) {
	            this.registerServiceWorker = null;
	            this.getRegistrationServiceWorker = null;
	        }
	        this.createContextualFragment = win.Range && win.Range.prototype.createContextualFragment;
	        var nativePerformance = win.performance;
	        if (nativePerformance) {
	            // eslint-disable-next-line no-restricted-properties
	            var nativePerformanceNow_1 = win.performance.now || win.Performance.prototype.now;
	            this.performanceNow = function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return nativePerformanceNow_1.apply(nativePerformance, args);
	            };
	        }
	        // Fetch
	        this.fetch = win.fetch;
	        this.Request = win.Request;
	        if (win.Headers) {
	            this.Headers = win.Headers;
	            this.headersSet = win.Headers.prototype.set;
	            this.headersGet = win.Headers.prototype.get;
	            this.headersDelete = win.Headers.prototype.delete;
	            this.headersEntries = win.Headers.prototype.entries;
	            this.headersForEach = win.Headers.prototype.forEach;
	            this.headersValues = win.Headers.prototype.values;
	        }
	        // Event
	        this.windowAddEventListener = win.addEventListener || winProto.addEventListener;
	        this.windowRemoveEventListener = win.removeEventListener || winProto.removeEventListener;
	        this.windowDispatchEvent = win.dispatchEvent;
	        this.WindowPointerEvent = win.PointerEvent || winProto.PointerEvent;
	        this.WindowMSPointerEvent = win.MSPointerEvent || winProto.MSPointerEvent;
	        this.WindowTouch = win.Touch || winProto.Touch;
	        this.WindowTouchEvent = win.TouchEvent || winProto.TouchEvent;
	        this.WindowKeyboardEvent = win.KeyboardEvent || winProto.KeyboardEvent;
	        this.WindowFocusEvent = win.FocusEvent || winProto.FocusEvent;
	        this.WindowTextEvent = win.TextEvent || winProto.TextEvent;
	        this.WindowInputEvent = win.InputEvent || winProto.InputEvent;
	        this.WindowMouseEvent = win.MouseEvent || winProto.MouseEvent;
	        this.canvasContextDrawImage = win.CanvasRenderingContext2D && win.CanvasRenderingContext2D.prototype.drawImage;
	        // FormData
	        this.formDataAppend = win.FormData && win.FormData.prototype.append;
	        // DateTime
	        this.date = win.Date;
	        this.dateNow = win.Date.now; // eslint-disable-line no-restricted-properties
	        // Math
	        this.math = win.Math;
	        this.mathRandom = win.Math.random;
	        // Object
	        this.objectToString = win.Object.prototype.toString;
	        this.objectAssign = win.Object.assign;
	        this.objectKeys = win.Object.keys;
	        this.objectDefineProperty = win.Object.defineProperty;
	        this.objectDefineProperties = win.Object.defineProperties;
	        this.objectCreate = win.Object.create;
	        this.objectIsExtensible = win.Object.isExtensible;
	        this.objectIsFrozen = win.Object.isFrozen;
	        this.objectGetOwnPropertyDescriptor = win.Object.getOwnPropertyDescriptor;
	        this.objectHasOwnProperty = win.Object.hasOwnProperty;
	        this.objectGetOwnPropertyNames = win.Object.getOwnPropertyNames;
	        this.objectGetPrototypeOf = win.Object.getPrototypeOf;
	        this.objectGetOwnPropertySymbols = win.Object.getOwnPropertySymbols;
	        // Array
	        this.arraySlice = win.Array.prototype.slice;
	        this.arrayConcat = win.Array.prototype.concat;
	        this.arrayFilter = win.Array.prototype.filter;
	        this.arrayFind = win.Array.prototype.find;
	        this.arrayMap = win.Array.prototype.map;
	        this.arrayJoin = win.Array.prototype.join;
	        this.arraySplice = win.Array.prototype.splice;
	        this.arrayUnshift = win.Array.prototype.unshift;
	        this.arrayForEach = win.Array.prototype.forEach;
	        this.arrayIndexOf = win.Array.prototype.indexOf;
	        this.arraySome = win.Array.prototype.some;
	        this.arrayReverse = win.Array.prototype.reverse;
	        this.arrayReduce = win.Array.prototype.reduce;
	        this.arrayFrom = win.Array.from;
	        this.DOMParserParseFromString = win.DOMParser && win.DOMParser.prototype.parseFromString;
	        this.arrayBufferIsView = win.ArrayBuffer.prototype.constructor.isView;
	        // NOTE: this section relates to getting properties from DOM classes
	        if (!isInWorker) {
	            // DOMTokenList
	            this.tokenListAdd = win.DOMTokenList.prototype.add;
	            this.tokenListRemove = win.DOMTokenList.prototype.remove;
	            this.tokenListReplace = win.DOMTokenList.prototype.replace;
	            this.tokenListSupports = win.DOMTokenList.prototype.supports;
	            this.tokenListToggle = win.DOMTokenList.prototype.toggle;
	            this.tokenListContains = win.DOMTokenList.prototype.contains;
	            var tokenListValueDescriptor = win.Object.getOwnPropertyDescriptor(win.DOMTokenList.prototype, 'value');
	            // NOTE: IE11 doesn't support the 'value' property of the DOMTokenList interface
	            if (tokenListValueDescriptor)
	                this.tokenListValueSetter = tokenListValueDescriptor.set;
	            // Stylesheets
	            this.styleGetPropertyValue = win.CSSStyleDeclaration.prototype.getPropertyValue;
	            this.styleSetProperty = win.CSSStyleDeclaration.prototype.setProperty;
	            this.styleRemoveProperty = win.CSSStyleDeclaration.prototype.removeProperty;
	            this.styleInsertRule = win.CSSStyleSheet.prototype.insertRule;
	            this.scrollTo = win.scrollTo;
	        }
	        if (win.Promise) {
	            this.promiseThen = win.Promise.prototype.then;
	            this.promiseReject = win.Promise.reject;
	        }
	        // Console
	        this.console = win.console;
	        if (this.console) {
	            this.consoleMeths = {
	                log: win.console.log,
	                warn: win.console.warn,
	                error: win.console.error,
	                info: win.console.info
	            };
	        }
	        this.crypto = win.crypto || win.msCrypto;
	        this.cryptoGetRandomValues = this.crypto && this.crypto.getRandomValues;
	        this.refreshClasses(win);
	        this._refreshGettersAndSetters(win, isInWorker);
	    };
	    NativeMethods.prototype.refreshClasses = function (win) {
	        this.windowClass = win.Window;
	        this.documentClass = win.Document;
	        this.locationClass = win.Location;
	        this.elementClass = win.Element;
	        this.svgElementClass = win.SVGElement;
	        this.Worker = win.Worker;
	        this.MessageChannel = win.MessageChannel;
	        this.ArrayBuffer = win.ArrayBuffer;
	        this.Uint8Array = win.Uint8Array;
	        this.Uint16Array = win.Uint16Array;
	        this.Uint32Array = win.Uint32Array;
	        this.DataView = win.DataView;
	        this.Blob = win.Blob;
	        this.XMLHttpRequest = win.XMLHttpRequest;
	        this.Image = win.Image;
	        this.Function = win.Function;
	        this.functionToString = win.Function.prototype.toString;
	        this.Error = win.Error;
	        this.FontFace = win.FontFace;
	        this.StorageEvent = win.StorageEvent;
	        this.MutationObserver = win.MutationObserver;
	        this.EventSource = win.EventSource;
	        this.Proxy = win.Proxy;
	        this.WebSocket = win.WebSocket;
	        this.HTMLCollection = win.HTMLCollection;
	        this.NodeList = win.NodeList;
	        this.Node = win.Node;
	        this.URL = win.URL;
	        this.DataTransfer = win.DataTransfer;
	        this.DataTransferItemList = win.DataTransferItemList;
	        this.DataTransferItem = win.DataTransferItem;
	        this.FileList = win.FileList;
	        // NOTE: non-IE11 case. window.File in IE11 is not constructable.
	        if (win.File && typeof win.File === 'function')
	            this.File = win.File;
	    };
	    NativeMethods.prototype.refreshElectronMeths = function (vmModule) {
	        if (this.createScript && isNativeFunction(vmModule.createScript))
	            return false;
	        this.createScript = vmModule.createScript;
	        this.runInDebugContext = vmModule.runInDebugContext;
	        this.runInContext = vmModule.runInContext;
	        this.runInNewContext = vmModule.runInNewContext;
	        this.runInThisContext = vmModule.runInThisContext;
	        return true;
	    };
	    NativeMethods._ensureDocumentMethodRestore = function (document, prototype, methodName, savedNativeMethod) {
	        prototype[methodName] = savedNativeMethod;
	        if (document[methodName] !== prototype[methodName])
	            document[methodName] = savedNativeMethod;
	    };
	    NativeMethods.prototype.restoreDocumentMeths = function (window, document) {
	        var docPrototype = window.Document.prototype;
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createDocumentFragment', this.createDocumentFragment);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createElement', this.createElement);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createElementNS', this.createElementNS);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'elementFromPoint', this.elementFromPoint);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'caretRangeFromPoint', this.caretRangeFromPoint);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'caretPositionFromPoint', this.caretPositionFromPoint);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementById', this.getElementById);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByClassName', this.getElementsByClassName);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByName', this.getElementsByName);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByTagName', this.getElementsByTagName);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'querySelector', this.querySelector);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'querySelectorAll', this.querySelectorAll);
	        // Event
	        // NOTE: IE11 has no EventTarget
	        if (!window.EventTarget) {
	            NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'addEventListener', this.documentAddEventListener);
	            NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'removeEventListener', this.documentRemoveEventListener);
	        }
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createEvent', this.documentCreateEvent);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createTouch', this.documentCreateTouch);
	        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createTouchList', this.documentCreateTouchList);
	        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentOpenPropOwnerName].prototype, 'open', this.documentOpen);
	        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentClosePropOwnerName].prototype, 'close', this.documentClose);
	        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentWritePropOwnerName].prototype, 'write', this.documentWrite);
	        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentWriteLnPropOwnerName].prototype, 'writeln', this.documentWriteLn);
	    };
	    NativeMethods.prototype.refreshIfNecessary = function (doc, win) {
	        var _this = this;
	        var tryToExecuteCode = function (func) {
	            try {
	                return func();
	            }
	            catch (e) {
	                return true;
	            }
	        };
	        var needToRefreshDocumentMethods = tryToExecuteCode(function () { return !doc.createElement ||
	            isNativeFunction(document.createElement); });
	        var needToRefreshElementMethods = tryToExecuteCode(function () {
	            var nativeElement = _this.createElement.call(doc, 'div');
	            return isNativeFunction(nativeElement.getAttribute);
	        });
	        var needToRefreshWindowMethods = tryToExecuteCode(function () {
	            _this.setTimeout.call(win, function () { return void 0; }, 0);
	            return isNativeFunction(win.XMLHttpRequest.prototype.open);
	        });
	        // NOTE: T173709
	        if (needToRefreshDocumentMethods)
	            this.refreshDocumentMeths(doc, win);
	        if (needToRefreshElementMethods)
	            this.refreshElementMeths(doc, win);
	        // NOTE: T239109
	        if (needToRefreshWindowMethods)
	            this.refreshWindowMeths(win);
	    };
	    NativeMethods.prototype.isNativeCode = function (fn) {
	        return NATIVE_CODE_RE.test(fn.toString());
	    };
	    return NativeMethods;
	}());
	var nativeMethods = new NativeMethods();

	/*!
	 * Bowser - a browser detector
	 * https://github.com/ded/bowser
	 * MIT License | (c) Dustin Diaz 2015
	 */

	var bowser = createCommonjsModule(function (module) {
	!function (root, name, definition) {
	  if (module.exports) module.exports = definition();
	  else root[name] = definition();
	}(commonjsGlobal, 'bowser', function () {
	  /**
	    * See useragents.js for examples of navigator.userAgent
	    */

	  var t = true;

	  function detect(ua) {

	    function getFirstMatch(regex) {
	      var match = ua.match(regex);
	      return (match && match.length > 1 && match[1]) || '';
	    }

	    function getSecondMatch(regex) {
	      var match = ua.match(regex);
	      return (match && match.length > 1 && match[2]) || '';
	    }

	    var iosdevice = getFirstMatch(/(ipod|iphone|ipad)/i).toLowerCase()
	      , likeAndroid = /like android/i.test(ua)
	      , android = !likeAndroid && /android/i.test(ua)
	      , nexusMobile = /nexus\s*[0-6]\s*/i.test(ua)
	      , nexusTablet = !nexusMobile && /nexus\s*[0-9]+/i.test(ua)
	      , chromeos = /CrOS/.test(ua)
	      , silk = /silk/i.test(ua)
	      , sailfish = /sailfish/i.test(ua)
	      , tizen = /tizen/i.test(ua)
	      , webos = /(web|hpw)os/i.test(ua)
	      , windowsphone = /windows phone/i.test(ua)
	      ; /SamsungBrowser/i.test(ua)
	      ; var windows = !windowsphone && /windows/i.test(ua)
	      , mac = !iosdevice && !silk && /macintosh/i.test(ua)
	      , linux = !android && !sailfish && !tizen && !webos && /linux/i.test(ua)
	      , edgeVersion = getFirstMatch(/edge\/(\d+(\.\d+)?)/i)
	      , versionIdentifier = getFirstMatch(/version\/(\d+(\.\d+)?)/i)
	      , tablet = /tablet/i.test(ua)
	      , mobile = !tablet && /[^-]mobi/i.test(ua)
	      , xbox = /xbox/i.test(ua)
	      , result;

	    if (/opera/i.test(ua)) {
	      //  an old Opera
	      result = {
	        name: 'Opera'
	      , opera: t
	      , version: versionIdentifier || getFirstMatch(/(?:opera|opr|opios)[\s\/](\d+(\.\d+)?)/i)
	      };
	    } else if (/opr|opios/i.test(ua)) {
	      // a new Opera
	      result = {
	        name: 'Opera'
	        , opera: t
	        , version: getFirstMatch(/(?:opr|opios)[\s\/](\d+(\.\d+)?)/i) || versionIdentifier
	      };
	    }
	    else if (/SamsungBrowser/i.test(ua)) {
	      result = {
	        name: 'Samsung Internet for Android'
	        , samsungBrowser: t
	        , version: versionIdentifier || getFirstMatch(/(?:SamsungBrowser)[\s\/](\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/coast/i.test(ua)) {
	      result = {
	        name: 'Opera Coast'
	        , coast: t
	        , version: versionIdentifier || getFirstMatch(/(?:coast)[\s\/](\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/yabrowser/i.test(ua)) {
	      result = {
	        name: 'Yandex Browser'
	      , yandexbrowser: t
	      , version: versionIdentifier || getFirstMatch(/(?:yabrowser)[\s\/](\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/ucbrowser/i.test(ua)) {
	      result = {
	          name: 'UC Browser'
	        , ucbrowser: t
	        , version: getFirstMatch(/(?:ucbrowser)[\s\/](\d+(?:\.\d+)+)/i)
	      };
	    }
	    else if (/mxios/i.test(ua)) {
	      result = {
	        name: 'Maxthon'
	        , maxthon: t
	        , version: getFirstMatch(/(?:mxios)[\s\/](\d+(?:\.\d+)+)/i)
	      };
	    }
	    else if (/epiphany/i.test(ua)) {
	      result = {
	        name: 'Epiphany'
	        , epiphany: t
	        , version: getFirstMatch(/(?:epiphany)[\s\/](\d+(?:\.\d+)+)/i)
	      };
	    }
	    else if (/puffin/i.test(ua)) {
	      result = {
	        name: 'Puffin'
	        , puffin: t
	        , version: getFirstMatch(/(?:puffin)[\s\/](\d+(?:\.\d+)?)/i)
	      };
	    }
	    else if (/sleipnir/i.test(ua)) {
	      result = {
	        name: 'Sleipnir'
	        , sleipnir: t
	        , version: getFirstMatch(/(?:sleipnir)[\s\/](\d+(?:\.\d+)+)/i)
	      };
	    }
	    else if (/k-meleon/i.test(ua)) {
	      result = {
	        name: 'K-Meleon'
	        , kMeleon: t
	        , version: getFirstMatch(/(?:k-meleon)[\s\/](\d+(?:\.\d+)+)/i)
	      };
	    }
	    else if (windowsphone) {
	      result = {
	        name: 'Windows Phone'
	      , windowsphone: t
	      };
	      if (edgeVersion) {
	        result.msedge = t;
	        result.version = edgeVersion;
	      }
	      else {
	        result.msie = t;
	        result.version = getFirstMatch(/iemobile\/(\d+(\.\d+)?)/i);
	      }
	    }
	    else if (/msie|trident/i.test(ua)) {
	      result = {
	        name: 'Internet Explorer'
	      , msie: t
	      , version: getFirstMatch(/(?:msie |rv:)(\d+(\.\d+)?)/i)
	      };
	    } else if (chromeos) {
	      result = {
	        name: 'Chrome'
	      , chromeos: t
	      , chromeBook: t
	      , chrome: t
	      , version: getFirstMatch(/(?:chrome|crios|crmo)\/(\d+(\.\d+)?)/i)
	      };
	    } else if (/chrome.+? edge/i.test(ua)) {
	      result = {
	        name: 'Microsoft Edge'
	      , msedge: t
	      , version: edgeVersion
	      };
	    }
	    else if (/vivaldi/i.test(ua)) {
	      result = {
	        name: 'Vivaldi'
	        , vivaldi: t
	        , version: getFirstMatch(/vivaldi\/(\d+(\.\d+)?)/i) || versionIdentifier
	      };
	    }
	    else if (sailfish) {
	      result = {
	        name: 'Sailfish'
	      , sailfish: t
	      , version: getFirstMatch(/sailfish\s?browser\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/seamonkey\//i.test(ua)) {
	      result = {
	        name: 'SeaMonkey'
	      , seamonkey: t
	      , version: getFirstMatch(/seamonkey\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/firefox|iceweasel|fxios/i.test(ua)) {
	      result = {
	        name: 'Firefox'
	      , firefox: t
	      , version: getFirstMatch(/(?:firefox|iceweasel|fxios)[ \/](\d+(\.\d+)?)/i)
	      };
	      if (/\((mobile|tablet);[^\)]*rv:[\d\.]+\)/i.test(ua)) {
	        result.firefoxos = t;
	      }
	    }
	    else if (silk) {
	      result =  {
	        name: 'Amazon Silk'
	      , silk: t
	      , version : getFirstMatch(/silk\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/phantom/i.test(ua)) {
	      result = {
	        name: 'PhantomJS'
	      , phantom: t
	      , version: getFirstMatch(/phantomjs\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/slimerjs/i.test(ua)) {
	      result = {
	        name: 'SlimerJS'
	        , slimer: t
	        , version: getFirstMatch(/slimerjs\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (/blackberry|\bbb\d+/i.test(ua) || /rim\stablet/i.test(ua)) {
	      result = {
	        name: 'BlackBerry'
	      , blackberry: t
	      , version: versionIdentifier || getFirstMatch(/blackberry[\d]+\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (webos) {
	      result = {
	        name: 'WebOS'
	      , webos: t
	      , version: versionIdentifier || getFirstMatch(/w(?:eb)?osbrowser\/(\d+(\.\d+)?)/i)
	      };
	      /touchpad\//i.test(ua) && (result.touchpad = t);
	    }
	    else if (/bada/i.test(ua)) {
	      result = {
	        name: 'Bada'
	      , bada: t
	      , version: getFirstMatch(/dolfin\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (tizen) {
	      result = {
	        name: 'Tizen'
	      , tizen: t
	      , version: getFirstMatch(/(?:tizen\s?)?browser\/(\d+(\.\d+)?)/i) || versionIdentifier
	      };
	    }
	    else if (/qupzilla/i.test(ua)) {
	      result = {
	        name: 'QupZilla'
	        , qupzilla: t
	        , version: getFirstMatch(/(?:qupzilla)[\s\/](\d+(?:\.\d+)+)/i) || versionIdentifier
	      };
	    }
	    else if (/chromium/i.test(ua)) {
	      result = {
	        name: 'Chromium'
	        , chromium: t
	        , version: getFirstMatch(/(?:chromium)[\s\/](\d+(?:\.\d+)?)/i) || versionIdentifier
	      };
	    }
	    else if (/chrome|crios|crmo/i.test(ua)) {
	      result = {
	        name: 'Chrome'
	        , chrome: t
	        , version: getFirstMatch(/(?:chrome|crios|crmo)\/(\d+(\.\d+)?)/i)
	      };
	    }
	    else if (android) {
	      result = {
	        name: 'Android'
	        , version: versionIdentifier
	      };
	    }
	    else if (/safari|applewebkit/i.test(ua)) {
	      result = {
	        name: 'Safari'
	      , safari: t
	      };
	      if (versionIdentifier) {
	        result.version = versionIdentifier;
	      }
	    }
	    else if (iosdevice) {
	      result = {
	        name : iosdevice == 'iphone' ? 'iPhone' : iosdevice == 'ipad' ? 'iPad' : 'iPod'
	      };
	      // WTF: version is not part of user agent in web apps
	      if (versionIdentifier) {
	        result.version = versionIdentifier;
	      }
	    }
	    else if(/googlebot/i.test(ua)) {
	      result = {
	        name: 'Googlebot'
	      , googlebot: t
	      , version: getFirstMatch(/googlebot\/(\d+(\.\d+))/i) || versionIdentifier
	      };
	    }
	    else {
	      result = {
	        name: getFirstMatch(/^(.*)\/(.*) /),
	        version: getSecondMatch(/^(.*)\/(.*) /)
	     };
	   }

	    // set webkit or gecko flag for browsers based on these engines
	    if (!result.msedge && /(apple)?webkit/i.test(ua)) {
	      if (/(apple)?webkit\/537\.36/i.test(ua)) {
	        result.name = result.name || "Blink";
	        result.blink = t;
	      } else {
	        result.name = result.name || "Webkit";
	        result.webkit = t;
	      }
	      if (!result.version && versionIdentifier) {
	        result.version = versionIdentifier;
	      }
	    } else if (!result.opera && /gecko\//i.test(ua)) {
	      result.name = result.name || "Gecko";
	      result.gecko = t;
	      result.version = result.version || getFirstMatch(/gecko\/(\d+(\.\d+)?)/i);
	    }

	    // set OS flags for platforms that have multiple browsers
	    if (!result.windowsphone && !result.msedge && (android || result.silk)) {
	      result.android = t;
	    } else if (!result.windowsphone && !result.msedge && iosdevice) {
	      result[iosdevice] = t;
	      result.ios = t;
	    } else if (mac) {
	      result.mac = t;
	    } else if (xbox) {
	      result.xbox = t;
	    } else if (windows) {
	      result.windows = t;
	    } else if (linux) {
	      result.linux = t;
	    }

	    // OS version extraction
	    var osVersion = '';
	    if (result.windowsphone) {
	      osVersion = getFirstMatch(/windows phone (?:os)?\s?(\d+(\.\d+)*)/i);
	    } else if (iosdevice) {
	      osVersion = getFirstMatch(/os (\d+([_\s]\d+)*) like mac os x/i);
	      osVersion = osVersion.replace(/[_\s]/g, '.');
	    } else if (android) {
	      osVersion = getFirstMatch(/android[ \/-](\d+(\.\d+)*)/i);
	    } else if (result.webos) {
	      osVersion = getFirstMatch(/(?:web|hpw)os\/(\d+(\.\d+)*)/i);
	    } else if (result.blackberry) {
	      osVersion = getFirstMatch(/rim\stablet\sos\s(\d+(\.\d+)*)/i);
	    } else if (result.bada) {
	      osVersion = getFirstMatch(/bada\/(\d+(\.\d+)*)/i);
	    } else if (result.tizen) {
	      osVersion = getFirstMatch(/tizen[\/\s](\d+(\.\d+)*)/i);
	    }
	    if (osVersion) {
	      result.osversion = osVersion;
	    }

	    // device type extraction
	    var osMajorVersion = osVersion.split('.')[0];
	    if (
	         tablet
	      || nexusTablet
	      || iosdevice == 'ipad'
	      || (android && (osMajorVersion == 3 || (osMajorVersion >= 4 && !mobile)))
	      || result.silk
	    ) {
	      result.tablet = t;
	    } else if (
	         mobile
	      || iosdevice == 'iphone'
	      || iosdevice == 'ipod'
	      || android
	      || nexusMobile
	      || result.blackberry
	      || result.webos
	      || result.bada
	    ) {
	      result.mobile = t;
	    }

	    // Graded Browser Support
	    // http://developer.yahoo.com/yui/articles/gbs
	    if (result.msedge ||
	        (result.msie && result.version >= 10) ||
	        (result.yandexbrowser && result.version >= 15) ||
			    (result.vivaldi && result.version >= 1.0) ||
	        (result.chrome && result.version >= 20) ||
	        (result.samsungBrowser && result.version >= 4) ||
	        (result.firefox && result.version >= 20.0) ||
	        (result.safari && result.version >= 6) ||
	        (result.opera && result.version >= 10.0) ||
	        (result.ios && result.osversion && result.osversion.split(".")[0] >= 6) ||
	        (result.blackberry && result.version >= 10.1)
	        || (result.chromium && result.version >= 20)
	        ) {
	      result.a = t;
	    }
	    else if ((result.msie && result.version < 10) ||
	        (result.chrome && result.version < 20) ||
	        (result.firefox && result.version < 20.0) ||
	        (result.safari && result.version < 6) ||
	        (result.opera && result.version < 10.0) ||
	        (result.ios && result.osversion && result.osversion.split(".")[0] < 6)
	        || (result.chromium && result.version < 20)
	        ) {
	      result.c = t;
	    } else result.x = t;

	    return result
	  }

	  var bowser = detect(typeof navigator !== 'undefined' ? navigator.userAgent || '' : '');

	  bowser.test = function (browserList) {
	    for (var i = 0; i < browserList.length; ++i) {
	      var browserItem = browserList[i];
	      if (typeof browserItem=== 'string') {
	        if (browserItem in bowser) {
	          return true;
	        }
	      }
	    }
	    return false;
	  };

	  /**
	   * Get version precisions count
	   *
	   * @example
	   *   getVersionPrecision("1.10.3") // 3
	   *
	   * @param  {string} version
	   * @return {number}
	   */
	  function getVersionPrecision(version) {
	    return version.split(".").length;
	  }

	  /**
	   * Array::map polyfill
	   *
	   * @param  {Array} arr
	   * @param  {Function} iterator
	   * @return {Array}
	   */
	  function map(arr, iterator) {
	    var result = [], i;
	    if (Array.prototype.map) {
	      return Array.prototype.map.call(arr, iterator);
	    }
	    for (i = 0; i < arr.length; i++) {
	      result.push(iterator(arr[i]));
	    }
	    return result;
	  }

	  /**
	   * Calculate browser version weight
	   *
	   * @example
	   *   compareVersions(['1.10.2.1',  '1.8.2.1.90'])    // 1
	   *   compareVersions(['1.010.2.1', '1.09.2.1.90']);  // 1
	   *   compareVersions(['1.10.2.1',  '1.10.2.1']);     // 0
	   *   compareVersions(['1.10.2.1',  '1.0800.2']);     // -1
	   *
	   * @param  {Array<String>} versions versions to compare
	   * @return {Number} comparison result
	   */
	  function compareVersions(versions) {
	    // 1) get common precision for both versions, for example for "10.0" and "9" it should be 2
	    var precision = Math.max(getVersionPrecision(versions[0]), getVersionPrecision(versions[1]));
	    var chunks = map(versions, function (version) {
	      var delta = precision - getVersionPrecision(version);

	      // 2) "9" -> "9.0" (for precision = 2)
	      version = version + new Array(delta + 1).join(".0");

	      // 3) "9.0" -> ["000000000"", "000000009"]
	      return map(version.split("."), function (chunk) {
	        return new Array(20 - chunk.length).join("0") + chunk;
	      }).reverse();
	    });

	    // iterate in reverse order by reversed chunks array
	    while (--precision >= 0) {
	      // 4) compare: "000000009" > "000000010" = false (but "9" > "10" = true)
	      if (chunks[0][precision] > chunks[1][precision]) {
	        return 1;
	      }
	      else if (chunks[0][precision] === chunks[1][precision]) {
	        if (precision === 0) {
	          // all version chunks are same
	          return 0;
	        }
	      }
	      else {
	        return -1;
	      }
	    }
	  }

	  /**
	   * Check if browser is unsupported
	   *
	   * @example
	   *   bowser.isUnsupportedBrowser({
	   *     msie: "10",
	   *     firefox: "23",
	   *     chrome: "29",
	   *     safari: "5.1",
	   *     opera: "16",
	   *     phantom: "534"
	   *   });
	   *
	   * @param  {Object}  minVersions map of minimal version to browser
	   * @param  {Boolean} [strictMode = false] flag to return false if browser wasn't found in map
	   * @param  {String}  [ua] user agent string
	   * @return {Boolean}
	   */
	  function isUnsupportedBrowser(minVersions, strictMode, ua) {
	    var _bowser = bowser;

	    // make strictMode param optional with ua param usage
	    if (typeof strictMode === 'string') {
	      ua = strictMode;
	      strictMode = void(0);
	    }

	    if (strictMode === void(0)) {
	      strictMode = false;
	    }
	    if (ua) {
	      _bowser = detect(ua);
	    }

	    var version = "" + _bowser.version;
	    for (var browser in minVersions) {
	      if (minVersions.hasOwnProperty(browser)) {
	        if (_bowser[browser]) {
	          if (typeof minVersions[browser] !== 'string') {
	            throw new Error('Browser version in the minVersion map should be a string: ' + browser + ': ' + String(minVersions));
	          }

	          // browser version and min supported version.
	          return compareVersions([version, minVersions[browser]]) < 0;
	        }
	      }
	    }

	    return strictMode; // not found
	  }

	  /**
	   * Check if browser is supported
	   *
	   * @param  {Object} minVersions map of minimal version to browser
	   * @param  {Boolean} [strictMode = false] flag to return false if browser wasn't found in map
	   * @param  {String}  [ua] user agent string
	   * @return {Boolean}
	   */
	  function check(minVersions, strictMode, ua) {
	    return !isUnsupportedBrowser(minVersions, strictMode, ua);
	  }

	  bowser.isUnsupportedBrowser = isUnsupportedBrowser;
	  bowser.compareVersions = compareVersions;
	  bowser.check = check;

	  /*
	   * Set our detect method to the main bowser object so we can
	   * reuse it to test other user agents.
	   * This is needed to implement future tests.
	   */
	  bowser._detect = detect;

	  return bowser
	});
	});

	var userAgent = navigator.userAgent.toLowerCase();
	//@ts-ignore
	var info = bowser._detect(userAgent);
	var webkitVersionMatch = userAgent.match(/applewebkit\/(\d+(:?\.\d+)*)/);
	//Helper
	//@ts-ignore
	var compareVersions = bowser.compareVersions;
	//Platforms
	var isMacPlatform = !!info.mac;
	var isAndroid = !!info.android;
	var isIOS = !!info.ios;
	var isMobile = !!info.mobile;
	var isTablet = !!info.tablet;
	//Browsers
	var version = parseInt(info.version, 10);
	var fullVersion = info.version;
	var webkitVersion = webkitVersionMatch && webkitVersionMatch[1] || '';
	var isIE = !!(info.msie || info.msedge);
	var isIE11 = isIE && version === 11;
	var isIE10 = isIE && version === 10;
	var isIE9 = isIE && version === 9;
	var isFirefox = !!info.firefox;
	var isMSEdge = !!info.msedge;
	var isChrome = !!info.chrome;
	var isSafari = !!info.safari;
	var isWebKit = !!(info.webkit || info.blink);
	var isElectron = /electron/g.test(userAgent);

	var browserUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		compareVersions: compareVersions,
		isMacPlatform: isMacPlatform,
		isAndroid: isAndroid,
		isIOS: isIOS,
		isMobile: isMobile,
		isTablet: isTablet,
		version: version,
		fullVersion: fullVersion,
		webkitVersion: webkitVersion,
		isIE: isIE,
		isIE11: isIE11,
		isIE10: isIE10,
		isIE9: isIE9,
		isFirefox: isFirefox,
		isMSEdge: isMSEdge,
		isChrome: isChrome,
		isSafari: isSafari,
		isWebKit: isWebKit,
		isElectron: isElectron
	});

	var EventEmitter = /** @class */ (function () {
	    function EventEmitter() {
	        this.eventsListeners = nativeMethods.objectCreate(null);
	    }
	    EventEmitter.prototype.emit = function (evt) {
	        var args = [];
	        for (var _i = 1; _i < arguments.length; _i++) {
	            args[_i - 1] = arguments[_i];
	        }
	        var listeners = this.eventsListeners[evt];
	        if (!listeners)
	            return;
	        var index = 0;
	        while (listeners[index]) {
	            // HACK: For IE: after calling document.write, the IFrameSandbox event handler throws the
	            // 'Can't execute code from a freed script' exception because the document has been
	            // recreated.
	            if (isIE) {
	                try {
	                    listeners[index].toString();
	                }
	                catch (e) {
	                    nativeMethods.arraySplice.call(listeners, index, 1);
	                    continue;
	                }
	            }
	            listeners[index++].apply(this, args);
	        }
	    };
	    EventEmitter.prototype.off = function (evt, listener) {
	        var listeners = this.eventsListeners[evt];
	        if (!listeners)
	            return;
	        this.eventsListeners[evt] = nativeMethods.arrayFilter.call(listeners, function (currentListener) { return currentListener !== listener; });
	    };
	    EventEmitter.prototype.on = function (evt, listener) {
	        this.eventsListeners[evt] = this.eventsListeners[evt] || [];
	        if (this.eventsListeners[evt].indexOf(listener) === -1)
	            this.eventsListeners[evt].push(listener);
	        return listener;
	    };
	    return EventEmitter;
	}());

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	var INTERNAL_ATTRIBUTES = {
	    storedAttrPostfix: '-hammerhead-stored-value',
	    hoverPseudoClass: 'data-hammerhead-hovered',
	    focusPseudoClass: 'data-hammerhead-focused',
	    uploadInfoHiddenInputName: 'hammerhead|upload-info-hidden-input-name'
	};

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	var POSTFIX = '-hammerhead-shadow-ui';
	var SHADOW_UI_CLASS_NAME = {
	    postfix: POSTFIX,
	    charset: 'charset' + POSTFIX,
	    script: 'script' + POSTFIX,
	    selfRemovingScript: 'self-removing-script' + POSTFIX,
	    uiStylesheet: 'ui-stylesheet' + POSTFIX
	};

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	// NOTE: Some websites override the String.prototype.trim method. When we use this function
	// in our scripts, we expect it to have the default behavior. Therefore, in order to protect
	// ourselves from spoofing, we must use our own implementation. Also, we cannot use the
	// String.prototype.trim method because on the client-side it is the same in the top window and
	// an iframe window. The client code may override this method in the top window before the
	// iframe is initialized, so that the iframe will lose access to the native method.
	function trim (str) {
	    return typeof str === 'string' ? str.replace(/(^\s+)|(\s+$)/g, '') : str;
	}

	// -------------------------------------------------------------
	var PROTOCOL_RE = /^([\w-]+?:)(\/\/|[^\\/]|$)/;
	var LEADING_SLASHES_RE = /^(\/\/)/;
	var HOST_RE = /^(.*?)(\/|%|\?|;|#|$)/;
	var PORT_RE = /:([0-9]*)$/;
	var QUERY_AND_HASH_RE = /(\?.+|#[^#]*)$/;
	var PATH_AFTER_HOST_RE = /^\/([^/]+?)\/([\S\s]+)$/;
	var HTTP_RE = /^https?:/;
	var FILE_RE = /^file:/i;
	var SHORT_ORIGIN_RE = /^http(s)?:\/\//;
	var IS_SECURE_ORIGIN_RE = /^s\*/;
	var SUPPORTED_PROTOCOL_RE = /^(?:https?|file):/i;
	var HASH_RE$1 = /^#/;
	var REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1 = '!';
	var REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR = '*';
	var TRAILING_SLASH_RE = /\/$/;
	var SPECIAL_BLANK_PAGE = 'about:blank';
	var SPECIAL_ERROR_PAGE = 'about:error';
	var SPECIAL_PAGES = [SPECIAL_BLANK_PAGE, SPECIAL_ERROR_PAGE];
	var HTTP_DEFAULT_PORT = '80';
	var HTTPS_DEFAULT_PORT = '443';
	var Credentials;
	(function (Credentials) {
	    Credentials[Credentials["include"] = 0] = "include";
	    Credentials[Credentials["sameOrigin"] = 1] = "sameOrigin";
	    Credentials[Credentials["omit"] = 2] = "omit";
	    Credentials[Credentials["unknown"] = 3] = "unknown";
	})(Credentials || (Credentials = {}));
	var SPECIAL_PAGE_DEST_RESOURCE_INFO = {
	    protocol: 'about:',
	    host: '',
	    hostname: '',
	    port: '',
	    partAfterHost: ''
	};
	var RESOURCE_TYPES = [
	    { name: 'isIframe', flag: 'i' },
	    { name: 'isForm', flag: 'f' },
	    { name: 'isScript', flag: 's' },
	    { name: 'isEventSource', flag: 'e' },
	    { name: 'isHtmlImport', flag: 'h' },
	    { name: 'isWebSocket', flag: 'w' },
	    { name: 'isServiceWorker', flag: 'c' },
	    { name: 'isAjax', flag: 'a' }
	];
	function parseResourceType$1(resourceType) {
	    var parsedResourceType = {};
	    if (!resourceType)
	        return parsedResourceType;
	    for (var _i = 0, RESOURCE_TYPES_1 = RESOURCE_TYPES; _i < RESOURCE_TYPES_1.length; _i++) {
	        var _a = RESOURCE_TYPES_1[_i], name_1 = _a.name, flag = _a.flag;
	        if (resourceType.indexOf(flag) > -1)
	            parsedResourceType[name_1] = true;
	    }
	    return parsedResourceType;
	}
	function getResourceTypeString(parsedResourceType) {
	    if (!parsedResourceType)
	        return null;
	    var resourceType = '';
	    for (var _i = 0, RESOURCE_TYPES_2 = RESOURCE_TYPES; _i < RESOURCE_TYPES_2.length; _i++) {
	        var _a = RESOURCE_TYPES_2[_i], name_2 = _a.name, flag = _a.flag;
	        if (parsedResourceType[name_2])
	            resourceType += flag;
	    }
	    return resourceType || null;
	}
	function makeShortOrigin(origin) {
	    return origin === 'null' ? '' : origin.replace(SHORT_ORIGIN_RE, function (_, secure) { return secure ? 's*' : ''; });
	}
	function restoreShortOrigin(origin) {
	    if (!origin)
	        return 'null';
	    return IS_SECURE_ORIGIN_RE.test(origin) ? origin.replace(IS_SECURE_ORIGIN_RE, 'https://') : 'http://' + origin;
	}
	function isSubDomain$1(domain, subDomain) {
	    domain = domain.replace(/^www./i, '');
	    subDomain = subDomain.replace(/^www./i, '');
	    if (domain === subDomain)
	        return true;
	    var index = subDomain.lastIndexOf(domain);
	    return subDomain[index - 1] === '.' && subDomain.length === index + domain.length;
	}
	function sameOriginCheck$1(location, checkedUrl) {
	    if (!checkedUrl)
	        return true;
	    var parsedCheckedUrl = parseUrl$1(checkedUrl);
	    var isRelative = !parsedCheckedUrl.host;
	    if (isRelative)
	        return true;
	    var parsedLocation = parseUrl$1(location);
	    var parsedProxyLocation = parseProxyUrl$1(location);
	    if (parsedCheckedUrl.host === parsedLocation.host && parsedCheckedUrl.protocol === parsedLocation.protocol)
	        return true;
	    var parsedDestUrl = parsedProxyLocation ? parsedProxyLocation.destResourceInfo : parsedLocation;
	    if (!parsedDestUrl)
	        return false;
	    var isSameProtocol = !parsedCheckedUrl.protocol || parsedCheckedUrl.protocol === parsedDestUrl.protocol;
	    var portsEq = !parsedDestUrl.port && !parsedCheckedUrl.port ||
	        parsedDestUrl.port && parsedDestUrl.port.toString() === parsedCheckedUrl.port;
	    return isSameProtocol && !!portsEq && parsedDestUrl.hostname === parsedCheckedUrl.hostname;
	}
	// NOTE: Convert the destination protocol and hostname to the lower case. (GH-1)
	function convertHostToLowerCase(url) {
	    var parsedUrl = parseUrl$1(url);
	    parsedUrl.protocol = parsedUrl.protocol && parsedUrl.protocol.toLowerCase();
	    parsedUrl.host = parsedUrl.host && parsedUrl.host.toLowerCase();
	    return formatUrl$1(parsedUrl);
	}
	function getURLString(url) {
	    // TODO: fix it
	    // eslint-disable-next-line no-undef
	    if (url === null && /iPad|iPhone/i.test(window.navigator.userAgent))
	        return '';
	    return String(url).replace(/\n|\t/g, '');
	}
	function getProxyUrl$1(url, opts) {
	    var sessionInfo = [opts.sessionId];
	    if (opts.windowId)
	        sessionInfo.push(opts.windowId);
	    var params = [sessionInfo.join(REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR)];
	    if (opts.resourceType)
	        params.push(opts.resourceType);
	    if (opts.charset)
	        params.push(opts.charset.toLowerCase());
	    if (typeof opts.credentials === 'number')
	        params.push(opts.credentials.toString());
	    if (opts.reqOrigin)
	        params.push(encodeURIComponent(makeShortOrigin(opts.reqOrigin)));
	    var descriptor = params.join(REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1);
	    var proxyProtocol = opts.proxyProtocol || 'http:';
	    return proxyProtocol + "//" + opts.proxyHostname + ":" + opts.proxyPort + "/" + descriptor + "/" + convertHostToLowerCase(url);
	}
	function getDomain(parsed) {
	    if (parsed.protocol === 'file:')
	        return 'null';
	    return formatUrl$1({
	        protocol: parsed.protocol,
	        host: parsed.host,
	        hostname: parsed.hostname,
	        port: String(parsed.port || '')
	    });
	}
	function parseRequestDescriptor(desc) {
	    var _a = desc.split(REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1), sessionInfo = _a[0], resourceType = _a[1], resourceData = _a.slice(2);
	    if (!sessionInfo)
	        return null;
	    var _b = sessionInfo.split(REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR), sessionId = _b[0], windowId = _b[1];
	    var parsedDesc = { sessionId: sessionId, resourceType: resourceType || null };
	    if (windowId)
	        parsedDesc.windowId = windowId;
	    if (resourceType && resourceData.length) {
	        var parsedResourceType = parseResourceType$1(resourceType);
	        if (parsedResourceType.isScript || parsedResourceType.isServiceWorker)
	            parsedDesc.charset = resourceData[0];
	        else if (parsedResourceType.isWebSocket)
	            parsedDesc.reqOrigin = decodeURIComponent(restoreShortOrigin(resourceData[0]));
	        else if (parsedResourceType.isAjax) {
	            parsedDesc.credentials = parseInt(resourceData[0]);
	            if (resourceData.length === 2)
	                parsedDesc.reqOrigin = decodeURIComponent(restoreShortOrigin(resourceData[1]));
	        }
	    }
	    return parsedDesc;
	}
	function parseProxyUrl$1(proxyUrl) {
	    // TODO: Remove it.
	    var parsedUrl = parseUrl$1(proxyUrl);
	    if (!parsedUrl.partAfterHost)
	        return null;
	    var match = parsedUrl.partAfterHost.match(PATH_AFTER_HOST_RE);
	    if (!match)
	        return null;
	    var parsedDesc = parseRequestDescriptor(match[1]);
	    // NOTE: We should have, at least, the job uid and the owner token.
	    if (!parsedDesc)
	        return null;
	    var destUrl = match[2];
	    // Browser can redirect to a special page with hash (GH-1671)
	    var destUrlWithoutHash = destUrl.replace(/#[\S\s]*$/, '');
	    if (!isSpecialPage$1(destUrlWithoutHash) && !SUPPORTED_PROTOCOL_RE.test(destUrl))
	        return null;
	    var destResourceInfo = null;
	    if (isSpecialPage$1(destUrlWithoutHash))
	        destResourceInfo = SPECIAL_PAGE_DEST_RESOURCE_INFO;
	    else {
	        destUrl = omitDefaultPort(destUrl);
	        destResourceInfo = parseUrl$1(destUrl);
	    }
	    return {
	        destUrl: destUrl,
	        destResourceInfo: destResourceInfo,
	        partAfterHost: parsedUrl.partAfterHost,
	        proxy: {
	            hostname: parsedUrl.hostname,
	            port: parsedUrl.port
	        },
	        sessionId: parsedDesc.sessionId,
	        resourceType: parsedDesc.resourceType,
	        charset: parsedDesc.charset,
	        reqOrigin: parsedDesc.reqOrigin,
	        windowId: parsedDesc.windowId,
	        credentials: parsedDesc.credentials
	    };
	}
	function getPathname(path) {
	    return path.replace(QUERY_AND_HASH_RE, '');
	}
	function parseUrl$1(url) {
	    var parsed = {};
	    url = processSpecialChars(url);
	    if (!url)
	        return parsed;
	    url = trim(url);
	    // Protocol
	    var hasImplicitProtocol = false;
	    var remainder = url
	        .replace(PROTOCOL_RE, function (_str, protocol, strAfterProtocol) {
	        parsed.protocol = protocol;
	        return strAfterProtocol;
	    })
	        .replace(LEADING_SLASHES_RE, function () {
	        hasImplicitProtocol = true;
	        return '';
	    });
	    // NOTE: the URL is relative.
	    if (!parsed.protocol && !hasImplicitProtocol) {
	        parsed.partAfterHost = url;
	        return parsed;
	    }
	    // Host
	    parsed.partAfterHost = remainder
	        .replace(HOST_RE, function (_str, host, restPartSeparator) {
	        parsed.host = host;
	        parsed.port = '';
	        return restPartSeparator;
	    });
	    if (typeof parsed.host === 'string') {
	        var authHostArr = parsed.host.split('@');
	        if (authHostArr.length === 2) {
	            parsed.auth = authHostArr[0];
	            parsed.host = authHostArr[1];
	        }
	    }
	    parsed.hostname = parsed.host ? parsed.host.replace(PORT_RE, function (_str, port) {
	        parsed.port = port;
	        return '';
	    }) : '';
	    return parsed;
	}
	function isSupportedProtocol$1(url) {
	    url = trim(url || '');
	    var isHash = HASH_RE$1.test(url);
	    if (isHash)
	        return false;
	    var protocol = url.match(PROTOCOL_RE);
	    if (!protocol)
	        return true;
	    return SUPPORTED_PROTOCOL_RE.test(protocol[0]);
	}
	function resolveUrlAsDest$1(url, getProxyUrlMeth) {
	    getProxyUrlMeth = getProxyUrlMeth || getProxyUrl$1;
	    if (isSupportedProtocol$1(url)) {
	        var proxyUrl = getProxyUrlMeth(url);
	        var parsedProxyUrl = parseProxyUrl$1(proxyUrl);
	        return parsedProxyUrl ? formatUrl$1(parsedProxyUrl.destResourceInfo) : url;
	    }
	    return url;
	}
	function formatUrl$1(parsedUrl) {
	    // NOTE: the URL is relative.
	    if (parsedUrl.protocol !== 'file:' && !parsedUrl.host && (!parsedUrl.hostname || !parsedUrl.port))
	        return parsedUrl.partAfterHost;
	    var url = parsedUrl.protocol || '';
	    if (parsedUrl.protocol !== 'about:')
	        url += '//';
	    if (parsedUrl.auth)
	        url += parsedUrl.auth + '@';
	    if (parsedUrl.host)
	        url += parsedUrl.host;
	    else {
	        url += parsedUrl.hostname;
	        if (parsedUrl.port)
	            url += ':' + parsedUrl.port;
	    }
	    if (parsedUrl.partAfterHost)
	        url += parsedUrl.partAfterHost;
	    return url;
	}
	function correctMultipleSlashes(url, pageProtocol) {
	    if (pageProtocol === void 0) { pageProtocol = ''; }
	    // NOTE: Remove unnecessary slashes from the beginning of the url and after scheme.
	    // For example:
	    // "//////example.com" -> "//example.com" (scheme-less HTTP(S) URL)
	    // "////home/testcafe/documents" -> "///home/testcafe/documents" (scheme-less unix file URL)
	    // "http:///example.com" -> "http://example.com"
	    //
	    // And add missing slashes after the file scheme.
	    // "file://C:/document.txt" -> "file:///C:/document.txt"
	    if (url.match(FILE_RE) || pageProtocol.match(FILE_RE)) {
	        return url
	            .replace(/^(file:)?\/+(\/\/\/.*$)/i, '$1$2')
	            .replace(/^(file:)?\/*([A-Za-z]):/i, '$1///$2:');
	    }
	    return url.replace(/^(https?:)?\/+(\/\/.*$)/i, '$1$2');
	}
	function processSpecialChars(url) {
	    return correctMultipleSlashes(getURLString(url));
	}
	function ensureTrailingSlash(srcUrl, processedUrl) {
	    if (!isValidUrl(processedUrl))
	        return processedUrl;
	    var srcUrlEndsWithTrailingSlash = TRAILING_SLASH_RE.test(srcUrl);
	    var processedUrlEndsWithTrailingSlash = TRAILING_SLASH_RE.test(processedUrl);
	    if (srcUrlEndsWithTrailingSlash && !processedUrlEndsWithTrailingSlash)
	        processedUrl += '/';
	    else if (srcUrl && !srcUrlEndsWithTrailingSlash && processedUrlEndsWithTrailingSlash)
	        processedUrl = processedUrl.replace(TRAILING_SLASH_RE, '');
	    return processedUrl;
	}
	function isSpecialPage$1(url) {
	    return SPECIAL_PAGES.indexOf(url) !== -1;
	}
	function isRelativeUrl(url) {
	    var parsedUrl = parseUrl$1(url);
	    return parsedUrl.protocol !== 'file:' && !parsedUrl.host;
	}
	function isValidPort(port) {
	    var parsedPort = parseInt(port, 10);
	    return parsedPort > 0 && parsedPort <= 65535;
	}
	function isValidUrl(url) {
	    var parsedUrl = parseUrl$1(url);
	    return parsedUrl.protocol === 'file:' || !!parsedUrl.hostname && (!parsedUrl.port || isValidPort(parsedUrl.port));
	}
	function ensureOriginTrailingSlash(url) {
	    // NOTE: If you request an url containing only port, host and protocol
	    // then browser adds the trailing slash itself.
	    var parsedUrl = parseUrl$1(url);
	    if (!parsedUrl.partAfterHost && HTTP_RE.test(parsedUrl.protocol))
	        return url + '/';
	    return url;
	}
	function omitDefaultPort(url) {
	    // NOTE: If you request an url containing default port
	    // then browser remove this one itself.
	    var parsedUrl = parseUrl$1(url);
	    var hasDefaultPort = parsedUrl.protocol === 'https:' && parsedUrl.port === HTTPS_DEFAULT_PORT ||
	        parsedUrl.protocol === 'http:' && parsedUrl.port === HTTP_DEFAULT_PORT;
	    if (hasDefaultPort) {
	        parsedUrl.host = parsedUrl.hostname;
	        parsedUrl.port = '';
	        return formatUrl$1(parsedUrl);
	    }
	    return url;
	}
	function prepareUrl(url) {
	    url = omitDefaultPort(url);
	    url = ensureOriginTrailingSlash(url);
	    return url;
	}
	function updateScriptImportUrls(cachedScript, serverInfo, sessionId, windowId) {
	    var regExp = new RegExp('(' + serverInfo.protocol + '//' + serverInfo.hostname + ':(?:' + serverInfo.port + '|' +
	        serverInfo.crossDomainPort + ')/)[^/' + REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1 + ']+', 'g');
	    var pattern = '$1' + sessionId + (windowId ? REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR + windowId : '');
	    return cachedScript.replace(regExp, pattern);
	}

	var sharedUrlUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		SUPPORTED_PROTOCOL_RE: SUPPORTED_PROTOCOL_RE,
		HASH_RE: HASH_RE$1,
		REQUEST_DESCRIPTOR_VALUES_SEPARATOR: REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1,
		REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR: REQUEST_DESCRIPTOR_SESSION_INFO_VALUES_SEPARATOR,
		TRAILING_SLASH_RE: TRAILING_SLASH_RE,
		SPECIAL_BLANK_PAGE: SPECIAL_BLANK_PAGE,
		SPECIAL_ERROR_PAGE: SPECIAL_ERROR_PAGE,
		SPECIAL_PAGES: SPECIAL_PAGES,
		HTTP_DEFAULT_PORT: HTTP_DEFAULT_PORT,
		HTTPS_DEFAULT_PORT: HTTPS_DEFAULT_PORT,
		get Credentials () { return Credentials; },
		parseResourceType: parseResourceType$1,
		getResourceTypeString: getResourceTypeString,
		restoreShortOrigin: restoreShortOrigin,
		isSubDomain: isSubDomain$1,
		sameOriginCheck: sameOriginCheck$1,
		getURLString: getURLString,
		getProxyUrl: getProxyUrl$1,
		getDomain: getDomain,
		parseProxyUrl: parseProxyUrl$1,
		getPathname: getPathname,
		parseUrl: parseUrl$1,
		isSupportedProtocol: isSupportedProtocol$1,
		resolveUrlAsDest: resolveUrlAsDest$1,
		formatUrl: formatUrl$1,
		correctMultipleSlashes: correctMultipleSlashes,
		processSpecialChars: processSpecialChars,
		ensureTrailingSlash: ensureTrailingSlash,
		isSpecialPage: isSpecialPage$1,
		isRelativeUrl: isRelativeUrl,
		isValidUrl: isValidUrl,
		ensureOriginTrailingSlash: ensureOriginTrailingSlash,
		omitDefaultPort: omitDefaultPort,
		prepareUrl: prepareUrl,
		updateScriptImportUrls: updateScriptImportUrls
	});

	var DOCUMENT_URL_RESOLVER = 'hammerhead|document-url-resolver';
	var urlResolver = {
	    _createResolver: function (doc) {
	        var htmlDocument = nativeMethods.createHTMLDocument.call(doc.implementation, 'title');
	        var a = nativeMethods.createElement.call(htmlDocument, 'a');
	        var base = nativeMethods.createElement.call(htmlDocument, 'base');
	        nativeMethods.appendChild.call(htmlDocument.body, a);
	        nativeMethods.appendChild.call(htmlDocument.head, base);
	        return htmlDocument;
	    },
	    _getResolver: function (doc) {
	        // NOTE: Once a document is recreated (document.open, document.write is called), nativeMethods will be refreshed.
	        // If we call urlResolve.updateBase after this,
	        // we will use native methods from an actual document.
	        // However, a document that contains an element for url resolving is created using a previous version of nativeMethods.
	        if (!doc[DOCUMENT_URL_RESOLVER]) {
	            nativeMethods.objectDefineProperty(doc, DOCUMENT_URL_RESOLVER, {
	                value: this._createResolver(doc),
	                writable: true
	            });
	        }
	        return doc[DOCUMENT_URL_RESOLVER];
	    },
	    _isNestedIframeWithoutSrc: function (win) {
	        if (!win || !win.parent || win.parent === win || win.parent.parent === win.parent)
	            return false;
	        var iframeElement = getFrameElement(window);
	        return !!iframeElement && isIframeWithoutSrc(iframeElement);
	    },
	    init: function (doc) {
	        this.updateBase(get$2(), doc);
	    },
	    getResolverElement: function (doc) {
	        return nativeMethods.nodeFirstChildGetter.call(this._getResolver(doc).body);
	    },
	    resolve: function (url, doc) {
	        var resolver = this.getResolverElement(doc);
	        var href = null;
	        if (url === null)
	            nativeMethods.removeAttribute.call(resolver, 'href');
	        else {
	            nativeMethods.anchorHrefSetter.call(resolver, url);
	            href = nativeMethods.anchorHrefGetter.call(resolver);
	            // NOTE: It looks like a Chrome bug: in a nested iframe without src (when an iframe is placed into another
	            // iframe) you cannot set a relative link href while the iframe loading is not completed. So, we'll do it with
	            // the parent's urlResolver Safari demonstrates similar behavior, but urlResolver.href has a relative URL value.
	            var needUseParentResolver = url && (!href || href.charAt(0) === '/') &&
	                this._isNestedIframeWithoutSrc(doc.defaultView);
	            if (needUseParentResolver)
	                return this.resolve(url, window.parent.document);
	        }
	        return ensureTrailingSlash(url, href);
	    },
	    updateBase: function (url, doc) {
	        var resolverDocument = this._getResolver(doc);
	        var baseElement = nativeMethods.elementGetElementsByTagName.call(resolverDocument.head, 'base')[0];
	        url = url || get$2();
	        /*eslint-disable no-restricted-properties*/
	        var parsedUrl = parseUrl$1(url);
	        var isRelativeUrl = parsedUrl.protocol !== 'file:' && !parsedUrl.host;
	        var isProtocolRelativeUrl = /^\/\//.test(url) && !!parsedUrl.host;
	        /*eslint-enable no-restricted-properties*/
	        if (isRelativeUrl || isProtocolRelativeUrl) {
	            var destinationLocation = get$2();
	            this.updateBase(destinationLocation, doc);
	            url = this.resolve(url, doc);
	        }
	        nativeMethods.setAttribute.call(baseElement, 'href', url);
	    },
	    getBaseUrl: function (doc) {
	        var baseElement = nativeMethods.elementGetElementsByTagName.call(this._getResolver(doc).head, 'base')[0];
	        return nativeMethods.getAttribute.call(baseElement, 'href');
	    },
	    changeUrlPart: function (url, nativePropSetter, value, doc) {
	        var resolver = this.getResolverElement(doc);
	        nativeMethods.anchorHrefSetter.call(resolver, url);
	        nativePropSetter.call(resolver, value);
	        return nativeMethods.anchorHrefGetter.call(resolver);
	    },
	    dispose: function (doc) {
	        doc[DOCUMENT_URL_RESOLVER] = null;
	    }
	};

	var Settings = /** @class */ (function () {
	    function Settings() {
	        this._settings = {
	            isFirstPageLoad: true,
	            sessionId: '',
	            forceProxySrcForImage: false,
	            crossDomainProxyPort: '',
	            referer: '',
	            serviceMsgUrl: '',
	            transportWorkerUrl: '',
	            iframeTaskScriptTemplate: '',
	            cookie: '',
	            allowMultipleWindows: false,
	            isRecordMode: false,
	            windowId: ''
	        };
	    }
	    Settings.prototype.set = function (value) {
	        this._settings = value;
	    };
	    Settings.prototype.get = function () {
	        return this._settings;
	    };
	    return Settings;
	}());
	var settings = new Settings();

	var forcedLocation = null;
	// NOTE: exposed only for tests
	function getLocation() {
	    // NOTE: Used for testing. Unfortunately, we cannot override the 'getLocation' method in a test.
	    if (forcedLocation)
	        return forcedLocation;
	    var frameElement = getFrameElement(globalContextInfo.global);
	    // NOTE: Fallback to the owner page's URL if we are in an iframe without src.
	    if (frameElement && isIframeWithoutSrc(frameElement))
	        return settings.get().referer;
	    return globalContextInfo.global.location.toString();
	}
	// NOTE: We need to be able to force the page location. During the test, Hammerhead should think that it is on the
	// proxied page, not in the test environment. Unfortunately, we cannot do it in any other way.
	function forceLocation(url) {
	    forcedLocation = url;
	}
	function sameOriginCheck(location, checkedUrl) {
	    if (checkedUrl)
	        checkedUrl = resolveUrl(checkedUrl);
	    return sameOriginCheck$1(location, checkedUrl);
	}
	function resolveUrl(url, doc) {
	    var preProcessedUrl = getURLString(url);
	    if (preProcessedUrl && preProcessedUrl.indexOf('//') === 0) {
	        // eslint-disable-next-line no-restricted-properties
	        var pageProtocol = getParsed().protocol;
	        preProcessedUrl = pageProtocol + correctMultipleSlashes(preProcessedUrl, pageProtocol);
	    }
	    else
	        preProcessedUrl = correctMultipleSlashes(preProcessedUrl);
	    if (globalContextInfo.isInWorker) {
	        if (self.location.protocol !== 'blob:') // eslint-disable-line no-restricted-properties
	            return new nativeMethods.URL(preProcessedUrl, get$2()).href; // eslint-disable-line no-restricted-properties
	        else
	            return url;
	    }
	    else {
	        return urlResolver.resolve(preProcessedUrl, doc || document);
	    }
	}
	var get$2 = function () {
	    var location = getLocation();
	    var parsedProxyUrl = parseProxyUrl$1(location);
	    return parsedProxyUrl ? parsedProxyUrl.destUrl : location;
	};
	function overrideGet(func) {
	    get$2 = func;
	}
	function withHash(hash) {
	    var location = get$2();
	    // NOTE: Remove the previous hash if there is any.
	    location = location.replace(/(#.*)$/, '');
	    return location + hash;
	}
	function parseLocationThroughAnchor(url) {
	    var resolver = urlResolver.getResolverElement(document);
	    // eslint-disable-next-line no-restricted-properties
	    var destPort = parseUrl$1(url).port;
	    // NOTE: IE browser adds the default port for the https protocol while resolving.
	    nativeMethods.anchorHrefSetter.call(resolver, get$2());
	    var hostname = nativeMethods.anchorHostnameGetter.call(resolver);
	    var pathname = nativeMethods.anchorPathnameGetter.call(resolver);
	    // NOTE: IE ignores the first '/' symbol in the pathname.
	    if (pathname.charAt(0) !== '/')
	        pathname = '/' + pathname;
	    // TODO: Describe default ports logic.
	    return {
	        protocol: nativeMethods.anchorProtocolGetter.call(resolver),
	        // NOTE: Remove the default port.
	        port: destPort ? nativeMethods.anchorPortGetter.call(resolver) : '',
	        hostname: hostname,
	        // NOTE: Remove the default port from the host.
	        host: destPort ? nativeMethods.anchorHostGetter.call(resolver) : hostname,
	        pathname: pathname,
	        hash: resolver.hash,
	        search: nativeMethods.anchorSearchGetter.call(resolver)
	    };
	}
	function parseLocationThroughURL(url) {
	    var parsedUrl = new nativeMethods.URL(url);
	    /* eslint-disable no-restricted-properties */
	    return {
	        protocol: parsedUrl.protocol,
	        port: parsedUrl.port,
	        hostname: parsedUrl.hostname,
	        host: parsedUrl.host,
	        pathname: parsedUrl.pathname,
	        hash: parsedUrl.hash,
	        search: parsedUrl.search
	    };
	    /* eslint-enable no-restricted-properties */
	}
	function getParsed() {
	    var dest = get$2();
	    return globalContextInfo.isInWorker ? parseLocationThroughURL(dest) : parseLocationThroughAnchor(dest);
	}
	function getOriginHeader() {
	    return getDomain(getParsed());
	}

	var destLocationUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		getLocation: getLocation,
		forceLocation: forceLocation,
		sameOriginCheck: sameOriginCheck,
		resolveUrl: resolveUrl,
		get get () { return get$2; },
		overrideGet: overrideGet,
		withHash: withHash,
		getParsed: getParsed,
		getOriginHeader: getOriginHeader
	});

	var HASH_RE = /#[\S\s]*$/;
	var SUPPORTED_WEB_SOCKET_PROTOCOL_RE = /^wss?:/i;
	var SCOPE_RE = /\/[^/]*$/;
	// NOTE: The window.location equals 'about:blank' in iframes without src
	// therefore we need to find a window with src to get the proxy settings
	var DEFAULT_PROXY_SETTINGS = (function () {
	    /*eslint-disable no-restricted-properties*/
	    var locationWindow = globalContextInfo.isInWorker ? { location: parseUrl(self.location.origin), parent: null } : window;
	    var proxyLocation = locationWindow.location;
	    while (!proxyLocation.hostname) {
	        locationWindow = locationWindow.parent;
	        proxyLocation = locationWindow.location;
	    }
	    return {
	        hostname: proxyLocation.hostname,
	        port: proxyLocation.port.toString(),
	        protocol: proxyLocation.protocol
	    };
	    /*eslint-enable no-restricted-properties*/
	})();
	var REQUEST_DESCRIPTOR_VALUES_SEPARATOR = REQUEST_DESCRIPTOR_VALUES_SEPARATOR$1;
	var getProxyUrl = function (url, opts) {
	    url = getURLString(url);
	    var resourceType = opts && opts.resourceType;
	    var parsedResourceType = parseResourceType$1(resourceType);
	    if (!parsedResourceType.isWebSocket && !isSupportedProtocol(url) && !isSpecialPage(url))
	        return url;
	    // NOTE: Resolves relative URLs.
	    var resolvedUrl = resolveUrl(url, opts && opts.doc);
	    if (parsedResourceType.isWebSocket && !isValidWebSocketUrl(resolvedUrl) || !isValidUrl(resolvedUrl))
	        return url;
	    /*eslint-disable no-restricted-properties*/
	    var proxyHostname = opts && opts.proxyHostname || DEFAULT_PROXY_SETTINGS.hostname;
	    var proxyPort = opts && opts.proxyPort || DEFAULT_PROXY_SETTINGS.port;
	    var proxyServerProtocol = opts && opts.proxyProtocol || DEFAULT_PROXY_SETTINGS.protocol;
	    /*eslint-enable no-restricted-properties*/
	    var proxyProtocol = parsedResourceType.isWebSocket
	        ? proxyServerProtocol.replace('http', 'ws')
	        : proxyServerProtocol;
	    var sessionId = opts && opts.sessionId || settings.get().sessionId;
	    var windowId = opts && opts.windowId || settings.get().windowId;
	    var credentials = opts && opts.credentials;
	    var charset = opts && opts.charset;
	    var reqOrigin = opts && opts.reqOrigin;
	    var crossDomainPort = getCrossDomainProxyPort(proxyPort);
	    // NOTE: If the relative URL contains no slash (e.g. 'img123'), the resolver will keep
	    // the original proxy information, so that we can return such URL as is.
	    // TODO: Implement the isProxyURL function.
	    var parsedProxyUrl = parseProxyUrl$1(resolvedUrl);
	    /*eslint-disable no-restricted-properties*/
	    var isValidProxyUrl = !!parsedProxyUrl && parsedProxyUrl.proxy.hostname === proxyHostname &&
	        (parsedProxyUrl.proxy.port === proxyPort || parsedProxyUrl.proxy.port === crossDomainPort);
	    /*eslint-enable no-restricted-properties*/
	    if (isValidProxyUrl) {
	        if (resourceType && parsedProxyUrl.resourceType === resourceType)
	            return resolvedUrl;
	        // NOTE: Need to change the proxy URL resource type.
	        var destUrl = formatUrl$1(parsedProxyUrl.destResourceInfo);
	        return getProxyUrl(destUrl, {
	            proxyProtocol: proxyProtocol,
	            proxyHostname: proxyHostname,
	            proxyPort: proxyPort,
	            sessionId: sessionId,
	            resourceType: resourceType,
	            charset: charset,
	            reqOrigin: reqOrigin,
	            credentials: credentials
	        });
	    }
	    var parsedUrl = parseUrl$1(resolvedUrl);
	    if (!parsedUrl.protocol) // eslint-disable-line no-restricted-properties
	        return url;
	    charset = charset || (parsedResourceType.isScript || parsedResourceType.isServiceWorker) &&
	        document[INTERNAL_PROPS.documentCharset];
	    // NOTE: It seems that the relative URL had the leading slash or dots, so that the proxy info path part was
	    // removed by the resolver and we have an origin URL with the incorrect host and protocol.
	    /*eslint-disable no-restricted-properties*/
	    if (parsedUrl.protocol === proxyServerProtocol && parsedUrl.hostname === proxyHostname &&
	        parsedUrl.port === proxyPort) {
	        var parsedDestLocation = getParsed();
	        parsedUrl.protocol = parsedDestLocation.protocol;
	        parsedUrl.host = parsedDestLocation.host;
	        parsedUrl.hostname = parsedDestLocation.hostname;
	        parsedUrl.port = parsedDestLocation.port || '';
	        resolvedUrl = formatUrl$1(parsedUrl);
	    }
	    /*eslint-enable no-restricted-properties*/
	    if (parsedResourceType.isWebSocket) {
	        // eslint-disable-next-line no-restricted-properties
	        parsedUrl.protocol = parsedUrl.protocol.replace('ws', 'http');
	        resolvedUrl = formatUrl$1(parsedUrl);
	        reqOrigin = reqOrigin || getOriginHeader();
	    }
	    return getProxyUrl$1(resolvedUrl, {
	        proxyProtocol: proxyProtocol,
	        proxyHostname: proxyHostname,
	        proxyPort: proxyPort,
	        sessionId: sessionId,
	        resourceType: resourceType,
	        charset: charset,
	        reqOrigin: reqOrigin,
	        windowId: windowId,
	        credentials: credentials
	    });
	};
	function overrideGetProxyUrl(func) {
	    getProxyUrl = func;
	}
	function getNavigationUrl(url, win) {
	    // NOTE: For the 'about:blank' page, we perform url proxing only for the top window, 'location' object and links.
	    // For images and iframes, we keep urls as they were.
	    // See details in https://github.com/DevExpress/testcafe-hammerhead/issues/339
	    var destinationLocation = null;
	    var isIframe = win.top !== win;
	    var winLocation = win.location.toString();
	    if (isIframe)
	        destinationLocation = winLocation;
	    else {
	        var parsedProxyUrl = parseProxyUrl(winLocation);
	        destinationLocation = parsedProxyUrl && parsedProxyUrl.destUrl;
	    }
	    if (isSpecialPage(destinationLocation) && isRelativeUrl(url))
	        return '';
	    url = prepareUrl(url);
	    return getProxyUrl(url);
	}
	var getCrossDomainIframeProxyUrl = function (url) {
	    return getProxyUrl(url, {
	        proxyPort: settings.get().crossDomainProxyPort,
	        resourceType: getResourceTypeString({ isIframe: true })
	    });
	};
	function overrideGetCrossDomainIframeProxyUrl(func) {
	    getCrossDomainIframeProxyUrl = func;
	}
	function getPageProxyUrl(url, windowId) {
	    var parsedProxyUrl = parseProxyUrl(url);
	    var resourceType = null;
	    if (parsedProxyUrl) {
	        url = parsedProxyUrl.destUrl;
	        resourceType = parsedProxyUrl.resourceType;
	    }
	    if (resourceType) {
	        var parsedResourceType = parseResourceType(resourceType);
	        parsedResourceType.isIframe = false;
	        resourceType = stringifyResourceType(parsedResourceType);
	    }
	    var isCrossDomainUrl = !sameOriginCheck(getLocation(), url);
	    var proxyPort = isCrossDomainUrl ? settings.get().crossDomainProxyPort : location.port.toString(); // eslint-disable-line no-restricted-properties
	    return getProxyUrl(url, { windowId: windowId, proxyPort: proxyPort, resourceType: resourceType });
	}
	function getCrossDomainProxyPort(proxyPort) {
	    return settings.get().crossDomainProxyPort === proxyPort
	        // eslint-disable-next-line no-restricted-properties
	        ? location.port.toString()
	        : settings.get().crossDomainProxyPort;
	}
	var resolveUrlAsDest = function (url) {
	    return resolveUrlAsDest$1(url, getProxyUrl);
	};
	function overrideResolveUrlAsDest(func) {
	    resolveUrlAsDest = func;
	}
	function formatUrl(parsedUrl) {
	    return formatUrl$1(parsedUrl);
	}
	var parseProxyUrl = function (proxyUrl) {
	    return parseProxyUrl$1(proxyUrl);
	};
	function overrideParseProxyUrl(func) {
	    parseProxyUrl = func;
	}
	function parseUrl(url) {
	    return parseUrl$1(url);
	}
	var convertToProxyUrl = function (url, resourceType, charset, isCrossDomain) {
	    if (isCrossDomain === void 0) { isCrossDomain = false; }
	    return getProxyUrl(url, {
	        resourceType: resourceType, charset: charset,
	        // eslint-disable-next-line no-restricted-properties
	        proxyPort: isCrossDomain ? settings.get().crossDomainProxyPort : DEFAULT_PROXY_SETTINGS.port
	    });
	};
	function overrideConvertToProxyUrl(func) {
	    convertToProxyUrl = func;
	}
	function changeDestUrlPart(proxyUrl, nativePropSetter, value, resourceType) {
	    var parsed = parseProxyUrl$1(proxyUrl);
	    if (parsed) {
	        var sessionId = parsed.sessionId;
	        var proxy = parsed.proxy;
	        var destUrl = urlResolver.changeUrlPart(parsed.destUrl, nativePropSetter, value, document);
	        return getProxyUrl(destUrl, {
	            /*eslint-disable no-restricted-properties*/
	            proxyHostname: proxy.hostname,
	            proxyPort: proxy.port,
	            /*eslint-enable no-restricted-properties*/
	            sessionId: sessionId,
	            resourceType: resourceType
	        });
	    }
	    return proxyUrl;
	}
	function isValidWebSocketUrl(url) {
	    var resolvedUrl = resolveUrlAsDest(url);
	    return SUPPORTED_WEB_SOCKET_PROTOCOL_RE.test(resolvedUrl);
	}
	function isSubDomain(domain, subDomain) {
	    return isSubDomain$1(domain, subDomain);
	}
	function isSupportedProtocol(url) {
	    return isSupportedProtocol$1(url);
	}
	function isSpecialPage(url) {
	    return isSpecialPage$1(url);
	}
	function parseResourceType(resourceType) {
	    return parseResourceType$1(resourceType);
	}
	function stringifyResourceType(resourceType) {
	    return getResourceTypeString(resourceType);
	}
	function isChangedOnlyHash(currentUrl, newUrl) {
	    // NOTE: we compare proxied urls because urls passed into the function may be proxied, non-proxied
	    // or relative. The getProxyUrl function solves all the corresponding problems.
	    return getProxyUrl(currentUrl).replace(HASH_RE, '') === getProxyUrl(newUrl).replace(HASH_RE, '');
	}
	function getDestinationUrl(proxyUrl) {
	    var parsedProxyUrl = parseProxyUrl(proxyUrl);
	    return parsedProxyUrl ? parsedProxyUrl.destUrl : proxyUrl;
	}
	function getScope(url) {
	    if (!isSupportedProtocol(url))
	        return null;
	    var parsedUrl = parseUrl(resolveUrlAsDest(url));
	    if (!parsedUrl)
	        return null;
	    // NOTE: Delete query and hash parts. These parts are not related to the scope (GH-2524)
	    var partAfterHostWithoutQueryAndHash = getPathname(parsedUrl.partAfterHost);
	    return partAfterHostWithoutQueryAndHash.replace(SCOPE_RE, '/') || '/';
	}
	function getAjaxProxyUrl(url, credentials) {
	    var isCrossDomain = !sameOriginCheck(getLocation(), url);
	    var opts = { resourceType: stringifyResourceType({ isAjax: true }), credentials: credentials };
	    if (isCrossDomain) {
	        opts.proxyPort = settings.get().crossDomainProxyPort;
	        opts.reqOrigin = getOriginHeader();
	    }
	    return getProxyUrl(url, opts);
	}

	var urlUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		DEFAULT_PROXY_SETTINGS: DEFAULT_PROXY_SETTINGS,
		REQUEST_DESCRIPTOR_VALUES_SEPARATOR: REQUEST_DESCRIPTOR_VALUES_SEPARATOR,
		get getProxyUrl () { return getProxyUrl; },
		overrideGetProxyUrl: overrideGetProxyUrl,
		getNavigationUrl: getNavigationUrl,
		get getCrossDomainIframeProxyUrl () { return getCrossDomainIframeProxyUrl; },
		overrideGetCrossDomainIframeProxyUrl: overrideGetCrossDomainIframeProxyUrl,
		getPageProxyUrl: getPageProxyUrl,
		getCrossDomainProxyPort: getCrossDomainProxyPort,
		get resolveUrlAsDest () { return resolveUrlAsDest; },
		overrideResolveUrlAsDest: overrideResolveUrlAsDest,
		formatUrl: formatUrl,
		get parseProxyUrl () { return parseProxyUrl; },
		overrideParseProxyUrl: overrideParseProxyUrl,
		parseUrl: parseUrl,
		get convertToProxyUrl () { return convertToProxyUrl; },
		overrideConvertToProxyUrl: overrideConvertToProxyUrl,
		changeDestUrlPart: changeDestUrlPart,
		isValidWebSocketUrl: isValidWebSocketUrl,
		isSubDomain: isSubDomain,
		isSupportedProtocol: isSupportedProtocol,
		isSpecialPage: isSpecialPage,
		parseResourceType: parseResourceType,
		stringifyResourceType: stringifyResourceType,
		isChangedOnlyHash: isChangedOnlyHash,
		getDestinationUrl: getDestinationUrl,
		getScope: getScope,
		getAjaxProxyUrl: getAjaxProxyUrl
	});

	var emptyActionAttrFallbacksToTheLocation = false;
	var instanceAndPrototypeToStringAreEqual = false;
	var hasTouchEvents = false;
	var hasTouchPoints = false;
	var isTouchDevice = false;
	var hasDataTransfer = false;
	var attrGetNamedItemIsNotEnumerable = false;
	var getElementsByNameReturnsHTMLCollection = false;
	if (nativeMethods.createElement) {
	    var form = nativeMethods.createElement.call(document, 'form');
	    var elements = nativeMethods.getElementsByName.call(document, '');
	    // NOTE: In some browsers, elements without the url attribute return the location url
	    // when accessing this attribute directly. See form.action in Edge 25 as an example.
	    emptyActionAttrFallbacksToTheLocation = nativeMethods.formActionGetter.call(form) === window.location.toString();
	    // NOTE: In Chrome, toString(window) equals '[object Window]' and toString(Window.prototype) equals '[object Blob]',
	    // this condition is also satisfied for Blob, Document, XMLHttpRequest, etc
	    instanceAndPrototypeToStringAreEqual = nativeMethods.objectToString.call(window) ===
	        nativeMethods.objectToString.call(Window.prototype);
	    hasTouchEvents = 'ontouchstart' in window;
	    // NOTE: We need to check touch points only for IE, because it has PointerEvent and MSPointerEvent (IE10, IE11)
	    // instead of TouchEvent (T109295).
	    hasTouchPoints = isIE && (navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0);
	    isTouchDevice = (isMobile || isTablet) && hasTouchEvents;
	    // @ts-ignore
	    hasDataTransfer = !!window.DataTransfer;
	    // NOTE: In the Edge 17, the getNamedItem method of attributes object is not enumerable
	    attrGetNamedItemIsNotEnumerable = !!nativeMethods.objectGetOwnPropertyDescriptor.call(window.Object, NamedNodeMap.prototype, 'getNamedItem');
	    // Both IE and Edge return an HTMLCollection, not a NodeList
	    // @ts-ignore
	    getElementsByNameReturnsHTMLCollection = nativeMethods.objectGetPrototypeOf.call(window.Object, elements) ===
	        nativeMethods.HTMLCollection.prototype;
	}

	var featureDetection = /*#__PURE__*/Object.freeze({
		__proto__: null,
		get emptyActionAttrFallbacksToTheLocation () { return emptyActionAttrFallbacksToTheLocation; },
		get instanceAndPrototypeToStringAreEqual () { return instanceAndPrototypeToStringAreEqual; },
		get hasTouchEvents () { return hasTouchEvents; },
		get hasTouchPoints () { return hasTouchPoints; },
		get isTouchDevice () { return isTouchDevice; },
		get hasDataTransfer () { return hasDataTransfer; },
		get attrGetNamedItemIsNotEnumerable () { return attrGetNamedItemIsNotEnumerable; },
		get getElementsByNameReturnsHTMLCollection () { return getElementsByNameReturnsHTMLCollection; }
	});

	// NOTE: For Chrome.
	var MIN_SELECT_SIZE_VALUE = 4;
	function getIntValue(value) {
	    value = value || '';
	    var parsedValue = parseInt(value.replace('px', ''), 10);
	    return isNaN(parsedValue) ? 0 : parsedValue;
	}
	function get$1(el, property, doc, win) {
	    el = el.documentElement || el;
	    var computedStyle = getComputedStyle(el, doc, win);
	    return computedStyle && computedStyle[property];
	}
	function set(el, property, value) {
	    el = el.documentElement || el;
	    el.style[property] = value;
	}
	function getBordersWidth(el) {
	    return {
	        bottom: getIntValue(get$1(el, 'borderBottomWidth')),
	        left: getIntValue(get$1(el, 'borderLeftWidth')),
	        right: getIntValue(get$1(el, 'borderRightWidth')),
	        top: getIntValue(get$1(el, 'borderTopWidth'))
	    };
	}
	function getComputedStyle(el, doc, win) {
	    // NOTE: In Firefox, after calling the 'document.write' function for nested iframes with html src value
	    // document.defaultView equals 'null'. But 'window.document' equals 'document'.
	    // This is why, we are forced to calculate the targetWindow instead of use document.defaultView.
	    doc = doc || document;
	    win = win || window;
	    var targetWin = doc.defaultView || win;
	    return targetWin.getComputedStyle(el, null);
	}
	function getElementMargin(el) {
	    return {
	        bottom: getIntValue(get$1(el, 'marginBottom')),
	        left: getIntValue(get$1(el, 'marginLeft')),
	        right: getIntValue(get$1(el, 'marginRight')),
	        top: getIntValue(get$1(el, 'marginTop'))
	    };
	}
	function getElementPadding(el) {
	    return {
	        bottom: getIntValue(get$1(el, 'paddingBottom')),
	        left: getIntValue(get$1(el, 'paddingLeft')),
	        right: getIntValue(get$1(el, 'paddingRight')),
	        top: getIntValue(get$1(el, 'paddingTop'))
	    };
	}
	function getElementScroll(el) {
	    var isHtmlElement$1 = isHtmlElement(el);
	    var currentWindow = window;
	    if (isHtmlElement$1 && isElementInIframe(el)) {
	        var currentIframe = getIframeByElement(el);
	        if (currentIframe)
	            currentWindow = nativeMethods.contentWindowGetter.call(currentIframe);
	    }
	    var targetEl = isHtmlElement$1 ? currentWindow : el;
	    return {
	        left: getScrollLeft(targetEl),
	        top: getScrollTop(targetEl)
	    };
	}
	function getWidth(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.document.documentElement.clientWidth;
	    if (isDocument(el)) {
	        var doc = el.documentElement;
	        var clientProp = 'clientWidth';
	        var scrollProp = 'scrollWidth';
	        var offsetProp = 'offsetWidth';
	        if (doc[clientProp] >= doc[scrollProp])
	            return doc[clientProp];
	        return Math.max(el.body[scrollProp], doc[scrollProp], el.body[offsetProp], doc[offsetProp]);
	    }
	    var value = el.offsetWidth;
	    value -= getIntValue(get$1(el, 'paddingLeft'));
	    value -= getIntValue(get$1(el, 'paddingRight'));
	    value -= getIntValue(get$1(el, 'borderLeftWidth'));
	    value -= getIntValue(get$1(el, 'borderRightWidth'));
	    return value;
	}
	function getHeight(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.document.documentElement.clientHeight;
	    if (isDocument(el)) {
	        var doc = el.documentElement;
	        var clientProp = 'clientHeight';
	        var scrollProp = 'scrollHeight';
	        var offsetProp = 'offsetHeight';
	        if (doc[clientProp] >= doc[scrollProp])
	            return doc[clientProp];
	        return Math.max(el.body[scrollProp], doc[scrollProp], el.body[offsetProp], doc[offsetProp]);
	    }
	    var value = el.offsetHeight;
	    value -= getIntValue(get$1(el, 'paddingTop'));
	    value -= getIntValue(get$1(el, 'paddingBottom'));
	    value -= getIntValue(get$1(el, 'borderTopWidth'));
	    value -= getIntValue(get$1(el, 'borderBottomWidth'));
	    return value;
	}
	function getInnerWidth(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.document.documentElement.clientWidth;
	    if (isDocument(el))
	        return el.documentElement.clientWidth;
	    var value = el.offsetWidth;
	    value -= getIntValue(get$1(el, 'borderLeftWidth'));
	    value -= getIntValue(get$1(el, 'borderRightWidth'));
	    return value;
	}
	function getInnerHeight(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.document.documentElement.clientHeight;
	    if (isDocument(el))
	        return el.documentElement.clientHeight;
	    var value = el.offsetHeight;
	    value -= getIntValue(get$1(el, 'borderTopWidth'));
	    value -= getIntValue(get$1(el, 'borderBottomWidth'));
	    return value;
	}
	function getOptionHeight(select) {
	    var realSizeValue = getSelectElementSize(select);
	    var selectPadding = getElementPadding(select);
	    var selectScrollHeight = select.scrollHeight - (selectPadding.top + selectPadding.bottom);
	    var childrenCount = getSelectVisibleChildren(select).length;
	    if (realSizeValue === 1)
	        return getHeight(select);
	    return isIE && realSizeValue > childrenCount
	        ? Math.round(selectScrollHeight / childrenCount)
	        : Math.round(selectScrollHeight / Math.max(childrenCount, realSizeValue));
	}
	function getSelectElementSize(select) {
	    // NOTE: iOS and Android ignore 'size' and 'multiple' attributes,
	    // all select elements behave like a select with size=1.
	    if (isSafari && hasTouchEvents || isAndroid)
	        return 1;
	    var sizeAttr = nativeMethods.getAttribute.call(select, 'size');
	    var multipleAttr = nativeMethods.getAttribute.call(select, 'multiple');
	    var size = !sizeAttr ? 1 : parseInt(sizeAttr, 10);
	    if (multipleAttr && (!sizeAttr || size < 1))
	        size = MIN_SELECT_SIZE_VALUE;
	    return size;
	}
	function isVisibleChild(el) {
	    var select = getSelectParent(el);
	    var tagName = getTagName(el);
	    return isSelectElement(select) && getSelectElementSize(select) > 1 &&
	        (tagName === 'option' || tagName === 'optgroup') &&
	        // NOTE: Firefox does not display groups without a label or with an empty label.
	        (!isFirefox || el.label);
	}
	function getScrollLeft(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.pageXOffset;
	    if (isDocument(el))
	        return el.defaultView.pageXOffset;
	    return el.scrollLeft;
	}
	function getScrollTop(el) {
	    if (!el)
	        return null;
	    if (isWindow(el))
	        return el.pageYOffset;
	    if (isDocument(el))
	        return el.defaultView.pageYOffset;
	    return el.scrollTop;
	}
	function setScrollLeft(el, value) {
	    if (!el)
	        return;
	    if (isWindow(el) || isDocument(el)) {
	        var win = findDocument(el).defaultView;
	        var scrollTop = getScrollTop(el);
	        nativeMethods.scrollTo.call(win, value, scrollTop);
	    }
	    else
	        el.scrollLeft = value;
	}
	function setScrollTop(el, value) {
	    if (!el)
	        return;
	    if (isWindow(el) || isDocument(el)) {
	        var win = findDocument(el).defaultView;
	        var scrollLeft = getScrollLeft(el);
	        nativeMethods.scrollTo.call(win, scrollLeft, value);
	    }
	    else
	        el.scrollTop = value;
	}
	function getOffsetParent(el) {
	    if (el) {
	        var offsetParent = el.offsetParent || document.body;
	        while (offsetParent && (!/^(?:body|html)$/i.test(offsetParent.nodeName) &&
	            get$1(offsetParent, 'position') === 'static'))
	            offsetParent = offsetParent.offsetParent;
	        return offsetParent;
	    }
	    return void 0;
	}
	function getOffset(el) {
	    if (!el || isWindow(el) || isDocument(el))
	        return null;
	    var clientRect = el.getBoundingClientRect();
	    // NOTE: A detached node or documentElement.
	    var doc = el.ownerDocument;
	    var docElement = doc.documentElement;
	    if (!docElement.contains(el) || el === docElement) {
	        return {
	            top: clientRect.top,
	            left: clientRect.left
	        };
	    }
	    var win = doc.defaultView;
	    var clientTop = docElement.clientTop || doc.body.clientTop || 0;
	    var clientLeft = docElement.clientLeft || doc.body.clientLeft || 0;
	    var scrollTop = win.pageYOffset || docElement.scrollTop || doc.body.scrollTop;
	    var scrollLeft = win.pageXOffset || docElement.scrollLeft || doc.body.scrollLeft;
	    clientRect = el.getBoundingClientRect();
	    return {
	        top: clientRect.top + scrollTop - clientTop,
	        left: clientRect.left + scrollLeft - clientLeft
	    };
	}
	function isElementVisible(el, doc) {
	    if (!isElementInDocument(el, doc))
	        return false;
	    while (el) {
	        if (get$1(el, 'display', doc) === 'none' || get$1(el, 'visibility', doc) === 'hidden')
	            return false;
	        el = getParentExceptShadowRoot(el);
	    }
	    return true;
	}
	function isElementInInvisibleIframe(el) {
	    var frameElement = getIframeByElement(el);
	    return frameElement && !isElementVisible(frameElement, findDocument(frameElement));
	}

	var styleUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		get: get$1,
		set: set,
		getBordersWidth: getBordersWidth,
		getComputedStyle: getComputedStyle,
		getElementMargin: getElementMargin,
		getElementPadding: getElementPadding,
		getElementScroll: getElementScroll,
		getWidth: getWidth,
		getHeight: getHeight,
		getInnerWidth: getInnerWidth,
		getInnerHeight: getInnerHeight,
		getOptionHeight: getOptionHeight,
		getSelectElementSize: getSelectElementSize,
		isVisibleChild: isVisibleChild,
		getScrollLeft: getScrollLeft,
		getScrollTop: getScrollTop,
		setScrollLeft: setScrollLeft,
		setScrollTop: setScrollTop,
		getOffsetParent: getOffsetParent,
		getOffset: getOffset,
		isElementVisible: isElementVisible,
		isElementInInvisibleIframe: isElementInInvisibleIframe
	});

	function getNativeQuerySelector(el) {
	    if (isDomElement(el))
	        return nativeMethods.elementQuerySelector;
	    return isDocumentFragmentNode(el) || isShadowRoot(el)
	        ? nativeMethods.documentFragmentQuerySelector
	        : nativeMethods.querySelector;
	}
	function getNativeQuerySelectorAll(el) {
	    // NOTE: Do not return the isDocument function instead of the isDomElement
	    // it leads to the `Invalid calling object` error in some cases in IE11 (GH-1846)
	    if (isDomElement(el))
	        return nativeMethods.elementQuerySelectorAll;
	    return isDocumentFragmentNode(el) || isShadowRoot(el)
	        ? nativeMethods.documentFragmentQuerySelectorAll
	        : nativeMethods.querySelectorAll;
	}

	var scrollbarSize = 0;
	var NATIVE_MAP_ELEMENT_STRINGS = [
	    '[object HTMLMapElement]',
	    '[object HTMLAreaElement]'
	];
	var WINDOW_IS_UNDEFINED = typeof window === 'undefined';
	var NATIVE_WINDOW_STR = WINDOW_IS_UNDEFINED ? '' : instanceToString(window);
	var IS_DOCUMENT_RE = /^\[object .*?Document]$/i;
	var IS_PROCESSING_INSTRUCTION_RE = /^\[object .*?ProcessingInstruction]$/i;
	var IS_SVG_ELEMENT_RE = /^\[object SVG\w+?Element]$/i;
	var IS_HTML_ELEMENT_RE = /^\[object HTML.*?Element]$/i;
	var IS_ARRAY_BUFFER_RE = /^\[object ArrayBuffer]$/i;
	var IS_DATA_VIEW_RE = /^\[object DataView]$/i;
	var NATIVE_TABLE_CELL_STR = WINDOW_IS_UNDEFINED ? '' : instanceToString(nativeMethods.createElement.call(document, 'td'));
	var ELEMENT_NODE_TYPE = WINDOW_IS_UNDEFINED ? -1 : Node.ELEMENT_NODE;
	var NOT_CONTENT_EDITABLE_ELEMENTS_RE = /^(select|option|applet|area|audio|canvas|datalist|keygen|map|meter|object|progress|source|track|video|img)$/;
	var INPUT_ELEMENTS_RE = /^(input|textarea|button)$/;
	var SCRIPT_OR_STYLE_RE = /^(script|style)$/i;
	var EDITABLE_INPUT_TYPES_RE = /^(email|number|password|search|tel|text|url)$/;
	var NUMBER_OR_EMAIL_INPUT_RE = /^(number|email)$/;
	// NOTE: input with 'file' type processed separately in 'UploadSandbox'
	var INPUT_WITH_NATIVE_DIALOG = /^(color|date|datetime-local|month|week)$/;
	// NOTE: We don't take into account the case of embedded contentEditable elements, and we
	// specify the contentEditable attribute for focusable elements.
	var FOCUSABLE_SELECTOR = 'input, select, textarea, button, body, iframe, [contenteditable="true"], [contenteditable=""], [tabIndex]';
	function isHidden(el) {
	    return el.offsetWidth <= 0 && el.offsetHeight <= 0;
	}
	function isAlwaysNotEditableElement(el) {
	    var tagName = getTagName(el);
	    return !!tagName && (NOT_CONTENT_EDITABLE_ELEMENTS_RE.test(tagName) || INPUT_ELEMENTS_RE.test(tagName));
	}
	function isLocationByProto(instance) {
	    var instanceCtor = null;
	    try {
	        // eslint-disable-next-line no-proto
	        instanceCtor = instance.__proto__;
	    }
	    catch (e) {
	        // NOTE: Try to detect cross-domain window location.
	        // A cross-domain location has no the "assign" function in Safari and Chrome.
	        var shouldNotHaveAssign = isSafari || isChrome;
	        return instance.replace && (shouldNotHaveAssign || !!instance.assign);
	    }
	    if (!instanceCtor)
	        return false;
	    var stringifiedInstanceCtor = nativeMethods.objectToString.call(instanceCtor);
	    return stringifiedInstanceCtor === '[object LocationPrototype]' ||
	        stringifiedInstanceCtor === '[object Location]'; // NOTE: "iPhone" Chrome device emulation case (GH-2080)
	}
	function closestFallback(el, selector) {
	    while (el) {
	        if (matches(el, selector))
	            return el;
	        el = nativeMethods.nodeParentNodeGetter.call(el);
	    }
	    return null;
	}
	function instanceToString(instance) {
	    if (!instanceAndPrototypeToStringAreEqual)
	        return nativeMethods.objectToString.call(instance);
	    return instance && typeof instance === 'object'
	        ? nativeMethods.objectToString.call(nativeMethods.objectGetPrototypeOf(instance))
	        : '';
	}
	function getActiveElement(currentDocument) {
	    // NOTE: Sometimes document.activeElement returns an empty object or null (IE11).
	    // https://github.com/DevExpress/testcafe-hammerhead/issues/768
	    var doc = currentDocument || document;
	    var activeElement = nativeMethods.documentActiveElementGetter.call(doc);
	    var el = isDomElement(activeElement) ? activeElement : doc.body;
	    while (el && el.shadowRoot) {
	        // eslint-disable-next-line no-restricted-properties
	        var shadowEl = el.shadowRoot.activeElement;
	        if (!shadowEl)
	            break;
	        el = shadowEl;
	    }
	    return el;
	}
	function getChildVisibleIndex(select, child) {
	    var childrenArray = getSelectVisibleChildren(select);
	    return childrenArray.indexOf(child);
	}
	function getIframeByElement(el) {
	    var elWindow = el[INTERNAL_PROPS.processedContext];
	    return getFrameElement(elWindow);
	}
	function getIframeLocation(iframe) {
	    var documentLocation = null;
	    try {
	        // eslint-disable-next-line no-restricted-properties
	        documentLocation = nativeMethods.contentDocumentGetter.call(iframe).location.href;
	    }
	    catch (e) {
	        documentLocation = null;
	    }
	    var srcLocation = nativeMethods.getAttribute.call(iframe, 'src' + INTERNAL_ATTRIBUTES.storedAttrPostfix) ||
	        nativeMethods.getAttribute.call(iframe, 'src') || nativeMethods.iframeSrcGetter.call(iframe);
	    var parsedProxyDocumentLocation = documentLocation && isSupportedProtocol(documentLocation) &&
	        parseProxyUrl(documentLocation);
	    var parsedProxySrcLocation = srcLocation && isSupportedProtocol(srcLocation) &&
	        parseProxyUrl(srcLocation);
	    return {
	        documentLocation: parsedProxyDocumentLocation ? parsedProxyDocumentLocation.destUrl : documentLocation,
	        srcLocation: parsedProxySrcLocation ? parsedProxySrcLocation.destUrl : srcLocation
	    };
	}
	function getFrameElement(win) {
	    try {
	        return win.frameElement;
	    }
	    catch (e) {
	        return null;
	    }
	}
	function getMapContainer(el) {
	    var closestMap = closest(el, 'map');
	    var closestMapName = nativeMethods.getAttribute.call(closestMap, 'name');
	    var containerSelector = '[usemap="#' + closestMapName + '"]';
	    return nativeMethods.querySelector.call(findDocument(el), containerSelector);
	}
	function getParentWindowWithSrc(window) {
	    var parent = window.parent;
	    var parentFrameElement = null;
	    if (window === window.top)
	        return window;
	    if (parent === window.top || isCrossDomainWindows(window, parent))
	        return parent;
	    try {
	        parentFrameElement = parent.frameElement;
	    }
	    catch (e) {
	        parentFrameElement = null;
	    }
	    if (parentFrameElement === null || !isIframeWithoutSrc(parentFrameElement))
	        return parent;
	    return getParentWindowWithSrc(parent);
	}
	function getScrollbarSize() {
	    if (!scrollbarSize) {
	        var scrollDiv = nativeMethods.createElement.call(document, 'div');
	        scrollDiv.style.height = '100px';
	        scrollDiv.style.overflow = 'scroll';
	        scrollDiv.style.position = 'absolute';
	        scrollDiv.style.top = '-9999px';
	        scrollDiv.style.width = '100px';
	        nativeMethods.appendChild.call(document.body, scrollDiv);
	        var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
	        scrollbarSize = scrollbarWidth;
	        var parent_1 = nativeMethods.nodeParentNodeGetter.call(scrollDiv);
	        parent_1.removeChild(scrollDiv);
	    }
	    return scrollbarSize;
	}
	function getSelectParent(child) {
	    var parent = nativeMethods.nodeParentNodeGetter.call(child);
	    return closest(parent, 'select');
	}
	function getSelectVisibleChildren(select) {
	    var children = nativeMethods.elementQuerySelectorAll.call(select, 'optgroup, option');
	    var result = [];
	    var length = nativeMethods.nodeListLengthGetter.call(children);
	    for (var i = 0; i < length; i++) {
	        var child = children[i];
	        // NOTE: Firefox does not display groups without a label and with an empty label.
	        var shouldAdd = isFirefox ? getTagName(child) !== 'optgroup' || child.label : true;
	        if (shouldAdd)
	            result.push(child);
	    }
	    return result;
	}
	function getTopSameDomainWindow(window) {
	    var result = window;
	    var currentWindow = window.parent;
	    if (result === window.top)
	        return result;
	    while (currentWindow) {
	        if (!isCrossDomainWindows(window, currentWindow)) {
	            var frameElement_1 = getFrameElement(currentWindow);
	            if (!frameElement_1 || !isIframeWithoutSrc(frameElement_1))
	                result = currentWindow;
	        }
	        currentWindow = currentWindow !== window.top ? currentWindow.parent : null;
	    }
	    return result;
	}
	function find(parent, selector, handler) {
	    var nodeList = getNativeQuerySelectorAll(parent).call(parent, selector);
	    if (handler) {
	        var length_1 = nativeMethods.nodeListLengthGetter.call(nodeList);
	        for (var i = 0; i < length_1; i++)
	            handler(nodeList[i]);
	    }
	    return nodeList;
	}
	function findDocument(el) {
	    if (el.documentElement)
	        return el;
	    if (el.ownerDocument && el.ownerDocument.defaultView)
	        return el.ownerDocument;
	    var parent = isElementNode(el) && nativeMethods.nodeParentNodeGetter.call(el);
	    return parent ? findDocument(parent) : document;
	}
	function isContentEditableElement(el) {
	    var isContentEditable = false;
	    var element = null;
	    if (isTextNode(el))
	        element = el.parentElement || nativeMethods.nodeParentNodeGetter.call(el);
	    else
	        element = el;
	    if (element) {
	        isContentEditable = element.isContentEditable && !isAlwaysNotEditableElement(element);
	        return isRenderedNode(element) && (isContentEditable || findDocument(el).designMode === 'on');
	    }
	    return false;
	}
	function isCrossDomainIframe(iframe, bySrc) {
	    var iframeLocation = getIframeLocation(iframe);
	    if (!bySrc && iframeLocation.documentLocation === null)
	        return true;
	    var currentLocation = bySrc ? iframeLocation.srcLocation : iframeLocation.documentLocation;
	    if (currentLocation && isSupportedProtocol(currentLocation))
	        return !sameOriginCheck(location.toString(), currentLocation);
	    return false;
	}
	function isCrossDomainWindows(window1, window2) {
	    try {
	        if (window1 === window2)
	            return false;
	        var window1Location = window1.location.toString();
	        var window2Location = window2.location.toString();
	        if (!isSupportedProtocol(window1Location) || !isSupportedProtocol(window2Location))
	            return false;
	        return !sameOriginCheck(window1Location, window2Location);
	    }
	    catch (e) {
	        return true;
	    }
	}
	function isIframeWindow(wnd) {
	    return wnd !== wnd.top;
	}
	function isDomElement(el) {
	    if (el instanceof nativeMethods.elementClass)
	        return true;
	    return el && IS_HTML_ELEMENT_RE.test(instanceToString(el)) && isElementNode(el) && el.tagName;
	}
	function getTagName(el) {
	    // NOTE: Check for tagName being a string, because it may be a function in an Angular app (T175340).
	    return el && typeof el.tagName === 'string' ? el.tagName.toLowerCase() : '';
	}
	var SHADOW_ROOT_PARENT_ELEMENT = 'hammerhead|element|shadow-root-parent';
	function getNodeShadowRootParent(el) {
	    var parent = nativeMethods.nodeParentNodeGetter.call(el);
	    while (parent && parent.nodeType !== Node.DOCUMENT_FRAGMENT_NODE)
	        parent = nativeMethods.nodeParentNodeGetter.call(parent);
	    return parent && parent[SHADOW_ROOT_PARENT_ELEMENT];
	}
	function getParentExceptShadowRoot(el) {
	    var parent = nativeMethods.nodeParentNodeGetter.call(el);
	    return parent && parent.nodeType === Node.DOCUMENT_FRAGMENT_NODE && parent[SHADOW_ROOT_PARENT_ELEMENT]
	        ? parent[SHADOW_ROOT_PARENT_ELEMENT]
	        : parent;
	}
	function isElementInDocument(el, currentDocument) {
	    var doc = currentDocument || document;
	    if (!doc.documentElement)
	        return false;
	    if (doc.documentElement.contains(el))
	        return true;
	    var shadowRootParent = getNodeShadowRootParent(el);
	    return shadowRootParent ? isElementInDocument(shadowRootParent) : false;
	}
	function isElementInIframe(el, currentDocument) {
	    var doc = currentDocument || findDocument(el);
	    return window.document !== doc;
	}
	function isHammerheadAttr(attr) {
	    return attr === INTERNAL_ATTRIBUTES.focusPseudoClass || attr === INTERNAL_ATTRIBUTES.hoverPseudoClass ||
	        attr.indexOf(INTERNAL_ATTRIBUTES.storedAttrPostfix) !== -1;
	}
	function isIframeElement(el) {
	    return instanceToString(el) === '[object HTMLIFrameElement]';
	}
	function isFrameElement(el) {
	    return instanceToString(el) === '[object HTMLFrameElement]';
	}
	function isIframeWithoutSrc(iframe) {
	    var iframeLocation = getIframeLocation(iframe);
	    var iframeSrcLocation = iframeLocation.srcLocation;
	    var iframeDocumentLocation = iframeLocation.documentLocation;
	    // NOTE: is a cross-domain iframe
	    if (iframeDocumentLocation === null)
	        return false;
	    // NOTE: after 'document.write' or 'document.open' call for iframe with/without src
	    // we will process it as iframe without src
	    if (nativeMethods.contentWindowGetter.call(iframe)[INTERNAL_PROPS.documentWasCleaned])
	        return true;
	    var iframeDocumentLocationHaveSupportedProtocol = isSupportedProtocol(iframeDocumentLocation);
	    // NOTE: When an iframe has an empty src attribute (<iframe src></iframe>) or has no src attribute (<iframe></iframe>),
	    // the iframe.src property is not empty but has different values in different browsers.
	    // Its document location is 'about:blank'. Therefore, we should check the src attribute.
	    if (!iframeDocumentLocationHaveSupportedProtocol && !nativeMethods.getAttribute.call(iframe, 'src'))
	        return true;
	    // In Chrome, when an iframe with the src attribute is added to DOM,
	    // its documentLocation is set to "about:blank" until the iframe has been loaded.
	    // So, we should check srcLocation in this case.
	    if (iframeSrcLocation && isSupportedProtocol(iframeSrcLocation))
	        return false;
	    return !iframeDocumentLocationHaveSupportedProtocol;
	}
	function isIframeWithSrcdoc(iframe) {
	    return nativeMethods.iframeSrcdocGetter && nativeMethods.hasAttribute.call(iframe, 'srcdoc');
	}
	function isImgElement(el) {
	    return instanceToString(el) === '[object HTMLImageElement]';
	}
	function isInputElement(el) {
	    return instanceToString(el) === '[object HTMLInputElement]';
	}
	function isTitleElement(el) {
	    return instanceToString(el) === '[object HTMLTitleElement]';
	}
	function isButtonElement(el) {
	    return instanceToString(el) === '[object HTMLButtonElement]';
	}
	function isFieldSetElement(el) {
	    return instanceToString(el) === '[object HTMLFieldSetElement]';
	}
	function isOptGroupElement(el) {
	    return instanceToString(el) === '[object HTMLOptGroupElement]';
	}
	function isHtmlElement(el) {
	    return instanceToString(el) === '[object HTMLHtmlElement]';
	}
	function isBodyElement(el) {
	    return instanceToString(el) === '[object HTMLBodyElement]';
	}
	function isHeadElement(el) {
	    return instanceToString(el) === '[object HTMLHeadElement]';
	}
	function isHeadOrBodyElement(el) {
	    var elString = instanceToString(el);
	    return elString === '[object HTMLHeadElement]' || elString === '[object HTMLBodyElement]';
	}
	function isHeadOrBodyOrHtmlElement(el) {
	    var elString = instanceToString(el);
	    return elString === '[object HTMLHeadElement]' || elString === '[object HTMLBodyElement]' ||
	        elString === '[object HTMLHtmlElement]';
	}
	function isBaseElement(el) {
	    return instanceToString(el) === '[object HTMLBaseElement]';
	}
	function isScriptElement(el) {
	    return instanceToString(el) === '[object HTMLScriptElement]';
	}
	function isStyleElement(el) {
	    return instanceToString(el) === '[object HTMLStyleElement]';
	}
	function isLabelElement(el) {
	    return instanceToString(el) === '[object HTMLLabelElement]';
	}
	function isTextAreaElement(el) {
	    return instanceToString(el) === '[object HTMLTextAreaElement]';
	}
	function isOptionElement(el) {
	    return instanceToString(el) === '[object HTMLOptionElement]';
	}
	function isRadioButtonElement(el) {
	    return isInputElement(el) && el.type.toLowerCase() === 'radio';
	}
	function isColorInputElement(el) {
	    return isInputElement(el) && el.type.toLowerCase() === 'color';
	}
	function isCheckboxElement(el) {
	    return isInputElement(el) && el.type.toLowerCase() === 'checkbox';
	}
	function isSelectElement(el) {
	    return instanceToString(el) === '[object HTMLSelectElement]';
	}
	function isFormElement(el) {
	    return instanceToString(el) === '[object HTMLFormElement]';
	}
	function isFileInput(el) {
	    return isInputElement(el) && el.type.toLowerCase() === 'file';
	}
	function isInputWithNativeDialog(el) {
	    return isInputElement(el) && INPUT_WITH_NATIVE_DIALOG.test(el.type.toLowerCase());
	}
	function isBodyElementWithChildren(el) {
	    return isBodyElement(el) && nativeMethods.htmlCollectionLengthGetter.call(nativeMethods.elementChildrenGetter.call(el));
	}
	function isMapElement(el) {
	    return NATIVE_MAP_ELEMENT_STRINGS.indexOf(instanceToString(el)) !== -1;
	}
	function isRenderedNode(node) {
	    return !(isProcessingInstructionNode(node) || isCommentNode(node) || SCRIPT_OR_STYLE_RE.test(node.nodeName));
	}
	function getTabIndex(el) {
	    // NOTE: we obtain the tabIndex value from an attribute because the el.tabIndex
	    // property returns -1 for some elements (e.g. for body) with no tabIndex assigned
	    var tabIndex = nativeMethods.getAttribute.call(el, 'tabIndex');
	    tabIndex = parseInt(tabIndex, 10);
	    return isNaN(tabIndex) ? null : tabIndex;
	}
	function isElementFocusable(el) {
	    if (!el)
	        return false;
	    var tabIndex = getTabIndex(el);
	    var isDisabledElement = matches(el, ':disabled');
	    var isInvisibleElement = get$1(el, 'visibility') === 'hidden';
	    var isNotDisplayedElement = get$1(el, 'display') === 'none';
	    var isHiddenElement = isWebKit ? isHidden(el) && !isOptionElement(el) : isHidden(el);
	    if (isDisabledElement || isInvisibleElement || isNotDisplayedElement || isHiddenElement)
	        return false;
	    if (isOptionElement(el) && isIE)
	        return false;
	    if (isAnchorElement(el)) {
	        if (tabIndex !== null)
	            return true;
	        return matches(el, 'a[href]');
	    }
	    if (isTableDataCellElement(el) && isIE)
	        return true;
	    return matches(el, FOCUSABLE_SELECTOR) || tabIndex !== null;
	}
	function isShadowUIElement(element) {
	    // @ts-ignore
	    return !!element[INTERNAL_PROPS.shadowUIElement];
	}
	function isWindow(instance) {
	    try {
	        // NOTE: The instanceToString call result has a strange values for the MessageEvent.target property:
	        // * [object DispHTMLWindow2] for IE11
	        // * [object Object] for MSEdge.
	        if ((isIE || isMSEdge) && instance && instance === instance.window)
	            instance = instance.window;
	        if (!instance || !instance.toString || NATIVE_WINDOW_STR !== instanceToString(instance))
	            return false;
	    }
	    catch (e) {
	        try {
	            // NOTE: If a cross-domain object has the 'top' field, this object is a window
	            // (not a document or location).
	            return !!instance.top;
	        }
	        catch (_a) {
	            return false;
	        }
	    }
	    try {
	        nativeMethods.winLocalStorageGetter.call(instance);
	    }
	    catch (_b) {
	        return false;
	    }
	    return true;
	}
	function isDocument(instance) {
	    if (instance instanceof nativeMethods.documentClass)
	        return true;
	    try {
	        return instance && IS_DOCUMENT_RE.test(instanceToString(instance));
	    }
	    catch (e) {
	        // NOTE: For cross-domain objects (windows, documents or locations), we return false because
	        // it's impossible to work with them in any case.
	        return false;
	    }
	}
	function isBlob(instance) {
	    return instance && instanceToString(instance) === '[object Blob]';
	}
	function isLocation(instance) {
	    if (!instance)
	        return false;
	    if (isIE || isSafari || isChrome)
	        return isLocationByProto(instance);
	    return instance instanceof nativeMethods.locationClass ||
	        nativeMethods.objectToString.call(instance) === '[object Location]';
	}
	function isSVGElement(instance) {
	    if (instance instanceof nativeMethods.svgElementClass)
	        return true;
	    return instance && IS_SVG_ELEMENT_RE.test(instanceToString(instance));
	}
	function isSVGElementOrChild(el) {
	    return !!closest(el, 'svg');
	}
	function isFetchHeaders(instance) {
	    if (nativeMethods.Headers && instance instanceof nativeMethods.Headers)
	        return true;
	    return instance && instanceToString(instance) === '[object Headers]';
	}
	function isFetchRequest(instance) {
	    if (nativeMethods.Request && instance instanceof nativeMethods.Request)
	        return true;
	    return instance && instanceToString(instance) === '[object Request]';
	}
	function isElementReadOnly(el) {
	    return el.readOnly || el.getAttribute('readonly') === 'readonly';
	}
	function isTextEditableInput(el) {
	    var attrType = el.getAttribute('type');
	    return isInputElement(el) &&
	        attrType ? EDITABLE_INPUT_TYPES_RE.test(attrType) : EDITABLE_INPUT_TYPES_RE.test(el.type);
	}
	function isTextEditableElement(el) {
	    return isTextEditableInput(el) || isTextAreaElement(el);
	}
	function isTextEditableElementAndEditingAllowed(el) {
	    return isTextEditableElement(el) && !isElementReadOnly(el);
	}
	function isElementNode(node) {
	    return node && node.nodeType === ELEMENT_NODE_TYPE;
	}
	function isTextNode(node) {
	    return instanceToString(node) === '[object Text]';
	}
	function isProcessingInstructionNode(node) {
	    return IS_PROCESSING_INSTRUCTION_RE.test(instanceToString(node));
	}
	function isCommentNode(node) {
	    return instanceToString(node) === '[object Comment]';
	}
	function isDocumentFragmentNode(node) {
	    return instanceToString(node) === '[object DocumentFragment]';
	}
	function isShadowRoot(root) {
	    return instanceToString(root) === '[object ShadowRoot]';
	}
	function isAnchorElement(el) {
	    return instanceToString(el) === '[object HTMLAnchorElement]';
	}
	function isTableElement(el) {
	    return instanceToString(el) === '[object HTMLTableElement]';
	}
	function isTableDataCellElement(el) {
	    return instanceToString(el) === NATIVE_TABLE_CELL_STR;
	}
	function isWebSocket(ws) {
	    return instanceToString(ws) === '[object WebSocket]';
	}
	function isMessageEvent(e) {
	    return instanceToString(e) === '[object MessageEvent]';
	}
	function isPerformanceNavigationTiming(entry) {
	    return instanceToString(entry) === '[object PerformanceNavigationTiming]';
	}
	function isArrayBuffer(data) {
	    if (data instanceof nativeMethods.ArrayBuffer)
	        return true;
	    return data && IS_ARRAY_BUFFER_RE.test(instanceToString(data));
	}
	function isArrayBufferView(data) {
	    return data && nativeMethods.arrayBufferIsView(data);
	}
	function isDataView(data) {
	    if (data instanceof nativeMethods.DataView)
	        return true;
	    return data && IS_DATA_VIEW_RE.test(instanceToString(data));
	}
	function matches(el, selector) {
	    if (!isElementNode(el))
	        return false;
	    return nativeMethods.matches.call(el, selector);
	}
	function closest(el, selector) {
	    if (!isElementNode(el))
	        return null;
	    if (nativeMethods.closest)
	        return nativeMethods.closest.call(el, selector);
	    return closestFallback(el, selector);
	}
	function addClass(el, className) {
	    if (!el)
	        return;
	    var classNames = className.split(/\s+/);
	    for (var _i = 0, classNames_1 = classNames; _i < classNames_1.length; _i++) {
	        var currentClassName = classNames_1[_i];
	        nativeMethods.tokenListAdd.call(nativeMethods.elementClassListGetter.call(el), currentClassName);
	    }
	}
	function removeClass(el, className) {
	    if (!el)
	        return;
	    var classNames = className.split(/\s+/);
	    for (var _i = 0, classNames_2 = classNames; _i < classNames_2.length; _i++) {
	        var currentClassName = classNames_2[_i];
	        nativeMethods.tokenListRemove.call(nativeMethods.elementClassListGetter.call(el), currentClassName);
	    }
	}
	function hasClass(el, className) {
	    if (!el)
	        return false;
	    return nativeMethods.tokenListContains.call(nativeMethods.elementClassListGetter.call(el), className);
	}
	function parseDocumentCharset() {
	    var metaCharset = nativeMethods.querySelector.call(document, '.' + SHADOW_UI_CLASS_NAME.charset);
	    return metaCharset && metaCharset.getAttribute('charset');
	}
	function getParents(el, selector) {
	    var parents = [];
	    var parent = getParent(el);
	    while (parent) {
	        if (!selector && isElementNode(parent) || selector && matches(parent, selector))
	            parents.push(parent);
	        parent = getParent(parent);
	    }
	    return parents;
	}
	function getParent(el) {
	    el = el.assignedSlot || el;
	    // eslint-disable-next-line no-restricted-properties
	    return nativeMethods.nodeParentNodeGetter.call(el) || el.host;
	}
	function findParent(node, includeSelf, predicate) {
	    if (includeSelf === void 0) { includeSelf = false; }
	    if (!includeSelf)
	        node = nativeMethods.nodeParentNodeGetter.call(node);
	    while (node) {
	        if (typeof predicate !== 'function' || predicate(node))
	            return node;
	        node = nativeMethods.nodeParentNodeGetter.call(node);
	    }
	    return null;
	}
	function nodeListToArray(nodeList) {
	    var result = [];
	    var length = nativeMethods.nodeListLengthGetter.call(nodeList);
	    for (var i = 0; i < length; i++)
	        result.push(nodeList[i]);
	    return result;
	}
	function getFileInputs(el) {
	    return isFileInput(el) ? [el] : nodeListToArray(getNativeQuerySelectorAll(el).call(el, 'input[type=file]'));
	}
	function getIframes(el) {
	    return isIframeElement(el) ? [el] : nodeListToArray(getNativeQuerySelectorAll(el).call(el, 'iframe,frame'));
	}
	function getScripts(el) {
	    return isScriptElement(el) ? [el] : nodeListToArray(getNativeQuerySelectorAll(el).call(el, 'script'));
	}
	function isNumberOrEmailInput(el) {
	    return isInputElement(el) && NUMBER_OR_EMAIL_INPUT_RE.test(el.type);
	}
	function isInputWithoutSelectionProperties(el) {
	    if (!isNumberOrEmailInput(el))
	        return false;
	    var hasSelectionProperties = typeof el.selectionStart === 'number' && typeof el.selectionEnd === 'number';
	    return !hasSelectionProperties;
	}
	function getAssociatedElement(el) {
	    if (!isLabelElement(el))
	        return null;
	    var doc = findDocument(el);
	    return el.control || el.htmlFor && nativeMethods.getElementById.call(doc, el.htmlFor);
	}

	var domUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		instanceToString: instanceToString,
		getActiveElement: getActiveElement,
		getChildVisibleIndex: getChildVisibleIndex,
		getIframeByElement: getIframeByElement,
		getIframeLocation: getIframeLocation,
		getFrameElement: getFrameElement,
		getMapContainer: getMapContainer,
		getParentWindowWithSrc: getParentWindowWithSrc,
		getScrollbarSize: getScrollbarSize,
		getSelectParent: getSelectParent,
		getSelectVisibleChildren: getSelectVisibleChildren,
		getTopSameDomainWindow: getTopSameDomainWindow,
		find: find,
		findDocument: findDocument,
		isContentEditableElement: isContentEditableElement,
		isCrossDomainIframe: isCrossDomainIframe,
		isCrossDomainWindows: isCrossDomainWindows,
		isIframeWindow: isIframeWindow,
		isDomElement: isDomElement,
		getTagName: getTagName,
		SHADOW_ROOT_PARENT_ELEMENT: SHADOW_ROOT_PARENT_ELEMENT,
		getNodeShadowRootParent: getNodeShadowRootParent,
		getParentExceptShadowRoot: getParentExceptShadowRoot,
		isElementInDocument: isElementInDocument,
		isElementInIframe: isElementInIframe,
		isHammerheadAttr: isHammerheadAttr,
		isIframeElement: isIframeElement,
		isFrameElement: isFrameElement,
		isIframeWithoutSrc: isIframeWithoutSrc,
		isIframeWithSrcdoc: isIframeWithSrcdoc,
		isImgElement: isImgElement,
		isInputElement: isInputElement,
		isTitleElement: isTitleElement,
		isButtonElement: isButtonElement,
		isFieldSetElement: isFieldSetElement,
		isOptGroupElement: isOptGroupElement,
		isHtmlElement: isHtmlElement,
		isBodyElement: isBodyElement,
		isHeadElement: isHeadElement,
		isHeadOrBodyElement: isHeadOrBodyElement,
		isHeadOrBodyOrHtmlElement: isHeadOrBodyOrHtmlElement,
		isBaseElement: isBaseElement,
		isScriptElement: isScriptElement,
		isStyleElement: isStyleElement,
		isLabelElement: isLabelElement,
		isTextAreaElement: isTextAreaElement,
		isOptionElement: isOptionElement,
		isRadioButtonElement: isRadioButtonElement,
		isColorInputElement: isColorInputElement,
		isCheckboxElement: isCheckboxElement,
		isSelectElement: isSelectElement,
		isFormElement: isFormElement,
		isFileInput: isFileInput,
		isInputWithNativeDialog: isInputWithNativeDialog,
		isBodyElementWithChildren: isBodyElementWithChildren,
		isMapElement: isMapElement,
		isRenderedNode: isRenderedNode,
		getTabIndex: getTabIndex,
		isElementFocusable: isElementFocusable,
		isShadowUIElement: isShadowUIElement,
		isWindow: isWindow,
		isDocument: isDocument,
		isBlob: isBlob,
		isLocation: isLocation,
		isSVGElement: isSVGElement,
		isSVGElementOrChild: isSVGElementOrChild,
		isFetchHeaders: isFetchHeaders,
		isFetchRequest: isFetchRequest,
		isElementReadOnly: isElementReadOnly,
		isTextEditableInput: isTextEditableInput,
		isTextEditableElement: isTextEditableElement,
		isTextEditableElementAndEditingAllowed: isTextEditableElementAndEditingAllowed,
		isElementNode: isElementNode,
		isTextNode: isTextNode,
		isProcessingInstructionNode: isProcessingInstructionNode,
		isCommentNode: isCommentNode,
		isDocumentFragmentNode: isDocumentFragmentNode,
		isShadowRoot: isShadowRoot,
		isAnchorElement: isAnchorElement,
		isTableElement: isTableElement,
		isTableDataCellElement: isTableDataCellElement,
		isWebSocket: isWebSocket,
		isMessageEvent: isMessageEvent,
		isPerformanceNavigationTiming: isPerformanceNavigationTiming,
		isArrayBuffer: isArrayBuffer,
		isArrayBufferView: isArrayBufferView,
		isDataView: isDataView,
		matches: matches,
		closest: closest,
		addClass: addClass,
		removeClass: removeClass,
		hasClass: hasClass,
		parseDocumentCharset: parseDocumentCharset,
		getParents: getParents,
		findParent: findParent,
		nodeListToArray: nodeListToArray,
		getFileInputs: getFileInputs,
		getIframes: getIframes,
		getScripts: getScripts,
		isNumberOrEmailInput: isNumberOrEmailInput,
		isInputWithoutSelectionProperties: isInputWithoutSelectionProperties,
		getAssociatedElement: getAssociatedElement
	});

	var SandboxBase = /** @class */ (function (_super) {
	    __extends(SandboxBase, _super);
	    function SandboxBase() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.window = null;
	        _this.nativeMethods = nativeMethods;
	        _this.document = null;
	        return _this;
	    }
	    // NOTE: The sandbox is deactivated when its window is removed from the DOM.
	    SandboxBase.prototype.isDeactivated = function () {
	        try {
	            // NOTE: In IE11, a situation when the document is not active may occur.
	            // eslint-disable-next-line no-unused-expressions
	            this.document.body;
	            if (this.window[INTERNAL_PROPS.hammerhead]) {
	                var frameElement_1 = getFrameElement(this.window);
	                return !!frameElement_1 && !isElementInDocument(frameElement_1, findDocument(frameElement_1));
	            }
	        }
	        catch (e) { // eslint-disable-line no-empty
	        }
	        return true;
	    };
	    SandboxBase.prototype.attach = function (window, document) {
	        this.window = window;
	        this.document = document || window.document;
	    };
	    return SandboxBase;
	}(EventEmitter));

	// -------------------------------------------------------------

	var isArray,
	    json$1,
	    renumber,
	    hexadecimal,
	    quotes,
	    escapeless,
	    parentheses,
	    semicolons,
	    safeConcatenation,
	    directive,
	    extra;


	var Syntax = {
	    AssignmentExpression:     'AssignmentExpression',
	    AssignmentPattern:        'AssignmentPattern',
	    ArrayExpression:          'ArrayExpression',
	    ArrayPattern:             'ArrayPattern',
	    ArrowFunctionExpression:  'ArrowFunctionExpression',
	    AwaitExpression:          'AwaitExpression',
	    BlockStatement:           'BlockStatement',
	    BinaryExpression:         'BinaryExpression',
	    BreakStatement:           'BreakStatement',
	    CallExpression:           'CallExpression',
	    CatchClause:              'CatchClause',
	    ClassBody:                'ClassBody',
	    ClassDeclaration:         'ClassDeclaration',
	    ClassExpression:          'ClassExpression',
	    ComprehensionBlock:       'ComprehensionBlock',
	    ComprehensionExpression:  'ComprehensionExpression',
	    ConditionalExpression:    'ConditionalExpression',
	    ContinueStatement:        'ContinueStatement',
	    DirectiveStatement:       'DirectiveStatement',
	    DoWhileStatement:         'DoWhileStatement',
	    DebuggerStatement:        'DebuggerStatement',
	    EmptyStatement:           'EmptyStatement',
	    ExportAllDeclaration:     'ExportAllDeclaration',
	    ExportBatchSpecifier:     'ExportBatchSpecifier',
	    ExportDeclaration:        'ExportDeclaration',
	    ExportNamedDeclaration:   'ExportNamedDeclaration',
	    ExportSpecifier:          'ExportSpecifier',
	    ExpressionStatement:      'ExpressionStatement',
	    ForStatement:             'ForStatement',
	    ForInStatement:           'ForInStatement',
	    ForOfStatement:           'ForOfStatement',
	    FunctionDeclaration:      'FunctionDeclaration',
	    FunctionExpression:       'FunctionExpression',
	    GeneratorExpression:      'GeneratorExpression',
	    Identifier:               'Identifier',
	    IfStatement:              'IfStatement',
	    ImportExpression:         'ImportExpression',
	    ImportSpecifier:          'ImportSpecifier',
	    ImportDeclaration:        'ImportDeclaration',
	    ChainExpression:          'ChainExpression',
	    Literal:                  'Literal',
	    LabeledStatement:         'LabeledStatement',
	    LogicalExpression:        'LogicalExpression',
	    MemberExpression:         'MemberExpression',
	    MetaProperty:             'MetaProperty',
	    MethodDefinition:         'MethodDefinition',
	    ModuleDeclaration:        'ModuleDeclaration',
	    NewExpression:            'NewExpression',
	    ObjectExpression:         'ObjectExpression',
	    ObjectPattern:            'ObjectPattern',
	    Program:                  'Program',
	    Property:                 'Property',
	    RestElement:              'RestElement',
	    ReturnStatement:          'ReturnStatement',
	    SequenceExpression:       'SequenceExpression',
	    SpreadElement:            'SpreadElement',
	    Super:                    'Super',
	    SwitchStatement:          'SwitchStatement',
	    SwitchCase:               'SwitchCase',
	    TaggedTemplateExpression: 'TaggedTemplateExpression',
	    TemplateElement:          'TemplateElement',
	    TemplateLiteral:          'TemplateLiteral',
	    ThisExpression:           'ThisExpression',
	    ThrowStatement:           'ThrowStatement',
	    TryStatement:             'TryStatement',
	    UnaryExpression:          'UnaryExpression',
	    UpdateExpression:         'UpdateExpression',
	    VariableDeclaration:      'VariableDeclaration',
	    VariableDeclarator:       'VariableDeclarator',
	    WhileStatement:           'WhileStatement',
	    WithStatement:            'WithStatement',
	    YieldExpression:          'YieldExpression'
	};

	var Syntax_1 = Syntax;

	var Precedence = {
	    Sequence:         0,
	    Yield:            1,
	    Assignment:       1,
	    Conditional:      2,
	    ArrowFunction:    2,
	    Coalesce:         3,
	    LogicalOR:        3,
	    LogicalAND:       4,
	    BitwiseOR:        5,
	    BitwiseXOR:       6,
	    BitwiseAND:       7,
	    Equality:         8,
	    Relational:       9,
	    BitwiseSHIFT:     10,
	    Additive:         11,
	    Multiplicative:   12,
	    Unary:            13,
	    Exponentiation:   14,
	    Postfix:          14,
	    Await:            14,
	    Call:             15,
	    New:              16,
	    TaggedTemplate:   17,
	    OptionalChaining: 17,
	    Member:           18,
	    Primary:          19
	};

	var BinaryPrecedence = {
	    '||':         Precedence.LogicalOR,
	    '&&':         Precedence.LogicalAND,
	    '|':          Precedence.BitwiseOR,
	    '^':          Precedence.BitwiseXOR,
	    '&':          Precedence.BitwiseAND,
	    '==':         Precedence.Equality,
	    '!=':         Precedence.Equality,
	    '===':        Precedence.Equality,
	    '!==':        Precedence.Equality,
	    'is':         Precedence.Equality,
	    'isnt':       Precedence.Equality,
	    '<':          Precedence.Relational,
	    '>':          Precedence.Relational,
	    '<=':         Precedence.Relational,
	    '>=':         Precedence.Relational,
	    'in':         Precedence.Relational,
	    'instanceof': Precedence.Relational,
	    '<<':         Precedence.BitwiseSHIFT,
	    '>>':         Precedence.BitwiseSHIFT,
	    '>>>':        Precedence.BitwiseSHIFT,
	    '+':          Precedence.Additive,
	    '-':          Precedence.Additive,
	    '*':          Precedence.Multiplicative,
	    '%':          Precedence.Multiplicative,
	    '/':          Precedence.Multiplicative,
	    '??':         Precedence.Coalesce,
	    '**':         Precedence.Exponentiation
	};

	function getDefaultOptions () {
	    // default options
	    return {
	        indent:    null,
	        base:      null,
	        parse:     null,
	        format:    {
	            indent:            {
	                style: '    ',
	                base:  0
	            },
	            newline:           '\n',
	            space:             ' ',
	            json:              false,
	            renumber:          false,
	            hexadecimal:       false,
	            quotes:            'single',
	            escapeless:        false,
	            compact:           false,
	            parentheses:       true,
	            semicolons:        true,
	            safeConcatenation: false
	        },
	        directive: false,
	        raw:       true,
	        verbatim:  null
	    };
	}

	//-------------------------------------------------===------------------------------------------------------
	//                                            Lexical utils
	//-------------------------------------------------===------------------------------------------------------

	//Const
	var NON_ASCII_WHITESPACES = [
	    0x1680, 0x180E, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005,
	    0x2006, 0x2007, 0x2008, 0x2009, 0x200A, 0x202F, 0x205F, 0x3000,
	    0xFEFF
	];

	//Regular expressions
	var NON_ASCII_IDENTIFIER_CHARACTERS_REGEXP = new RegExp(
	    '[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376' +
	    '\u0377\u037A-\u037D\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-' +
	    '\u0527\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA' +
	    '\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-' +
	    '\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0\u08A2-\u08AC\u08E4-\u08FE\u0900-' +
	    '\u0963\u0966-\u096F\u0971-\u0977\u0979-\u097F\u0981-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-' +
	    '\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-' +
	    '\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38' +
	    '\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83' +
	    '\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9' +
	    '\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-' +
	    '\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-' +
	    '\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E' +
	    '\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-' +
	    '\u0BEF\u0C01-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C33\u0C35-\u0C39\u0C3D-\u0C44\u0C46-' +
	    '\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58\u0C59\u0C60-\u0C63\u0C66-\u0C6F\u0C82\u0C83\u0C85-\u0C8C\u0C8E-' +
	    '\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE' +
	    '\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D02\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44' +
	    '\u0D46-\u0D48\u0D4A-\u0D4E\u0D57\u0D60-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-' +
	    '\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DF2\u0DF3\u0E01-\u0E3A' +
	    '\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-' +
	    '\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9' +
	    '\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84' +
	    '\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-' +
	    '\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5' +
	    '\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-' +
	    '\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F0\u1700-\u170C\u170E-' +
	    '\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD' +
	    '\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191C\u1920-\u192B' +
	    '\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E' +
	    '\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-' +
	    '\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1CD0-\u1CD2\u1CD4-\u1CF6\u1D00-\u1DE6\u1DFC-\u1F15\u1F18-\u1F1D\u1F20-' +
	    '\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-' +
	    '\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F' +
	    '\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115' +
	    '\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188' +
	    '\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-' +
	    '\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-' +
	    '\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A' +
	    '\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5' +
	    '\u4E00-\u9FCC\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA697' +
	    '\uA69F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA78E\uA790-\uA793\uA7A0-\uA7AA\uA7F8-\uA827\uA840-\uA873' +
	    '\uA880-\uA8C4\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-' +
	    '\uA9D9\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A\uAA7B\uAA80-\uAAC2\uAADB-\uAADD\uAAE0-' +
	    '\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uABC0-\uABEA\uABEC' +
	    '\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-' +
	    '\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D' +
	    '\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE26\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74' +
	    '\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-' +
	    '\uFFD7\uFFDA-\uFFDC]'
	);


	//Methods
	function isIdentifierCh (cp) {
	    if (cp < 0x80) {
	        return cp >= 97 && cp <= 122 ||      // a..z
	               cp >= 65 && cp <= 90 ||       // A..Z
	               cp >= 48 && cp <= 57 ||       // 0..9
	               cp === 36 || cp === 95 ||     // $ (dollar) and _ (underscore)
	               cp === 92;                    // \ (backslash)
	    }

	    var ch = String.fromCharCode(cp);

	    return NON_ASCII_IDENTIFIER_CHARACTERS_REGEXP.test(ch);
	}

	function isLineTerminator (cp) {
	    return cp === 0x0A || cp === 0x0D || cp === 0x2028 || cp === 0x2029;
	}

	function isWhitespace (cp) {
	    return cp === 0x20 || cp === 0x09 || isLineTerminator(cp) || cp === 0x0B || cp === 0x0C || cp === 0xA0 ||
	           (cp >= 0x1680 && NON_ASCII_WHITESPACES.indexOf(cp) >= 0);
	}

	function isDecimalDigit (cp) {
	    return cp >= 48 && cp <= 57;
	}

	function stringRepeat (str, num) {
	    var result = '';

	    for (num |= 0; num > 0; num >>>= 1, str += str) {
	        if (num & 1) {
	            result += str;
	        }
	    }

	    return result;
	}

	isArray = Array.isArray;
	if (!isArray) {
	    isArray = function isArray (array) {
	        return Object.prototype.toString.call(array) === '[object Array]';
	    };
	}


	function updateDeeply (target, override) {
	    var key, val;

	    function isHashObject (target) {
	        return typeof target === 'object' && target instanceof Object && !(target instanceof RegExp);
	    }

	    for (key in override) {
	        if (override.hasOwnProperty(key)) {
	            val = override[key];
	            if (isHashObject(val)) {
	                if (isHashObject(target[key])) {
	                    updateDeeply(target[key], val);
	                }
	                else {
	                    target[key] = updateDeeply({}, val);
	                }
	            }
	            else {
	                target[key] = val;
	            }
	        }
	    }
	    return target;
	}

	function generateNumber (value) {
	    var result, point, temp, exponent, pos;

	    if (value === 1 / 0) {
	        return json$1 ? 'null' : renumber ? '1e400' : '1e+400';
	    }

	    result = '' + value;
	    if (!renumber || result.length < 3) {
	        return result;
	    }

	    point = result.indexOf('.');
	    //NOTE: 0x30 == '0'
	    if (!json$1 && result.charCodeAt(0) === 0x30 && point === 1) {
	        point  = 0;
	        result = result.slice(1);
	    }
	    temp     = result;
	    result   = result.replace('e+', 'e');
	    exponent = 0;
	    if ((pos = temp.indexOf('e')) > 0) {
	        exponent = +temp.slice(pos + 1);
	        temp     = temp.slice(0, pos);
	    }
	    if (point >= 0) {
	        exponent -= temp.length - point - 1;
	        temp = +(temp.slice(0, point) + temp.slice(point + 1)) + '';
	    }
	    pos = 0;

	    //NOTE: 0x30 == '0'
	    while (temp.charCodeAt(temp.length + pos - 1) === 0x30) {
	        --pos;
	    }
	    if (pos !== 0) {
	        exponent -= pos;
	        temp = temp.slice(0, pos);
	    }
	    if (exponent !== 0) {
	        temp += 'e' + exponent;
	    }
	    if ((temp.length < result.length ||
	         (hexadecimal && value > 1e12 && Math.floor(value) === value &&
	          (temp = '0x' + value.toString(16)).length
	          < result.length)) &&
	        +temp === value) {
	        result = temp;
	    }

	    return result;
	}

	// Generate valid RegExp expression.
	// This function is based on https://github.com/Constellation/iv Engine

	function escapeRegExpCharacter (ch, previousIsBackslash) {
	    // not handling '\' and handling \u2028 or \u2029 to unicode escape sequence
	    if ((ch & ~1) === 0x2028) {
	        return (previousIsBackslash ? 'u' : '\\u') + ((ch === 0x2028) ? '2028' : '2029');
	    }
	    else if (ch === 10 || ch === 13) {  // \n, \r
	        return (previousIsBackslash ? '' : '\\') + ((ch === 10) ? 'n' : 'r');
	    }
	    return String.fromCharCode(ch);
	}

	function generateRegExp (reg) {
	    var match, result, flags, i, iz, ch, characterInBrack, previousIsBackslash;

	    result = reg.toString();

	    if (reg.source) {
	        // extract flag from toString result
	        match = result.match(/\/([^/]*)$/);
	        if (!match) {
	            return result;
	        }

	        flags  = match[1];
	        result = '';

	        characterInBrack    = false;
	        previousIsBackslash = false;
	        for (i = 0, iz = reg.source.length; i < iz; ++i) {
	            ch = reg.source.charCodeAt(i);

	            if (!previousIsBackslash) {
	                if (characterInBrack) {
	                    if (ch === 93) {  // ]
	                        characterInBrack = false;
	                    }
	                }
	                else {
	                    if (ch === 47) {  // /
	                        result += '\\';
	                    }
	                    else if (ch === 91) {  // [
	                        characterInBrack = true;
	                    }
	                }
	                result += escapeRegExpCharacter(ch, previousIsBackslash);
	                previousIsBackslash = ch === 92;  // \
	            }
	            else {
	                // if new RegExp("\\\n') is provided, create /\n/
	                result += escapeRegExpCharacter(ch, previousIsBackslash);
	                // prevent like /\\[/]/
	                previousIsBackslash = false;
	            }
	        }

	        return '/' + result + '/' + flags;
	    }

	    return result;
	}

	function escapeAllowedCharacter (code, next) {
	    var hex, result = '\\';

	    switch (code) {
	        case 0x08:          // \b
	            result += 'b';
	            break;
	        case 0x0C:          // \f
	            result += 'f';
	            break;
	        case 0x09:          // \t
	            result += 't';
	            break;
	        default:
	            hex = code.toString(16).toUpperCase();
	            if (json$1 || code > 0xFF) {
	                result += 'u' + '0000'.slice(hex.length) + hex;
	            }

	            else if (code === 0x0000 && !isDecimalDigit(next)) {
	                result += '0';
	            }

	            else if (code === 0x000B) {     // \v
	                result += 'x0B';
	            }

	            else {
	                result += 'x' + '00'.slice(hex.length) + hex;
	            }
	            break;
	    }

	    return result;
	}

	function escapeDisallowedCharacter (code) {
	    var result = '\\';
	    switch (code) {
	        case 0x5C       // \
	        :
	            result += '\\';
	            break;
	        case 0x0A       // \n
	        :
	            result += 'n';
	            break;
	        case 0x0D       // \r
	        :
	            result += 'r';
	            break;
	        case 0x2028:
	            result += 'u2028';
	            break;
	        case 0x2029:
	            result += 'u2029';
	            break;
	    }

	    return result;
	}

	function escapeDirective (str) {
	    var i, iz, code, quote;

	    quote = quotes === 'double' ? '"' : '\'';
	    for (i = 0, iz = str.length; i < iz; ++i) {
	        code = str.charCodeAt(i);
	        if (code === 0x27) {            // '
	            quote = '"';
	            break;
	        }
	        else if (code === 0x22) {     // "
	            quote = '\'';
	            break;
	        }
	        else if (code === 0x5C) {     // \
	            ++i;
	        }
	    }

	    return quote + str + quote;
	}

	function escapeString (str) {
	    var result = '', i, len, code, singleQuotes = 0, doubleQuotes = 0, single, quote;
	    //TODO http://jsperf.com/character-counting/8
	    for (i = 0, len = str.length; i < len; ++i) {
	        code = str.charCodeAt(i);
	        if (code === 0x27) {           // '
	            ++singleQuotes;
	        }
	        else if (code === 0x22) { // "
	            ++doubleQuotes;
	        }
	        else if (code === 0x2F && json$1) { // /
	            result += '\\';
	        }
	        else if (isLineTerminator(code) || code === 0x5C) { // \
	            result += escapeDisallowedCharacter(code);
	            continue;
	        }
	        else if ((json$1 && code < 0x20) ||                                     // SP
	                 !(json$1 || escapeless || (code >= 0x20 && code <= 0x7E))) {   // SP, ~
	            result += escapeAllowedCharacter(code, str.charCodeAt(i + 1));
	            continue;
	        }
	        result += String.fromCharCode(code);
	    }

	    single = !(quotes === 'double' || (quotes === 'auto' && doubleQuotes < singleQuotes));
	    quote  = single ? '\'' : '"';

	    if (!(single ? singleQuotes : doubleQuotes)) {
	        return quote + result + quote;
	    }

	    str    = result;
	    result = quote;

	    for (i = 0, len = str.length; i < len; ++i) {
	        code = str.charCodeAt(i);
	        if ((code === 0x27 && single) || (code === 0x22 && !single)) {    // ', "
	            result += '\\';
	        }
	        result += String.fromCharCode(code);
	    }

	    return result + quote;
	}


	function join (l, r) {
	    if (!l.length)
	        return r;

	    if (!r.length)
	        return l;

	    var lCp = l.charCodeAt(l.length - 1),
	        rCp = r.charCodeAt(0);

	    if (isIdentifierCh(lCp) && isIdentifierCh(rCp) ||
	        lCp === rCp && (lCp === 0x2B || lCp === 0x2D) ||   // + +, - -
	        lCp === 0x2F && rCp === 0x69) {                    // /re/ instanceof foo
	        return l + _.space + r;
	    }

	    else if (isWhitespace(lCp) || isWhitespace(rCp))
	        return l + r;

	    return l + _.optSpace + r;
	}

	function shiftIndent () {
	    var prevIndent = _.indent;

	    _.indent += _.indentUnit;
	    return prevIndent;
	}

	function adoptionPrefix ($stmt) {
	    if ($stmt.type === Syntax.BlockStatement)
	        return _.optSpace;

	    if ($stmt.type === Syntax.EmptyStatement)
	        return '';

	    return _.newline + _.indent + _.indentUnit;
	}

	function adoptionSuffix ($stmt) {
	    if ($stmt.type === Syntax.BlockStatement)
	        return _.optSpace;

	    return _.newline + _.indent;
	}

	//Subentities generators
	function generateVerbatim ($expr, settings) {
	    var verbatim     = $expr[extra.verbatim],
	        strVerbatim  = typeof verbatim === 'string',
	        precedence   = !strVerbatim &&
	                       verbatim.precedence !== void 0 ? verbatim.precedence : Precedence.Sequence,
	        parenthesize = precedence < settings.precedence,
	        content      = strVerbatim ? verbatim : verbatim.content,
	        chunks       = content.split(/\r\n|\n/),
	        chunkCount   = chunks.length;

	    if (parenthesize)
	        _.js += '(';

	    _.js += chunks[0];

	    for (var i = 1; i < chunkCount; i++)
	        _.js += _.newline + _.indent + chunks[i];

	    if (parenthesize)
	        _.js += ')';
	}

	function generateFunctionParams ($node) {
	    var $params                     = $node.params,
	        paramCount                  = $params.length,
	        lastParamIdx                = paramCount - 1,
	        arrowFuncWithoutParentheses = $node.type === Syntax.ArrowFunctionExpression && paramCount === 1 &&
	                                      $params[0].type === Syntax.Identifier;

	    //NOTE: arg => { } case
	    if (arrowFuncWithoutParentheses)
	        _.js += $params[0].name;

	    else {
	        _.js += '(';

	        for (var i = 0; i < paramCount; ++i) {
	            var $param = $params[i];

	            if ($params[i].type === Syntax.Identifier)
	                _.js += $param.name;

	            else
	                ExprGen[$param.type]($param, Preset.e4);

	            if (i !== lastParamIdx)
	                _.js += ',' + _.optSpace;
	        }

	        _.js += ')';
	    }
	}

	function generateFunctionBody ($node) {
	    var $body = $node.body;

	    generateFunctionParams($node);

	    if ($node.type === Syntax.ArrowFunctionExpression)
	        _.js += _.optSpace + '=>';

	    if ($node.expression) {
	        _.js += _.optSpace;

	        var exprJs = exprToJs($body, Preset.e4);

	        if (exprJs.charAt(0) === '{')
	            exprJs = '(' + exprJs + ')';

	        _.js += exprJs;
	    }

	    else {
	        _.js += adoptionPrefix($body);
	        StmtGen[$body.type]($body, Preset.s8);
	    }
	}


	//-------------------------------------------------===------------------------------------------------------
	//                                Syntactic entities generation presets
	//-------------------------------------------------===------------------------------------------------------

	var Preset = {
	    e1: function (allowIn) {
	        return {
	            precedence:              Precedence.Assignment,
	            allowIn:                 allowIn,
	            allowCall:               true,
	            allowUnparenthesizedNew: true
	        };
	    },

	    e2: function (allowIn) {
	        return {
	            precedence:              Precedence.LogicalOR,
	            allowIn:                 allowIn,
	            allowCall:               true,
	            allowUnparenthesizedNew: true
	        };
	    },

	    e3: {
	        precedence:              Precedence.Call,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: false
	    },

	    e4: {
	        precedence:              Precedence.Assignment,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e5: {
	        precedence:              Precedence.Sequence,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e6: function (allowUnparenthesizedNew) {
	        return {
	            precedence:              Precedence.New,
	            allowIn:                 true,
	            allowCall:               false,
	            allowUnparenthesizedNew: allowUnparenthesizedNew
	        };
	    },

	    e7: {
	        precedence:              Precedence.Unary,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e8: {
	        precedence:              Precedence.Postfix,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e9: {
	        precedence:              void 0,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e10: {
	        precedence:              Precedence.Call,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },

	    e11: function (allowCall) {
	        return {
	            precedence:              Precedence.Call,
	            allowIn:                 true,
	            allowCall:               allowCall,
	            allowUnparenthesizedNew: false
	        };
	    },

	    e12: {
	        precedence:              Precedence.Primary,
	        allowIn:                 false,
	        allowCall:               false,
	        allowUnparenthesizedNew: true
	    },

	    e13: {
	        precedence:              Precedence.Primary,
	        allowIn:                 true,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },


	    e14: {
	        precedence:              Precedence.Sequence,
	        allowIn:                 false,
	        allowCall:               true,
	        allowUnparenthesizedNew: true
	    },


	    e15: function (allowCall) {
	        return {
	            precedence:              Precedence.Sequence,
	            allowIn:                 true,
	            allowCall:               allowCall,
	            allowUnparenthesizedNew: true
	        };
	    },

	    e16: function (precedence, allowIn) {
	        return {
	            precedence:              precedence,
	            allowIn:                 allowIn,
	            allowCall:               true,
	            allowUnparenthesizedNew: true
	        };
	    },

	    e17: function (allowIn) {
	        return {
	            precedence:              Precedence.Call,
	            allowIn:                 allowIn,
	            allowCall:               true,
	            allowUnparenthesizedNew: true
	        }
	    },

	    e18: function (allowIn) {
	        return {
	            precedence:              Precedence.Assignment,
	            allowIn:                 allowIn,
	            allowCall:               true,
	            allowUnparenthesizedNew: true
	        }
	    },

	    e19: {
	        precedence:        Precedence.Sequence,
	        allowIn:           true,
	        allowCall:         true,
	        semicolonOptional: false
	    },

	    e20: {
	        precedence: Precedence.Await,
	        allowCall:  true
	    },

	    s1: function (functionBody, semicolonOptional) {
	        return {
	            allowIn:           true,
	            functionBody:      false,
	            directiveContext:  functionBody,
	            semicolonOptional: semicolonOptional
	        };
	    },

	    s2: {
	        allowIn:           true,
	        functionBody:      false,
	        directiveContext:  false,
	        semicolonOptional: true
	    },

	    s3: function (allowIn) {
	        return {
	            allowIn:           allowIn,
	            functionBody:      false,
	            directiveContext:  false,
	            semicolonOptional: false
	        };
	    },

	    s4: function (semicolonOptional) {
	        return {
	            allowIn:           true,
	            functionBody:      false,
	            directiveContext:  false,
	            semicolonOptional: semicolonOptional
	        };
	    },

	    s5: function (semicolonOptional) {
	        return {
	            allowIn:           true,
	            functionBody:      false,
	            directiveContext:  true,
	            semicolonOptional: semicolonOptional,
	        };
	    },

	    s6: {
	        allowIn:           false,
	        functionBody:      false,
	        directiveContext:  false,
	        semicolonOptional: false
	    },

	    s7: {
	        allowIn:           true,
	        functionBody:      false,
	        directiveContext:  false,
	        semicolonOptional: false
	    },

	    s8: {
	        allowIn:           true,
	        functionBody:      true,
	        directiveContext:  false,
	        semicolonOptional: false
	    }
	};


	//-------------------------------------------------===-------------------------------------------------------
	//                                             Expressions
	//-------------------------------------------------===-------------------------------------------------------

	//Regular expressions
	var FLOATING_OR_OCTAL_REGEXP  = /[.eExX]|^0[0-9]+/,
	    LAST_DECIMAL_DIGIT_REGEXP = /[0-9]$/;


	//Common expression generators
	function generateLogicalOrBinaryExpression ($expr, settings) {
	    var op                 = $expr.operator,
	        precedence         = BinaryPrecedence[$expr.operator],
	        parenthesize       = precedence < settings.precedence,
	        allowIn            = settings.allowIn || parenthesize,
	        operandGenSettings = Preset.e16(precedence, allowIn),
	        exprJs             = exprToJs($expr.left, operandGenSettings);

	    parenthesize |= op === 'in' && !allowIn;

	    if (parenthesize)
	        _.js += '(';

	    // 0x2F = '/'
	    if (exprJs.charCodeAt(exprJs.length - 1) === 0x2F && isIdentifierCh(op.charCodeAt(0)))
	        exprJs = exprJs + _.space + op;

	    else
	        exprJs = join(exprJs, op);

	    operandGenSettings.precedence++;

	    var rightJs = exprToJs($expr.right, operandGenSettings);

	    //NOTE: If '/' concats with '/' or `<` concats with `!--`, it is interpreted as comment start
	    if (op === '/' && rightJs.charAt(0) === '/' || op.slice(-1) === '<' && rightJs.slice(0, 3) === '!--')
	        exprJs += _.space + rightJs;

	    else
	        exprJs = join(exprJs, rightJs);

	    _.js += exprJs;

	    if (parenthesize)
	        _.js += ')';
	}

	function generateArrayPatternOrExpression ($expr) {
	    var $elems    = $expr.elements,
	        elemCount = $elems.length;

	    if (elemCount) {
	        var lastElemIdx = elemCount - 1,
	            multiline   = elemCount > 1,
	            prevIndent  = shiftIndent(),
	            itemPrefix  = _.newline + _.indent;

	        _.js += '[';

	        for (var i = 0; i < elemCount; i++) {
	            var $elem = $elems[i];

	            if (multiline)
	                _.js += itemPrefix;

	            if ($elem)
	                ExprGen[$elem.type]($elem, Preset.e4);

	            if (i !== lastElemIdx || !$elem)
	                _.js += ',';
	        }

	        _.indent = prevIndent;

	        if (multiline)
	            _.js += _.newline + _.indent;

	        _.js += ']';
	    }

	    else
	        _.js += '[]';
	}

	function generateGeneratorOrComprehensionExpression ($expr) {
	    //NOTE: GeneratorExpression should be parenthesized with (...), ComprehensionExpression with [...]
	    var $blocks     = $expr.blocks,
	        $filter     = $expr.filter,
	        isGenerator = $expr.type === Syntax.GeneratorExpression,
	        exprJs      = isGenerator ? '(' : '[',
	        bodyJs      = exprToJs($expr.body, Preset.e4);

	    if ($blocks) {
	        var prevIndent = shiftIndent(),
	            blockCount = $blocks.length;

	        for (var i = 0; i < blockCount; ++i) {
	            var blockJs = exprToJs($blocks[i], Preset.e5);

	            exprJs = i > 0 ? join(exprJs, blockJs) : (exprJs + blockJs);
	        }

	        _.indent = prevIndent;
	    }

	    if ($filter) {
	        var filterJs = exprToJs($filter, Preset.e5);

	        exprJs = join(exprJs, 'if' + _.optSpace);
	        exprJs = join(exprJs, '(' + filterJs + ')');
	    }

	    exprJs = join(exprJs, bodyJs);
	    exprJs += isGenerator ? ')' : ']';

	    _.js += exprJs;
	}


	//Expression raw generator dictionary
	var ExprRawGen = {
	    SequenceExpression: function generateSequenceExpression ($expr, settings) {
	        var $children       = $expr.expressions,
	            childrenCount   = $children.length,
	            lastChildIdx    = childrenCount - 1,
	            parenthesize    = Precedence.Sequence < settings.precedence,
	            exprGenSettings = Preset.e1(settings.allowIn || parenthesize);

	        if (parenthesize)
	            _.js += '(';

	        for (var i = 0; i < childrenCount; i++) {
	            var $child = $children[i];

	            ExprGen[$child.type]($child, exprGenSettings);

	            if (i !== lastChildIdx)
	                _.js += ',' + _.optSpace;
	        }

	        if (parenthesize)
	            _.js += ')';
	    },

	    AssignmentExpression: function generateAssignmentExpression ($expr, settings) {
	        var $left        = $expr.left,
	            $right       = $expr.right,
	            parenthesize = Precedence.Assignment < settings.precedence,
	            allowIn      = settings.allowIn || parenthesize;

	        if (parenthesize)
	            _.js += '(';

	        ExprGen[$left.type]($left, Preset.e17(allowIn));
	        _.js += _.optSpace + $expr.operator + _.optSpace;
	        ExprGen[$right.type]($right, Preset.e18(allowIn));

	        if (parenthesize)
	            _.js += ')';
	    },

	    AssignmentPattern: function generateAssignmentPattern ($node) {
	        var $fakeAssign = {
	            left:     $node.left,
	            right:    $node.right,
	            operator: '='
	        };

	        ExprGen.AssignmentExpression($fakeAssign, Preset.e4);
	    },

	    ArrowFunctionExpression: function generateArrowFunctionExpression ($expr, settings) {
	        var parenthesize = Precedence.ArrowFunction < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        if ($expr.async)
	            _.js += 'async ';

	        generateFunctionBody($expr);

	        if (parenthesize)
	            _.js += ')';
	    },

	    AwaitExpression: function generateAwaitExpression ($expr, settings) {
	        var parenthesize = Precedence.Await < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        _.js += $expr.all ? 'await* ' : 'await ';

	        ExprGen[$expr.argument.type]($expr.argument, Preset.e20);

	        if (parenthesize)
	            _.js += ')';
	    },

	    ConditionalExpression: function generateConditionalExpression ($expr, settings) {
	        var $test             = $expr.test,
	            $conseq           = $expr.consequent,
	            $alt              = $expr.alternate,
	            parenthesize      = Precedence.Conditional < settings.precedence,
	            allowIn           = settings.allowIn || parenthesize,
	            testGenSettings   = Preset.e2(allowIn),
	            branchGenSettings = Preset.e1(allowIn);

	        if (parenthesize)
	            _.js += '(';

	        ExprGen[$test.type]($test, testGenSettings);
	        _.js += _.optSpace + '?' + _.optSpace;
	        ExprGen[$conseq.type]($conseq, branchGenSettings);
	        _.js += _.optSpace + ':' + _.optSpace;
	        ExprGen[$alt.type]($alt, branchGenSettings);

	        if (parenthesize)
	            _.js += ')';
	    },

	    LogicalExpression: generateLogicalOrBinaryExpression,

	    BinaryExpression: generateLogicalOrBinaryExpression,

	    CallExpression: function generateCallExpression ($expr, settings) {
	        var $callee      = $expr.callee,
	            $args        = $expr['arguments'],
	            argCount     = $args.length,
	            lastArgIdx   = argCount - 1,
	            parenthesize = !settings.allowCall || Precedence.Call < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        ExprGen[$callee.type]($callee, Preset.e3);

	        if ($expr.optional)
	            _.js += '?.';

	        _.js += '(';

	        for (var i = 0; i < argCount; ++i) {
	            var $arg = $args[i];

	            ExprGen[$arg.type]($arg, Preset.e4);

	            if (i !== lastArgIdx)
	                _.js += ',' + _.optSpace;
	        }

	        _.js += ')';

	        if (parenthesize)
	            _.js += ')';
	    },

	    NewExpression: function generateNewExpression ($expr, settings) {
	        var $args        = $expr['arguments'],
	            parenthesize = Precedence.New < settings.precedence,
	            argCount     = $args.length,
	            lastArgIdx   = argCount - 1,
	            withCall     = !settings.allowUnparenthesizedNew || parentheses || argCount > 0,
	            calleeJs     = exprToJs($expr.callee, Preset.e6(!withCall));

	        if (parenthesize)
	            _.js += '(';

	        _.js += join('new', calleeJs);

	        if (withCall) {
	            _.js += '(';

	            for (var i = 0; i < argCount; ++i) {
	                var $arg = $args[i];

	                ExprGen[$arg.type]($arg, Preset.e4);

	                if (i !== lastArgIdx)
	                    _.js += ',' + _.optSpace;
	            }

	            _.js += ')';
	        }

	        if (parenthesize)
	            _.js += ')';
	    },

	    MemberExpression: function generateMemberExpression ($expr, settings) {
	        var $obj         = $expr.object,
	            $prop        = $expr.property,
	            parenthesize = Precedence.Member < settings.precedence,
	            isNumObj     = !$expr.computed && $obj.type === Syntax.Literal && typeof $obj.value === 'number';

	        if (parenthesize)
	            _.js += '(';

	        if (isNumObj) {

	            //NOTE: When the following conditions are all true:
	            //   1. No floating point
	            //   2. Don't have exponents
	            //   3. The last character is a decimal digit
	            //   4. Not hexadecimal OR octal number literal
	            // then we should add a floating point.

	            var numJs     = exprToJs($obj, Preset.e11(settings.allowCall)),
	                withPoint = LAST_DECIMAL_DIGIT_REGEXP.test(numJs) && !FLOATING_OR_OCTAL_REGEXP.test(numJs);

	            _.js += withPoint ? (numJs + '.') : numJs;
	        }

	        else
	            ExprGen[$obj.type]($obj, Preset.e11(settings.allowCall));

	        if ($expr.computed) {
	            if ($expr.optional)
	                _.js += '?.';

	            _.js += '[';
	            ExprGen[$prop.type]($prop, Preset.e15(settings.allowCall));
	            _.js += ']';
	        }

	        else
	            _.js += ($expr.optional ? '?.' : '.') + $prop.name;

	        if (parenthesize)
	            _.js += ')';
	    },

	    UnaryExpression: function generateUnaryExpression ($expr, settings) {
	        var parenthesize = Precedence.Unary < settings.precedence,
	            op           = $expr.operator,
	            argJs        = exprToJs($expr.argument, Preset.e7);

	        if (parenthesize)
	            _.js += '(';

	        //NOTE: delete, void, typeof
	        // get `typeof []`, not `typeof[]`
	        if (_.optSpace === '' || op.length > 2)
	            _.js += join(op, argJs);

	        else {
	            _.js += op;

	            //NOTE: Prevent inserting spaces between operator and argument if it is unnecessary
	            // like, `!cond`
	            var leftCp  = op.charCodeAt(op.length - 1),
	                rightCp = argJs.charCodeAt(0);

	            // 0x2B = '+', 0x2D =  '-'
	            if (leftCp === rightCp && (leftCp === 0x2B || leftCp === 0x2D) ||
	                isIdentifierCh(leftCp) && isIdentifierCh(rightCp)) {
	                _.js += _.space;
	            }

	            _.js += argJs;
	        }

	        if (parenthesize)
	            _.js += ')';
	    },

	    YieldExpression: function generateYieldExpression ($expr, settings) {
	        var $arg         = $expr.argument,
	            js           = $expr.delegate ? 'yield*' : 'yield',
	            parenthesize = Precedence.Yield < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        if ($arg) {
	            var argJs = exprToJs($arg, Preset.e4);

	            js = join(js, argJs);
	        }

	        _.js += js;

	        if (parenthesize)
	            _.js += ')';
	    },

	    UpdateExpression: function generateUpdateExpression ($expr, settings) {
	        var $arg         = $expr.argument,
	            $op          = $expr.operator,
	            prefix       = $expr.prefix,
	            precedence   = prefix ? Precedence.Unary : Precedence.Postfix,
	            parenthesize = precedence < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        if (prefix) {
	            _.js += $op;
	            ExprGen[$arg.type]($arg, Preset.e8);

	        }

	        else {
	            ExprGen[$arg.type]($arg, Preset.e8);
	            _.js += $op;
	        }

	        if (parenthesize)
	            _.js += ')';
	    },

	    FunctionExpression: function generateFunctionExpression ($expr) {
	        var isGenerator = !!$expr.generator;

	        if ($expr.async)
	            _.js += 'async ';

	        _.js += isGenerator ? 'function*' : 'function';

	        if ($expr.id) {
	            _.js += isGenerator ? _.optSpace : _.space;
	            _.js += $expr.id.name;
	        }
	        else
	            _.js += _.optSpace;

	        generateFunctionBody($expr);
	    },

	    ExportBatchSpecifier: function generateExportBatchSpecifier () {
	        _.js += '*';
	    },

	    ArrayPattern: generateArrayPatternOrExpression,

	    ArrayExpression: generateArrayPatternOrExpression,

	    ClassExpression: function generateClassExpression ($expr) {
	        var $id    = $expr.id,
	            $super = $expr.superClass,
	            $body  = $expr.body,
	            exprJs = 'class';

	        if ($id) {
	            var idJs = exprToJs($id, Preset.e9);

	            exprJs = join(exprJs, idJs);
	        }

	        if ($super) {
	            var superJs = exprToJs($super, Preset.e4);

	            superJs = join('extends', superJs);
	            exprJs  = join(exprJs, superJs);
	        }

	        _.js += exprJs + _.optSpace;
	        StmtGen[$body.type]($body, Preset.s2);
	    },

	    MetaProperty: function generateMetaProperty ($expr, settings) {
	        var $meta        = $expr.meta,
	            $property    = $expr.property,
	            parenthesize = Precedence.Member < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        _.js += (typeof $meta === "string" ? $meta : $meta.name) +
	            '.' + (typeof $property === "string" ? $property : $property.name);

	        if (parenthesize)
	            _.js += ')';
	    },

	    MethodDefinition: function generateMethodDefinition ($expr) {
	        var exprJs = $expr['static'] ? 'static' + _.optSpace : '',
	            keyJs  = exprToJs($expr.key, Preset.e5);

	        if ($expr.computed)
	            keyJs = '[' + keyJs + ']';

	        if ($expr.kind === 'get' || $expr.kind === 'set') {
	            keyJs = join($expr.kind, keyJs);
	            _.js += join(exprJs, keyJs);
	        }

	        else {
	            if ($expr.value.generator)
	                _.js += exprJs + '*' + keyJs;
	            else if ($expr.value.async)
	                _.js += exprJs + 'async ' + keyJs;
	            else
	                _.js += join(exprJs, keyJs);
	        }

	        generateFunctionBody($expr.value);
	    },

	    Property: function generateProperty ($expr) {
	        var $val  = $expr.value,
	            $kind = $expr.kind,
	            keyJs = exprToJs($expr.key, Preset.e4);

	        if ($expr.computed)
	            keyJs = '[' + keyJs + ']';

	        if ($kind === 'get' || $kind === 'set') {
	            _.js += $kind + _.space + keyJs;
	            generateFunctionBody($val);
	        }

	        else {
	            if ($expr.shorthand)
	                _.js += keyJs;

	            else if ($expr.method) {
	                if ($val.generator)
	                    keyJs = '*' + keyJs;
	                else if ($val.async)
	                    keyJs = 'async ' + keyJs;

	                _.js += keyJs;
	                generateFunctionBody($val);
	            }

	            else {
	                _.js += keyJs + ':' + _.optSpace;
	                ExprGen[$val.type]($val, Preset.e4);
	            }
	        }
	    },

	    ObjectExpression: function generateObjectExpression ($expr) {
	        var $props    = $expr.properties,
	            propCount = $props.length;

	        if (propCount) {
	            var lastPropIdx = propCount - 1,
	                prevIndent  = shiftIndent();

	            _.js += '{';

	            for (var i = 0; i < propCount; i++) {
	                var $prop    = $props[i],
	                    propType = $prop.type || Syntax.Property;

	                _.js += _.newline + _.indent;
	                ExprGen[propType]($prop, Preset.e5);

	                if (i !== lastPropIdx)
	                    _.js += ',';
	            }

	            _.indent = prevIndent;
	            _.js += _.newline + _.indent + '}';
	        }

	        else
	            _.js += '{}';
	    },

	    ObjectPattern: function generateObjectPattern ($expr) {
	        var $props    = $expr.properties,
	            propCount = $props.length;

	        if (propCount) {
	            var lastPropIdx = propCount - 1,
	                multiline   = false;

	            if (propCount === 1)
	                multiline = $props[0].value.type !== Syntax.Identifier;

	            else {
	                for (var i = 0; i < propCount; i++) {
	                    if (!$props[i].shorthand) {
	                        multiline = true;
	                        break;
	                    }
	                }
	            }

	            _.js += multiline ? ('{' + _.newline) : '{';

	            var prevIndent = shiftIndent(),
	                propSuffix = ',' + (multiline ? _.newline : _.optSpace);

	            for (var i = 0; i < propCount; i++) {
	                var $prop = $props[i];

	                if (multiline)
	                    _.js += _.indent;

	                ExprGen[$prop.type]($prop, Preset.e5);

	                if (i !== lastPropIdx)
	                    _.js += propSuffix;
	            }

	            _.indent = prevIndent;
	            _.js += multiline ? (_.newline + _.indent + '}') : '}';
	        }
	        else
	            _.js += '{}';
	    },

	    ThisExpression: function generateThisExpression () {
	        _.js += 'this';
	    },

	    Identifier: function generateIdentifier ($expr, precedence, flag) {
	        _.js += $expr.name;
	    },

	    ImportExpression: function generateImportExpression ($expr, settings) {
	        var parenthesize = Precedence.Call < settings.precedence;
	        var $source      = $expr.source;

	        if (parenthesize)
	            _.js += '(';

	        _.js += 'import(';

	        ExprGen[$source.type]($source, Preset.e4);

	        _.js += ')';

	        if (parenthesize)
	            _.js += ')';
	    },

	    ImportSpecifier: function generateImportSpecifier ($expr) {
	        _.js += $expr.imported.name;

	        if ($expr.local)
	            _.js += _.space + 'as' + _.space + $expr.local.name;
	    },

	    ExportSpecifier: function generateImportOrExportSpecifier ($expr) {
	        _.js += $expr.local.name;

	        if ($expr.exported)
	            _.js += _.space + 'as' + _.space + $expr.exported.name;
	    },

	    ChainExpression: function generateChainExpression ($expr, settings) {
	        var parenthesize = Precedence.OptionalChaining < settings.precedence;
	        var $expression  = $expr.expression;

	        settings = settings || {};

	        var newSettings  = {
	            precedence: Precedence.OptionalChaining,
	            allowIn:    settings.allowIn ,
	            allowCall:  settings.allowCall,

	            allowUnparenthesizedNew: settings.allowUnparenthesizedNew
	        };

	        if (parenthesize) {
	            newSettings.allowCall = true;
	            _.js += '(';
	        }

	        ExprGen[$expression.type]($expression, newSettings);

	        if (parenthesize)
	            _.js += ')';
	    },

	    Literal: function generateLiteral ($expr) {
	        if (extra.raw && $expr.raw !== void 0)
	            _.js += $expr.raw;

	        else if ($expr.value === null)
	            _.js += 'null';

	        else {
	            var valueType = typeof $expr.value;

	            if (valueType === 'string')
	                _.js += escapeString($expr.value);

	            else if (valueType === 'number')
	                _.js += generateNumber($expr.value);

	            else if (valueType === 'boolean')
	                _.js += $expr.value ? 'true' : 'false';

	            else
	                _.js += generateRegExp($expr.value);
	        }
	    },

	    GeneratorExpression: generateGeneratorOrComprehensionExpression,

	    ComprehensionExpression: generateGeneratorOrComprehensionExpression,

	    ComprehensionBlock: function generateComprehensionBlock ($expr) {
	        var $left   = $expr.left,
	            leftJs  = void 0,
	            rightJs = exprToJs($expr.right, Preset.e5);

	        if ($left.type === Syntax.VariableDeclaration)
	            leftJs = $left.kind + _.space + stmtToJs($left.declarations[0], Preset.s6);

	        else
	            leftJs = exprToJs($left, Preset.e10);

	        leftJs = join(leftJs, $expr.of ? 'of' : 'in');

	        _.js += 'for' + _.optSpace + '(' + join(leftJs, rightJs) + ')';
	    },

	    RestElement: function generateRestElement ($node) {
	        _.js += '...' + $node.argument.name;
	    },

	    SpreadElement: function generateSpreadElement ($expr) {
	        var $arg = $expr.argument;

	        _.js += '...';
	        ExprGen[$arg.type]($arg, Preset.e4);
	    },

	    TaggedTemplateExpression: function generateTaggedTemplateExpression ($expr, settings) {
	        var $tag         = $expr.tag,
	            $quasi       = $expr.quasi,
	            parenthesize = Precedence.TaggedTemplate < settings.precedence;

	        if (parenthesize)
	            _.js += '(';

	        ExprGen[$tag.type]($tag, Preset.e11(settings.allowCall));
	        ExprGen[$quasi.type]($quasi, Preset.e12);

	        if (parenthesize)
	            _.js += ')';
	    },

	    TemplateElement: function generateTemplateElement ($expr) {
	        //NOTE: Don't use "cooked". Since tagged template can use raw template
	        // representation. So if we do so, it breaks the script semantics.
	        _.js += $expr.value.raw;
	    },

	    TemplateLiteral: function generateTemplateLiteral ($expr) {
	        var $quasis      = $expr.quasis,
	            $childExprs  = $expr.expressions,
	            quasiCount   = $quasis.length,
	            lastQuasiIdx = quasiCount - 1;

	        _.js += '`';

	        for (var i = 0; i < quasiCount; ++i) {
	            var $quasi = $quasis[i];

	            ExprGen[$quasi.type]($quasi, Preset.e13);

	            if (i !== lastQuasiIdx) {
	                var $childExpr = $childExprs[i];

	                _.js += '${' + _.optSpace;
	                ExprGen[$childExpr.type]($childExpr, Preset.e5);
	                _.js += _.optSpace + '}';
	            }
	        }

	        _.js += '`';
	    },

	    Super: function generateSuper () {
	        _.js += 'super';
	    }
	};


	//-------------------------------------------------===------------------------------------------------------
	//                                              Statements
	//-------------------------------------------------===------------------------------------------------------


	//Regular expressions
	var EXPR_STMT_UNALLOWED_EXPR_REGEXP = /^{|^class(?:\s|{)|^(async )?function(?:\s|\*|\()/;


	//Common statement generators
	function generateTryStatementHandlers (stmtJs, $finalizer, handlers) {
	    var handlerCount   = handlers.length,
	        lastHandlerIdx = handlerCount - 1;

	    for (var i = 0; i < handlerCount; ++i) {
	        var handlerJs = stmtToJs(handlers[i], Preset.s7);

	        stmtJs = join(stmtJs, handlerJs);

	        if ($finalizer || i !== lastHandlerIdx)
	            stmtJs += adoptionSuffix(handlers[i].body);
	    }

	    return stmtJs;
	}

	function generateForStatementIterator ($op, $stmt, settings) {
	    var $body                 = $stmt.body,
	        $left                 = $stmt.left,
	        bodySemicolonOptional = !semicolons && settings.semicolonOptional,
	        prevIndent1           = shiftIndent(),
	        awaitStr              = $stmt.await ? ' await' : '',
	        stmtJs                = 'for' + awaitStr + _.optSpace + '(';

	    if ($left.type === Syntax.VariableDeclaration) {
	        var prevIndent2 = shiftIndent();

	        stmtJs += $left.kind + _.space + stmtToJs($left.declarations[0], Preset.s6);
	        _.indent = prevIndent2;
	    }

	    else
	        stmtJs += exprToJs($left, Preset.e10);

	    stmtJs = join(stmtJs, $op);

	    var rightJs = exprToJs($stmt.right, Preset.e4);

	    stmtJs = join(stmtJs, rightJs) + ')';

	    _.indent = prevIndent1;

	    _.js += stmtJs + adoptionPrefix($body);
	    StmtGen[$body.type]($body, Preset.s4(bodySemicolonOptional));
	}


	//Statement generator dictionary
	var StmtRawGen = {
	    BlockStatement: function generateBlockStatement ($stmt, settings) {
	        var $body      = $stmt.body,
	            len        = $body.length,
	            lastIdx    = len - 1,
	            prevIndent = shiftIndent();

	        _.js += '{' + _.newline;

	        for (var i = 0; i < len; i++) {
	            var $item = $body[i];

	            _.js += _.indent;
	            StmtGen[$item.type]($item, Preset.s1(settings.functionBody, i === lastIdx));
	            _.js += _.newline;
	        }

	        _.indent = prevIndent;
	        _.js += _.indent + '}';
	    },

	    BreakStatement: function generateBreakStatement ($stmt, settings) {
	        if ($stmt.label)
	            _.js += 'break ' + $stmt.label.name;

	        else
	            _.js += 'break';

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    ContinueStatement: function generateContinueStatement ($stmt, settings) {
	        if ($stmt.label)
	            _.js += 'continue ' + $stmt.label.name;

	        else
	            _.js += 'continue';

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    ClassBody: function generateClassBody ($stmt) {
	        var $body       = $stmt.body,
	            itemCount   = $body.length,
	            lastItemIdx = itemCount - 1,
	            prevIndent  = shiftIndent();

	        _.js += '{' + _.newline;

	        for (var i = 0; i < itemCount; i++) {
	            var $item    = $body[i],
	                itemType = $item.type || Syntax.Property;

	            _.js += _.indent;
	            ExprGen[itemType]($item, Preset.e5);

	            if (i !== lastItemIdx)
	                _.js += _.newline;
	        }

	        _.indent = prevIndent;
	        _.js += _.newline + _.indent + '}';
	    },

	    ClassDeclaration: function generateClassDeclaration ($stmt) {
	        var $body  = $stmt.body,
	            $super = $stmt.superClass,
	            js     = 'class ' + $stmt.id.name;

	        if ($super) {
	            var superJs = exprToJs($super, Preset.e4);

	            js += _.space + join('extends', superJs);
	        }

	        _.js += js + _.optSpace;
	        StmtGen[$body.type]($body, Preset.s2);
	    },

	    DirectiveStatement: function generateDirectiveStatement ($stmt, settings) {
	        if (extra.raw && $stmt.raw)
	            _.js += $stmt.raw;

	        else
	            _.js += escapeDirective($stmt.directive);

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    DoWhileStatement: function generateDoWhileStatement ($stmt, settings) {
	        var $body  = $stmt.body,
	            $test  = $stmt.test,
	            bodyJs = adoptionPrefix($body) +
	                     stmtToJs($body, Preset.s7) +
	                     adoptionSuffix($body);

	        //NOTE: Because `do 42 while (cond)` is Syntax Error. We need semicolon.
	        var stmtJs = join('do', bodyJs);

	        _.js += join(stmtJs, 'while' + _.optSpace + '(');
	        ExprGen[$test.type]($test, Preset.e5);
	        _.js += ')';

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    CatchClause: function generateCatchClause ($stmt) {
	        var $param     = $stmt.param,
	            $guard     = $stmt.guard,
	            $body      = $stmt.body,
	            prevIndent = shiftIndent();

	        _.js += 'catch' + _.optSpace;
	        
	        if ($param) {
	           _.js += '(';
	           ExprGen[$param.type]($param, Preset.e5);
	        }

	        if ($guard) {
	            _.js += ' if ';
	            ExprGen[$guard.type]($guard, Preset.e5);
	        }

	        _.indent = prevIndent;
	        if ($param) {
	           _.js += ')';
	        } 
	     
	        _.js += adoptionPrefix($body);
	        StmtGen[$body.type]($body, Preset.s7);
	    },

	    DebuggerStatement: function generateDebuggerStatement ($stmt, settings) {
	        _.js += 'debugger';

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    EmptyStatement: function generateEmptyStatement () {
	        _.js += ';';
	    },

	    ExportAllDeclaration: function ($stmt, settings) {
	        StmtRawGen.ExportDeclaration($stmt, settings, true);
	    },

	    ExportDeclaration: function generateExportDeclaration ($stmt, settings, exportAll) {
	        var $specs        = $stmt.specifiers,
	            $decl         = $stmt.declaration,
	            withSemicolon = semicolons || !settings.semicolonOptional;

	        // export default AssignmentExpression[In] ;
	        if ($stmt['default']) {
	            var declJs = exprToJs($decl, Preset.e4);

	            _.js += join('export default', declJs);

	            if (withSemicolon)
	                _.js += ';';
	        }

	        // export * FromClause ;
	        // export ExportClause[NoReference] FromClause ;
	        // export ExportClause ;
	        else if ($specs || exportAll) {
	            var stmtJs = 'export';

	            if (exportAll)
	                stmtJs += _.optSpace + '*';

	            else if ($specs.length === 0)
	                stmtJs += _.optSpace + '{' + _.optSpace + '}';

	            else if ($specs[0].type === Syntax.ExportBatchSpecifier) {
	                var specJs = exprToJs($specs[0], Preset.e5);

	                stmtJs = join(stmtJs, specJs);
	            }

	            else {
	                var prevIndent  = shiftIndent(),
	                    specCount   = $specs.length,
	                    lastSpecIdx = specCount - 1;

	                stmtJs += _.optSpace + '{';

	                for (var i = 0; i < specCount; ++i) {
	                    stmtJs += _.newline + _.indent;
	                    stmtJs += exprToJs($specs[i], Preset.e5);

	                    if (i !== lastSpecIdx)
	                        stmtJs += ',';
	                }

	                _.indent = prevIndent;
	                stmtJs += _.newline + _.indent + '}';
	            }

	            if ($stmt.source) {
	                _.js += join(stmtJs, 'from' + _.optSpace);
	                ExprGen.Literal($stmt.source);
	            }

	            else
	                _.js += stmtJs;

	            if (withSemicolon)
	                _.js += ';';
	        }

	        // export VariableStatement
	        // export Declaration[Default]
	        else if ($decl) {
	            var declJs = stmtToJs($decl, Preset.s4(!withSemicolon));

	            _.js += join('export', declJs);
	        }
	    },

	    ExportNamedDeclaration: function ($stmt, settings) {
	        StmtRawGen.ExportDeclaration($stmt, settings);
	    },

	    ExpressionStatement: function generateExpressionStatement ($stmt, settings) {
	        var exprJs       = exprToJs($stmt.expression, Preset.e5),
	            parenthesize = EXPR_STMT_UNALLOWED_EXPR_REGEXP.test(exprJs) ||
	                           (directive &&
	                            settings.directiveContext &&
	                            $stmt.expression.type === Syntax.Literal &&
	                            typeof $stmt.expression.value === 'string');

	        //NOTE: '{', 'function', 'class' are not allowed in expression statement.
	        // Therefore, they should be parenthesized.
	        if (parenthesize)
	            _.js += '(' + exprJs + ')';

	        else
	            _.js += exprJs;

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    ImportDeclaration: function generateImportDeclaration ($stmt, settings) {
	        var $specs    = $stmt.specifiers,
	            stmtJs    = 'import',
	            specCount = $specs.length;

	        //NOTE: If no ImportClause is present,
	        // this should be `import ModuleSpecifier` so skip `from`
	        // ModuleSpecifier is StringLiteral.
	        if (specCount) {
	            var hasBinding    = !!$specs[0]['default'],
	                firstNamedIdx = hasBinding ? 1 : 0,
	                lastSpecIdx   = specCount - 1;

	            // ImportedBinding
	            if (hasBinding)
	                stmtJs = join(stmtJs, $specs[0].id.name);

	            // NamedImports
	            if (firstNamedIdx < specCount) {
	                if (hasBinding)
	                    stmtJs += ',';

	                stmtJs += _.optSpace + '{';

	                // import { ... } from "...";
	                if (firstNamedIdx === lastSpecIdx)
	                    stmtJs += _.optSpace + exprToJs($specs[firstNamedIdx], Preset.e5) + _.optSpace;

	                else {
	                    var prevIndent = shiftIndent();

	                    // import {
	                    //    ...,
	                    //    ...,
	                    // } from "...";
	                    for (var i = firstNamedIdx; i < specCount; i++) {
	                        stmtJs += _.newline + _.indent + exprToJs($specs[i], Preset.e5);

	                        if (i !== lastSpecIdx)
	                            stmtJs += ',';
	                    }

	                    _.indent = prevIndent;
	                    stmtJs += _.newline + _.indent;
	                }

	                stmtJs += '}' + _.optSpace;
	            }

	            stmtJs = join(stmtJs, 'from');
	        }

	        _.js += stmtJs + _.optSpace;
	        ExprGen.Literal($stmt.source);

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    VariableDeclarator: function generateVariableDeclarator ($stmt, settings) {
	        var $id         = $stmt.id,
	            $init       = $stmt.init,
	            genSettings = Preset.e1(settings.allowIn);

	        if ($init) {
	            ExprGen[$id.type]($id, genSettings);
	            _.js += _.optSpace + '=' + _.optSpace;
	            ExprGen[$init.type]($init, genSettings);
	        }

	        else {
	            if ($id.type === Syntax.Identifier)
	                _.js += $id.name;

	            else
	                ExprGen[$id.type]($id, genSettings);
	        }
	    },

	    VariableDeclaration: function generateVariableDeclaration ($stmt, settings) {
	        var $decls          = $stmt.declarations,
	            len             = $decls.length,
	            prevIndent      = len > 1 ? shiftIndent() : _.indent,
	            declGenSettings = Preset.s3(settings.allowIn);

	        _.js += $stmt.kind;

	        for (var i = 0; i < len; i++) {
	            var $decl = $decls[i];

	            _.js += i === 0 ? _.space : (',' + _.optSpace);
	            StmtGen[$decl.type]($decl, declGenSettings);
	        }

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';

	        _.indent = prevIndent;
	    },

	    ThrowStatement: function generateThrowStatement ($stmt, settings) {
	        var argJs = exprToJs($stmt.argument, Preset.e5);

	        _.js += join('throw', argJs);

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    TryStatement: function generateTryStatement ($stmt) {
	        var $block     = $stmt.block,
	            $finalizer = $stmt.finalizer,
	            stmtJs     = 'try' +
	                         adoptionPrefix($block) +
	                         stmtToJs($block, Preset.s7) +
	                         adoptionSuffix($block);

	        var $handlers = $stmt.handlers || $stmt.guardedHandlers;

	        if ($handlers)
	            stmtJs = generateTryStatementHandlers(stmtJs, $finalizer, $handlers);

	        if ($stmt.handler) {
	            $handlers = isArray($stmt.handler) ? $stmt.handler : [$stmt.handler];
	            stmtJs    = generateTryStatementHandlers(stmtJs, $finalizer, $handlers);
	        }

	        if ($finalizer) {
	            stmtJs = join(stmtJs, 'finally' + adoptionPrefix($finalizer));
	            stmtJs += stmtToJs($finalizer, Preset.s7);
	        }

	        _.js += stmtJs;
	    },

	    SwitchStatement: function generateSwitchStatement ($stmt) {
	        var $cases     = $stmt.cases,
	            $discr     = $stmt.discriminant,
	            prevIndent = shiftIndent();

	        _.js += 'switch' + _.optSpace + '(';
	        ExprGen[$discr.type]($discr, Preset.e5);
	        _.js += ')' + _.optSpace + '{' + _.newline;
	        _.indent = prevIndent;

	        if ($cases) {
	            var caseCount   = $cases.length,
	                lastCaseIdx = caseCount - 1;

	            for (var i = 0; i < caseCount; i++) {
	                var $case = $cases[i];

	                _.js += _.indent;
	                StmtGen[$case.type]($case, Preset.s4(i === lastCaseIdx));
	                _.js += _.newline;
	            }
	        }

	        _.js += _.indent + '}';
	    },

	    SwitchCase: function generateSwitchCase ($stmt, settings) {
	        var $conseqs                = $stmt.consequent,
	            $firstConseq            = $conseqs[0],
	            $test                   = $stmt.test,
	            i                       = 0,
	            conseqSemicolonOptional = !semicolons && settings.semicolonOptional,
	            conseqCount             = $conseqs.length,
	            lastConseqIdx           = conseqCount - 1,
	            prevIndent              = shiftIndent();

	        if ($test) {
	            var testJs = exprToJs($test, Preset.e5);

	            _.js += join('case', testJs) + ':';
	        }

	        else
	            _.js += 'default:';


	        if (conseqCount && $firstConseq.type === Syntax.BlockStatement) {
	            i++;
	            _.js += adoptionPrefix($firstConseq);
	            StmtGen[$firstConseq.type]($firstConseq, Preset.s7);
	        }

	        for (; i < conseqCount; i++) {
	            var $conseq           = $conseqs[i],
	                semicolonOptional = i === lastConseqIdx && conseqSemicolonOptional;

	            _.js += _.newline + _.indent;
	            StmtGen[$conseq.type]($conseq, Preset.s4(semicolonOptional));
	        }

	        _.indent = prevIndent;
	    },

	    IfStatement: function generateIfStatement ($stmt, settings) {
	        var $conseq           = $stmt.consequent,
	            $test             = $stmt.test,
	            prevIndent        = shiftIndent(),
	            semicolonOptional = !semicolons && settings.semicolonOptional;

	        _.js += 'if' + _.optSpace + '(';
	        ExprGen[$test.type]($test, Preset.e5);
	        _.js += ')';
	        _.indent = prevIndent;
	        _.js += adoptionPrefix($conseq);

	        if ($stmt.alternate) {
	            var conseq = stmtToJs($conseq, Preset.s7) + adoptionSuffix($conseq),
	                alt    = stmtToJs($stmt.alternate, Preset.s4(semicolonOptional));

	            if ($stmt.alternate.type === Syntax.IfStatement)
	                alt = 'else ' + alt;

	            else
	                alt = join('else', adoptionPrefix($stmt.alternate) + alt);

	            _.js += join(conseq, alt);
	        }

	        else
	            StmtGen[$conseq.type]($conseq, Preset.s4(semicolonOptional));
	    },

	    ForStatement: function generateForStatement ($stmt, settings) {
	        var $init                 = $stmt.init,
	            $test                 = $stmt.test,
	            $body                 = $stmt.body,
	            $update               = $stmt.update,
	            bodySemicolonOptional = !semicolons && settings.semicolonOptional,
	            prevIndent            = shiftIndent();

	        _.js += 'for' + _.optSpace + '(';

	        if ($init) {
	            if ($init.type === Syntax.VariableDeclaration)
	                StmtGen[$init.type]($init, Preset.s6);

	            else {
	                ExprGen[$init.type]($init, Preset.e14);
	                _.js += ';';
	            }
	        }

	        else
	            _.js += ';';

	        if ($test) {
	            _.js += _.optSpace;
	            ExprGen[$test.type]($test, Preset.e5);
	        }

	        _.js += ';';

	        if ($update) {
	            _.js += _.optSpace;
	            ExprGen[$update.type]($update, Preset.e5);
	        }

	        _.js += ')';
	        _.indent = prevIndent;
	        _.js += adoptionPrefix($body);
	        StmtGen[$body.type]($body, Preset.s4(bodySemicolonOptional));
	    },

	    ForInStatement: function generateForInStatement ($stmt, settings) {
	        generateForStatementIterator('in', $stmt, settings);
	    },

	    ForOfStatement: function generateForOfStatement ($stmt, settings) {
	        generateForStatementIterator('of', $stmt, settings);
	    },

	    LabeledStatement: function generateLabeledStatement ($stmt, settings) {
	        var $body                 = $stmt.body,
	            bodySemicolonOptional = !semicolons && settings.semicolonOptional,
	            prevIndent            = _.indent;

	        _.js += $stmt.label.name + ':' + adoptionPrefix($body);

	        if ($body.type !== Syntax.BlockStatement)
	            prevIndent = shiftIndent();

	        StmtGen[$body.type]($body, Preset.s4(bodySemicolonOptional));
	        _.indent       = prevIndent;
	    },

	    ModuleDeclaration: function generateModuleDeclaration ($stmt, settings) {
	        _.js += 'module' + _.space + $stmt.id.name + _.space + 'from' + _.optSpace;

	        ExprGen.Literal($stmt.source);

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    Program: function generateProgram ($stmt) {
	        var $body   = $stmt.body,
	            len     = $body.length,
	            lastIdx = len - 1;

	        if (safeConcatenation && len > 0)
	            _.js += '\n';

	        for (var i = 0; i < len; i++) {
	            var $item = $body[i];

	            _.js += _.indent;
	            StmtGen[$item.type]($item, Preset.s5(!safeConcatenation && i === lastIdx));

	            if (i !== lastIdx)
	                _.js += _.newline;
	        }
	    },

	    FunctionDeclaration: function generateFunctionDeclaration ($stmt) {
	        var isGenerator = !!$stmt.generator;

	        if ($stmt.async)
	            _.js += 'async ';

	        _.js += isGenerator ? ('function*' + _.optSpace) : ('function' + _.space );
	        _.js += $stmt.id.name;
	        generateFunctionBody($stmt);
	    },

	    ReturnStatement: function generateReturnStatement ($stmt, settings) {
	        var $arg = $stmt.argument;

	        if ($arg) {
	            var argJs = exprToJs($arg, Preset.e5);

	            _.js += join('return', argJs);
	        }

	        else
	            _.js += 'return';

	        if (semicolons || !settings.semicolonOptional)
	            _.js += ';';
	    },

	    WhileStatement: function generateWhileStatement ($stmt, settings) {
	        var $body                 = $stmt.body,
	            $test                 = $stmt.test,
	            bodySemicolonOptional = !semicolons && settings.semicolonOptional,
	            prevIndent            = shiftIndent();

	        _.js += 'while' + _.optSpace + '(';
	        ExprGen[$test.type]($test, Preset.e5);
	        _.js += ')';
	        _.indent = prevIndent;

	        _.js += adoptionPrefix($body);
	        StmtGen[$body.type]($body, Preset.s4(bodySemicolonOptional));
	    },

	    WithStatement: function generateWithStatement ($stmt, settings) {
	        var $body                 = $stmt.body,
	            $obj                  = $stmt.object,
	            bodySemicolonOptional = !semicolons && settings.semicolonOptional,
	            prevIndent            = shiftIndent();

	        _.js += 'with' + _.optSpace + '(';
	        ExprGen[$obj.type]($obj, Preset.e5);
	        _.js += ')';
	        _.indent = prevIndent;
	        _.js += adoptionPrefix($body);
	        StmtGen[$body.type]($body, Preset.s4(bodySemicolonOptional));
	    }
	};

	//CodeGen
	//-----------------------------------------------------------------------------------
	function exprToJs ($expr, settings) {
	    var savedJs = _.js;
	    _.js        = '';

	    ExprGen[$expr.type]($expr, settings);

	    var src = _.js;
	    _.js    = savedJs;

	    return src;
	}

	function stmtToJs ($stmt, settings) {
	    var savedJs = _.js;
	    _.js        = '';

	    StmtGen[$stmt.type]($stmt, settings);

	    var src = _.js;
	    _.js    = savedJs;

	    return src;
	}

	function run ($node) {
	    _.js = '';

	    if (StmtGen[$node.type])
	        StmtGen[$node.type]($node, Preset.s7);

	    else
	        ExprGen[$node.type]($node, Preset.e19);

	    return _.js;
	}

	function wrapExprGen (gen) {
	    return function ($expr, settings) {
	        if (extra.verbatim && $expr.hasOwnProperty(extra.verbatim))
	            generateVerbatim($expr, settings);

	        else
	            gen($expr, settings);
	    }
	}

	function createExprGenWithExtras () {
	    var gens = {};

	    for (var key in ExprRawGen) {
	        if (ExprRawGen.hasOwnProperty(key))
	            gens[key] = wrapExprGen(ExprRawGen[key]);
	    }

	    return gens;
	}


	//Strings
	var _ = {
	    js:         '',
	    newline:    '\n',
	    optSpace:   ' ',
	    space:      ' ',
	    indentUnit: '    ',
	    indent:     ''
	};


	//Generators
	var ExprGen = void 0,
	    StmtGen = StmtRawGen;


	var generate = function ($node, options) {
	    var defaultOptions = getDefaultOptions();

	    if (options != null) {
	        //NOTE: Obsolete options
	        //
	        //   `options.indent`
	        //   `options.base`
	        //
	        // Instead of them, we can use `option.format.indent`.
	        if (typeof options.indent === 'string') {
	            defaultOptions.format.indent.style = options.indent;
	        }
	        if (typeof options.base === 'number') {
	            defaultOptions.format.indent.base = options.base;
	        }
	        options      = updateDeeply(defaultOptions, options);
	        _.indentUnit = options.format.indent.style;
	        if (typeof options.base === 'string') {
	            _.indent = options.base;
	        }
	        else {
	            _.indent = stringRepeat(_.indentUnit, options.format.indent.base);
	        }
	    }
	    else {
	        options      = defaultOptions;
	        _.indentUnit = options.format.indent.style;
	        _.indent     = stringRepeat(_.indentUnit, options.format.indent.base);
	    }
	    json$1        = options.format.json;
	    renumber    = options.format.renumber;
	    hexadecimal = json$1 ? false : options.format.hexadecimal;
	    quotes      = json$1 ? 'double' : options.format.quotes;
	    escapeless  = options.format.escapeless;

	    _.newline  = options.format.newline;
	    _.optSpace = options.format.space;

	    if (options.format.compact)
	        _.newline = _.optSpace = _.indentUnit = _.indent = '';

	    _.space           = _.optSpace ? _.optSpace : ' ';
	    parentheses       = options.format.parentheses;
	    semicolons        = options.format.semicolons;
	    safeConcatenation = options.format.safeConcatenation;
	    directive         = options.directive;
	    json$1 ? null : options.parse;
	    extra             = options;

	    if (extra.verbatim)
	        ExprGen = createExprGenWithExtras();

	    else
	        ExprGen = ExprRawGen;

	    return run($node);
	};

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	var SCRIPT_PROCESSING_INSTRUCTIONS = {
	    getLocation: '__get$Loc',
	    setLocation: '__set$Loc',
	    getProperty: '__get$',
	    setProperty: '__set$',
	    callMethod: '__call$',
	    processScript: '__proc$Script',
	    processHtml: '__proc$Html',
	    getEval: '__get$Eval',
	    getPostMessage: '__get$PostMessage',
	    getProxyUrl: '__get$ProxyUrl',
	    restArray: '__rest$Array',
	    restObject: '__rest$Object',
	    swScopeHeaderValue: '__swScopeHeaderValue'
	};

	var TEMP_VARIABLE_PREFIX = '_hh$temp';
	var TempVariables = /** @class */ (function () {
	    function TempVariables() {
	        this._list = [];
	    }
	    TempVariables.resetCounter = function () {
	        TempVariables._counter = 0;
	    };
	    TempVariables.generateName = function (baseName, key, index) {
	        if (!baseName)
	            return TEMP_VARIABLE_PREFIX + TempVariables._counter++;
	        if (key) {
	            if (key.type === Syntax_1.Identifier)
	                return baseName + '$' + key.name;
	            if (key.type === Syntax_1.Literal)
	                return baseName + '$' + key.value.toString().replace(/[^a-zA-Z0-9]/g, '');
	        }
	        return baseName + '$i' + index;
	    };
	    TempVariables.isHHTempVariable = function (name) {
	        return name.indexOf(TEMP_VARIABLE_PREFIX) === 0;
	    };
	    TempVariables.prototype.append = function (name) {
	        this._list.push(name);
	    };
	    TempVariables.prototype.get = function () {
	        return this._list;
	    };
	    TempVariables._counter = 0;
	    return TempVariables;
	}());

	function createIdentifier(name) {
	    return { type: Syntax_1.Identifier, name: name };
	}
	function createExpressionStatement(expression) {
	    return { type: Syntax_1.ExpressionStatement, expression: expression };
	}
	function createAssignmentExpression(left, operator, right) {
	    return { type: Syntax_1.AssignmentExpression, operator: operator, left: left, right: right };
	}
	function createSimpleCallExpression(callee, args) {
	    return { type: Syntax_1.CallExpression, callee: callee, arguments: args, optional: false };
	}
	function createArrayExpression(elements) {
	    return { type: Syntax_1.ArrayExpression, elements: elements };
	}
	function createMemberExpression(object, property, computed) {
	    return { type: Syntax_1.MemberExpression, object: object, property: property, computed: computed, optional: false };
	}
	function createBinaryExpression(left, operator, right) {
	    return { type: Syntax_1.BinaryExpression, left: left, right: right, operator: operator };
	}
	function createSequenceExpression(expressions) {
	    return { type: Syntax_1.SequenceExpression, expressions: expressions };
	}
	function createThisExpression() {
	    return { type: Syntax_1.ThisExpression };
	}
	function createLogicalExpression(left, operator, right) {
	    return { type: Syntax_1.LogicalExpression, left: left, right: right, operator: operator };
	}
	function createReturnStatement(argument) {
	    if (argument === void 0) { argument = null; }
	    return { type: Syntax_1.ReturnStatement, argument: argument };
	}
	function createFunctionExpression(id, params, body, async, generator) {
	    if (async === void 0) { async = false; }
	    if (generator === void 0) { generator = false; }
	    return { type: Syntax_1.FunctionExpression, id: id, params: params, body: body, async: async, generator: generator };
	}
	function createUnaryExpression(operator, argument) {
	    return { type: Syntax_1.UnaryExpression, operator: operator, prefix: true, argument: argument };
	}
	function createUndefined() {
	    return createUnaryExpression('void', createSimpleLiteral(0));
	}
	function createConditionalExpression(test, consequent, alternate) {
	    return { type: Syntax_1.ConditionalExpression, test: test, consequent: consequent, alternate: alternate };
	}
	function createSimpleLiteral(value) {
	    return { type: Syntax_1.Literal, value: value };
	}
	function createAssignmentExprStmt(left, right) {
	    return createExpressionStatement(createAssignmentExpression(left, '=', right));
	}
	function createBlockStatement(body) {
	    return { type: Syntax_1.BlockStatement, body: body };
	}
	function createVariableDeclarator(id, init) {
	    if (init === void 0) { init = null; }
	    return { type: Syntax_1.VariableDeclarator, id: id, init: init };
	}
	function createVariableDeclaration(kind, declarations) {
	    return { type: Syntax_1.VariableDeclaration, declarations: declarations, kind: kind };
	}
	function createProcessScriptMethodCall(arg, isApply) {
	    var args = [arg];
	    var processScriptIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.processScript);
	    if (isApply)
	        args.push(createSimpleLiteral(true));
	    return createSimpleCallExpression(processScriptIdentifier, args);
	}
	function createLocationGetWrapper(location) {
	    var getLocationIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getLocation);
	    return createSimpleCallExpression(getLocationIdentifier, [location]);
	}
	function createLocationSetWrapper(locationIdentifier, value, wrapWithSequence) {
	    var tempIdentifier = createIdentifier(TempVariables.generateName());
	    var setLocationIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.setLocation);
	    var setLocationCall = createSimpleCallExpression(setLocationIdentifier, [locationIdentifier, tempIdentifier]);
	    var locationAssignment = createAssignmentExpression(locationIdentifier, '=', tempIdentifier);
	    var callIdentifier = createIdentifier('call');
	    var functionWrapper = createFunctionExpression(null, [], createBlockStatement([
	        createVariableDeclaration('var', [createVariableDeclarator(tempIdentifier, value)]),
	        createReturnStatement(createLogicalExpression(setLocationCall, '||', locationAssignment))
	    ]));
	    var functionWrapperCallMember = createMemberExpression(functionWrapper, callIdentifier, false);
	    var functionWrapperCall = createSimpleCallExpression(functionWrapperCallMember, [createThisExpression()]);
	    if (wrapWithSequence)
	        return createSequenceExpression([createSimpleLiteral(0), functionWrapperCall]);
	    return functionWrapperCall;
	}
	function createPropertySetWrapper(propertyName, obj, value) {
	    var setPropertyIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.setProperty);
	    return createSimpleCallExpression(setPropertyIdentifier, [obj, createSimpleLiteral(propertyName), value]);
	}
	function createMethodCallWrapper(owner, method, args) {
	    var callMethodIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.callMethod);
	    var methodArgsArray = createArrayExpression(args);
	    return createSimpleCallExpression(callMethodIdentifier, [owner, method, methodArgsArray]);
	}
	function createPropertyGetWrapper(propertyName, owner) {
	    var getPropertyIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getProperty);
	    return createSimpleCallExpression(getPropertyIdentifier, [owner, createSimpleLiteral(propertyName)]);
	}
	function createComputedPropertyGetWrapper(property, owner) {
	    var getPropertyIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getProperty);
	    return createSimpleCallExpression(getPropertyIdentifier, [owner, property]);
	}
	function createComputedPropertySetWrapper(property, owner, value) {
	    var setPropertyIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.setProperty);
	    return createSimpleCallExpression(setPropertyIdentifier, [owner, property, value]);
	}
	function createGetEvalMethodCall(node) {
	    var getEvalIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getEval);
	    return createSimpleCallExpression(getEvalIdentifier, [node]);
	}
	function getProxyUrlLiteral(source, resolver) {
	    var proxyUrl = resolver(String(source.value), getResourceTypeString({ isScript: true }));
	    return createSimpleLiteral(proxyUrl);
	}
	function createGetProxyUrlMethodCall(arg, baseUrl) {
	    var getProxyUrlIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl);
	    var args = [arg];
	    if (baseUrl)
	        args.push(createSimpleLiteral(baseUrl));
	    return createSimpleCallExpression(getProxyUrlIdentifier, args);
	}
	function createGetPostMessageMethodCall(node) {
	    var getPostMessageIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage);
	    var args = node.type === Syntax_1.MemberExpression
	        ? [node.object]
	        : [createSimpleLiteral(null), node];
	    return createSimpleCallExpression(getPostMessageIdentifier, args);
	}
	function createExpandedConcatOperation(left, right) {
	    return createAssignmentExpression(left, '=', createBinaryExpression(left, '+', right));
	}
	function createHtmlProcessorWrapper(node) {
	    var processHtmlIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.processHtml);
	    var parentIdentifier = createIdentifier('parent');
	    var windowIdentifier = createIdentifier('window');
	    var processHtmlThroughParent = createMemberExpression(parentIdentifier, processHtmlIdentifier, false);
	    var processHtmlCall = createSimpleCallExpression(processHtmlThroughParent, [windowIdentifier, node.expression]);
	    return createExpressionStatement(processHtmlCall);
	}
	function createTempVarsDeclaration(tempVars) {
	    var declarations = [];
	    for (var _i = 0, tempVars_1 = tempVars; _i < tempVars_1.length; _i++) {
	        var variable = tempVars_1[_i];
	        declarations.push(createVariableDeclarator(createIdentifier(variable)));
	    }
	    return createVariableDeclaration('var', declarations);
	}

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	// NOTE: constants are exported for the testing purposes
	var METHODS = [
	    'postMessage',
	    'replace',
	    'assign'
	];
	var PROPERTIES = [
	    'href',
	    'location'
	];
	var INSTRUMENTED_METHOD_RE = new RegExp("^(" + METHODS.join('|') + ")$");
	var INSTRUMENTED_PROPERTY_RE = new RegExp("^(" + PROPERTIES.join('|') + ")$");
	// NOTE: Mootools framework contains code that removes the RegExp.prototype.test
	// method and restores it later.
	//    delete z[A]; // z = RegExp.prototype, A = "test"
	//    __set$(z, A, x.protect()); // x.protect - returns the removed method
	// The __set$ function calls the test method of the regular expression. (GH-331)
	var reTest = RegExp.prototype.test;
	// NOTE: The Function.prototype.call method can also be removed.
	// But only one of the methods can be removed at a time.
	var test = function (regexp, str) { return regexp.test ? regexp.test(str) : reTest.call(regexp, str); };
	// NOTE: we can't use the map approach here, because
	// cases like `WRAPPABLE_METHOD['toString']` will fail.
	// We could use the hasOwnProperty test, but it is
	// significantly slower than the regular expression test
	function shouldInstrumentMethod(name) {
	    return test(INSTRUMENTED_METHOD_RE, String(name));
	}
	function shouldInstrumentProperty(name) {
	    return test(INSTRUMENTED_PROPERTY_RE, String(name));
	}

	var instrumentationProcessingUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		METHODS: METHODS,
		PROPERTIES: PROPERTIES,
		shouldInstrumentMethod: shouldInstrumentMethod,
		shouldInstrumentProperty: shouldInstrumentProperty
	});

	// -------------------------------------------------------------
	// Transform:
	// obj[prop] -->
	// __get$(obj, prop)
	var transformer$m = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.MemberExpression,
	    condition: function (node, parent) {
	        if (!node.computed || !parent)
	            return false;
	        if (node.property.type === Syntax_1.Literal && !shouldInstrumentProperty(node.property.value))
	            return false;
	        // super[prop]
	        if (node.object.type === Syntax_1.Super)
	            return false;
	        // object[prop] = value
	        if (parent.type === Syntax_1.AssignmentExpression && parent.left === node)
	            return false;
	        // delete object[prop]
	        if (parent.type === Syntax_1.UnaryExpression && parent.operator === 'delete')
	            return false;
	        // object[prop]++ || object[prop]-- || ++object[prop] || --object[prop]
	        if (parent.type === Syntax_1.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
	            return false;
	        // object[prop]()
	        if (parent.type === Syntax_1.CallExpression && parent.callee === node)
	            return false;
	        // new (object[prop])() || new (object[prop])
	        if (parent.type === Syntax_1.NewExpression && parent.callee === node)
	            return false;
	        // for(object[prop] in source)
	        if (parent.type === Syntax_1.ForInStatement && parent.left === node)
	            return false;
	        return true;
	    },
	    run: function (node) { return createComputedPropertyGetWrapper(node.property, node.object); }
	};

	// -------------------------------------------------------------
	// Transform:
	// obj[prop] = value -->
	// __set$(object, prop, value)
	var transformer$l = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.AssignmentExpression,
	    condition: function (node) {
	        var left = node.left;
	        // super[prop] = value
	        if (left.type === Syntax_1.MemberExpression && left.object.type === Syntax_1.Super)
	            return false;
	        if (node.operator === '=' && left.type === Syntax_1.MemberExpression && left.computed)
	            return left.property.type === Syntax_1.Literal ? shouldInstrumentProperty(left.property.value) : true;
	        return false;
	    },
	    run: function (node) {
	        var memberExpression = node.left;
	        return createComputedPropertySetWrapper(memberExpression.property, memberExpression.object, node.right);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// val1 += val2
	// --> val1 = val1 + val2
	var transformer$k = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.AssignmentExpression,
	    condition: function (node) {
	        if (node.operator !== '+=')
	            return false;
	        var left = node.left;
	        // location
	        if (left.type === Syntax_1.Identifier)
	            return shouldInstrumentProperty(left.name);
	        if (left.type === Syntax_1.MemberExpression) {
	            // something['location'] or something[propname]
	            if (left.computed)
	                return left.property.type === Syntax_1.Literal
	                    ? shouldInstrumentProperty(left.property.value)
	                    : left.property.type !== Syntax_1.UpdateExpression;
	            // something.location
	            else if (left.property.type === Syntax_1.Identifier)
	                return shouldInstrumentProperty(left.property.name);
	        }
	        return false;
	    },
	    run: function (node) { return createExpandedConcatOperation(node.left, node.right); }
	};

	function replaceNode(node, newNode, parent, key) {
	    var oldNode = parent[key];
	    if (oldNode instanceof Array) {
	        if (node)
	            oldNode[oldNode.indexOf(node)] = newNode;
	        else
	            oldNode.unshift(newNode);
	    }
	    else {
	        // @ts-ignore
	        parent[key] = newNode;
	    }
	    if (node) {
	        newNode.originStart = newNode.start = node.start;
	        newNode.originEnd = newNode.end = node.end;
	    }
	    else {
	        /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
	        newNode.start = newNode.end = newNode.originStart = newNode.originEnd = oldNode[1] ? oldNode[1].start : parent.start + 1;
	    }
	}

	// -------------------------------------------------------------
	// Transform:
	// eval(script); --> eval(__proc$Script(script));
	var transformer$j = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.CallExpression,
	    condition: function (node) {
	        if (!node.arguments.length)
	            return false;
	        var callee = node.callee;
	        // eval()
	        if (callee.type === Syntax_1.Identifier && callee.name === 'eval')
	            return true;
	        // obj.eval(), obj['eval'](),
	        return callee.type === Syntax_1.MemberExpression &&
	            (callee.property.type === Syntax_1.Identifier && callee.property.name ||
	                callee.property.type === Syntax_1.Literal && callee.property.value) === 'eval';
	    },
	    run: function (node) {
	        var newArgs = createProcessScriptMethodCall(node.arguments[0]);
	        replaceNode(node.arguments[0], newArgs, node, 'arguments');
	        return null;
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// foo = eval.bind(...); -->
	// foo = __get$Eval(eval).bind(...);
	var transformer$i = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.CallExpression,
	    condition: function (node) {
	        if (node.callee.type === Syntax_1.MemberExpression && node.callee.property.type === Syntax_1.Identifier &&
	            node.callee.property.name === 'bind') {
	            var obj = node.callee.object;
	            // obj.eval.bind(), obj[eval].bind()
	            if (obj.type === Syntax_1.MemberExpression &&
	                (obj.property.type === Syntax_1.Identifier && obj.property.name ||
	                    obj.property.type === Syntax_1.Literal && obj.property.value) === 'eval')
	                return true;
	            // eval.bind()
	            if (obj.type === Syntax_1.Identifier && obj.name === 'eval')
	                return true;
	        }
	        return false;
	    },
	    run: function (node) {
	        var callee = node.callee;
	        var getEvalNode = createGetEvalMethodCall(callee.object);
	        replaceNode(callee.object, getEvalNode, callee, 'object');
	        return null;
	    }
	};

	// -------------------------------------------------------------
	var INVOCATION_FUNC_NAME_RE$1 = /^(call|apply)$/;
	// Transform:
	// eval.call(ctx, script);
	// eval.apply(ctx, script); -->
	// eval.call(ctx, __proc$Script(script));
	// eval.apply(ctx, __proc$Script(script, true));
	var transformer$h = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.CallExpression,
	    condition: function (node) {
	        // eval.<meth>(ctx, script, ...)
	        if (node.arguments.length < 2)
	            return false;
	        if (node.callee.type === Syntax_1.MemberExpression && node.callee.property.type === Syntax_1.Identifier &&
	            INVOCATION_FUNC_NAME_RE$1.test(node.callee.property.name)) {
	            var obj = node.callee.object;
	            // eval.<meth>()
	            if (obj.type === Syntax_1.Identifier && obj.name === 'eval')
	                return true;
	            // obj.eval.<meth>(), obj[eval].<meth>()
	            if (obj.type === Syntax_1.MemberExpression &&
	                (obj.property.type === Syntax_1.Identifier && obj.property.name ||
	                    obj.property.type === Syntax_1.Literal && obj.property.value) === 'eval')
	                return true;
	        }
	        return false;
	    },
	    run: function (node) {
	        var callee = node.callee;
	        var property = callee.property;
	        var newArg = createProcessScriptMethodCall(node.arguments[1], property.name === 'apply');
	        replaceNode(node.arguments[1], newArg, node, 'arguments');
	        return null;
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// const foo = eval; foo = eval; { _eval: eval }; return eval;
	// -->
	// const foo = _get$Eval(eval); foo = _get$Eval(eval); { _eval: _get$Eval(eval) }; return _get$Eval(eval);
	var transformer$g = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.Identifier,
	    condition: function (node, parent) {
	        if (node.name === 'eval' && parent) {
	            // Skip: eval()
	            if (parent.type === Syntax_1.CallExpression && parent.callee === node)
	                return false;
	            // Skip: class X { eval () {} }
	            if (parent.type === Syntax_1.MethodDefinition)
	                return false;
	            // Skip: class eval { x () {} }
	            if (parent.type === Syntax_1.ClassDeclaration)
	                return false;
	            // Skip: window.eval, eval.call
	            if (parent.type === Syntax_1.MemberExpression)
	                return false;
	            // Skip: function eval () { ... }
	            if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration) &&
	                parent.id === node)
	                return false;
	            // Skip: function (eval) { ... } || function func(eval) { ... } || eval => { ... }
	            if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration ||
	                parent.type === Syntax_1.ArrowFunctionExpression) && parent.params.indexOf(node) !== -1)
	                return false;
	            // Skip: { eval: value }
	            if (parent.type === Syntax_1.Property && parent.key === node)
	                return false;
	            // Skip: { eval }
	            if (parent.type === Syntax_1.Property && parent.value === node && parent.shorthand)
	                return false;
	            // Skip: eval = value || function x (eval = value) { ... }
	            if ((parent.type === Syntax_1.AssignmentExpression || parent.type === Syntax_1.AssignmentPattern) &&
	                parent.left === node)
	                return false;
	            // Skip: const eval = value;
	            if (parent.type === Syntax_1.VariableDeclarator && parent.id === node)
	                return false;
	            // Skip: eval++ || eval-- || ++eval || --eval
	            if (parent.type === Syntax_1.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
	                return false;
	            // Skip already transformed: __get$Eval(eval)
	            if (parent.type === Syntax_1.CallExpression && parent.callee.type === Syntax_1.Identifier &&
	                parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.getEval)
	                return false;
	            // Skip: function x (...eval) {}
	            if (parent.type === Syntax_1.RestElement)
	                return false;
	            // Skip: export { eval } from "module";
	            if (parent.type === Syntax_1.ExportSpecifier)
	                return false;
	            // Skip: import { eval } from "module";
	            if (parent.type === Syntax_1.ImportSpecifier)
	                return false;
	            return true;
	        }
	        return false;
	    },
	    run: createGetEvalMethodCall
	};

	// -------------------------------------------------------------
	// Transform:
	// const foo = window.eval; foo = window.eval; { _eval: window.eval }; return window.eval;
	// -->
	// const foo = _get$Eval(window.eval); foo = _get$Eval(window.eval); { _eval: _get$Eval(window.eval) }; return _get$Eval(window.eval);
	var transformer$f = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.MemberExpression,
	    condition: function (node, parent) {
	        if (!parent)
	            return false;
	        // Skip: window.eval.field
	        if (parent.type === Syntax_1.MemberExpression && (parent.property === node || parent.object === node))
	            return false;
	        // Skip: window.eval()
	        if (parent.type === Syntax_1.CallExpression && parent.callee === node)
	            return false;
	        // Skip: window.eval = 1, window["eval"] = 1
	        if (parent.type === Syntax_1.AssignmentExpression && parent.left === node)
	            return false;
	        // Skip already transformed: __get$Eval(window.eval), __get$Eval(window["eval"])
	        if (parent.type === Syntax_1.CallExpression && parent.callee.type === Syntax_1.Identifier &&
	            parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.getEval)
	            return false;
	        // window.eval
	        if (node.property.type === Syntax_1.Identifier && node.property.name === 'eval')
	            return true;
	        // window['eval']
	        if (node.property.type === Syntax_1.Literal && node.property.value === 'eval')
	            return true;
	        return false;
	    },
	    run: createGetEvalMethodCall
	};

	// -------------------------------------------------------------
	// Transform:
	// const foo = postMessage; foo = postMessage; { _postMessage: postMessage }; return postMessage;
	// -->
	// const foo = _get$PostMessage(postMessage); foo = _get$PostMessage(postMessage); { _postMessage: _get$PostMessage(postMessage) }; return _get$PostMessage(postMessage);
	var transformer$e = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.Identifier,
	    condition: function (node, parent) {
	        if (node.name !== 'postMessage' || !parent)
	            return false;
	        // Skip: window.postMessage, postMessage.call
	        if (parent.type === Syntax_1.MemberExpression)
	            return false;
	        // Skip: class X { postMessage () {} }
	        if (parent.type === Syntax_1.MethodDefinition)
	            return false;
	        // Skip: class postMessage { x () {} }
	        if (parent.type === Syntax_1.ClassDeclaration)
	            return false;
	        // Skip: function postMessage () { ... }
	        if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration) &&
	            parent.id === node)
	            return false;
	        // Skip: function (postMessage) { ... } || function func(postMessage) { ... } || postMessage => { ... }
	        if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration ||
	            parent.type === Syntax_1.ArrowFunctionExpression) && parent.params.indexOf(node) !== -1)
	            return false;
	        // Skip: { postMessage: value }
	        if (parent.type === Syntax_1.Property && parent.key === node)
	            return false;
	        // Skip: { postMessage }
	        if (parent.type === Syntax_1.Property && parent.value === node && parent.shorthand)
	            return false;
	        // Skip: postMessage = value || function x (postMessage = value) { ... }
	        if ((parent.type === Syntax_1.AssignmentExpression || parent.type === Syntax_1.AssignmentPattern) &&
	            parent.left === node)
	            return false;
	        // Skip: const postMessage = value;
	        if (parent.type === Syntax_1.VariableDeclarator && parent.id === node)
	            return false;
	        // Skip: postMessage++ || postMessage-- || ++postMessage || --postMessage
	        if (parent.type === Syntax_1.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
	            return false;
	        // Skip already transformed: __get$PostMessage(postMessage) || __call$(obj, postMessage, args...);
	        if (parent.type === Syntax_1.CallExpression && parent.callee.type === Syntax_1.Identifier &&
	            (parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage ||
	                parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.callMethod && parent.arguments[1] === node))
	            return false;
	        // Skip: function x (...postMessage) {}
	        if (parent.type === Syntax_1.RestElement)
	            return false;
	        // Skip: export { postMessage } from "module";
	        if (parent.type === Syntax_1.ExportSpecifier)
	            return false;
	        // Skip: import { postMessage } from "module";
	        if (parent.type === Syntax_1.ImportSpecifier)
	            return false;
	        return true;
	    },
	    run: createGetPostMessageMethodCall
	};

	// -------------------------------------------------------------
	// Transform:
	// const foo = window.postMessage; foo = window.postMessage; { _postMessage: window.postMessage }; return window.postMessage;
	// -->
	// const foo = _get$PostMessage(window.postMessage); foo = _get$PostMessage(window.postMessage); { _postMessage: _get$PostMessage(window.postMessage) }; return _get$PostMessage(window.postMessage);
	var transformer$d = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.MemberExpression,
	    condition: function (node, parent) {
	        if (!parent)
	            return false;
	        // Skip: window.postMessage.field
	        if (parent.type === Syntax_1.MemberExpression && (parent.property === node || parent.object === node))
	            return false;
	        // Skip: window.postMessage()
	        if (parent.type === Syntax_1.CallExpression && parent.callee === node)
	            return false;
	        // Skip: window.postMessage = 1, window["postMessage"] = 1
	        if (parent.type === Syntax_1.AssignmentExpression && parent.left === node)
	            return false;
	        // Skip already transformed: __get$PostMessage(window.postMessage), __get$PostMessage(window["postMessage"])
	        if (parent.type === Syntax_1.CallExpression && parent.callee.type === Syntax_1.Identifier &&
	            parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage)
	            return false;
	        // window.postMessage
	        if (node.property.type === Syntax_1.Identifier && node.property.name === 'postMessage')
	            return true;
	        // window['postMessage']
	        if (node.property.type === Syntax_1.Literal && node.property.value === 'postMessage')
	            return true;
	        return false;
	    },
	    run: createGetPostMessageMethodCall
	};

	// -------------------------------------------------------------
	var INVOCATION_FUNC_NAME_RE = /^(call|apply|bind)$/;
	// Transform:
	// postMessage.call(ctx, script);
	// postMessage.apply(ctx, script);
	// postMessage.bind(...); -->
	// __get$PostMessage(postMessage).call(ctx, script);
	// __get$PostMessage(postMessage).apply(ctx, script);
	// __get$PostMessage(postMessage).bind(...);
	var transformer$c = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.CallExpression,
	    condition: function (node) {
	        if (node.callee.type === Syntax_1.MemberExpression && node.callee.property.type === Syntax_1.Identifier &&
	            INVOCATION_FUNC_NAME_RE.test(node.callee.property.name)) {
	            // postMessage.<call|apply>(ctx, script, ...)
	            if (node.arguments.length < 2 && node.callee.property.name !== 'bind')
	                return false;
	            var obj = node.callee.object;
	            // obj.postMessage.<meth>(), obj[postMessage].<meth>(),
	            if (obj.type === Syntax_1.MemberExpression &&
	                (obj.property.type === Syntax_1.Identifier && obj.property.name ||
	                    obj.property.type === Syntax_1.Literal && obj.property.value) === 'postMessage')
	                return true;
	            // postMessage.<meth>()
	            if (obj.type === Syntax_1.Identifier && obj.name === 'postMessage')
	                return true;
	        }
	        return false;
	    },
	    run: function (node) {
	        var callee = node.callee;
	        var getPostMessageNode = createGetPostMessageMethodCall(callee.object);
	        replaceNode(callee.object, getPostMessageNode, callee, 'object');
	        return null;
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// for(obj[prop] in src), for(obj.prop in src) -->
	// for(const _hh$temp0 in src) { obj[prop] = _hh$temp0; }
	var transformer$b = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.ForInStatement,
	    condition: function (node) { return node.left.type === Syntax_1.MemberExpression; },
	    run: function (node) {
	        var tempVarAst = createIdentifier(TempVariables.generateName());
	        var varDeclaration = createVariableDeclaration('var', [createVariableDeclarator(tempVarAst)]);
	        var assignmentExprStmt = createAssignmentExprStmt(node.left, tempVarAst);
	        if (node.body.type !== Syntax_1.BlockStatement)
	            replaceNode(node.body, createBlockStatement([assignmentExprStmt, node.body]), node, 'body');
	        else
	            replaceNode(null, assignmentExprStmt, node.body, 'body');
	        replaceNode(node.left, varDeclaration, node, 'left');
	        return null;
	    }
	};

	function walkDeclarators(node, action) {
	    var declarators = [];
	    var identifiers = [];
	    for (var _i = 0, _a = node.body; _i < _a.length; _i++) {
	        var statement = _a[_i];
	        if (statement.type === Syntax_1.VariableDeclaration)
	            declarators.push.apply(declarators, statement.declarations);
	    }
	    for (var _b = 0, declarators_1 = declarators; _b < declarators_1.length; _b++) {
	        var declarator = declarators_1[_b];
	        if (declarator.type === Syntax_1.VariableDeclarator) {
	            if (declarator.id.type === Syntax_1.Identifier)
	                identifiers.push(declarator.id);
	            if (declarator.id.type === Syntax_1.ArrayPattern)
	                identifiers.push.apply(identifiers, declarator.id.elements);
	            if (declarator.id.type === Syntax_1.ObjectPattern)
	                for (var _c = 0, _d = declarator.id.properties; _c < _d.length; _c++) {
	                    var prop = _d[_c];
	                    identifiers.push(prop.value);
	                }
	        }
	    }
	    for (var _e = 0, identifiers_1 = identifiers; _e < identifiers_1.length; _e++) {
	        var identifier = identifiers_1[_e];
	        if (identifier.type === Syntax_1.Identifier)
	            action(identifier);
	    }
	}
	function replaceDuplicateDeclarators(forOfNode) {
	    var _a;
	    var forOfLeft = forOfNode.left;
	    var nodesToReplace = [];
	    var isArrayPatternDeclaration = ((_a = forOfLeft.declarations[0]) === null || _a === void 0 ? void 0 : _a.id.type) === Syntax_1.ArrayPattern;
	    var isBlockStatement = forOfNode.body.type === Syntax_1.BlockStatement;
	    if (!isArrayPatternDeclaration || !isBlockStatement)
	        return;
	    var leftDeclaration = forOfLeft.declarations[0].id;
	    var leftIdentifiers = leftDeclaration.elements;
	    walkDeclarators(forOfNode.body, function (node) {
	        for (var _i = 0, leftIdentifiers_1 = leftIdentifiers; _i < leftIdentifiers_1.length; _i++) {
	            var identifier = leftIdentifiers_1[_i];
	            if (identifier.name === node.name)
	                nodesToReplace.push(identifier);
	        }
	    });
	    for (var _i = 0, nodesToReplace_1 = nodesToReplace; _i < nodesToReplace_1.length; _i++) {
	        var nodeToReplace = nodesToReplace_1[_i];
	        var destIdentifier = createIdentifier(TempVariables.generateName());
	        replaceNode(nodeToReplace, destIdentifier, leftDeclaration, 'elements');
	    }
	}
	// Transform:
	// for (let {href, postMessage} of wins) {} -->
	// for (let _hh$temp0 of wins) { let {href, postMessage} = _hh$temp0; }
	var transformer$a = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.ForOfStatement,
	    condition: function (node) {
	        var left = node.left;
	        if (left.type === Syntax_1.VariableDeclaration)
	            left = left.declarations[0].id;
	        return left.type === Syntax_1.ObjectPattern || left.type === Syntax_1.ArrayPattern;
	    },
	    run: function (node) {
	        var tempIdentifier = createIdentifier(TempVariables.generateName());
	        var forOfLeft = node.left;
	        var statementWithTempAssignment;
	        if (forOfLeft.type === Syntax_1.VariableDeclaration) {
	            replaceDuplicateDeclarators(node);
	            statementWithTempAssignment = createVariableDeclaration(forOfLeft.kind, [
	                createVariableDeclarator(forOfLeft.declarations[0].id, tempIdentifier)
	            ]);
	            statementWithTempAssignment.reTransform = true;
	            replaceNode(forOfLeft.declarations[0].id, tempIdentifier, forOfLeft.declarations[0], 'id');
	        }
	        else {
	            var varDeclaration = createVariableDeclaration('var', [createVariableDeclarator(tempIdentifier)]);
	            statementWithTempAssignment = createAssignmentExprStmt(forOfLeft, tempIdentifier);
	            replaceNode(forOfLeft, varDeclaration, node, 'left');
	        }
	        if (node.body.type === Syntax_1.BlockStatement)
	            replaceNode(null, statementWithTempAssignment, node.body, 'body');
	        else
	            replaceNode(node.body, createBlockStatement([statementWithTempAssignment, node.body]), node, 'body');
	        return null;
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// location -->
	// __get$Loc(location)
	var transformer$9 = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.Identifier,
	    condition: function (node, parent) {
	        if (node.name !== 'location' || !parent)
	            return false;
	        // Skip: const location = value;
	        if (parent.type === Syntax_1.VariableDeclarator && parent.id === node)
	            return false;
	        // Skip: location = value || function x (location = value) { ... }
	        if ((parent.type === Syntax_1.AssignmentExpression || parent.type === Syntax_1.AssignmentPattern) &&
	            parent.left === node)
	            return false;
	        // Skip: function location() {}
	        if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration) &&
	            parent.id === node)
	            return false;
	        // Skip: object.location || location.field
	        if (parent.type === Syntax_1.MemberExpression && parent.property === node)
	            return false;
	        // Skip: { location: value }
	        if (parent.type === Syntax_1.Property && parent.key === node)
	            return false;
	        // Skip: { location }
	        if (parent.type === Syntax_1.Property && parent.value === node && parent.shorthand)
	            return false;
	        // Skip: location++ || location-- || ++location || --location
	        if (parent.type === Syntax_1.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
	            return false;
	        // Skip: function (location) { ... } || function func(location) { ... } || location => { ... }
	        if ((parent.type === Syntax_1.FunctionExpression || parent.type === Syntax_1.FunctionDeclaration ||
	            parent.type === Syntax_1.ArrowFunctionExpression) && parent.params.indexOf(node) !== -1)
	            return false;
	        // Skip already transformed: __get$Loc(location)
	        if (parent.type === Syntax_1.CallExpression && parent.callee.type === Syntax_1.Identifier &&
	            parent.callee.name === SCRIPT_PROCESSING_INSTRUCTIONS.getLocation)
	            return false;
	        // Skip: class X { location () {} }
	        if (parent.type === Syntax_1.MethodDefinition)
	            return false;
	        // Skip: class location { x () {} }
	        if (parent.type === Syntax_1.ClassDeclaration)
	            return false;
	        // Skip: function x (...location) {}
	        if (parent.type === Syntax_1.RestElement)
	            return false;
	        // Skip: export { location } from "module";
	        if (parent.type === Syntax_1.ExportSpecifier)
	            return false;
	        // Skip: import { location } from "module";
	        if (parent.type === Syntax_1.ImportSpecifier)
	            return false;
	        return true;
	    },
	    run: createLocationGetWrapper
	};

	// -------------------------------------------------------------
	// Transform:
	// location = value -->
	// (function(){ return __set$Loc(location, value) || location = value;}.apply(this))
	var transformer$8 = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.AssignmentExpression,
	    condition: function (node) { return node.operator === '=' && node.left.type === Syntax_1.Identifier && node.left.name === 'location'; },
	    run: function (node, parent, key) {
	        if (!parent)
	            return null;
	        var wrapWithSequence = key !== 'arguments' && key !== 'consequent' && key !== 'alternate' &&
	            // @ts-ignore
	            (parent.type !== Syntax_1.SequenceExpression || parent.expressions[0] === node);
	        return createLocationSetWrapper(node.left, node.right, wrapWithSequence);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// obj.<wrappable-property> -->
	// __get$(obj, '<wrappable-property>')
	var transformer$7 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.MemberExpression,
	    condition: function (node, parent) {
	        if (node.computed || !parent)
	            return false;
	        if (node.property.type === Syntax_1.Identifier && !shouldInstrumentProperty(node.property.name))
	            return false;
	        // Skip: super.prop
	        if (node.object.type === Syntax_1.Super)
	            return false;
	        // Skip: object.prop = value
	        if (parent.type === Syntax_1.AssignmentExpression && parent.left === node)
	            return false;
	        // Skip: delete object.prop
	        if (parent.type === Syntax_1.UnaryExpression && parent.operator === 'delete')
	            return false;
	        // Skip: object.prop()
	        if (parent.type === Syntax_1.CallExpression && parent.callee === node)
	            return false;
	        // Skip: object.prop++ || object.prop-- || ++object.prop || --object.prop
	        if (parent.type === Syntax_1.UpdateExpression && (parent.operator === '++' || parent.operator === '--'))
	            return false;
	        // Skip: new (object.prop)() || new (object.prop)
	        if (parent.type === Syntax_1.NewExpression && parent.callee === node)
	            return false;
	        // Skip: for(object.prop in source)
	        if (parent.type === Syntax_1.ForInStatement && parent.left === node)
	            return false;
	        return true;
	    },
	    // eslint-disable-next-line
	    run: function (node) { return createPropertyGetWrapper(node.property.name, node.object); }
	};

	// -------------------------------------------------------------
	// Transform:
	// obj.<wrappable-property> = value -->
	// __set$(obj, '<wrappable-property>', value)
	var transformer$6 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.AssignmentExpression,
	    condition: function (node) {
	        // super.prop = value
	        if (node.left.type === Syntax_1.MemberExpression && node.left.object.type === Syntax_1.Super)
	            return false;
	        return node.operator === '=' &&
	            node.left.type === Syntax_1.MemberExpression && !node.left.computed &&
	            node.left.property.type === Syntax_1.Identifier &&
	            shouldInstrumentProperty(node.left.property.name);
	    },
	    run: function (node) {
	        var memberExpression = node.left;
	        var identifier = memberExpression.property;
	        return createPropertySetWrapper(identifier.name, memberExpression.object, node.right);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// obj.method(args...); obj[method](args...); -->
	// _call$(obj, 'method', args...); _call$(obj, method, args...);
	var transformer$5 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.CallExpression,
	    condition: function (node) {
	        var callee = node.callee;
	        if (callee.type === Syntax_1.MemberExpression) {
	            // Skip: super.meth()
	            if (callee.object.type === Syntax_1.Super)
	                return false;
	            if (callee.computed)
	                return callee.property.type === Syntax_1.Literal ? shouldInstrumentMethod(callee.property.value) : true;
	            return callee.property.type === Syntax_1.Identifier && shouldInstrumentMethod(callee.property.name);
	        }
	        return false;
	    },
	    run: function (node) {
	        var callee = node.callee;
	        var method = callee.computed
	            ? callee.property
	            : createSimpleLiteral(callee.property.name); // eslint-disable-line no-extra-parens
	        return createMethodCallWrapper(callee.object, method, node.arguments);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// x = 5; "hello" --> x = 5; parent.__proc$Html(window, "hello")
	// someAction(); generateHtmlPage() --> someAction(); parent.__proc$Html(window, generateHtmlPage())
	var transformer$4 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.ExpressionStatement,
	    condition: function (node, parent) { return !!transformer$4.wrapLastExpr && !!parent && parent.type === Syntax_1.Program &&
	        parent.body[parent.body.length - 1] === node; },
	    run: function (node) {
	        transformer$4.wrapLastExpr = false;
	        return createHtmlProcessorWrapper(node);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// import something from 'url';   -->   import something from 'processed-url';
	// export * from 'url';   -->   export * from 'processed-url';
	// export { x as y } from 'url';   -->   export { x as y } from 'processed-url';
	var transformer$3 = {
	    nodeReplacementRequireTransform: false,
	    nodeTypes: Syntax_1.Literal,
	    condition: function (node, parent) { return !!parent && (parent.type === Syntax_1.ImportDeclaration ||
	        parent.type === Syntax_1.ExportAllDeclaration ||
	        parent.type === Syntax_1.ExportNamedDeclaration) && parent.source === node; },
	    run: function (node) { return transformer$3.resolver ? getProxyUrlLiteral(node, transformer$3.resolver) : null; }
	};

	// -------------------------------------------------------------
	// Transform:
	// import(something).then()
	// -->
	// import(__get$ProxyUrl(something)).then()
	var transformer$2 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.ImportExpression,
	    condition: function () { return true; },
	    run: function (node) {
	        var newSource = createGetProxyUrlMethodCall(node.source, transformer$2.baseUrl);
	        replaceNode(node.source, newSource, node, 'source');
	        return null;
	    }
	};

	function processObjectProperty(prop, temp, build, baseTempName) {
	    var pattern = prop.value;
	    var computed = prop.computed || prop.key.type === Syntax_1.Literal;
	    var value = createMemberExpression(temp, prop.key, computed);
	    process(pattern, value, build, baseTempName);
	}
	function createObjectRest(tempIdentifier, keys) {
	    var restObjectIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.restObject);
	    return createSimpleCallExpression(restObjectIdentifier, [tempIdentifier, createArrayExpression(keys)]);
	}
	function createRestArray(array, startIndex) {
	    var restArrayIdentifier = createIdentifier(SCRIPT_PROCESSING_INSTRUCTIONS.restArray);
	    return createSimpleCallExpression(restArrayIdentifier, [array, createSimpleLiteral(startIndex)]);
	}
	function createTempIdentifierOrUseExisting(value, build, baseTempName) {
	    if (value.type === Syntax_1.Identifier && TempVariables.isHHTempVariable(value.name))
	        return value;
	    var tempIdentifier = createIdentifier(baseTempName || TempVariables.generateName(baseTempName));
	    build(tempIdentifier, value, true);
	    return tempIdentifier;
	}
	function processObjectPattern(pattern, value, build, baseTempName) {
	    var properties = pattern.properties;
	    var hasRest = properties.length && properties[properties.length - 1].type === Syntax_1.RestElement;
	    var tempIdentifier = createTempIdentifierOrUseExisting(value, build, baseTempName);
	    var propNames = [];
	    var baseTempNames = [];
	    if (!baseTempName)
	        baseTempName = tempIdentifier.name;
	    if (hasRest) {
	        for (var i = 0; i < properties.length - 1; i++) {
	            var prop = properties[i];
	            var key = prop.key;
	            if (key.type === Syntax_1.Identifier)
	                propNames.push(prop.computed ? key : createSimpleLiteral(key.name));
	            else if (key.type === Syntax_1.Literal)
	                propNames.push(key);
	            else {
	                var tempPropKey = createIdentifier(TempVariables.generateName());
	                build(tempPropKey, key, true);
	                propNames.push(tempPropKey);
	                prop.key = tempPropKey;
	            }
	        }
	    }
	    for (var i = 0; i < properties.length; i++) {
	        var prop = properties[i];
	        if (prop.type === Syntax_1.RestElement)
	            build(prop.argument, createObjectRest(tempIdentifier, propNames));
	        else {
	            var assignmentProp = prop;
	            var newBaseTempName = TempVariables.generateName(baseTempName, assignmentProp.key, i);
	            if (baseTempNames.indexOf(newBaseTempName) > -1)
	                newBaseTempName = TempVariables.generateName(newBaseTempName, void 0, i);
	            baseTempNames.push(newBaseTempName);
	            processObjectProperty(assignmentProp, tempIdentifier, build, newBaseTempName);
	        }
	    }
	}
	function processArrayPattern(pattern, value, build, baseTempName) {
	    var tempIdentifier = createTempIdentifierOrUseExisting(value, build, baseTempName);
	    if (!baseTempName)
	        baseTempName = tempIdentifier.name;
	    for (var i = 0; i < pattern.elements.length; i++) {
	        var elem = pattern.elements[i];
	        if (!elem)
	            continue;
	        if (elem.type === Syntax_1.RestElement) {
	            value = createRestArray(tempIdentifier, i);
	            elem = elem.argument;
	        }
	        else
	            value = createMemberExpression(tempIdentifier, createSimpleLiteral(i), true);
	        process(elem, value, build, TempVariables.generateName(baseTempName, null, i));
	    }
	}
	function processAssignmentPattern(pattern, value, build, baseTempName) {
	    var left = pattern.left, right = pattern.right;
	    var tempIdentifier = createTempIdentifierOrUseExisting(value, build, baseTempName);
	    var tempCondition = createBinaryExpression(tempIdentifier, '===', createUndefined());
	    var tempConditional = createConditionalExpression(tempCondition, right, tempIdentifier);
	    if (!baseTempName)
	        baseTempName = tempIdentifier.name;
	    baseTempName += '$assign';
	    process(left, tempConditional, build, baseTempName);
	}
	function process(pattern, value, build, baseTempName) {
	    if (pattern.type === Syntax_1.ObjectPattern)
	        processObjectPattern(pattern, value, build, baseTempName);
	    else if (pattern.type === Syntax_1.ArrayPattern)
	        processArrayPattern(pattern, value, build, baseTempName);
	    else if (pattern.type === Syntax_1.AssignmentPattern)
	        processAssignmentPattern(pattern, value, build, baseTempName);
	    else
	        build(pattern, value);
	}

	// -------------------------------------------------------------
	// Transform:
	// var { location: loc } = window,
	//     [{ location }, item] = [window, 6]
	// -->
	// var _hh$temp0 = window,
	//     loc = _hh$temp0.location,
	//     _hh$temp1 = [window, 6],
	//     _hh$temp1$0 = _hh$temp1[0],
	//     location = _hh$temp1$0.location,
	//     item = _hh$temp1[1];
	var transformer$1 = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.VariableDeclaration,
	    // @ts-ignore
	    condition: function (node, parent) {
	        // Skip: for (let { x } in some);
	        if (parent.type === Syntax_1.ForInStatement)
	            return false;
	        for (var _i = 0, _a = node.declarations; _i < _a.length; _i++) {
	            var declarator = _a[_i];
	            if (declarator.id.type === Syntax_1.ObjectPattern || declarator.id.type === Syntax_1.ArrayPattern)
	                return true;
	        }
	        return false;
	    },
	    run: function (node) {
	        var declarations = [];
	        for (var _i = 0, _a = node.declarations; _i < _a.length; _i++) {
	            var declarator = _a[_i];
	            process(declarator.id, declarator.init, function (pattern, value) {
	                return declarations.push(createVariableDeclarator(pattern, value));
	            });
	        }
	        return createVariableDeclaration(node.kind, declarations);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// ({ location: loc } = window);
	// [{ location }, item] = [window, 6]
	// -->
	// var _hh$temp0, _hh$temp1, _hh$temp1$0;
	//
	// (_hh$temp0 = window, loc = _hh$temp0.location, _hh$temp0);
	// (_hh$temp1 = [window, 6], _hh$temp1$0 = _hh$temp1[0], location = _hh$temp1$0.location, item = _hh$temp1[1], _hh$temp1);
	var transformer = {
	    nodeReplacementRequireTransform: true,
	    nodeTypes: Syntax_1.AssignmentExpression,
	    condition: function (node) { return node.operator === '=' && (node.left.type === Syntax_1.ObjectPattern || node.left.type === Syntax_1.ArrayPattern); },
	    run: function (node, _parent, _key, tempVars) {
	        var assignments = [];
	        var firstTemp = void 0;
	        process(node.left, node.right, function (pattern, value, isTemp) {
	            if (firstTemp === void 0) {
	                if (isTemp)
	                    firstTemp = pattern;
	                else
	                    firstTemp = null;
	            }
	            assignments.push(createAssignmentExpression(pattern, '=', value));
	            if (isTemp)
	                tempVars.append(pattern.name);
	        });
	        if (firstTemp)
	            assignments.push(firstTemp);
	        return createSequenceExpression(assignments);
	    }
	};

	// -------------------------------------------------------------
	// Transform:
	// function x ({a, b}, [c, d]) {}
	// -->
	// function x (_hh$temp0, _hh$temp1) {
	//     var {a, b} = _hh$temp0,
	//         [c, d] = _hh$temp1;
	// }
	function create$2(type) {
	    return {
	        nodeReplacementRequireTransform: false,
	        nodeTypes: type,
	        condition: function (node) {
	            for (var _i = 0, _a = node.params; _i < _a.length; _i++) {
	                var param = _a[_i];
	                if (param.type === Syntax_1.AssignmentPattern)
	                    param = param.left;
	                if (param.type === Syntax_1.ObjectPattern || param.type === Syntax_1.ArrayPattern)
	                    return true;
	            }
	            return false;
	        },
	        run: function (node) {
	            var declarations = [];
	            for (var _i = 0, _a = node.params; _i < _a.length; _i++) {
	                var param = _a[_i];
	                var tempVarParent = node;
	                var tempVarKey = 'params';
	                if (param.type === Syntax_1.AssignmentPattern) {
	                    // @ts-ignore
	                    tempVarParent = param;
	                    param = param.left;
	                    tempVarKey = 'left';
	                }
	                if (param.type === Syntax_1.ObjectPattern && param.properties.length ||
	                    param.type === Syntax_1.ArrayPattern && param.elements.length) {
	                    var tempVar = createIdentifier(TempVariables.generateName());
	                    // @ts-ignore
	                    replaceNode(param, tempVar, tempVarParent, tempVarKey);
	                    declarations.push(createVariableDeclarator(param, tempVar));
	                }
	            }
	            if (!declarations.length)
	                return null;
	            var declaration = createVariableDeclaration('var', declarations);
	            if (node.body.type !== Syntax_1.BlockStatement) {
	                // @ts-ignore
	                var returnStmt = createReturnStatement(node.body);
	                replaceNode(node.body, createBlockStatement([declaration, returnStmt]), node, 'body');
	                // @ts-ignore
	                node.expression = false;
	                return node;
	            }
	            else
	                replaceNode(null, declaration, node.body, 'body');
	            declaration.reTransform = true;
	            return null;
	        }
	    };
	}

	// -------------------------------------------------------------
	var TRANSFORMERS = [
	    create$2(Syntax_1.FunctionDeclaration),
	    create$2(Syntax_1.FunctionExpression),
	    create$2(Syntax_1.ArrowFunctionExpression),
	    transformer,
	    transformer$m,
	    transformer$l,
	    transformer$k,
	    transformer$j,
	    transformer$i,
	    transformer$h,
	    transformer$g,
	    transformer$f,
	    transformer$e,
	    transformer$d,
	    transformer$c,
	    transformer$b,
	    transformer$a,
	    transformer$9,
	    transformer$8,
	    transformer$7,
	    transformer$6,
	    transformer$5,
	    transformer$4,
	    transformer$3,
	    transformer$2,
	    transformer$1
	];
	function createTransformerMap() {
	    var transformerMap = new Map();
	    for (var _i = 0, TRANSFORMERS_1 = TRANSFORMERS; _i < TRANSFORMERS_1.length; _i++) {
	        var transformer = TRANSFORMERS_1[_i];
	        var nodeType = transformer.nodeTypes;
	        var transformers = transformerMap.get(nodeType);
	        if (!transformers) {
	            transformers = [];
	            transformerMap.set(nodeType, transformers);
	        }
	        transformers.push(transformer);
	    }
	    return transformerMap;
	}
	var transformers = createTransformerMap();

	var STACK_FRAME_REG_EXPS = [
	    /^\s*at .*\((\S+)\)/,
	    /^\s*at (\S+)/,
	    /^.*@(\S+)/,
	    /(.+)/ // Any string
	];
	var ROW_COLUMN_NUMBER_REG_EX = /:\d+:\d+$/;
	function getDestSource(source) {
	    var parsedProxiedUrl = parseProxyUrl$1(source);
	    return parsedProxiedUrl && parsedProxiedUrl.destUrl;
	}
	function replaceUrlWithProxied(str, source) {
	    source = source.replace(ROW_COLUMN_NUMBER_REG_EX, '');
	    var destUrl = getDestSource(source);
	    return destUrl ? str.replace(source, destUrl) : str;
	}
	function replaceProxiedUrlsInStack(stack) {
	    if (!stack)
	        return stack;
	    var stackFrames = stack.split('\n');
	    for (var i = 0; i < stackFrames.length; i++) {
	        var stackFrame = stackFrames[i];
	        for (var _i = 0, STACK_FRAME_REG_EXPS_1 = STACK_FRAME_REG_EXPS; _i < STACK_FRAME_REG_EXPS_1.length; _i++) {
	            var stackFrameRegExp = STACK_FRAME_REG_EXPS_1[_i];
	            if (stackFrameRegExp.test(stackFrame)) {
	                stackFrames[i] = stackFrame.replace(stackFrameRegExp, replaceUrlWithProxied);
	                break;
	            }
	        }
	    }
	    return stackFrames.join('\n');
	}
	function getFirstDestUrl(stack) {
	    if (!stack)
	        return null;
	    var stackFrames = stack.split('\n');
	    for (var _i = 0, stackFrames_1 = stackFrames; _i < stackFrames_1.length; _i++) {
	        var stackFrame = stackFrames_1[_i];
	        var _loop_1 = function (stackFrameRegExp) {
	            if (!stackFrameRegExp.test(stackFrame))
	                return "continue";
	            var destSource = null;
	            stackFrame.replace(stackFrameRegExp, function (str, source) {
	                source = source.replace(ROW_COLUMN_NUMBER_REG_EX, '');
	                destSource = getDestSource(source);
	                return str;
	            });
	            if (destSource)
	                return { value: destSource };
	        };
	        for (var _a = 0, STACK_FRAME_REG_EXPS_2 = STACK_FRAME_REG_EXPS; _a < STACK_FRAME_REG_EXPS_2.length; _a++) {
	            var stackFrameRegExp = STACK_FRAME_REG_EXPS_2[_a];
	            var state_1 = _loop_1(stackFrameRegExp);
	            if (typeof state_1 === "object")
	                return state_1.value;
	        }
	    }
	    return null;
	}

	var sharedStackProcessingUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		replaceProxiedUrlsInStack: replaceProxiedUrlsInStack,
		getFirstDestUrl: getFirstDestUrl
	});

	var State = /** @class */ (function () {
	    function State() {
	        this.hasTransformedAncestor = false;
	    }
	    // NOTE: There is an issue with processing `new` expressions. `new a.src.b()` will be transformed
	    // to `new __get$(a, 'src').b()`, which is wrong. The correct result is `new (__get$(a, 'src')).b()`.
	    // To solve this problem, we add a 'state' entity. This entity stores the "new" expression, so that
	    // we can add it to the changes when the transformation is found.
	    State.create = function (currState, node, parent, key, hasTransformedAncestor) {
	        if (hasTransformedAncestor === void 0) { hasTransformedAncestor = false; }
	        var isNewExpression = node.type === Syntax_1.NewExpression;
	        var isNewExpressionAncestor = isNewExpression && !currState.newExpressionAncestor;
	        var newState = new State();
	        newState.hasTransformedAncestor = currState.hasTransformedAncestor || hasTransformedAncestor;
	        newState.newExpressionAncestor = isNewExpressionAncestor ? node : currState.newExpressionAncestor;
	        newState.newExpressionAncestorParent = isNewExpressionAncestor ? parent : currState.newExpressionAncestorParent;
	        // @ts-ignore
	        newState.newExpressionAncestorKey = isNewExpressionAncestor ? key : currState.newExpressionAncestorKey;
	        return newState;
	    };
	    return State;
	}());
	// NOTE: We should avoid using native object prototype methods,
	// since they can be overriden by the client code. (GH-245)
	var objectToString = Object.prototype.toString;
	var objectKeys = Object.keys;
	function getChange(node, parentType) {
	    /*eslint-disable @typescript-eslint/no-non-null-assertion*/
	    var start = node.originStart;
	    var end = node.originEnd;
	    /*eslint-disable @typescript-eslint/no-non-null-assertion*/
	    return { start: start, end: end, node: node, parentType: parentType };
	}
	function transformChildNodes(node, changes, state, tempVars) {
	    // @ts-ignore
	    var nodeKeys = objectKeys(node);
	    for (var _i = 0, nodeKeys_1 = nodeKeys; _i < nodeKeys_1.length; _i++) {
	        var key = nodeKeys_1[_i];
	        var childNode = node[key];
	        var stringifiedNode = objectToString.call(childNode);
	        if (stringifiedNode === '[object Array]') {
	            // @ts-ignore
	            var childNodes = childNode;
	            for (var _a = 0, childNodes_1 = childNodes; _a < childNodes_1.length; _a++) {
	                var nthNode = childNodes_1[_a];
	                // NOTE: Some items of ArrayExpression can be null
	                if (nthNode)
	                    transform(nthNode, changes, state, node, key, tempVars);
	            }
	        }
	        else if (stringifiedNode === '[object Object]') {
	            // @ts-ignore
	            transform(childNode, changes, state, node, key, tempVars);
	        }
	    }
	}
	function isNodeTransformed(node) {
	    return node.originStart !== void 0 && node.originEnd !== void 0;
	}
	function addChangeForTransformedNode(state, changes, replacement, parentType) {
	    var hasTransformedAncestor = state.hasTransformedAncestor ||
	        state.newExpressionAncestor && isNodeTransformed(state.newExpressionAncestor);
	    if (hasTransformedAncestor)
	        return;
	    if (state.newExpressionAncestor) {
	        replaceNode(state.newExpressionAncestor, state.newExpressionAncestor, state.newExpressionAncestorParent, state.newExpressionAncestorKey);
	        changes.push(getChange(state.newExpressionAncestor, state.newExpressionAncestorParent.type));
	    }
	    else
	        changes.push(getChange(replacement, parentType));
	}
	function addTempVarsDeclaration(node, changes, state, tempVars) {
	    var names = tempVars.get();
	    if (!names.length)
	        return;
	    var declaration = createTempVarsDeclaration(names);
	    replaceNode(null, declaration, node, 'body');
	    addChangeForTransformedNode(state, changes, declaration, node.type);
	}
	function beforeTransform(wrapLastExprWithProcessHtml, resolver) {
	    if (wrapLastExprWithProcessHtml === void 0) { wrapLastExprWithProcessHtml = false; }
	    transformer$4.wrapLastExpr = wrapLastExprWithProcessHtml;
	    transformer$3.resolver = resolver;
	    var isServerSide = typeof window === 'undefined';
	    if (isServerSide)
	        transformer$2.baseUrl = resolver ? parseProxyUrl$1(resolver('./')).destUrl : '';
	    else {
	        var currentStack = new Error().stack;
	        // NOTE: IE11 doesn't give the error stack without the 'throw' statement and doesn't support the 'import' statement
	        transformer$2.baseUrl = currentStack && getFirstDestUrl(currentStack) || '';
	    }
	}
	function afterTransform() {
	    transformer$4.wrapLastExpr = false;
	    transformer$3.resolver = void 0;
	    transformer$2.baseUrl = void 0;
	}
	function findTransformer(node, parent) {
	    var nodeTransformers = transformers.get(node.type);
	    if (nodeTransformers) {
	        for (var _i = 0, nodeTransformers_1 = nodeTransformers; _i < nodeTransformers_1.length; _i++) {
	            var transformer = nodeTransformers_1[_i];
	            if (transformer.condition(node, parent))
	                return transformer;
	        }
	    }
	    return null;
	}
	function transform(node, changes, state, parent, key, tempVars) {
	    var allowTempVarAdd = node.type === Syntax_1.BlockStatement;
	    var nodeTransformed = false;
	    if (allowTempVarAdd)
	        tempVars = new TempVariables();
	    if (!node.reTransform && isNodeTransformed(node)) {
	        addChangeForTransformedNode(state, changes, node, parent.type);
	        nodeTransformed = true;
	    }
	    else {
	        var storedNode = node;
	        var transformer = findTransformer(node, parent);
	        var replacement = null;
	        while (transformer) {
	            replacement = transformer.run(replacement || node, parent, key, tempVars);
	            if (!replacement)
	                break;
	            nodeTransformed = true;
	            if (!transformer.nodeReplacementRequireTransform)
	                break;
	            transformer = findTransformer(replacement, parent);
	            node = replacement;
	        }
	        if (nodeTransformed) {
	            replaceNode(storedNode, replacement, parent, key);
	            addChangeForTransformedNode(state, changes, replacement, parent.type);
	        }
	    }
	    state = State.create(state, node, parent, key, nodeTransformed);
	    transformChildNodes(node, changes, state, tempVars);
	    if (allowTempVarAdd)
	        addTempVarsDeclaration(node, changes, state, tempVars);
	}
	function transformProgram(node, wrapLastExprWithProcessHtml, resolver) {
	    if (wrapLastExprWithProcessHtml === void 0) { wrapLastExprWithProcessHtml = false; }
	    var changes = [];
	    var state = new State();
	    var tempVars = new TempVariables();
	    TempVariables.resetCounter();
	    beforeTransform(wrapLastExprWithProcessHtml, resolver);
	    transformChildNodes(node, changes, state, tempVars);
	    addTempVarsDeclaration(node, changes, state, tempVars);
	    afterTransform();
	    return changes;
	}

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	// NOTE: taken from https://github.com/benjamingr/RegExp.escape
	function reEscape (str) {
	    return str.replace(/[\\^$*+?.()|[\]{}]/g, '\\$&');
	}

	var SERVICE_ROUTES = {
	    hammerhead: '/hammerhead.js',
	    task: '/task.js',
	    iframeTask: '/iframe-task.js',
	    messaging: '/messaging',
	    transportWorker: '/transport-worker.js',
	    workerHammerhead: '/worker-hammerhead.js'
	};

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	// NOTE: We should store the methods of the `JSON` object
	// since they can be overriden by the client code.
	var parse$1 = JSON.parse;
	var stringify = JSON.stringify;
	function isDOMNode(obj) {
	    if (typeof Node === 'object')
	        return obj instanceof Node;
	    return typeof obj.nodeType === 'number' && typeof obj.nodeName === 'string';
	}
	function isJQueryObj(obj) {
	    return !!(obj && obj.jquery);
	}
	function isSerializable(value) {
	    if (value) {
	        // NOTE: jquery object, DOM nodes and functions are disallowed obj types because we can't serialize them correctly
	        if (typeof value === 'function' || isJQueryObj(value) || isDOMNode(value))
	            return false;
	        if (typeof value === 'object') {
	            for (var prop in value) {
	                if (value.hasOwnProperty(prop) && !isSerializable(value[prop]))
	                    return false;
	            }
	        }
	    }
	    return true;
	}

	var json = /*#__PURE__*/Object.freeze({
		__proto__: null,
		parse: parse$1,
		stringify: stringify,
		isSerializable: isSerializable
	});

	// -------------------------------------------------------------
	var SCRIPT_PROCESSING_START_COMMENT = '/*hammerhead|script|start*/';
	var SCRIPT_PROCESSING_END_COMMENT = '/*hammerhead|script|end*/';
	var SCRIPT_PROCESSING_END_HEADER_COMMENT = '/*hammerhead|script|processing-header-end*/';
	var STRICT_MODE_PLACEHOLDER = '{strict-placeholder}';
	var SW_SCOPE_HEADER_VALUE = '{sw-scope-header-value}';
	var HEADER = ("\n    " + SCRIPT_PROCESSING_START_COMMENT + "\n    " + STRICT_MODE_PLACEHOLDER + "\n    " + SW_SCOPE_HEADER_VALUE + "\n\n    if (typeof window !== 'undefined' && window){\n        window['" + INTERNAL_PROPS.processDomMethodName + "'] && window['" + INTERNAL_PROPS.processDomMethodName + "']();\n\n        if (window." + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + " && typeof " + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + " === 'undefined')\n            var " + SCRIPT_PROCESSING_INSTRUCTIONS.getLocation + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.getLocation + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.setLocation + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.setLocation + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.setProperty + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.setProperty + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.callMethod + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.callMethod + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getEval + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.getEval + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.processScript + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.processScript + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.processHtml + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.processHtml + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.restArray + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.restArray + ",\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.restObject + " = window." + SCRIPT_PROCESSING_INSTRUCTIONS.restObject + ";\n    } else {\n        if (typeof " + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + " === 'undefined')\n            var " + SCRIPT_PROCESSING_INSTRUCTIONS.getLocation + " = function(l){return l},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.setLocation + " = function(l,v){return l = v},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.setProperty + " = function(o,p,v){return o[p] = v},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getProperty + " = function(o,p){return o[p]},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.callMethod + " = function(o,p,a){return o[p].apply(o,a)},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getEval + " = function(e){return e},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.processScript + " = function(s){return s},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.processHtml + " = function(h){return h},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage + " = function(w,p){return arguments.length===1?w.postMessage:p},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl + " = function(u,d){return u},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.restArray + " = function(a,i){return Array.prototype.slice.call(a, i)},\n                " + SCRIPT_PROCESSING_INSTRUCTIONS.restObject + " = function(o,p){var k=Object.keys(o),n={};for(var i=0;i<k.length;++i)if(p.indexOf(k[i])<0)n[k[i]]=o[k[i]];return n};\n        \n        if (typeof importScripts !== \"undefined\" && /\\[native code]/g.test(importScripts.toString()))\n            importScripts((location.origin || (location.protocol + \"//\" + location.host)) + \"" + SERVICE_ROUTES.workerHammerhead + "\");\n    }\n    " + SCRIPT_PROCESSING_END_HEADER_COMMENT + "\n").replace(/\n(?!$)\s*/g, '');
	// NOTE: IE removes trailing newlines in script.textContent,
	// so a trailing newline in RegExp is optional
	var HEADER_RE = new RegExp(reEscape(SCRIPT_PROCESSING_START_COMMENT) + "[\\S\\s]+?" + reEscape(SCRIPT_PROCESSING_END_HEADER_COMMENT) + "\n?", 'gi');
	var PROCESSING_END_COMMENT_RE = new RegExp("\n?" + reEscape(SCRIPT_PROCESSING_END_COMMENT) + "\\s*", 'gi');
	function remove$1(code) {
	    return code
	        .replace(HEADER_RE, '')
	        .replace(PROCESSING_END_COMMENT_RE, '');
	}
	function add$1(code, isStrictMode, swScopeHeaderValue) {
	    var header = HEADER
	        .replace(STRICT_MODE_PLACEHOLDER, isStrictMode ? '"use strict";' : '')
	        .replace(SW_SCOPE_HEADER_VALUE, swScopeHeaderValue ? "var " + SCRIPT_PROCESSING_INSTRUCTIONS.swScopeHeaderValue + " = " + stringify(swScopeHeaderValue) + ";" : '');
	    return header + code + '\n' + SCRIPT_PROCESSING_END_COMMENT;
	}

	var headerProcessingUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		SCRIPT_PROCESSING_START_COMMENT: SCRIPT_PROCESSING_START_COMMENT,
		SCRIPT_PROCESSING_END_COMMENT: SCRIPT_PROCESSING_END_COMMENT,
		SCRIPT_PROCESSING_END_HEADER_COMMENT: SCRIPT_PROCESSING_END_HEADER_COMMENT,
		remove: remove$1,
		add: add$1
	});

	var identifier = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.isIdentifierStart = isIdentifierStart;
	exports.isIdentifierChar = isIdentifierChar;
	// Reserved word lists for various dialects of the language
	exports.reservedWords = {
	    3: "abstract boolean byte char class double enum export extends final float goto implements import int interface long native package private protected public short static super synchronized throws transient volatile",
	    5: "class enum extends super const export import",
	    6: "enum",
	    strict: "implements interface let package private protected public static yield",
	    strictBind: "eval arguments"
	    // And the keywords
	};
	var ecma5AndLessKeywords = "break case catch continue debugger default do else finally for function if return switch throw try var while with null true false instanceof typeof void delete new in this";
	exports.keywords = {
	    5: ecma5AndLessKeywords,
	    "5module": ecma5AndLessKeywords + " export import",
	    6: ecma5AndLessKeywords + " const class extends export import super"
	};
	exports.keywordRelationalOperator = /^in(stanceof)?$/;
	// ## Character categories
	// Big ugly regular expressions that match characters in the
	// whitespace, identifier, and identifier-start categories. These
	// are only applied when a character is found to actually have a
	// code point above 128.
	// Generated by `bin/generate-identifier-regex.js`.
	var nonASCIIidentifierStartChars = "\xaa\xb5\xba\xc0-\xd6\xd8-\xf6\xf8-\u02c1\u02c6-\u02d1\u02e0-\u02e4\u02ec\u02ee\u0370-\u0374\u0376\u0377\u037a-\u037d\u037f\u0386\u0388-\u038a\u038c\u038e-\u03a1\u03a3-\u03f5\u03f7-\u0481\u048a-\u052f\u0531-\u0556\u0559\u0560-\u0588\u05d0-\u05ea\u05ef-\u05f2\u0620-\u064a\u066e\u066f\u0671-\u06d3\u06d5\u06e5\u06e6\u06ee\u06ef\u06fa-\u06fc\u06ff\u0710\u0712-\u072f\u074d-\u07a5\u07b1\u07ca-\u07ea\u07f4\u07f5\u07fa\u0800-\u0815\u081a\u0824\u0828\u0840-\u0858\u0860-\u086a\u08a0-\u08b4\u08b6-\u08c7\u0904-\u0939\u093d\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098c\u098f\u0990\u0993-\u09a8\u09aa-\u09b0\u09b2\u09b6-\u09b9\u09bd\u09ce\u09dc\u09dd\u09df-\u09e1\u09f0\u09f1\u09fc\u0a05-\u0a0a\u0a0f\u0a10\u0a13-\u0a28\u0a2a-\u0a30\u0a32\u0a33\u0a35\u0a36\u0a38\u0a39\u0a59-\u0a5c\u0a5e\u0a72-\u0a74\u0a85-\u0a8d\u0a8f-\u0a91\u0a93-\u0aa8\u0aaa-\u0ab0\u0ab2\u0ab3\u0ab5-\u0ab9\u0abd\u0ad0\u0ae0\u0ae1\u0af9\u0b05-\u0b0c\u0b0f\u0b10\u0b13-\u0b28\u0b2a-\u0b30\u0b32\u0b33\u0b35-\u0b39\u0b3d\u0b5c\u0b5d\u0b5f-\u0b61\u0b71\u0b83\u0b85-\u0b8a\u0b8e-\u0b90\u0b92-\u0b95\u0b99\u0b9a\u0b9c\u0b9e\u0b9f\u0ba3\u0ba4\u0ba8-\u0baa\u0bae-\u0bb9\u0bd0\u0c05-\u0c0c\u0c0e-\u0c10\u0c12-\u0c28\u0c2a-\u0c39\u0c3d\u0c58-\u0c5a\u0c60\u0c61\u0c80\u0c85-\u0c8c\u0c8e-\u0c90\u0c92-\u0ca8\u0caa-\u0cb3\u0cb5-\u0cb9\u0cbd\u0cde\u0ce0\u0ce1\u0cf1\u0cf2\u0d04-\u0d0c\u0d0e-\u0d10\u0d12-\u0d3a\u0d3d\u0d4e\u0d54-\u0d56\u0d5f-\u0d61\u0d7a-\u0d7f\u0d85-\u0d96\u0d9a-\u0db1\u0db3-\u0dbb\u0dbd\u0dc0-\u0dc6\u0e01-\u0e30\u0e32\u0e33\u0e40-\u0e46\u0e81\u0e82\u0e84\u0e86-\u0e8a\u0e8c-\u0ea3\u0ea5\u0ea7-\u0eb0\u0eb2\u0eb3\u0ebd\u0ec0-\u0ec4\u0ec6\u0edc-\u0edf\u0f00\u0f40-\u0f47\u0f49-\u0f6c\u0f88-\u0f8c\u1000-\u102a\u103f\u1050-\u1055\u105a-\u105d\u1061\u1065\u1066\u106e-\u1070\u1075-\u1081\u108e\u10a0-\u10c5\u10c7\u10cd\u10d0-\u10fa\u10fc-\u1248\u124a-\u124d\u1250-\u1256\u1258\u125a-\u125d\u1260-\u1288\u128a-\u128d\u1290-\u12b0\u12b2-\u12b5\u12b8-\u12be\u12c0\u12c2-\u12c5\u12c8-\u12d6\u12d8-\u1310\u1312-\u1315\u1318-\u135a\u1380-\u138f\u13a0-\u13f5\u13f8-\u13fd\u1401-\u166c\u166f-\u167f\u1681-\u169a\u16a0-\u16ea\u16ee-\u16f8\u1700-\u170c\u170e-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176c\u176e-\u1770\u1780-\u17b3\u17d7\u17dc\u1820-\u1878\u1880-\u18a8\u18aa\u18b0-\u18f5\u1900-\u191e\u1950-\u196d\u1970-\u1974\u1980-\u19ab\u19b0-\u19c9\u1a00-\u1a16\u1a20-\u1a54\u1aa7\u1b05-\u1b33\u1b45-\u1b4b\u1b83-\u1ba0\u1bae\u1baf\u1bba-\u1be5\u1c00-\u1c23\u1c4d-\u1c4f\u1c5a-\u1c7d\u1c80-\u1c88\u1c90-\u1cba\u1cbd-\u1cbf\u1ce9-\u1cec\u1cee-\u1cf3\u1cf5\u1cf6\u1cfa\u1d00-\u1dbf\u1e00-\u1f15\u1f18-\u1f1d\u1f20-\u1f45\u1f48-\u1f4d\u1f50-\u1f57\u1f59\u1f5b\u1f5d\u1f5f-\u1f7d\u1f80-\u1fb4\u1fb6-\u1fbc\u1fbe\u1fc2-\u1fc4\u1fc6-\u1fcc\u1fd0-\u1fd3\u1fd6-\u1fdb\u1fe0-\u1fec\u1ff2-\u1ff4\u1ff6-\u1ffc\u2071\u207f\u2090-\u209c\u2102\u2107\u210a-\u2113\u2115\u2118-\u211d\u2124\u2126\u2128\u212a-\u2139\u213c-\u213f\u2145-\u2149\u214e\u2160-\u2188\u2c00-\u2c2e\u2c30-\u2c5e\u2c60-\u2ce4\u2ceb-\u2cee\u2cf2\u2cf3\u2d00-\u2d25\u2d27\u2d2d\u2d30-\u2d67\u2d6f\u2d80-\u2d96\u2da0-\u2da6\u2da8-\u2dae\u2db0-\u2db6\u2db8-\u2dbe\u2dc0-\u2dc6\u2dc8-\u2dce\u2dd0-\u2dd6\u2dd8-\u2dde\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303c\u3041-\u3096\u309b-\u309f\u30a1-\u30fa\u30fc-\u30ff\u3105-\u312f\u3131-\u318e\u31a0-\u31bf\u31f0-\u31ff\u3400-\u4dbf\u4e00-\u9ffc\ua000-\ua48c\ua4d0-\ua4fd\ua500-\ua60c\ua610-\ua61f\ua62a\ua62b\ua640-\ua66e\ua67f-\ua69d\ua6a0-\ua6ef\ua717-\ua71f\ua722-\ua788\ua78b-\ua7bf\ua7c2-\ua7ca\ua7f5-\ua801\ua803-\ua805\ua807-\ua80a\ua80c-\ua822\ua840-\ua873\ua882-\ua8b3\ua8f2-\ua8f7\ua8fb\ua8fd\ua8fe\ua90a-\ua925\ua930-\ua946\ua960-\ua97c\ua984-\ua9b2\ua9cf\ua9e0-\ua9e4\ua9e6-\ua9ef\ua9fa-\ua9fe\uaa00-\uaa28\uaa40-\uaa42\uaa44-\uaa4b\uaa60-\uaa76\uaa7a\uaa7e-\uaaaf\uaab1\uaab5\uaab6\uaab9-\uaabd\uaac0\uaac2\uaadb-\uaadd\uaae0-\uaaea\uaaf2-\uaaf4\uab01-\uab06\uab09-\uab0e\uab11-\uab16\uab20-\uab26\uab28-\uab2e\uab30-\uab5a\uab5c-\uab69\uab70-\uabe2\uac00-\ud7a3\ud7b0-\ud7c6\ud7cb-\ud7fb\uf900-\ufa6d\ufa70-\ufad9\ufb00-\ufb06\ufb13-\ufb17\ufb1d\ufb1f-\ufb28\ufb2a-\ufb36\ufb38-\ufb3c\ufb3e\ufb40\ufb41\ufb43\ufb44\ufb46-\ufbb1\ufbd3-\ufd3d\ufd50-\ufd8f\ufd92-\ufdc7\ufdf0-\ufdfb\ufe70-\ufe74\ufe76-\ufefc\uff21-\uff3a\uff41-\uff5a\uff66-\uffbe\uffc2-\uffc7\uffca-\uffcf\uffd2-\uffd7\uffda-\uffdc";
	var nonASCIIidentifierChars = "\u200c\u200d\xb7\u0300-\u036f\u0387\u0483-\u0487\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u0669\u0670\u06d6-\u06dc\u06df-\u06e4\u06e7\u06e8\u06ea-\u06ed\u06f0-\u06f9\u0711\u0730-\u074a\u07a6-\u07b0\u07c0-\u07c9\u07eb-\u07f3\u07fd\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0859-\u085b\u08d3-\u08e1\u08e3-\u0903\u093a-\u093c\u093e-\u094f\u0951-\u0957\u0962\u0963\u0966-\u096f\u0981-\u0983\u09bc\u09be-\u09c4\u09c7\u09c8\u09cb-\u09cd\u09d7\u09e2\u09e3\u09e6-\u09ef\u09fe\u0a01-\u0a03\u0a3c\u0a3e-\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a66-\u0a71\u0a75\u0a81-\u0a83\u0abc\u0abe-\u0ac5\u0ac7-\u0ac9\u0acb-\u0acd\u0ae2\u0ae3\u0ae6-\u0aef\u0afa-\u0aff\u0b01-\u0b03\u0b3c\u0b3e-\u0b44\u0b47\u0b48\u0b4b-\u0b4d\u0b55-\u0b57\u0b62\u0b63\u0b66-\u0b6f\u0b82\u0bbe-\u0bc2\u0bc6-\u0bc8\u0bca-\u0bcd\u0bd7\u0be6-\u0bef\u0c00-\u0c04\u0c3e-\u0c44\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0c66-\u0c6f\u0c81-\u0c83\u0cbc\u0cbe-\u0cc4\u0cc6-\u0cc8\u0cca-\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0ce6-\u0cef\u0d00-\u0d03\u0d3b\u0d3c\u0d3e-\u0d44\u0d46-\u0d48\u0d4a-\u0d4d\u0d57\u0d62\u0d63\u0d66-\u0d6f\u0d81-\u0d83\u0dca\u0dcf-\u0dd4\u0dd6\u0dd8-\u0ddf\u0de6-\u0def\u0df2\u0df3\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0e50-\u0e59\u0eb1\u0eb4-\u0ebc\u0ec8-\u0ecd\u0ed0-\u0ed9\u0f18\u0f19\u0f20-\u0f29\u0f35\u0f37\u0f39\u0f3e\u0f3f\u0f71-\u0f84\u0f86\u0f87\u0f8d-\u0f97\u0f99-\u0fbc\u0fc6\u102b-\u103e\u1040-\u1049\u1056-\u1059\u105e-\u1060\u1062-\u1064\u1067-\u106d\u1071-\u1074\u1082-\u108d\u108f-\u109d\u135d-\u135f\u1369-\u1371\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b4-\u17d3\u17dd\u17e0-\u17e9\u180b-\u180d\u1810-\u1819\u18a9\u1920-\u192b\u1930-\u193b\u1946-\u194f\u19d0-\u19da\u1a17-\u1a1b\u1a55-\u1a5e\u1a60-\u1a7c\u1a7f-\u1a89\u1a90-\u1a99\u1ab0-\u1abd\u1abf\u1ac0\u1b00-\u1b04\u1b34-\u1b44\u1b50-\u1b59\u1b6b-\u1b73\u1b80-\u1b82\u1ba1-\u1bad\u1bb0-\u1bb9\u1be6-\u1bf3\u1c24-\u1c37\u1c40-\u1c49\u1c50-\u1c59\u1cd0-\u1cd2\u1cd4-\u1ce8\u1ced\u1cf4\u1cf7-\u1cf9\u1dc0-\u1df9\u1dfb-\u1dff\u203f\u2040\u2054\u20d0-\u20dc\u20e1\u20e5-\u20f0\u2cef-\u2cf1\u2d7f\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua620-\ua629\ua66f\ua674-\ua67d\ua69e\ua69f\ua6f0\ua6f1\ua802\ua806\ua80b\ua823-\ua827\ua82c\ua880\ua881\ua8b4-\ua8c5\ua8d0-\ua8d9\ua8e0-\ua8f1\ua8ff-\ua909\ua926-\ua92d\ua947-\ua953\ua980-\ua983\ua9b3-\ua9c0\ua9d0-\ua9d9\ua9e5\ua9f0-\ua9f9\uaa29-\uaa36\uaa43\uaa4c\uaa4d\uaa50-\uaa59\uaa7b-\uaa7d\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uaaeb-\uaaef\uaaf5\uaaf6\uabe3-\uabea\uabec\uabed\uabf0-\uabf9\ufb1e\ufe00-\ufe0f\ufe20-\ufe2f\ufe33\ufe34\ufe4d-\ufe4f\uff10-\uff19\uff3f";
	var nonASCIIidentifierStart = new RegExp("[" + nonASCIIidentifierStartChars + "]");
	var nonASCIIidentifier = new RegExp("[" + nonASCIIidentifierStartChars + nonASCIIidentifierChars + "]");
	nonASCIIidentifierStartChars = nonASCIIidentifierChars = null;
	// These are a run-length and offset encoded representation of the
	// >0xffff code points that are a valid part of identifiers. The
	// offset starts at 0x10000, and each pair of numbers represents an
	// offset to the next range, and then a size of the range. They were
	// generated by bin/generate-identifier-regex.js
	// eslint-disable-next-line comma-spacing
	var astralIdentifierStartCodes = [0, 11, 2, 25, 2, 18, 2, 1, 2, 14, 3, 13, 35, 122, 70, 52, 268, 28, 4, 48, 48, 31, 14, 29, 6, 37, 11, 29, 3, 35, 5, 7, 2, 4, 43, 157, 19, 35, 5, 35, 5, 39, 9, 51, 157, 310, 10, 21, 11, 7, 153, 5, 3, 0, 2, 43, 2, 1, 4, 0, 3, 22, 11, 22, 10, 30, 66, 18, 2, 1, 11, 21, 11, 25, 71, 55, 7, 1, 65, 0, 16, 3, 2, 2, 2, 28, 43, 28, 4, 28, 36, 7, 2, 27, 28, 53, 11, 21, 11, 18, 14, 17, 111, 72, 56, 50, 14, 50, 14, 35, 349, 41, 7, 1, 79, 28, 11, 0, 9, 21, 107, 20, 28, 22, 13, 52, 76, 44, 33, 24, 27, 35, 30, 0, 3, 0, 9, 34, 4, 0, 13, 47, 15, 3, 22, 0, 2, 0, 36, 17, 2, 24, 85, 6, 2, 0, 2, 3, 2, 14, 2, 9, 8, 46, 39, 7, 3, 1, 3, 21, 2, 6, 2, 1, 2, 4, 4, 0, 19, 0, 13, 4, 159, 52, 19, 3, 21, 2, 31, 47, 21, 1, 2, 0, 185, 46, 42, 3, 37, 47, 21, 0, 60, 42, 14, 0, 72, 26, 230, 43, 117, 63, 32, 7, 3, 0, 3, 7, 2, 1, 2, 23, 16, 0, 2, 0, 95, 7, 3, 38, 17, 0, 2, 0, 29, 0, 11, 39, 8, 0, 22, 0, 12, 45, 20, 0, 35, 56, 264, 8, 2, 36, 18, 0, 50, 29, 113, 6, 2, 1, 2, 37, 22, 0, 26, 5, 2, 1, 2, 31, 15, 0, 328, 18, 190, 0, 80, 921, 103, 110, 18, 195, 2749, 1070, 4050, 582, 8634, 568, 8, 30, 114, 29, 19, 47, 17, 3, 32, 20, 6, 18, 689, 63, 129, 74, 6, 0, 67, 12, 65, 1, 2, 0, 29, 6135, 9, 1237, 43, 8, 8952, 286, 50, 2, 18, 3, 9, 395, 2309, 106, 6, 12, 4, 8, 8, 9, 5991, 84, 2, 70, 2, 1, 3, 0, 3, 1, 3, 3, 2, 11, 2, 0, 2, 6, 2, 64, 2, 3, 3, 7, 2, 6, 2, 27, 2, 3, 2, 4, 2, 0, 4, 6, 2, 339, 3, 24, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 7, 2357, 44, 11, 6, 17, 0, 370, 43, 1301, 196, 60, 67, 8, 0, 1205, 3, 2, 26, 2, 1, 2, 0, 3, 0, 2, 9, 2, 3, 2, 0, 2, 0, 7, 0, 5, 0, 2, 0, 2, 0, 2, 2, 2, 1, 2, 0, 3, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 2, 0, 3, 3, 2, 6, 2, 3, 2, 3, 2, 0, 2, 9, 2, 16, 6, 2, 2, 4, 2, 16, 4421, 42717, 35, 4148, 12, 221, 3, 5761, 15, 7472, 3104, 541, 1507, 4938];
	// eslint-disable-next-line comma-spacing
	var astralIdentifierCodes = [509, 0, 227, 0, 150, 4, 294, 9, 1368, 2, 2, 1, 6, 3, 41, 2, 5, 0, 166, 1, 574, 3, 9, 9, 370, 1, 154, 10, 176, 2, 54, 14, 32, 9, 16, 3, 46, 10, 54, 9, 7, 2, 37, 13, 2, 9, 6, 1, 45, 0, 13, 2, 49, 13, 9, 3, 2, 11, 83, 11, 7, 0, 161, 11, 6, 9, 7, 3, 56, 1, 2, 6, 3, 1, 3, 2, 10, 0, 11, 1, 3, 6, 4, 4, 193, 17, 10, 9, 5, 0, 82, 19, 13, 9, 214, 6, 3, 8, 28, 1, 83, 16, 16, 9, 82, 12, 9, 9, 84, 14, 5, 9, 243, 14, 166, 9, 71, 5, 2, 1, 3, 3, 2, 0, 2, 1, 13, 9, 120, 6, 3, 6, 4, 0, 29, 9, 41, 6, 2, 3, 9, 0, 10, 10, 47, 15, 406, 7, 2, 7, 17, 9, 57, 21, 2, 13, 123, 5, 4, 0, 2, 1, 2, 6, 2, 0, 9, 9, 49, 4, 2, 1, 2, 4, 9, 9, 330, 3, 19306, 9, 135, 4, 60, 6, 26, 9, 1014, 0, 2, 54, 8, 3, 82, 0, 12, 1, 19628, 1, 5319, 4, 4, 5, 9, 7, 3, 6, 31, 3, 149, 2, 1418, 49, 513, 54, 5, 49, 9, 0, 15, 0, 23, 4, 2, 14, 1361, 6, 2, 16, 3, 6, 2, 1, 2, 4, 262, 6, 10, 9, 419, 13, 1495, 6, 110, 6, 6, 9, 4759, 9, 787719, 239];
	// This has a complexity linear to the value of the code. The
	// assumption is that looking up astral identifier characters is
	// rare.
	function isInAstralSet(code, set) {
	    var pos = 0x10000;
	    for (var i = 0; i < set.length; i += 2) {
	        pos += set[i];
	        if (pos > code)
	            return false;
	        pos += set[i + 1];
	        if (pos >= code)
	            return true;
	    }
	}
	// Test whether a given character code starts an identifier.
	function isIdentifierStart(code, astral) {
	    if (code < 65)
	        return code === 36;
	    if (code < 91)
	        return true;
	    if (code < 97)
	        return code === 95;
	    if (code < 123)
	        return true;
	    if (code <= 0xffff)
	        return code >= 0xaa && nonASCIIidentifierStart.test(String.fromCharCode(code));
	    if (astral === false)
	        return false;
	    return isInAstralSet(code, astralIdentifierStartCodes);
	}
	// Test whether a given character is part of an identifier.
	function isIdentifierChar(code, astral) {
	    if (code < 48)
	        return code === 36;
	    if (code < 58)
	        return true;
	    if (code < 65)
	        return false;
	    if (code < 91)
	        return true;
	    if (code < 97)
	        return code === 95;
	    if (code < 123)
	        return true;
	    if (code <= 0xffff)
	        return code >= 0xaa && nonASCIIidentifier.test(String.fromCharCode(code));
	    if (astral === false)
	        return false;
	    return isInAstralSet(code, astralIdentifierStartCodes) || isInAstralSet(code, astralIdentifierCodes);
	}
	});

	var tokentype = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	// ## Token types
	// The assignment of fine-grained, information-carrying type objects
	// allows the tokenizer to store the information it has about a
	// token in a way that is very cheap for the parser to look up.
	// All token type variables start with an underscore, to make them
	// easy to recognize.
	// The `beforeExpr` property is used to disambiguate between regular
	// expressions and divisions. It is set on all token types that can
	// be followed by an expression (thus, a slash after them would be a
	// regular expression).
	//
	// The `startsExpr` property is used to check if the token ends a
	// `yield` expression. It is set on all token types that either can
	// directly start an expression (like a quotation mark) or can
	// continue an expression (like the body of a string).
	//
	// `isLoop` marks a keyword as starting a loop, which is important
	// to know when parsing a label, in order to allow or disallow
	// continue jumps to that label.
	var TokenType = /** @class */ (function () {
	    function TokenType(label, conf) {
	        if (conf === void 0) { conf = {}; }
	        this.label = label;
	        this.keyword = conf.keyword;
	        this.beforeExpr = !!conf.beforeExpr;
	        this.startsExpr = !!conf.startsExpr;
	        this.isLoop = !!conf.isLoop;
	        this.isAssign = !!conf.isAssign;
	        this.prefix = !!conf.prefix;
	        this.postfix = !!conf.postfix;
	        this.binop = conf.binop || null;
	        this.updateContext = null;
	    }
	    return TokenType;
	}());
	exports.TokenType = TokenType;
	function binop(name, prec) {
	    return new TokenType(name, { beforeExpr: true, binop: prec });
	}
	var beforeExpr = { beforeExpr: true }, startsExpr = { startsExpr: true
	    // Map keyword names to token types.
	};
	var keywords = exports.keywords = {};
	// Succinct definitions of keyword token types
	function kw(name, options) {
	    if (options === void 0) { options = {}; }
	    options.keyword = name;
	    return keywords[name] = new TokenType(name, options);
	}
	exports.types = {
	    num: new TokenType("num", startsExpr),
	    regexp: new TokenType("regexp", startsExpr),
	    string: new TokenType("string", startsExpr),
	    name: new TokenType("name", startsExpr),
	    eof: new TokenType("eof"),
	    // Punctuation token types.
	    bracketL: new TokenType("[", { beforeExpr: true, startsExpr: true }),
	    bracketR: new TokenType("]"),
	    braceL: new TokenType("{", { beforeExpr: true, startsExpr: true }),
	    braceR: new TokenType("}"),
	    parenL: new TokenType("(", { beforeExpr: true, startsExpr: true }),
	    parenR: new TokenType(")"),
	    comma: new TokenType(",", beforeExpr),
	    semi: new TokenType(";", beforeExpr),
	    colon: new TokenType(":", beforeExpr),
	    dot: new TokenType("."),
	    question: new TokenType("?", beforeExpr),
	    questionDot: new TokenType("?."),
	    arrow: new TokenType("=>", beforeExpr),
	    template: new TokenType("template"),
	    invalidTemplate: new TokenType("invalidTemplate"),
	    ellipsis: new TokenType("...", beforeExpr),
	    backQuote: new TokenType("`", startsExpr),
	    dollarBraceL: new TokenType("${", { beforeExpr: true, startsExpr: true }),
	    // Operators. These carry several kinds of properties to help the
	    // parser use them properly (the presence of these properties is
	    // what categorizes them as operators).
	    //
	    // `binop`, when present, specifies that this operator is a binary
	    // operator, and will refer to its precedence.
	    //
	    // `prefix` and `postfix` mark the operator as a prefix or postfix
	    // unary operator.
	    //
	    // `isAssign` marks all of `=`, `+=`, `-=` etcetera, which act as
	    // binary operators with a very low precedence, that should result
	    // in AssignmentExpression nodes.
	    eq: new TokenType("=", { beforeExpr: true, isAssign: true }),
	    assign: new TokenType("_=", { beforeExpr: true, isAssign: true }),
	    incDec: new TokenType("++/--", { prefix: true, postfix: true, startsExpr: true }),
	    prefix: new TokenType("!/~", { beforeExpr: true, prefix: true, startsExpr: true }),
	    logicalOR: binop("||", 1),
	    logicalAND: binop("&&", 2),
	    bitwiseOR: binop("|", 3),
	    bitwiseXOR: binop("^", 4),
	    bitwiseAND: binop("&", 5),
	    equality: binop("==/!=/===/!==", 6),
	    relational: binop("</>/<=/>=", 7),
	    bitShift: binop("<</>>/>>>", 8),
	    plusMin: new TokenType("+/-", { beforeExpr: true, binop: 9, prefix: true, startsExpr: true }),
	    modulo: binop("%", 10),
	    star: binop("*", 10),
	    slash: binop("/", 10),
	    starstar: new TokenType("**", { beforeExpr: true }),
	    coalesce: binop("??", 1),
	    // Keyword token types.
	    _break: kw("break"),
	    _case: kw("case", beforeExpr),
	    _catch: kw("catch"),
	    _continue: kw("continue"),
	    _debugger: kw("debugger"),
	    _default: kw("default", beforeExpr),
	    _do: kw("do", { isLoop: true, beforeExpr: true }),
	    _else: kw("else", beforeExpr),
	    _finally: kw("finally"),
	    _for: kw("for", { isLoop: true }),
	    _function: kw("function", startsExpr),
	    _if: kw("if"),
	    _return: kw("return", beforeExpr),
	    _switch: kw("switch"),
	    _throw: kw("throw", beforeExpr),
	    _try: kw("try"),
	    _var: kw("var"),
	    _const: kw("const"),
	    _while: kw("while", { isLoop: true }),
	    _with: kw("with"),
	    _new: kw("new", { beforeExpr: true, startsExpr: true }),
	    _this: kw("this", startsExpr),
	    _super: kw("super", startsExpr),
	    _class: kw("class", startsExpr),
	    _extends: kw("extends", beforeExpr),
	    _export: kw("export"),
	    _import: kw("import", startsExpr),
	    _null: kw("null", startsExpr),
	    _true: kw("true", startsExpr),
	    _false: kw("false", startsExpr),
	    _in: kw("in", { beforeExpr: true, binop: 7 }),
	    _instanceof: kw("instanceof", { beforeExpr: true, binop: 7 }),
	    _typeof: kw("typeof", { beforeExpr: true, prefix: true, startsExpr: true }),
	    _void: kw("void", { beforeExpr: true, prefix: true, startsExpr: true }),
	    _delete: kw("delete", { beforeExpr: true, prefix: true, startsExpr: true })
	};
	});

	var whitespace = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.isNewLine = isNewLine;
	// Matches a whole line break (where CRLF is considered a single
	// line break). Used to count lines.
	var lineBreak = exports.lineBreak = /\r\n?|\n|\u2028|\u2029/;
	exports.lineBreakG = new RegExp(lineBreak.source, "g");
	function isNewLine(code, ecma2019String) {
	    return code === 10 || code === 13 || !ecma2019String && (code === 0x2028 || code === 0x2029);
	}
	exports.nonASCIIwhitespace = /[\u1680\u2000-\u200a\u202f\u205f\u3000\ufeff]/;
	exports.skipWhiteSpace = /(?:\s|\/\/.*|\/\*[^]*?\*\/)*/g;
	});

	var util = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.has = has;
	exports.wordsRegexp = wordsRegexp;
	var _Object$prototype = Object.prototype;
	var hasOwnProperty = _Object$prototype.hasOwnProperty, toString = _Object$prototype.toString;
	// Checks if an object has a property.
	function has(obj, propName) {
	    return hasOwnProperty.call(obj, propName);
	}
	exports.isArray = Array.isArray || (function (obj) { return toString.call(obj) === "[object Array]"; });
	function wordsRegexp(words) {
	    return new RegExp("^(?:" + words.replace(/ /g, "|") + ")$");
	}
	});

	var locutil = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.SourceLocation = exports.Position = undefined;
	exports.getLineInfo = getLineInfo;

	// These are used when `options.locations` is on, for the
	// `startLoc` and `endLoc` properties.
	var Position = /** @class */ (function () {
	    function Position(line, col) {
	        this.line = line;
	        this.column = col;
	    }
	    Position.prototype.offset = function (n) {
	        return new Position(this.line, this.column + n);
	    };
	    return Position;
	}());
	exports.Position = Position;
	var SourceLocation = /** @class */ (function () {
	    function SourceLocation(p, start, end) {
	        this.start = start;
	        this.end = end;
	        if (p.sourceFile !== null)
	            this.source = p.sourceFile;
	    }
	    return SourceLocation;
	}());
	exports.SourceLocation = SourceLocation; // The `getLineInfo` function is mostly useful when the
	// `locations` option is off (for performance reasons) and you
	// want to find the line/column position for a given character
	// offset. `input` should be the code string that the offset refers
	// into.
	function getLineInfo(input, offset) {
	    for (var line = 1, cur = 0;;) {
	        whitespace.lineBreakG.lastIndex = cur;
	        var match = whitespace.lineBreakG.exec(input);
	        if (match && match.index < offset) {
	            ++line;
	            cur = match.index + match[0].length;
	        }
	        else {
	            return new Position(line, offset - cur);
	        }
	    }
	}
	});

	var options = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.defaultOptions = undefined;
	exports.getOptions = getOptions;


	// A second argument must be given to configure the parser process.
	// These options are recognized (only `ecmaVersion` is required):
	var defaultOptions = exports.defaultOptions = {
	    // `ecmaVersion` indicates the ECMAScript version to parse. Must be
	    // either 3, 5, 6 (or 2015), 7 (2016), 8 (2017), 9 (2018), 10
	    // (2019), 11 (2020), 12 (2021), or `"latest"` (the latest version
	    // the library supports). This influences support for strict mode,
	    // the set of reserved words, and support for new syntax features.
	    ecmaVersion: null,
	    // `sourceType` indicates the mode the code should be parsed in.
	    // Can be either `"script"` or `"module"`. This influences global
	    // strict mode and parsing of `import` and `export` declarations.
	    sourceType: "script",
	    // `onInsertedSemicolon` can be a callback that will be called
	    // when a semicolon is automatically inserted. It will be passed
	    // the position of the comma as an offset, and if `locations` is
	    // enabled, it is given the location as a `{line, column}` object
	    // as second argument.
	    onInsertedSemicolon: null,
	    // `onTrailingComma` is similar to `onInsertedSemicolon`, but for
	    // trailing commas.
	    onTrailingComma: null,
	    // By default, reserved words are only enforced if ecmaVersion >= 5.
	    // Set `allowReserved` to a boolean value to explicitly turn this on
	    // an off. When this option has the value "never", reserved words
	    // and keywords can also not be used as property names.
	    allowReserved: null,
	    // When enabled, a return at the top level is not considered an
	    // error.
	    allowReturnOutsideFunction: false,
	    // When enabled, import/export statements are not constrained to
	    // appearing at the top of the program, and an import.meta expression
	    // in a script isn't considered an error.
	    allowImportExportEverywhere: false,
	    // When enabled, await identifiers are allowed to appear at the top-level scope,
	    // but they are still not allowed in non-async functions.
	    allowAwaitOutsideFunction: false,
	    // When enabled, hashbang directive in the beginning of file
	    // is allowed and treated as a line comment.
	    allowHashBang: false,
	    // When `locations` is on, `loc` properties holding objects with
	    // `start` and `end` properties in `{line, column}` form (with
	    // line being 1-based and column 0-based) will be attached to the
	    // nodes.
	    locations: false,
	    // A function can be passed as `onToken` option, which will
	    // cause Acorn to call that function with object in the same
	    // format as tokens returned from `tokenizer().getToken()`. Note
	    // that you are not allowed to call the parser from the
	    // callback—that will corrupt its internal state.
	    onToken: null,
	    // A function can be passed as `onComment` option, which will
	    // cause Acorn to call that function with `(block, text, start,
	    // end)` parameters whenever a comment is skipped. `block` is a
	    // boolean indicating whether this is a block (`/* */`) comment,
	    // `text` is the content of the comment, and `start` and `end` are
	    // character offsets that denote the start and end of the comment.
	    // When the `locations` option is on, two more parameters are
	    // passed, the full `{line, column}` locations of the start and
	    // end of the comments. Note that you are not allowed to call the
	    // parser from the callback—that will corrupt its internal state.
	    onComment: null,
	    // Nodes have their start and end characters offsets recorded in
	    // `start` and `end` properties (directly on the node, rather than
	    // the `loc` object, which holds line/column data. To also add a
	    // [semi-standardized][range] `range` property holding a `[start,
	    // end]` array with the same numbers, set the `ranges` option to
	    // `true`.
	    //
	    // [range]: https://bugzilla.mozilla.org/show_bug.cgi?id=745678
	    ranges: false,
	    // It is possible to parse multiple files into a single AST by
	    // passing the tree produced by parsing the first file as
	    // `program` option in subsequent parses. This will add the
	    // toplevel forms of the parsed file to the `Program` (top) node
	    // of an existing parse tree.
	    program: null,
	    // When `locations` is on, you can pass this to record the source
	    // file in every node's `loc` object.
	    sourceFile: null,
	    // This value, if given, is stored in every node, whether
	    // `locations` is on or off.
	    directSourceFile: null,
	    // When enabled, parenthesized expressions are represented by
	    // (non-standard) ParenthesizedExpression nodes
	    preserveParens: false
	    // Interpret and default an options object
	};
	var warnedAboutEcmaVersion = false;
	function getOptions(opts) {
	    var options = {};
	    for (var opt in defaultOptions)
	        options[opt] = opts && (0, util.has)(opts, opt) ? opts[opt] : defaultOptions[opt];
	    if (options.ecmaVersion === "latest") {
	        options.ecmaVersion = 1e8;
	    }
	    else if (options.ecmaVersion == null) {
	        if (!warnedAboutEcmaVersion && typeof console === "object" && console.warn) {
	            warnedAboutEcmaVersion = true;
	            console.warn("Since Acorn 8.0.0, options.ecmaVersion is required.\nDefaulting to 2020, but this will stop working in the future.");
	        }
	        options.ecmaVersion = 11;
	    }
	    else if (options.ecmaVersion >= 2015) {
	        options.ecmaVersion -= 2009;
	    }
	    if (options.allowReserved == null)
	        options.allowReserved = options.ecmaVersion < 5;
	    if ((0, util.isArray)(options.onToken)) {
	        var tokens_1 = options.onToken;
	        options.onToken = function (token) { return tokens_1.push(token); };
	    }
	    if ((0, util.isArray)(options.onComment))
	        options.onComment = pushComment(options, options.onComment);
	    return options;
	}
	function pushComment(options, array) {
	    return function (block, text, start, end, startLoc, endLoc) {
	        var comment = {
	            type: block ? "Block" : "Line",
	            value: text,
	            start: start,
	            end: end
	        };
	        if (options.locations)
	            comment.loc = new locutil.SourceLocation(this, startLoc, endLoc);
	        if (options.ranges)
	            comment.range = [start, end];
	        array.push(comment);
	    };
	}
	});

	var scopeflags = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.functionFlags = functionFlags;
	// Each scope gets a bitset that may contain these flags
	var SCOPE_TOP = exports.SCOPE_TOP = 1, SCOPE_FUNCTION = exports.SCOPE_FUNCTION = 2; exports.SCOPE_VAR = SCOPE_TOP | SCOPE_FUNCTION; var SCOPE_ASYNC = exports.SCOPE_ASYNC = 4, SCOPE_GENERATOR = exports.SCOPE_GENERATOR = 8; exports.SCOPE_ARROW = 16; exports.SCOPE_SIMPLE_CATCH = 32; exports.SCOPE_SUPER = 64; exports.SCOPE_DIRECT_SUPER = 128;
	function functionFlags(async, generator) {
	    return SCOPE_FUNCTION | (async ? SCOPE_ASYNC : 0) | (generator ? SCOPE_GENERATOR : 0);
	}
	// Used in checkLVal* and declareName to determine the type of a binding
	exports.BIND_NONE = 0; 
	// Not a binding
	exports.BIND_VAR = 1; 
	// Var-style binding
	exports.BIND_LEXICAL = 2; 
	// Let- or const-style binding
	exports.BIND_FUNCTION = 3; 
	// Function declaration
	exports.BIND_SIMPLE_CATCH = 4; 
	// Simple (identifier pattern) catch binding
	exports.BIND_OUTSIDE = 5; // Special case for function names as bound inside the function
	});

	var state = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.Parser = undefined;






	var Parser = /** @class */ (function () {
	    function Parser(options$1, input, startPos) {
	        this.options = options$1 = (0, options.getOptions)(options$1);
	        this.sourceFile = options$1.sourceFile;
	        this.keywords = (0, util.wordsRegexp)(identifier.keywords[options$1.ecmaVersion >= 6 ? 6 : options$1.sourceType === "module" ? "5module" : 5]);
	        var reserved = "";
	        if (options$1.allowReserved !== true) {
	            reserved = identifier.reservedWords[options$1.ecmaVersion >= 6 ? 6 : options$1.ecmaVersion === 5 ? 5 : 3];
	            if (options$1.sourceType === "module")
	                reserved += " await";
	        }
	        this.reservedWords = (0, util.wordsRegexp)(reserved);
	        var reservedStrict = (reserved ? reserved + " " : "") + identifier.reservedWords.strict;
	        this.reservedWordsStrict = (0, util.wordsRegexp)(reservedStrict);
	        this.reservedWordsStrictBind = (0, util.wordsRegexp)(reservedStrict + " " + identifier.reservedWords.strictBind);
	        this.input = String(input);
	        // Used to signal to callers of `readWord1` whether the word
	        // contained any escape sequences. This is needed because words with
	        // escape sequences must not be interpreted as keywords.
	        this.containsEsc = false;
	        // Set up token state
	        // The current position of the tokenizer in the input.
	        if (startPos) {
	            this.pos = startPos;
	            this.lineStart = this.input.lastIndexOf("\n", startPos - 1) + 1;
	            this.curLine = this.input.slice(0, this.lineStart).split(whitespace.lineBreak).length;
	        }
	        else {
	            this.pos = this.lineStart = 0;
	            this.curLine = 1;
	        }
	        // Properties of the current token:
	        // Its type
	        this.type = tokentype.types.eof;
	        // For tokens that include more information than their type, the value
	        this.value = null;
	        // Its start and end offset
	        this.start = this.end = this.pos;
	        // And, if locations are used, the {line, column} object
	        // corresponding to those offsets
	        this.startLoc = this.endLoc = this.curPosition();
	        // Position information for the previous token
	        this.lastTokEndLoc = this.lastTokStartLoc = null;
	        this.lastTokStart = this.lastTokEnd = this.pos;
	        // The context stack is used to superficially track syntactic
	        // context to predict whether a regular expression is allowed in a
	        // given position.
	        this.context = this.initialContext();
	        this.exprAllowed = true;
	        // Figure out if it's a module code.
	        this.inModule = options$1.sourceType === "module";
	        this.strict = this.inModule || this.strictDirective(this.pos);
	        // Used to signify the start of a potential arrow function
	        this.potentialArrowAt = -1;
	        // Positions to delayed-check that yield/await does not exist in default parameters.
	        this.yieldPos = this.awaitPos = this.awaitIdentPos = 0;
	        // Labels in scope.
	        this.labels = [];
	        // Thus-far undefined exports.
	        this.undefinedExports = Object.create(null);
	        // If enabled, skip leading hashbang line.
	        if (this.pos === 0 && options$1.allowHashBang && this.input.slice(0, 2) === "#!")
	            this.skipLineComment(2);
	        // Scope tracking for duplicate variable names (see scope.js)
	        this.scopeStack = [];
	        this.enterScope(scopeflags.SCOPE_TOP);
	        // For RegExp validation
	        this.regexpState = null;
	    }
	    Parser.prototype.parse = function () {
	        var node = this.options.program || this.startNode();
	        this.nextToken();
	        return this.parseTopLevel(node);
	    };
	    Object.defineProperty(Parser.prototype, "inFunction", {
	        get: function () {
	            return (this.currentVarScope().flags & scopeflags.SCOPE_FUNCTION) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "inGenerator", {
	        get: function () {
	            return (this.currentVarScope().flags & scopeflags.SCOPE_GENERATOR) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "inAsync", {
	        get: function () {
	            return (this.currentVarScope().flags & scopeflags.SCOPE_ASYNC) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "allowSuper", {
	        get: function () {
	            return (this.currentThisScope().flags & scopeflags.SCOPE_SUPER) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "allowDirectSuper", {
	        get: function () {
	            return (this.currentThisScope().flags & scopeflags.SCOPE_DIRECT_SUPER) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "treatFunctionsAsVar", {
	        get: function () {
	            return this.treatFunctionsAsVarInScope(this.currentScope());
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Object.defineProperty(Parser.prototype, "inNonArrowFunction", {
	        get: function () {
	            return (this.currentThisScope().flags & scopeflags.SCOPE_FUNCTION) > 0;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    Parser.extend = function () {
	        var plugins = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            plugins[_i] = arguments[_i];
	        }
	        var cls = this;
	        for (var i = 0; i < plugins.length; i++)
	            cls = plugins[i](cls);
	        return cls;
	    };
	    Parser.parse = function (input, options) {
	        return new this(options, input).parse();
	    };
	    Parser.parseExpressionAt = function (input, pos, options) {
	        var parser = new this(options, input, pos);
	        parser.nextToken();
	        return parser.parseExpression();
	    };
	    Parser.tokenizer = function (input, options) {
	        return new this(options, input);
	    };
	    return Parser;
	}());
	exports.Parser = Parser;
	});

	var DestructuringErrors_1 = DestructuringErrors;



	var pp$5 = state.Parser.prototype;
	// ## Parser utilities
	var literal = /^(?:'((?:\\.|[^'\\])*?)'|"((?:\\.|[^"\\])*?)")/;
	pp$5.strictDirective = function (start) {
	    for (;;) {
	        // Try to find string literal.
	        whitespace.skipWhiteSpace.lastIndex = start;
	        start += whitespace.skipWhiteSpace.exec(this.input)[0].length;
	        var match = literal.exec(this.input.slice(start));
	        if (!match)
	            return false;
	        if ((match[1] || match[2]) === "use strict")
	            return false;
	        start += match[0].length;
	        // Skip semicolon, if any.
	        whitespace.skipWhiteSpace.lastIndex = start;
	        start += whitespace.skipWhiteSpace.exec(this.input)[0].length;
	        if (this.input[start] === ";")
	            start++;
	    }
	};
	// Predicate that tests whether the next token is of the given
	// type, and if yes, consumes it as a side effect.
	pp$5.eat = function (type) {
	    if (this.type === type) {
	        this.next();
	        return true;
	    }
	    else {
	        return false;
	    }
	};
	// Tests whether parsed token is a contextual keyword.
	pp$5.isContextual = function (name) {
	    return this.type === tokentype.types.name && this.value === name && !this.containsEsc;
	};
	// Consumes contextual keyword if possible.
	pp$5.eatContextual = function (name) {
	    if (!this.isContextual(name))
	        return false;
	    this.next();
	    return true;
	};
	// Asserts that following token is given contextual keyword.
	pp$5.expectContextual = function (name) {
	    if (!this.eatContextual(name))
	        this.unexpected();
	};
	// Test whether a semicolon can be inserted at the current position.
	pp$5.canInsertSemicolon = function () {
	    return this.type === tokentype.types.eof || this.type === tokentype.types.braceR || whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
	};
	pp$5.insertSemicolon = function () {
	    if (this.canInsertSemicolon()) {
	        if (this.options.onInsertedSemicolon)
	            this.options.onInsertedSemicolon(this.lastTokEnd, this.lastTokEndLoc);
	        return true;
	    }
	};
	// Consume a semicolon, or, failing that, see if we are allowed to
	// pretend that there is a semicolon at this position.
	pp$5.semicolon = function () {
	    if (!this.eat(tokentype.types.semi) && !this.insertSemicolon())
	        this.unexpected();
	};
	pp$5.afterTrailingComma = function (tokType, notNext) {
	    if (this.type === tokType) {
	        if (this.options.onTrailingComma)
	            this.options.onTrailingComma(this.lastTokStart, this.lastTokStartLoc);
	        if (!notNext)
	            this.next();
	        return true;
	    }
	};
	// Expect a token of a given type. If found, consume it, otherwise,
	// raise an unexpected token error.
	pp$5.expect = function (type) {
	    this.eat(type) || this.unexpected();
	};
	// Raise an unexpected token error.
	pp$5.unexpected = function (pos) {
	    this.raise(pos != null ? pos : this.start, "Unexpected token");
	};
	function DestructuringErrors() {
	    this.shorthandAssign = this.trailingComma = this.parenthesizedAssign = this.parenthesizedBind = this.doubleProto = -1;
	}
	pp$5.checkPatternErrors = function (refDestructuringErrors, isAssign) {
	    if (!refDestructuringErrors)
	        return;
	    if (refDestructuringErrors.trailingComma > -1)
	        this.raiseRecoverable(refDestructuringErrors.trailingComma, "Comma is not permitted after the rest element");
	    var parens = isAssign ? refDestructuringErrors.parenthesizedAssign : refDestructuringErrors.parenthesizedBind;
	    if (parens > -1)
	        this.raiseRecoverable(parens, "Parenthesized pattern");
	};
	pp$5.checkExpressionErrors = function (refDestructuringErrors, andThrow) {
	    if (!refDestructuringErrors)
	        return false;
	    var shorthandAssign = refDestructuringErrors.shorthandAssign, doubleProto = refDestructuringErrors.doubleProto;
	    if (!andThrow)
	        return shorthandAssign >= 0 || doubleProto >= 0;
	    if (shorthandAssign >= 0)
	        this.raise(shorthandAssign, "Shorthand property assignments are valid only in destructuring patterns");
	    if (doubleProto >= 0)
	        this.raiseRecoverable(doubleProto, "Redefinition of __proto__ property");
	};
	pp$5.checkYieldAwaitInDefaultParams = function () {
	    if (this.yieldPos && (!this.awaitPos || this.yieldPos < this.awaitPos))
	        this.raise(this.yieldPos, "Yield expression cannot be a default value");
	    if (this.awaitPos)
	        this.raise(this.awaitPos, "Await expression cannot be a default value");
	};
	pp$5.isSimpleAssignTarget = function (expr) {
	    if (expr.type === "ParenthesizedExpression")
	        return this.isSimpleAssignTarget(expr.expression);
	    return expr.type === "Identifier" || expr.type === "MemberExpression";
	};

	var parseutil = /*#__PURE__*/Object.defineProperty({
		DestructuringErrors: DestructuringErrors_1
	}, '__esModule', {value: true});

	var pp$4 = state.Parser.prototype;
	// ### Statement parsing
	// Parse a program. Initializes the parser, reads any number of
	// statements, and wraps them in a Program node.  Optionally takes a
	// `program` argument.  If present, the statements will be appended
	// to its body instead of creating a new node.
	pp$4.parseTopLevel = function (node) {
	    var exports = Object.create(null);
	    if (!node.body)
	        node.body = [];
	    while (this.type !== tokentype.types.eof) {
	        var stmt = this.parseStatement(null, true, exports);
	        node.body.push(stmt);
	    }
	    if (this.inModule) {
	        for (var _iterator = Object.keys(this.undefinedExports), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
	            var _ref;
	            if (_isArray) {
	                if (_i >= _iterator.length)
	                    break;
	                _ref = _iterator[_i++];
	            }
	            else {
	                _i = _iterator.next();
	                if (_i.done)
	                    break;
	                _ref = _i.value;
	            }
	            var name_1 = _ref;
	            this.raiseRecoverable(this.undefinedExports[name_1].start, "Export '" + name_1 + "' is not defined");
	        }
	    }
	    this.adaptDirectivePrologue(node.body);
	    this.next();
	    node.sourceType = this.options.sourceType;
	    return this.finishNode(node, "Program");
	};
	var loopLabel = { kind: "loop" }, switchLabel = { kind: "switch" };
	pp$4.isLet = function (context) {
	    if (this.options.ecmaVersion < 6 || !this.isContextual("let"))
	        return false;
	    whitespace.skipWhiteSpace.lastIndex = this.pos;
	    var skip = whitespace.skipWhiteSpace.exec(this.input);
	    var next = this.pos + skip[0].length, nextCh = this.input.charCodeAt(next);
	    // For ambiguous cases, determine if a LexicalDeclaration (or only a
	    // Statement) is allowed here. If context is not empty then only a Statement
	    // is allowed. However, `let [` is an explicit negative lookahead for
	    // ExpressionStatement, so special-case it first.
	    if (nextCh === 91)
	        return true; // '['
	    if (context)
	        return false;
	    if (nextCh === 123)
	        return true; // '{'
	    if ((0, identifier.isIdentifierStart)(nextCh, true)) {
	        var pos = next + 1;
	        while ((0, identifier.isIdentifierChar)(this.input.charCodeAt(pos), true))
	            ++pos;
	        var ident = this.input.slice(next, pos);
	        if (!identifier.keywordRelationalOperator.test(ident))
	            return true;
	    }
	    return false;
	};
	// check 'async [no LineTerminator here] function'
	// - 'async /*foo*/ function' is OK.
	// - 'async /*\n*/ function' is invalid.
	pp$4.isAsyncFunction = function () {
	    if (this.options.ecmaVersion < 8 || !this.isContextual("async"))
	        return false;
	    whitespace.skipWhiteSpace.lastIndex = this.pos;
	    var skip = whitespace.skipWhiteSpace.exec(this.input);
	    var next = this.pos + skip[0].length;
	    return !whitespace.lineBreak.test(this.input.slice(this.pos, next)) && this.input.slice(next, next + 8) === "function" && (next + 8 === this.input.length || !(0, identifier.isIdentifierChar)(this.input.charAt(next + 8)));
	};
	// Parse a single statement.
	//
	// If expecting a statement and finding a slash operator, parse a
	// regular expression literal. This is to handle cases like
	// `if (foo) /blah/.exec(foo)`, where looking at the previous token
	// does not help.
	pp$4.parseStatement = function (context, topLevel, exports) {
	    var starttype = this.type, node = this.startNode(), kind;
	    if (this.isLet(context)) {
	        starttype = tokentype.types._var;
	        kind = "let";
	    }
	    // Most types of statements are recognized by the keyword they
	    // start with. Many are trivial to parse, some require a bit of
	    // complexity.
	    switch (starttype) {
	        case tokentype.types._break:
	        case tokentype.types._continue:
	            return this.parseBreakContinueStatement(node, starttype.keyword);
	        case tokentype.types._debugger:
	            return this.parseDebuggerStatement(node);
	        case tokentype.types._do:
	            return this.parseDoStatement(node);
	        case tokentype.types._for:
	            return this.parseForStatement(node);
	        case tokentype.types._function:
	            // Function as sole body of either an if statement or a labeled statement
	            // works, but not when it is part of a labeled statement that is the sole
	            // body of an if statement.
	            if (context && (this.strict || context !== "if" && context !== "label") && this.options.ecmaVersion >= 6)
	                this.unexpected();
	            return this.parseFunctionStatement(node, false, !context);
	        case tokentype.types._class:
	            if (context)
	                this.unexpected();
	            return this.parseClass(node, true);
	        case tokentype.types._if:
	            return this.parseIfStatement(node);
	        case tokentype.types._return:
	            return this.parseReturnStatement(node);
	        case tokentype.types._switch:
	            return this.parseSwitchStatement(node);
	        case tokentype.types._throw:
	            return this.parseThrowStatement(node);
	        case tokentype.types._try:
	            return this.parseTryStatement(node);
	        case tokentype.types._const:
	        case tokentype.types._var:
	            kind = kind || this.value;
	            if (context && kind !== "var")
	                this.unexpected();
	            return this.parseVarStatement(node, kind);
	        case tokentype.types._while:
	            return this.parseWhileStatement(node);
	        case tokentype.types._with:
	            return this.parseWithStatement(node);
	        case tokentype.types.braceL:
	            return this.parseBlock(true, node);
	        case tokentype.types.semi:
	            return this.parseEmptyStatement(node);
	        case tokentype.types._export:
	        case tokentype.types._import:
	            if (this.options.ecmaVersion > 10 && starttype === tokentype.types._import) {
	                whitespace.skipWhiteSpace.lastIndex = this.pos;
	                var skip = whitespace.skipWhiteSpace.exec(this.input);
	                var next = this.pos + skip[0].length, nextCh = this.input.charCodeAt(next);
	                if (nextCh === 40 || nextCh === 46) // '(' or '.'
	                    return this.parseExpressionStatement(node, this.parseExpression());
	            }
	            if (!this.options.allowImportExportEverywhere) {
	                if (!topLevel)
	                    this.raise(this.start, "'import' and 'export' may only appear at the top level");
	                if (!this.inModule)
	                    this.raise(this.start, "'import' and 'export' may appear only with 'sourceType: module'");
	            }
	            return starttype === tokentype.types._import ? this.parseImport(node) : this.parseExport(node, exports);
	        // If the statement does not start with a statement keyword or a
	        // brace, it's an ExpressionStatement or LabeledStatement. We
	        // simply start parsing an expression, and afterwards, if the
	        // next token is a colon and the expression was a simple
	        // Identifier node, we switch to interpreting it as a label.
	        default:
	            if (this.isAsyncFunction()) {
	                if (context)
	                    this.unexpected();
	                this.next();
	                return this.parseFunctionStatement(node, true, !context);
	            }
	            var maybeName = this.value, expr = this.parseExpression();
	            if (starttype === tokentype.types.name && expr.type === "Identifier" && this.eat(tokentype.types.colon))
	                return this.parseLabeledStatement(node, maybeName, expr, context);
	            else
	                return this.parseExpressionStatement(node, expr);
	    }
	};
	pp$4.parseBreakContinueStatement = function (node, keyword) {
	    var isBreak = keyword === "break";
	    this.next();
	    if (this.eat(tokentype.types.semi) || this.insertSemicolon())
	        node.label = null;
	    else if (this.type !== tokentype.types.name)
	        this.unexpected();
	    else {
	        node.label = this.parseIdent();
	        this.semicolon();
	    }
	    // Verify that there is an actual destination to break or
	    // continue to.
	    var i = 0;
	    for (; i < this.labels.length; ++i) {
	        var lab = this.labels[i];
	        if (node.label == null || lab.name === node.label.name) {
	            if (lab.kind != null && (isBreak || lab.kind === "loop"))
	                break;
	            if (node.label && isBreak)
	                break;
	        }
	    }
	    if (i === this.labels.length)
	        this.raise(node.start, "Unsyntactic " + keyword);
	    return this.finishNode(node, isBreak ? "BreakStatement" : "ContinueStatement");
	};
	pp$4.parseDebuggerStatement = function (node) {
	    this.next();
	    this.semicolon();
	    return this.finishNode(node, "DebuggerStatement");
	};
	pp$4.parseDoStatement = function (node) {
	    this.next();
	    this.labels.push(loopLabel);
	    node.body = this.parseStatement("do");
	    this.labels.pop();
	    this.expect(tokentype.types._while);
	    node.test = this.parseParenExpression();
	    if (this.options.ecmaVersion >= 6)
	        this.eat(tokentype.types.semi);
	    else
	        this.semicolon();
	    return this.finishNode(node, "DoWhileStatement");
	};
	// Disambiguating between a `for` and a `for`/`in` or `for`/`of`
	// loop is non-trivial. Basically, we have to parse the init `var`
	// statement or expression, disallowing the `in` operator (see
	// the second parameter to `parseExpression`), and then check
	// whether the next token is `in` or `of`. When there is no init
	// part (semicolon immediately after the opening parenthesis), it
	// is a regular `for` loop.
	pp$4.parseForStatement = function (node) {
	    this.next();
	    var awaitAt = this.options.ecmaVersion >= 9 && (this.inAsync || !this.inFunction && this.options.allowAwaitOutsideFunction) && this.eatContextual("await") ? this.lastTokStart : -1;
	    this.labels.push(loopLabel);
	    this.enterScope(0);
	    this.expect(tokentype.types.parenL);
	    if (this.type === tokentype.types.semi) {
	        if (awaitAt > -1)
	            this.unexpected(awaitAt);
	        return this.parseFor(node, null);
	    }
	    var isLet = this.isLet();
	    if (this.type === tokentype.types._var || this.type === tokentype.types._const || isLet) {
	        var init_1 = this.startNode(), kind = isLet ? "let" : this.value;
	        this.next();
	        this.parseVar(init_1, true, kind);
	        this.finishNode(init_1, "VariableDeclaration");
	        if ((this.type === tokentype.types._in || this.options.ecmaVersion >= 6 && this.isContextual("of")) && init_1.declarations.length === 1) {
	            if (this.options.ecmaVersion >= 9) {
	                if (this.type === tokentype.types._in) {
	                    if (awaitAt > -1)
	                        this.unexpected(awaitAt);
	                }
	                else
	                    node.await = awaitAt > -1;
	            }
	            return this.parseForIn(node, init_1);
	        }
	        if (awaitAt > -1)
	            this.unexpected(awaitAt);
	        return this.parseFor(node, init_1);
	    }
	    var refDestructuringErrors = new parseutil.DestructuringErrors();
	    var init = this.parseExpression(true, refDestructuringErrors);
	    if (this.type === tokentype.types._in || this.options.ecmaVersion >= 6 && this.isContextual("of")) {
	        if (this.options.ecmaVersion >= 9) {
	            if (this.type === tokentype.types._in) {
	                if (awaitAt > -1)
	                    this.unexpected(awaitAt);
	            }
	            else
	                node.await = awaitAt > -1;
	        }
	        this.toAssignable(init, false, refDestructuringErrors);
	        this.checkLValPattern(init);
	        return this.parseForIn(node, init);
	    }
	    else {
	        this.checkExpressionErrors(refDestructuringErrors, true);
	    }
	    if (awaitAt > -1)
	        this.unexpected(awaitAt);
	    return this.parseFor(node, init);
	};
	pp$4.parseFunctionStatement = function (node, isAsync, declarationPosition) {
	    this.next();
	    return this.parseFunction(node, FUNC_STATEMENT | (declarationPosition ? 0 : FUNC_HANGING_STATEMENT), false, isAsync);
	};
	pp$4.parseIfStatement = function (node) {
	    this.next();
	    node.test = this.parseParenExpression();
	    // allow function declarations in branches, but only in non-strict mode
	    node.consequent = this.parseStatement("if");
	    node.alternate = this.eat(tokentype.types._else) ? this.parseStatement("if") : null;
	    return this.finishNode(node, "IfStatement");
	};
	pp$4.parseReturnStatement = function (node) {
	    if (!this.inFunction && !this.options.allowReturnOutsideFunction)
	        this.raise(this.start, "'return' outside of function");
	    this.next();
	    // In `return` (and `break`/`continue`), the keywords with
	    // optional arguments, we eagerly look for a semicolon or the
	    // possibility to insert one.
	    if (this.eat(tokentype.types.semi) || this.insertSemicolon())
	        node.argument = null;
	    else {
	        node.argument = this.parseExpression();
	        this.semicolon();
	    }
	    return this.finishNode(node, "ReturnStatement");
	};
	pp$4.parseSwitchStatement = function (node) {
	    this.next();
	    node.discriminant = this.parseParenExpression();
	    node.cases = [];
	    this.expect(tokentype.types.braceL);
	    this.labels.push(switchLabel);
	    this.enterScope(0);
	    // Statements under must be grouped (by label) in SwitchCase
	    // nodes. `cur` is used to keep the node that we are currently
	    // adding statements to.
	    var cur;
	    for (var sawDefault = false; this.type !== tokentype.types.braceR;) {
	        if (this.type === tokentype.types._case || this.type === tokentype.types._default) {
	            var isCase = this.type === tokentype.types._case;
	            if (cur)
	                this.finishNode(cur, "SwitchCase");
	            node.cases.push(cur = this.startNode());
	            cur.consequent = [];
	            this.next();
	            if (isCase) {
	                cur.test = this.parseExpression();
	            }
	            else {
	                if (sawDefault)
	                    this.raiseRecoverable(this.lastTokStart, "Multiple default clauses");
	                sawDefault = true;
	                cur.test = null;
	            }
	            this.expect(tokentype.types.colon);
	        }
	        else {
	            if (!cur)
	                this.unexpected();
	            cur.consequent.push(this.parseStatement(null));
	        }
	    }
	    this.exitScope();
	    if (cur)
	        this.finishNode(cur, "SwitchCase");
	    this.next(); // Closing brace
	    this.labels.pop();
	    return this.finishNode(node, "SwitchStatement");
	};
	pp$4.parseThrowStatement = function (node) {
	    this.next();
	    if (whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.start)))
	        this.raise(this.lastTokEnd, "Illegal newline after throw");
	    node.argument = this.parseExpression();
	    this.semicolon();
	    return this.finishNode(node, "ThrowStatement");
	};
	// Reused empty array added for node fields that are always empty.
	var empty$1 = [];
	pp$4.parseTryStatement = function (node) {
	    this.next();
	    node.block = this.parseBlock();
	    node.handler = null;
	    if (this.type === tokentype.types._catch) {
	        var clause = this.startNode();
	        this.next();
	        if (this.eat(tokentype.types.parenL)) {
	            clause.param = this.parseBindingAtom();
	            var simple = clause.param.type === "Identifier";
	            this.enterScope(simple ? scopeflags.SCOPE_SIMPLE_CATCH : 0);
	            this.checkLValPattern(clause.param, simple ? scopeflags.BIND_SIMPLE_CATCH : scopeflags.BIND_LEXICAL);
	            this.expect(tokentype.types.parenR);
	        }
	        else {
	            if (this.options.ecmaVersion < 10)
	                this.unexpected();
	            clause.param = null;
	            this.enterScope(0);
	        }
	        clause.body = this.parseBlock(false);
	        this.exitScope();
	        node.handler = this.finishNode(clause, "CatchClause");
	    }
	    node.finalizer = this.eat(tokentype.types._finally) ? this.parseBlock() : null;
	    if (!node.handler && !node.finalizer)
	        this.raise(node.start, "Missing catch or finally clause");
	    return this.finishNode(node, "TryStatement");
	};
	pp$4.parseVarStatement = function (node, kind) {
	    this.next();
	    this.parseVar(node, false, kind);
	    this.semicolon();
	    return this.finishNode(node, "VariableDeclaration");
	};
	pp$4.parseWhileStatement = function (node) {
	    this.next();
	    node.test = this.parseParenExpression();
	    this.labels.push(loopLabel);
	    node.body = this.parseStatement("while");
	    this.labels.pop();
	    return this.finishNode(node, "WhileStatement");
	};
	pp$4.parseWithStatement = function (node) {
	    if (this.strict)
	        this.raise(this.start, "'with' in strict mode");
	    this.next();
	    node.object = this.parseParenExpression();
	    node.body = this.parseStatement("with");
	    return this.finishNode(node, "WithStatement");
	};
	pp$4.parseEmptyStatement = function (node) {
	    this.next();
	    return this.finishNode(node, "EmptyStatement");
	};
	pp$4.parseLabeledStatement = function (node, maybeName, expr, context) {
	    for (var _iterator2 = this.labels, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
	        var _ref2;
	        if (_isArray2) {
	            if (_i2 >= _iterator2.length)
	                break;
	            _ref2 = _iterator2[_i2++];
	        }
	        else {
	            _i2 = _iterator2.next();
	            if (_i2.done)
	                break;
	            _ref2 = _i2.value;
	        }
	        var label = _ref2;
	        if (label.name === maybeName)
	            this.raise(expr.start, "Label '" + maybeName + "' is already declared");
	    }
	    var kind = this.type.isLoop ? "loop" : this.type === tokentype.types._switch ? "switch" : null;
	    for (var i = this.labels.length - 1; i >= 0; i--) {
	        var label = this.labels[i];
	        if (label.statementStart === node.start) {
	            // Update information about previous labels on this node
	            label.statementStart = this.start;
	            label.kind = kind;
	        }
	        else
	            break;
	    }
	    this.labels.push({ name: maybeName, kind: kind, statementStart: this.start });
	    node.body = this.parseStatement(context ? context.indexOf("label") === -1 ? context + "label" : context : "label");
	    this.labels.pop();
	    node.label = expr;
	    return this.finishNode(node, "LabeledStatement");
	};
	pp$4.parseExpressionStatement = function (node, expr) {
	    node.expression = expr;
	    this.semicolon();
	    return this.finishNode(node, "ExpressionStatement");
	};
	// Parse a semicolon-enclosed block of statements, handling `"use
	// strict"` declarations when `allowStrict` is true (used for
	// function bodies).
	pp$4.parseBlock = function (createNewLexicalScope, node, exitStrict) {
	    if (createNewLexicalScope === void 0) { createNewLexicalScope = true; }
	    if (node === void 0) { node = this.startNode(); }
	    node.body = [];
	    this.expect(tokentype.types.braceL);
	    if (createNewLexicalScope)
	        this.enterScope(0);
	    while (this.type !== tokentype.types.braceR) {
	        var stmt = this.parseStatement(null);
	        node.body.push(stmt);
	    }
	    if (exitStrict)
	        this.strict = false;
	    this.next();
	    if (createNewLexicalScope)
	        this.exitScope();
	    return this.finishNode(node, "BlockStatement");
	};
	// Parse a regular `for` loop. The disambiguation code in
	// `parseStatement` will already have parsed the init statement or
	// expression.
	pp$4.parseFor = function (node, init) {
	    node.init = init;
	    this.expect(tokentype.types.semi);
	    node.test = this.type === tokentype.types.semi ? null : this.parseExpression();
	    this.expect(tokentype.types.semi);
	    node.update = this.type === tokentype.types.parenR ? null : this.parseExpression();
	    this.expect(tokentype.types.parenR);
	    node.body = this.parseStatement("for");
	    this.exitScope();
	    this.labels.pop();
	    return this.finishNode(node, "ForStatement");
	};
	// Parse a `for`/`in` and `for`/`of` loop, which are almost
	// same from parser's perspective.
	pp$4.parseForIn = function (node, init) {
	    var isForIn = this.type === tokentype.types._in;
	    this.next();
	    if (init.type === "VariableDeclaration" && init.declarations[0].init != null && (!isForIn || this.options.ecmaVersion < 8 || this.strict || init.kind !== "var" || init.declarations[0].id.type !== "Identifier")) {
	        this.raise(init.start, (isForIn ? "for-in" : "for-of") + " loop variable declaration may not have an initializer");
	    }
	    node.left = init;
	    node.right = isForIn ? this.parseExpression() : this.parseMaybeAssign();
	    this.expect(tokentype.types.parenR);
	    node.body = this.parseStatement("for");
	    this.exitScope();
	    this.labels.pop();
	    return this.finishNode(node, isForIn ? "ForInStatement" : "ForOfStatement");
	};
	// Parse a list of variable declarations.
	pp$4.parseVar = function (node, isFor, kind) {
	    node.declarations = [];
	    node.kind = kind;
	    for (;;) {
	        var decl = this.startNode();
	        this.parseVarId(decl, kind);
	        if (this.eat(tokentype.types.eq)) {
	            decl.init = this.parseMaybeAssign(isFor);
	        }
	        else if (kind === "const" && !(this.type === tokentype.types._in || this.options.ecmaVersion >= 6 && this.isContextual("of"))) {
	            this.unexpected();
	        }
	        else if (decl.id.type !== "Identifier" && !(isFor && (this.type === tokentype.types._in || this.isContextual("of")))) {
	            this.raise(this.lastTokEnd, "Complex binding patterns require an initialization value");
	        }
	        else {
	            decl.init = null;
	        }
	        node.declarations.push(this.finishNode(decl, "VariableDeclarator"));
	        if (!this.eat(tokentype.types.comma))
	            break;
	    }
	    return node;
	};
	pp$4.parseVarId = function (decl, kind) {
	    decl.id = this.parseBindingAtom();
	    this.checkLValPattern(decl.id, kind === "var" ? scopeflags.BIND_VAR : scopeflags.BIND_LEXICAL, false);
	};
	var FUNC_STATEMENT = 1, FUNC_HANGING_STATEMENT = 2, FUNC_NULLABLE_ID = 4;
	// Parse a function declaration or literal (depending on the
	// `statement & FUNC_STATEMENT`).
	// Remove `allowExpressionBody` for 7.0.0, as it is only called with false
	pp$4.parseFunction = function (node, statement, allowExpressionBody, isAsync) {
	    this.initFunction(node);
	    if (this.options.ecmaVersion >= 9 || this.options.ecmaVersion >= 6 && !isAsync) {
	        if (this.type === tokentype.types.star && statement & FUNC_HANGING_STATEMENT)
	            this.unexpected();
	        node.generator = this.eat(tokentype.types.star);
	    }
	    if (this.options.ecmaVersion >= 8)
	        node.async = !!isAsync;
	    if (statement & FUNC_STATEMENT) {
	        node.id = statement & FUNC_NULLABLE_ID && this.type !== tokentype.types.name ? null : this.parseIdent();
	        if (node.id && !(statement & FUNC_HANGING_STATEMENT))
	            // If it is a regular function declaration in sloppy mode, then it is
	            // subject to Annex B semantics (BIND_FUNCTION). Otherwise, the binding
	            // mode depends on properties of the current scope (see
	            // treatFunctionsAsVar).
	            this.checkLValSimple(node.id, this.strict || node.generator || node.async ? this.treatFunctionsAsVar ? scopeflags.BIND_VAR : scopeflags.BIND_LEXICAL : scopeflags.BIND_FUNCTION);
	    }
	    var oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
	    this.yieldPos = 0;
	    this.awaitPos = 0;
	    this.awaitIdentPos = 0;
	    this.enterScope((0, scopeflags.functionFlags)(node.async, node.generator));
	    if (!(statement & FUNC_STATEMENT))
	        node.id = this.type === tokentype.types.name ? this.parseIdent() : null;
	    this.parseFunctionParams(node);
	    this.parseFunctionBody(node, allowExpressionBody, false);
	    this.yieldPos = oldYieldPos;
	    this.awaitPos = oldAwaitPos;
	    this.awaitIdentPos = oldAwaitIdentPos;
	    return this.finishNode(node, statement & FUNC_STATEMENT ? "FunctionDeclaration" : "FunctionExpression");
	};
	pp$4.parseFunctionParams = function (node) {
	    this.expect(tokentype.types.parenL);
	    node.params = this.parseBindingList(tokentype.types.parenR, false, this.options.ecmaVersion >= 8);
	    this.checkYieldAwaitInDefaultParams();
	};
	// Parse a class declaration or literal (depending on the
	// `isStatement` parameter).
	pp$4.parseClass = function (node, isStatement) {
	    this.next();
	    // ecma-262 14.6 Class Definitions
	    // A class definition is always strict mode code.
	    var oldStrict = this.strict;
	    this.strict = true;
	    this.parseClassId(node, isStatement);
	    this.parseClassSuper(node);
	    var classBody = this.startNode();
	    var hadConstructor = false;
	    classBody.body = [];
	    this.expect(tokentype.types.braceL);
	    while (this.type !== tokentype.types.braceR) {
	        var element = this.parseClassElement(node.superClass !== null);
	        if (element) {
	            classBody.body.push(element);
	            if (element.type === "MethodDefinition" && element.kind === "constructor") {
	                if (hadConstructor)
	                    this.raise(element.start, "Duplicate constructor in the same class");
	                hadConstructor = true;
	            }
	        }
	    }
	    this.strict = oldStrict;
	    this.next();
	    node.body = this.finishNode(classBody, "ClassBody");
	    return this.finishNode(node, isStatement ? "ClassDeclaration" : "ClassExpression");
	};
	pp$4.parseClassElement = function (constructorAllowsSuper) {
	    var _this = this;
	    if (this.eat(tokentype.types.semi))
	        return null;
	    var method = this.startNode();
	    var tryContextual = function (k, noLineBreak) {
	        if (noLineBreak === void 0) { noLineBreak = false; }
	        var start = _this.start, startLoc = _this.startLoc;
	        if (!_this.eatContextual(k))
	            return false;
	        if (_this.type !== tokentype.types.parenL && (!noLineBreak || !_this.canInsertSemicolon()))
	            return true;
	        if (method.key)
	            _this.unexpected();
	        method.computed = false;
	        method.key = _this.startNodeAt(start, startLoc);
	        method.key.name = k;
	        _this.finishNode(method.key, "Identifier");
	        return false;
	    };
	    method.kind = "method";
	    method.static = tryContextual("static");
	    var isGenerator = this.eat(tokentype.types.star);
	    var isAsync = false;
	    if (!isGenerator) {
	        if (this.options.ecmaVersion >= 8 && tryContextual("async", true)) {
	            isAsync = true;
	            isGenerator = this.options.ecmaVersion >= 9 && this.eat(tokentype.types.star);
	        }
	        else if (tryContextual("get")) {
	            method.kind = "get";
	        }
	        else if (tryContextual("set")) {
	            method.kind = "set";
	        }
	    }
	    if (!method.key)
	        this.parsePropertyName(method);
	    var key = method.key;
	    var allowsDirectSuper = false;
	    if (!method.computed && !method.static && (key.type === "Identifier" && key.name === "constructor" || key.type === "Literal" && key.value === "constructor")) {
	        if (method.kind !== "method")
	            this.raise(key.start, "Constructor can't have get/set modifier");
	        if (isGenerator)
	            this.raise(key.start, "Constructor can't be a generator");
	        if (isAsync)
	            this.raise(key.start, "Constructor can't be an async method");
	        method.kind = "constructor";
	        allowsDirectSuper = constructorAllowsSuper;
	    }
	    else if (method.static && key.type === "Identifier" && key.name === "prototype") {
	        this.raise(key.start, "Classes may not have a static property named prototype");
	    }
	    this.parseClassMethod(method, isGenerator, isAsync, allowsDirectSuper);
	    if (method.kind === "get" && method.value.params.length !== 0)
	        this.raiseRecoverable(method.value.start, "getter should have no params");
	    if (method.kind === "set" && method.value.params.length !== 1)
	        this.raiseRecoverable(method.value.start, "setter should have exactly one param");
	    if (method.kind === "set" && method.value.params[0].type === "RestElement")
	        this.raiseRecoverable(method.value.params[0].start, "Setter cannot use rest params");
	    return method;
	};
	pp$4.parseClassMethod = function (method, isGenerator, isAsync, allowsDirectSuper) {
	    method.value = this.parseMethod(isGenerator, isAsync, allowsDirectSuper);
	    return this.finishNode(method, "MethodDefinition");
	};
	pp$4.parseClassId = function (node, isStatement) {
	    if (this.type === tokentype.types.name) {
	        node.id = this.parseIdent();
	        if (isStatement)
	            this.checkLValSimple(node.id, scopeflags.BIND_LEXICAL, false);
	    }
	    else {
	        if (isStatement === true)
	            this.unexpected();
	        node.id = null;
	    }
	};
	pp$4.parseClassSuper = function (node) {
	    node.superClass = this.eat(tokentype.types._extends) ? this.parseExprSubscripts() : null;
	};
	// Parses module export declaration.
	pp$4.parseExport = function (node, exports) {
	    this.next();
	    // export * from '...'
	    if (this.eat(tokentype.types.star)) {
	        if (this.options.ecmaVersion >= 11) {
	            if (this.eatContextual("as")) {
	                node.exported = this.parseIdent(true);
	                this.checkExport(exports, node.exported.name, this.lastTokStart);
	            }
	            else {
	                node.exported = null;
	            }
	        }
	        this.expectContextual("from");
	        if (this.type !== tokentype.types.string)
	            this.unexpected();
	        node.source = this.parseExprAtom();
	        this.semicolon();
	        return this.finishNode(node, "ExportAllDeclaration");
	    }
	    if (this.eat(tokentype.types._default)) {
	        // export default ...
	        this.checkExport(exports, "default", this.lastTokStart);
	        var isAsync = void 0;
	        if (this.type === tokentype.types._function || (isAsync = this.isAsyncFunction())) {
	            var fNode = this.startNode();
	            this.next();
	            if (isAsync)
	                this.next();
	            node.declaration = this.parseFunction(fNode, FUNC_STATEMENT | FUNC_NULLABLE_ID, false, isAsync);
	        }
	        else if (this.type === tokentype.types._class) {
	            var cNode = this.startNode();
	            node.declaration = this.parseClass(cNode, "nullableID");
	        }
	        else {
	            node.declaration = this.parseMaybeAssign();
	            this.semicolon();
	        }
	        return this.finishNode(node, "ExportDefaultDeclaration");
	    }
	    // export var|const|let|function|class ...
	    if (this.shouldParseExportStatement()) {
	        node.declaration = this.parseStatement(null);
	        if (node.declaration.type === "VariableDeclaration")
	            this.checkVariableExport(exports, node.declaration.declarations);
	        else
	            this.checkExport(exports, node.declaration.id.name, node.declaration.id.start);
	        node.specifiers = [];
	        node.source = null;
	    }
	    else {
	        // export { x, y as z } [from '...']
	        node.declaration = null;
	        node.specifiers = this.parseExportSpecifiers(exports);
	        if (this.eatContextual("from")) {
	            if (this.type !== tokentype.types.string)
	                this.unexpected();
	            node.source = this.parseExprAtom();
	        }
	        else {
	            for (var _iterator3 = node.specifiers, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
	                var _ref3;
	                if (_isArray3) {
	                    if (_i3 >= _iterator3.length)
	                        break;
	                    _ref3 = _iterator3[_i3++];
	                }
	                else {
	                    _i3 = _iterator3.next();
	                    if (_i3.done)
	                        break;
	                    _ref3 = _i3.value;
	                }
	                var spec = _ref3;
	                // check for keywords used as local names
	                this.checkUnreserved(spec.local);
	                // check if export is defined
	                this.checkLocalExport(spec.local);
	            }
	            node.source = null;
	        }
	        this.semicolon();
	    }
	    return this.finishNode(node, "ExportNamedDeclaration");
	};
	pp$4.checkExport = function (exports, name, pos) {
	    if (!exports)
	        return;
	    if ((0, util.has)(exports, name))
	        this.raiseRecoverable(pos, "Duplicate export '" + name + "'");
	    exports[name] = true;
	};
	pp$4.checkPatternExport = function (exports, pat) {
	    var type = pat.type;
	    if (type === "Identifier")
	        this.checkExport(exports, pat.name, pat.start);
	    else if (type === "ObjectPattern") {
	        for (var _iterator4 = pat.properties, _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
	            var _ref4;
	            if (_isArray4) {
	                if (_i4 >= _iterator4.length)
	                    break;
	                _ref4 = _iterator4[_i4++];
	            }
	            else {
	                _i4 = _iterator4.next();
	                if (_i4.done)
	                    break;
	                _ref4 = _i4.value;
	            }
	            var prop = _ref4;
	            this.checkPatternExport(exports, prop);
	        }
	    }
	    else if (type === "ArrayPattern") {
	        for (var _iterator5 = pat.elements, _isArray5 = Array.isArray(_iterator5), _i5 = 0, _iterator5 = _isArray5 ? _iterator5 : _iterator5[Symbol.iterator]();;) {
	            var _ref5;
	            if (_isArray5) {
	                if (_i5 >= _iterator5.length)
	                    break;
	                _ref5 = _iterator5[_i5++];
	            }
	            else {
	                _i5 = _iterator5.next();
	                if (_i5.done)
	                    break;
	                _ref5 = _i5.value;
	            }
	            var elt = _ref5;
	            if (elt)
	                this.checkPatternExport(exports, elt);
	        }
	    }
	    else if (type === "Property")
	        this.checkPatternExport(exports, pat.value);
	    else if (type === "AssignmentPattern")
	        this.checkPatternExport(exports, pat.left);
	    else if (type === "RestElement")
	        this.checkPatternExport(exports, pat.argument);
	    else if (type === "ParenthesizedExpression")
	        this.checkPatternExport(exports, pat.expression);
	};
	pp$4.checkVariableExport = function (exports, decls) {
	    if (!exports)
	        return;
	    for (var _iterator6 = decls, _isArray6 = Array.isArray(_iterator6), _i6 = 0, _iterator6 = _isArray6 ? _iterator6 : _iterator6[Symbol.iterator]();;) {
	        var _ref6;
	        if (_isArray6) {
	            if (_i6 >= _iterator6.length)
	                break;
	            _ref6 = _iterator6[_i6++];
	        }
	        else {
	            _i6 = _iterator6.next();
	            if (_i6.done)
	                break;
	            _ref6 = _i6.value;
	        }
	        var decl = _ref6;
	        this.checkPatternExport(exports, decl.id);
	    }
	};
	pp$4.shouldParseExportStatement = function () {
	    return this.type.keyword === "var" || this.type.keyword === "const" || this.type.keyword === "class" || this.type.keyword === "function" || this.isLet() || this.isAsyncFunction();
	};
	// Parses a comma-separated list of module exports.
	pp$4.parseExportSpecifiers = function (exports) {
	    var nodes = [], first = true;
	    // export { x, y as z } [from '...']
	    this.expect(tokentype.types.braceL);
	    while (!this.eat(tokentype.types.braceR)) {
	        if (!first) {
	            this.expect(tokentype.types.comma);
	            if (this.afterTrailingComma(tokentype.types.braceR))
	                break;
	        }
	        else
	            first = false;
	        var node = this.startNode();
	        node.local = this.parseIdent(true);
	        node.exported = this.eatContextual("as") ? this.parseIdent(true) : node.local;
	        this.checkExport(exports, node.exported.name, node.exported.start);
	        nodes.push(this.finishNode(node, "ExportSpecifier"));
	    }
	    return nodes;
	};
	// Parses import declaration.
	pp$4.parseImport = function (node) {
	    this.next();
	    // import '...'
	    if (this.type === tokentype.types.string) {
	        node.specifiers = empty$1;
	        node.source = this.parseExprAtom();
	    }
	    else {
	        node.specifiers = this.parseImportSpecifiers();
	        this.expectContextual("from");
	        node.source = this.type === tokentype.types.string ? this.parseExprAtom() : this.unexpected();
	    }
	    this.semicolon();
	    return this.finishNode(node, "ImportDeclaration");
	};
	// Parses a comma-separated list of module imports.
	pp$4.parseImportSpecifiers = function () {
	    var nodes = [], first = true;
	    if (this.type === tokentype.types.name) {
	        // import defaultObj, { x, y as z } from '...'
	        var node = this.startNode();
	        node.local = this.parseIdent();
	        this.checkLValSimple(node.local, scopeflags.BIND_LEXICAL);
	        nodes.push(this.finishNode(node, "ImportDefaultSpecifier"));
	        if (!this.eat(tokentype.types.comma))
	            return nodes;
	    }
	    if (this.type === tokentype.types.star) {
	        var node = this.startNode();
	        this.next();
	        this.expectContextual("as");
	        node.local = this.parseIdent();
	        this.checkLValSimple(node.local, scopeflags.BIND_LEXICAL);
	        nodes.push(this.finishNode(node, "ImportNamespaceSpecifier"));
	        return nodes;
	    }
	    this.expect(tokentype.types.braceL);
	    while (!this.eat(tokentype.types.braceR)) {
	        if (!first) {
	            this.expect(tokentype.types.comma);
	            if (this.afterTrailingComma(tokentype.types.braceR))
	                break;
	        }
	        else
	            first = false;
	        var node = this.startNode();
	        node.imported = this.parseIdent(true);
	        if (this.eatContextual("as")) {
	            node.local = this.parseIdent();
	        }
	        else {
	            this.checkUnreserved(node.imported);
	            node.local = node.imported;
	        }
	        this.checkLValSimple(node.local, scopeflags.BIND_LEXICAL);
	        nodes.push(this.finishNode(node, "ImportSpecifier"));
	    }
	    return nodes;
	};
	// Set `ExpressionStatement#directive` property for directive prologues.
	pp$4.adaptDirectivePrologue = function (statements) {
	    for (var i = 0; i < statements.length && this.isDirectiveCandidate(statements[i]); ++i) {
	        statements[i].directive = statements[i].expression.raw.slice(1, -1);
	    }
	};
	pp$4.isDirectiveCandidate = function (statement) {
	    return statement.type === "ExpressionStatement" && statement.expression.type === "Literal" && typeof statement.expression.value === "string" && (
	    // Reject parenthesized strings.
	    this.input[statement.start] === "\"" || this.input[statement.start] === "'");
	};

	var pp$3 = state.Parser.prototype;
	// Convert existing expression atom to assignable pattern
	// if possible.
	pp$3.toAssignable = function (node, isBinding, refDestructuringErrors) {
	    if (this.options.ecmaVersion >= 6 && node) {
	        switch (node.type) {
	            case "Identifier":
	                if (this.inAsync && node.name === "await")
	                    this.raise(node.start, "Cannot use 'await' as identifier inside an async function");
	                break;
	            case "ObjectPattern":
	            case "ArrayPattern":
	            case "AssignmentPattern":
	            case "RestElement":
	                break;
	            case "ObjectExpression":
	                node.type = "ObjectPattern";
	                if (refDestructuringErrors)
	                    this.checkPatternErrors(refDestructuringErrors, true);
	                for (var _iterator = node.properties, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
	                    var _ref;
	                    if (_isArray) {
	                        if (_i >= _iterator.length)
	                            break;
	                        _ref = _iterator[_i++];
	                    }
	                    else {
	                        _i = _iterator.next();
	                        if (_i.done)
	                            break;
	                        _ref = _i.value;
	                    }
	                    var prop = _ref;
	                    this.toAssignable(prop, isBinding);
	                    // Early error:
	                    //   AssignmentRestProperty[Yield, Await] :
	                    //     `...` DestructuringAssignmentTarget[Yield, Await]
	                    //
	                    //   It is a Syntax Error if |DestructuringAssignmentTarget| is an |ArrayLiteral| or an |ObjectLiteral|.
	                    if (prop.type === "RestElement" && (prop.argument.type === "ArrayPattern" || prop.argument.type === "ObjectPattern")) {
	                        this.raise(prop.argument.start, "Unexpected token");
	                    }
	                }
	                break;
	            case "Property":
	                // AssignmentProperty has type === "Property"
	                if (node.kind !== "init")
	                    this.raise(node.key.start, "Object pattern can't contain getter or setter");
	                this.toAssignable(node.value, isBinding);
	                break;
	            case "ArrayExpression":
	                node.type = "ArrayPattern";
	                if (refDestructuringErrors)
	                    this.checkPatternErrors(refDestructuringErrors, true);
	                this.toAssignableList(node.elements, isBinding);
	                break;
	            case "SpreadElement":
	                node.type = "RestElement";
	                this.toAssignable(node.argument, isBinding);
	                if (node.argument.type === "AssignmentPattern")
	                    this.raise(node.argument.start, "Rest elements cannot have a default value");
	                break;
	            case "AssignmentExpression":
	                if (node.operator !== "=")
	                    this.raise(node.left.end, "Only '=' operator can be used for specifying default value.");
	                node.type = "AssignmentPattern";
	                delete node.operator;
	                this.toAssignable(node.left, isBinding);
	                break;
	            case "ParenthesizedExpression":
	                this.toAssignable(node.expression, isBinding, refDestructuringErrors);
	                break;
	            case "ChainExpression":
	                this.raiseRecoverable(node.start, "Optional chaining cannot appear in left-hand side");
	                break;
	            case "MemberExpression":
	                if (!isBinding)
	                    break;
	            default:
	                this.raise(node.start, "Assigning to rvalue");
	        }
	    }
	    else if (refDestructuringErrors)
	        this.checkPatternErrors(refDestructuringErrors, true);
	    return node;
	};
	// Convert list of expression atoms to binding list.
	pp$3.toAssignableList = function (exprList, isBinding) {
	    var end = exprList.length;
	    for (var i = 0; i < end; i++) {
	        var elt = exprList[i];
	        if (elt)
	            this.toAssignable(elt, isBinding);
	    }
	    if (end) {
	        var last = exprList[end - 1];
	        if (this.options.ecmaVersion === 6 && isBinding && last && last.type === "RestElement" && last.argument.type !== "Identifier")
	            this.unexpected(last.argument.start);
	    }
	    return exprList;
	};
	// Parses spread element.
	pp$3.parseSpread = function (refDestructuringErrors) {
	    var node = this.startNode();
	    this.next();
	    node.argument = this.parseMaybeAssign(false, refDestructuringErrors);
	    return this.finishNode(node, "SpreadElement");
	};
	pp$3.parseRestBinding = function () {
	    var node = this.startNode();
	    this.next();
	    // RestElement inside of a function parameter must be an identifier
	    if (this.options.ecmaVersion === 6 && this.type !== tokentype.types.name)
	        this.unexpected();
	    node.argument = this.parseBindingAtom();
	    return this.finishNode(node, "RestElement");
	};
	// Parses lvalue (assignable) atom.
	pp$3.parseBindingAtom = function () {
	    if (this.options.ecmaVersion >= 6) {
	        switch (this.type) {
	            case tokentype.types.bracketL:
	                var node = this.startNode();
	                this.next();
	                node.elements = this.parseBindingList(tokentype.types.bracketR, true, true);
	                return this.finishNode(node, "ArrayPattern");
	            case tokentype.types.braceL:
	                return this.parseObj(true);
	        }
	    }
	    return this.parseIdent();
	};
	pp$3.parseBindingList = function (close, allowEmpty, allowTrailingComma) {
	    var elts = [], first = true;
	    while (!this.eat(close)) {
	        if (first)
	            first = false;
	        else
	            this.expect(tokentype.types.comma);
	        if (allowEmpty && this.type === tokentype.types.comma) {
	            elts.push(null);
	        }
	        else if (allowTrailingComma && this.afterTrailingComma(close)) {
	            break;
	        }
	        else if (this.type === tokentype.types.ellipsis) {
	            var rest = this.parseRestBinding();
	            this.parseBindingListItem(rest);
	            elts.push(rest);
	            if (this.type === tokentype.types.comma)
	                this.raise(this.start, "Comma is not permitted after the rest element");
	            this.expect(close);
	            break;
	        }
	        else {
	            var elem = this.parseMaybeDefault(this.start, this.startLoc);
	            this.parseBindingListItem(elem);
	            elts.push(elem);
	        }
	    }
	    return elts;
	};
	pp$3.parseBindingListItem = function (param) {
	    return param;
	};
	// Parses assignment pattern around given atom if possible.
	pp$3.parseMaybeDefault = function (startPos, startLoc, left) {
	    left = left || this.parseBindingAtom();
	    if (this.options.ecmaVersion < 6 || !this.eat(tokentype.types.eq))
	        return left;
	    var node = this.startNodeAt(startPos, startLoc);
	    node.left = left;
	    node.right = this.parseMaybeAssign();
	    return this.finishNode(node, "AssignmentPattern");
	};
	// The following three functions all verify that a node is an lvalue —
	// something that can be bound, or assigned to. In order to do so, they perform
	// a variety of checks:
	//
	// - Check that none of the bound/assigned-to identifiers are reserved words.
	// - Record name declarations for bindings in the appropriate scope.
	// - Check duplicate argument names, if checkClashes is set.
	//
	// If a complex binding pattern is encountered (e.g., object and array
	// destructuring), the entire pattern is recursively checked.
	//
	// There are three versions of checkLVal*() appropriate for different
	// circumstances:
	//
	// - checkLValSimple() shall be used if the syntactic construct supports
	//   nothing other than identifiers and member expressions. Parenthesized
	//   expressions are also correctly handled. This is generally appropriate for
	//   constructs for which the spec says
	//
	//   > It is a Syntax Error if AssignmentTargetType of [the production] is not
	//   > simple.
	//
	//   It is also appropriate for checking if an identifier is valid and not
	//   defined elsewhere, like import declarations or function/class identifiers.
	//
	//   Examples where this is used include:
	//     a += …;
	//     import a from '…';
	//   where a is the node to be checked.
	//
	// - checkLValPattern() shall be used if the syntactic construct supports
	//   anything checkLValSimple() supports, as well as object and array
	//   destructuring patterns. This is generally appropriate for constructs for
	//   which the spec says
	//
	//   > It is a Syntax Error if [the production] is neither an ObjectLiteral nor
	//   > an ArrayLiteral and AssignmentTargetType of [the production] is not
	//   > simple.
	//
	//   Examples where this is used include:
	//     (a = …);
	//     const a = …;
	//     try { … } catch (a) { … }
	//   where a is the node to be checked.
	//
	// - checkLValInnerPattern() shall be used if the syntactic construct supports
	//   anything checkLValPattern() supports, as well as default assignment
	//   patterns, rest elements, and other constructs that may appear within an
	//   object or array destructuring pattern.
	//
	//   As a special case, function parameters also use checkLValInnerPattern(),
	//   as they also support defaults and rest constructs.
	//
	// These functions deliberately support both assignment and binding constructs,
	// as the logic for both is exceedingly similar. If the node is the target of
	// an assignment, then bindingType should be set to BIND_NONE. Otherwise, it
	// should be set to the appropriate BIND_* constant, like BIND_VAR or
	// BIND_LEXICAL.
	//
	// If the function is called with a non-BIND_NONE bindingType, then
	// additionally a checkClashes object may be specified to allow checking for
	// duplicate argument names. checkClashes is ignored if the provided construct
	// is an assignment (i.e., bindingType is BIND_NONE).
	pp$3.checkLValSimple = function (expr, bindingType, checkClashes) {
	    if (bindingType === void 0) { bindingType = scopeflags.BIND_NONE; }
	    var isBind = bindingType !== scopeflags.BIND_NONE;
	    switch (expr.type) {
	        case "Identifier":
	            if (this.strict && this.reservedWordsStrictBind.test(expr.name))
	                this.raiseRecoverable(expr.start, (isBind ? "Binding " : "Assigning to ") + expr.name + " in strict mode");
	            if (isBind) {
	                if (bindingType === scopeflags.BIND_LEXICAL && expr.name === "let")
	                    this.raiseRecoverable(expr.start, "let is disallowed as a lexically bound name");
	                if (checkClashes) {
	                    if ((0, util.has)(checkClashes, expr.name))
	                        this.raiseRecoverable(expr.start, "Argument name clash");
	                    checkClashes[expr.name] = true;
	                }
	                if (bindingType !== scopeflags.BIND_OUTSIDE)
	                    this.declareName(expr.name, bindingType, expr.start);
	            }
	            break;
	        case "ChainExpression":
	            this.raiseRecoverable(expr.start, "Optional chaining cannot appear in left-hand side");
	            break;
	        case "MemberExpression":
	            if (isBind)
	                this.raiseRecoverable(expr.start, "Binding member expression");
	            break;
	        case "ParenthesizedExpression":
	            if (isBind)
	                this.raiseRecoverable(expr.start, "Binding parenthesized expression");
	            return this.checkLValSimple(expr.expression, bindingType, checkClashes);
	        default:
	            this.raise(expr.start, (isBind ? "Binding" : "Assigning to") + " rvalue");
	    }
	};
	pp$3.checkLValPattern = function (expr, bindingType, checkClashes) {
	    if (bindingType === void 0) { bindingType = scopeflags.BIND_NONE; }
	    switch (expr.type) {
	        case "ObjectPattern":
	            for (var _iterator2 = expr.properties, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
	                var _ref2;
	                if (_isArray2) {
	                    if (_i2 >= _iterator2.length)
	                        break;
	                    _ref2 = _iterator2[_i2++];
	                }
	                else {
	                    _i2 = _iterator2.next();
	                    if (_i2.done)
	                        break;
	                    _ref2 = _i2.value;
	                }
	                var prop = _ref2;
	                this.checkLValInnerPattern(prop, bindingType, checkClashes);
	            }
	            break;
	        case "ArrayPattern":
	            for (var _iterator3 = expr.elements, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
	                var _ref3;
	                if (_isArray3) {
	                    if (_i3 >= _iterator3.length)
	                        break;
	                    _ref3 = _iterator3[_i3++];
	                }
	                else {
	                    _i3 = _iterator3.next();
	                    if (_i3.done)
	                        break;
	                    _ref3 = _i3.value;
	                }
	                var elem = _ref3;
	                if (elem)
	                    this.checkLValInnerPattern(elem, bindingType, checkClashes);
	            }
	            break;
	        default:
	            this.checkLValSimple(expr, bindingType, checkClashes);
	    }
	};
	pp$3.checkLValInnerPattern = function (expr, bindingType, checkClashes) {
	    if (bindingType === void 0) { bindingType = scopeflags.BIND_NONE; }
	    switch (expr.type) {
	        case "Property":
	            // AssignmentProperty has type === "Property"
	            this.checkLValInnerPattern(expr.value, bindingType, checkClashes);
	            break;
	        case "AssignmentPattern":
	            this.checkLValPattern(expr.left, bindingType, checkClashes);
	            break;
	        case "RestElement":
	            this.checkLValPattern(expr.argument, bindingType, checkClashes);
	            break;
	        default:
	            this.checkLValPattern(expr, bindingType, checkClashes);
	    }
	};

	var pp$2 = state.Parser.prototype;
	// Check if property name clashes with already added.
	// Object/class getters and setters are not allowed to clash —
	// either with each other or with an init property — and in
	// strict mode, init properties are also not allowed to be repeated.
	// A recursive descent parser operates by defining functions for all
	// syntactic elements, and recursively calling those, each function
	// advancing the input stream and returning an AST node. Precedence
	// of constructs (for example, the fact that `!x[1]` means `!(x[1])`
	// instead of `(!x)[1]` is handled by the fact that the parser
	// function that parses unary prefix operators is called first, and
	// in turn calls the function that parses `[]` subscripts — that
	// way, it'll receive the node for `x[1]` already parsed, and wraps
	// *that* in the unary operator node.
	//
	// Acorn uses an [operator precedence parser][opp] to handle binary
	// operator precedence, because it is much more compact than using
	// the technique outlined above, which uses different, nesting
	// functions to specify precedence, for all of the ten binary
	// precedence levels that JavaScript defines.
	//
	// [opp]: http://en.wikipedia.org/wiki/Operator-precedence_parser
	pp$2.checkPropClash = function (prop, propHash, refDestructuringErrors) {
	    if (this.options.ecmaVersion >= 9 && prop.type === "SpreadElement")
	        return;
	    if (this.options.ecmaVersion >= 6 && (prop.computed || prop.method || prop.shorthand))
	        return;
	    var key = prop.key, name;
	    switch (key.type) {
	        case "Identifier":
	            name = key.name;
	            break;
	        case "Literal":
	            name = String(key.value);
	            break;
	        default:
	            return;
	    }
	    var kind = prop.kind;
	    if (this.options.ecmaVersion >= 6) {
	        if (name === "__proto__" && kind === "init") {
	            if (propHash.proto) {
	                if (refDestructuringErrors) {
	                    if (refDestructuringErrors.doubleProto < 0)
	                        refDestructuringErrors.doubleProto = key.start;
	                    // Backwards-compat kludge. Can be removed in version 6.0
	                }
	                else
	                    this.raiseRecoverable(key.start, "Redefinition of __proto__ property");
	            }
	            propHash.proto = true;
	        }
	        return;
	    }
	    name = "$" + name;
	    var other = propHash[name];
	    if (other) {
	        var redefinition = void 0;
	        if (kind === "init") {
	            redefinition = this.strict && other.init || other.get || other.set;
	        }
	        else {
	            redefinition = other.init || other[kind];
	        }
	        if (redefinition)
	            this.raiseRecoverable(key.start, "Redefinition of property");
	    }
	    else {
	        other = propHash[name] = {
	            init: false,
	            get: false,
	            set: false
	        };
	    }
	    other[kind] = true;
	};
	// ### Expression parsing
	// These nest, from the most general expression type at the top to
	// 'atomic', nondivisible expression types at the bottom. Most of
	// the functions will simply let the function(s) below them parse,
	// and, *if* the syntactic construct they handle is present, wrap
	// the AST node that the inner parser gave them in another node.
	// Parse a full expression. The optional arguments are used to
	// forbid the `in` operator (in for loops initalization expressions)
	// and provide reference for storing '=' operator inside shorthand
	// property assignment in contexts where both object expression
	// and object pattern might appear (so it's possible to raise
	// delayed syntax error at correct position).
	pp$2.parseExpression = function (noIn, refDestructuringErrors) {
	    var startPos = this.start, startLoc = this.startLoc;
	    var expr = this.parseMaybeAssign(noIn, refDestructuringErrors);
	    if (this.type === tokentype.types.comma) {
	        var node = this.startNodeAt(startPos, startLoc);
	        node.expressions = [expr];
	        while (this.eat(tokentype.types.comma))
	            node.expressions.push(this.parseMaybeAssign(noIn, refDestructuringErrors));
	        return this.finishNode(node, "SequenceExpression");
	    }
	    return expr;
	};
	// Parse an assignment expression. This includes applications of
	// operators like `+=`.
	pp$2.parseMaybeAssign = function (noIn, refDestructuringErrors, afterLeftParse) {
	    if (this.isContextual("yield")) {
	        if (this.inGenerator)
	            return this.parseYield(noIn);
	        // The tokenizer will assume an expression is allowed after
	        // `yield`, but this isn't that kind of yield
	        else
	            this.exprAllowed = false;
	    }
	    var ownDestructuringErrors = false, oldParenAssign = -1, oldTrailingComma = -1;
	    if (refDestructuringErrors) {
	        oldParenAssign = refDestructuringErrors.parenthesizedAssign;
	        oldTrailingComma = refDestructuringErrors.trailingComma;
	        refDestructuringErrors.parenthesizedAssign = refDestructuringErrors.trailingComma = -1;
	    }
	    else {
	        refDestructuringErrors = new parseutil.DestructuringErrors();
	        ownDestructuringErrors = true;
	    }
	    var startPos = this.start, startLoc = this.startLoc;
	    if (this.type === tokentype.types.parenL || this.type === tokentype.types.name)
	        this.potentialArrowAt = this.start;
	    var left = this.parseMaybeConditional(noIn, refDestructuringErrors);
	    if (afterLeftParse)
	        left = afterLeftParse.call(this, left, startPos, startLoc);
	    if (this.type.isAssign) {
	        var node = this.startNodeAt(startPos, startLoc);
	        node.operator = this.value;
	        if (this.type === tokentype.types.eq)
	            left = this.toAssignable(left, false, refDestructuringErrors);
	        if (!ownDestructuringErrors) {
	            refDestructuringErrors.parenthesizedAssign = refDestructuringErrors.trailingComma = refDestructuringErrors.doubleProto = -1;
	        }
	        if (refDestructuringErrors.shorthandAssign >= left.start)
	            refDestructuringErrors.shorthandAssign = -1; // reset because shorthand default was used correctly
	        if (this.type === tokentype.types.eq)
	            this.checkLValPattern(left);
	        else
	            this.checkLValSimple(left);
	        node.left = left;
	        this.next();
	        node.right = this.parseMaybeAssign(noIn);
	        return this.finishNode(node, "AssignmentExpression");
	    }
	    else {
	        if (ownDestructuringErrors)
	            this.checkExpressionErrors(refDestructuringErrors, true);
	    }
	    if (oldParenAssign > -1)
	        refDestructuringErrors.parenthesizedAssign = oldParenAssign;
	    if (oldTrailingComma > -1)
	        refDestructuringErrors.trailingComma = oldTrailingComma;
	    return left;
	};
	// Parse a ternary conditional (`?:`) operator.
	pp$2.parseMaybeConditional = function (noIn, refDestructuringErrors) {
	    var startPos = this.start, startLoc = this.startLoc;
	    var expr = this.parseExprOps(noIn, refDestructuringErrors);
	    if (this.checkExpressionErrors(refDestructuringErrors))
	        return expr;
	    if (this.eat(tokentype.types.question)) {
	        var node = this.startNodeAt(startPos, startLoc);
	        node.test = expr;
	        node.consequent = this.parseMaybeAssign();
	        this.expect(tokentype.types.colon);
	        node.alternate = this.parseMaybeAssign(noIn);
	        return this.finishNode(node, "ConditionalExpression");
	    }
	    return expr;
	};
	// Start the precedence parser.
	pp$2.parseExprOps = function (noIn, refDestructuringErrors) {
	    var startPos = this.start, startLoc = this.startLoc;
	    var expr = this.parseMaybeUnary(refDestructuringErrors, false);
	    if (this.checkExpressionErrors(refDestructuringErrors))
	        return expr;
	    return expr.start === startPos && expr.type === "ArrowFunctionExpression" ? expr : this.parseExprOp(expr, startPos, startLoc, -1, noIn);
	};
	// Parse binary operators with the operator precedence parsing
	// algorithm. `left` is the left-hand side of the operator.
	// `minPrec` provides context that allows the function to stop and
	// defer further parser to one of its callers when it encounters an
	// operator that has a lower precedence than the set it is parsing.
	pp$2.parseExprOp = function (left, leftStartPos, leftStartLoc, minPrec, noIn) {
	    var prec = this.type.binop;
	    if (prec != null && (!noIn || this.type !== tokentype.types._in)) {
	        if (prec > minPrec) {
	            var logical = this.type === tokentype.types.logicalOR || this.type === tokentype.types.logicalAND;
	            var coalesce = this.type === tokentype.types.coalesce;
	            if (coalesce) {
	                // Handle the precedence of `tt.coalesce` as equal to the range of logical expressions.
	                // In other words, `node.right` shouldn't contain logical expressions in order to check the mixed error.
	                prec = tokentype.types.logicalAND.binop;
	            }
	            var op = this.value;
	            this.next();
	            var startPos = this.start, startLoc = this.startLoc;
	            var right = this.parseExprOp(this.parseMaybeUnary(null, false), startPos, startLoc, prec, noIn);
	            var node = this.buildBinary(leftStartPos, leftStartLoc, left, right, op, logical || coalesce);
	            if (logical && this.type === tokentype.types.coalesce || coalesce && (this.type === tokentype.types.logicalOR || this.type === tokentype.types.logicalAND)) {
	                this.raiseRecoverable(this.start, "Logical expressions and coalesce expressions cannot be mixed. Wrap either by parentheses");
	            }
	            return this.parseExprOp(node, leftStartPos, leftStartLoc, minPrec, noIn);
	        }
	    }
	    return left;
	};
	pp$2.buildBinary = function (startPos, startLoc, left, right, op, logical) {
	    var node = this.startNodeAt(startPos, startLoc);
	    node.left = left;
	    node.operator = op;
	    node.right = right;
	    return this.finishNode(node, logical ? "LogicalExpression" : "BinaryExpression");
	};
	// Parse unary operators, both prefix and postfix.
	pp$2.parseMaybeUnary = function (refDestructuringErrors, sawUnary) {
	    var startPos = this.start, startLoc = this.startLoc, expr;
	    if (this.isContextual("await") && (this.inAsync || !this.inFunction && this.options.allowAwaitOutsideFunction)) {
	        expr = this.parseAwait();
	        sawUnary = true;
	    }
	    else if (this.type.prefix) {
	        var node = this.startNode(), update = this.type === tokentype.types.incDec;
	        node.operator = this.value;
	        node.prefix = true;
	        this.next();
	        node.argument = this.parseMaybeUnary(null, true);
	        this.checkExpressionErrors(refDestructuringErrors, true);
	        if (update)
	            this.checkLValSimple(node.argument);
	        else if (this.strict && node.operator === "delete" && node.argument.type === "Identifier")
	            this.raiseRecoverable(node.start, "Deleting local variable in strict mode");
	        else
	            sawUnary = true;
	        expr = this.finishNode(node, update ? "UpdateExpression" : "UnaryExpression");
	    }
	    else {
	        expr = this.parseExprSubscripts(refDestructuringErrors);
	        if (this.checkExpressionErrors(refDestructuringErrors))
	            return expr;
	        while (this.type.postfix && !this.canInsertSemicolon()) {
	            var node = this.startNodeAt(startPos, startLoc);
	            node.operator = this.value;
	            node.prefix = false;
	            node.argument = expr;
	            this.checkLValSimple(expr);
	            this.next();
	            expr = this.finishNode(node, "UpdateExpression");
	        }
	    }
	    if (!sawUnary && this.eat(tokentype.types.starstar))
	        return this.buildBinary(startPos, startLoc, expr, this.parseMaybeUnary(null, false), "**", false);
	    else
	        return expr;
	};
	// Parse call, dot, and `[]`-subscript expressions.
	pp$2.parseExprSubscripts = function (refDestructuringErrors) {
	    var startPos = this.start, startLoc = this.startLoc;
	    var expr = this.parseExprAtom(refDestructuringErrors);
	    if (expr.type === "ArrowFunctionExpression" && this.input.slice(this.lastTokStart, this.lastTokEnd) !== ")")
	        return expr;
	    var result = this.parseSubscripts(expr, startPos, startLoc);
	    if (refDestructuringErrors && result.type === "MemberExpression") {
	        if (refDestructuringErrors.parenthesizedAssign >= result.start)
	            refDestructuringErrors.parenthesizedAssign = -1;
	        if (refDestructuringErrors.parenthesizedBind >= result.start)
	            refDestructuringErrors.parenthesizedBind = -1;
	        if (refDestructuringErrors.trailingComma >= result.start)
	            refDestructuringErrors.trailingComma = -1;
	    }
	    return result;
	};
	pp$2.parseSubscripts = function (base, startPos, startLoc, noCalls) {
	    var maybeAsyncArrow = this.options.ecmaVersion >= 8 && base.type === "Identifier" && base.name === "async" && this.lastTokEnd === base.end && !this.canInsertSemicolon() && base.end - base.start === 5 && this.potentialArrowAt === base.start;
	    var optionalChained = false;
	    while (true) {
	        var element = this.parseSubscript(base, startPos, startLoc, noCalls, maybeAsyncArrow, optionalChained);
	        if (element.optional)
	            optionalChained = true;
	        if (element === base || element.type === "ArrowFunctionExpression") {
	            if (optionalChained) {
	                var chainNode = this.startNodeAt(startPos, startLoc);
	                chainNode.expression = element;
	                element = this.finishNode(chainNode, "ChainExpression");
	            }
	            return element;
	        }
	        base = element;
	    }
	};
	pp$2.parseSubscript = function (base, startPos, startLoc, noCalls, maybeAsyncArrow, optionalChained) {
	    var optionalSupported = this.options.ecmaVersion >= 11;
	    var optional = optionalSupported && this.eat(tokentype.types.questionDot);
	    if (noCalls && optional)
	        this.raise(this.lastTokStart, "Optional chaining cannot appear in the callee of new expressions");
	    var computed = this.eat(tokentype.types.bracketL);
	    if (computed || optional && this.type !== tokentype.types.parenL && this.type !== tokentype.types.backQuote || this.eat(tokentype.types.dot)) {
	        var node = this.startNodeAt(startPos, startLoc);
	        node.object = base;
	        node.property = computed ? this.parseExpression() : this.parseIdent(this.options.allowReserved !== "never");
	        node.computed = !!computed;
	        if (computed)
	            this.expect(tokentype.types.bracketR);
	        if (optionalSupported) {
	            node.optional = optional;
	        }
	        base = this.finishNode(node, "MemberExpression");
	    }
	    else if (!noCalls && this.eat(tokentype.types.parenL)) {
	        var refDestructuringErrors = new parseutil.DestructuringErrors(), oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
	        this.yieldPos = 0;
	        this.awaitPos = 0;
	        this.awaitIdentPos = 0;
	        var exprList = this.parseExprList(tokentype.types.parenR, this.options.ecmaVersion >= 8, false, refDestructuringErrors);
	        if (maybeAsyncArrow && !optional && !this.canInsertSemicolon() && this.eat(tokentype.types.arrow)) {
	            this.checkPatternErrors(refDestructuringErrors, false);
	            this.checkYieldAwaitInDefaultParams();
	            if (this.awaitIdentPos > 0)
	                this.raise(this.awaitIdentPos, "Cannot use 'await' as identifier inside an async function");
	            this.yieldPos = oldYieldPos;
	            this.awaitPos = oldAwaitPos;
	            this.awaitIdentPos = oldAwaitIdentPos;
	            return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), exprList, true);
	        }
	        this.checkExpressionErrors(refDestructuringErrors, true);
	        this.yieldPos = oldYieldPos || this.yieldPos;
	        this.awaitPos = oldAwaitPos || this.awaitPos;
	        this.awaitIdentPos = oldAwaitIdentPos || this.awaitIdentPos;
	        var node = this.startNodeAt(startPos, startLoc);
	        node.callee = base;
	        node.arguments = exprList;
	        if (optionalSupported) {
	            node.optional = optional;
	        }
	        base = this.finishNode(node, "CallExpression");
	    }
	    else if (this.type === tokentype.types.backQuote) {
	        if (optional || optionalChained) {
	            this.raise(this.start, "Optional chaining cannot appear in the tag of tagged template expressions");
	        }
	        var node = this.startNodeAt(startPos, startLoc);
	        node.tag = base;
	        node.quasi = this.parseTemplate({ isTagged: true });
	        base = this.finishNode(node, "TaggedTemplateExpression");
	    }
	    return base;
	};
	// Parse an atomic expression — either a single token that is an
	// expression, an expression started by a keyword like `function` or
	// `new`, or an expression wrapped in punctuation like `()`, `[]`,
	// or `{}`.
	pp$2.parseExprAtom = function (refDestructuringErrors) {
	    // If a division operator appears in an expression position, the
	    // tokenizer got confused, and we force it to read a regexp instead.
	    if (this.type === tokentype.types.slash)
	        this.readRegexp();
	    var node, canBeArrow = this.potentialArrowAt === this.start;
	    switch (this.type) {
	        case tokentype.types._super:
	            if (!this.allowSuper)
	                this.raise(this.start, "'super' keyword outside a method");
	            node = this.startNode();
	            this.next();
	            if (this.type === tokentype.types.parenL && !this.allowDirectSuper)
	                this.raise(node.start, "super() call outside constructor of a subclass");
	            // The `super` keyword can appear at below:
	            // SuperProperty:
	            //     super [ Expression ]
	            //     super . IdentifierName
	            // SuperCall:
	            //     super ( Arguments )
	            if (this.type !== tokentype.types.dot && this.type !== tokentype.types.bracketL && this.type !== tokentype.types.parenL)
	                this.unexpected();
	            return this.finishNode(node, "Super");
	        case tokentype.types._this:
	            node = this.startNode();
	            this.next();
	            return this.finishNode(node, "ThisExpression");
	        case tokentype.types.name:
	            var startPos = this.start, startLoc = this.startLoc, containsEsc = this.containsEsc;
	            var id = this.parseIdent(false);
	            if (this.options.ecmaVersion >= 8 && !containsEsc && id.name === "async" && !this.canInsertSemicolon() && this.eat(tokentype.types._function))
	                return this.parseFunction(this.startNodeAt(startPos, startLoc), 0, false, true);
	            if (canBeArrow && !this.canInsertSemicolon()) {
	                if (this.eat(tokentype.types.arrow))
	                    return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), [id], false);
	                if (this.options.ecmaVersion >= 8 && id.name === "async" && this.type === tokentype.types.name && !containsEsc) {
	                    id = this.parseIdent(false);
	                    if (this.canInsertSemicolon() || !this.eat(tokentype.types.arrow))
	                        this.unexpected();
	                    return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), [id], true);
	                }
	            }
	            return id;
	        case tokentype.types.regexp:
	            var value = this.value;
	            node = this.parseLiteral(value.value);
	            node.regex = { pattern: value.pattern, flags: value.flags };
	            return node;
	        case tokentype.types.num:
	        case tokentype.types.string:
	            return this.parseLiteral(this.value);
	        case tokentype.types._null:
	        case tokentype.types._true:
	        case tokentype.types._false:
	            node = this.startNode();
	            node.value = this.type === tokentype.types._null ? null : this.type === tokentype.types._true;
	            node.raw = this.type.keyword;
	            this.next();
	            return this.finishNode(node, "Literal");
	        case tokentype.types.parenL:
	            var start = this.start, expr = this.parseParenAndDistinguishExpression(canBeArrow);
	            if (refDestructuringErrors) {
	                if (refDestructuringErrors.parenthesizedAssign < 0 && !this.isSimpleAssignTarget(expr))
	                    refDestructuringErrors.parenthesizedAssign = start;
	                if (refDestructuringErrors.parenthesizedBind < 0)
	                    refDestructuringErrors.parenthesizedBind = start;
	            }
	            return expr;
	        case tokentype.types.bracketL:
	            node = this.startNode();
	            this.next();
	            node.elements = this.parseExprList(tokentype.types.bracketR, true, true, refDestructuringErrors);
	            return this.finishNode(node, "ArrayExpression");
	        case tokentype.types.braceL:
	            return this.parseObj(false, refDestructuringErrors);
	        case tokentype.types._function:
	            node = this.startNode();
	            this.next();
	            return this.parseFunction(node, 0);
	        case tokentype.types._class:
	            return this.parseClass(this.startNode(), false);
	        case tokentype.types._new:
	            return this.parseNew();
	        case tokentype.types.backQuote:
	            return this.parseTemplate();
	        case tokentype.types._import:
	            if (this.options.ecmaVersion >= 11) {
	                return this.parseExprImport();
	            }
	            else {
	                return this.unexpected();
	            }
	        default:
	            this.unexpected();
	    }
	};
	pp$2.parseExprImport = function () {
	    var node = this.startNode();
	    // Consume `import` as an identifier for `import.meta`.
	    // Because `this.parseIdent(true)` doesn't check escape sequences, it needs the check of `this.containsEsc`.
	    if (this.containsEsc)
	        this.raiseRecoverable(this.start, "Escape sequence in keyword import");
	    var meta = this.parseIdent(true);
	    switch (this.type) {
	        case tokentype.types.parenL:
	            return this.parseDynamicImport(node);
	        case tokentype.types.dot:
	            node.meta = meta;
	            return this.parseImportMeta(node);
	        default:
	            this.unexpected();
	    }
	};
	pp$2.parseDynamicImport = function (node) {
	    this.next(); // skip `(`
	    // Parse node.source.
	    node.source = this.parseMaybeAssign();
	    // Verify ending.
	    if (!this.eat(tokentype.types.parenR)) {
	        var errorPos = this.start;
	        if (this.eat(tokentype.types.comma) && this.eat(tokentype.types.parenR)) {
	            this.raiseRecoverable(errorPos, "Trailing comma is not allowed in import()");
	        }
	        else {
	            this.unexpected(errorPos);
	        }
	    }
	    return this.finishNode(node, "ImportExpression");
	};
	pp$2.parseImportMeta = function (node) {
	    this.next(); // skip `.`
	    var containsEsc = this.containsEsc;
	    node.property = this.parseIdent(true);
	    if (node.property.name !== "meta")
	        this.raiseRecoverable(node.property.start, "The only valid meta property for import is 'import.meta'");
	    if (containsEsc)
	        this.raiseRecoverable(node.start, "'import.meta' must not contain escaped characters");
	    if (this.options.sourceType !== "module" && !this.options.allowImportExportEverywhere)
	        this.raiseRecoverable(node.start, "Cannot use 'import.meta' outside a module");
	    return this.finishNode(node, "MetaProperty");
	};
	pp$2.parseLiteral = function (value) {
	    var node = this.startNode();
	    node.value = value;
	    node.raw = this.input.slice(this.start, this.end);
	    if (node.raw.charCodeAt(node.raw.length - 1) === 110)
	        node.bigint = node.raw.slice(0, -1).replace(/_/g, "");
	    this.next();
	    return this.finishNode(node, "Literal");
	};
	pp$2.parseParenExpression = function () {
	    this.expect(tokentype.types.parenL);
	    var val = this.parseExpression();
	    this.expect(tokentype.types.parenR);
	    return val;
	};
	pp$2.parseParenAndDistinguishExpression = function (canBeArrow) {
	    var startPos = this.start, startLoc = this.startLoc, val, allowTrailingComma = this.options.ecmaVersion >= 8;
	    if (this.options.ecmaVersion >= 6) {
	        this.next();
	        var innerStartPos = this.start, innerStartLoc = this.startLoc;
	        var exprList = [], first = true, lastIsComma = false;
	        var refDestructuringErrors = new parseutil.DestructuringErrors(), oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, spreadStart = void 0;
	        this.yieldPos = 0;
	        this.awaitPos = 0;
	        // Do not save awaitIdentPos to allow checking awaits nested in parameters
	        while (this.type !== tokentype.types.parenR) {
	            first ? first = false : this.expect(tokentype.types.comma);
	            if (allowTrailingComma && this.afterTrailingComma(tokentype.types.parenR, true)) {
	                lastIsComma = true;
	                break;
	            }
	            else if (this.type === tokentype.types.ellipsis) {
	                spreadStart = this.start;
	                exprList.push(this.parseParenItem(this.parseRestBinding()));
	                if (this.type === tokentype.types.comma)
	                    this.raise(this.start, "Comma is not permitted after the rest element");
	                break;
	            }
	            else {
	                exprList.push(this.parseMaybeAssign(false, refDestructuringErrors, this.parseParenItem));
	            }
	        }
	        var innerEndPos = this.start, innerEndLoc = this.startLoc;
	        this.expect(tokentype.types.parenR);
	        if (canBeArrow && !this.canInsertSemicolon() && this.eat(tokentype.types.arrow)) {
	            this.checkPatternErrors(refDestructuringErrors, false);
	            this.checkYieldAwaitInDefaultParams();
	            this.yieldPos = oldYieldPos;
	            this.awaitPos = oldAwaitPos;
	            return this.parseParenArrowList(startPos, startLoc, exprList);
	        }
	        if (!exprList.length || lastIsComma)
	            this.unexpected(this.lastTokStart);
	        if (spreadStart)
	            this.unexpected(spreadStart);
	        this.checkExpressionErrors(refDestructuringErrors, true);
	        this.yieldPos = oldYieldPos || this.yieldPos;
	        this.awaitPos = oldAwaitPos || this.awaitPos;
	        if (exprList.length > 1) {
	            val = this.startNodeAt(innerStartPos, innerStartLoc);
	            val.expressions = exprList;
	            this.finishNodeAt(val, "SequenceExpression", innerEndPos, innerEndLoc);
	        }
	        else {
	            val = exprList[0];
	        }
	    }
	    else {
	        val = this.parseParenExpression();
	    }
	    if (this.options.preserveParens) {
	        var par = this.startNodeAt(startPos, startLoc);
	        par.expression = val;
	        return this.finishNode(par, "ParenthesizedExpression");
	    }
	    else {
	        return val;
	    }
	};
	pp$2.parseParenItem = function (item) {
	    return item;
	};
	pp$2.parseParenArrowList = function (startPos, startLoc, exprList) {
	    return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), exprList);
	};
	// New's precedence is slightly tricky. It must allow its argument to
	// be a `[]` or dot subscript expression, but not a call — at least,
	// not without wrapping it in parentheses. Thus, it uses the noCalls
	// argument to parseSubscripts to prevent it from consuming the
	// argument list.
	var empty = [];
	pp$2.parseNew = function () {
	    if (this.containsEsc)
	        this.raiseRecoverable(this.start, "Escape sequence in keyword new");
	    var node = this.startNode();
	    var meta = this.parseIdent(true);
	    if (this.options.ecmaVersion >= 6 && this.eat(tokentype.types.dot)) {
	        node.meta = meta;
	        var containsEsc = this.containsEsc;
	        node.property = this.parseIdent(true);
	        if (node.property.name !== "target")
	            this.raiseRecoverable(node.property.start, "The only valid meta property for new is 'new.target'");
	        if (containsEsc)
	            this.raiseRecoverable(node.start, "'new.target' must not contain escaped characters");
	        if (!this.inNonArrowFunction)
	            this.raiseRecoverable(node.start, "'new.target' can only be used in functions");
	        return this.finishNode(node, "MetaProperty");
	    }
	    var startPos = this.start, startLoc = this.startLoc, isImport = this.type === tokentype.types._import;
	    node.callee = this.parseSubscripts(this.parseExprAtom(), startPos, startLoc, true);
	    if (isImport && node.callee.type === "ImportExpression") {
	        this.raise(startPos, "Cannot use new with import()");
	    }
	    if (this.eat(tokentype.types.parenL))
	        node.arguments = this.parseExprList(tokentype.types.parenR, this.options.ecmaVersion >= 8, false);
	    else
	        node.arguments = empty;
	    return this.finishNode(node, "NewExpression");
	};
	// Parse template expression.
	pp$2.parseTemplateElement = function (_a) {
	    var isTagged = _a.isTagged;
	    var elem = this.startNode();
	    if (this.type === tokentype.types.invalidTemplate) {
	        if (!isTagged) {
	            this.raiseRecoverable(this.start, "Bad escape sequence in untagged template literal");
	        }
	        elem.value = {
	            raw: this.value,
	            cooked: null
	        };
	    }
	    else {
	        elem.value = {
	            raw: this.input.slice(this.start, this.end).replace(/\r\n?/g, "\n"),
	            cooked: this.value
	        };
	    }
	    this.next();
	    elem.tail = this.type === tokentype.types.backQuote;
	    return this.finishNode(elem, "TemplateElement");
	};
	pp$2.parseTemplate = function (_a) {
	    var _b = (_a === void 0 ? {} : _a).isTagged, isTagged = _b === void 0 ? false : _b;
	    var node = this.startNode();
	    this.next();
	    node.expressions = [];
	    var curElt = this.parseTemplateElement({ isTagged: isTagged });
	    node.quasis = [curElt];
	    while (!curElt.tail) {
	        if (this.type === tokentype.types.eof)
	            this.raise(this.pos, "Unterminated template literal");
	        this.expect(tokentype.types.dollarBraceL);
	        node.expressions.push(this.parseExpression());
	        this.expect(tokentype.types.braceR);
	        node.quasis.push(curElt = this.parseTemplateElement({ isTagged: isTagged }));
	    }
	    this.next();
	    return this.finishNode(node, "TemplateLiteral");
	};
	pp$2.isAsyncProp = function (prop) {
	    return !prop.computed && prop.key.type === "Identifier" && prop.key.name === "async" && (this.type === tokentype.types.name || this.type === tokentype.types.num || this.type === tokentype.types.string || this.type === tokentype.types.bracketL || this.type.keyword || this.options.ecmaVersion >= 9 && this.type === tokentype.types.star) && !whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
	};
	// Parse an object literal or binding pattern.
	pp$2.parseObj = function (isPattern, refDestructuringErrors) {
	    var node = this.startNode(), first = true, propHash = {};
	    node.properties = [];
	    this.next();
	    while (!this.eat(tokentype.types.braceR)) {
	        if (!first) {
	            this.expect(tokentype.types.comma);
	            if (this.options.ecmaVersion >= 5 && this.afterTrailingComma(tokentype.types.braceR))
	                break;
	        }
	        else
	            first = false;
	        var prop = this.parseProperty(isPattern, refDestructuringErrors);
	        if (!isPattern)
	            this.checkPropClash(prop, propHash, refDestructuringErrors);
	        node.properties.push(prop);
	    }
	    return this.finishNode(node, isPattern ? "ObjectPattern" : "ObjectExpression");
	};
	pp$2.parseProperty = function (isPattern, refDestructuringErrors) {
	    var prop = this.startNode(), isGenerator, isAsync, startPos, startLoc;
	    if (this.options.ecmaVersion >= 9 && this.eat(tokentype.types.ellipsis)) {
	        if (isPattern) {
	            prop.argument = this.parseIdent(false);
	            if (this.type === tokentype.types.comma) {
	                this.raise(this.start, "Comma is not permitted after the rest element");
	            }
	            return this.finishNode(prop, "RestElement");
	        }
	        // To disallow parenthesized identifier via `this.toAssignable()`.
	        if (this.type === tokentype.types.parenL && refDestructuringErrors) {
	            if (refDestructuringErrors.parenthesizedAssign < 0) {
	                refDestructuringErrors.parenthesizedAssign = this.start;
	            }
	            if (refDestructuringErrors.parenthesizedBind < 0) {
	                refDestructuringErrors.parenthesizedBind = this.start;
	            }
	        }
	        // Parse argument.
	        prop.argument = this.parseMaybeAssign(false, refDestructuringErrors);
	        // To disallow trailing comma via `this.toAssignable()`.
	        if (this.type === tokentype.types.comma && refDestructuringErrors && refDestructuringErrors.trailingComma < 0) {
	            refDestructuringErrors.trailingComma = this.start;
	        }
	        // Finish
	        return this.finishNode(prop, "SpreadElement");
	    }
	    if (this.options.ecmaVersion >= 6) {
	        prop.method = false;
	        prop.shorthand = false;
	        if (isPattern || refDestructuringErrors) {
	            startPos = this.start;
	            startLoc = this.startLoc;
	        }
	        if (!isPattern)
	            isGenerator = this.eat(tokentype.types.star);
	    }
	    var containsEsc = this.containsEsc;
	    this.parsePropertyName(prop);
	    if (!isPattern && !containsEsc && this.options.ecmaVersion >= 8 && !isGenerator && this.isAsyncProp(prop)) {
	        isAsync = true;
	        isGenerator = this.options.ecmaVersion >= 9 && this.eat(tokentype.types.star);
	        this.parsePropertyName(prop, refDestructuringErrors);
	    }
	    else {
	        isAsync = false;
	    }
	    this.parsePropertyValue(prop, isPattern, isGenerator, isAsync, startPos, startLoc, refDestructuringErrors, containsEsc);
	    return this.finishNode(prop, "Property");
	};
	pp$2.parsePropertyValue = function (prop, isPattern, isGenerator, isAsync, startPos, startLoc, refDestructuringErrors, containsEsc) {
	    if ((isGenerator || isAsync) && this.type === tokentype.types.colon)
	        this.unexpected();
	    if (this.eat(tokentype.types.colon)) {
	        prop.value = isPattern ? this.parseMaybeDefault(this.start, this.startLoc) : this.parseMaybeAssign(false, refDestructuringErrors);
	        prop.kind = "init";
	    }
	    else if (this.options.ecmaVersion >= 6 && this.type === tokentype.types.parenL) {
	        if (isPattern)
	            this.unexpected();
	        prop.kind = "init";
	        prop.method = true;
	        prop.value = this.parseMethod(isGenerator, isAsync);
	    }
	    else if (!isPattern && !containsEsc && this.options.ecmaVersion >= 5 && !prop.computed && prop.key.type === "Identifier" && (prop.key.name === "get" || prop.key.name === "set") && this.type !== tokentype.types.comma && this.type !== tokentype.types.braceR && this.type !== tokentype.types.eq) {
	        if (isGenerator || isAsync)
	            this.unexpected();
	        prop.kind = prop.key.name;
	        this.parsePropertyName(prop);
	        prop.value = this.parseMethod(false);
	        var paramCount = prop.kind === "get" ? 0 : 1;
	        if (prop.value.params.length !== paramCount) {
	            var start = prop.value.start;
	            if (prop.kind === "get")
	                this.raiseRecoverable(start, "getter should have no params");
	            else
	                this.raiseRecoverable(start, "setter should have exactly one param");
	        }
	        else {
	            if (prop.kind === "set" && prop.value.params[0].type === "RestElement")
	                this.raiseRecoverable(prop.value.params[0].start, "Setter cannot use rest params");
	        }
	    }
	    else if (this.options.ecmaVersion >= 6 && !prop.computed && prop.key.type === "Identifier") {
	        if (isGenerator || isAsync)
	            this.unexpected();
	        this.checkUnreserved(prop.key);
	        if (prop.key.name === "await" && !this.awaitIdentPos)
	            this.awaitIdentPos = startPos;
	        prop.kind = "init";
	        if (isPattern) {
	            prop.value = this.parseMaybeDefault(startPos, startLoc, this.copyNode(prop.key));
	        }
	        else if (this.type === tokentype.types.eq && refDestructuringErrors) {
	            if (refDestructuringErrors.shorthandAssign < 0)
	                refDestructuringErrors.shorthandAssign = this.start;
	            prop.value = this.parseMaybeDefault(startPos, startLoc, this.copyNode(prop.key));
	        }
	        else {
	            prop.value = this.copyNode(prop.key);
	        }
	        prop.shorthand = true;
	    }
	    else
	        this.unexpected();
	};
	pp$2.parsePropertyName = function (prop) {
	    if (this.options.ecmaVersion >= 6) {
	        if (this.eat(tokentype.types.bracketL)) {
	            prop.computed = true;
	            prop.key = this.parseMaybeAssign();
	            this.expect(tokentype.types.bracketR);
	            return prop.key;
	        }
	        else {
	            prop.computed = false;
	        }
	    }
	    return prop.key = this.type === tokentype.types.num || this.type === tokentype.types.string ? this.parseExprAtom() : this.parseIdent(this.options.allowReserved !== "never");
	};
	// Initialize empty function node.
	pp$2.initFunction = function (node) {
	    node.id = null;
	    if (this.options.ecmaVersion >= 6)
	        node.generator = node.expression = false;
	    if (this.options.ecmaVersion >= 8)
	        node.async = false;
	};
	// Parse object or class method.
	pp$2.parseMethod = function (isGenerator, isAsync, allowDirectSuper) {
	    var node = this.startNode(), oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
	    this.initFunction(node);
	    if (this.options.ecmaVersion >= 6)
	        node.generator = isGenerator;
	    if (this.options.ecmaVersion >= 8)
	        node.async = !!isAsync;
	    this.yieldPos = 0;
	    this.awaitPos = 0;
	    this.awaitIdentPos = 0;
	    this.enterScope((0, scopeflags.functionFlags)(isAsync, node.generator) | scopeflags.SCOPE_SUPER | (allowDirectSuper ? scopeflags.SCOPE_DIRECT_SUPER : 0));
	    this.expect(tokentype.types.parenL);
	    node.params = this.parseBindingList(tokentype.types.parenR, false, this.options.ecmaVersion >= 8);
	    this.checkYieldAwaitInDefaultParams();
	    this.parseFunctionBody(node, false, true);
	    this.yieldPos = oldYieldPos;
	    this.awaitPos = oldAwaitPos;
	    this.awaitIdentPos = oldAwaitIdentPos;
	    return this.finishNode(node, "FunctionExpression");
	};
	// Parse arrow function expression with given parameters.
	pp$2.parseArrowExpression = function (node, params, isAsync) {
	    var oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
	    this.enterScope((0, scopeflags.functionFlags)(isAsync, false) | scopeflags.SCOPE_ARROW);
	    this.initFunction(node);
	    if (this.options.ecmaVersion >= 8)
	        node.async = !!isAsync;
	    this.yieldPos = 0;
	    this.awaitPos = 0;
	    this.awaitIdentPos = 0;
	    node.params = this.toAssignableList(params, true);
	    this.parseFunctionBody(node, true, false);
	    this.yieldPos = oldYieldPos;
	    this.awaitPos = oldAwaitPos;
	    this.awaitIdentPos = oldAwaitIdentPos;
	    return this.finishNode(node, "ArrowFunctionExpression");
	};
	// Parse function body and check parameters.
	pp$2.parseFunctionBody = function (node, isArrowFunction, isMethod) {
	    var isExpression = isArrowFunction && this.type !== tokentype.types.braceL;
	    var oldStrict = this.strict, useStrict = false;
	    if (isExpression) {
	        node.body = this.parseMaybeAssign();
	        node.expression = true;
	        this.checkParams(node, false);
	    }
	    else {
	        var nonSimple = this.options.ecmaVersion >= 7 && !this.isSimpleParamList(node.params);
	        if (!oldStrict || nonSimple) {
	            useStrict = this.strictDirective(this.end);
	            // If this is a strict mode function, verify that argument names
	            // are not repeated, and it does not try to bind the words `eval`
	            // or `arguments`.
	            if (useStrict && nonSimple)
	                this.raiseRecoverable(node.start, "Illegal 'use strict' directive in function with non-simple parameter list");
	        }
	        // Start a new scope with regard to labels and the `inFunction`
	        // flag (restore them to their old value afterwards).
	        var oldLabels = this.labels;
	        this.labels = [];
	        if (useStrict)
	            this.strict = true;
	        // Add the params to varDeclaredNames to ensure that an error is thrown
	        // if a let/const declaration in the function clashes with one of the params.
	        this.checkParams(node, !oldStrict && !useStrict && !isArrowFunction && !isMethod && this.isSimpleParamList(node.params));
	        // Ensure the function name isn't a forbidden identifier in strict mode, e.g. 'eval'
	        if (this.strict && node.id)
	            this.checkLValSimple(node.id, scopeflags.BIND_OUTSIDE);
	        node.body = this.parseBlock(false, undefined, useStrict && !oldStrict);
	        node.expression = false;
	        this.adaptDirectivePrologue(node.body.body);
	        this.labels = oldLabels;
	    }
	    this.exitScope();
	};
	pp$2.isSimpleParamList = function (params) {
	    for (var _iterator = params, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
	        var _ref;
	        if (_isArray) {
	            if (_i >= _iterator.length)
	                break;
	            _ref = _iterator[_i++];
	        }
	        else {
	            _i = _iterator.next();
	            if (_i.done)
	                break;
	            _ref = _i.value;
	        }
	        var param = _ref;
	        if (param.type !== "Identifier")
	            return false;
	    }
	    return true;
	};
	// Checks function params for various disallowed patterns such as using "eval"
	// or "arguments" and duplicate parameters.
	pp$2.checkParams = function (node, allowDuplicates) {
	    var nameHash = Object.create(null);
	    for (var _iterator2 = node.params, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
	        var _ref2;
	        if (_isArray2) {
	            if (_i2 >= _iterator2.length)
	                break;
	            _ref2 = _iterator2[_i2++];
	        }
	        else {
	            _i2 = _iterator2.next();
	            if (_i2.done)
	                break;
	            _ref2 = _i2.value;
	        }
	        var param = _ref2;
	        this.checkLValInnerPattern(param, scopeflags.BIND_VAR, allowDuplicates ? null : nameHash);
	    }
	};
	// Parses a comma-separated list of expressions, and returns them as
	// an array. `close` is the token type that ends the list, and
	// `allowEmpty` can be turned on to allow subsequent commas with
	// nothing in between them to be parsed as `null` (which is needed
	// for array literals).
	pp$2.parseExprList = function (close, allowTrailingComma, allowEmpty, refDestructuringErrors) {
	    var elts = [], first = true;
	    while (!this.eat(close)) {
	        if (!first) {
	            this.expect(tokentype.types.comma);
	            if (allowTrailingComma && this.afterTrailingComma(close))
	                break;
	        }
	        else
	            first = false;
	        var elt = void 0;
	        if (allowEmpty && this.type === tokentype.types.comma)
	            elt = null;
	        else if (this.type === tokentype.types.ellipsis) {
	            elt = this.parseSpread(refDestructuringErrors);
	            if (refDestructuringErrors && this.type === tokentype.types.comma && refDestructuringErrors.trailingComma < 0)
	                refDestructuringErrors.trailingComma = this.start;
	        }
	        else {
	            elt = this.parseMaybeAssign(false, refDestructuringErrors);
	        }
	        elts.push(elt);
	    }
	    return elts;
	};
	pp$2.checkUnreserved = function (_a) {
	    var start = _a.start, end = _a.end, name = _a.name;
	    if (this.inGenerator && name === "yield")
	        this.raiseRecoverable(start, "Cannot use 'yield' as identifier inside a generator");
	    if (this.inAsync && name === "await")
	        this.raiseRecoverable(start, "Cannot use 'await' as identifier inside an async function");
	    if (this.keywords.test(name))
	        this.raise(start, "Unexpected keyword '" + name + "'");
	    if (this.options.ecmaVersion < 6 && this.input.slice(start, end).indexOf("\\") !== -1)
	        return;
	    var re = this.strict ? this.reservedWordsStrict : this.reservedWords;
	    if (re.test(name)) {
	        if (!this.inAsync && name === "await")
	            this.raiseRecoverable(start, "Cannot use keyword 'await' outside an async function");
	        this.raiseRecoverable(start, "The keyword '" + name + "' is reserved");
	    }
	};
	// Parse the next token as an identifier. If `liberal` is true (used
	// when parsing properties), it will also convert keywords into
	// identifiers.
	pp$2.parseIdent = function (liberal, isBinding) {
	    var node = this.startNode();
	    if (this.type === tokentype.types.name) {
	        node.name = this.value;
	    }
	    else if (this.type.keyword) {
	        node.name = this.type.keyword;
	        // To fix https://github.com/acornjs/acorn/issues/575
	        // `class` and `function` keywords push new context into this.context.
	        // But there is no chance to pop the context if the keyword is consumed as an identifier such as a property name.
	        // If the previous token is a dot, this does not apply because the context-managing code already ignored the keyword
	        if ((node.name === "class" || node.name === "function") && (this.lastTokEnd !== this.lastTokStart + 1 || this.input.charCodeAt(this.lastTokStart) !== 46)) {
	            this.context.pop();
	        }
	    }
	    else {
	        this.unexpected();
	    }
	    this.next(!!liberal);
	    this.finishNode(node, "Identifier");
	    if (!liberal) {
	        this.checkUnreserved(node);
	        if (node.name === "await" && !this.awaitIdentPos)
	            this.awaitIdentPos = node.start;
	    }
	    return node;
	};
	// Parses yield expression inside generator.
	pp$2.parseYield = function (noIn) {
	    if (!this.yieldPos)
	        this.yieldPos = this.start;
	    var node = this.startNode();
	    this.next();
	    if (this.type === tokentype.types.semi || this.canInsertSemicolon() || this.type !== tokentype.types.star && !this.type.startsExpr) {
	        node.delegate = false;
	        node.argument = null;
	    }
	    else {
	        node.delegate = this.eat(tokentype.types.star);
	        node.argument = this.parseMaybeAssign(noIn);
	    }
	    return this.finishNode(node, "YieldExpression");
	};
	pp$2.parseAwait = function () {
	    if (!this.awaitPos)
	        this.awaitPos = this.start;
	    var node = this.startNode();
	    this.next();
	    node.argument = this.parseMaybeUnary(null, true);
	    return this.finishNode(node, "AwaitExpression");
	};

	var pp$1 = state.Parser.prototype;
	// This function is used to raise exceptions on parse errors. It
	// takes an offset integer (into the current `input`) to indicate
	// the location of the error, attaches the position to the end
	// of the error message, and then raises a `SyntaxError` with that
	// message.
	pp$1.raise = function (pos, message) {
	    var loc = (0, locutil.getLineInfo)(this.input, pos);
	    message += " (" + loc.line + ":" + loc.column + ")";
	    var err = new SyntaxError(message);
	    err.pos = pos;
	    err.loc = loc;
	    err.raisedAt = this.pos;
	    throw err;
	};
	pp$1.raiseRecoverable = pp$1.raise;
	pp$1.curPosition = function () {
	    if (this.options.locations) {
	        return new locutil.Position(this.curLine, this.pos - this.lineStart);
	    }
	};

	var pp = state.Parser.prototype;
	var Scope = /** @class */ (function () {
	    function Scope(flags) {
	        this.flags = flags;
	        // A list of var-declared names in the current lexical scope
	        this.var = [];
	        // A list of lexically-declared names in the current lexical scope
	        this.lexical = [];
	        // A list of lexically-declared FunctionDeclaration names in the current lexical scope
	        this.functions = [];
	    }
	    return Scope;
	}());
	// The functions in this module keep track of declared variables in the current scope in order to detect duplicate variable names.
	pp.enterScope = function (flags) {
	    this.scopeStack.push(new Scope(flags));
	};
	pp.exitScope = function () {
	    this.scopeStack.pop();
	};
	// The spec says:
	// > At the top level of a function, or script, function declarations are
	// > treated like var declarations rather than like lexical declarations.
	pp.treatFunctionsAsVarInScope = function (scope) {
	    return scope.flags & scopeflags.SCOPE_FUNCTION || !this.inModule && scope.flags & scopeflags.SCOPE_TOP;
	};
	pp.declareName = function (name, bindingType, pos) {
	    var redeclared = false;
	    if (bindingType === scopeflags.BIND_LEXICAL) {
	        var scope = this.currentScope();
	        redeclared = scope.lexical.indexOf(name) > -1 || scope.functions.indexOf(name) > -1 || scope.var.indexOf(name) > -1;
	        scope.lexical.push(name);
	        if (this.inModule && scope.flags & scopeflags.SCOPE_TOP)
	            delete this.undefinedExports[name];
	    }
	    else if (bindingType === scopeflags.BIND_SIMPLE_CATCH) {
	        var scope = this.currentScope();
	        scope.lexical.push(name);
	    }
	    else if (bindingType === scopeflags.BIND_FUNCTION) {
	        var scope = this.currentScope();
	        if (this.treatFunctionsAsVar)
	            redeclared = scope.lexical.indexOf(name) > -1;
	        else
	            redeclared = scope.lexical.indexOf(name) > -1 || scope.var.indexOf(name) > -1;
	        scope.functions.push(name);
	    }
	    else {
	        for (var i = this.scopeStack.length - 1; i >= 0; --i) {
	            var scope = this.scopeStack[i];
	            if (scope.lexical.indexOf(name) > -1 && !(scope.flags & scopeflags.SCOPE_SIMPLE_CATCH && scope.lexical[0] === name) || !this.treatFunctionsAsVarInScope(scope) && scope.functions.indexOf(name) > -1) {
	                redeclared = true;
	                break;
	            }
	            scope.var.push(name);
	            if (this.inModule && scope.flags & scopeflags.SCOPE_TOP)
	                delete this.undefinedExports[name];
	            if (scope.flags & scopeflags.SCOPE_VAR)
	                break;
	        }
	    }
	    if (redeclared)
	        this.raiseRecoverable(pos, "Identifier '" + name + "' has already been declared");
	};
	pp.checkLocalExport = function (id) {
	    // scope.functions must be empty as Module code is always strict.
	    if (this.scopeStack[0].lexical.indexOf(id.name) === -1 && this.scopeStack[0].var.indexOf(id.name) === -1) {
	        this.undefinedExports[id.name] = id;
	    }
	};
	pp.currentScope = function () {
	    return this.scopeStack[this.scopeStack.length - 1];
	};
	pp.currentVarScope = function () {
	    for (var i = this.scopeStack.length - 1;; i--) {
	        var scope = this.scopeStack[i];
	        if (scope.flags & scopeflags.SCOPE_VAR)
	            return scope;
	    }
	};
	// Could be useful for `this`, `new.target`, `super()`, `super.property`, and `super[property]`.
	pp.currentThisScope = function () {
	    for (var i = this.scopeStack.length - 1;; i--) {
	        var scope = this.scopeStack[i];
	        if (scope.flags & scopeflags.SCOPE_VAR && !(scope.flags & scopeflags.SCOPE_ARROW))
	            return scope;
	    }
	};

	var node = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.Node = undefined;


	var Node = /** @class */ (function () {
	    function Node(parser, pos, loc) {
	        this.type = "";
	        this.start = pos;
	        this.end = 0;
	        if (parser.options.locations)
	            this.loc = new locutil.SourceLocation(parser, loc);
	        if (parser.options.directSourceFile)
	            this.sourceFile = parser.options.directSourceFile;
	        if (parser.options.ranges)
	            this.range = [pos, 0];
	    }
	    return Node;
	}());
	exports.Node = Node; // Start an AST node, attaching a start offset.
	var pp = state.Parser.prototype;
	pp.startNode = function () {
	    return new Node(this, this.start, this.startLoc);
	};
	pp.startNodeAt = function (pos, loc) {
	    return new Node(this, pos, loc);
	};
	// Finish an AST node, adding `type` and `end` properties.
	function finishNodeAt(node, type, pos, loc) {
	    node.type = type;
	    node.end = pos;
	    if (this.options.locations)
	        node.loc.end = loc;
	    if (this.options.ranges)
	        node.range[1] = pos;
	    return node;
	}
	pp.finishNode = function (node, type) {
	    return finishNodeAt.call(this, node, type, this.lastTokEnd, this.lastTokEndLoc);
	};
	// Finish node at given position
	pp.finishNodeAt = function (node, type, pos, loc) {
	    return finishNodeAt.call(this, node, type, pos, loc);
	};
	pp.copyNode = function (node) {
	    var newNode = new Node(this, node.start, this.startLoc);
	    for (var prop in node)
	        newNode[prop] = node[prop];
	    return newNode;
	};
	});

	var tokencontext = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.types = exports.TokContext = undefined;



	var TokContext = /** @class */ (function () {
	    function TokContext(token, isExpr, preserveSpace, override, generator) {
	        this.token = token;
	        this.isExpr = !!isExpr;
	        this.preserveSpace = !!preserveSpace;
	        this.override = override;
	        this.generator = !!generator;
	    }
	    return TokContext;
	}());
	exports.TokContext = TokContext; // The algorithm used to determine whether a regexp can appear at a
	// given point in the program is loosely based on sweet.js' approach.
	// See https://github.com/mozilla/sweet.js/wiki/design
	var types = exports.types = {
	    b_stat: new TokContext("{", false),
	    b_expr: new TokContext("{", true),
	    b_tmpl: new TokContext("${", false),
	    p_stat: new TokContext("(", false),
	    p_expr: new TokContext("(", true),
	    q_tmpl: new TokContext("`", true, true, function (p) { return p.tryReadTemplateToken(); }),
	    f_stat: new TokContext("function", false),
	    f_expr: new TokContext("function", true),
	    f_expr_gen: new TokContext("function", true, false, null, true),
	    f_gen: new TokContext("function", false, false, null, true)
	};
	var pp = state.Parser.prototype;
	pp.initialContext = function () {
	    return [types.b_stat];
	};
	pp.braceIsBlock = function (prevType) {
	    var parent = this.curContext();
	    if (parent === types.f_expr || parent === types.f_stat)
	        return true;
	    if (prevType === tokentype.types.colon && (parent === types.b_stat || parent === types.b_expr))
	        return !parent.isExpr;
	    // The check for `tt.name && exprAllowed` detects whether we are
	    // after a `yield` or `of` construct. See the `updateContext` for
	    // `tt.name`.
	    if (prevType === tokentype.types._return || prevType === tokentype.types.name && this.exprAllowed)
	        return whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
	    if (prevType === tokentype.types._else || prevType === tokentype.types.semi || prevType === tokentype.types.eof || prevType === tokentype.types.parenR || prevType === tokentype.types.arrow)
	        return true;
	    if (prevType === tokentype.types.braceL)
	        return parent === types.b_stat;
	    if (prevType === tokentype.types._var || prevType === tokentype.types._const || prevType === tokentype.types.name)
	        return false;
	    return !this.exprAllowed;
	};
	pp.inGeneratorContext = function () {
	    for (var i = this.context.length - 1; i >= 1; i--) {
	        var context = this.context[i];
	        if (context.token === "function")
	            return context.generator;
	    }
	    return false;
	};
	pp.updateContext = function (prevType) {
	    var update, type = this.type;
	    if (type.keyword && prevType === tokentype.types.dot)
	        this.exprAllowed = false;
	    else if (update = type.updateContext)
	        update.call(this, prevType);
	    else
	        this.exprAllowed = type.beforeExpr;
	};
	// Token-specific context update code
	tokentype.types.parenR.updateContext = tokentype.types.braceR.updateContext = function () {
	    if (this.context.length === 1) {
	        this.exprAllowed = true;
	        return;
	    }
	    var out = this.context.pop();
	    if (out === types.b_stat && this.curContext().token === "function") {
	        out = this.context.pop();
	    }
	    this.exprAllowed = !out.isExpr;
	};
	tokentype.types.braceL.updateContext = function (prevType) {
	    this.context.push(this.braceIsBlock(prevType) ? types.b_stat : types.b_expr);
	    this.exprAllowed = true;
	};
	tokentype.types.dollarBraceL.updateContext = function () {
	    this.context.push(types.b_tmpl);
	    this.exprAllowed = true;
	};
	tokentype.types.parenL.updateContext = function (prevType) {
	    var statementParens = prevType === tokentype.types._if || prevType === tokentype.types._for || prevType === tokentype.types._with || prevType === tokentype.types._while;
	    this.context.push(statementParens ? types.p_stat : types.p_expr);
	    this.exprAllowed = true;
	};
	tokentype.types.incDec.updateContext = function () {
	    // tokExprAllowed stays unchanged
	};
	tokentype.types._function.updateContext = tokentype.types._class.updateContext = function (prevType) {
	    if (prevType.beforeExpr && prevType !== tokentype.types._else && !(prevType === tokentype.types.semi && this.curContext() !== types.p_stat) && !(prevType === tokentype.types._return && whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.start))) && !((prevType === tokentype.types.colon || prevType === tokentype.types.braceL) && this.curContext() === types.b_stat))
	        this.context.push(types.f_expr);
	    else
	        this.context.push(types.f_stat);
	    this.exprAllowed = false;
	};
	tokentype.types.backQuote.updateContext = function () {
	    if (this.curContext() === types.q_tmpl)
	        this.context.pop();
	    else
	        this.context.push(types.q_tmpl);
	    this.exprAllowed = false;
	};
	tokentype.types.star.updateContext = function (prevType) {
	    if (prevType === tokentype.types._function) {
	        var index = this.context.length - 1;
	        if (this.context[index] === types.f_expr)
	            this.context[index] = types.f_expr_gen;
	        else
	            this.context[index] = types.f_gen;
	    }
	    this.exprAllowed = true;
	};
	tokentype.types.name.updateContext = function (prevType) {
	    var allowed = false;
	    if (this.options.ecmaVersion >= 6 && prevType !== tokentype.types.dot) {
	        if (this.value === "of" && !this.exprAllowed || this.value === "yield" && this.inGeneratorContext())
	            allowed = true;
	    }
	    this.exprAllowed = allowed;
	};
	});

	var unicodePropertyData = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;

	// This file contains Unicode properties extracted from the ECMAScript
	// specification. The lists are extracted like so:
	// $$('#table-binary-unicode-properties > figure > table > tbody > tr > td:nth-child(1) code').map(el => el.innerText)
	// #table-binary-unicode-properties
	var ecma9BinaryProperties = "ASCII ASCII_Hex_Digit AHex Alphabetic Alpha Any Assigned Bidi_Control Bidi_C Bidi_Mirrored Bidi_M Case_Ignorable CI Cased Changes_When_Casefolded CWCF Changes_When_Casemapped CWCM Changes_When_Lowercased CWL Changes_When_NFKC_Casefolded CWKCF Changes_When_Titlecased CWT Changes_When_Uppercased CWU Dash Default_Ignorable_Code_Point DI Deprecated Dep Diacritic Dia Emoji Emoji_Component Emoji_Modifier Emoji_Modifier_Base Emoji_Presentation Extender Ext Grapheme_Base Gr_Base Grapheme_Extend Gr_Ext Hex_Digit Hex IDS_Binary_Operator IDSB IDS_Trinary_Operator IDST ID_Continue IDC ID_Start IDS Ideographic Ideo Join_Control Join_C Logical_Order_Exception LOE Lowercase Lower Math Noncharacter_Code_Point NChar Pattern_Syntax Pat_Syn Pattern_White_Space Pat_WS Quotation_Mark QMark Radical Regional_Indicator RI Sentence_Terminal STerm Soft_Dotted SD Terminal_Punctuation Term Unified_Ideograph UIdeo Uppercase Upper Variation_Selector VS White_Space space XID_Continue XIDC XID_Start XIDS";
	var ecma10BinaryProperties = ecma9BinaryProperties + " Extended_Pictographic";
	var ecma11BinaryProperties = ecma10BinaryProperties;
	var ecma12BinaryProperties = ecma11BinaryProperties + " EBase EComp EMod EPres ExtPict";
	var unicodeBinaryProperties = {
	    9: ecma9BinaryProperties,
	    10: ecma10BinaryProperties,
	    11: ecma11BinaryProperties,
	    12: ecma12BinaryProperties
	    // #table-unicode-general-category-values
	};
	var unicodeGeneralCategoryValues = "Cased_Letter LC Close_Punctuation Pe Connector_Punctuation Pc Control Cc cntrl Currency_Symbol Sc Dash_Punctuation Pd Decimal_Number Nd digit Enclosing_Mark Me Final_Punctuation Pf Format Cf Initial_Punctuation Pi Letter L Letter_Number Nl Line_Separator Zl Lowercase_Letter Ll Mark M Combining_Mark Math_Symbol Sm Modifier_Letter Lm Modifier_Symbol Sk Nonspacing_Mark Mn Number N Open_Punctuation Ps Other C Other_Letter Lo Other_Number No Other_Punctuation Po Other_Symbol So Paragraph_Separator Zp Private_Use Co Punctuation P punct Separator Z Space_Separator Zs Spacing_Mark Mc Surrogate Cs Symbol S Titlecase_Letter Lt Unassigned Cn Uppercase_Letter Lu";
	// #table-unicode-script-values
	var ecma9ScriptValues = "Adlam Adlm Ahom Ahom Anatolian_Hieroglyphs Hluw Arabic Arab Armenian Armn Avestan Avst Balinese Bali Bamum Bamu Bassa_Vah Bass Batak Batk Bengali Beng Bhaiksuki Bhks Bopomofo Bopo Brahmi Brah Braille Brai Buginese Bugi Buhid Buhd Canadian_Aboriginal Cans Carian Cari Caucasian_Albanian Aghb Chakma Cakm Cham Cham Cherokee Cher Common Zyyy Coptic Copt Qaac Cuneiform Xsux Cypriot Cprt Cyrillic Cyrl Deseret Dsrt Devanagari Deva Duployan Dupl Egyptian_Hieroglyphs Egyp Elbasan Elba Ethiopic Ethi Georgian Geor Glagolitic Glag Gothic Goth Grantha Gran Greek Grek Gujarati Gujr Gurmukhi Guru Han Hani Hangul Hang Hanunoo Hano Hatran Hatr Hebrew Hebr Hiragana Hira Imperial_Aramaic Armi Inherited Zinh Qaai Inscriptional_Pahlavi Phli Inscriptional_Parthian Prti Javanese Java Kaithi Kthi Kannada Knda Katakana Kana Kayah_Li Kali Kharoshthi Khar Khmer Khmr Khojki Khoj Khudawadi Sind Lao Laoo Latin Latn Lepcha Lepc Limbu Limb Linear_A Lina Linear_B Linb Lisu Lisu Lycian Lyci Lydian Lydi Mahajani Mahj Malayalam Mlym Mandaic Mand Manichaean Mani Marchen Marc Masaram_Gondi Gonm Meetei_Mayek Mtei Mende_Kikakui Mend Meroitic_Cursive Merc Meroitic_Hieroglyphs Mero Miao Plrd Modi Modi Mongolian Mong Mro Mroo Multani Mult Myanmar Mymr Nabataean Nbat New_Tai_Lue Talu Newa Newa Nko Nkoo Nushu Nshu Ogham Ogam Ol_Chiki Olck Old_Hungarian Hung Old_Italic Ital Old_North_Arabian Narb Old_Permic Perm Old_Persian Xpeo Old_South_Arabian Sarb Old_Turkic Orkh Oriya Orya Osage Osge Osmanya Osma Pahawh_Hmong Hmng Palmyrene Palm Pau_Cin_Hau Pauc Phags_Pa Phag Phoenician Phnx Psalter_Pahlavi Phlp Rejang Rjng Runic Runr Samaritan Samr Saurashtra Saur Sharada Shrd Shavian Shaw Siddham Sidd SignWriting Sgnw Sinhala Sinh Sora_Sompeng Sora Soyombo Soyo Sundanese Sund Syloti_Nagri Sylo Syriac Syrc Tagalog Tglg Tagbanwa Tagb Tai_Le Tale Tai_Tham Lana Tai_Viet Tavt Takri Takr Tamil Taml Tangut Tang Telugu Telu Thaana Thaa Thai Thai Tibetan Tibt Tifinagh Tfng Tirhuta Tirh Ugaritic Ugar Vai Vaii Warang_Citi Wara Yi Yiii Zanabazar_Square Zanb";
	var ecma10ScriptValues = ecma9ScriptValues + " Dogra Dogr Gunjala_Gondi Gong Hanifi_Rohingya Rohg Makasar Maka Medefaidrin Medf Old_Sogdian Sogo Sogdian Sogd";
	var ecma11ScriptValues = ecma10ScriptValues + " Elymaic Elym Nandinagari Nand Nyiakeng_Puachue_Hmong Hmnp Wancho Wcho";
	var ecma12ScriptValues = ecma11ScriptValues + " Chorasmian Chrs Diak Dives_Akuru Khitan_Small_Script Kits Yezi Yezidi";
	var unicodeScriptValues = {
	    9: ecma9ScriptValues,
	    10: ecma10ScriptValues,
	    11: ecma11ScriptValues,
	    12: ecma12ScriptValues
	};
	var data = {};
	function buildUnicodeData(ecmaVersion) {
	    var d = data[ecmaVersion] = {
	        binary: (0, util.wordsRegexp)(unicodeBinaryProperties[ecmaVersion] + " " + unicodeGeneralCategoryValues),
	        nonBinary: {
	            General_Category: (0, util.wordsRegexp)(unicodeGeneralCategoryValues),
	            Script: (0, util.wordsRegexp)(unicodeScriptValues[ecmaVersion])
	        }
	    };
	    d.nonBinary.Script_Extensions = d.nonBinary.Script;
	    d.nonBinary.gc = d.nonBinary.General_Category;
	    d.nonBinary.sc = d.nonBinary.Script;
	    d.nonBinary.scx = d.nonBinary.Script_Extensions;
	}
	buildUnicodeData(9);
	buildUnicodeData(10);
	buildUnicodeData(11);
	buildUnicodeData(12);
	exports.default = data;
	module.exports = exports['default'];
	});

	var regexp = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.RegExpValidationState = undefined;



	var _unicodePropertyData2 = _interopRequireDefault(unicodePropertyData);

	function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
	var pp = state.Parser.prototype;
	var RegExpValidationState = /** @class */ (function () {
	    function RegExpValidationState(parser) {
	        this.parser = parser;
	        this.validFlags = "gim" + (parser.options.ecmaVersion >= 6 ? "uy" : "") + (parser.options.ecmaVersion >= 9 ? "s" : "");
	        this.unicodeProperties = _unicodePropertyData2.default[parser.options.ecmaVersion >= 12 ? 12 : parser.options.ecmaVersion];
	        this.source = "";
	        this.flags = "";
	        this.start = 0;
	        this.switchU = false;
	        this.switchN = false;
	        this.pos = 0;
	        this.lastIntValue = 0;
	        this.lastStringValue = "";
	        this.lastAssertionIsQuantifiable = false;
	        this.numCapturingParens = 0;
	        this.maxBackReference = 0;
	        this.groupNames = [];
	        this.backReferenceNames = [];
	    }
	    RegExpValidationState.prototype.reset = function (start, pattern, flags) {
	        var unicode = flags.indexOf("u") !== -1;
	        this.start = start | 0;
	        this.source = pattern + "";
	        this.flags = flags;
	        this.switchU = unicode && this.parser.options.ecmaVersion >= 6;
	        this.switchN = unicode && this.parser.options.ecmaVersion >= 9;
	    };
	    RegExpValidationState.prototype.raise = function (message) {
	        this.parser.raiseRecoverable(this.start, "Invalid regular expression: /" + this.source + "/: " + message);
	    };
	    // If u flag is given, this returns the code point at the index (it combines a surrogate pair).
	    // Otherwise, this returns the code unit of the index (can be a part of a surrogate pair).
	    RegExpValidationState.prototype.at = function (i, forceU) {
	        if (forceU === void 0) { forceU = false; }
	        var s = this.source;
	        var l = s.length;
	        if (i >= l) {
	            return -1;
	        }
	        var c = s.charCodeAt(i);
	        if (!(forceU || this.switchU) || c <= 0xD7FF || c >= 0xE000 || i + 1 >= l) {
	            return c;
	        }
	        var next = s.charCodeAt(i + 1);
	        return next >= 0xDC00 && next <= 0xDFFF ? (c << 10) + next - 0x35FDC00 : c;
	    };
	    RegExpValidationState.prototype.nextIndex = function (i, forceU) {
	        if (forceU === void 0) { forceU = false; }
	        var s = this.source;
	        var l = s.length;
	        if (i >= l) {
	            return l;
	        }
	        var c = s.charCodeAt(i), next;
	        if (!(forceU || this.switchU) || c <= 0xD7FF || c >= 0xE000 || i + 1 >= l || (next = s.charCodeAt(i + 1)) < 0xDC00 || next > 0xDFFF) {
	            return i + 1;
	        }
	        return i + 2;
	    };
	    RegExpValidationState.prototype.current = function (forceU) {
	        if (forceU === void 0) { forceU = false; }
	        return this.at(this.pos, forceU);
	    };
	    RegExpValidationState.prototype.lookahead = function (forceU) {
	        if (forceU === void 0) { forceU = false; }
	        return this.at(this.nextIndex(this.pos, forceU), forceU);
	    };
	    RegExpValidationState.prototype.advance = function (forceU) {
	        if (forceU === void 0) { forceU = false; }
	        this.pos = this.nextIndex(this.pos, forceU);
	    };
	    RegExpValidationState.prototype.eat = function (ch, forceU) {
	        if (forceU === void 0) { forceU = false; }
	        if (this.current(forceU) === ch) {
	            this.advance(forceU);
	            return true;
	        }
	        return false;
	    };
	    return RegExpValidationState;
	}());
	exports.RegExpValidationState = RegExpValidationState;
	function codePointToString(ch) {
	    if (ch <= 0xFFFF)
	        return String.fromCharCode(ch);
	    ch -= 0x10000;
	    return String.fromCharCode((ch >> 10) + 0xD800, (ch & 0x03FF) + 0xDC00);
	}
	/**
	 * Validate the flags part of a given RegExpLiteral.
	 *
	 * @param {RegExpValidationState} state The state to validate RegExp.
	 * @returns {void}
	 */
	pp.validateRegExpFlags = function (state) {
	    var validFlags = state.validFlags;
	    var flags = state.flags;
	    for (var i = 0; i < flags.length; i++) {
	        var flag = flags.charAt(i);
	        if (validFlags.indexOf(flag) === -1) {
	            this.raise(state.start, "Invalid regular expression flag");
	        }
	        if (flags.indexOf(flag, i + 1) > -1) {
	            this.raise(state.start, "Duplicate regular expression flag");
	        }
	    }
	};
	/**
	 * Validate the pattern part of a given RegExpLiteral.
	 *
	 * @param {RegExpValidationState} state The state to validate RegExp.
	 * @returns {void}
	 */
	pp.validateRegExpPattern = function (state) {
	    this.regexp_pattern(state);
	    // The goal symbol for the parse is |Pattern[~U, ~N]|. If the result of
	    // parsing contains a |GroupName|, reparse with the goal symbol
	    // |Pattern[~U, +N]| and use this result instead. Throw a *SyntaxError*
	    // exception if _P_ did not conform to the grammar, if any elements of _P_
	    // were not matched by the parse, or if any Early Error conditions exist.
	    if (!state.switchN && this.options.ecmaVersion >= 9 && state.groupNames.length > 0) {
	        state.switchN = true;
	        this.regexp_pattern(state);
	    }
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Pattern
	pp.regexp_pattern = function (state) {
	    state.pos = 0;
	    state.lastIntValue = 0;
	    state.lastStringValue = "";
	    state.lastAssertionIsQuantifiable = false;
	    state.numCapturingParens = 0;
	    state.maxBackReference = 0;
	    state.groupNames.length = 0;
	    state.backReferenceNames.length = 0;
	    this.regexp_disjunction(state);
	    if (state.pos !== state.source.length) {
	        // Make the same messages as V8.
	        if (state.eat(0x29 /* ) */)) {
	            state.raise("Unmatched ')'");
	        }
	        if (state.eat(0x5D /* ] */) || state.eat(0x7D /* } */)) {
	            state.raise("Lone quantifier brackets");
	        }
	    }
	    if (state.maxBackReference > state.numCapturingParens) {
	        state.raise("Invalid escape");
	    }
	    for (var _iterator = state.backReferenceNames, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
	        var _ref;
	        if (_isArray) {
	            if (_i >= _iterator.length)
	                break;
	            _ref = _iterator[_i++];
	        }
	        else {
	            _i = _iterator.next();
	            if (_i.done)
	                break;
	            _ref = _i.value;
	        }
	        var name_1 = _ref;
	        if (state.groupNames.indexOf(name_1) === -1) {
	            state.raise("Invalid named capture referenced");
	        }
	    }
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Disjunction
	pp.regexp_disjunction = function (state) {
	    this.regexp_alternative(state);
	    while (state.eat(0x7C /* | */)) {
	        this.regexp_alternative(state);
	    }
	    // Make the same message as V8.
	    if (this.regexp_eatQuantifier(state, true)) {
	        state.raise("Nothing to repeat");
	    }
	    if (state.eat(0x7B /* { */)) {
	        state.raise("Lone quantifier brackets");
	    }
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Alternative
	pp.regexp_alternative = function (state) {
	    while (state.pos < state.source.length && this.regexp_eatTerm(state))
	        ;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-Term
	pp.regexp_eatTerm = function (state) {
	    if (this.regexp_eatAssertion(state)) {
	        // Handle `QuantifiableAssertion Quantifier` alternative.
	        // `state.lastAssertionIsQuantifiable` is true if the last eaten Assertion
	        // is a QuantifiableAssertion.
	        if (state.lastAssertionIsQuantifiable && this.regexp_eatQuantifier(state)) {
	            // Make the same message as V8.
	            if (state.switchU) {
	                state.raise("Invalid quantifier");
	            }
	        }
	        return true;
	    }
	    if (state.switchU ? this.regexp_eatAtom(state) : this.regexp_eatExtendedAtom(state)) {
	        this.regexp_eatQuantifier(state);
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-Assertion
	pp.regexp_eatAssertion = function (state) {
	    var start = state.pos;
	    state.lastAssertionIsQuantifiable = false;
	    // ^, $
	    if (state.eat(0x5E /* ^ */) || state.eat(0x24 /* $ */)) {
	        return true;
	    }
	    // \b \B
	    if (state.eat(0x5C /* \ */)) {
	        if (state.eat(0x42 /* B */) || state.eat(0x62 /* b */)) {
	            return true;
	        }
	        state.pos = start;
	    }
	    // Lookahead / Lookbehind
	    if (state.eat(0x28 /* ( */) && state.eat(0x3F /* ? */)) {
	        var lookbehind = false;
	        if (this.options.ecmaVersion >= 9) {
	            lookbehind = state.eat(0x3C /* < */);
	        }
	        if (state.eat(0x3D /* = */) || state.eat(0x21 /* ! */)) {
	            this.regexp_disjunction(state);
	            if (!state.eat(0x29 /* ) */)) {
	                state.raise("Unterminated group");
	            }
	            state.lastAssertionIsQuantifiable = !lookbehind;
	            return true;
	        }
	    }
	    state.pos = start;
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Quantifier
	pp.regexp_eatQuantifier = function (state, noError) {
	    if (noError === void 0) { noError = false; }
	    if (this.regexp_eatQuantifierPrefix(state, noError)) {
	        state.eat(0x3F /* ? */);
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-QuantifierPrefix
	pp.regexp_eatQuantifierPrefix = function (state, noError) {
	    return state.eat(0x2A /* * */) || state.eat(0x2B /* + */) || state.eat(0x3F /* ? */) || this.regexp_eatBracedQuantifier(state, noError);
	};
	pp.regexp_eatBracedQuantifier = function (state, noError) {
	    var start = state.pos;
	    if (state.eat(0x7B /* { */)) {
	        var min = 0, max = -1;
	        if (this.regexp_eatDecimalDigits(state)) {
	            min = state.lastIntValue;
	            if (state.eat(0x2C /* , */) && this.regexp_eatDecimalDigits(state)) {
	                max = state.lastIntValue;
	            }
	            if (state.eat(0x7D /* } */)) {
	                // SyntaxError in https://www.ecma-international.org/ecma-262/8.0/#sec-term
	                if (max !== -1 && max < min && !noError) {
	                    state.raise("numbers out of order in {} quantifier");
	                }
	                return true;
	            }
	        }
	        if (state.switchU && !noError) {
	            state.raise("Incomplete quantifier");
	        }
	        state.pos = start;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Atom
	pp.regexp_eatAtom = function (state) {
	    return this.regexp_eatPatternCharacters(state) || state.eat(0x2E /* . */) || this.regexp_eatReverseSolidusAtomEscape(state) || this.regexp_eatCharacterClass(state) || this.regexp_eatUncapturingGroup(state) || this.regexp_eatCapturingGroup(state);
	};
	pp.regexp_eatReverseSolidusAtomEscape = function (state) {
	    var start = state.pos;
	    if (state.eat(0x5C /* \ */)) {
	        if (this.regexp_eatAtomEscape(state)) {
	            return true;
	        }
	        state.pos = start;
	    }
	    return false;
	};
	pp.regexp_eatUncapturingGroup = function (state) {
	    var start = state.pos;
	    if (state.eat(0x28 /* ( */)) {
	        if (state.eat(0x3F /* ? */) && state.eat(0x3A /* : */)) {
	            this.regexp_disjunction(state);
	            if (state.eat(0x29 /* ) */)) {
	                return true;
	            }
	            state.raise("Unterminated group");
	        }
	        state.pos = start;
	    }
	    return false;
	};
	pp.regexp_eatCapturingGroup = function (state) {
	    if (state.eat(0x28 /* ( */)) {
	        if (this.options.ecmaVersion >= 9) {
	            this.regexp_groupSpecifier(state);
	        }
	        else if (state.current() === 0x3F /* ? */) {
	            state.raise("Invalid group");
	        }
	        this.regexp_disjunction(state);
	        if (state.eat(0x29 /* ) */)) {
	            state.numCapturingParens += 1;
	            return true;
	        }
	        state.raise("Unterminated group");
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ExtendedAtom
	pp.regexp_eatExtendedAtom = function (state) {
	    return state.eat(0x2E /* . */) || this.regexp_eatReverseSolidusAtomEscape(state) || this.regexp_eatCharacterClass(state) || this.regexp_eatUncapturingGroup(state) || this.regexp_eatCapturingGroup(state) || this.regexp_eatInvalidBracedQuantifier(state) || this.regexp_eatExtendedPatternCharacter(state);
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-InvalidBracedQuantifier
	pp.regexp_eatInvalidBracedQuantifier = function (state) {
	    if (this.regexp_eatBracedQuantifier(state, true)) {
	        state.raise("Nothing to repeat");
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-SyntaxCharacter
	pp.regexp_eatSyntaxCharacter = function (state) {
	    var ch = state.current();
	    if (isSyntaxCharacter(ch)) {
	        state.lastIntValue = ch;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	function isSyntaxCharacter(ch) {
	    return ch === 0x24 /* $ */ || ch >= 0x28 /* ( */ && ch <= 0x2B /* + */ || ch === 0x2E /* . */ || ch === 0x3F /* ? */ || ch >= 0x5B /* [ */ && ch <= 0x5E /* ^ */ || ch >= 0x7B /* { */ && ch <= 0x7D /* } */;
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-PatternCharacter
	// But eat eager.
	pp.regexp_eatPatternCharacters = function (state) {
	    var start = state.pos;
	    var ch = 0;
	    while ((ch = state.current()) !== -1 && !isSyntaxCharacter(ch)) {
	        state.advance();
	    }
	    return state.pos !== start;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ExtendedPatternCharacter
	pp.regexp_eatExtendedPatternCharacter = function (state) {
	    var ch = state.current();
	    if (ch !== -1 && ch !== 0x24 /* $ */ && !(ch >= 0x28 /* ( */ && ch <= 0x2B /* + */) && ch !== 0x2E /* . */ && ch !== 0x3F /* ? */ && ch !== 0x5B /* [ */ && ch !== 0x5E /* ^ */ && ch !== 0x7C /* | */) {
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// GroupSpecifier ::
	//   [empty]
	//   `?` GroupName
	pp.regexp_groupSpecifier = function (state) {
	    if (state.eat(0x3F /* ? */)) {
	        if (this.regexp_eatGroupName(state)) {
	            if (state.groupNames.indexOf(state.lastStringValue) !== -1) {
	                state.raise("Duplicate capture group name");
	            }
	            state.groupNames.push(state.lastStringValue);
	            return;
	        }
	        state.raise("Invalid group");
	    }
	};
	// GroupName ::
	//   `<` RegExpIdentifierName `>`
	// Note: this updates `state.lastStringValue` property with the eaten name.
	pp.regexp_eatGroupName = function (state) {
	    state.lastStringValue = "";
	    if (state.eat(0x3C /* < */)) {
	        if (this.regexp_eatRegExpIdentifierName(state) && state.eat(0x3E /* > */)) {
	            return true;
	        }
	        state.raise("Invalid capture group name");
	    }
	    return false;
	};
	// RegExpIdentifierName ::
	//   RegExpIdentifierStart
	//   RegExpIdentifierName RegExpIdentifierPart
	// Note: this updates `state.lastStringValue` property with the eaten name.
	pp.regexp_eatRegExpIdentifierName = function (state) {
	    state.lastStringValue = "";
	    if (this.regexp_eatRegExpIdentifierStart(state)) {
	        state.lastStringValue += codePointToString(state.lastIntValue);
	        while (this.regexp_eatRegExpIdentifierPart(state)) {
	            state.lastStringValue += codePointToString(state.lastIntValue);
	        }
	        return true;
	    }
	    return false;
	};
	// RegExpIdentifierStart ::
	//   UnicodeIDStart
	//   `$`
	//   `_`
	//   `\` RegExpUnicodeEscapeSequence[+U]
	pp.regexp_eatRegExpIdentifierStart = function (state) {
	    var start = state.pos;
	    var forceU = this.options.ecmaVersion >= 11;
	    var ch = state.current(forceU);
	    state.advance(forceU);
	    if (ch === 0x5C /* \ */ && this.regexp_eatRegExpUnicodeEscapeSequence(state, forceU)) {
	        ch = state.lastIntValue;
	    }
	    if (isRegExpIdentifierStart(ch)) {
	        state.lastIntValue = ch;
	        return true;
	    }
	    state.pos = start;
	    return false;
	};
	function isRegExpIdentifierStart(ch) {
	    return (0, identifier.isIdentifierStart)(ch, true) || ch === 0x24 /* $ */ || ch === 0x5F; /* _ */
	}
	// RegExpIdentifierPart ::
	//   UnicodeIDContinue
	//   `$`
	//   `_`
	//   `\` RegExpUnicodeEscapeSequence[+U]
	//   <ZWNJ>
	//   <ZWJ>
	pp.regexp_eatRegExpIdentifierPart = function (state) {
	    var start = state.pos;
	    var forceU = this.options.ecmaVersion >= 11;
	    var ch = state.current(forceU);
	    state.advance(forceU);
	    if (ch === 0x5C /* \ */ && this.regexp_eatRegExpUnicodeEscapeSequence(state, forceU)) {
	        ch = state.lastIntValue;
	    }
	    if (isRegExpIdentifierPart(ch)) {
	        state.lastIntValue = ch;
	        return true;
	    }
	    state.pos = start;
	    return false;
	};
	function isRegExpIdentifierPart(ch) {
	    return (0, identifier.isIdentifierChar)(ch, true) || ch === 0x24 /* $ */ || ch === 0x5F /* _ */ || ch === 0x200C /* <ZWNJ> */ || ch === 0x200D; /* <ZWJ> */
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-AtomEscape
	pp.regexp_eatAtomEscape = function (state) {
	    if (this.regexp_eatBackReference(state) || this.regexp_eatCharacterClassEscape(state) || this.regexp_eatCharacterEscape(state) || state.switchN && this.regexp_eatKGroupName(state)) {
	        return true;
	    }
	    if (state.switchU) {
	        // Make the same message as V8.
	        if (state.current() === 0x63 /* c */) {
	            state.raise("Invalid unicode escape");
	        }
	        state.raise("Invalid escape");
	    }
	    return false;
	};
	pp.regexp_eatBackReference = function (state) {
	    var start = state.pos;
	    if (this.regexp_eatDecimalEscape(state)) {
	        var n = state.lastIntValue;
	        if (state.switchU) {
	            // For SyntaxError in https://www.ecma-international.org/ecma-262/8.0/#sec-atomescape
	            if (n > state.maxBackReference) {
	                state.maxBackReference = n;
	            }
	            return true;
	        }
	        if (n <= state.numCapturingParens) {
	            return true;
	        }
	        state.pos = start;
	    }
	    return false;
	};
	pp.regexp_eatKGroupName = function (state) {
	    if (state.eat(0x6B /* k */)) {
	        if (this.regexp_eatGroupName(state)) {
	            state.backReferenceNames.push(state.lastStringValue);
	            return true;
	        }
	        state.raise("Invalid named reference");
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-CharacterEscape
	pp.regexp_eatCharacterEscape = function (state) {
	    return this.regexp_eatControlEscape(state) || this.regexp_eatCControlLetter(state) || this.regexp_eatZero(state) || this.regexp_eatHexEscapeSequence(state) || this.regexp_eatRegExpUnicodeEscapeSequence(state, false) || !state.switchU && this.regexp_eatLegacyOctalEscapeSequence(state) || this.regexp_eatIdentityEscape(state);
	};
	pp.regexp_eatCControlLetter = function (state) {
	    var start = state.pos;
	    if (state.eat(0x63 /* c */)) {
	        if (this.regexp_eatControlLetter(state)) {
	            return true;
	        }
	        state.pos = start;
	    }
	    return false;
	};
	pp.regexp_eatZero = function (state) {
	    if (state.current() === 0x30 /* 0 */ && !isDecimalDigit(state.lookahead())) {
	        state.lastIntValue = 0;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-ControlEscape
	pp.regexp_eatControlEscape = function (state) {
	    var ch = state.current();
	    if (ch === 0x74 /* t */) {
	        state.lastIntValue = 0x09; /* \t */
	        state.advance();
	        return true;
	    }
	    if (ch === 0x6E /* n */) {
	        state.lastIntValue = 0x0A; /* \n */
	        state.advance();
	        return true;
	    }
	    if (ch === 0x76 /* v */) {
	        state.lastIntValue = 0x0B; /* \v */
	        state.advance();
	        return true;
	    }
	    if (ch === 0x66 /* f */) {
	        state.lastIntValue = 0x0C; /* \f */
	        state.advance();
	        return true;
	    }
	    if (ch === 0x72 /* r */) {
	        state.lastIntValue = 0x0D; /* \r */
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-ControlLetter
	pp.regexp_eatControlLetter = function (state) {
	    var ch = state.current();
	    if (isControlLetter(ch)) {
	        state.lastIntValue = ch % 0x20;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	function isControlLetter(ch) {
	    return ch >= 0x41 /* A */ && ch <= 0x5A /* Z */ || ch >= 0x61 /* a */ && ch <= 0x7A /* z */;
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-RegExpUnicodeEscapeSequence
	pp.regexp_eatRegExpUnicodeEscapeSequence = function (state, forceU) {
	    if (forceU === void 0) { forceU = false; }
	    var start = state.pos;
	    var switchU = forceU || state.switchU;
	    if (state.eat(0x75 /* u */)) {
	        if (this.regexp_eatFixedHexDigits(state, 4)) {
	            var lead = state.lastIntValue;
	            if (switchU && lead >= 0xD800 && lead <= 0xDBFF) {
	                var leadSurrogateEnd = state.pos;
	                if (state.eat(0x5C /* \ */) && state.eat(0x75 /* u */) && this.regexp_eatFixedHexDigits(state, 4)) {
	                    var trail = state.lastIntValue;
	                    if (trail >= 0xDC00 && trail <= 0xDFFF) {
	                        state.lastIntValue = (lead - 0xD800) * 0x400 + (trail - 0xDC00) + 0x10000;
	                        return true;
	                    }
	                }
	                state.pos = leadSurrogateEnd;
	                state.lastIntValue = lead;
	            }
	            return true;
	        }
	        if (switchU && state.eat(0x7B /* { */) && this.regexp_eatHexDigits(state) && state.eat(0x7D /* } */) && isValidUnicode(state.lastIntValue)) {
	            return true;
	        }
	        if (switchU) {
	            state.raise("Invalid unicode escape");
	        }
	        state.pos = start;
	    }
	    return false;
	};
	function isValidUnicode(ch) {
	    return ch >= 0 && ch <= 0x10FFFF;
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-IdentityEscape
	pp.regexp_eatIdentityEscape = function (state) {
	    if (state.switchU) {
	        if (this.regexp_eatSyntaxCharacter(state)) {
	            return true;
	        }
	        if (state.eat(0x2F /* / */)) {
	            state.lastIntValue = 0x2F; /* / */
	            return true;
	        }
	        return false;
	    }
	    var ch = state.current();
	    if (ch !== 0x63 /* c */ && (!state.switchN || ch !== 0x6B /* k */)) {
	        state.lastIntValue = ch;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-DecimalEscape
	pp.regexp_eatDecimalEscape = function (state) {
	    state.lastIntValue = 0;
	    var ch = state.current();
	    if (ch >= 0x31 /* 1 */ && ch <= 0x39 /* 9 */) {
	        do {
	            state.lastIntValue = 10 * state.lastIntValue + (ch - 0x30 /* 0 */);
	            state.advance();
	        } while ((ch = state.current()) >= 0x30 /* 0 */ && ch <= 0x39 /* 9 */);
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-CharacterClassEscape
	pp.regexp_eatCharacterClassEscape = function (state) {
	    var ch = state.current();
	    if (isCharacterClassEscape(ch)) {
	        state.lastIntValue = -1;
	        state.advance();
	        return true;
	    }
	    if (state.switchU && this.options.ecmaVersion >= 9 && (ch === 0x50 /* P */ || ch === 0x70 /* p */)) {
	        state.lastIntValue = -1;
	        state.advance();
	        if (state.eat(0x7B /* { */) && this.regexp_eatUnicodePropertyValueExpression(state) && state.eat(0x7D /* } */)) {
	            return true;
	        }
	        state.raise("Invalid property name");
	    }
	    return false;
	};
	function isCharacterClassEscape(ch) {
	    return ch === 0x64 /* d */ || ch === 0x44 /* D */ || ch === 0x73 /* s */ || ch === 0x53 /* S */ || ch === 0x77 /* w */ || ch === 0x57 /* W */;
	}
	// UnicodePropertyValueExpression ::
	//   UnicodePropertyName `=` UnicodePropertyValue
	//   LoneUnicodePropertyNameOrValue
	pp.regexp_eatUnicodePropertyValueExpression = function (state) {
	    var start = state.pos;
	    // UnicodePropertyName `=` UnicodePropertyValue
	    if (this.regexp_eatUnicodePropertyName(state) && state.eat(0x3D /* = */)) {
	        var name_2 = state.lastStringValue;
	        if (this.regexp_eatUnicodePropertyValue(state)) {
	            var value = state.lastStringValue;
	            this.regexp_validateUnicodePropertyNameAndValue(state, name_2, value);
	            return true;
	        }
	    }
	    state.pos = start;
	    // LoneUnicodePropertyNameOrValue
	    if (this.regexp_eatLoneUnicodePropertyNameOrValue(state)) {
	        var nameOrValue = state.lastStringValue;
	        this.regexp_validateUnicodePropertyNameOrValue(state, nameOrValue);
	        return true;
	    }
	    return false;
	};
	pp.regexp_validateUnicodePropertyNameAndValue = function (state, name, value) {
	    if (!(0, util.has)(state.unicodeProperties.nonBinary, name))
	        state.raise("Invalid property name");
	    if (!state.unicodeProperties.nonBinary[name].test(value))
	        state.raise("Invalid property value");
	};
	pp.regexp_validateUnicodePropertyNameOrValue = function (state, nameOrValue) {
	    if (!state.unicodeProperties.binary.test(nameOrValue))
	        state.raise("Invalid property name");
	};
	// UnicodePropertyName ::
	//   UnicodePropertyNameCharacters
	pp.regexp_eatUnicodePropertyName = function (state) {
	    var ch = 0;
	    state.lastStringValue = "";
	    while (isUnicodePropertyNameCharacter(ch = state.current())) {
	        state.lastStringValue += codePointToString(ch);
	        state.advance();
	    }
	    return state.lastStringValue !== "";
	};
	function isUnicodePropertyNameCharacter(ch) {
	    return isControlLetter(ch) || ch === 0x5F; /* _ */
	}
	// UnicodePropertyValue ::
	//   UnicodePropertyValueCharacters
	pp.regexp_eatUnicodePropertyValue = function (state) {
	    var ch = 0;
	    state.lastStringValue = "";
	    while (isUnicodePropertyValueCharacter(ch = state.current())) {
	        state.lastStringValue += codePointToString(ch);
	        state.advance();
	    }
	    return state.lastStringValue !== "";
	};
	function isUnicodePropertyValueCharacter(ch) {
	    return isUnicodePropertyNameCharacter(ch) || isDecimalDigit(ch);
	}
	// LoneUnicodePropertyNameOrValue ::
	//   UnicodePropertyValueCharacters
	pp.regexp_eatLoneUnicodePropertyNameOrValue = function (state) {
	    return this.regexp_eatUnicodePropertyValue(state);
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-CharacterClass
	pp.regexp_eatCharacterClass = function (state) {
	    if (state.eat(0x5B /* [ */)) {
	        state.eat(0x5E /* ^ */);
	        this.regexp_classRanges(state);
	        if (state.eat(0x5D /* ] */)) {
	            return true;
	        }
	        // Unreachable since it threw "unterminated regular expression" error before.
	        state.raise("Unterminated character class");
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-ClassRanges
	// https://www.ecma-international.org/ecma-262/8.0/#prod-NonemptyClassRanges
	// https://www.ecma-international.org/ecma-262/8.0/#prod-NonemptyClassRangesNoDash
	pp.regexp_classRanges = function (state) {
	    while (this.regexp_eatClassAtom(state)) {
	        var left = state.lastIntValue;
	        if (state.eat(0x2D /* - */) && this.regexp_eatClassAtom(state)) {
	            var right = state.lastIntValue;
	            if (state.switchU && (left === -1 || right === -1)) {
	                state.raise("Invalid character class");
	            }
	            if (left !== -1 && right !== -1 && left > right) {
	                state.raise("Range out of order in character class");
	            }
	        }
	    }
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-ClassAtom
	// https://www.ecma-international.org/ecma-262/8.0/#prod-ClassAtomNoDash
	pp.regexp_eatClassAtom = function (state) {
	    var start = state.pos;
	    if (state.eat(0x5C /* \ */)) {
	        if (this.regexp_eatClassEscape(state)) {
	            return true;
	        }
	        if (state.switchU) {
	            // Make the same message as V8.
	            var ch_1 = state.current();
	            if (ch_1 === 0x63 /* c */ || isOctalDigit(ch_1)) {
	                state.raise("Invalid class escape");
	            }
	            state.raise("Invalid escape");
	        }
	        state.pos = start;
	    }
	    var ch = state.current();
	    if (ch !== 0x5D /* ] */) {
	        state.lastIntValue = ch;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ClassEscape
	pp.regexp_eatClassEscape = function (state) {
	    var start = state.pos;
	    if (state.eat(0x62 /* b */)) {
	        state.lastIntValue = 0x08; /* <BS> */
	        return true;
	    }
	    if (state.switchU && state.eat(0x2D /* - */)) {
	        state.lastIntValue = 0x2D; /* - */
	        return true;
	    }
	    if (!state.switchU && state.eat(0x63 /* c */)) {
	        if (this.regexp_eatClassControlLetter(state)) {
	            return true;
	        }
	        state.pos = start;
	    }
	    return this.regexp_eatCharacterClassEscape(state) || this.regexp_eatCharacterEscape(state);
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ClassControlLetter
	pp.regexp_eatClassControlLetter = function (state) {
	    var ch = state.current();
	    if (isDecimalDigit(ch) || ch === 0x5F /* _ */) {
	        state.lastIntValue = ch % 0x20;
	        state.advance();
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-HexEscapeSequence
	pp.regexp_eatHexEscapeSequence = function (state) {
	    var start = state.pos;
	    if (state.eat(0x78 /* x */)) {
	        if (this.regexp_eatFixedHexDigits(state, 2)) {
	            return true;
	        }
	        if (state.switchU) {
	            state.raise("Invalid escape");
	        }
	        state.pos = start;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-DecimalDigits
	pp.regexp_eatDecimalDigits = function (state) {
	    var start = state.pos;
	    var ch = 0;
	    state.lastIntValue = 0;
	    while (isDecimalDigit(ch = state.current())) {
	        state.lastIntValue = 10 * state.lastIntValue + (ch - 0x30 /* 0 */);
	        state.advance();
	    }
	    return state.pos !== start;
	};
	function isDecimalDigit(ch) {
	    return ch >= 0x30 /* 0 */ && ch <= 0x39; /* 9 */
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-HexDigits
	pp.regexp_eatHexDigits = function (state) {
	    var start = state.pos;
	    var ch = 0;
	    state.lastIntValue = 0;
	    while (isHexDigit(ch = state.current())) {
	        state.lastIntValue = 16 * state.lastIntValue + hexToInt(ch);
	        state.advance();
	    }
	    return state.pos !== start;
	};
	function isHexDigit(ch) {
	    return ch >= 0x30 /* 0 */ && ch <= 0x39 /* 9 */ || ch >= 0x41 /* A */ && ch <= 0x46 /* F */ || ch >= 0x61 /* a */ && ch <= 0x66 /* f */;
	}
	function hexToInt(ch) {
	    if (ch >= 0x41 /* A */ && ch <= 0x46 /* F */) {
	        return 10 + (ch - 0x41 /* A */);
	    }
	    if (ch >= 0x61 /* a */ && ch <= 0x66 /* f */) {
	        return 10 + (ch - 0x61 /* a */);
	    }
	    return ch - 0x30; /* 0 */
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-LegacyOctalEscapeSequence
	// Allows only 0-377(octal) i.e. 0-255(decimal).
	pp.regexp_eatLegacyOctalEscapeSequence = function (state) {
	    if (this.regexp_eatOctalDigit(state)) {
	        var n1 = state.lastIntValue;
	        if (this.regexp_eatOctalDigit(state)) {
	            var n2 = state.lastIntValue;
	            if (n1 <= 3 && this.regexp_eatOctalDigit(state)) {
	                state.lastIntValue = n1 * 64 + n2 * 8 + state.lastIntValue;
	            }
	            else {
	                state.lastIntValue = n1 * 8 + n2;
	            }
	        }
	        else {
	            state.lastIntValue = n1;
	        }
	        return true;
	    }
	    return false;
	};
	// https://www.ecma-international.org/ecma-262/8.0/#prod-OctalDigit
	pp.regexp_eatOctalDigit = function (state) {
	    var ch = state.current();
	    if (isOctalDigit(ch)) {
	        state.lastIntValue = ch - 0x30; /* 0 */
	        state.advance();
	        return true;
	    }
	    state.lastIntValue = 0;
	    return false;
	};
	function isOctalDigit(ch) {
	    return ch >= 0x30 /* 0 */ && ch <= 0x37; /* 7 */
	}
	// https://www.ecma-international.org/ecma-262/8.0/#prod-Hex4Digits
	// https://www.ecma-international.org/ecma-262/8.0/#prod-HexDigit
	// And HexDigit HexDigit in https://www.ecma-international.org/ecma-262/8.0/#prod-HexEscapeSequence
	pp.regexp_eatFixedHexDigits = function (state, length) {
	    var start = state.pos;
	    state.lastIntValue = 0;
	    for (var i = 0; i < length; ++i) {
	        var ch = state.current();
	        if (!isHexDigit(ch)) {
	            state.pos = start;
	            return false;
	        }
	        state.lastIntValue = 16 * state.lastIntValue + hexToInt(ch);
	        state.advance();
	    }
	    return true;
	};
	});

	var tokenize = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.Token = undefined;






	// Object type used to represent tokens. Note that normally, tokens
	// simply exist as properties on the parser object. This is only
	// used for the onToken callback and the external tokenizer.
	var Token = /** @class */ (function () {
	    function Token(p) {
	        this.type = p.type;
	        this.value = p.value;
	        this.start = p.start;
	        this.end = p.end;
	        if (p.options.locations)
	            this.loc = new locutil.SourceLocation(p, p.startLoc, p.endLoc);
	        if (p.options.ranges)
	            this.range = [p.start, p.end];
	    }
	    return Token;
	}());
	exports.Token = Token; // ## Tokenizer
	var pp = state.Parser.prototype;
	// Move to the next token
	pp.next = function (ignoreEscapeSequenceInKeyword) {
	    if (!ignoreEscapeSequenceInKeyword && this.type.keyword && this.containsEsc)
	        this.raiseRecoverable(this.start, "Escape sequence in keyword " + this.type.keyword);
	    if (this.options.onToken)
	        this.options.onToken(new Token(this));
	    this.lastTokEnd = this.end;
	    this.lastTokStart = this.start;
	    this.lastTokEndLoc = this.endLoc;
	    this.lastTokStartLoc = this.startLoc;
	    this.nextToken();
	};
	pp.getToken = function () {
	    this.next();
	    return new Token(this);
	};
	// If we're in an ES6 environment, make parsers iterable
	if (typeof Symbol !== "undefined")
	    pp[Symbol.iterator] = function () {
	        var _this = this;
	        return {
	            next: function () {
	                var token = _this.getToken();
	                return {
	                    done: token.type === tokentype.types.eof,
	                    value: token
	                };
	            }
	        };
	    };
	// Toggle strict mode. Re-reads the next number or string to please
	// pedantic tests (`"use strict"; 010;` should fail).
	pp.curContext = function () {
	    return this.context[this.context.length - 1];
	};
	// Read a single token, updating the parser object's token-related
	// properties.
	pp.nextToken = function () {
	    var curContext = this.curContext();
	    if (!curContext || !curContext.preserveSpace)
	        this.skipSpace();
	    this.start = this.pos;
	    if (this.options.locations)
	        this.startLoc = this.curPosition();
	    if (this.pos >= this.input.length)
	        return this.finishToken(tokentype.types.eof);
	    if (curContext.override)
	        return curContext.override(this);
	    else
	        this.readToken(this.fullCharCodeAtPos());
	};
	pp.readToken = function (code) {
	    // Identifier or keyword. '\uXXXX' sequences are allowed in
	    // identifiers, so '\' also dispatches to that.
	    if ((0, identifier.isIdentifierStart)(code, this.options.ecmaVersion >= 6) || code === 92 /* '\' */)
	        return this.readWord();
	    return this.getTokenFromCode(code);
	};
	pp.fullCharCodeAtPos = function () {
	    var code = this.input.charCodeAt(this.pos);
	    if (code <= 0xd7ff || code >= 0xe000)
	        return code;
	    var next = this.input.charCodeAt(this.pos + 1);
	    return (code << 10) + next - 0x35fdc00;
	};
	pp.skipBlockComment = function () {
	    var startLoc = this.options.onComment && this.curPosition();
	    var start = this.pos, end = this.input.indexOf("*/", this.pos += 2);
	    if (end === -1)
	        this.raise(this.pos - 2, "Unterminated comment");
	    this.pos = end + 2;
	    if (this.options.locations) {
	        whitespace.lineBreakG.lastIndex = start;
	        var match = void 0;
	        while ((match = whitespace.lineBreakG.exec(this.input)) && match.index < this.pos) {
	            ++this.curLine;
	            this.lineStart = match.index + match[0].length;
	        }
	    }
	    if (this.options.onComment)
	        this.options.onComment(true, this.input.slice(start + 2, end), start, this.pos, startLoc, this.curPosition());
	};
	pp.skipLineComment = function (startSkip) {
	    var start = this.pos;
	    var startLoc = this.options.onComment && this.curPosition();
	    var ch = this.input.charCodeAt(this.pos += startSkip);
	    while (this.pos < this.input.length && !(0, whitespace.isNewLine)(ch)) {
	        ch = this.input.charCodeAt(++this.pos);
	    }
	    if (this.options.onComment)
	        this.options.onComment(false, this.input.slice(start + startSkip, this.pos), start, this.pos, startLoc, this.curPosition());
	};
	// Called at the start of the parse and after every token. Skips
	// whitespace and comments, and.
	pp.skipSpace = function () {
	    loop: while (this.pos < this.input.length) {
	        var ch = this.input.charCodeAt(this.pos);
	        switch (ch) {
	            case 32:
	            case 160:
	                // ' '
	                ++this.pos;
	                break;
	            case 13:
	                if (this.input.charCodeAt(this.pos + 1) === 10) {
	                    ++this.pos;
	                }
	            case 10:
	            case 8232:
	            case 8233:
	                ++this.pos;
	                if (this.options.locations) {
	                    ++this.curLine;
	                    this.lineStart = this.pos;
	                }
	                break;
	            case 47:
	                // '/'
	                switch (this.input.charCodeAt(this.pos + 1)) {
	                    case 42:
	                        // '*'
	                        this.skipBlockComment();
	                        break;
	                    case 47:
	                        this.skipLineComment(2);
	                        break;
	                    default:
	                        break loop;
	                }
	                break;
	            default:
	                if (ch > 8 && ch < 14 || ch >= 5760 && whitespace.nonASCIIwhitespace.test(String.fromCharCode(ch))) {
	                    ++this.pos;
	                }
	                else {
	                    break loop;
	                }
	        }
	    }
	};
	// Called at the end of every token. Sets `end`, `val`, and
	// maintains `context` and `exprAllowed`, and skips the space after
	// the token, so that the next one's `start` will point at the
	// right position.
	pp.finishToken = function (type, val) {
	    this.end = this.pos;
	    if (this.options.locations)
	        this.endLoc = this.curPosition();
	    var prevType = this.type;
	    this.type = type;
	    this.value = val;
	    this.updateContext(prevType);
	};
	// ### Token reading
	// This is the function that is called to fetch the next token. It
	// is somewhat obscure, because it works in character codes rather
	// than characters, and because operator parsing has been inlined
	// into it.
	//
	// All in the name of speed.
	//
	pp.readToken_dot = function () {
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (next >= 48 && next <= 57)
	        return this.readNumber(true);
	    var next2 = this.input.charCodeAt(this.pos + 2);
	    if (this.options.ecmaVersion >= 6 && next === 46 && next2 === 46) {
	        // 46 = dot '.'
	        this.pos += 3;
	        return this.finishToken(tokentype.types.ellipsis);
	    }
	    else {
	        ++this.pos;
	        return this.finishToken(tokentype.types.dot);
	    }
	};
	pp.readToken_slash = function () {
	    // '/'
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (this.exprAllowed) {
	        ++this.pos;
	        return this.readRegexp();
	    }
	    if (next === 61)
	        return this.finishOp(tokentype.types.assign, 2);
	    return this.finishOp(tokentype.types.slash, 1);
	};
	pp.readToken_mult_modulo_exp = function (code) {
	    // '%*'
	    var next = this.input.charCodeAt(this.pos + 1);
	    var size = 1;
	    var tokentype$1 = code === 42 ? tokentype.types.star : tokentype.types.modulo;
	    // exponentiation operator ** and **=
	    if (this.options.ecmaVersion >= 7 && code === 42 && next === 42) {
	        ++size;
	        tokentype$1 = tokentype.types.starstar;
	        next = this.input.charCodeAt(this.pos + 2);
	    }
	    if (next === 61)
	        return this.finishOp(tokentype.types.assign, size + 1);
	    return this.finishOp(tokentype$1, size);
	};
	pp.readToken_pipe_amp = function (code) {
	    // '|&'
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (next === code) {
	        if (this.options.ecmaVersion >= 12) {
	            var next2 = this.input.charCodeAt(this.pos + 2);
	            if (next2 === 61)
	                return this.finishOp(tokentype.types.assign, 3);
	        }
	        return this.finishOp(code === 124 ? tokentype.types.logicalOR : tokentype.types.logicalAND, 2);
	    }
	    if (next === 61)
	        return this.finishOp(tokentype.types.assign, 2);
	    return this.finishOp(code === 124 ? tokentype.types.bitwiseOR : tokentype.types.bitwiseAND, 1);
	};
	pp.readToken_caret = function () {
	    // '^'
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (next === 61)
	        return this.finishOp(tokentype.types.assign, 2);
	    return this.finishOp(tokentype.types.bitwiseXOR, 1);
	};
	pp.readToken_plus_min = function (code) {
	    // '+-'
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (next === code) {
	        if (next === 45 && !this.inModule && this.input.charCodeAt(this.pos + 2) === 62 && (this.lastTokEnd === 0 || whitespace.lineBreak.test(this.input.slice(this.lastTokEnd, this.pos)))) {
	            // A `-->` line comment
	            this.skipLineComment(3);
	            this.skipSpace();
	            return this.nextToken();
	        }
	        return this.finishOp(tokentype.types.incDec, 2);
	    }
	    if (next === 61)
	        return this.finishOp(tokentype.types.assign, 2);
	    return this.finishOp(tokentype.types.plusMin, 1);
	};
	pp.readToken_lt_gt = function (code) {
	    // '<>'
	    var next = this.input.charCodeAt(this.pos + 1);
	    var size = 1;
	    if (next === code) {
	        size = code === 62 && this.input.charCodeAt(this.pos + 2) === 62 ? 3 : 2;
	        if (this.input.charCodeAt(this.pos + size) === 61)
	            return this.finishOp(tokentype.types.assign, size + 1);
	        return this.finishOp(tokentype.types.bitShift, size);
	    }
	    if (next === 33 && code === 60 && !this.inModule && this.input.charCodeAt(this.pos + 2) === 45 && this.input.charCodeAt(this.pos + 3) === 45) {
	        // `<!--`, an XML-style comment that should be interpreted as a line comment
	        this.skipLineComment(4);
	        this.skipSpace();
	        return this.nextToken();
	    }
	    if (next === 61)
	        size = 2;
	    return this.finishOp(tokentype.types.relational, size);
	};
	pp.readToken_eq_excl = function (code) {
	    // '=!'
	    var next = this.input.charCodeAt(this.pos + 1);
	    if (next === 61)
	        return this.finishOp(tokentype.types.equality, this.input.charCodeAt(this.pos + 2) === 61 ? 3 : 2);
	    if (code === 61 && next === 62 && this.options.ecmaVersion >= 6) {
	        // '=>'
	        this.pos += 2;
	        return this.finishToken(tokentype.types.arrow);
	    }
	    return this.finishOp(code === 61 ? tokentype.types.eq : tokentype.types.prefix, 1);
	};
	pp.readToken_question = function () {
	    // '?'
	    var ecmaVersion = this.options.ecmaVersion;
	    if (ecmaVersion >= 11) {
	        var next = this.input.charCodeAt(this.pos + 1);
	        if (next === 46) {
	            var next2 = this.input.charCodeAt(this.pos + 2);
	            if (next2 < 48 || next2 > 57)
	                return this.finishOp(tokentype.types.questionDot, 2);
	        }
	        if (next === 63) {
	            if (ecmaVersion >= 12) {
	                var next2 = this.input.charCodeAt(this.pos + 2);
	                if (next2 === 61)
	                    return this.finishOp(tokentype.types.assign, 3);
	            }
	            return this.finishOp(tokentype.types.coalesce, 2);
	        }
	    }
	    return this.finishOp(tokentype.types.question, 1);
	};
	pp.getTokenFromCode = function (code) {
	    switch (code) {
	        // The interpretation of a dot depends on whether it is followed
	        // by a digit or another two dots.
	        case 46:
	            // '.'
	            return this.readToken_dot();
	        // Punctuation tokens.
	        case 40:
	            ++this.pos;
	            return this.finishToken(tokentype.types.parenL);
	        case 41:
	            ++this.pos;
	            return this.finishToken(tokentype.types.parenR);
	        case 59:
	            ++this.pos;
	            return this.finishToken(tokentype.types.semi);
	        case 44:
	            ++this.pos;
	            return this.finishToken(tokentype.types.comma);
	        case 91:
	            ++this.pos;
	            return this.finishToken(tokentype.types.bracketL);
	        case 93:
	            ++this.pos;
	            return this.finishToken(tokentype.types.bracketR);
	        case 123:
	            ++this.pos;
	            return this.finishToken(tokentype.types.braceL);
	        case 125:
	            ++this.pos;
	            return this.finishToken(tokentype.types.braceR);
	        case 58:
	            ++this.pos;
	            return this.finishToken(tokentype.types.colon);
	        case 96:
	            // '`'
	            if (this.options.ecmaVersion < 6)
	                break;
	            ++this.pos;
	            return this.finishToken(tokentype.types.backQuote);
	        case 48:
	            // '0'
	            var next = this.input.charCodeAt(this.pos + 1);
	            if (next === 120 || next === 88)
	                return this.readRadixNumber(16); // '0x', '0X' - hex number
	            if (this.options.ecmaVersion >= 6) {
	                if (next === 111 || next === 79)
	                    return this.readRadixNumber(8); // '0o', '0O' - octal number
	                if (next === 98 || next === 66)
	                    return this.readRadixNumber(2); // '0b', '0B' - binary number
	            }
	        // Anything else beginning with a digit is an integer, octal
	        // number, or float.
	        case 49:
	        case 50:
	        case 51:
	        case 52:
	        case 53:
	        case 54:
	        case 55:
	        case 56:
	        case 57:
	            // 1-9
	            return this.readNumber(false);
	        // Quotes produce strings.
	        case 34:
	        case 39:
	            // '"', "'"
	            return this.readString(code);
	        // Operators are parsed inline in tiny state machines. '=' (61) is
	        // often referred to. `finishOp` simply skips the amount of
	        // characters it is given as second argument, and returns a token
	        // of the type given by its first argument.
	        case 47:
	            // '/'
	            return this.readToken_slash();
	        case 37:
	        case 42:
	            // '%*'
	            return this.readToken_mult_modulo_exp(code);
	        case 124:
	        case 38:
	            // '|&'
	            return this.readToken_pipe_amp(code);
	        case 94:
	            // '^'
	            return this.readToken_caret();
	        case 43:
	        case 45:
	            // '+-'
	            return this.readToken_plus_min(code);
	        case 60:
	        case 62:
	            // '<>'
	            return this.readToken_lt_gt(code);
	        case 61:
	        case 33:
	            // '=!'
	            return this.readToken_eq_excl(code);
	        case 63:
	            // '?'
	            return this.readToken_question();
	        case 126:
	            // '~'
	            return this.finishOp(tokentype.types.prefix, 1);
	    }
	    this.raise(this.pos, "Unexpected character '" + codePointToString(code) + "'");
	};
	pp.finishOp = function (type, size) {
	    var str = this.input.slice(this.pos, this.pos + size);
	    this.pos += size;
	    return this.finishToken(type, str);
	};
	pp.readRegexp = function () {
	    var escaped, inClass, start = this.pos;
	    for (;;) {
	        if (this.pos >= this.input.length)
	            this.raise(start, "Unterminated regular expression");
	        var ch = this.input.charAt(this.pos);
	        if (whitespace.lineBreak.test(ch))
	            this.raise(start, "Unterminated regular expression");
	        if (!escaped) {
	            if (ch === "[")
	                inClass = true;
	            else if (ch === "]" && inClass)
	                inClass = false;
	            else if (ch === "/" && !inClass)
	                break;
	            escaped = ch === "\\";
	        }
	        else
	            escaped = false;
	        ++this.pos;
	    }
	    var pattern = this.input.slice(start, this.pos);
	    ++this.pos;
	    var flagsStart = this.pos;
	    var flags = this.readWord1();
	    if (this.containsEsc)
	        this.unexpected(flagsStart);
	    // Validate pattern
	    var state = this.regexpState || (this.regexpState = new regexp.RegExpValidationState(this));
	    state.reset(start, pattern, flags);
	    this.validateRegExpFlags(state);
	    this.validateRegExpPattern(state);
	    // Create Literal#value property value.
	    var value = null;
	    try {
	        value = new RegExp(pattern, flags);
	    }
	    catch (e) {
	        // ESTree requires null if it failed to instantiate RegExp object.
	        // https://github.com/estree/estree/blob/a27003adf4fd7bfad44de9cef372a2eacd527b1c/es5.md#regexpliteral
	    }
	    return this.finishToken(tokentype.types.regexp, { pattern: pattern, flags: flags, value: value });
	};
	// Read an integer in the given radix. Return null if zero digits
	// were read, the integer value otherwise. When `len` is given, this
	// will return `null` unless the integer has exactly `len` digits.
	pp.readInt = function (radix, len, maybeLegacyOctalNumericLiteral) {
	    // `len` is used for character escape sequences. In that case, disallow separators.
	    var allowSeparators = this.options.ecmaVersion >= 12 && len === undefined;
	    // `maybeLegacyOctalNumericLiteral` is true if it doesn't have prefix (0x,0o,0b)
	    // and isn't fraction part nor exponent part. In that case, if the first digit
	    // is zero then disallow separators.
	    var isLegacyOctalNumericLiteral = maybeLegacyOctalNumericLiteral && this.input.charCodeAt(this.pos) === 48;
	    var start = this.pos, total = 0, lastCode = 0;
	    for (var i = 0, e = len == null ? Infinity : len; i < e; ++i, ++this.pos) {
	        var code = this.input.charCodeAt(this.pos), val = void 0;
	        if (allowSeparators && code === 95) {
	            if (isLegacyOctalNumericLiteral)
	                this.raiseRecoverable(this.pos, "Numeric separator is not allowed in legacy octal numeric literals");
	            if (lastCode === 95)
	                this.raiseRecoverable(this.pos, "Numeric separator must be exactly one underscore");
	            if (i === 0)
	                this.raiseRecoverable(this.pos, "Numeric separator is not allowed at the first of digits");
	            lastCode = code;
	            continue;
	        }
	        if (code >= 97)
	            val = code - 97 + 10; // a
	        else if (code >= 65)
	            val = code - 65 + 10; // A
	        else if (code >= 48 && code <= 57)
	            val = code - 48; // 0-9
	        else
	            val = Infinity;
	        if (val >= radix)
	            break;
	        lastCode = code;
	        total = total * radix + val;
	    }
	    if (allowSeparators && lastCode === 95)
	        this.raiseRecoverable(this.pos - 1, "Numeric separator is not allowed at the last of digits");
	    if (this.pos === start || len != null && this.pos - start !== len)
	        return null;
	    return total;
	};
	function stringToNumber(str, isLegacyOctalNumericLiteral) {
	    if (isLegacyOctalNumericLiteral) {
	        return parseInt(str, 8);
	    }
	    // `parseFloat(value)` stops parsing at the first numeric separator then returns a wrong value.
	    return parseFloat(str.replace(/_/g, ""));
	}
	function stringToBigInt(str) {
	    if (typeof BigInt !== "function") {
	        return null;
	    }
	    // `BigInt(value)` throws syntax error if the string contains numeric separators.
	    return BigInt(str.replace(/_/g, ""));
	}
	pp.readRadixNumber = function (radix) {
	    var start = this.pos;
	    this.pos += 2; // 0x
	    var val = this.readInt(radix);
	    if (val == null)
	        this.raise(this.start + 2, "Expected number in radix " + radix);
	    if (this.options.ecmaVersion >= 11 && this.input.charCodeAt(this.pos) === 110) {
	        val = stringToBigInt(this.input.slice(start, this.pos));
	        ++this.pos;
	    }
	    else if ((0, identifier.isIdentifierStart)(this.fullCharCodeAtPos()))
	        this.raise(this.pos, "Identifier directly after number");
	    return this.finishToken(tokentype.types.num, val);
	};
	// Read an integer, octal integer, or floating-point number.
	pp.readNumber = function (startsWithDot) {
	    var start = this.pos;
	    if (!startsWithDot && this.readInt(10, undefined, true) === null)
	        this.raise(start, "Invalid number");
	    var octal = this.pos - start >= 2 && this.input.charCodeAt(start) === 48;
	    if (octal && this.strict)
	        this.raise(start, "Invalid number");
	    var next = this.input.charCodeAt(this.pos);
	    if (!octal && !startsWithDot && this.options.ecmaVersion >= 11 && next === 110) {
	        var val_1 = stringToBigInt(this.input.slice(start, this.pos));
	        ++this.pos;
	        if ((0, identifier.isIdentifierStart)(this.fullCharCodeAtPos()))
	            this.raise(this.pos, "Identifier directly after number");
	        return this.finishToken(tokentype.types.num, val_1);
	    }
	    if (octal && /[89]/.test(this.input.slice(start, this.pos)))
	        octal = false;
	    if (next === 46 && !octal) {
	        // '.'
	        ++this.pos;
	        this.readInt(10);
	        next = this.input.charCodeAt(this.pos);
	    }
	    if ((next === 69 || next === 101) && !octal) {
	        // 'eE'
	        next = this.input.charCodeAt(++this.pos);
	        if (next === 43 || next === 45)
	            ++this.pos; // '+-'
	        if (this.readInt(10) === null)
	            this.raise(start, "Invalid number");
	    }
	    if ((0, identifier.isIdentifierStart)(this.fullCharCodeAtPos()))
	        this.raise(this.pos, "Identifier directly after number");
	    var val = stringToNumber(this.input.slice(start, this.pos), octal);
	    return this.finishToken(tokentype.types.num, val);
	};
	// Read a string value, interpreting backslash-escapes.
	pp.readCodePoint = function () {
	    var ch = this.input.charCodeAt(this.pos), code;
	    if (ch === 123) {
	        // '{'
	        if (this.options.ecmaVersion < 6)
	            this.unexpected();
	        var codePos = ++this.pos;
	        code = this.readHexChar(this.input.indexOf("}", this.pos) - this.pos);
	        ++this.pos;
	        if (code > 0x10FFFF)
	            this.invalidStringToken(codePos, "Code point out of bounds");
	    }
	    else {
	        code = this.readHexChar(4);
	    }
	    return code;
	};
	function codePointToString(code) {
	    // UTF-16 Decoding
	    if (code <= 0xFFFF)
	        return String.fromCharCode(code);
	    code -= 0x10000;
	    return String.fromCharCode((code >> 10) + 0xD800, (code & 1023) + 0xDC00);
	}
	pp.readString = function (quote) {
	    var out = "", chunkStart = ++this.pos;
	    for (;;) {
	        if (this.pos >= this.input.length)
	            this.raise(this.start, "Unterminated string constant");
	        var ch = this.input.charCodeAt(this.pos);
	        if (ch === quote)
	            break;
	        if (ch === 92) {
	            // '\'
	            out += this.input.slice(chunkStart, this.pos);
	            out += this.readEscapedChar(false);
	            chunkStart = this.pos;
	        }
	        else {
	            if ((0, whitespace.isNewLine)(ch, this.options.ecmaVersion >= 10))
	                this.raise(this.start, "Unterminated string constant");
	            ++this.pos;
	        }
	    }
	    out += this.input.slice(chunkStart, this.pos++);
	    return this.finishToken(tokentype.types.string, out);
	};
	// Reads template string tokens.
	var INVALID_TEMPLATE_ESCAPE_ERROR = {};
	pp.tryReadTemplateToken = function () {
	    this.inTemplateElement = true;
	    try {
	        this.readTmplToken();
	    }
	    catch (err) {
	        if (err === INVALID_TEMPLATE_ESCAPE_ERROR) {
	            this.readInvalidTemplateToken();
	        }
	        else {
	            throw err;
	        }
	    }
	    this.inTemplateElement = false;
	};
	pp.invalidStringToken = function (position, message) {
	    if (this.inTemplateElement && this.options.ecmaVersion >= 9) {
	        throw INVALID_TEMPLATE_ESCAPE_ERROR;
	    }
	    else {
	        this.raise(position, message);
	    }
	};
	pp.readTmplToken = function () {
	    var out = "", chunkStart = this.pos;
	    for (;;) {
	        if (this.pos >= this.input.length)
	            this.raise(this.start, "Unterminated template");
	        var ch = this.input.charCodeAt(this.pos);
	        if (ch === 96 || ch === 36 && this.input.charCodeAt(this.pos + 1) === 123) {
	            // '`', '${'
	            if (this.pos === this.start && (this.type === tokentype.types.template || this.type === tokentype.types.invalidTemplate)) {
	                if (ch === 36) {
	                    this.pos += 2;
	                    return this.finishToken(tokentype.types.dollarBraceL);
	                }
	                else {
	                    ++this.pos;
	                    return this.finishToken(tokentype.types.backQuote);
	                }
	            }
	            out += this.input.slice(chunkStart, this.pos);
	            return this.finishToken(tokentype.types.template, out);
	        }
	        if (ch === 92) {
	            // '\'
	            out += this.input.slice(chunkStart, this.pos);
	            out += this.readEscapedChar(true);
	            chunkStart = this.pos;
	        }
	        else if ((0, whitespace.isNewLine)(ch)) {
	            out += this.input.slice(chunkStart, this.pos);
	            ++this.pos;
	            switch (ch) {
	                case 13:
	                    if (this.input.charCodeAt(this.pos) === 10)
	                        ++this.pos;
	                case 10:
	                    out += "\n";
	                    break;
	                default:
	                    out += String.fromCharCode(ch);
	                    break;
	            }
	            if (this.options.locations) {
	                ++this.curLine;
	                this.lineStart = this.pos;
	            }
	            chunkStart = this.pos;
	        }
	        else {
	            ++this.pos;
	        }
	    }
	};
	// Reads a template token to search for the end, without validating any escape sequences
	pp.readInvalidTemplateToken = function () {
	    for (; this.pos < this.input.length; this.pos++) {
	        switch (this.input[this.pos]) {
	            case "\\":
	                ++this.pos;
	                break;
	            case "$":
	                if (this.input[this.pos + 1] !== "{") {
	                    break;
	                }
	            // falls through
	            case "`":
	                return this.finishToken(tokentype.types.invalidTemplate, this.input.slice(this.start, this.pos));
	            // no default
	        }
	    }
	    this.raise(this.start, "Unterminated template");
	};
	// Used to read escaped characters
	pp.readEscapedChar = function (inTemplate) {
	    var ch = this.input.charCodeAt(++this.pos);
	    ++this.pos;
	    switch (ch) {
	        case 110:
	            return "\n"; // 'n' -> '\n'
	        case 114:
	            return "\r"; // 'r' -> '\r'
	        case 120:
	            return String.fromCharCode(this.readHexChar(2)); // 'x'
	        case 117:
	            return codePointToString(this.readCodePoint()); // 'u'
	        case 116:
	            return "\t"; // 't' -> '\t'
	        case 98:
	            return "\b"; // 'b' -> '\b'
	        case 118:
	            return "\u000b"; // 'v' -> '\u000b'
	        case 102:
	            return "\f"; // 'f' -> '\f'
	        case 13:
	            if (this.input.charCodeAt(this.pos) === 10)
	                ++this.pos; // '\r\n'
	        case 10:
	            // ' \n'
	            if (this.options.locations) {
	                this.lineStart = this.pos;
	                ++this.curLine;
	            }
	            return "";
	        case 56:
	        case 57:
	            if (this.strict) {
	                this.invalidStringToken(this.pos - 1, "Invalid escape sequence");
	            }
	            if (inTemplate) {
	                var codePos = this.pos - 1;
	                this.invalidStringToken(codePos, "Invalid escape sequence in template string");
	                return null;
	            }
	        default:
	            if (ch >= 48 && ch <= 55) {
	                var octalStr = this.input.substr(this.pos - 1, 3).match(/^[0-7]+/)[0];
	                var octal = parseInt(octalStr, 8);
	                if (octal > 255) {
	                    octalStr = octalStr.slice(0, -1);
	                    octal = parseInt(octalStr, 8);
	                }
	                this.pos += octalStr.length - 1;
	                ch = this.input.charCodeAt(this.pos);
	                if ((octalStr !== "0" || ch === 56 || ch === 57) && (this.strict || inTemplate)) {
	                    this.invalidStringToken(this.pos - 1 - octalStr.length, inTemplate ? "Octal literal in template string" : "Octal literal in strict mode");
	                }
	                return String.fromCharCode(octal);
	            }
	            if ((0, whitespace.isNewLine)(ch)) {
	                // Unicode new line characters after \ get removed from output in both
	                // template literals and strings
	                return "";
	            }
	            return String.fromCharCode(ch);
	    }
	};
	// Used to read character escape sequences ('\x', '\u', '\U').
	pp.readHexChar = function (len) {
	    var codePos = this.pos;
	    var n = this.readInt(16, len);
	    if (n === null)
	        this.invalidStringToken(codePos, "Bad character escape sequence");
	    return n;
	};
	// Read an identifier, and return it as a string. Sets `this.containsEsc`
	// to whether the word contained a '\u' escape.
	//
	// Incrementally adds only escaped chars, adding other chunks as-is
	// as a micro-optimization.
	pp.readWord1 = function () {
	    this.containsEsc = false;
	    var word = "", first = true, chunkStart = this.pos;
	    var astral = this.options.ecmaVersion >= 6;
	    while (this.pos < this.input.length) {
	        var ch = this.fullCharCodeAtPos();
	        if ((0, identifier.isIdentifierChar)(ch, astral)) {
	            this.pos += ch <= 0xffff ? 1 : 2;
	        }
	        else if (ch === 92) {
	            // "\"
	            this.containsEsc = true;
	            word += this.input.slice(chunkStart, this.pos);
	            var escStart = this.pos;
	            if (this.input.charCodeAt(++this.pos) !== 117) // "u"
	                this.invalidStringToken(this.pos, "Expecting Unicode escape sequence \\uXXXX");
	            ++this.pos;
	            var esc = this.readCodePoint();
	            if (!(first ? identifier.isIdentifierStart : identifier.isIdentifierChar)(esc, astral))
	                this.invalidStringToken(escStart, "Invalid Unicode escape");
	            word += this.input.substr(this.pos - 6, 6);
	            chunkStart = this.pos;
	        }
	        else {
	            break;
	        }
	        first = false;
	    }
	    return word + this.input.slice(chunkStart, this.pos);
	};
	// Read an identifier or keyword token. Will check for reserved
	// words when necessary.
	pp.readWord = function () {
	    var word = this.readWord1();
	    var type = tokentype.types.name;
	    if (this.keywords.test(word)) {
	        type = tokentype.keywords[word];
	    }
	    return this.finishToken(type, word);
	};
	});

	var lib = createCommonjsModule(function (module, exports) {
	exports.__esModule = true;
	exports.nonASCIIwhitespace = exports.lineBreakG = exports.lineBreak = exports.isNewLine = exports.Token = exports.isIdentifierStart = exports.isIdentifierChar = exports.tokContexts = exports.TokContext = exports.keywordTypes = exports.tokTypes = exports.TokenType = exports.Node = exports.getLineInfo = exports.SourceLocation = exports.Position = exports.defaultOptions = exports.Parser = exports.version = undefined;
	exports.parse = parse;
	exports.parseExpressionAt = parseExpressionAt;
	exports.tokenizer = tokenizer;















	var version = exports.version = "8.1.0"; // Acorn is a tiny, fast JavaScript parser written in JavaScript.
	//
	// Acorn was written by Marijn Haverbeke, Ingvar Stepanyan, and
	// various contributors and released under an MIT license.
	//
	// Git repositories for Acorn are available at
	//
	//     http://marijnhaverbeke.nl/git/acorn
	//     https://github.com/acornjs/acorn.git
	//
	// Please use the [github bug tracker][ghbt] to report issues.
	//
	// [ghbt]: https://github.com/acornjs/acorn/issues
	//
	// [walk]: util/walk.js
	exports.Parser = state.Parser;
	exports.defaultOptions = options.defaultOptions;
	exports.Position = locutil.Position;
	exports.SourceLocation = locutil.SourceLocation;
	exports.getLineInfo = locutil.getLineInfo;
	exports.Node = node.Node;
	exports.TokenType = tokentype.TokenType;
	exports.tokTypes = tokentype.types;
	exports.keywordTypes = tokentype.keywords;
	exports.TokContext = tokencontext.TokContext;
	exports.tokContexts = tokencontext.types;
	exports.isIdentifierChar = identifier.isIdentifierChar;
	exports.isIdentifierStart = identifier.isIdentifierStart;
	exports.Token = tokenize.Token;
	exports.isNewLine = whitespace.isNewLine;
	exports.lineBreak = whitespace.lineBreak;
	exports.lineBreakG = whitespace.lineBreakG;
	exports.nonASCIIwhitespace = whitespace.nonASCIIwhitespace;
	state.Parser.acorn = {
	    Parser: state.Parser,
	    version: version,
	    defaultOptions: options.defaultOptions,
	    Position: locutil.Position,
	    SourceLocation: locutil.SourceLocation,
	    getLineInfo: locutil.getLineInfo,
	    Node: node.Node,
	    TokenType: tokentype.TokenType,
	    tokTypes: tokentype.types,
	    keywordTypes: tokentype.keywords,
	    TokContext: tokencontext.TokContext,
	    tokContexts: tokencontext.types,
	    isIdentifierChar: identifier.isIdentifierChar,
	    isIdentifierStart: identifier.isIdentifierStart,
	    Token: tokenize.Token,
	    isNewLine: whitespace.isNewLine,
	    lineBreak: whitespace.lineBreak,
	    lineBreakG: whitespace.lineBreakG,
	    nonASCIIwhitespace: whitespace.nonASCIIwhitespace
	    // The main exported interface (under `self.acorn` when in the
	    // browser) is a `parse` function that takes a code string and
	    // returns an abstract syntax tree as specified by [Mozilla parser
	    // API][api].
	    //
	    // [api]: https://developer.mozilla.org/en-US/docs/SpiderMonkey/Parser_API
	};
	function parse(input, options) {
	    return state.Parser.parse(input, options);
	}
	// This function tries to parse a single expression at a given
	// offset in a string. Useful for parsing mixed-language formats
	// that embed JavaScript expressions.
	function parseExpressionAt(input, pos, options) {
	    return state.Parser.parseExpressionAt(input, pos, options);
	}
	// Acorn is organized as a tokenizer and a recursive-descent parser.
	// The `tokenizer` export provides an interface to the tokenizer.
	function tokenizer(input, options) {
	    return state.Parser.tokenizer(input, options);
	}
	});

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	var BOM_RE = /^(\xEF\xBB\xBF|\xFE\xFF|\xFF\xFE|\x00\x00\xFE\xFF|\xFF\xFE\x00\x00|\x2B\x2F\x76\x38|\x2B\x2F\x76\x39|\x2B\x2F\x76\x2B|\x2B\x2F\x76\x2F|\xF7\x64\x4C|\xDD\x73\x66\x73|\x0E\xFE\xFF|\xFB\xEE\x28|\x84\x31\x95\x33)/;
	function getBOM (str) {
	    var match = str.match(BOM_RE);
	    return match ? match[0] : null;
	}

	var HTML_COMMENT_RE = /(^|\n)\s*<!--[^\n]*(\n|$)/g;
	var OBJECT_RE = /^\s*\{.*\}\s*$/;
	var TRAILING_SEMICOLON_RE = /;\s*$/;
	var OBJECT_WRAPPER_RE = /^\s*\((.*)\);\s*$/;
	var SOURCEMAP_RE = /(?:\/\/[@#][ \t]+sourceMappingURL=([^\s'"]+?)[ \t]*$)/gm;
	var PROCESSED_SCRIPT_RE = new RegExp([
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.getLocation),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.setLocation),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.getProperty),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.setProperty),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.callMethod),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.processScript),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.processHtml),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage),
	    reEscape(SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl)
	].join('|'));
	var PARSING_OPTIONS = {
	    allowReturnOutsideFunction: true,
	    allowImportExportEverywhere: true,
	    ecmaVersion: 11
	};
	// Code pre/post-processing
	function removeHtmlComments(code) {
	    // NOTE: The JS parser removes the line that follows'<!--'. (T226589)
	    do
	        code = code.replace(HTML_COMMENT_RE, '\n');
	    while (HTML_COMMENT_RE.test(code));
	    return code;
	}
	function preprocess(code) {
	    var bom = getBOM(code);
	    var preprocessed = bom ? code.substring(bom.length) : code;
	    preprocessed = remove$1(preprocessed);
	    preprocessed = removeSourceMap(preprocessed);
	    return { bom: bom, preprocessed: preprocessed };
	}
	function removeSourceMap(code) {
	    return code.replace(SOURCEMAP_RE, '');
	}
	function postprocess(processed, withHeader, bom, strictMode, swScopeHeaderValue) {
	    // NOTE: If the 'use strict' directive is not in the beginning of the file, it is ignored.
	    // As we insert our header in the beginning of the script, we must put a new 'use strict'
	    // before the header, otherwise it will be ignored.
	    if (withHeader)
	        processed = add$1(processed, strictMode, swScopeHeaderValue);
	    return bom ? bom + processed : processed;
	}
	// Parse/generate code
	function removeTrailingSemicolon(processed, src) {
	    return TRAILING_SEMICOLON_RE.test(src) ? processed : processed.replace(TRAILING_SEMICOLON_RE, '');
	}
	function getAst(src, isObject) {
	    // NOTE: In case of objects (e.g.eval('{ 1: 2}')) without wrapping
	    // object will be parsed as label. To avoid this we parenthesize src
	    src = isObject ? "(" + src + ")" : src;
	    try {
	        return lib.parse(src, PARSING_OPTIONS);
	    }
	    catch (err) {
	        return null;
	    }
	}
	function getCode(ast, src) {
	    var code = generate(ast, {
	        format: {
	            quotes: 'double',
	            escapeless: true,
	            compact: true
	        }
	    });
	    return src ? removeTrailingSemicolon(code, src) : code;
	}
	// Analyze code
	function analyze(code) {
	    var isObject = OBJECT_RE.test(code);
	    var ast = getAst(code, isObject);
	    // NOTE: `{ const a = 'foo'; }` edge case
	    if (!ast && isObject) {
	        ast = getAst(code, false);
	        isObject = false;
	    }
	    return { ast: ast, isObject: isObject };
	}
	function isArrayDataScript(ast) {
	    var firstChild = ast.body[0];
	    return ast.body.length === 1 &&
	        firstChild.type === Syntax_1.ExpressionStatement &&
	        firstChild.expression.type === Syntax_1.ArrayExpression;
	}
	function isStrictMode(ast) {
	    if (ast.body.length) {
	        var firstChild = ast.body[0];
	        if (firstChild.type === Syntax_1.ExpressionStatement && firstChild.expression.type === Syntax_1.Literal)
	            return firstChild.expression.value === 'use strict';
	    }
	    return false;
	}
	function applyChanges(script, changes, isObject) {
	    var indexOffset = isObject ? -1 : 0;
	    var chunks = [];
	    var index = 0;
	    if (!changes.length)
	        return script;
	    changes.sort(function (a, b) { return (a.start - b.start) || (a.end - b.end); });
	    for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
	        var change = changes_1[_i];
	        var changeStart = change.start + indexOffset;
	        var changeEnd = change.end + indexOffset;
	        var parentheses = change.node.type === Syntax_1.SequenceExpression &&
	            change.parentType !== Syntax_1.ExpressionStatement &&
	            change.parentType !== Syntax_1.SequenceExpression;
	        chunks.push(script.substring(index, changeStart));
	        chunks.push(parentheses ? '(' : ' ');
	        chunks.push(getCode(change.node, script.substring(changeStart, changeEnd)));
	        chunks.push(parentheses ? ')' : ' ');
	        index += changeEnd - index;
	    }
	    chunks.push(script.substring(index));
	    return chunks.join('');
	}
	function isScriptProcessed(code) {
	    return PROCESSED_SCRIPT_RE.test(code);
	}
	function processScript(src, withHeader, wrapLastExprWithProcessHtml, resolver, swScopeHeaderValue) {
	    if (withHeader === void 0) { withHeader = false; }
	    if (wrapLastExprWithProcessHtml === void 0) { wrapLastExprWithProcessHtml = false; }
	    var _a = preprocess(src), bom = _a.bom, preprocessed = _a.preprocessed;
	    var withoutHtmlComments = removeHtmlComments(preprocessed);
	    var _b = analyze(withoutHtmlComments), ast = _b.ast, isObject = _b.isObject;
	    if (!ast)
	        return src;
	    withHeader = withHeader && !isObject && !isArrayDataScript(ast);
	    var changes = transformProgram(ast, wrapLastExprWithProcessHtml, resolver);
	    var processed = changes.length ? applyChanges(withoutHtmlComments, changes, isObject) : preprocessed;
	    processed = postprocess(processed, withHeader, bom, isStrictMode(ast), swScopeHeaderValue);
	    if (isObject)
	        processed = processed.replace(OBJECT_WRAPPER_RE, '$1');
	    return processed;
	}

	var scriptProcessingUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		isScriptProcessed: isScriptProcessed,
		processScript: processScript
	});

	// -------------------------------------------------------------
	var SOURCE_MAP_RE = /(?:\/\*\s*(?:#|@)\s*sourceMappingURL\s*=[\s\S]*?\*\/)|(?:\/\/[\t ]*(?:#|@)[\t ]*sourceMappingURL[\t ]*=.*)/ig;
	var CSS_URL_PROPERTY_VALUE_PATTERN = /(url\s*\(\s*)(?:(')([^\s']*)(')|(")([^\s"]*)(")|([^\s)]*))(\s*\))|(@import\s+)(?:(')([^\s']*)(')|(")([^\s"]*)("))/g;
	var STYLESHEET_PROCESSING_START_COMMENT = '/*hammerhead|stylesheet|start*/';
	var STYLESHEET_PROCESSING_END_COMMENT = '/*hammerhead|stylesheet|end*/';
	var HOVER_PSEUDO_CLASS_RE = /:\s*hover(\W)/gi;
	var PSEUDO_CLASS_RE = new RegExp("\\[" + INTERNAL_ATTRIBUTES.hoverPseudoClass + "\\](\\W)", 'ig');
	var IS_STYLE_SHEET_PROCESSED_RE = new RegExp("\\s*" + reEscape(STYLESHEET_PROCESSING_START_COMMENT), 'gi');
	var STYLESHEET_PROCESSING_COMMENTS_RE = new RegExp(reEscape(STYLESHEET_PROCESSING_START_COMMENT) + "\n?|" +
	    ("\n?" + reEscape(STYLESHEET_PROCESSING_END_COMMENT) + "\\s*"), 'gi');
	var StyleProcessor = /** @class */ (function () {
	    function StyleProcessor() {
	        this.STYLESHEET_PROCESSING_START_COMMENT = STYLESHEET_PROCESSING_START_COMMENT;
	        this.STYLESHEET_PROCESSING_END_COMMENT = STYLESHEET_PROCESSING_END_COMMENT;
	    }
	    StyleProcessor.prototype.process = function (css, urlReplacer, shouldIncludeProcessingComments) {
	        if (!css || typeof css !== 'string' || shouldIncludeProcessingComments && IS_STYLE_SHEET_PROCESSED_RE.test(css))
	            return css;
	        // NOTE: Replace the :hover pseudo-class.
	        css = css.replace(HOVER_PSEUDO_CLASS_RE, '[' + INTERNAL_ATTRIBUTES.hoverPseudoClass + ']$1');
	        // NOTE: Remove all 'source map' directives.
	        css = css.replace(SOURCE_MAP_RE, '');
	        // NOTE: Replace URLs in CSS rules with proxy URLs.
	        css = this._replaceStylsheetUrls(css, urlReplacer);
	        if (shouldIncludeProcessingComments)
	            css = STYLESHEET_PROCESSING_START_COMMENT + "\n" + css + "\n" + STYLESHEET_PROCESSING_END_COMMENT;
	        return css;
	    };
	    StyleProcessor.prototype.cleanUp = function (css, parseProxyUrl) {
	        if (typeof css !== 'string')
	            return css;
	        css = css
	            .replace(PSEUDO_CLASS_RE, ':hover$1');
	        css = this._removeStylesheetProcessingComments(css);
	        return this._replaceStylsheetUrls(css, function (url) {
	            var parsedProxyUrl = parseProxyUrl(url);
	            return parsedProxyUrl ? parsedProxyUrl.destUrl : url;
	        });
	    };
	    StyleProcessor.prototype._removeStylesheetProcessingComments = function (css) {
	        var parts = css.split(STYLESHEET_PROCESSING_COMMENTS_RE);
	        var stylesheepPartsFound = parts.length >= 3;
	        if (!stylesheepPartsFound)
	            return css;
	        for (var i = 0; i < parts.length; i += 2) {
	            var whiteSpaceCount = 0;
	            // NOTE: search for whitespaces from the end of the string
	            // we do not use /\s*$/ regex intentionally to improve performance
	            for (var j = parts[i].length - 1; j >= 0; j--) {
	                if (!(/\s/.test(parts[i][j])))
	                    break;
	                whiteSpaceCount++;
	            }
	            parts[i] = parts[i].substring(0, parts[i].length - whiteSpaceCount);
	        }
	        return parts.join('');
	    };
	    StyleProcessor.prototype._replaceStylsheetUrls = function (css, processor) {
	        return css.replace(CSS_URL_PROPERTY_VALUE_PATTERN, function (match, prefix1, openQuote1, url1, closeQuote1, openQuote2, url2, closeQuote2, url3, postfix, prefix2, openQuote3, url4, closeQuote3, openQuote4, url5, closeQuote4) {
	            var prefix = prefix1 || prefix2;
	            var openQuote = openQuote1 || openQuote2 || openQuote3 || openQuote4 || '';
	            var url = url1 || url2 || url3 || url4 || url5;
	            var closeQuote = closeQuote1 || closeQuote2 || closeQuote3 || closeQuote4 || '';
	            postfix = postfix || '';
	            var processedUrl = isSpecialPage$1(url) ? url : processor(url);
	            return url ? prefix + openQuote + processedUrl + closeQuote + postfix : match;
	        });
	    };
	    return StyleProcessor;
	}());
	var styleProcessor = new StyleProcessor();

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	var BUILTIN_HEADERS = {
	    authorization: 'authorization',
	    wwwAuthenticate: 'www-authenticate',
	    proxyAuthorization: 'proxy-authorization',
	    proxyAuthenticate: 'proxy-authenticate',
	    host: 'host',
	    referer: 'referer',
	    origin: 'origin',
	    contentLength: 'content-length',
	    cookie: 'cookie',
	    setCookie: 'set-cookie',
	    ifModifiedSince: 'if-modified-since',
	    ifNoneMatch: 'if-none-match',
	    contentType: 'content-type',
	    location: 'location',
	    xFrameOptions: 'x-frame-options',
	    sourceMap: 'sourcemap',
	    referrerPolicy: 'referrer-policy',
	    refresh: 'refresh',
	    link: 'link',
	    cacheControl: 'cache-control',
	    pragma: 'pragma',
	    eTag: 'etag',
	    contentDisposition: 'content-disposition',
	    accept: 'accept',
	    contentEncoding: 'content-encoding',
	    expires: 'expires',
	    serviceWorkerAllowed: 'service-worker-allowed',
	    accessControlAllowOrigin: 'access-control-allow-origin',
	    accessControlAllowCredentials: 'access-control-allow-credentials',
	    contentSecurityPolicy: 'content-security-policy',
	    contentSecurityPolicyReportOnly: 'content-security-policy-report-only',
	    xContentSecurityPolicy: 'x-content-security-policy',
	    xContentSecurityPolicyReportOnly: 'x-content-security-policy-report-only',
	    xWebkitCsp: 'x-webkit-csp'
	};

	var XML_NAMESPACE = 'http://www.w3.org/XML/1998/namespace';
	var XLINK_NAMESPACE = 'http://www.w3.org/1999/xlink';

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	var URL_ATTR_TAGS = {
	    href: ['a', 'link', 'image', 'area', 'base'],
	    src: ['img', 'embed', 'script', 'source', 'video', 'audio', 'input', 'frame', 'iframe'],
	    action: ['form'],
	    formaction: ['button', 'input'],
	    manifest: ['html'],
	    data: ['object']
	};
	var URL_ATTRS = ['href', 'src', 'action', 'formaction', 'manifest', 'data'];
	var TARGET_ATTR_TAGS = {
	    target: ['a', 'form', 'area', 'base'],
	    formtarget: ['input', 'button']
	};
	var TARGET_ATTRS = ['target', 'formtarget'];
	var ATTRS_WITH_SPECIAL_PROXYING_LOGIC = ['sandbox', 'autocomplete', 'target', 'formtarget', 'style'];

	// -------------------------------------------------------------
	var CDATA_REG_EX = /^(\s)*\/\/<!\[CDATA\[([\s\S]*)\/\/\]\]>(\s)*$/;
	var HTML_COMMENT_POSTFIX_REG_EX = /(\/\/[^\n]*|\n\s*)-->[^\n]*([\n\s]*)?$/;
	var HTML_COMMENT_PREFIX_REG_EX = /^(\s)*<!--[^\n]*\n/;
	var HTML_COMMENT_SIMPLE_POSTFIX_REG_EX = /-->\s*$/;
	var JAVASCRIPT_PROTOCOL_REG_EX = /^\s*javascript\s*:/i;
	var EXECUTABLE_SCRIPT_TYPES_REG_EX = /^\s*(application\/(x-)?(ecma|java)script|text\/(javascript(1\.[0-5])?|((x-)?ecma|x-java|js|live)script)|module)\s*$/i;
	var SVG_XLINK_HREF_TAGS = [
	    'animate', 'animateColor', 'animateMotion', 'animateTransform', 'mpath', 'set',
	    'linearGradient', 'radialGradient', 'stop',
	    'a', 'altglyph', 'color-profile', 'cursor', 'feimage', 'filter', 'font-face-uri', 'glyphref', 'image',
	    'mpath', 'pattern', 'script', 'textpath', 'use', 'tref'
	];
	var INTEGRITY_ATTR_TAGS = ['script', 'link'];
	var IFRAME_FLAG_TAGS = ['a', 'form', 'area', 'input', 'button'];
	var PROCESSED_PRELOAD_LINK_CONTENT_TYPE = 'script';
	var MODULE_PRELOAD_LINK_REL = 'modulepreload';
	var ELEMENT_PROCESSED = 'hammerhead|element-processed';
	var AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER = 'hammerhead|autocomplete-attribute-absence-marker';
	var DomProcessor = /** @class */ (function () {
	    function DomProcessor(adapter) {
	        this.adapter = adapter;
	        this.HTML_PROCESSING_REQUIRED_EVENT = 'hammerhead|event|html-processing-required';
	        this.SVG_XLINK_HREF_TAGS = SVG_XLINK_HREF_TAGS;
	        this.AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER = AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER;
	        this.PROCESSED_PRELOAD_LINK_CONTENT_TYPE = PROCESSED_PRELOAD_LINK_CONTENT_TYPE;
	        this.MODULE_PRELOAD_LINK_REL = MODULE_PRELOAD_LINK_REL;
	        this.forceProxySrcForImage = false;
	        this.allowMultipleWindows = false;
	        this.EVENTS = this.adapter.EVENTS;
	        this.elementProcessorPatterns = this._createProcessorPatterns(this.adapter);
	    }
	    DomProcessor.isTagWithTargetAttr = function (tagName) {
	        return !!tagName && TARGET_ATTR_TAGS.target.indexOf(tagName) > -1;
	    };
	    DomProcessor.isTagWithFormTargetAttr = function (tagName) {
	        return !!tagName && TARGET_ATTR_TAGS.formtarget.indexOf(tagName) > -1;
	    };
	    DomProcessor.isTagWithIntegrityAttr = function (tagName) {
	        return !!tagName && INTEGRITY_ATTR_TAGS.indexOf(tagName) !== -1;
	    };
	    DomProcessor.isIframeFlagTag = function (tagName) {
	        return !!tagName && IFRAME_FLAG_TAGS.indexOf(tagName) !== -1;
	    };
	    DomProcessor.isAddedAutocompleteAttr = function (attrName, storedAttrValue) {
	        return attrName === 'autocomplete' && storedAttrValue === AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER;
	    };
	    DomProcessor.processJsAttrValue = function (value, _a) {
	        var isJsProtocol = _a.isJsProtocol, isEventAttr = _a.isEventAttr;
	        if (isJsProtocol)
	            value = value.replace(JAVASCRIPT_PROTOCOL_REG_EX, '');
	        value = processScript(value, false, isJsProtocol && !isEventAttr, void 0);
	        if (isJsProtocol)
	            value = 'javascript:' + value; // eslint-disable-line no-script-url
	        return value;
	    };
	    DomProcessor.getStoredAttrName = function (attr) {
	        return attr + INTERNAL_ATTRIBUTES.storedAttrPostfix;
	    };
	    DomProcessor.isJsProtocol = function (value) {
	        return JAVASCRIPT_PROTOCOL_REG_EX.test(value);
	    };
	    DomProcessor._isHtmlImportLink = function (tagName, relAttr) {
	        return !!tagName && !!relAttr && tagName === 'link' && relAttr === 'import';
	    };
	    DomProcessor.prototype._getRelAttribute = function (el) {
	        return String(this.adapter.getAttr(el, 'rel')).toLowerCase();
	    };
	    DomProcessor.prototype._getAsAttribute = function (el) {
	        return String(this.adapter.getAttr(el, 'as')).toLowerCase();
	    };
	    DomProcessor.prototype._createProcessorPatterns = function (adapter) {
	        var _this = this;
	        var selectors = {
	            HAS_HREF_ATTR: function (el) { return _this.isUrlAttr(el, 'href'); },
	            HAS_SRC_ATTR: function (el) { return _this.isUrlAttr(el, 'src'); },
	            HAS_ACTION_ATTR: function (el) { return _this.isUrlAttr(el, 'action'); },
	            HAS_FORMACTION_ATTR: function (el) { return _this.isUrlAttr(el, 'formaction'); },
	            HAS_FORMTARGET_ATTR: function (el) {
	                return DomProcessor.isTagWithFormTargetAttr(adapter.getTagName(el)) && adapter.hasAttr(el, 'formtarget');
	            },
	            HAS_MANIFEST_ATTR: function (el) { return _this.isUrlAttr(el, 'manifest'); },
	            HAS_DATA_ATTR: function (el) { return _this.isUrlAttr(el, 'data'); },
	            HAS_SRCDOC_ATTR: function (el) {
	                var tagName = _this.adapter.getTagName(el);
	                return (tagName === 'iframe' || tagName === 'frame') && adapter.hasAttr(el, 'srcdoc');
	            },
	            HTTP_EQUIV_META: function (el) {
	                var tagName = adapter.getTagName(el);
	                return tagName === 'meta' && adapter.hasAttr(el, 'http-equiv');
	            },
	            ALL: function () { return true; },
	            IS_SCRIPT: function (el) { return adapter.getTagName(el) === 'script'; },
	            IS_LINK: function (el) { return adapter.getTagName(el) === 'link'; },
	            IS_INPUT: function (el) { return adapter.getTagName(el) === 'input'; },
	            IS_FILE_INPUT: function (el) {
	                return adapter.getTagName(el) === 'input' &&
	                    adapter.hasAttr(el, 'type') &&
	                    adapter.getAttr(el, 'type').toLowerCase() === 'file';
	            },
	            IS_STYLE: function (el) { return adapter.getTagName(el) === 'style'; },
	            HAS_EVENT_HANDLER: function (el) { return adapter.hasEventHandler(el); },
	            IS_SANDBOXED_IFRAME: function (el) {
	                var tagName = adapter.getTagName(el);
	                return (tagName === 'iframe' || tagName === 'frame') && adapter.hasAttr(el, 'sandbox');
	            },
	            IS_SVG_ELEMENT_WITH_XLINK_HREF_ATTR: function (el) {
	                return adapter.isSVGElement(el) &&
	                    adapter.hasAttr(el, 'xlink:href') &&
	                    SVG_XLINK_HREF_TAGS.indexOf(adapter.getTagName(el)) !== -1;
	            },
	            IS_SVG_ELEMENT_WITH_XML_BASE_ATTR: function (el) { return adapter.isSVGElement(el) && adapter.hasAttr(el, 'xml:base'); }
	        };
	        return [
	            {
	                selector: selectors.HAS_FORMTARGET_ATTR,
	                targetAttr: 'formtarget',
	                elementProcessors: [this._processTargetBlank]
	            },
	            {
	                selector: selectors.HAS_HREF_ATTR,
	                urlAttr: 'href',
	                targetAttr: 'target',
	                elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_SRC_ATTR,
	                urlAttr: 'src',
	                targetAttr: 'target',
	                elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_ACTION_ATTR,
	                urlAttr: 'action',
	                targetAttr: 'target',
	                elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_FORMACTION_ATTR,
	                urlAttr: 'formaction',
	                targetAttr: 'formtarget',
	                elementProcessors: [this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_MANIFEST_ATTR,
	                urlAttr: 'manifest',
	                elementProcessors: [this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_DATA_ATTR,
	                urlAttr: 'data',
	                elementProcessors: [this._processUrlAttrs, this._processUrlJsAttr]
	            },
	            {
	                selector: selectors.HAS_SRCDOC_ATTR,
	                elementProcessors: [this._processSrcdocAttr]
	            },
	            {
	                selector: selectors.HTTP_EQUIV_META,
	                urlAttr: 'content',
	                elementProcessors: [this._processMetaElement]
	            },
	            {
	                selector: selectors.IS_SCRIPT,
	                elementProcessors: [this._processScriptElement, this._processIntegrityAttr]
	            },
	            { selector: selectors.ALL, elementProcessors: [this._processStyleAttr] },
	            {
	                selector: selectors.IS_LINK,
	                relAttr: 'rel',
	                elementProcessors: [this._processIntegrityAttr, this._processRelPrefetch]
	            },
	            { selector: selectors.IS_STYLE, elementProcessors: [this._processStylesheetElement] },
	            { selector: selectors.IS_INPUT, elementProcessors: [this._processAutoComplete] },
	            { selector: selectors.IS_FILE_INPUT, elementProcessors: [this._processRequired] },
	            { selector: selectors.HAS_EVENT_HANDLER, elementProcessors: [this._processEvtAttr] },
	            { selector: selectors.IS_SANDBOXED_IFRAME, elementProcessors: [this._processSandboxedIframe] },
	            {
	                selector: selectors.IS_SVG_ELEMENT_WITH_XLINK_HREF_ATTR,
	                urlAttr: 'xlink:href',
	                elementProcessors: [this._processSVGXLinkHrefAttr, this._processUrlAttrs]
	            },
	            {
	                selector: selectors.IS_SVG_ELEMENT_WITH_XML_BASE_ATTR,
	                urlAttr: 'xml:base',
	                elementProcessors: [this._processUrlAttrs]
	            }
	        ];
	    };
	    // API
	    DomProcessor.prototype.processElement = function (el, urlReplacer) {
	        // @ts-ignore
	        if (el[ELEMENT_PROCESSED])
	            return;
	        for (var _i = 0, _a = this.elementProcessorPatterns; _i < _a.length; _i++) {
	            var pattern = _a[_i];
	            if (pattern.selector(el) && !this._isShadowElement(el)) {
	                for (var _b = 0, _c = pattern.elementProcessors; _b < _c.length; _b++) {
	                    var processor = _c[_b];
	                    processor.call(this, el, urlReplacer, pattern);
	                }
	                // @ts-ignore
	                el[ELEMENT_PROCESSED] = true;
	            }
	        }
	    };
	    // Utils
	    DomProcessor.prototype.getElementResourceType = function (el) {
	        var tagName = this.adapter.getTagName(el);
	        if (tagName === 'link' && (this._getAsAttribute(el) === PROCESSED_PRELOAD_LINK_CONTENT_TYPE ||
	            this._getRelAttribute(el) === MODULE_PRELOAD_LINK_REL))
	            return getResourceTypeString({ isScript: true });
	        return getResourceTypeString({
	            isIframe: tagName === 'iframe' || tagName === 'frame' || this._isOpenLinkInIframe(el),
	            isForm: tagName === 'form' || tagName === 'input' || tagName === 'button',
	            isScript: tagName === 'script',
	            isHtmlImport: tagName === 'link' && this._getRelAttribute(el) === 'import'
	        });
	    };
	    DomProcessor.prototype.isUrlAttr = function (el, attr, ns) {
	        var tagName = this.adapter.getTagName(el);
	        attr = attr ? attr.toLowerCase() : attr;
	        // @ts-ignore
	        if (URL_ATTR_TAGS[attr] && URL_ATTR_TAGS[attr].indexOf(tagName) !== -1)
	            return true;
	        return this.adapter.isSVGElement(el) && (attr === 'xml:base' || attr === 'base' && ns === XML_NAMESPACE);
	    };
	    DomProcessor.prototype.getUrlAttr = function (el) {
	        var tagName = this.adapter.getTagName(el);
	        for (var _i = 0, URL_ATTRS_1 = URL_ATTRS; _i < URL_ATTRS_1.length; _i++) {
	            var urlAttr = URL_ATTRS_1[_i];
	            // @ts-ignore
	            if (URL_ATTR_TAGS[urlAttr].indexOf(tagName) !== -1)
	                return urlAttr;
	        }
	        return null;
	    };
	    DomProcessor.prototype.getTargetAttr = function (el) {
	        var tagName = this.adapter.getTagName(el);
	        for (var _i = 0, TARGET_ATTRS_1 = TARGET_ATTRS; _i < TARGET_ATTRS_1.length; _i++) {
	            var targetAttr = TARGET_ATTRS_1[_i];
	            // @ts-ignore
	            if (TARGET_ATTR_TAGS[targetAttr].indexOf(tagName) > -1)
	                return targetAttr;
	        }
	        return null;
	    };
	    DomProcessor.prototype._isOpenLinkInIframe = function (el) {
	        var tagName = this.adapter.getTagName(el);
	        var targetAttr = this.getTargetAttr(el);
	        var target = this.adapter.getAttr(el, targetAttr);
	        var rel = this._getRelAttribute(el);
	        if (target !== '_top') {
	            var isImageInput = tagName === 'input' && this.adapter.getAttr(el, 'type') === 'image';
	            var mustProcessTag = !isImageInput && DomProcessor.isIframeFlagTag(tagName) ||
	                DomProcessor._isHtmlImportLink(tagName, rel);
	            var isNameTarget = target ? target[0] !== '_' : false;
	            if (target === '_parent')
	                return mustProcessTag && !this.adapter.isTopParentIframe(el);
	            if (mustProcessTag && (this.adapter.hasIframeParent(el) || isNameTarget && this.adapter.isExistingTarget(target, el)))
	                return true;
	        }
	        return false;
	    };
	    DomProcessor.prototype._isShadowElement = function (el) {
	        var className = this.adapter.getClassName(el);
	        return typeof className === 'string' && className.indexOf(SHADOW_UI_CLASS_NAME.postfix) > -1;
	    };
	    // Element processors
	    DomProcessor.prototype._processAutoComplete = function (el) {
	        var storedUrlAttr = DomProcessor.getStoredAttrName('autocomplete');
	        var processed = this.adapter.hasAttr(el, storedUrlAttr);
	        var attrValue = this.adapter.getAttr(el, processed ? storedUrlAttr : 'autocomplete');
	        if (!processed) {
	            this.adapter.setAttr(el, storedUrlAttr, attrValue || attrValue ===
	                ''
	                ? attrValue
	                : AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER);
	        }
	        this.adapter.setAttr(el, 'autocomplete', 'off');
	    };
	    DomProcessor.prototype._processRequired = function (el) {
	        var storedRequired = DomProcessor.getStoredAttrName('required');
	        var hasRequiredAttr = this.adapter.hasAttr(el, 'required');
	        var processed = this.adapter.hasAttr(el, storedRequired) && !hasRequiredAttr;
	        if (!processed && hasRequiredAttr) {
	            var attrValue = this.adapter.getAttr(el, 'required');
	            this.adapter.setAttr(el, storedRequired, attrValue);
	            this.adapter.removeAttr(el, 'required');
	        }
	    };
	    // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
	    // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
	    // the correct SHA for the changed script.
	    // In addition, we create stored 'integrity' attribute with the current 'integrity' attribute value. (GH-235)
	    DomProcessor.prototype._processIntegrityAttr = function (el) {
	        var storedIntegrityAttr = DomProcessor.getStoredAttrName('integrity');
	        var processed = this.adapter.hasAttr(el, storedIntegrityAttr) && !this.adapter.hasAttr(el, 'integrity');
	        var attrValue = this.adapter.getAttr(el, processed ? storedIntegrityAttr : 'integrity');
	        if (attrValue)
	            this.adapter.setAttr(el, storedIntegrityAttr, attrValue);
	        if (!processed)
	            this.adapter.removeAttr(el, 'integrity');
	    };
	    // NOTE: We simply remove the 'rel' attribute if rel='prefetch' and use stored 'rel' attribute, because the prefetch
	    // resource type is unknown. https://github.com/DevExpress/testcafe/issues/2528
	    DomProcessor.prototype._processRelPrefetch = function (el, _urlReplacer, pattern) {
	        var storedRelAttr = DomProcessor.getStoredAttrName(pattern.relAttr);
	        var processed = this.adapter.hasAttr(el, storedRelAttr) && !this.adapter.hasAttr(el, pattern.relAttr);
	        var attrValue = this.adapter.getAttr(el, processed ? storedRelAttr : pattern.relAttr);
	        if (attrValue) {
	            var formatedValue = trim(attrValue.toLowerCase());
	            if (formatedValue === 'prefetch') {
	                this.adapter.setAttr(el, storedRelAttr, attrValue);
	                if (!processed)
	                    this.adapter.removeAttr(el, pattern.relAttr);
	            }
	        }
	    };
	    DomProcessor.prototype._processJsAttr = function (el, attrName, _a) {
	        var isJsProtocol = _a.isJsProtocol, isEventAttr = _a.isEventAttr;
	        var storedUrlAttr = DomProcessor.getStoredAttrName(attrName);
	        var processed = this.adapter.hasAttr(el, storedUrlAttr);
	        var attrValue = this.adapter.getAttr(el, processed ? storedUrlAttr : attrName);
	        var processedValue = DomProcessor.processJsAttrValue(attrValue, { isJsProtocol: isJsProtocol, isEventAttr: isEventAttr });
	        if (attrValue !== processedValue) {
	            this.adapter.setAttr(el, storedUrlAttr, attrValue);
	            this.adapter.setAttr(el, attrName, processedValue);
	        }
	    };
	    DomProcessor.prototype._processEvtAttr = function (el) {
	        var events = this.adapter.EVENTS;
	        for (var i = 0; i < events.length; i++) {
	            var attrValue = this.adapter.getAttr(el, events[i]);
	            if (attrValue) {
	                this._processJsAttr(el, events[i], {
	                    isJsProtocol: DomProcessor.isJsProtocol(attrValue),
	                    isEventAttr: true
	                });
	            }
	        }
	    };
	    DomProcessor.prototype._processMetaElement = function (el, urlReplacer, pattern) {
	        var httpEquivAttrValue = this.adapter.getAttr(el, 'http-equiv').toLowerCase();
	        if (httpEquivAttrValue === BUILTIN_HEADERS.refresh) {
	            var attr = this.adapter.getAttr(el, pattern.urlAttr);
	            attr = attr.replace(/(url=)(.*)$/i, function (_match, prefix, url) { return prefix + urlReplacer(url); });
	            this.adapter.setAttr(el, pattern.urlAttr, attr);
	        }
	        // TODO: remove after https://github.com/DevExpress/testcafe-hammerhead/issues/244 implementation
	        else if (httpEquivAttrValue === BUILTIN_HEADERS.contentSecurityPolicy) {
	            this.adapter.removeAttr(el, 'http-equiv');
	            this.adapter.removeAttr(el, 'content');
	        }
	    };
	    DomProcessor.prototype._processSandboxedIframe = function (el) {
	        var attrValue = this.adapter.getAttr(el, 'sandbox');
	        var allowSameOrigin = attrValue.indexOf('allow-same-origin') !== -1;
	        var allowScripts = attrValue.indexOf('allow-scripts') !== -1;
	        var storedAttr = DomProcessor.getStoredAttrName('sandbox');
	        this.adapter.setAttr(el, storedAttr, attrValue);
	        if (!allowSameOrigin || !allowScripts) {
	            attrValue += !allowSameOrigin ? ' allow-same-origin' : '';
	            attrValue += !allowScripts ? ' allow-scripts' : '';
	        }
	        this.adapter.setAttr(el, 'sandbox', attrValue);
	    };
	    DomProcessor.prototype._processScriptElement = function (script, urlReplacer) {
	        var scriptContent = this.adapter.getScriptContent(script);
	        if (!scriptContent || !this.adapter.needToProcessContent(script))
	            return;
	        var scriptProcessedOnServer = isScriptProcessed(scriptContent);
	        if (scriptProcessedOnServer)
	            return;
	        // NOTE: We do not process scripts that are not executed during page load. We process scripts of types like
	        // text/javascript, application/javascript etc. (a complete list of MIME types is specified in the w3c.org
	        // html5 specification). If the type is not set, it is considered 'text/javascript' by default.
	        var scriptType = this.adapter.getAttr(script, 'type');
	        var isExecutableScript = !scriptType || EXECUTABLE_SCRIPT_TYPES_REG_EX.test(scriptType);
	        if (isExecutableScript) {
	            var result = scriptContent;
	            var commentPrefix = '';
	            var commentPrefixMatch = result.match(HTML_COMMENT_PREFIX_REG_EX);
	            var commentPostfix = '';
	            var commentPostfixMatch = null;
	            var hasCDATA = CDATA_REG_EX.test(result);
	            if (commentPrefixMatch) {
	                commentPrefix = commentPrefixMatch[0];
	                commentPostfixMatch = result.match(HTML_COMMENT_POSTFIX_REG_EX);
	                if (commentPostfixMatch)
	                    commentPostfix = commentPostfixMatch[0];
	                else if (!HTML_COMMENT_SIMPLE_POSTFIX_REG_EX.test(commentPrefix))
	                    commentPostfix = '//-->';
	                result = result.replace(commentPrefix, '').replace(commentPostfix, '');
	            }
	            if (hasCDATA)
	                result = result.replace(CDATA_REG_EX, '$2');
	            result = commentPrefix + processScript(result, true, false, urlReplacer) + commentPostfix;
	            if (hasCDATA)
	                result = '\n//<![CDATA[\n' + result + '//]]>';
	            this.adapter.setScriptContent(script, result);
	        }
	    };
	    DomProcessor.prototype._processStyleAttr = function (el, urlReplacer) {
	        var style = this.adapter.getAttr(el, 'style');
	        if (style)
	            this.adapter.setAttr(el, 'style', styleProcessor.process(style, urlReplacer, false));
	    };
	    DomProcessor.prototype._processStylesheetElement = function (el, urlReplacer) {
	        var content = this.adapter.getStyleContent(el);
	        if (content && urlReplacer && this.adapter.needToProcessContent(el)) {
	            content = styleProcessor.process(content, urlReplacer, true);
	            this.adapter.setStyleContent(el, content);
	        }
	    };
	    DomProcessor.prototype._processTargetBlank = function (el, _urlReplacer, pattern) {
	        if (this.allowMultipleWindows)
	            return;
	        var storedTargetAttr = DomProcessor.getStoredAttrName(pattern.targetAttr);
	        var processed = this.adapter.hasAttr(el, storedTargetAttr);
	        if (processed)
	            return;
	        var attrValue = this.adapter.getAttr(el, pattern.targetAttr);
	        // NOTE: Value may have whitespace.
	        attrValue = attrValue && attrValue.replace(/\s/g, '');
	        if (attrValue === '_blank') {
	            this.adapter.setAttr(el, pattern.targetAttr, '_top');
	            this.adapter.setAttr(el, storedTargetAttr, attrValue);
	        }
	    };
	    DomProcessor.prototype._processUrlAttrs = function (el, urlReplacer, pattern) {
	        var storedUrlAttr = DomProcessor.getStoredAttrName(pattern.urlAttr);
	        var resourceUrl = this.adapter.getAttr(el, pattern.urlAttr);
	        var isSpecialPageUrl = isSpecialPage$1(resourceUrl);
	        var processedOnServer = this.adapter.hasAttr(el, storedUrlAttr);
	        if ((!resourceUrl && resourceUrl !== '' || processedOnServer) ||
	            !isSupportedProtocol$1(resourceUrl) && !isSpecialPageUrl)
	            return;
	        var elTagName = this.adapter.getTagName(el);
	        var isIframe = elTagName === 'iframe' || elTagName === 'frame';
	        var isScript = elTagName === 'script';
	        var isAnchor = elTagName === 'a';
	        var target = this.adapter.getAttr(el, pattern.targetAttr);
	        // NOTE: Elements with target=_parent shouldn’t be processed on the server,because we don't
	        // know what is the parent of the processed page (an iframe or the top window).
	        if (!this.adapter.needToProcessUrl(elTagName, target))
	            return;
	        var resourceType = this.getElementResourceType(el);
	        var parsedResourceUrl = parseUrl$1(resourceUrl);
	        var isRelativePath = parsedResourceUrl.protocol !== 'file:' && !parsedResourceUrl.host;
	        var charsetAttrValue = isScript && this.adapter.getAttr(el, 'charset');
	        var isImgWithoutSrc = elTagName === 'img' && resourceUrl === '';
	        var isIframeWithEmptySrc = isIframe && resourceUrl === '';
	        var isCrossDomainSrc = false;
	        var proxyUrl = resourceUrl;
	        // NOTE: Only a non-relative iframe src can be cross-domain.
	        if (isIframe && !isSpecialPageUrl && !isRelativePath)
	            isCrossDomainSrc = !this.adapter.sameOriginCheck(parseProxyUrl$1(urlReplacer('/')).destUrl, resourceUrl);
	        if ((!isSpecialPageUrl || isAnchor) && !isImgWithoutSrc && !isIframeWithEmptySrc) {
	            proxyUrl = elTagName === 'img' && !this.forceProxySrcForImage
	                ? resolveUrlAsDest$1(resourceUrl, urlReplacer)
	                : urlReplacer(resourceUrl, resourceType, charsetAttrValue, isCrossDomainSrc);
	        }
	        this.adapter.setAttr(el, storedUrlAttr, resourceUrl);
	        this.adapter.setAttr(el, pattern.urlAttr, proxyUrl);
	    };
	    DomProcessor.prototype._processSrcdocAttr = function (el) {
	        var storedAttr = DomProcessor.getStoredAttrName('srcdoc');
	        var html = this.adapter.getAttr(el, 'srcdoc');
	        var processedHtml = this.adapter.processSrcdocAttr(html);
	        this.adapter.setAttr(el, storedAttr, html);
	        this.adapter.setAttr(el, 'srcdoc', processedHtml);
	    };
	    DomProcessor.prototype._processUrlJsAttr = function (el, _urlReplacer, pattern) {
	        if (DomProcessor.isJsProtocol(this.adapter.getAttr(el, pattern.urlAttr)))
	            this._processJsAttr(el, pattern.urlAttr, { isJsProtocol: true, isEventAttr: false });
	    };
	    DomProcessor.prototype._processSVGXLinkHrefAttr = function (el, _urlReplacer, pattern) {
	        var attrValue = this.adapter.getAttr(el, pattern.urlAttr);
	        if (HASH_RE$1.test(attrValue)) {
	            var storedUrlAttr = DomProcessor.getStoredAttrName(pattern.urlAttr);
	            this.adapter.setAttr(el, storedUrlAttr, attrValue);
	        }
	    };
	    return DomProcessor;
	}());

	var lengthWeakMap = new WeakMap();
	function updateOrigin(ancestorOrigins, wrapper, index, origin) {
	    var descriptor = createOverriddenDescriptor(ancestorOrigins, index, { value: origin });
	    nativeMethods.objectDefineProperty(wrapper, index, descriptor);
	}
	var DOMStringListInheritor = /** @class */ (function () {
	    function DOMStringListInheritor() {
	    }
	    return DOMStringListInheritor;
	}());
	DOMStringListInheritor.prototype = DOMStringList.prototype;
	var DOMStringListWrapper = /** @class */ (function (_super) {
	    __extends(DOMStringListWrapper, _super);
	    function DOMStringListWrapper(window, getCrossDomainOrigin) {
	        var _this = _super.call(this) || this;
	        var nativeOrigins = window.location.ancestorOrigins;
	        var length = nativeOrigins.length;
	        var parentWindow = window.parent;
	        lengthWeakMap.set(_this, length);
	        var _loop_1 = function (i) {
	            var parentLocationWrapper = LocationAccessorsInstrumentation.getLocationWrapper(parentWindow);
	            var isCrossDomainParent = parentLocationWrapper === parentWindow.location;
	            // eslint-disable-next-line no-restricted-properties
	            updateOrigin(nativeOrigins, this_1, i.toString(), isCrossDomainParent ? '' : parentLocationWrapper.origin);
	            if (isCrossDomainParent && getCrossDomainOrigin)
	                //@ts-ignore
	                getCrossDomainOrigin(parentWindow, function (origin) { return updateOrigin(nativeOrigins, _this, i, origin); });
	            parentWindow = parentWindow.parent;
	        };
	        var this_1 = this;
	        for (var i = 0; i < length; i++) {
	            _loop_1(i);
	        }
	        return _this;
	    }
	    DOMStringListWrapper.prototype.item = function (index) {
	        return this[index];
	    };
	    DOMStringListWrapper.prototype.contains = function (origin) {
	        if (typeof origin !== 'string')
	            origin = String(origin);
	        var length = lengthWeakMap.get(this) || 0;
	        for (var i = 0; i < length; i++) {
	            if (this[i] === origin)
	                return true;
	        }
	        return false;
	    };
	    Object.defineProperty(DOMStringListWrapper.prototype, "length", {
	        get: function () {
	            return lengthWeakMap.get(this);
	        },
	        enumerable: true,
	        configurable: true
	    });
	    return DOMStringListWrapper;
	}(DOMStringListInheritor));

	var MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER || 9007199254740991;
	var MIN_SAFE_INTEGER = Number.MIN_SAFE_INTEGER || -9007199254740991;
	var IntegerIdGenerator = /** @class */ (function () {
	    function IntegerIdGenerator() {
	        this._id = MIN_SAFE_INTEGER;
	    }
	    IntegerIdGenerator.prototype.increment = function () {
	        this._id = this._id === MAX_SAFE_INTEGER ? MIN_SAFE_INTEGER : this._id + 1;
	        return this._id;
	    };
	    Object.defineProperty(IntegerIdGenerator.prototype, "value", {
	        get: function () {
	            return this._id;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    return IntegerIdGenerator;
	}());

	var GET_ORIGIN_CMD = 'hammerhead|command|get-origin';
	var ORIGIN_RECEIVED_CMD = 'hammerhead|command|origin-received';
	function getLocationUrl(window) {
	    try {
	        return window.location.toString();
	    }
	    catch (e) {
	        return void 0;
	    }
	}
	var LocationInheritor = /** @class */ (function () {
	    function LocationInheritor() {
	    }
	    return LocationInheritor;
	}());
	LocationInheritor.prototype = Location.prototype;
	var LocationWrapper = /** @class */ (function (_super) {
	    __extends(LocationWrapper, _super);
	    function LocationWrapper(window, messageSandbox, onChanged) {
	        var _this = _super.call(this) || this;
	        var parsedLocation = parseProxyUrl(getLocationUrl(window));
	        var locationResourceType = parsedLocation ? parsedLocation.resourceType : '';
	        var parsedResourceType = parseResourceType(locationResourceType);
	        // @ts-ignore
	        var isLocationPropsInProto = nativeMethods.objectHasOwnProperty.call(window.Location.prototype, 'href');
	        // @ts-ignore
	        var locationPropsOwner = isLocationPropsInProto ? window.Location.prototype : window.location;
	        var locationProps = {};
	        parsedResourceType.isIframe = parsedResourceType.isIframe || isIframeWindow(window);
	        var resourceType = getResourceTypeString({
	            isIframe: parsedResourceType.isIframe,
	            isForm: parsedResourceType.isForm
	        });
	        var getHref = function () {
	            // eslint-disable-next-line no-restricted-properties
	            if (isIframeWindow(window) && window.location.href === SPECIAL_BLANK_PAGE)
	                return SPECIAL_BLANK_PAGE;
	            var locationUrl = get$2();
	            var resolveElement = urlResolver.getResolverElement(window.document);
	            nativeMethods.anchorHrefSetter.call(resolveElement, locationUrl);
	            var href = nativeMethods.anchorHrefGetter.call(resolveElement);
	            return ensureTrailingSlash(href, locationUrl);
	        };
	        var getProxiedHref = function (href) {
	            if (typeof href !== 'string')
	                href = String(href);
	            href = prepareUrl(href);
	            if (DomProcessor.isJsProtocol(href))
	                return DomProcessor.processJsAttrValue(href, { isJsProtocol: true, isEventAttr: false });
	            var locationUrl = getLocationUrl(window);
	            var proxyPort = null;
	            if (window !== window.parent) {
	                var parentLocationUrl = getLocationUrl(window.parent);
	                var parsedParentLocationUrl = parseProxyUrl(parentLocationUrl);
	                if (parsedParentLocationUrl && parsedParentLocationUrl.proxy) {
	                    // eslint-disable-next-line no-restricted-properties
	                    var parentProxyPort = parsedParentLocationUrl.proxy.port;
	                    proxyPort = sameOriginCheck$1(parentLocationUrl, href)
	                        ? parentProxyPort
	                        : getCrossDomainProxyPort(parentProxyPort);
	                }
	            }
	            var changedOnlyHash = locationUrl && isChangedOnlyHash(locationUrl, href);
	            var currentResourceType = changedOnlyHash ? locationResourceType : resourceType;
	            return getProxyUrl(href, { resourceType: currentResourceType, proxyPort: proxyPort });
	        };
	        // eslint-disable-next-line no-restricted-properties
	        locationProps.href = createOverriddenDescriptor(locationPropsOwner, 'href', {
	            getter: getHref,
	            setter: function (href) {
	                var proxiedHref = getProxiedHref(href);
	                // eslint-disable-next-line no-restricted-properties
	                window.location.href = proxiedHref;
	                onChanged(proxiedHref);
	                return href;
	            }
	        });
	        // eslint-disable-next-line no-restricted-properties
	        locationProps.search = createOverriddenDescriptor(locationPropsOwner, 'search', {
	            // eslint-disable-next-line no-restricted-properties
	            getter: function () { return window.location.search; },
	            setter: function (search) {
	                var newLocation = changeDestUrlPart(window.location.toString(), nativeMethods.anchorSearchSetter, search, resourceType);
	                // @ts-ignore
	                window.location = newLocation;
	                onChanged(newLocation);
	                return search;
	            }
	        });
	        // eslint-disable-next-line no-restricted-properties
	        locationProps.origin = createOverriddenDescriptor(locationPropsOwner, 'origin', {
	            getter: function () { return getDomain(getParsed()); },
	            setter: function (origin) { return origin; }
	        });
	        locationProps.hash = createOverriddenDescriptor(locationPropsOwner, 'hash', {
	            getter: function () { return window.location.hash; },
	            setter: function (hash) {
	                window.location.hash = hash;
	                return hash;
	            }
	        });
	        if (window.location.ancestorOrigins) {
	            var callbacks_1 = nativeMethods.objectCreate(null);
	            var idGenerator_1 = new IntegerIdGenerator();
	            var getCrossDomainOrigin = function (win, callback) {
	                var id = idGenerator_1.increment();
	                callbacks_1[id] = callback;
	                messageSandbox.sendServiceMsg({ id: id, cmd: GET_ORIGIN_CMD }, win);
	            };
	            if (messageSandbox) {
	                messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (_a) {
	                    var message = _a.message, source = _a.source;
	                    if (message.cmd === GET_ORIGIN_CMD) {
	                        // @ts-ignore
	                        messageSandbox.sendServiceMsg({ id: message.id, cmd: ORIGIN_RECEIVED_CMD, origin: _this.origin }, source); // eslint-disable-line no-restricted-properties
	                    }
	                    else if (message.cmd === ORIGIN_RECEIVED_CMD) {
	                        var callback = callbacks_1[message.id];
	                        if (callback)
	                            callback(message.origin); // eslint-disable-line no-restricted-properties
	                    }
	                });
	            }
	            var ancestorOrigins_1 = new DOMStringListWrapper(window, messageSandbox ? getCrossDomainOrigin : void 0);
	            locationProps.ancestorOrigins = createOverriddenDescriptor(locationPropsOwner, 'ancestorOrigins', {
	                getter: function () { return ancestorOrigins_1; }
	            });
	        }
	        var createLocationPropertyDesc = function (property, nativePropSetter) {
	            locationProps[property] = createOverriddenDescriptor(locationPropsOwner, property, {
	                getter: function () {
	                    var frameElement = getFrameElement(window);
	                    var inIframeWithoutSrc = frameElement && isIframeWithoutSrc(frameElement);
	                    var parsedDestLocation = inIframeWithoutSrc ? window.location : getParsed();
	                    return parsedDestLocation[property];
	                },
	                setter: function (value) {
	                    var newLocation = changeDestUrlPart(window.location.toString(), nativePropSetter, value, resourceType);
	                    // @ts-ignore
	                    window.location = newLocation;
	                    onChanged(newLocation);
	                    return value;
	                }
	            });
	        };
	        createLocationPropertyDesc('port', nativeMethods.anchorPortSetter);
	        createLocationPropertyDesc('host', nativeMethods.anchorHostSetter);
	        createLocationPropertyDesc('hostname', nativeMethods.anchorHostnameSetter);
	        createLocationPropertyDesc('pathname', nativeMethods.anchorPathnameSetter);
	        createLocationPropertyDesc('protocol', nativeMethods.anchorProtocolSetter);
	        locationProps.assign = createOverriddenDescriptor(locationPropsOwner, 'assign', {
	            value: function (url) {
	                var proxiedHref = getProxiedHref(url);
	                var result = window.location.assign(proxiedHref);
	                onChanged(proxiedHref);
	                return result;
	            }
	        });
	        locationProps.replace = createOverriddenDescriptor(locationPropsOwner, 'replace', {
	            value: function (url) {
	                var proxiedHref = getProxiedHref(url);
	                var result = window.location.replace(proxiedHref);
	                onChanged(proxiedHref);
	                return result;
	            }
	        });
	        locationProps.reload = createOverriddenDescriptor(locationPropsOwner, 'reload', {
	            value: function (forcedReload) {
	                var result = window.location.reload(forcedReload);
	                onChanged(window.location.toString());
	                return result;
	            }
	        });
	        locationProps.toString = createOverriddenDescriptor(locationPropsOwner, 'toString', { value: getHref });
	        if (!isLocationPropsInProto && nativeMethods.objectHasOwnProperty.call(window.location, 'valueOf'))
	            locationProps.valueOf = createOverriddenDescriptor(locationPropsOwner, 'valueOf', { value: function () { return _this; } });
	        nativeMethods.objectDefineProperties(_this, locationProps);
	        // NOTE: We shouldn't break the client script if the browser add the new API. For example:
	        // > From Chrome 80 to Chrome 85, the fragmentDirective property was defined on Location.prototype.
	        if (isIE11)
	            return _this;
	        var protoKeys = nativeMethods.objectKeys(Location.prototype);
	        var locWrapper = _this;
	        var rewriteDescriptorFn = function (descriptor, key) {
	            if (typeof descriptor[key] !== 'function')
	                return;
	            var nativeMethod = descriptor[key];
	            descriptor[key] = function () {
	                var ctx = this === locWrapper ? window.location : this;
	                return nativeMethod.apply(ctx, arguments);
	            };
	        };
	        for (var _i = 0, protoKeys_1 = protoKeys; _i < protoKeys_1.length; _i++) {
	            var protoKey = protoKeys_1[_i];
	            if (protoKey in locationProps)
	                continue;
	            var protoKeyDescriptor = nativeMethods.objectGetOwnPropertyDescriptor(Location.prototype, protoKey);
	            rewriteDescriptorFn(protoKeyDescriptor, 'get');
	            rewriteDescriptorFn(protoKeyDescriptor, 'set');
	            rewriteDescriptorFn(protoKeyDescriptor, 'value');
	            nativeMethods.objectDefineProperty(locWrapper, protoKey, protoKeyDescriptor);
	            // NOTE: We hide errors with a new browser API and we should know about it.
	            nativeMethods.consoleMeths.log("testcafe-hammerhead: unwrapped Location.prototype." + protoKey + " descriptor!");
	        }
	        return _this;
	    }
	    return LocationWrapper;
	}(LocationInheritor));
	// NOTE: window.Location in IE11 is object
	if (typeof Location !== 'function')
	    LocationWrapper.toString = function () { return Location.toString(); };
	else
	    overrideStringRepresentation(LocationWrapper, Location);

	var LOCATION_WRAPPER = 'hammerhead|location-wrapper';
	var LocationAccessorsInstrumentation = /** @class */ (function (_super) {
	    __extends(LocationAccessorsInstrumentation, _super);
	    function LocationAccessorsInstrumentation(_messageSandbox) {
	        var _this = _super.call(this) || this;
	        _this._messageSandbox = _messageSandbox;
	        _this.LOCATION_CHANGED_EVENT = 'hammerhead|event|location-changed';
	        _this._locationChangedEventCallback = function (e) { return _this.emit(_this.LOCATION_CHANGED_EVENT, e); };
	        return _this;
	    }
	    LocationAccessorsInstrumentation.isLocationWrapper = function (obj) {
	        return obj instanceof LocationWrapper;
	    };
	    LocationAccessorsInstrumentation.getLocationWrapper = function (owner) {
	        // NOTE: IE11 case. We can get cross-domain location wrapper without any exceptions.
	        // We return owner.location in this case, as in other browsers.
	        if (isIE && isCrossDomainWindows(window, owner))
	            return owner.location;
	        // NOTE: When the owner is cross-domain, we cannot get its location wrapper, so we return the original
	        // location, which cannot be accessed but behaves like a real one. Cross-domain location retains the 'replace'
	        // and 'assign' methods, so we intercept calls to them through MethodCallInstrumentation.
	        try {
	            return owner[LOCATION_WRAPPER];
	        }
	        catch (e) {
	            return owner.location;
	        }
	    };
	    LocationAccessorsInstrumentation.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        var document = window.document;
	        var locationWrapper = new LocationWrapper(window, this._messageSandbox, this._locationChangedEventCallback);
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        nativeMethods.objectDefineProperty(window, LOCATION_WRAPPER, { value: locationWrapper, configurable: true });
	        nativeMethods.objectDefineProperty(document, LOCATION_WRAPPER, { value: locationWrapper, configurable: true });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.getLocation, {
	            value: function (location) { return isLocation(location) ? locationWrapper : location; },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.setLocation, {
	            value: function (location, value) {
	                if (isLocation(location) && typeof value === 'string') {
	                    // @ts-ignore
	                    locationWrapper.href = value; // eslint-disable-line no-restricted-properties
	                    return value;
	                }
	                return null;
	            },
	            configurable: true
	        });
	    };
	    return LocationAccessorsInstrumentation;
	}(SandboxBase));

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	function inaccessibleTypeToStr(obj) {
	    return obj === null ? 'null' : 'undefined';
	}
	function isNullOrUndefined(obj) {
	    return obj === null || obj === void 0;
	}
	function isPrimitiveType(obj) {
	    var objType = typeof obj;
	    return objType !== 'object' && objType !== 'function';
	}

	var typeUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		inaccessibleTypeToStr: inaccessibleTypeToStr,
		isNullOrUndefined: isNullOrUndefined,
		isPrimitiveType: isPrimitiveType
	});

	var IFRAME_WINDOW_INITED = 'hammerhead|iframe-window-inited';
	var IframeSandbox = /** @class */ (function (_super) {
	    __extends(IframeSandbox, _super);
	    function IframeSandbox(_nodeMutation, _cookieSandbox) {
	        var _this = _super.call(this) || this;
	        _this._nodeMutation = _nodeMutation;
	        _this._cookieSandbox = _cookieSandbox;
	        _this.RUN_TASK_SCRIPT_EVENT = 'hammerhead|event|run-task-script';
	        _this.EVAL_HAMMERHEAD_SCRIPT_EVENT = 'hammerhead|event|eval-hammerhead-script';
	        _this.EVAL_EXTERNAL_SCRIPT_EVENT = 'hammerhead|event|eval-external-script';
	        _this.IFRAME_DOCUMENT_CREATED_EVENT = 'hammerhead|event|iframe-document-created';
	        _this.on(_this.RUN_TASK_SCRIPT_EVENT, _this.iframeReadyToInitHandler);
	        _this._nodeMutation.on(_this._nodeMutation.IFRAME_ADDED_TO_DOM_EVENT, function (iframe) { return _this.processIframe(iframe); });
	        _this.iframeNativeMethodsBackup = null;
	        return _this;
	    }
	    IframeSandbox.prototype._shouldSaveIframeNativeMethods = function (iframe) {
	        if (!isWebKit)
	            return false;
	        var iframeSrc = this.nativeMethods.getAttribute.call(iframe, 'src');
	        return DomProcessor.isJsProtocol(iframeSrc);
	    };
	    IframeSandbox.prototype._ensureIframeNativeMethodsForChrome = function (iframe) {
	        var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	        var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	        if (this.iframeNativeMethodsBackup) {
	            this.iframeNativeMethodsBackup.restoreDocumentMeths(contentWindow, contentDocument);
	            this.iframeNativeMethodsBackup = null;
	        }
	        else if (this._shouldSaveIframeNativeMethods(iframe))
	            // @ts-ignore
	            this.iframeNativeMethodsBackup = new this.nativeMethods.constructor(contentDocument, contentWindow);
	    };
	    IframeSandbox.prototype._ensureIframeNativeMethodsForIE = function (iframe) {
	        var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	        var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	        var iframeNativeMethods = contentWindow[INTERNAL_PROPS.iframeNativeMethods];
	        if (iframeNativeMethods) {
	            iframeNativeMethods.restoreDocumentMeths(contentWindow, contentDocument);
	            delete contentWindow[INTERNAL_PROPS.iframeNativeMethods];
	        }
	    };
	    IframeSandbox.prototype._ensureIframeNativeMethods = function (iframe) {
	        // NOTE: In Chrome, iframe with javascript protocol src raises the load event twice.
	        // As a result, when the second load event is raised, we write the overridden methods to the native methods.
	        // So, we need to save the native methods when the first load event is raised.
	        // https://code.google.com/p/chromium/issues/detail?id=578812
	        this._ensureIframeNativeMethodsForChrome(iframe);
	        // NOTE: Restore native document methods for the iframe's document if it overrided earlier (IE9, IE10 only)
	        // https://github.com/DevExpress/testcafe-hammerhead/issues/279
	        this._ensureIframeNativeMethodsForIE(iframe);
	    };
	    IframeSandbox.prototype._emitEvents = function (iframe) {
	        // NOTE: Raise this internal event to eval the Hammerhead code script.
	        this.emit(this.EVAL_HAMMERHEAD_SCRIPT_EVENT, { iframe: iframe });
	        // NOTE: Raise this event to eval external code script.
	        this.emit(this.EVAL_EXTERNAL_SCRIPT_EVENT, { iframe: iframe });
	        // NOTE: Raise this event to eval the "task" script and to call the Hammerhead initialization method
	        // and external script initialization code.
	        this.emit(this.RUN_TASK_SCRIPT_EVENT, iframe);
	    };
	    IframeSandbox.prototype._raiseReadyToInitEvent = function (iframe) {
	        if (!isIframeWithoutSrc(iframe))
	            return;
	        var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	        var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	        if (!IframeSandbox.isIframeInitialized(iframe)) {
	            // NOTE: Even if iframe is not loaded (iframe.contentDocument.documentElement does not exist), we
	            // still need to override the document.write method without initializing Hammerhead. This method can
	            // be called before iframe is fully loaded, we should override it now.
	            if (isNativeFunction(contentDocument.write))
	                this.emit(this.IFRAME_DOCUMENT_CREATED_EVENT, { iframe: iframe });
	        }
	        else if (!contentWindow[IFRAME_WINDOW_INITED] && !contentWindow[INTERNAL_PROPS.hammerhead]) {
	            this._ensureIframeNativeMethods(iframe);
	            // NOTE: Ok, the iframe is fully loaded now, but Hammerhead is not injected.
	            nativeMethods.objectDefineProperty(contentWindow, IFRAME_WINDOW_INITED, { value: true });
	            this._emitEvents(iframe);
	            contentWindow[INTERNAL_PROPS.processDomMethodName]();
	        }
	    };
	    IframeSandbox.isIframeInitialized = function (iframe) {
	        var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	        var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	        var isFFIframeUninitialized = isFirefox && contentWindow.document.readyState === 'uninitialized';
	        return !isFFIframeUninitialized && !!contentDocument.documentElement ||
	            isIE && contentWindow[INTERNAL_PROPS.documentWasCleaned];
	    };
	    IframeSandbox.isWindowInited = function (window) {
	        return window[IFRAME_WINDOW_INITED];
	    };
	    IframeSandbox.prototype.iframeReadyToInitHandler = function (iframe) {
	        // NOTE: We are using String.replace in order to avoid adding Mustache scripts on the client side.
	        // If it is needed elsewhere in a certain place, we should consider using Mustache.
	        var taskScriptTemplate = settings.get().iframeTaskScriptTemplate;
	        var escapeStringPatterns = function (str) { return str.replace(/\$/g, '$$$$'); };
	        var cookie = stringify(this._cookieSandbox.getCookie());
	        var referer = settings.get().referer || this.window.location.toString();
	        var iframeTaskScriptTemplate = stringify(taskScriptTemplate);
	        var taskScript = taskScriptTemplate
	            .replace('{{{cookie}}}', escapeStringPatterns(cookie))
	            .replace('{{{referer}}}', escapeStringPatterns(stringify(referer)))
	            .replace('{{{iframeTaskScriptTemplate}}}', escapeStringPatterns(iframeTaskScriptTemplate));
	        var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	        contentWindow.eval.call(contentWindow, taskScript);
	    };
	    IframeSandbox.prototype.onIframeBeganToRun = function (iframe) {
	        this._raiseReadyToInitEvent(iframe);
	    };
	    IframeSandbox.prototype.processIframe = function (el) {
	        var _this = this;
	        if (isShadowUIElement(el))
	            return;
	        // NOTE: the ready to init event will be raised by the self removing script
	        if (isIframeWithSrcdoc(el))
	            return;
	        if (isIframeElement(el) && nativeMethods.contentWindowGetter.call(el) ||
	            isFrameElement(el) && nativeMethods.frameContentWindowGetter.call(el))
	            this._raiseReadyToInitEvent(el);
	        // NOTE: This handler exists for iframes without the src attribute. In some the browsers (e.g. Chrome)
	        // the load event is triggering immediately after an iframe added to DOM. In other browsers,
	        // the _raiseReadyToInitEvent function is calling in our function wrapper after an iframe added to DOM.
	        this.nativeMethods.addEventListener.call(el, 'load', function () { return _this._raiseReadyToInitEvent(el); });
	    };
	    return IframeSandbox;
	}(SandboxBase));

	var BaseDomAdapter = /** @class */ (function () {
	    function BaseDomAdapter() {
	        this.EVENTS = ['onblur', 'onchange', 'onclick', 'oncontextmenu', 'oncopy', 'oncut',
	            'ondblclick', 'onerror', 'onfocus', 'onfocusin', 'onfocusout', 'onhashchange', 'onkeydown',
	            'onkeypress', 'onkeyup', 'onload', 'onmousedown', 'onmouseenter', 'onmouseleave',
	            'onmousemove', 'onmouseout', 'onmouseover', 'onmouseup', 'onmousewheel', 'onpaste', 'onreset',
	            'onresize', 'onscroll', 'onselect', 'onsubmit', 'ontextinput', 'onunload', 'onwheel',
	            'onpointerdown', 'onpointerup', 'onpointercancel', 'onpointermove', 'onpointerover', 'onpointerout',
	            'onpointerenter', 'onpointerleave', 'ongotpointercapture', 'onlostpointercapture',
	            'onmspointerdown', 'onmspointerup', 'onmspointercancel', 'onmspointermove', 'onmspointerover',
	            'onmspointerout', 'onmspointerenter', 'onmspointerleave', 'onmsgotpointercapture', 'onmslostpointercapture'
	        ];
	    }
	    return BaseDomAdapter;
	}());

	// -------------------------------------------------------------
	function create$1(script) {
	    return ("\n        <script class=\"" + SHADOW_UI_CLASS_NAME.selfRemovingScript + "\">\n            (function () {\n                var currentScript = document.currentScript;\n\n                /* NOTE: IE11 doesn't support the 'currentScript' property */\n                if (!currentScript) {\n                    var scripts       = document.scripts;\n                    var scriptsLength = scripts.length;\n\n                    currentScript = scripts[scriptsLength - 1];\n                }\n\n                currentScript.parentNode.removeChild(currentScript);\n\n                " + script + "\n            })();\n        </script>\n    ").replace(/\n\s*|\/\*[\S\s]*?\*\//g, '');
	}
	var sharedSelfRemovingScripts = {
	    iframeInit: create$1("\n        var parentHammerhead = null;\n\n        if (!window[\"" + INTERNAL_PROPS.hammerhead + "\"])\n            Object.defineProperty(window, \"" + INTERNAL_PROPS.documentWasCleaned + "\", { value: true, configurable: true });\n\n        try {\n            parentHammerhead = window.parent[\"" + INTERNAL_PROPS.hammerhead + "\"];\n        } catch(e) {}\n\n        if (parentHammerhead)\n            parentHammerhead.sandbox.onIframeDocumentRecreated(window.frameElement);\n    "),
	    onWindowRecreation: create$1("\n        var hammerhead = window[\"" + INTERNAL_PROPS.hammerhead + "\"];\n        var sandbox    = hammerhead && hammerhead.sandbox;\n\n        if (!sandbox) {\n            try {\n                sandbox = window.parent[\"" + INTERNAL_PROPS.hammerhead + "\"].sandboxUtils.backup.get(window);\n            } catch(e) {}\n        }\n\n        if (sandbox) {\n            Object.defineProperty(window, \"" + INTERNAL_PROPS.documentWasCleaned + "\", { value: true, configurable: true });\n\n            sandbox.node.mutation.onDocumentCleaned(window, document);\n\n            /* NOTE: B234357 */\n            sandbox.node.processNodes(null, document);\n        }\n    "),
	    onBodyCreated: create$1("\n        if (window[\"" + INTERNAL_PROPS.hammerhead + "\"])\n            window[\"" + INTERNAL_PROPS.hammerhead + "\"].sandbox.node.raiseBodyCreatedEvent();\n    "),
	    onOriginFirstTitleLoaded: create$1("\n        window[\"" + INTERNAL_PROPS.hammerhead + "\"].sandbox.node.onOriginFirstTitleElementInHeadLoaded();\n    "),
	    restoreStorages: create$1("\n        window.localStorage.setItem(\"%s\", %s);\n        window.sessionStorage.setItem(\"%s\", %s);\n    ")
	};

	var InsertPosition;
	(function (InsertPosition) {
	    InsertPosition["beforeBegin"] = "beforebegin";
	    InsertPosition["afterBegin"] = "afterbegin";
	    InsertPosition["beforeEnd"] = "beforeend";
	    InsertPosition["afterEnd"] = "afterend";
	})(InsertPosition || (InsertPosition = {}));
	var InsertPosition$1 = InsertPosition;

	function removeElement (element) {
	    var parent = nativeMethods.nodeParentNodeGetter.call(element);
	    if (parent)
	        nativeMethods.removeChild.call(parent, element);
	    return element;
	}

	var FAKE_TAG_NAME_PREFIX = 'hh_fake_tag_name_';
	var FAKE_DOCTYPE_TAG_NAME = 'hh_fake_doctype';
	var FAKE_HEAD_TAG_NAME = FAKE_TAG_NAME_PREFIX + "head";
	var FAKE_BODY_TAG_NAME = FAKE_TAG_NAME_PREFIX + "body";
	var FAKE_ATTR_WITH_TAG_NAME = 'hh_fake_attr';
	var FAKE_TAG_NAME_RE = new RegExp('(<\\/?)' + FAKE_TAG_NAME_PREFIX, 'ig');
	var WRAP_TAGS_RE = /(<\/?)(html|head|body|table|tbody|tfoot|thead|tr|td|th|caption|colgroup)((?:\s[^>]*)?>)/ig;
	var WRAP_TAGS_TEMPLATE = "$1" + FAKE_TAG_NAME_PREFIX + "$2$3";
	var WRAP_COL_NOSCRIPT_TAGS_RE = /<(\/?(?:col|noscript))(\s[^>]*?)?(\s?\/)?>/ig;
	var WRAP_COL_NOSCRIPT_TAGS_TEMPLATE = "<br " + FAKE_ATTR_WITH_TAG_NAME + "=\"$1|$3\"$2>";
	var UNWRAP_COL_NOSCRIPT_TAGS_RE = new RegExp("<br([^>]*?) " + FAKE_ATTR_WITH_TAG_NAME + "=\"([^|]+)\\|([^\"]*)\"([^>]*)", 'ig');
	var WRAP_DOCTYPE_RE = /<!doctype([^>]*)>/ig;
	var WRAP_DOCTYPE_TEMPLATE = "<" + FAKE_DOCTYPE_TAG_NAME + ">$1</" + FAKE_DOCTYPE_TAG_NAME + ">";
	var UNWRAP_DOCTYPE_RE = new RegExp("<" + FAKE_DOCTYPE_TAG_NAME + ">([\\S\\s]*?)</" + FAKE_DOCTYPE_TAG_NAME + ">", 'ig');
	var FIND_SVG_RE = /<svg\s?[^>]*>/ig;
	var FIND_NS_ATTRS_RE = /\s(?:NS[0-9]+:[^"']+('|")[\S\s]*?\1|[^:]+:NS[0-9]+=(?:""|''))/g;
	var STORED_ATTRS_SELECTOR = (function () {
	    var storedAttrs = [];
	    for (var _i = 0, URL_ATTRS_1 = URL_ATTRS; _i < URL_ATTRS_1.length; _i++) {
	        var attr = URL_ATTRS_1[_i];
	        storedAttrs.push(DomProcessor.getStoredAttrName(attr));
	    }
	    for (var _a = 0, ATTRS_WITH_SPECIAL_PROXYING_LOGIC_1 = ATTRS_WITH_SPECIAL_PROXYING_LOGIC; _a < ATTRS_WITH_SPECIAL_PROXYING_LOGIC_1.length; _a++) {
	        var attr = ATTRS_WITH_SPECIAL_PROXYING_LOGIC_1[_a];
	        storedAttrs.push(DomProcessor.getStoredAttrName(attr));
	    }
	    return '[' + storedAttrs.join('],[') + ']';
	})();
	var SHADOW_UI_ELEMENTS_SELECTOR = "[class*=\"" + SHADOW_UI_CLASS_NAME.postfix + "\"]";
	var HOVER_AND_FOCUS_PSEUDO_CLASS_ELEMENTS_SELECTOR = "[" + INTERNAL_ATTRIBUTES.hoverPseudoClass + "],[" + INTERNAL_ATTRIBUTES.focusPseudoClass + "]";
	var FAKE_ELEMENTS_SELECTOR = FAKE_HEAD_TAG_NAME + ", " + FAKE_BODY_TAG_NAME;
	var HTML_PARSER_ELEMENT_FLAG = 'hammerhead|html-parser-element-flag';
	var SCRIPT_AND_STYLE_SELECTOR = 'script,link[rel="stylesheet"]';
	var htmlDocument = nativeMethods.createHTMLDocument.call(document.implementation, 'title');
	var htmlParser = nativeMethods.createDocumentFragment.call(htmlDocument);
	htmlParser[HTML_PARSER_ELEMENT_FLAG] = true;
	function getHtmlDocument() {
	    try {
	        // NOTE: IE bug: access denied.
	        if (htmlDocument.location)
	            htmlDocument.location.toString();
	    }
	    catch (e) {
	        htmlDocument = nativeMethods.createHTMLDocument.call(document.implementation, 'title');
	        htmlParser = nativeMethods.createDocumentFragment.call(htmlDocument);
	        htmlParser[HTML_PARSER_ELEMENT_FLAG] = true;
	    }
	    return htmlDocument;
	}
	function wrapHtmlText(html) {
	    return html
	        .replace(WRAP_DOCTYPE_RE, WRAP_DOCTYPE_TEMPLATE)
	        .replace(WRAP_COL_NOSCRIPT_TAGS_RE, WRAP_COL_NOSCRIPT_TAGS_TEMPLATE)
	        .replace(WRAP_TAGS_RE, WRAP_TAGS_TEMPLATE);
	}
	function unwrapHtmlText(html) {
	    return html
	        .replace(UNWRAP_DOCTYPE_RE, '<!doctype$1>')
	        .replace(UNWRAP_COL_NOSCRIPT_TAGS_RE, '<$2$1$4$3')
	        .replace(FAKE_TAG_NAME_RE, '$1');
	}
	function isPageHtml(html) {
	    return /^\s*(<\s*(!doctype|html|head|body)[^>]*>)/i.test(html);
	}
	function processHtmlInternal(html, process) {
	    var container = nativeMethods.createElement.call(getHtmlDocument(), 'div');
	    html = wrapHtmlText(html);
	    nativeMethods.appendChild.call(htmlParser, container);
	    nativeMethods.elementInnerHTMLSetter.call(container, html);
	    var processedHtml = process(container) ? nativeMethods.elementInnerHTMLGetter.call(container) : html;
	    removeElement(container);
	    processedHtml = unwrapHtmlText(processedHtml);
	    // NOTE: hack for IE (GH-1083)
	    if (isIE && !isMSEdge && html !== processedHtml)
	        processedHtml = removeExtraSvgNamespaces(html, processedHtml);
	    return processedHtml;
	}
	function cleanUpUrlAttr(el) {
	    var urlAttr = domProcessor.getUrlAttr(el);
	    if (!urlAttr || !nativeMethods.hasAttribute.call(el, urlAttr))
	        return;
	    var storedAttr = DomProcessor.getStoredAttrName(urlAttr);
	    if (nativeMethods.hasAttribute.call(el, storedAttr)) {
	        nativeMethods.setAttribute.call(el, urlAttr, nativeMethods.getAttribute.call(el, storedAttr));
	        nativeMethods.removeAttribute.call(el, storedAttr);
	    }
	}
	function cleanUpAutocompleteAttr(el) {
	    if (!nativeMethods.hasAttribute.call(el, 'autocomplete'))
	        return;
	    var storedAttr = DomProcessor.getStoredAttrName('autocomplete');
	    if (nativeMethods.hasAttribute.call(el, storedAttr)) {
	        var storedAttrValue = nativeMethods.getAttribute.call(el, storedAttr);
	        if (DomProcessor.isAddedAutocompleteAttr('autocomplete', storedAttrValue))
	            nativeMethods.removeAttribute.call(el, 'autocomplete');
	        else
	            nativeMethods.setAttribute.call(el, 'autocomplete', storedAttrValue);
	        nativeMethods.removeAttribute.call(el, storedAttr);
	    }
	}
	function cleanUpTargetAttr(el) {
	    var targetAttr = domProcessor.getTargetAttr(el);
	    if (!targetAttr || !nativeMethods.hasAttribute.call(el, targetAttr))
	        return;
	    var storedAttr = DomProcessor.getStoredAttrName(targetAttr);
	    if (nativeMethods.hasAttribute.call(el, storedAttr)) {
	        nativeMethods.setAttribute.call(el, targetAttr, nativeMethods.getAttribute.call(el, storedAttr));
	        nativeMethods.removeAttribute.call(el, storedAttr);
	    }
	}
	function cleanUpSandboxAttr(el) {
	    if (domProcessor.adapter.getTagName(el) !== 'iframe' || !nativeMethods.hasAttribute.call(el, 'sandbox'))
	        return;
	    var storedAttr = DomProcessor.getStoredAttrName('sandbox');
	    if (nativeMethods.hasAttribute.call(el, storedAttr)) {
	        nativeMethods.setAttribute.call(el, 'sandbox', nativeMethods.getAttribute.call(el, storedAttr));
	        nativeMethods.removeAttribute.call(el, storedAttr);
	    }
	}
	function cleanUpStyleAttr(el) {
	    if (!nativeMethods.hasAttribute.call(el, 'style'))
	        return;
	    var storedAttr = DomProcessor.getStoredAttrName('style');
	    if (nativeMethods.hasAttribute.call(el, storedAttr)) {
	        nativeMethods.setAttribute.call(el, 'style', nativeMethods.getAttribute.call(el, storedAttr));
	        nativeMethods.removeAttribute.call(el, storedAttr);
	    }
	}
	function cleanUpHtml(html) {
	    return processHtmlInternal(html, function (container) {
	        var changed = false;
	        find(container, STORED_ATTRS_SELECTOR, function (el) {
	            cleanUpUrlAttr(el);
	            cleanUpAutocompleteAttr(el);
	            cleanUpTargetAttr(el);
	            cleanUpSandboxAttr(el);
	            cleanUpStyleAttr(el);
	            changed = true;
	        });
	        find(container, SHADOW_UI_ELEMENTS_SELECTOR, function (el) {
	            var parent = nativeMethods.nodeParentNodeGetter.call(el);
	            if (parent) {
	                nativeMethods.removeChild.call(parent, el);
	                changed = true;
	            }
	        });
	        find(container, 'script', function (el) {
	            var textContent = nativeMethods.nodeTextContentGetter.call(el);
	            var cleanedTextContent = remove$1(textContent);
	            if (textContent !== cleanedTextContent) {
	                nativeMethods.nodeTextContentSetter.call(el, cleanedTextContent);
	                changed = true;
	            }
	        });
	        find(container, 'style', function (el) {
	            var textContent = nativeMethods.nodeTextContentGetter.call(el);
	            var cleanedTextContent = styleProcessor.cleanUp(textContent, parseProxyUrl);
	            if (textContent !== cleanedTextContent) {
	                nativeMethods.nodeTextContentSetter.call(el, cleanedTextContent);
	                changed = true;
	            }
	        });
	        find(container, HOVER_AND_FOCUS_PSEUDO_CLASS_ELEMENTS_SELECTOR, function (el) {
	            nativeMethods.removeAttribute.call(el, INTERNAL_ATTRIBUTES.hoverPseudoClass);
	            nativeMethods.removeAttribute.call(el, INTERNAL_ATTRIBUTES.focusPseudoClass);
	            changed = true;
	        });
	        find(container, FAKE_ELEMENTS_SELECTOR, function (el) {
	            var innerHtml = nativeMethods.elementInnerHTMLGetter.call(el);
	            if (innerHtml.indexOf(sharedSelfRemovingScripts.iframeInit) !== -1) {
	                nativeMethods.elementInnerHTMLSetter.call(el, innerHtml.replace(sharedSelfRemovingScripts.iframeInit, ''));
	                changed = true;
	            }
	        });
	        return changed;
	    });
	}
	function processHtml(html, options) {
	    if (options === void 0) { options = {}; }
	    var parentTag = options.parentTag, prepareDom = options.prepareDom, processedContext = options.processedContext, isPage = options.isPage;
	    return processHtmlInternal(html, function (container) {
	        var doctypeElement = null;
	        var htmlElements = [];
	        var children = [];
	        var length = 0;
	        var storedBaseUrl = urlResolver.getBaseUrl(document);
	        if (prepareDom)
	            prepareDom(container);
	        if (nativeMethods.htmlCollectionLengthGetter.call(nativeMethods.elementChildrenGetter.call(container))) {
	            children = nativeMethods.elementQuerySelectorAll.call(container, '*');
	            length = nativeMethods.nodeListLengthGetter.call(children);
	        }
	        var base = nativeMethods.elementQuerySelector.call(container, 'base');
	        if (base)
	            urlResolver.updateBase(nativeMethods.getAttribute.call(base, 'href'), document);
	        for (var i = 0; i < length; i++) {
	            var child = children[i];
	            if (isScriptElement(child)) {
	                var scriptContent = nativeMethods.nodeTextContentGetter.call(child);
	                nativeMethods.nodeTextContentSetter.call(child, unwrapHtmlText(scriptContent));
	            }
	            child[INTERNAL_PROPS.processedContext] = processedContext;
	            domProcessor.processElement(child, convertToProxyUrl);
	            var elTagName = getTagName(child);
	            if (elTagName === FAKE_HEAD_TAG_NAME || elTagName === FAKE_BODY_TAG_NAME)
	                htmlElements.push(child);
	            else if (elTagName === FAKE_DOCTYPE_TAG_NAME)
	                doctypeElement = child;
	        }
	        if (!parentTag) {
	            if (htmlElements.length) {
	                for (var _i = 0, htmlElements_1 = htmlElements; _i < htmlElements_1.length; _i++) {
	                    var htmlElement = htmlElements_1[_i];
	                    var firstScriptOrStyle = nativeMethods.elementQuerySelector.call(htmlElement, SCRIPT_AND_STYLE_SELECTOR);
	                    if (firstScriptOrStyle)
	                        nativeMethods.insertAdjacentHTML.call(firstScriptOrStyle, InsertPosition$1.beforeBegin, sharedSelfRemovingScripts.iframeInit);
	                    else
	                        nativeMethods.insertAdjacentHTML.call(htmlElement, InsertPosition$1.beforeEnd, sharedSelfRemovingScripts.iframeInit);
	                }
	            }
	            else if (doctypeElement && isIE)
	                nativeMethods.insertAdjacentHTML.call(doctypeElement, InsertPosition$1.afterEnd, sharedSelfRemovingScripts.iframeInit);
	            else if (isPage)
	                nativeMethods.insertAdjacentHTML.call(container, InsertPosition$1.afterBegin, sharedSelfRemovingScripts.iframeInit);
	        }
	        urlResolver.updateBase(storedBaseUrl, document);
	        return true;
	    });
	}
	function dispose$1() {
	    htmlParser = null;
	    htmlDocument = null;
	}
	function isInternalHtmlParserElement(el) {
	    while (nativeMethods.nodeParentNodeGetter.call(el))
	        el = nativeMethods.nodeParentNodeGetter.call(el);
	    return !!el[HTML_PARSER_ELEMENT_FLAG];
	}
	function removeExtraSvgNamespaces(html, processedHtml) {
	    var initialSvgStrs = html.match(FIND_SVG_RE);
	    var index = 0;
	    if (!initialSvgStrs)
	        return processedHtml;
	    return processedHtml.replace(FIND_SVG_RE, function (svgStr) {
	        var initialSvgStr = initialSvgStrs[index];
	        var initialNSAttrs = initialSvgStr ? initialSvgStr.match(FIND_NS_ATTRS_RE) : null;
	        if (initialSvgStr)
	            index++;
	        return initialSvgStr ? svgStr.replace(FIND_NS_ATTRS_RE, function () {
	            var replacement = initialNSAttrs ? initialNSAttrs.join('') : '';
	            if (initialNSAttrs)
	                initialNSAttrs = null;
	            return replacement;
	        }) : svgStr;
	    });
	}

	var htmlUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		isPageHtml: isPageHtml,
		cleanUpHtml: cleanUpHtml,
		processHtml: processHtml,
		dispose: dispose$1,
		isInternalHtmlParserElement: isInternalHtmlParserElement
	});

	var BEGIN_MARKER_TAG_NAME = 'hammerhead_write_marker_begin';
	var END_MARKER_TAG_NAME = 'hammerhead_write_marker_end';
	var BEGIN_MARKER_MARKUP = "<" + BEGIN_MARKER_TAG_NAME + "></" + BEGIN_MARKER_TAG_NAME + ">";
	var END_MARKER_MARKUP = "<" + END_MARKER_TAG_NAME + "></" + END_MARKER_TAG_NAME + ">";
	var BEGIN_REMOVE_RE = new RegExp("^[\\S\\s]*" + BEGIN_MARKER_MARKUP, 'g');
	var END_REMOVE_RE = new RegExp(END_MARKER_MARKUP + "[\\S\\s]*$", 'g');
	var REMOVE_OPENING_TAG_RE = /^<[^>]+>/g;
	var REMOVE_CLOSING_TAG_RE = /<\/[^<>]+>$/g;
	var PENDING_RE = /<\/?(?:[A-Za-z][^>]*)?$/g;
	var UNCLOSED_ELEMENT_FLAG = 'hammerhead|unclosed-element-flag';
	var DocumentWriter = /** @class */ (function () {
	    function DocumentWriter(window, document) {
	        this.window = window;
	        this.document = document;
	        this.pending = '';
	        this.parentTagChain = [];
	        this.isBeginMarkerInDOM = false;
	        this.isEndMarkerInDOM = false;
	        this.isClosingContentEl = false;
	        this.isNonClosedComment = false;
	        this.isAddContentToEl = false;
	        this.contentForProcessing = '';
	        this.nonClosedEl = null;
	        this.cachedStartsWithClosingTagRegExps = {};
	    }
	    DocumentWriter.prototype._cutPending = function (htmlChunk) {
	        var match = htmlChunk.match(PENDING_RE);
	        this.pending = match ? match[0] : '';
	        return this.pending ? htmlChunk.substring(0, htmlChunk.length - this.pending.length) : htmlChunk;
	    };
	    DocumentWriter.prototype._wrapHtmlChunk = function (htmlChunk) {
	        var parentTagChainMarkup = this.parentTagChain.length ? '<' + this.parentTagChain.join('><') + '>' : '';
	        if (this.isNonClosedComment)
	            parentTagChainMarkup += '<!--';
	        return parentTagChainMarkup + BEGIN_MARKER_MARKUP + htmlChunk + END_MARKER_MARKUP;
	    };
	    DocumentWriter.prototype._unwrapHtmlChunk = function (htmlChunk) {
	        if (!htmlChunk)
	            return htmlChunk;
	        htmlChunk = htmlChunk
	            .replace(BEGIN_REMOVE_RE, '')
	            .replace(END_REMOVE_RE, '');
	        if (!this.isBeginMarkerInDOM)
	            htmlChunk = this.isNonClosedComment ? htmlChunk.slice(4) : htmlChunk.replace(REMOVE_OPENING_TAG_RE, '');
	        if (!this.isEndMarkerInDOM)
	            htmlChunk = this.isNonClosedComment ? htmlChunk.slice(0, -3) : htmlChunk.replace(REMOVE_CLOSING_TAG_RE, '');
	        if (!this.isBeginMarkerInDOM && this.isEndMarkerInDOM)
	            this.isNonClosedComment = false;
	        return htmlChunk;
	    };
	    DocumentWriter._setUnclosedElementFlag = function (el) {
	        if (isScriptElement(el) || isStyleElement(el))
	            el[UNCLOSED_ELEMENT_FLAG] = true;
	    };
	    DocumentWriter.hasUnclosedElementFlag = function (el) {
	        return !!el[UNCLOSED_ELEMENT_FLAG];
	    };
	    DocumentWriter._searchBeginMarker = function (container) {
	        var beginMarker = nativeMethods.elementQuerySelector.call(container, BEGIN_MARKER_TAG_NAME);
	        if (beginMarker)
	            return beginMarker;
	        beginMarker = container;
	        while (nativeMethods.elementFirstElementChildGetter.call(beginMarker))
	            beginMarker = nativeMethods.elementFirstElementChildGetter.call(beginMarker);
	        var beginMarkerParent = nativeMethods.nodeParentNodeGetter.call(beginMarker);
	        if (nativeMethods.nodeFirstChildGetter.call(beginMarkerParent) !== beginMarker)
	            beginMarker = nativeMethods.nodeFirstChildGetter.call(beginMarkerParent);
	        else if (isCommentNode(nativeMethods.nodeFirstChildGetter.call(beginMarker)))
	            beginMarker = nativeMethods.nodeFirstChildGetter.call(beginMarker);
	        return beginMarker;
	    };
	    DocumentWriter._searchEndMarker = function (container) {
	        var endMarker = nativeMethods.elementQuerySelector.call(container, END_MARKER_TAG_NAME);
	        if (endMarker)
	            return endMarker;
	        endMarker = container;
	        while (nativeMethods.elementLastElementChildGetter.call(endMarker))
	            endMarker = nativeMethods.elementLastElementChildGetter.call(endMarker);
	        var endMarkerParent = nativeMethods.nodeParentNodeGetter.call(endMarker);
	        if (nativeMethods.nodeLastChildGetter.call(endMarkerParent) !== endMarker)
	            endMarker = nativeMethods.nodeLastChildGetter.call(endMarkerParent);
	        else if (isCommentNode(nativeMethods.nodeLastChildGetter.call(endMarker)))
	            endMarker = nativeMethods.nodeLastChildGetter.call(endMarker);
	        return endMarker;
	    };
	    DocumentWriter.prototype._updateParentTagChain = function (container, endMarker) {
	        var endMarkerParent = getTagName(endMarker) !== END_MARKER_TAG_NAME ? endMarker : nativeMethods.nodeParentNodeGetter.call(endMarker);
	        if (isCommentNode(endMarker)) {
	            this.isNonClosedComment = true;
	            endMarkerParent = nativeMethods.nodeParentNodeGetter.call(endMarker);
	        }
	        this.parentTagChain = [];
	        while (endMarkerParent !== container) {
	            this.parentTagChain.unshift(getTagName(endMarkerParent));
	            endMarkerParent = nativeMethods.nodeParentNodeGetter.call(endMarkerParent);
	        }
	    };
	    DocumentWriter.prototype._processBeginMarkerInContent = function (beginMarker) {
	        var elWithContent = beginMarker;
	        DocumentWriter._setUnclosedElementFlag(elWithContent);
	        if (this.isClosingContentEl && (isScriptElement(elWithContent) || isStyleElement(elWithContent))) {
	            this.contentForProcessing = nativeMethods.nodeTextContentGetter.call(this.nonClosedEl) +
	                nativeMethods.nodeTextContentGetter.call(elWithContent).replace(BEGIN_REMOVE_RE, '');
	            nativeMethods.nodeTextContentSetter.call(elWithContent, '');
	        }
	        else {
	            var textContent = nativeMethods.nodeTextContentGetter.call(elWithContent);
	            nativeMethods.nodeTextContentSetter.call(elWithContent, textContent.replace(BEGIN_REMOVE_RE, ''));
	        }
	        beginMarker = nativeMethods.createElement.call(document, BEGIN_MARKER_TAG_NAME);
	        var elWithContentParent = nativeMethods.nodeParentNodeGetter.call(elWithContent);
	        nativeMethods.insertBefore.call(elWithContentParent, beginMarker, elWithContent);
	    };
	    DocumentWriter._createStartsWithClosingTagRegExp = function (tagName) {
	        var regExpStrParts = [tagName.charAt(tagName.length - 1), '?'];
	        for (var i = tagName.length - 2; i > -1; i--) {
	            regExpStrParts.unshift('(?:', tagName.charAt(i));
	            regExpStrParts.push(')?');
	        }
	        regExpStrParts.unshift('^</');
	        regExpStrParts.push('$');
	        return new RegExp(regExpStrParts.join(''), 'i');
	    };
	    DocumentWriter.prototype._getStartsWithClosingTagRegExp = function (tagName) {
	        tagName = tagName.toLowerCase();
	        if (!this.cachedStartsWithClosingTagRegExps[tagName])
	            this.cachedStartsWithClosingTagRegExps[tagName] = DocumentWriter._createStartsWithClosingTagRegExp(tagName);
	        return this.cachedStartsWithClosingTagRegExps[tagName];
	    };
	    DocumentWriter.prototype._processEndMarkerInContent = function (endMarker) {
	        var elWithContent = endMarker;
	        var textContent = nativeMethods.nodeTextContentGetter.call(elWithContent);
	        DocumentWriter._setUnclosedElementFlag(elWithContent);
	        nativeMethods.nodeTextContentSetter.call(elWithContent, textContent.replace(END_REMOVE_RE, ''));
	        endMarker = nativeMethods.createElement.call(document, END_MARKER_TAG_NAME);
	        if (this.pending) {
	            var startsWithClosingTagRegExp = this._getStartsWithClosingTagRegExp(elWithContent.tagName);
	            var isPendingStartsWithClosingTagPart = startsWithClosingTagRegExp.test(this.pending);
	            if (!isPendingStartsWithClosingTagPart) {
	                var newContent = nativeMethods.nodeTextContentGetter.call(elWithContent) + this.pending;
	                nativeMethods.nodeTextContentSetter.call(elWithContent, newContent);
	                this.pending = '';
	            }
	        }
	        var elWithContentParent = nativeMethods.nodeParentNodeGetter.call(elWithContent);
	        nativeMethods.appendChild.call(elWithContentParent, endMarker);
	    };
	    DocumentWriter._addOnDocumentRecreationScript = function (endMarker) {
	        var span = nativeMethods.createElement.call(endMarker.ownerDocument, 'span');
	        var endMarkerParent = nativeMethods.nodeParentNodeGetter.call(endMarker);
	        nativeMethods.insertBefore.call(endMarkerParent, span, endMarker);
	        nativeMethods.elementOuterHTMLSetter.call(span, sharedSelfRemovingScripts.onWindowRecreation);
	    };
	    DocumentWriter.prototype._prepareDom = function (container, isDocumentCleaned) {
	        var beginMarker = DocumentWriter._searchBeginMarker(container);
	        var endMarker = DocumentWriter._searchEndMarker(container);
	        this.isBeginMarkerInDOM = getTagName(beginMarker) === BEGIN_MARKER_TAG_NAME;
	        this.isEndMarkerInDOM = getTagName(endMarker) === END_MARKER_TAG_NAME;
	        this.isAddContentToEl = beginMarker === endMarker;
	        this.isClosingContentEl = !this.isBeginMarkerInDOM && !this.isAddContentToEl;
	        if (!this.isAddContentToEl) {
	            this._updateParentTagChain(container, endMarker);
	            if (isDocumentCleaned)
	                DocumentWriter._addOnDocumentRecreationScript(endMarker);
	        }
	        if (!this.isBeginMarkerInDOM && !this.isEndMarkerInDOM) {
	            this._processBeginMarkerInContent(beginMarker);
	            this._processEndMarkerInContent(endMarker);
	        }
	        else if (this.isBeginMarkerInDOM && !this.isEndMarkerInDOM)
	            this._processEndMarkerInContent(endMarker);
	        else if (!this.isBeginMarkerInDOM && this.isEndMarkerInDOM)
	            this._processBeginMarkerInContent(beginMarker);
	    };
	    DocumentWriter.prototype._processHtmlChunk = function (htmlChunk, isDocumentCleaned) {
	        var _this = this;
	        htmlChunk = this._cutPending(this.pending + htmlChunk);
	        htmlChunk = this._wrapHtmlChunk(htmlChunk);
	        htmlChunk = processHtml(htmlChunk, {
	            prepareDom: function (container) { return _this._prepareDom(container, isDocumentCleaned); },
	            processedContext: this.window
	        });
	        htmlChunk = this._unwrapHtmlChunk(htmlChunk);
	        // NOTE: Firefox and IE recreate a window instance during the document.write function execution (T213930).
	        if (htmlChunk && this.isBeginMarkerInDOM && (isFirefox || isIE) && !isPageHtml(htmlChunk))
	            htmlChunk = sharedSelfRemovingScripts.iframeInit + htmlChunk;
	        return htmlChunk;
	    };
	    DocumentWriter.prototype.write = function (args, ln, isDocumentCleaned) {
	        var htmlChunk = this._processHtmlChunk(nativeMethods.arrayJoin.call(args, ''), isDocumentCleaned);
	        if (this.nonClosedEl && this.contentForProcessing) {
	            var processedContent = this.contentForProcessing;
	            if (isScriptElement(this.nonClosedEl))
	                processedContent = processScript(this.contentForProcessing, true, false, convertToProxyUrl);
	            else if (isStyleElement(this.nonClosedEl))
	                processedContent = styleProcessor.process(this.contentForProcessing, getProxyUrl, true);
	            nativeMethods.nodeTextContentSetter.call(this.nonClosedEl, processedContent);
	            this.contentForProcessing = '';
	        }
	        var nativeWriteMethod = ln ? nativeMethods.documentWriteLn : nativeMethods.documentWrite;
	        var result = nativeWriteMethod.call(this.document, htmlChunk);
	        if (isDocumentCleaned && isIE)
	            return result;
	        if (!this.isEndMarkerInDOM && !this.isAddContentToEl) {
	            var el = this.document.documentElement;
	            while (nativeMethods.elementLastElementChildGetter.call(el))
	                el = nativeMethods.elementLastElementChildGetter.call(el);
	            this.nonClosedEl = el;
	        }
	        return result;
	    };
	    return DocumentWriter;
	}());

	var WINDOWS_STORAGE = 'hammerhead|windows-storage';
	function getStorage() {
	    var topSameDomainWindow = getTopSameDomainWindow(window);
	    var storage = topSameDomainWindow[WINDOWS_STORAGE];
	    if (!storage) {
	        storage = [];
	        nativeMethods.objectDefineProperty(topSameDomainWindow, WINDOWS_STORAGE, { value: storage });
	    }
	    return storage;
	}
	function add(wnd) {
	    var storage = getStorage();
	    for (var i = storage.length - 1; i >= 0; i--) {
	        try {
	            if (storage[i] === wnd)
	                return;
	        }
	        catch (e) {
	            storage.splice(i, 1);
	        }
	    }
	    storage.push(wnd);
	}
	function remove(wnd) {
	    var storage = getStorage();
	    var index = storage.indexOf(wnd);
	    if (index !== -1)
	        storage.splice(index, 1);
	}
	function findByName(name) {
	    var storage = getStorage();
	    for (var i = 0; i < storage.length; i++) {
	        try {
	            if (storage[i].name === name)
	                return storage[i];
	        }
	        catch (e) {
	            // NOTE: During loading, an iframe can be changed from same-domain to cross-domain.
	            // Iframe's window is reinitialized, and we add 2 windows to the window storages:
	            // one to the same-domain storage and another one to the cross-domain storage.
	            // We remove the cross-domain window from this storage
	            // because it is already added to the cross-domain window storage.
	            storage.splice(i, 1);
	            i--;
	        }
	    }
	    return null;
	}

	var windowStorage = /*#__PURE__*/Object.freeze({
		__proto__: null,
		add: add,
		remove: remove,
		findByName: findByName
	});

	var ClientDomAdapter = /** @class */ (function (_super) {
	    __extends(ClientDomAdapter, _super);
	    function ClientDomAdapter() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this._srcdocMode = false;
	        return _this;
	    }
	    ClientDomAdapter.prototype.removeAttr = function (el, attr) {
	        return nativeMethods.removeAttribute.call(el, attr);
	    };
	    ClientDomAdapter.prototype.getAttr = function (el, attr) {
	        return nativeMethods.getAttribute.call(el, attr);
	    };
	    ClientDomAdapter.prototype.hasAttr = function (el, attr) {
	        return el.hasAttribute(attr);
	    };
	    ClientDomAdapter.prototype.isSVGElement = function (el) {
	        return isSVGElement(el);
	    };
	    ClientDomAdapter.prototype.getClassName = function (el) {
	        return el.className;
	    };
	    ClientDomAdapter.prototype.hasEventHandler = function (el) {
	        var attributes = nativeMethods.elementAttributesGetter.call(el);
	        for (var _i = 0, attributes_1 = attributes; _i < attributes_1.length; _i++) {
	            var attr = attributes_1[_i];
	            if (this.EVENTS.indexOf(attr.name) !== -1)
	                return true;
	        }
	        return false;
	    };
	    ClientDomAdapter.prototype.getTagName = function (el) {
	        return getTagName(el);
	    };
	    ClientDomAdapter.prototype.setAttr = function (el, attr, value) {
	        return nativeMethods.setAttribute.call(el, attr, value);
	    };
	    ClientDomAdapter.prototype.setScriptContent = function (script, content) {
	        nativeMethods.scriptTextSetter.call(script, content);
	    };
	    ClientDomAdapter.prototype.getScriptContent = function (script) {
	        return nativeMethods.scriptTextGetter.call(script);
	    };
	    ClientDomAdapter.prototype.getStyleContent = function (style) {
	        return nativeMethods.elementInnerHTMLGetter.call(style);
	    };
	    ClientDomAdapter.prototype.setStyleContent = function (style, content) {
	        nativeMethods.elementInnerHTMLSetter.call(style, content);
	    };
	    ClientDomAdapter.prototype.needToProcessContent = function (el) {
	        return !DocumentWriter.hasUnclosedElementFlag(el);
	    };
	    ClientDomAdapter.prototype.needToProcessUrl = function () {
	        return true;
	    };
	    ClientDomAdapter.prototype.hasIframeParent = function (el) {
	        if (this._srcdocMode)
	            return true;
	        try {
	            if (el[INTERNAL_PROPS.processedContext])
	                return window.top !== el[INTERNAL_PROPS.processedContext];
	            return window.top.document !== findDocument(el);
	        }
	        catch (e) {
	            return true;
	        }
	    };
	    ClientDomAdapter.prototype.getProxyUrl = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        return getProxyUrl.apply(void 0, args);
	    };
	    ClientDomAdapter.prototype.isTopParentIframe = function (el) {
	        var elWindow = el[INTERNAL_PROPS.processedContext];
	        return elWindow && window.top === elWindow.parent;
	    };
	    ClientDomAdapter.prototype.sameOriginCheck = function (location, checkedUrl) {
	        return sameOriginCheck(location, checkedUrl);
	    };
	    ClientDomAdapter.prototype.isExistingTarget = function (target) {
	        return !!findByName(target);
	    };
	    ClientDomAdapter.prototype.processSrcdocAttr = function (html) {
	        this._srcdocMode = true;
	        var processedHtml = processHtml(html, { isPage: true });
	        this._srcdocMode = false;
	        return processedHtml;
	    };
	    return ClientDomAdapter;
	}(BaseDomAdapter));

	var domProcessor = new DomProcessor(new ClientDomAdapter());

	var DocumentSandbox = /** @class */ (function (_super) {
	    __extends(DocumentSandbox, _super);
	    function DocumentSandbox(_nodeSandbox, _shadowUI, _cookieSandbox, _documentTitleStorageInitializer) {
	        var _this = _super.call(this) || this;
	        _this._nodeSandbox = _nodeSandbox;
	        _this._shadowUI = _shadowUI;
	        _this._cookieSandbox = _cookieSandbox;
	        _this._documentTitleStorageInitializer = _documentTitleStorageInitializer;
	        _this.documentWriter = null;
	        return _this;
	    }
	    DocumentSandbox.forceProxySrcForImageIfNecessary = function (element) {
	        if (isImgElement(element) && settings.get().forceProxySrcForImage)
	            element[INTERNAL_PROPS.forceProxySrcForImage] = true;
	    };
	    DocumentSandbox._isDocumentInDesignMode = function (doc) {
	        return doc.designMode === 'on';
	    };
	    DocumentSandbox.prototype._isUninitializedIframeWithoutSrc = function (win) {
	        var frameElement = getFrameElement(win);
	        return win !== win.top && frameElement && isIframeWithoutSrc(frameElement) &&
	            !IframeSandbox.isIframeInitialized(frameElement);
	    };
	    DocumentSandbox.prototype._beforeDocumentCleaned = function () {
	        this._nodeSandbox.mutation.onBeforeDocumentCleaned(this.document);
	    };
	    DocumentSandbox.prototype._onDocumentClosed = function () {
	        this._nodeSandbox.mutation.onDocumentClosed(this.document);
	    };
	    DocumentSandbox._shouldEmitDocumentCleanedEvents = function (doc) {
	        if (isIE) {
	            if (doc.readyState !== 'loading')
	                return true;
	            var window_1 = doc.defaultView;
	            if (window_1[INTERNAL_PROPS.documentWasCleaned])
	                return false;
	            var iframe = window_1 && getFrameElement(window_1);
	            return iframe && isIframeWithoutSrc(iframe);
	        }
	        return doc.readyState !== 'loading' && doc.readyState !== 'uninitialized';
	    };
	    DocumentSandbox.prototype._performDocumentWrite = function (args, ln) {
	        var shouldEmitEvents = DocumentSandbox._shouldEmitDocumentCleanedEvents(this.document);
	        if (shouldEmitEvents)
	            this._beforeDocumentCleaned();
	        var result = this.documentWriter.write(args, ln, shouldEmitEvents);
	        // NOTE: B234357
	        if (!shouldEmitEvents)
	            this._nodeSandbox.processNodes(null, this.document);
	        return result;
	    };
	    DocumentSandbox.prototype._needToUpdateDocumentWriter = function (window, document) {
	        try {
	            return !this.documentWriter || this.window !== window || this.document !== document;
	        }
	        catch (e) {
	            return true;
	        }
	    };
	    DocumentSandbox._definePropertyDescriptor = function (owner, childOfOwner, prop, overriddenDescriptor) {
	        // NOTE: The 'URL', 'domain' and 'referrer' properties are non configurable in IE and Edge
	        if (!overriddenDescriptor.configurable) {
	            // NOTE: property doesn't redefined yet
	            if (!childOfOwner.hasOwnProperty(prop))
	                nativeMethods.objectDefineProperty(childOfOwner, prop, overriddenDescriptor);
	        }
	        else
	            nativeMethods.objectDefineProperty(owner, prop, overriddenDescriptor);
	    };
	    DocumentSandbox.prototype.iframeDocumentOpen = function (window, document, args) {
	        var iframe = window.frameElement;
	        var result = nativeMethods.documentOpen.apply(document, args);
	        nativeMethods.objectDefineProperty(window, INTERNAL_PROPS.documentWasCleaned, { value: true, configurable: true });
	        this._nodeSandbox.iframeSandbox.onIframeBeganToRun(iframe);
	        return result;
	    };
	    DocumentSandbox.prototype.attach = function (window, document, partialInitializationForNotLoadedIframe) {
	        var _this = this;
	        if (partialInitializationForNotLoadedIframe === void 0) { partialInitializationForNotLoadedIframe = false; }
	        if (this._needToUpdateDocumentWriter(window, document)) {
	            this.documentWriter = new DocumentWriter(window, document);
	            this._nodeSandbox.mutation.on(this._nodeSandbox.mutation.BEFORE_DOCUMENT_CLEANED_EVENT, function () {
	                _this.documentWriter = new DocumentWriter(window, document);
	            });
	        }
	        _super.prototype.attach.call(this, window, document);
	        var documentSandbox = this;
	        var docPrototype = window.Document.prototype;
	        var overriddenMethods = {
	            open: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var isUninitializedIframe = documentSandbox._isUninitializedIframeWithoutSrc(window);
	                if (!isUninitializedIframe)
	                    documentSandbox._beforeDocumentCleaned();
	                if (isIE)
	                    return window.parent[INTERNAL_PROPS.hammerhead].sandbox.node.doc.iframeDocumentOpen(window, this, args);
	                var result = nativeMethods.documentOpen.apply(this, args);
	                // NOTE: Chrome does not remove the "%hammerhead%" property from window
	                // after document.open call
	                var objectDefinePropertyFn = window[INTERNAL_PROPS.hammerhead]
	                    ? window[INTERNAL_PROPS.hammerhead].nativeMethods.objectDefineProperty
	                    : window.Object.defineProperty;
	                objectDefinePropertyFn(window, INTERNAL_PROPS.documentWasCleaned, { value: true, configurable: true });
	                if (!isUninitializedIframe)
	                    documentSandbox._nodeSandbox.mutation.onDocumentCleaned(window, this);
	                else
	                    // NOTE: If iframe initialization is in progress, we need to override the document.write and document.open
	                    // methods once again, because they were cleaned after the native document.open method call.
	                    documentSandbox.attach(window, this);
	                return result;
	            },
	            close: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                // NOTE: IE11 raise the "load" event only when the document.close method is called. We need to
	                // restore the overridden document.open and document.write methods before Hammerhead injection, if the
	                // window is not initialized.
	                if (isIE && !IframeSandbox.isWindowInited(window))
	                    nativeMethods.restoreDocumentMeths(window, this);
	                // NOTE: IE doesn't run scripts in iframe if iframe.documentContent.designMode equals 'on' (GH-871)
	                if (DocumentSandbox._isDocumentInDesignMode(this))
	                    ShadowUI.removeSelfRemovingScripts(this);
	                var result = nativeMethods.documentClose.apply(this, args);
	                if (!documentSandbox._isUninitializedIframeWithoutSrc(window))
	                    documentSandbox._onDocumentClosed();
	                var iframe = getFrameElement(window);
	                // NOTE: Firefox misses the Hammerhead instance after the iframe.contentDocument.close function calling (GH-1821)
	                if (iframe)
	                    documentSandbox._nodeSandbox.iframeSandbox.onIframeBeganToRun(iframe);
	                return result;
	            },
	            write: function () {
	                return documentSandbox._performDocumentWrite(arguments);
	            },
	            writeln: function () {
	                return documentSandbox._performDocumentWrite(arguments, true);
	            }
	        };
	        overrideFunction(window[nativeMethods.documentOpenPropOwnerName].prototype, 'open', overriddenMethods.open);
	        overrideFunction(window[nativeMethods.documentClosePropOwnerName].prototype, 'close', overriddenMethods.close);
	        overrideFunction(window[nativeMethods.documentWritePropOwnerName].prototype, 'write', overriddenMethods.write);
	        overrideFunction(window[nativeMethods.documentWriteLnPropOwnerName].prototype, 'writeln', overriddenMethods.writeln);
	        overrideFunction(document, 'open', overriddenMethods.open);
	        overrideFunction(document, 'close', overriddenMethods.close);
	        overrideFunction(document, 'write', overriddenMethods.write);
	        overrideFunction(document, 'writeln', overriddenMethods.writeln);
	        if (document.open !== overriddenMethods.open)
	            overrideFunction(document, 'open', overriddenMethods.open);
	        overrideFunction(docPrototype, 'createElement', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var el = nativeMethods.createElement.apply(this, args);
	            DocumentSandbox.forceProxySrcForImageIfNecessary(el);
	            domProcessor.processElement(el, convertToProxyUrl);
	            documentSandbox._nodeSandbox.processNodes(el);
	            return el;
	        });
	        overrideFunction(docPrototype, 'createElementNS', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var el = nativeMethods.createElementNS.apply(this, args);
	            DocumentSandbox.forceProxySrcForImageIfNecessary(el);
	            domProcessor.processElement(el, convertToProxyUrl);
	            documentSandbox._nodeSandbox.processNodes(el);
	            return el;
	        });
	        overrideFunction(docPrototype, 'createDocumentFragment', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var fragment = nativeMethods.createDocumentFragment.apply(this, args);
	            documentSandbox._nodeSandbox.processNodes(fragment);
	            return fragment;
	        });
	        var htmlDocPrototype = window.HTMLDocument.prototype;
	        var storedDomain = '';
	        if (nativeMethods.documentDocumentURIGetter) {
	            overrideDescriptor(docPrototype, 'documentURI', {
	                getter: function () {
	                    return getDestinationUrl(nativeMethods.documentDocumentURIGetter.call(this));
	                }
	            });
	        }
	        var referrerOverriddenDescriptor = createOverriddenDescriptor(docPrototype, 'referrer', {
	            getter: function () {
	                var referrer = getDestinationUrl(nativeMethods.documentReferrerGetter.call(this));
	                return isSpecialPage(referrer) ? '' : referrer;
	            }
	        });
	        DocumentSandbox._definePropertyDescriptor(docPrototype, htmlDocPrototype, 'referrer', referrerOverriddenDescriptor);
	        var urlOverriddenDescriptor = createOverriddenDescriptor(docPrototype, 'URL', {
	            getter: function () {
	                // eslint-disable-next-line no-restricted-properties
	                return LocationAccessorsInstrumentation.getLocationWrapper(this).href;
	            }
	        });
	        DocumentSandbox._definePropertyDescriptor(docPrototype, htmlDocPrototype, 'URL', urlOverriddenDescriptor);
	        var domainPropertyOwner = nativeMethods.objectHasOwnProperty.call(docPrototype, 'domain')
	            ? docPrototype
	            : htmlDocPrototype;
	        var domainOverriddenDescriptor = createOverriddenDescriptor(domainPropertyOwner, 'domain', {
	            getter: function () {
	                // eslint-disable-next-line no-restricted-properties
	                return storedDomain || LocationAccessorsInstrumentation.getLocationWrapper(window).hostname;
	            },
	            setter: function (value) {
	                storedDomain = value;
	            }
	        });
	        DocumentSandbox._definePropertyDescriptor(domainPropertyOwner, htmlDocPrototype, 'domain', domainOverriddenDescriptor);
	        overrideDescriptor(docPrototype, 'styleSheets', {
	            getter: function () {
	                var styleSheets = nativeMethods.documentStyleSheetsGetter.call(this);
	                return documentSandbox._shadowUI._filterStyleSheetList(styleSheets, styleSheets.length);
	            }
	        });
	        var documentCookiePropOwnerPrototype = window[nativeMethods.documentCookiePropOwnerName].prototype;
	        overrideDescriptor(documentCookiePropOwnerPrototype, 'cookie', {
	            getter: function () { return documentSandbox._cookieSandbox.getCookie(); },
	            setter: function (value) { return documentSandbox._cookieSandbox.setCookie(String(value)); }
	        });
	        overrideDescriptor(docPrototype, 'activeElement', {
	            getter: function () {
	                var activeElement = nativeMethods.documentActiveElementGetter.call(this);
	                if (activeElement && isShadowUIElement(activeElement))
	                    return documentSandbox._shadowUI.getLastActiveElement() || this.body;
	                return activeElement;
	            }
	        });
	        if (this._documentTitleStorageInitializer && !partialInitializationForNotLoadedIframe) {
	            overrideDescriptor(docPrototype, 'title', {
	                getter: function () {
	                    return documentSandbox._documentTitleStorageInitializer.storage.getTitle();
	                },
	                setter: function (value) {
	                    documentSandbox._documentTitleStorageInitializer.storage.setTitle(value);
	                }
	            });
	        }
	    };
	    return DocumentSandbox;
	}(SandboxBase));

	function createInput(form) {
	    var hiddenInput = nativeMethods.createElement.call(document, 'input');
	    hiddenInput.type = 'hidden';
	    hiddenInput.name = INTERNAL_ATTRIBUTES.uploadInfoHiddenInputName;
	    ShadowUI.markElementAsShadow(hiddenInput);
	    nativeMethods.inputValueSetter.call(hiddenInput, '[]');
	    nativeMethods.appendChild.call(form, hiddenInput);
	    ShadowUI.markFormAsShadow(form);
	    return hiddenInput;
	}
	function getInput(form) {
	    var inputSelector = '[name="' + INTERNAL_ATTRIBUTES.uploadInfoHiddenInputName + '"]';
	    return nativeMethods.elementQuerySelector.call(form, inputSelector) || createInput(form);
	}
	function indexOf(info, input) {
	    for (var index = 0; index < info.length; index++) {
	        if (info[index].id === input.id || info[index].name === input.name)
	            return index;
	    }
	    return -1;
	}
	function addInputInfo(input, fileList, value) {
	    var formInfo = getFormInfo(input);
	    if (formInfo) {
	        var files = [];
	        fileList = nativeMethods.arraySlice.call(fileList);
	        for (var i = 0, len = fileList.length; i < len; i++) {
	            var file = fileList[i];
	            files.push({
	                name: file.name,
	                type: file.type,
	                data: file.base64
	            });
	        }
	        var inputInfoIndex = indexOf(formInfo, input);
	        var inputInfo = {
	            id: input.id,
	            name: input.name,
	            files: files,
	            value: value
	        };
	        if (inputInfoIndex === -1)
	            formInfo.push(inputInfo);
	        else
	            formInfo[inputInfoIndex] = inputInfo;
	        setFormInfo(input, formInfo);
	    }
	}
	function getFormInfo(input) {
	    return input.form ? parse$1(nativeMethods.inputValueGetter.call(getInput(input.form))) : null;
	}
	function setFormInfo(input, info) {
	    if (input.form) {
	        var hiddenInput = getInput(input.form);
	        nativeMethods.inputValueSetter.call(hiddenInput, stringify(info));
	    }
	}
	function removeInputInfo(input) {
	    var uploadInfo = getFormInfo(input);
	    if (uploadInfo) {
	        var inputInfoIndex = indexOf(uploadInfo, input);
	        if (inputInfoIndex !== -1) {
	            uploadInfo.splice(inputInfoIndex, 1);
	            setFormInfo(input, uploadInfo);
	            return true;
	        }
	    }
	    return false;
	}

	var hiddenInfoUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		addInputInfo: addInputInfo,
		getFormInfo: getFormInfo,
		setFormInfo: setFormInfo,
		removeInputInfo: removeInputInfo
	});

	var COMPOSED_EVENTS = [
	    'blur',
	    'focus',
	    'focusin',
	    'focusout',
	    'click',
	    'dblclick',
	    'mousedown',
	    'mousemove',
	    'mouseout',
	    'mouseover',
	    'mouseup',
	    'beforeinput',
	    'input',
	    'keydown',
	    'keyup'
	];
	var BUTTON = {
	    left: 0,
	    middle: 1,
	    right: 2
	};
	var BUTTONS_PARAMETER = {
	    noButton: 0,
	    leftButton: 1,
	    rightButton: 2
	};
	var WHICH_PARAMETER = {
	    noButton: 0,
	    leftButton: 1,
	    middleButton: 2,
	    rightButton: 3
	};
	var KEYBOARD_MODIFIERS_PARAMETER = {
	    altKey: 'Alt',
	    ctrlKey: 'Control',
	    shiftKey: 'Shift',
	    metaKey: 'Meta'
	};
	var DOM_EVENTS = [
	    'click', 'dblclick', 'contextmenu',
	    'mousedown', 'mouseup', 'mousemove', 'mouseover', 'mouseout', 'mouseenter', 'mouseleave',
	    'touchstart', 'touchmove', 'touchend',
	    'keydown', 'keypress', 'keyup',
	    'textInput', 'textinput', 'input', 'change',
	    'focus', 'blur',
	    'MSPointerDown', 'MSPointerMove', 'MSPointerOver', 'MSPointerOut', 'MSPointerUp',
	    'pointerdown', 'pointermove', 'pointerover', 'pointerout', 'pointerup', 'pointerenter', 'pointerleave',
	    'dragstart', 'drop',
	    'focusin', 'focusout'
	];
	function preventDefault(ev, allowBubbling) {
	    if (ev.preventDefault)
	        ev.preventDefault();
	    else
	        ev.returnValue = false;
	    if (!allowBubbling)
	        stopPropagation(ev);
	}
	function stopPropagation(ev) {
	    if (ev.stopImmediatePropagation)
	        ev.stopImmediatePropagation();
	    else if (ev.stopPropagation)
	        ev.stopPropagation();
	    ev.cancelBubble = true;
	}
	function isObjectEventListener(listener) {
	    return typeof listener === 'object' && listener && typeof listener.handleEvent === 'function';
	}
	function isValidEventListener(listener) {
	    return typeof listener === 'function' || isObjectEventListener(listener);
	}
	function callEventListener(ctx, listener, e) {
	    if (isObjectEventListener(listener))
	        return listener.handleEvent.call(listener, e);
	    return listener.call(ctx, e);
	}
	function isComposedEvent(event) {
	    return COMPOSED_EVENTS.indexOf(event) !== -1;
	}
	var hasPointerEvents = !!(nativeMethods.WindowPointerEvent || nativeMethods.WindowMSPointerEvent);

	var eventUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		BUTTON: BUTTON,
		BUTTONS_PARAMETER: BUTTONS_PARAMETER,
		WHICH_PARAMETER: WHICH_PARAMETER,
		KEYBOARD_MODIFIERS_PARAMETER: KEYBOARD_MODIFIERS_PARAMETER,
		DOM_EVENTS: DOM_EVENTS,
		preventDefault: preventDefault,
		stopPropagation: stopPropagation,
		isObjectEventListener: isObjectEventListener,
		isValidEventListener: isValidEventListener,
		callEventListener: callEventListener,
		isComposedEvent: isComposedEvent,
		hasPointerEvents: hasPointerEvents
	});

	// OPTIMIZATION: http://jsperf.com/bind-apply
	function fnBind(func, thisObj) {
	    return function () {
	        return func.apply(thisObj, arguments);
	    };
	}

	var ATTRIBUTES_METHODS = ['setNamedItem', 'setNamedItemNS', 'removeNamedItem', 'removeNamedItemNS', 'getNamedItem', 'getNamedItemNS'];
	function createMethodWrapper(el, attributes, method) {
	    return function () {
	        var result = attributes[method].apply(attributes, arguments);
	        refreshAttributesWrapper(el);
	        return result;
	    };
	}
	var AttributesWrapper = /** @class */ (function () {
	    function AttributesWrapper(el, attributes) {
	        var _this = this;
	        assignAttributes(this, attributes);
	        this.item = function (index) { return _this[index]; };
	        for (var field in attributes) {
	            if (typeof this[field] === 'function' && field !== 'item') {
	                this[field] = ATTRIBUTES_METHODS.indexOf(field) !== -1
	                    ? createMethodWrapper(el, attributes, field)
	                    : fnBind(attributes[field], attributes);
	            }
	        }
	        if (attrGetNamedItemIsNotEnumerable)
	            this['getNamedItem'] = createMethodWrapper(el, attributes, 'getNamedItem');
	    }
	    return AttributesWrapper;
	}());

	var ATTRIBUTES_WRAPPER = 'hammerhead|element-attribute-wrappers';
	function cleanAttributes(wrapper) {
	    if (wrapper.length) {
	        for (var i = 0; i < wrapper.length; i++) {
	            delete wrapper[wrapper[i].name];
	            delete wrapper[i];
	        }
	    }
	}
	function assignAttributes(wrapper, attributes) {
	    var length = 0;
	    var properties = {};
	    for (var _i = 0, attributes_1 = attributes; _i < attributes_1.length; _i++) {
	        var attr = attributes_1[_i];
	        if (!isHammerheadAttr(attr.name)) {
	            var storedAttr = attributes[DomProcessor.getStoredAttrName(attr.name)];
	            if (storedAttr) {
	                // eslint-disable-next-line no-restricted-properties
	                if (DomProcessor.isAddedAutocompleteAttr(attr.name, storedAttr.value))
	                    continue;
	                attr = nativeMethods.cloneNode.call(attr);
	                // eslint-disable-next-line no-restricted-properties
	                attr.value = storedAttr.value;
	            }
	            properties[attr.name] = { value: attr, configurable: true, enumerable: true };
	            properties[length] = { value: attr, configurable: true };
	            length++;
	        }
	    }
	    properties['length'] = { value: length, configurable: true };
	    nativeMethods.objectDefineProperties(wrapper, properties);
	}
	function getAttributes(el) {
	    if (el[ATTRIBUTES_WRAPPER]) {
	        refreshAttributesWrapper(el);
	        return el[ATTRIBUTES_WRAPPER];
	    }
	    var attributes = nativeMethods.elementAttributesGetter.call(el);
	    if (!attributes)
	        return attributes;
	    for (var _i = 0, attributes_2 = attributes; _i < attributes_2.length; _i++) {
	        var attr = attributes_2[_i];
	        if (isHammerheadAttr(attr.name)) {
	            AttributesWrapper.prototype = attributes;
	            el[ATTRIBUTES_WRAPPER] = new AttributesWrapper(el, attributes);
	            return el[ATTRIBUTES_WRAPPER];
	        }
	    }
	    return attributes;
	}
	function refreshAttributesWrapper(el) {
	    var attributesWrapper = el[ATTRIBUTES_WRAPPER];
	    if (attributesWrapper) {
	        cleanAttributes(attributesWrapper);
	        assignAttributes(attributesWrapper, nativeMethods.elementAttributesGetter.call(el));
	    }
	}

	var DOMMutationTracker = /** @class */ (function () {
	    function DOMMutationTracker() {
	        var _this = this;
	        this._mutations = nativeMethods.objectCreate(null);
	        this._isDomContentLoaded = false;
	        nativeMethods.addEventListener.call(document, 'DOMContentLoaded', function () {
	            for (var _i = 0, _a = nativeMethods.objectKeys(_this._mutations); _i < _a.length; _i++) {
	                var tagName = _a[_i];
	                _this._updateVersion(tagName);
	            }
	            _this._isDomContentLoaded = true;
	        });
	    }
	    DOMMutationTracker.prototype._updateVersion = function (tagName) {
	        if (tagName in this._mutations)
	            this._mutations[tagName].increment();
	    };
	    DOMMutationTracker.prototype._processElement = function (el) {
	        if (!el.tagName || isShadowUIElement(el))
	            return;
	        var tagName = getTagName(el);
	        this._updateVersion('*');
	        this._updateVersion(tagName);
	    };
	    DOMMutationTracker.prototype._processChildren = function (el) {
	        if (!el.querySelectorAll)
	            return;
	        var children = getNativeQuerySelectorAll(el).call(el, '*');
	        var length = nativeMethods.nodeListLengthGetter.call(children);
	        for (var i = 0; i < length; i++)
	            this._processElement(children[i]);
	    };
	    DOMMutationTracker.prototype.onElementChanged = function (el) {
	        this._processElement(el);
	        this._processChildren(el);
	    };
	    DOMMutationTracker.prototype.onChildrenChanged = function (el) {
	        this._processChildren(el);
	    };
	    DOMMutationTracker.prototype.isDomContentLoaded = function () {
	        return this._isDomContentLoaded;
	    };
	    DOMMutationTracker.prototype.isOutdated = function (tagName, version) {
	        var isTagTracked = tagName in this._mutations;
	        if (!isTagTracked)
	            this._mutations[tagName] = new IntegerIdGenerator();
	        var lastVersion = this._mutations[tagName].value; // eslint-disable-line no-restricted-properties
	        return version < lastVersion;
	    };
	    DOMMutationTracker.prototype.getVersion = function (tagName) {
	        if (tagName in this._mutations)
	            return this._mutations[tagName].value; // eslint-disable-line no-restricted-properties
	        return -Infinity;
	    };
	    return DOMMutationTracker;
	}());
	var domMutationTracker = new DOMMutationTracker();

	var KEYWORD_TARGETS = ['_blank', '_self', '_parent', '_top'];
	function isKeywordTarget (value) {
	    if (value === void 0) { value = ''; }
	    value = value.toLowerCase();
	    return KEYWORD_TARGETS.indexOf(value) !== -1;
	}

	var RESTRICTED_META_HTTP_EQUIV_VALUES = [BUILTIN_HEADERS.refresh, BUILTIN_HEADERS.contentSecurityPolicy];
	var ElementSandbox = /** @class */ (function (_super) {
	    __extends(ElementSandbox, _super);
	    function ElementSandbox(_nodeSandbox, _uploadSandbox, _iframeSandbox, _shadowUI, _eventSandbox, _childWindowSandbox) {
	        var _this = _super.call(this) || this;
	        _this._nodeSandbox = _nodeSandbox;
	        _this._uploadSandbox = _uploadSandbox;
	        _this._iframeSandbox = _iframeSandbox;
	        _this._shadowUI = _shadowUI;
	        _this._eventSandbox = _eventSandbox;
	        _this._childWindowSandbox = _childWindowSandbox;
	        _this.BEFORE_FORM_SUBMIT_EVENT = 'hammerhead|event|before-form-submit';
	        _this.SCRIPT_ELEMENT_ADDED_EVENT = 'hammerhead|event|script-added';
	        _this.overriddenMethods = null;
	        return _this;
	    }
	    ElementSandbox._onTargetChanged = function (el) {
	        var tagName = getTagName(el);
	        var targetAttr = domProcessor.getTargetAttr(el);
	        if (!DomProcessor.isIframeFlagTag(tagName))
	            return;
	        var urlAttr = '';
	        if (targetAttr === 'target')
	            urlAttr = tagName === 'form' ? 'action' : 'href';
	        else if (targetAttr === 'formtarget')
	            urlAttr = 'formaction';
	        var storedUrlAttr = DomProcessor.getStoredAttrName(urlAttr);
	        if (el.hasAttribute(storedUrlAttr)) {
	            var url = el.getAttribute(storedUrlAttr);
	            if (isSupportedProtocol(url))
	                el.setAttribute(urlAttr, url);
	        }
	    };
	    ElementSandbox._setProxiedSrc = function (img) {
	        if (img[INTERNAL_PROPS.forceProxySrcForImage])
	            return;
	        var imgSrc = nativeMethods.imageSrcGetter.call(img);
	        var skipNextLoadEvent = !!imgSrc && img.complete && !img[INTERNAL_PROPS.cachedImage];
	        img[INTERNAL_PROPS.forceProxySrcForImage] = true;
	        if (imgSrc)
	            img.setAttribute('src', imgSrc);
	        img[INTERNAL_PROPS.skipNextLoadEventForImage] = skipNextLoadEvent;
	    };
	    ElementSandbox.prototype.getAttributeCore = function (el, args, isNs) {
	        var attr = String(args[isNs ? 1 : 0]);
	        var loweredAttr = attr.toLowerCase();
	        var ns = isNs ? args[0] : null;
	        var getAttrMeth = isNs ? nativeMethods.getAttributeNS : nativeMethods.getAttribute;
	        var tagName = getTagName(el);
	        if (loweredAttr === 'style')
	            return styleProcessor.cleanUp(getAttrMeth.apply(el, args), parseProxyUrl);
	        // OPTIMIZATION: The hasAttribute method is very slow.
	        if (domProcessor.isUrlAttr(el, loweredAttr, ns) ||
	            domProcessor.EVENTS.indexOf(loweredAttr) !== -1 ||
	            ATTRS_WITH_SPECIAL_PROXYING_LOGIC.indexOf(loweredAttr) !== -1) {
	            var storedAttrName = DomProcessor.getStoredAttrName(attr);
	            var storedAttrValue = getAttrMeth.apply(el, isNs ? [ns, storedAttrName] : [storedAttrName]);
	            if (DomProcessor.isAddedAutocompleteAttr(loweredAttr, storedAttrValue))
	                return null;
	            else if (el.hasAttribute(storedAttrName))
	                args[isNs ? 1 : 0] = storedAttrName;
	        }
	        else if (!isNs && (
	        // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
	        // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
	        // the correct SHA for the changed script. (GH-235)
	        loweredAttr === 'integrity' && DomProcessor.isTagWithIntegrityAttr(tagName) ||
	            // NOTE: We simply remove the 'rel' attribute if rel='prefetch' and use stored 'rel' attribute, because the prefetch
	            // resource type is unknown. https://github.com/DevExpress/testcafe/issues/2528
	            loweredAttr === 'rel' && tagName === 'link' ||
	            loweredAttr === 'required' && isFileInput(el) ||
	            loweredAttr === 'srcdoc' && tagName === 'iframe')) {
	            var storedAttr = DomProcessor.getStoredAttrName(attr);
	            if (nativeMethods.hasAttribute.call(el, storedAttr))
	                args[0] = storedAttr;
	        }
	        return getAttrMeth.apply(el, args);
	    };
	    ElementSandbox.prototype.setAttributeCore = function (el, args, isNs) {
	        var ns = isNs ? args[0] : null;
	        var attr = String(args[isNs ? 1 : 0]);
	        var loweredAttr = attr.toLowerCase();
	        var valueIndex = isNs ? 2 : 1;
	        var value = String(args[valueIndex]);
	        var setAttrMeth = isNs ? nativeMethods.setAttributeNS : nativeMethods.setAttribute;
	        var tagName = getTagName(el);
	        var isUrlAttr = domProcessor.isUrlAttr(el, attr, ns);
	        var isEventAttr = domProcessor.EVENTS.indexOf(attr) !== -1;
	        var needToCallTargetChanged = false;
	        var needToRecalcHref = false;
	        var isSpecialPage$1 = isSpecialPage(value);
	        var isSupportedProtocol$1 = isSupportedProtocol(value);
	        if (isUrlAttr && !isSupportedProtocol$1 && !isSpecialPage$1 || isEventAttr) {
	            var isJsProtocol = DomProcessor.isJsProtocol(value);
	            var storedJsAttr = DomProcessor.getStoredAttrName(attr);
	            if (isUrlAttr && isJsProtocol || isEventAttr)
	                args[valueIndex] = DomProcessor.processJsAttrValue(value, { isJsProtocol: isJsProtocol, isEventAttr: isEventAttr });
	            setAttrMeth.apply(el, isNs ? [ns, storedJsAttr, value] : [storedJsAttr, value]);
	        }
	        else if (isUrlAttr && (isSupportedProtocol$1 || isSpecialPage$1)) {
	            var storedUrlAttr = DomProcessor.getStoredAttrName(attr);
	            setAttrMeth.apply(el, isNs ? [ns, storedUrlAttr, value] : [storedUrlAttr, value]);
	            if (tagName !== 'img' || el[INTERNAL_PROPS.forceProxySrcForImage]) {
	                if (tagName === 'img')
	                    el[INTERNAL_PROPS.skipNextLoadEventForImage] = false;
	                if (value !== '' && (!isSpecialPage$1 || tagName === 'a')) {
	                    var isIframe = tagName === 'iframe' || tagName === 'frame';
	                    var isScript = tagName === 'script';
	                    var isCrossDomainUrl = isSupportedProtocol$1 && !sameOriginCheck(location.toString(), value);
	                    var resourceType = domProcessor.getElementResourceType(el);
	                    var elCharset = isScript && el.charset; // eslint-disable-line no-extra-parens
	                    var currentDocument = el.ownerDocument || this.document;
	                    if (loweredAttr === 'formaction' && !nativeMethods.hasAttribute.call(el, 'formtarget')) {
	                        resourceType = stringifyResourceType({ isForm: true });
	                        if (el.form && nativeMethods.hasAttribute.call(el.form, 'action')) { // eslint-disable-line no-extra-parens
	                            var parsedFormAction = parseProxyUrl(nativeMethods.formActionGetter.call(el.form)); // eslint-disable-line no-extra-parens
	                            if (parsedFormAction)
	                                resourceType = parsedFormAction.resourceType;
	                        }
	                    }
	                    if (ElementSandbox._isHrefAttrForBaseElement(el, attr) &&
	                        isElementInDocument(el, currentDocument))
	                        urlResolver.updateBase(value, currentDocument);
	                    args[valueIndex] = isIframe && isCrossDomainUrl
	                        ? getCrossDomainIframeProxyUrl(value)
	                        : getProxyUrl(value, { resourceType: resourceType, charset: elCharset, doc: currentDocument });
	                }
	            }
	            else if (value && !isSpecialPage$1 && !parseProxyUrl(value)) {
	                args[valueIndex] = el[INTERNAL_PROPS.forceProxySrcForImage]
	                    ? getProxyUrl(value)
	                    : resolveUrlAsDest(value);
	            }
	        }
	        else if (loweredAttr === 'autocomplete') {
	            var storedAutocompleteAttr = DomProcessor.getStoredAttrName(attr);
	            setAttrMeth.apply(el, isNs ? [ns, storedAutocompleteAttr, value] : [storedAutocompleteAttr, value]);
	            args[valueIndex] = 'off';
	        }
	        else if (loweredAttr === 'target' && DomProcessor.isTagWithTargetAttr(tagName) ||
	            loweredAttr === 'formtarget' && DomProcessor.isTagWithFormTargetAttr(tagName)) {
	            var currentTarget = nativeMethods.getAttribute.call(el, loweredAttr);
	            var newTarget = this.getCorrectedTarget(value);
	            if (newTarget !== currentTarget) {
	                var storedTargetAttr = DomProcessor.getStoredAttrName(attr);
	                setAttrMeth.apply(el, isNs ? [ns, storedTargetAttr, value] : [storedTargetAttr, value]);
	                args[valueIndex] = newTarget;
	                needToCallTargetChanged = true;
	            }
	            else
	                return null;
	        }
	        else if (attr === 'sandbox') {
	            var storedSandboxAttr = DomProcessor.getStoredAttrName(attr);
	            var allowSameOrigin = value.indexOf('allow-same-origin') !== -1;
	            var allowScripts = value.indexOf('allow-scripts') !== -1;
	            setAttrMeth.apply(el, isNs ? [ns, storedSandboxAttr, value] : [storedSandboxAttr, value]);
	            if (!allowSameOrigin || !allowScripts) {
	                args[valueIndex] += !allowSameOrigin ? ' allow-same-origin' : '';
	                args[valueIndex] += !allowScripts ? ' allow-scripts' : '';
	            }
	            if (el[this._nodeSandbox.win.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN])
	                el[this._nodeSandbox.win.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN](value);
	        }
	        // TODO: remove after https://github.com/DevExpress/testcafe-hammerhead/issues/244 implementation
	        else if (tagName === 'meta' && attr === 'http-equiv') {
	            var loweredValue = value.toLowerCase();
	            if (RESTRICTED_META_HTTP_EQUIV_VALUES.indexOf(loweredValue) !== -1)
	                return null;
	        }
	        else if (loweredAttr === 'xlink:href' &&
	            domProcessor.SVG_XLINK_HREF_TAGS.indexOf(tagName) !== -1 &&
	            isSVGElement(el)) {
	            var storedXLinkHrefAttr = DomProcessor.getStoredAttrName(attr);
	            setAttrMeth.apply(el, isNs ? [ns, storedXLinkHrefAttr, value] : [storedXLinkHrefAttr, value]);
	            if (!HASH_RE$1.test(value))
	                args[valueIndex] = getProxyUrl(value);
	        }
	        else if (loweredAttr === 'style')
	            args[valueIndex] = styleProcessor.process(value, getProxyUrl);
	        else if (!isNs && loweredAttr === 'integrity' && DomProcessor.isTagWithIntegrityAttr(tagName)) {
	            var storedIntegrityAttr = DomProcessor.getStoredAttrName(attr);
	            return setAttrMeth.apply(el, [storedIntegrityAttr, value]);
	        }
	        else if (!isNs && loweredAttr === 'rel' && tagName === 'link') {
	            var currentValue = nativeMethods.getAttribute.call(el, 'rel');
	            var formatedValue = trim(value.toLowerCase());
	            var storedRelAttr = DomProcessor.getStoredAttrName(attr);
	            needToRecalcHref = value !== currentValue && (value === domProcessor.MODULE_PRELOAD_LINK_REL ||
	                currentValue === domProcessor.MODULE_PRELOAD_LINK_REL);
	            if (formatedValue === 'prefetch') {
	                nativeMethods.removeAttribute.call(el, attr);
	                args[0] = storedRelAttr;
	            }
	            else
	                nativeMethods.removeAttribute.call(el, storedRelAttr);
	        }
	        else if (!isNs && loweredAttr === 'as' && tagName === 'link') {
	            var currentValue = nativeMethods.getAttribute.call(el, 'as');
	            needToRecalcHref = value !== currentValue && (value === domProcessor.PROCESSED_PRELOAD_LINK_CONTENT_TYPE ||
	                currentValue === domProcessor.PROCESSED_PRELOAD_LINK_CONTENT_TYPE);
	        }
	        else if (!isNs && loweredAttr === 'required' && isFileInput(el)) {
	            var storedRequiredAttr = DomProcessor.getStoredAttrName(attr);
	            nativeMethods.removeAttribute.call(el, attr);
	            args[0] = storedRequiredAttr;
	        }
	        else if (!isNs && loweredAttr === 'type' && isInputElement(el)) {
	            var currentType = nativeMethods.getAttribute.call(el, loweredAttr);
	            var newType = value.toLowerCase();
	            var storedRequiredAttr = DomProcessor.getStoredAttrName('required');
	            var currentRequired = nativeMethods.hasAttribute.call(el, storedRequiredAttr)
	                ? nativeMethods.getAttribute.call(el, storedRequiredAttr)
	                : nativeMethods.getAttribute.call(el, 'required');
	            var typeIsChanged = !currentType || newType !== currentType.toLowerCase();
	            if (typeIsChanged && currentRequired !== null) {
	                if (newType === 'file') {
	                    nativeMethods.setAttribute.call(el, storedRequiredAttr, currentRequired);
	                    nativeMethods.removeAttribute.call(el, 'required');
	                }
	                else if (currentType === 'file') {
	                    nativeMethods.setAttribute.call(el, 'required', currentRequired);
	                    nativeMethods.removeAttribute.call(el, storedRequiredAttr);
	                }
	            }
	        }
	        else if (!isNs && loweredAttr === 'srcdoc' && tagName === 'iframe') {
	            var storedAttr = DomProcessor.getStoredAttrName(attr);
	            setAttrMeth.apply(el, [storedAttr, value]);
	            args[valueIndex] = domProcessor.adapter.processSrcdocAttr(value);
	        }
	        var result = setAttrMeth.apply(el, args);
	        if (tagName === 'img' && !el[INTERNAL_PROPS.forceProxySrcForImage] && el.complete && !isFirefox) // eslint-disable-line no-extra-parens
	            el[INTERNAL_PROPS.cachedImage] = true;
	        if (needToCallTargetChanged)
	            ElementSandbox._onTargetChanged(el);
	        if (needToRecalcHref && nativeMethods.hasAttribute.call(el, 'href'))
	            this.setAttributeCore(el, ['href', nativeMethods.getAttribute.call(el, 'href')]);
	        return result;
	    };
	    ElementSandbox.prototype._hasAttributeCore = function (el, args, isNs) {
	        var attributeNameArgIndex = isNs ? 1 : 0;
	        var hasAttrMeth = isNs ? nativeMethods.hasAttributeNS : nativeMethods.hasAttribute;
	        var storedAutocompleteAttrName = DomProcessor.getStoredAttrName('autocomplete');
	        var storedAutocompleteAttrValue = nativeMethods.getAttribute.call(el, storedAutocompleteAttrName);
	        var tagName = getTagName(el);
	        if (typeof args[attributeNameArgIndex] === 'string' &&
	            DomProcessor.isAddedAutocompleteAttr(args[attributeNameArgIndex], storedAutocompleteAttrValue))
	            return false;
	        // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
	        // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
	        // the correct SHA for the changed script.
	        // _hasAttributeCore returns true for 'integrity' attribute if the stored attribute is exists. (GH-235)
	        else if (!isNs && args[0] === 'integrity' &&
	            DomProcessor.isTagWithIntegrityAttr(tagName))
	            args[0] = DomProcessor.getStoredAttrName('integrity');
	        // NOTE: We simply remove the 'rel' attribute if rel='prefetch' and use stored 'rel' attribute, because the prefetch
	        // resource type is unknown.
	        // _hasAttributeCore returns true for 'rel' attribute if the original 'rel' or stored attribute is exists.
	        // https://github.com/DevExpress/testcafe/issues/2528
	        else if (!isNs && args[0] === 'rel' && tagName === 'link') {
	            var storedRelAttr = DomProcessor.getStoredAttrName(args[0]);
	            return hasAttrMeth.apply(el, args) || hasAttrMeth.apply(el, [storedRelAttr]);
	        }
	        else if (!isNs && args[0] === 'required' && isFileInput(el)) {
	            var storedRequiredAttr = DomProcessor.getStoredAttrName(args[0]);
	            return hasAttrMeth.apply(el, args) || hasAttrMeth.call(el, storedRequiredAttr);
	        }
	        return hasAttrMeth.apply(el, args);
	    };
	    ElementSandbox.prototype.removeAttributeCore = function (el, args, isNs) {
	        var attr = String(args[isNs ? 1 : 0]);
	        var formatedAttr = attr.toLowerCase();
	        var removeAttrFunc = isNs ? nativeMethods.removeAttributeNS : nativeMethods.removeAttribute;
	        var tagName = getTagName(el);
	        var result = void 0;
	        if (domProcessor.isUrlAttr(el, formatedAttr, isNs ? args[0] : null) || formatedAttr === 'sandbox' ||
	            formatedAttr === 'autocomplete' ||
	            domProcessor.EVENTS.indexOf(formatedAttr) !== -1 ||
	            formatedAttr === 'target' && DomProcessor.isTagWithTargetAttr(tagName) ||
	            formatedAttr === 'formtarget' && DomProcessor.isTagWithFormTargetAttr(tagName)) {
	            var storedAttr = DomProcessor.getStoredAttrName(attr);
	            if (formatedAttr === 'autocomplete')
	                nativeMethods.setAttribute.call(el, storedAttr, domProcessor.AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER);
	            else
	                removeAttrFunc.apply(el, isNs ? [args[0], storedAttr] : [storedAttr]);
	        }
	        else if (!isNs && formatedAttr === 'rel' && tagName === 'link') {
	            var storedRelAttr = DomProcessor.getStoredAttrName(attr);
	            removeAttrFunc.apply(el, [storedRelAttr]);
	        }
	        else if (!isNs && formatedAttr === 'required' && isFileInput(el)) {
	            var storedRequiredAttr = DomProcessor.getStoredAttrName(attr);
	            removeAttrFunc.call(el, storedRequiredAttr);
	        }
	        else if (!isNs && formatedAttr === 'type' && isInputElement(el)) {
	            var storedRequiredAttr = DomProcessor.getStoredAttrName('required');
	            if (nativeMethods.hasAttribute.call(el, storedRequiredAttr)) {
	                var currentRequired = nativeMethods.getAttribute.call(el, storedRequiredAttr);
	                nativeMethods.setAttribute.call(el, 'required', currentRequired);
	                nativeMethods.removeAttribute.call(el, storedRequiredAttr);
	            }
	        }
	        if (ElementSandbox._isHrefAttrForBaseElement(el, formatedAttr))
	            urlResolver.updateBase(get$2(), this.document);
	        if (formatedAttr !== 'autocomplete')
	            result = removeAttrFunc.apply(el, args);
	        if (formatedAttr === 'target' && DomProcessor.isTagWithTargetAttr(tagName) ||
	            formatedAttr === 'formtarget' && DomProcessor.isTagWithFormTargetAttr(tagName))
	            ElementSandbox._onTargetChanged(el);
	        return result;
	    };
	    ElementSandbox._getChildNodesArray = function (args, range) {
	        var result = [];
	        var start = range[0], end = range[1];
	        for (var i = start; i < end; i++) {
	            var node = args[i];
	            if (isDocumentFragmentNode(node)) {
	                var childNodes = nativeMethods.nodeChildNodesGetter.call(node);
	                result.push.apply(result, nodeListToArray(childNodes));
	            }
	            else if (typeof node !== 'string')
	                result.push(node);
	        }
	        return result;
	    };
	    ElementSandbox.prototype._addNodeCore = function (parent, context, newNodesRange, args, nativeFn, checkBody) {
	        if (checkBody === void 0) { checkBody = true; }
	        this._prepareNodesForInsertion(args, newNodesRange, parent);
	        var result = null;
	        var childNodesArray = ElementSandbox._getChildNodesArray(args, newNodesRange);
	        // NOTE: Before the page's <body> is processed and added to DOM,
	        // some javascript frameworks create their own body element, perform
	        // certain manipulations and then remove it.
	        // Therefore, we need to check if the body element is present in DOM
	        if (checkBody && isBodyElementWithChildren(parent) && isElementInDocument(parent)) {
	            var newNodes = nativeMethods.arraySlice.apply(args, newNodesRange);
	            result = this._shadowUI.insertBeforeRoot(newNodes);
	        }
	        else
	            result = nativeFn.apply(context, args);
	        for (var _i = 0, childNodesArray_1 = childNodesArray; _i < childNodesArray_1.length; _i++) {
	            var child = childNodesArray_1[_i];
	            this._onElementAdded(child);
	        }
	        return result;
	    };
	    ElementSandbox.prototype._removeNodeCore = function (context, args, removingNode, nativeFn) {
	        this._onRemoveFileInputInfo(removingNode);
	        this._onRemoveIframe(removingNode);
	        var result = nativeFn.apply(context, args);
	        this._onElementRemoved(removingNode);
	        return result;
	    };
	    ElementSandbox.prototype._prepareNodesForInsertion = function (args, newNodesRange, parentNode) {
	        var start = newNodesRange[0], end = newNodesRange[1];
	        for (var i = start; i < end; i++) {
	            var node = args[i];
	            if (typeof node === 'string')
	                args[i] = ElementSandbox._processTextContent(node, parentNode);
	            else if (isTextNode(node))
	                node.data = ElementSandbox._processTextContent(node.data, parentNode);
	            else
	                this._nodeSandbox.processNodes(node);
	        }
	    };
	    ElementSandbox.prototype._insertAdjacentTextOrElement = function (context, args, nativeFn) {
	        var _a, _b, _c;
	        var position = (_c = (_a = args[0]) === null || _a === void 0 ? void 0 : (_b = _a).toLocaleLowerCase) === null || _c === void 0 ? void 0 : _c.call(_b);
	        var parent = position === InsertPosition$1.beforeBegin || position === InsertPosition$1.afterEnd
	            ? nativeMethods.nodeParentNodeGetter.call(context)
	            : context;
	        if (!parent)
	            return nativeMethods.insertAdjacentElement.apply(context, args);
	        return this._addNodeCore(parent, context, [1, 2], args, nativeFn, position === InsertPosition$1.beforeEnd);
	    };
	    ElementSandbox.prototype._createOverriddenMethods = function () {
	        // NOTE: We need the closure because a context of overridden methods is an html element
	        var sandbox = this;
	        this.overriddenMethods = {
	            appendData: function (text) {
	                var parentNode = nativeMethods.nodeParentNodeGetter.call(this);
	                nativeMethods.nodeTextContentSetter.call(this, nativeMethods.nodeTextContentGetter.call(this) + text);
	                if (parentNode)
	                    this.data = ElementSandbox._processTextContent(this.data, parentNode);
	            },
	            insertRow: function () {
	                var nativeMeth = isTableElement(this)
	                    ? nativeMethods.insertTableRow
	                    : nativeMethods.insertTBodyRow;
	                var row = nativeMeth.apply(this, arguments);
	                sandbox._nodeSandbox.processNodes(row);
	                return row;
	            },
	            insertCell: function () {
	                var cell = nativeMethods.insertCell.apply(this, arguments);
	                sandbox._nodeSandbox.processNodes(cell);
	                return cell;
	            },
	            insertAdjacentHTML: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var _a, _b, _c;
	                var position = (_c = (_a = args[0]) === null || _a === void 0 ? void 0 : (_b = _a).toLocaleLowerCase) === null || _c === void 0 ? void 0 : _c.call(_b);
	                var html = args[1];
	                var parent = position === InsertPosition$1.beforeBegin || position === InsertPosition$1.afterEnd
	                    ? nativeMethods.nodeParentNodeGetter.call(this)
	                    : this;
	                if (args.length > 1 && html !== null && parent) {
	                    args[1] = processHtml(String(html), {
	                        parentTag: parent['tagName'],
	                        processedContext: parent[INTERNAL_PROPS.processedContext]
	                    });
	                }
	                nativeMethods.insertAdjacentHTML.apply(this, args);
	                if (!parent)
	                    return;
	                sandbox._nodeSandbox.processNodes(parent);
	                domMutationTracker.onChildrenChanged(parent);
	            },
	            insertAdjacentElement: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._insertAdjacentTextOrElement(this, args, nativeMethods.insertAdjacentElement);
	            },
	            insertAdjacentText: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._insertAdjacentTextOrElement(this, args, nativeMethods.insertAdjacentText);
	            },
	            formSubmit: function () {
	                sandbox._ensureTargetContainsExistingBrowsingContext(this);
	                var args = { form: this, preventSubmit: false };
	                sandbox.emit(sandbox.BEFORE_FORM_SUBMIT_EVENT, args);
	                // HACK: For https://github.com/DevExpress/testcafe/issues/3560
	                // We have to cancel every form submit after a test is done
	                // to prevent requests to a closed session
	                if (!args.preventSubmit)
	                    return nativeMethods.formSubmit.apply(this, arguments);
	                return null;
	            },
	            insertBefore: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._addNodeCore(this, this, [0, 1], args, nativeMethods.insertBefore, !args[1]);
	            },
	            appendChild: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._addNodeCore(this, this, [0, 1], args, nativeMethods.appendChild);
	            },
	            append: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._addNodeCore(this, this, [0, args.length], args, nativeMethods.append);
	            },
	            prepend: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._addNodeCore(this, this, [0, args.length], args, nativeMethods.prepend, false);
	            },
	            after: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var parent = nativeMethods.nodeParentNodeGetter.call(this);
	                if (!parent)
	                    return nativeMethods.after.apply(this, args);
	                return sandbox._addNodeCore(parent, this, [0, args.length], args, nativeMethods.after, false);
	            },
	            removeChild: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._removeNodeCore(this, args, args[0], nativeMethods.removeChild);
	            },
	            remove: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                return sandbox._removeNodeCore(this, args, this, nativeMethods.remove);
	            },
	            elementReplaceWith: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var parentNode = nativeMethods.nodeParentNodeGetter.call(this);
	                if (!parentNode)
	                    return nativeMethods.elementReplaceWith.apply(this, args);
	                var newNodesRange = [0, args.length];
	                sandbox._prepareNodesForInsertion(args, newNodesRange, parentNode);
	                var childNodesArray = ElementSandbox._getChildNodesArray(args, newNodesRange);
	                sandbox._onRemoveFileInputInfo(this);
	                sandbox._onRemoveIframe(this);
	                var result = nativeMethods.elementReplaceWith.apply(this, args);
	                sandbox._onElementRemoved(this);
	                for (var _a = 0, childNodesArray_2 = childNodesArray; _a < childNodesArray_2.length; _a++) {
	                    var child = childNodesArray_2[_a];
	                    sandbox._onElementAdded(child);
	                }
	                return result;
	            },
	            replaceChild: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var newChild = args[0], oldChild = args[1];
	                if (isTextNode(newChild))
	                    newChild.data = ElementSandbox._processTextContent(newChild.data, this);
	                sandbox._onRemoveFileInputInfo(oldChild);
	                var result = nativeMethods.replaceChild.apply(this, arguments);
	                sandbox._onAddFileInputInfo(newChild);
	                domMutationTracker.onElementChanged(newChild);
	                domMutationTracker.onElementChanged(oldChild);
	                return result;
	            },
	            cloneNode: function () {
	                var clone = nativeMethods.cloneNode.apply(this, arguments);
	                sandbox._nodeSandbox.processNodes(clone);
	                return clone;
	            },
	            attachShadow: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var root = nativeMethods.attachShadow.apply(this, args);
	                nativeMethods.objectDefineProperty(root, SHADOW_ROOT_PARENT_ELEMENT, { value: this });
	                return root;
	            },
	            getAttribute: function () {
	                return sandbox.getAttributeCore(this, arguments);
	            },
	            getAttributeNS: function () {
	                return sandbox.getAttributeCore(this, arguments, true);
	            },
	            setAttribute: function () {
	                var result = sandbox.setAttributeCore(this, arguments);
	                refreshAttributesWrapper(this);
	                return result;
	            },
	            setAttributeNS: function () {
	                var result = sandbox.setAttributeCore(this, arguments, true);
	                refreshAttributesWrapper(this);
	                return result;
	            },
	            removeAttribute: function () {
	                var result = sandbox.removeAttributeCore(this, arguments);
	                refreshAttributesWrapper(this);
	                return result;
	            },
	            removeAttributeNS: function () {
	                var result = sandbox.removeAttributeCore(this, arguments, true);
	                refreshAttributesWrapper(this);
	                return result;
	            },
	            querySelector: function () {
	                if (typeof arguments[0] === 'string')
	                    arguments[0] = NodeSandbox.processSelector(arguments[0]);
	                return getNativeQuerySelector(this).apply(this, arguments);
	            },
	            querySelectorAll: function () {
	                if (typeof arguments[0] === 'string')
	                    arguments[0] = NodeSandbox.processSelector(arguments[0]);
	                return getNativeQuerySelectorAll(this).apply(this, arguments);
	            },
	            hasAttribute: function () {
	                return sandbox._hasAttributeCore(this, arguments, false);
	            },
	            hasAttributeNS: function () {
	                return sandbox._hasAttributeCore(this, arguments, true);
	            },
	            hasAttributes: function () {
	                if (nativeMethods.elementAttributesGetter.call(this).length === 2 &&
	                    nativeMethods.elementAttributesGetter.call(this).getNamedItem('autocomplete') &&
	                    nativeMethods.elementAttributesGetter.call(this).getNamedItem(DomProcessor.getStoredAttrName('autocomplete')))
	                    return sandbox._hasAttributeCore(this, ['autocomplete'], false);
	                return nativeMethods.hasAttributes.apply(this, arguments);
	            },
	            anchorToString: function () {
	                return getDestinationUrl(nativeMethods.anchorToString.call(this));
	            },
	            registerElement: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var opts = args[1];
	                if (opts && opts.prototype && opts.prototype.createdCallback) {
	                    var storedCreatedCallback_1 = opts.prototype.createdCallback;
	                    opts.prototype.createdCallback = function () {
	                        if (!isInternalHtmlParserElement(this))
	                            storedCreatedCallback_1.call(this);
	                    };
	                }
	                return nativeMethods.registerElement.apply(this, args);
	            }
	        };
	    };
	    ElementSandbox._processTextContent = function (str, parentNode) {
	        if (!parentNode['tagName'])
	            return str;
	        if (isScriptElement(parentNode))
	            return processScript(str, true, false, convertToProxyUrl);
	        if (isStyleElement(parentNode))
	            return styleProcessor.process(str, getProxyUrl);
	        return str;
	    };
	    ElementSandbox._isHrefAttrForBaseElement = function (el, attr) {
	        return isBaseElement(el) && attr === 'href';
	    };
	    ElementSandbox._removeFileInputInfo = function (el) {
	        removeInputInfo(el);
	    };
	    ElementSandbox._hasShadowUIParentOrContainsShadowUIClassPostfix = function (el) {
	        var parent = nativeMethods.nodeParentNodeGetter.call(el);
	        return parent && isShadowUIElement(parent) || ShadowUI.containsShadowUIClassPostfix(el);
	    };
	    ElementSandbox.prototype._isFirstBaseTagOnPage = function (el) {
	        var doc = el.ownerDocument || this.document;
	        return nativeMethods.querySelector.call(doc, 'base') === el;
	    };
	    ElementSandbox.prototype._onAddFileInputInfo = function (el) {
	        if (!isDomElement(el))
	            return;
	        var fileInputs = getFileInputs(el);
	        for (var _i = 0, fileInputs_1 = fileInputs; _i < fileInputs_1.length; _i++) {
	            var fileInput = fileInputs_1[_i];
	            this.addFileInputInfo(fileInput);
	        }
	    };
	    ElementSandbox.prototype._onRemoveFileInputInfo = function (el) {
	        if (!isDomElement(el))
	            return;
	        if (isFileInput(el))
	            ElementSandbox._removeFileInputInfo(el);
	        else
	            find(el, 'input[type=file]', ElementSandbox._removeFileInputInfo);
	    };
	    ElementSandbox.prototype._onRemoveIframe = function (el) {
	        if (isDomElement(el) && isIframeElement(el))
	            remove(nativeMethods.contentWindowGetter.call(el));
	    };
	    ElementSandbox.prototype._onElementAdded = function (el) {
	        if (ElementSandbox._hasShadowUIParentOrContainsShadowUIClassPostfix(el))
	            ShadowUI.markElementAndChildrenAsShadow(el);
	        if ((isDomElement(el) || isDocument(el)) && isElementInDocument(el)) {
	            var iframes = getIframes(el);
	            for (var _i = 0, iframes_1 = iframes; _i < iframes_1.length; _i++) {
	                var iframe = iframes_1[_i];
	                this.onIframeAddedToDOM(iframe);
	                add(nativeMethods.contentWindowGetter.call(iframe));
	            }
	            var scripts = getScripts(el);
	            for (var _a = 0, scripts_1 = scripts; _a < scripts_1.length; _a++) {
	                var script = scripts_1[_a];
	                this.emit(this.SCRIPT_ELEMENT_ADDED_EVENT, { el: script });
	            }
	            domMutationTracker.onElementChanged(el);
	        }
	        // NOTE: recalculate `formaction` attribute value if it placed in the dom
	        if ((isInputElement(el) || isButtonElement(el)) && el.form &&
	            nativeMethods.hasAttribute.call(el, 'formaction'))
	            el.setAttribute('formaction', el.getAttribute('formaction'));
	        if (isBodyElement(el))
	            this._shadowUI.onBodyElementMutation();
	        this._onAddFileInputInfo(el);
	        if (isBaseElement(el) && this._isFirstBaseTagOnPage(el)) {
	            var storedHrefAttrName = DomProcessor.getStoredAttrName('href');
	            var storedHrefAttrValue = el.getAttribute(storedHrefAttrName);
	            if (storedHrefAttrValue !== null)
	                urlResolver.updateBase(storedHrefAttrValue, this.document);
	        }
	    };
	    ElementSandbox.prototype._onElementRemoved = function (el) {
	        if (isBodyElement(el))
	            this._shadowUI.onBodyElementMutation();
	        else if (isBaseElement(el)) {
	            var firstBaseEl = nativeMethods.querySelector.call(this.document, 'base');
	            var storedHrefAttr = firstBaseEl && firstBaseEl.getAttribute(DomProcessor.getStoredAttrName('href'));
	            urlResolver.updateBase(storedHrefAttr || get$2(), this.document);
	        }
	        domMutationTracker.onElementChanged(el);
	    };
	    ElementSandbox.prototype.addFileInputInfo = function (el) {
	        var infoManager = this._uploadSandbox.infoManager;
	        addInputInfo(el, infoManager.getFiles(el), infoManager.getValue(el));
	    };
	    ElementSandbox.prototype.onIframeAddedToDOM = function (iframe) {
	        if (!isCrossDomainIframe(iframe, true))
	            this._nodeSandbox.mutation.onIframeAddedToDOM(iframe);
	    };
	    ElementSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        this._createOverriddenMethods();
	        overrideFunction(window.Element.prototype, 'setAttribute', this.overriddenMethods.setAttribute);
	        overrideFunction(window.Element.prototype, 'setAttributeNS', this.overriddenMethods.setAttributeNS);
	        overrideFunction(window.Element.prototype, 'getAttribute', this.overriddenMethods.getAttribute);
	        overrideFunction(window.Element.prototype, 'getAttributeNS', this.overriddenMethods.getAttributeNS);
	        overrideFunction(window.Element.prototype, 'removeAttribute', this.overriddenMethods.removeAttribute);
	        overrideFunction(window.Element.prototype, 'removeAttributeNS', this.overriddenMethods.removeAttributeNS);
	        overrideFunction(window.Element.prototype, 'cloneNode', this.overriddenMethods.cloneNode);
	        overrideFunction(window.Element.prototype, 'querySelector', this.overriddenMethods.querySelector);
	        overrideFunction(window.Element.prototype, 'querySelectorAll', this.overriddenMethods.querySelectorAll);
	        overrideFunction(window.Element.prototype, 'hasAttribute', this.overriddenMethods.hasAttribute);
	        overrideFunction(window.Element.prototype, 'hasAttributeNS', this.overriddenMethods.hasAttributeNS);
	        overrideFunction(window.Element.prototype, 'hasAttributes', this.overriddenMethods.hasAttributes);
	        if (nativeMethods.attachShadow)
	            overrideFunction(window.Element.prototype, 'attachShadow', this.overriddenMethods.attachShadow);
	        overrideFunction(window.Node.prototype, 'cloneNode', this.overriddenMethods.cloneNode);
	        overrideFunction(window.Node.prototype, 'appendChild', this.overriddenMethods.appendChild);
	        overrideFunction(window.Node.prototype, 'removeChild', this.overriddenMethods.removeChild);
	        overrideFunction(window.Node.prototype, 'insertBefore', this.overriddenMethods.insertBefore);
	        overrideFunction(window.Node.prototype, 'replaceChild', this.overriddenMethods.replaceChild);
	        if (nativeMethods.append)
	            overrideFunction(window.Element.prototype, 'append', this.overriddenMethods.append);
	        if (nativeMethods.prepend)
	            overrideFunction(window.Element.prototype, 'prepend', this.overriddenMethods.prepend);
	        if (nativeMethods.after)
	            overrideFunction(window.Element.prototype, 'after', this.overriddenMethods.after);
	        if (nativeMethods.remove)
	            overrideFunction(window.Element.prototype, 'remove', this.overriddenMethods.remove);
	        if (nativeMethods.elementReplaceWith)
	            overrideFunction(window.Element.prototype, 'replaceWith', this.overriddenMethods.elementReplaceWith);
	        overrideFunction(window.DocumentFragment.prototype, 'querySelector', this.overriddenMethods.querySelector);
	        overrideFunction(window.DocumentFragment.prototype, 'querySelectorAll', this.overriddenMethods.querySelectorAll);
	        overrideFunction(window.HTMLTableElement.prototype, 'insertRow', this.overriddenMethods.insertRow);
	        overrideFunction(window.HTMLTableSectionElement.prototype, 'insertRow', this.overriddenMethods.insertRow);
	        overrideFunction(window.HTMLTableRowElement.prototype, 'insertCell', this.overriddenMethods.insertCell);
	        overrideFunction(window.HTMLFormElement.prototype, 'submit', this.overriddenMethods.formSubmit);
	        overrideFunction(window.HTMLAnchorElement.prototype, 'toString', this.overriddenMethods.anchorToString);
	        overrideFunction(window.CharacterData.prototype, 'appendData', this.overriddenMethods.appendData);
	        if (window.Document.prototype.registerElement)
	            overrideFunction(window.Document.prototype, 'registerElement', this.overriddenMethods.registerElement);
	        overrideFunction(nativeMethods.insertAdjacentMethodsOwner, 'insertAdjacentHTML', this.overriddenMethods.insertAdjacentHTML);
	        overrideFunction(nativeMethods.insertAdjacentMethodsOwner, 'insertAdjacentElement', this.overriddenMethods.insertAdjacentElement);
	        overrideFunction(nativeMethods.insertAdjacentMethodsOwner, 'insertAdjacentText', this.overriddenMethods.insertAdjacentText);
	        this._setValidBrowsingContextOnElementClick(window);
	        // NOTE: Cookie can be set up for the page by using the request initiated by img.
	        // For example: img.src = '<url that responds with the Set-Cookie header>'
	        // If img has the 'load' event handler, we redirect the request through proxy.
	        // For details, see https://github.com/DevExpress/testcafe-hammerhead/issues/651
	        this._eventSandbox.listeners.on(this._eventSandbox.listeners.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
	            if (e.eventType === 'load' && isImgElement(e.el))
	                ElementSandbox._setProxiedSrc(e.el);
	        });
	        overrideDescriptor(window.HTMLElement.prototype, 'onload', {
	            getter: null,
	            setter: function (handler) {
	                if (isImgElement(this) && isValidEventListener(handler))
	                    ElementSandbox._setProxiedSrc(this);
	                nativeMethods.htmlElementOnloadSetter.call(this, handler);
	            }
	        });
	    };
	    ElementSandbox.prototype._ensureTargetContainsExistingBrowsingContext = function (el) {
	        if (settings.get().allowMultipleWindows)
	            return;
	        if (!nativeMethods.hasAttribute.call(el, 'target'))
	            return;
	        var attr = nativeMethods.getAttribute.call(el, 'target');
	        var storedAttr = nativeMethods.getAttribute.call(el, DomProcessor.getStoredAttrName('target'));
	        el.setAttribute('target', storedAttr || attr);
	    };
	    ElementSandbox.prototype._setValidBrowsingContextOnElementClick = function (window) {
	        var _this = this;
	        this._eventSandbox.listeners.initElementListening(window, ['click']);
	        this._eventSandbox.listeners.addInternalEventBeforeListener(window, ['click'], function (e) {
	            var el = e.target;
	            if (isInputElement(el) && el.form)
	                el = el.form;
	            var tagName = getTagName(el);
	            if (!DomProcessor.isTagWithTargetAttr(tagName))
	                return;
	            _this._ensureTargetContainsExistingBrowsingContext(el);
	        });
	    };
	    ElementSandbox.prototype._setProxiedSrcUrlOnError = function (img) {
	        img.addEventListener('error', function (e) {
	            var storedAttr = nativeMethods.getAttribute.call(img, DomProcessor.getStoredAttrName('src'));
	            var imgSrc = nativeMethods.imageSrcGetter.call(img);
	            if (storedAttr && !parseProxyUrl(imgSrc) &&
	                isSupportedProtocol(imgSrc) && !isSpecialPage(imgSrc)) {
	                nativeMethods.setAttribute.call(img, 'src', getProxyUrl(storedAttr));
	                stopPropagation(e);
	            }
	        }, false);
	    };
	    ElementSandbox.prototype.getCorrectedTarget = function (target) {
	        if (target === void 0) { target = ''; }
	        if (settings.get().allowMultipleWindows)
	            return target;
	        if (target && !isKeywordTarget(target) && !findByName(target) ||
	            /_blank/i.test(target))
	            return '_top';
	        return target;
	    };
	    ElementSandbox.prototype._handleImageLoadEventRaising = function (el) {
	        this._eventSandbox.listeners.initElementListening(el, ['load']);
	        this._eventSandbox.listeners.addInternalEventBeforeListener(el, ['load'], function (_e, _dispatched, preventEvent, _cancelHandlers, stopEventPropagation) {
	            if (el[INTERNAL_PROPS.cachedImage])
	                el[INTERNAL_PROPS.cachedImage] = false;
	            if (!el[INTERNAL_PROPS.skipNextLoadEventForImage])
	                return;
	            el[INTERNAL_PROPS.skipNextLoadEventForImage] = false;
	            preventEvent();
	            stopEventPropagation();
	        });
	        if (!el[INTERNAL_PROPS.forceProxySrcForImage] && !settings.get().forceProxySrcForImage)
	            this._setProxiedSrcUrlOnError(el);
	    };
	    ElementSandbox.prototype._processBaseTag = function (el) {
	        if (!this._isFirstBaseTagOnPage(el))
	            return;
	        var storedUrlAttr = nativeMethods.getAttribute.call(el, DomProcessor.getStoredAttrName('href'));
	        if (storedUrlAttr !== null)
	            urlResolver.updateBase(storedUrlAttr, el.ownerDocument || this.document);
	    };
	    ElementSandbox.prototype._reProcessElementWithTargetAttr = function (el, tagName) {
	        var targetAttr = domProcessor.getTargetAttr(el);
	        if (DomProcessor.isIframeFlagTag(tagName) && nativeMethods.getAttribute.call(el, targetAttr) === '_parent')
	            domProcessor.processElement(el, convertToProxyUrl);
	    };
	    ElementSandbox.prototype.processElement = function (el) {
	        var tagName = getTagName(el);
	        switch (tagName) {
	            case 'a':
	                this._childWindowSandbox.handleClickOnLinkOrArea(el);
	                break;
	            case 'img':
	                this._handleImageLoadEventRaising(el);
	                break;
	            case 'iframe':
	            case 'frame':
	                this._iframeSandbox.processIframe(el);
	                break;
	            case 'base':
	                this._processBaseTag(el);
	                break;
	            case 'area':
	                this._childWindowSandbox.handleClickOnLinkOrArea(el);
	                break;
	        }
	        // NOTE: we need to reprocess a tag client-side if it wasn't processed on the server.
	        // See the usage of Parse5DomAdapter.needToProcessUrl
	        this._reProcessElementWithTargetAttr(el, tagName);
	    };
	    return ElementSandbox;
	}(SandboxBase));

	var DEFAULT_TITLE_VALUE = '';
	// NOTE: Hammehead always add the <title> element for each test before test execution.
	// It's necessary to TestCafe can find the browser tab by page title.
	// This <title> element stores the sandboxed title value in the internal property.
	var INTERNAL_TITLE_PROP_NAME = 'hammerhead|document-title-storage|internal-prop-name';
	var DocumentTitleStorage = /** @class */ (function (_super) {
	    __extends(DocumentTitleStorage, _super);
	    function DocumentTitleStorage(document) {
	        var _this = _super.call(this) || this;
	        _this._document = document;
	        return _this;
	    }
	    DocumentTitleStorage.prototype._ensureFirstTitleElement = function () {
	        var firstTitle = this.getFirstTitleElement();
	        if (firstTitle)
	            return firstTitle;
	        firstTitle = nativeMethods.createElement.call(this._document, 'title');
	        nativeMethods.appendChild.call(this._document.head, firstTitle);
	        this.emit('titleElementAdded');
	        return firstTitle;
	    };
	    DocumentTitleStorage.prototype._getValueFromFirstTitleElement = function () {
	        var firstTitle = this.getFirstTitleElement();
	        if (!firstTitle)
	            return DEFAULT_TITLE_VALUE;
	        return this.getTitleElementPropertyValue(firstTitle);
	    };
	    DocumentTitleStorage.prototype._setValueForFirstTitleElement = function (value) {
	        var firstTitle = this._ensureFirstTitleElement();
	        this.setTitleElementPropertyValue(firstTitle, value);
	    };
	    DocumentTitleStorage.prototype._getTitleElement = function (index) {
	        return this._document &&
	            this._document.head &&
	            nativeMethods.elementQuerySelectorAll.call(this._document.head, 'title')[index];
	    };
	    DocumentTitleStorage.prototype.getFirstTitleElement = function () {
	        return this._getTitleElement(0);
	    };
	    DocumentTitleStorage.prototype.getSecondTitleElement = function () {
	        return this._getTitleElement(1);
	    };
	    DocumentTitleStorage.prototype.getTitle = function () {
	        return this._getValueFromFirstTitleElement();
	    };
	    DocumentTitleStorage.prototype.setTitle = function (value) {
	        value = String(value);
	        this._setValueForFirstTitleElement(value);
	    };
	    DocumentTitleStorage.prototype.getTitleElementPropertyValue = function (element) {
	        return element[INTERNAL_TITLE_PROP_NAME] || DEFAULT_TITLE_VALUE;
	    };
	    DocumentTitleStorage.prototype.setTitleElementPropertyValue = function (element, value) {
	        value = String(value);
	        if (this.isElementProcessed(element))
	            element[INTERNAL_TITLE_PROP_NAME] = value;
	        else
	            nativeMethods.objectDefineProperty(element, INTERNAL_TITLE_PROP_NAME, {
	                value: value,
	                writable: true
	            });
	    };
	    DocumentTitleStorage.prototype.getDocument = function () {
	        return this._document;
	    };
	    DocumentTitleStorage.prototype.isElementProcessed = function (titleElement) {
	        return INTERNAL_TITLE_PROP_NAME in titleElement;
	    };
	    Object.defineProperty(DocumentTitleStorage, "DEFAULT_TITLE_VALUE", {
	        get: function () {
	            return DEFAULT_TITLE_VALUE;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    return DocumentTitleStorage;
	}(EventEmitter));

	var DocumentTitleStorageInitializer = /** @class */ (function () {
	    function DocumentTitleStorageInitializer(storage) {
	        var _this = this;
	        this.storage = storage;
	        this.storage.on('titleElementAdded', function () { return _this._processFirstTitleElement(); });
	    }
	    DocumentTitleStorageInitializer.prototype._setProxiedTitleValue = function () {
	        var _a = settings.get(), sessionId = _a.sessionId, windowId = _a.windowId;
	        var value = sessionId + "*" + windowId;
	        nativeMethods.documentTitleSetter.call(this.storage.getDocument(), value);
	    };
	    DocumentTitleStorageInitializer.prototype._processFirstTitleElement = function () {
	        var firstTitle = this.storage.getFirstTitleElement();
	        if (!firstTitle)
	            return false;
	        if (this.storage.isElementProcessed(firstTitle))
	            return false;
	        var value = nativeMethods.titleElementTextGetter.call(firstTitle);
	        this.storage.setTitleElementPropertyValue(firstTitle, value);
	        this._setProxiedTitleValue();
	        return true;
	    };
	    DocumentTitleStorageInitializer.prototype.onAttach = function () {
	        this._processFirstTitleElement();
	    };
	    DocumentTitleStorageInitializer.prototype.onPageTitleLoaded = function () {
	        if (this._processFirstTitleElement())
	            return;
	        var firstTitle = this.storage.getFirstTitleElement();
	        var secondTitle = this.storage.getSecondTitleElement();
	        if (!secondTitle)
	            return;
	        // NOTE: IE11 returns an empty string for the second <title> tag in the elements hierarchy.
	        var pageOriginValue = nativeMethods.titleElementTextGetter.call(secondTitle) ||
	            nativeMethods.htmlElementInnerTextGetter.call(secondTitle);
	        var serviceValue = nativeMethods.titleElementTextGetter.call(firstTitle);
	        nativeMethods.titleElementTextSetter.call(secondTitle, serviceValue);
	        removeElement(firstTitle);
	        this.storage.setTitleElementPropertyValue(secondTitle, pageOriginValue);
	    };
	    return DocumentTitleStorageInitializer;
	}());

	var ATTRIBUTE_SELECTOR_REG_EX = /\[([\w-]+)(\^?=.+?)]/g;
	var ATTRIBUTE_OPERATOR_WITH_HASH_VALUE = /^\W+\s*#/;
	var PSEUDO_CLASS_FOCUS_REG_EX = /\s*:focus\b/gi;
	var PSEUDO_CLASS_HOVER_REG_EX = /:hover\b/gi;
	var NodeSandbox = /** @class */ (function (_super) {
	    __extends(NodeSandbox, _super);
	    function NodeSandbox(mutation, iframeSandbox, _eventSandbox, _uploadSandbox, shadowUI, _cookieSandbox, _childWindowSandbox) {
	        var _this = _super.call(this) || this;
	        _this.mutation = mutation;
	        _this.iframeSandbox = iframeSandbox;
	        _this._eventSandbox = _eventSandbox;
	        _this._uploadSandbox = _uploadSandbox;
	        _this.shadowUI = shadowUI;
	        _this._cookieSandbox = _cookieSandbox;
	        _this._childWindowSandbox = _childWindowSandbox;
	        _this.raiseBodyCreatedEvent = _this._onBodyCreated;
	        // NOTE: We need to define the property with the 'writable' descriptor for testing purposes
	        nativeMethods.objectDefineProperty(document, INTERNAL_PROPS.documentCharset, {
	            value: parseDocumentCharset(),
	            writable: true
	        });
	        _this._documentTitleStorageInitializer = NodeSandbox._createDocumentTitleStorageInitializer();
	        _this.doc = new DocumentSandbox(_this, _this.shadowUI, _this._cookieSandbox, _this._documentTitleStorageInitializer);
	        _this.win = new WindowSandbox(_this, _this._eventSandbox, _this._uploadSandbox, _this.mutation, _this._childWindowSandbox, _this._documentTitleStorageInitializer);
	        _this.element = new ElementSandbox(_this, _this._uploadSandbox, _this.iframeSandbox, _this.shadowUI, _this._eventSandbox, _this._childWindowSandbox);
	        return _this;
	    }
	    NodeSandbox._createDocumentTitleStorageInitializer = function () {
	        if (isIframeWindow(window))
	            return null;
	        var documentTitleStorage = new DocumentTitleStorage(document);
	        return new DocumentTitleStorageInitializer(documentTitleStorage);
	    };
	    NodeSandbox.prototype._onBodyCreated = function () {
	        this._eventSandbox.listeners.initDocumentBodyListening(this.document);
	        this.mutation.onBodyCreated(this.document.body);
	    };
	    NodeSandbox.prototype._processElement = function (el) {
	        var processedContext = el[INTERNAL_PROPS.processedContext];
	        if (isShadowUIElement(el) || processedContext === this.window)
	            return;
	        var urlAttrName = null;
	        if (processedContext) {
	            urlAttrName = domProcessor.getUrlAttr(el);
	            urlAttrName = urlAttrName && el.hasAttribute(urlAttrName) ? urlAttrName : null;
	        }
	        var canAddNewProp = nativeMethods.objectIsExtensible(el);
	        var canUpdateExistingProp = processedContext && !nativeMethods.objectIsFrozen(el);
	        if (canAddNewProp || canUpdateExistingProp) {
	            nativeMethods.objectDefineProperty(el, INTERNAL_PROPS.processedContext, {
	                value: this.window,
	                writable: true
	            });
	        }
	        // NOTE: We need to reprocess url attribute of element, if it's moved to different window (GH-564)
	        if (urlAttrName)
	            el.setAttribute(urlAttrName, el.getAttribute(urlAttrName));
	        this.element.processElement(el);
	    };
	    NodeSandbox.prototype.onOriginFirstTitleElementInHeadLoaded = function () {
	        if (this._documentTitleStorageInitializer)
	            this._documentTitleStorageInitializer.onPageTitleLoaded();
	    };
	    NodeSandbox.prototype.processNodes = function (el, doc) {
	        if (!el) {
	            doc = doc || this.document;
	            if (doc.documentElement)
	                this.processNodes(doc.documentElement);
	        }
	        else if (el.querySelectorAll) {
	            if (el.nodeType !== Node.DOCUMENT_FRAGMENT_NODE)
	                this._processElement(el);
	            var children = getNativeQuerySelectorAll(el).call(el, '*');
	            var length_1 = nativeMethods.nodeListLengthGetter.call(children);
	            for (var i = 0; i < length_1; i++)
	                this._processElement(children[i]);
	        }
	    };
	    // NOTE: DOM sandbox hides evidence of the content proxying from a page native script. Proxy replaces URLs for
	    // resources. Our goal is to make the native script think that all resources are fetched from the destination
	    // resource, not from proxy, and also provide proxying for dynamically created elements.
	    NodeSandbox.prototype.attach = function (window) {
	        var _this = this;
	        var document = window.document;
	        var domContentLoadedEventRaised = false;
	        _super.prototype.attach.call(this, window, document);
	        if (this._documentTitleStorageInitializer)
	            this._documentTitleStorageInitializer.onAttach();
	        this.iframeSandbox.on(this.iframeSandbox.IFRAME_DOCUMENT_CREATED_EVENT, function (_a) {
	            var iframe = _a.iframe;
	            var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	            var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	            // NOTE: Before overriding the iframe, we must restore native document methods.
	            // Therefore, we save them before they are overridden.
	            // @ts-ignore
	            var iframeNativeMethods = new _this.nativeMethods.constructor(contentDocument, contentWindow);
	            contentWindow[INTERNAL_PROPS.iframeNativeMethods] = iframeNativeMethods;
	            // NOTE: Override only the document (in fact, we only need the 'write' and 'writeln' methods).
	            _this.doc.attach(contentWindow, contentDocument, true);
	        });
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        nativeMethods.objectDefineProperty(window, INTERNAL_PROPS.processDomMethodName, {
	            value: function (el, doc) {
	                // NOTE: TestCafe creates a shadow-ui root before the DOMContentLoaded event (once document.body is
	                // available). Sometimes for a very heavy DOM or a very slow loading the body doesn't contain all
	                // elements at that moment and as a result after a full page loading our root element becomes not
	                // the last child of the body. So we need to make the root last body child manually on every script
	                // loading until the DOMContentLoaded event is raised.
	                if (!domContentLoadedEventRaised)
	                    _this.shadowUI.onBodyElementMutation();
	                _this.processNodes(el, doc);
	            },
	            configurable: true
	        });
	        // NOTE: In some browsers (for example Firefox), the 'window.document' object is different when iframe is
	        // created and when the document’s ready event is raised. Therefore, we need to update the 'document' object
	        // to override its methods (Q527555).
	        document.addEventListener('DOMContentLoaded', function () {
	            domContentLoadedEventRaised = true;
	            _this.processNodes(null, document);
	        }, false);
	        this.doc.attach(window, document);
	        this.win.attach(window);
	        this.element.attach(window);
	    };
	    NodeSandbox._processAttributeSelector = function (selector) {
	        if (!ATTRIBUTE_SELECTOR_REG_EX.test(selector))
	            return selector;
	        return selector + ',' + selector.replace(ATTRIBUTE_SELECTOR_REG_EX, function (str, name, operatorWithValue) {
	            if (URL_ATTRS.indexOf(name) !== -1 &&
	                !ATTRIBUTE_OPERATOR_WITH_HASH_VALUE.test(operatorWithValue)) {
	                name = DomProcessor.getStoredAttrName(name);
	                return '[' + name + operatorWithValue + ']';
	            }
	            return str;
	        });
	    };
	    NodeSandbox._processPseudoClassSelectors = function (selector) {
	        // NOTE: When a selector that contains the ':focus' pseudo-class is used in the querySelector and
	        // querySelectorAll functions, these functions return an empty result if the browser is not focused.
	        // This replaces ':focus' with a custom CSS class to return the current active element in that case.
	        // IE returns a valid element, so there is no need to replace the selector for it.
	        if (!isIE)
	            selector = selector.replace(PSEUDO_CLASS_FOCUS_REG_EX, '[' + INTERNAL_ATTRIBUTES.focusPseudoClass + ']');
	        selector = selector.replace(PSEUDO_CLASS_HOVER_REG_EX, '[' + INTERNAL_ATTRIBUTES.hoverPseudoClass + ']');
	        return selector;
	    };
	    NodeSandbox.processSelector = function (selector) {
	        if (selector) {
	            selector = NodeSandbox._processPseudoClassSelectors(selector);
	            selector = NodeSandbox._processAttributeSelector(selector);
	        }
	        return selector;
	    };
	    return NodeSandbox;
	}(SandboxBase));

	function getAreaElementRectangle(el, mapContainer) {
	    var shape = nativeMethods.getAttribute.call(el, 'shape');
	    var coords = nativeMethods.getAttribute.call(el, 'coords');
	    var i = 0;
	    if (shape === 'default')
	        return getElementRectangle(mapContainer);
	    if (!shape || !coords)
	        return null;
	    coords = coords.split(',');
	    if (!coords.length)
	        return null;
	    for (i = 0; i < coords.length; i++) {
	        coords[i] = parseInt(coords[i], 10);
	        if (typeof coords[i] !== 'number')
	            return null;
	    }
	    var rectangle = null;
	    switch (shape) {
	        case 'rect':
	            if (coords.length === 4) {
	                rectangle = {
	                    height: coords[3] - coords[1],
	                    left: coords[0],
	                    top: coords[1],
	                    width: coords[2] - coords[0]
	                };
	            }
	            break;
	        case 'circle':
	            if (coords.length === 3) {
	                rectangle = {
	                    height: coords[2] * 2,
	                    left: coords[0] - coords[2],
	                    top: coords[1] - coords[2],
	                    width: coords[2] * 2
	                };
	            }
	            break;
	        case 'poly':
	            if (coords.length >= 6 && coords.length % 2 === 0) {
	                rectangle = {};
	                rectangle.left = rectangle.right = coords[0];
	                rectangle.top = rectangle.bottom = coords[1];
	                for (i = 2; i < coords.length; i += 2) {
	                    rectangle.left = coords[i] < rectangle.left ? coords[i] : rectangle.left;
	                    rectangle.right = coords[i] > rectangle.right ? coords[i] : rectangle.right;
	                }
	                for (i = 3; i < coords.length; i += 2) {
	                    rectangle.top = coords[i] < rectangle.top ? coords[i] : rectangle.top;
	                    rectangle.bottom = coords[i] > rectangle.bottom ? coords[i] : rectangle.bottom;
	                }
	                rectangle.height = rectangle.bottom - rectangle.top;
	                rectangle.width = rectangle.right - rectangle.left;
	            }
	            break;
	    }
	    if (rectangle) {
	        var containerOffset = getOffsetPosition(mapContainer);
	        rectangle.left += containerOffset.left;
	        rectangle.top += containerOffset.top;
	    }
	    return rectangle;
	}
	function getMapElementRectangle(el) {
	    var mapContainer = getMapContainer(el);
	    if (mapContainer) {
	        if (/^map$/i.test(el.tagName))
	            return getElementRectangle(mapContainer);
	        else if (/^area$/i.test(el.tagName)) {
	            var areaElementRectangle = getAreaElementRectangle(el, mapContainer);
	            if (areaElementRectangle)
	                return areaElementRectangle;
	        }
	    }
	    return {
	        height: 0,
	        left: 0,
	        top: 0,
	        width: 0
	    };
	}
	function getSelectChildRectangle(el) {
	    var select = getSelectParent(el);
	    if (select) {
	        var selectRectangle = getElementRectangle(select);
	        var selectBorders = getBordersWidth(select);
	        var selectRightScrollbar = getInnerWidth(select) ===
	            select.clientWidth ? 0 : getScrollbarSize();
	        var optionHeight = getOptionHeight(select);
	        var optionRealIndex = getChildVisibleIndex(select, el);
	        var optionVisibleIndex = Math.max(optionRealIndex - getScrollTop(select) / optionHeight, 0);
	        return {
	            height: optionHeight,
	            left: selectRectangle.left + selectBorders.left,
	            top: selectRectangle.top + selectBorders.top + getElementPadding(select).top +
	                optionVisibleIndex * optionHeight,
	            width: selectRectangle.width - (selectBorders.left + selectBorders.right) - selectRightScrollbar
	        };
	    }
	    return getElementRectangle(el);
	}
	function getSvgElementRelativeRectangle(el) {
	    var isSvgTextElement = matches(el, 'tspan') || matches(el, 'tref') ||
	        getTagName(el) === 'textpath';
	    var boundingClientRect = el.getBoundingClientRect();
	    var elementRect = {
	        height: !isSvgTextElement ? boundingClientRect.height : el.offsetHeight,
	        left: boundingClientRect.left + (document.body.scrollLeft || document.documentElement.scrollLeft),
	        top: boundingClientRect.top + (document.body.scrollTop || document.documentElement.scrollTop),
	        width: !isSvgTextElement ? boundingClientRect.width : el.offsetWidth
	    };
	    if (isSvgTextElement) {
	        var offsetParent = getOffsetParent(el);
	        var elOffset = getOffset(el);
	        var offsetParentOffset = getOffset(offsetParent);
	        var offsetParentIsBody = matches(offsetParent, 'body');
	        return {
	            height: elementRect.height || boundingClientRect.height,
	            left: offsetParentIsBody ? el.offsetLeft || elOffset.left : offsetParentOffset.left + el.offsetLeft,
	            top: offsetParentIsBody ? el.offsetTop || elOffset.top : offsetParentOffset.top + el.offsetTop,
	            width: elementRect.width || boundingClientRect.width
	        };
	    }
	    if (isFirefox || isIE)
	        return elementRect;
	    var strokeWidth = nativeMethods.getAttribute.call(el, 'stroke-width') || get$1(el, 'stroke-width');
	    // NOTE: We assume that the 'stroke-width' attribute can only be set in pixels.
	    strokeWidth = strokeWidth ? +strokeWidth.replace(/px|em|ex|pt|pc|cm|mm|in/, '') : 1;
	    if (strokeWidth && +strokeWidth % 2 !== 0)
	        strokeWidth = +strokeWidth + 1;
	    if ((matches(el, 'line') || matches(el, 'polyline') || matches(el, 'polygon') ||
	        matches(el, 'path')) &&
	        (!elementRect.width || !elementRect.height)) {
	        if (!elementRect.width && elementRect.height) {
	            elementRect.left -= strokeWidth / 2;
	            elementRect.width = strokeWidth;
	        }
	        else if (elementRect.width && !elementRect.height) {
	            elementRect.height = strokeWidth;
	            elementRect.top -= strokeWidth / 2;
	        }
	    }
	    else {
	        if (matches(el, 'polygon')) {
	            elementRect.height += 2 * strokeWidth;
	            elementRect.left -= strokeWidth;
	            elementRect.top -= strokeWidth;
	            elementRect.width += 2 * strokeWidth;
	        }
	        elementRect.height += strokeWidth;
	        elementRect.left -= strokeWidth / 2;
	        elementRect.top -= strokeWidth / 2;
	        elementRect.width += strokeWidth;
	    }
	    return elementRect;
	}
	function getElementRectangle(el) {
	    var rectangle = {};
	    if (isMapElement(el))
	        rectangle = getMapElementRectangle(el);
	    else if (isVisibleChild(el))
	        rectangle = getSelectChildRectangle(el);
	    else {
	        var elementOffset = getOffsetPosition(el);
	        var relativeRectangle = isSVGElementOrChild(el) ? getSvgElementRelativeRectangle(el) : el.getBoundingClientRect();
	        rectangle = {
	            height: relativeRectangle.height,
	            left: elementOffset.left,
	            top: elementOffset.top,
	            width: relativeRectangle.width
	        };
	    }
	    rectangle.height = Math.round(rectangle.height);
	    rectangle.left = Math.round(rectangle.left);
	    rectangle.top = Math.round(rectangle.top);
	    rectangle.width = Math.round(rectangle.width);
	    return rectangle;
	}
	function shouldIgnoreMouseEventInsideIframe(el, x, y) {
	    if (getTagName(el) !== 'iframe')
	        return false;
	    var rect = getElementRectangle(el);
	    var borders = getBordersWidth(el);
	    var padding = getElementPadding(el);
	    // NOTE: we detect element's 'content' position: left, right, top and bottom
	    // which does not consider borders and paddings, so we need to
	    // subtract it for right and bottom, and add for left and top
	    var left = rect.left + borders.left + padding.left;
	    var top = rect.top + borders.top + padding.top;
	    var right = rect.left + rect.width - borders.right - padding.right;
	    var bottom = rect.top + rect.height - borders.bottom - padding.bottom;
	    return x >= left && x <= right && y >= top && y <= bottom;
	}
	function calcOffsetPosition(el, borders, offsetPosition) {
	    var isSvg = isSVGElementOrChild(el);
	    var relativeRectangle = isSvg ? getSvgElementRelativeRectangle(el) : null;
	    return {
	        left: isSvg ? relativeRectangle.left + borders.left : offsetPosition.left + borders.left,
	        top: isSvg ? relativeRectangle.top + borders.top : offsetPosition.top + borders.top
	    };
	}
	function calcOffsetPositionInIframe(el, borders, offsetPosition, doc, currentIframe) {
	    var iframeBorders = getBordersWidth(currentIframe);
	    borders.left += iframeBorders.left;
	    borders.top += iframeBorders.top;
	    var iframeOffset = getOffsetPosition(currentIframe);
	    var iframePadding = getElementPadding(currentIframe);
	    var clientPosition = null;
	    if (isSVGElementOrChild(el)) {
	        var relativeRectangle = getSvgElementRelativeRectangle(el);
	        clientPosition = {
	            x: relativeRectangle.left - (document.body.scrollLeft || document.documentElement.scrollLeft) +
	                borders.left,
	            y: relativeRectangle.top - (document.body.scrollTop || document.documentElement.scrollTop) + borders.top
	        };
	    }
	    else {
	        clientPosition = offsetToClientCoords({
	            x: offsetPosition.left + borders.left,
	            y: offsetPosition.top + borders.top
	        }, doc);
	    }
	    return {
	        left: iframeOffset.left + clientPosition.x + iframePadding.left,
	        top: iframeOffset.top + clientPosition.y + iframePadding.top
	    };
	}
	function getOffsetPosition(el, roundFn) {
	    if (roundFn === void 0) { roundFn = Math.round; }
	    if (isMapElement(el)) {
	        var rectangle = getMapElementRectangle(el);
	        return {
	            left: rectangle.left,
	            top: rectangle.top
	        };
	    }
	    var doc = findDocument(el);
	    var isInIframe = isElementInIframe(el, doc);
	    var currentIframe = isInIframe ? getIframeByElement(doc) : null;
	    var offsetPosition = doc === el ? getOffset(doc.documentElement) : getOffset(el);
	    // NOTE: The jquery .offset() function doesn't take the body's border into account (except IE7)
	    // http://bugs.jquery.com/ticket/7948.
	    // NOTE: Sometimes, in IE, the getElementFromPoint method returns a cross-domain iframe's documentElement,
	    // but there’s no way to access its body.
	    var borders = doc.body ? getBordersWidth(doc.body) : {
	        left: 0,
	        top: 0
	    };
	    var calcOffsetPositionFn = !isInIframe || !currentIframe ? calcOffsetPosition : calcOffsetPositionInIframe;
	    var _a = calcOffsetPositionFn(el, borders, offsetPosition, doc, currentIframe), left = _a.left, top = _a.top;
	    if (typeof roundFn === 'function') {
	        left = roundFn(left);
	        top = roundFn(top);
	    }
	    return { left: left, top: top };
	}
	function offsetToClientCoords(coords, currentDocument) {
	    var doc = currentDocument || document;
	    var documentScrollLeft = getScrollLeft(doc);
	    var documentScrollTop = getScrollTop(doc);
	    var bodyScrollLeft = getScrollLeft(doc.body);
	    var bodyScrollTop = getScrollTop(doc.body);
	    var scrollLeft = documentScrollLeft === 0 && bodyScrollLeft !== 0 ? bodyScrollLeft : documentScrollLeft;
	    var scrollTop = documentScrollTop === 0 && bodyScrollTop !== 0 ? bodyScrollTop : documentScrollTop;
	    return {
	        x: coords.x - scrollLeft,
	        y: coords.y - scrollTop
	    };
	}

	var positionUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		getElementRectangle: getElementRectangle,
		shouldIgnoreMouseEventInsideIframe: shouldIgnoreMouseEventInsideIframe,
		getOffsetPosition: getOffsetPosition,
		offsetToClientCoords: offsetToClientCoords
	});

	// NOTE: Tags from https://www.w3schools.com/tags/att_name.asp
	var ELEMENTS_WITH_NAME_ATTRIBUTE = ['button', 'fieldset', 'form', 'iframe',
	    'input', 'map', 'meta', 'object', 'output', 'param', 'select', 'textarea'];
	var COLLECTION_PROTO_GETTERS_RESERVE = 10;
	var collectionProtoGettersCount = 0;
	var HTMLCollectionInheritor = /** @class */ (function () {
	    function HTMLCollectionInheritor() {
	    }
	    return HTMLCollectionInheritor;
	}());
	HTMLCollectionInheritor.prototype = HTMLCollection.prototype;
	var HTMLCollectionWrapper = /** @class */ (function (_super) {
	    __extends(HTMLCollectionWrapper, _super);
	    function HTMLCollectionWrapper(collection, tagName) {
	        var _this = _super.call(this) || this;
	        tagName = tagName.toLowerCase();
	        nativeMethods.objectDefineProperties(_this, {
	            _collection: { value: collection },
	            _filteredCollection: { value: [] },
	            _tagName: { value: tagName },
	            _version: { value: -Infinity, writable: true },
	            _namedProps: { value: ELEMENTS_WITH_NAME_ATTRIBUTE.indexOf(tagName) !== -1 ? [] : null },
	            _lastNativeLength: { value: 0, writable: true }
	        });
	        _this._refreshCollection();
	        return _this;
	    }
	    HTMLCollectionWrapper.prototype.item = function (index) {
	        this._refreshCollection();
	        return this._filteredCollection[index];
	    };
	    Object.defineProperty(HTMLCollectionWrapper.prototype, "length", {
	        get: function () {
	            this._refreshCollection();
	            return this._filteredCollection.length;
	        },
	        enumerable: true,
	        configurable: true
	    });
	    HTMLCollectionWrapper.prototype._refreshCollection = function () {
	        var storedNativeCollectionLength = this._lastNativeLength;
	        var nativeCollectionLength = nativeMethods.htmlCollectionLengthGetter.call(this._collection);
	        this._lastNativeLength = nativeCollectionLength;
	        if (!domMutationTracker.isOutdated(this._tagName, this._version) &&
	            (domMutationTracker.isDomContentLoaded() || storedNativeCollectionLength === nativeCollectionLength))
	            return;
	        var storedFilteredCollectionLength = this._filteredCollection.length;
	        var currentNamedProps = filterCollection(this, nativeCollectionLength);
	        this._version = domMutationTracker.getVersion(this._tagName);
	        updateCollectionIndexGetters(this, storedFilteredCollectionLength, this._filteredCollection.length);
	        updateNamedProps(this, this._namedProps, currentNamedProps);
	    };
	    return HTMLCollectionWrapper;
	}(HTMLCollectionInheritor));
	var additionalProtoMethods = {
	    constructor: {
	        value: HTMLCollectionWrapper.constructor,
	        configurable: true,
	        enumerable: false,
	        writable: true
	    },
	    _refreshCollection: {
	        value: HTMLCollectionWrapper.prototype._refreshCollection,
	        enumerable: false
	    }
	};
	if (HTMLCollection.prototype.namedItem) {
	    additionalProtoMethods.namedItem = {
	        value: function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            this._refreshCollection();
	            var namedItem = this._collection.namedItem.apply(this._collection, args);
	            return namedItem && isShadowUIElement(namedItem) ? null : namedItem;
	        },
	        enumerable: true,
	        configurable: true,
	        writable: true
	    };
	}
	nativeMethods.objectDefineProperties(HTMLCollectionWrapper.prototype, additionalProtoMethods);
	addShadowGetters(COLLECTION_PROTO_GETTERS_RESERVE);
	function addShadowGetters(count) {
	    var _loop_1 = function (i) {
	        var idx = collectionProtoGettersCount++;
	        nativeMethods.objectDefineProperty(HTMLCollectionWrapper.prototype, idx, {
	            get: function () {
	                this.item(idx);
	            }
	        });
	    };
	    for (var i = 0; i < count; i++) {
	        _loop_1();
	    }
	}
	function updateCollectionIndexGetters(wrapper, oldLength, currentLength) {
	    if (oldLength === currentLength)
	        return;
	    var _loop_2 = function () {
	        var idx = oldLength++;
	        nativeMethods.objectDefineProperty(wrapper, idx, {
	            enumerable: true,
	            configurable: true,
	            get: function () { return wrapper.item(idx); }
	        });
	    };
	    while (oldLength < currentLength) {
	        _loop_2();
	    }
	    while (oldLength > currentLength)
	        delete wrapper[--oldLength];
	    var maxCollectionLength = collectionProtoGettersCount - COLLECTION_PROTO_GETTERS_RESERVE;
	    if (currentLength > maxCollectionLength)
	        addShadowGetters(currentLength - maxCollectionLength);
	}
	function updateNamedProps(wrapper, oldNamedProps, currentNamedProps) {
	    if (!currentNamedProps)
	        return;
	    for (var _i = 0, oldNamedProps_1 = oldNamedProps; _i < oldNamedProps_1.length; _i++) {
	        var oldProp = oldNamedProps_1[_i];
	        if (currentNamedProps.indexOf(oldProp) === -1)
	            delete wrapper[oldProp];
	    }
	    var _loop_3 = function (prop) {
	        if (!wrapper._collection[prop])
	            return "continue";
	        nativeMethods.objectDefineProperty(wrapper, prop, {
	            configurable: true,
	            get: function () {
	                this._refreshCollection();
	                return wrapper._collection[prop];
	            }
	        });
	    };
	    for (var _a = 0, currentNamedProps_1 = currentNamedProps; _a < currentNamedProps_1.length; _a++) {
	        var prop = currentNamedProps_1[_a];
	        _loop_3(prop);
	    }
	}
	function filterCollection(wrapper, nativeCollectionLength) {
	    var nativeCollection = wrapper._collection;
	    var currentNamedProps = wrapper._namedProps ? [] : null;
	    var filteredCollection = wrapper._filteredCollection;
	    filteredCollection.length = 0;
	    for (var i = 0; i < nativeCollectionLength; i++) {
	        var el = nativeCollection[i];
	        if (isShadowUIElement(el))
	            continue;
	        filteredCollection.push(el);
	        if (!currentNamedProps)
	            continue;
	        var nameAttr = nativeMethods.getAttribute.call(el, 'name');
	        if (nameAttr !== null)
	            currentNamedProps.push(nameAttr);
	    }
	    return currentNamedProps;
	}

	var IS_NON_STATIC_POSITION_RE = /fixed|relative|absolute/;
	var CLASSNAME_RE = /\.((?:\\.|[-\w]|[^\x00-\xa0])+)/g;
	var IS_SHADOW_CONTAINER_FLAG = 'hammerhead|shadow-ui|container-flag';
	var IS_SHADOW_CONTAINER_COLLECTION_FLAG = 'hammerhead|shadow-ui|container-collection-flag';
	var HTML_COLLECTION_WRAPPER = 'hammerhead|shadow-ui|html-collection-wrapper';
	var ShadowUI = /** @class */ (function (_super) {
	    __extends(ShadowUI, _super);
	    function ShadowUI(_nodeMutation, _messageSandbox, _iframeSandbox, _ieDebugSandbox) {
	        var _this = _super.call(this) || this;
	        _this._nodeMutation = _nodeMutation;
	        _this._messageSandbox = _messageSandbox;
	        _this._iframeSandbox = _iframeSandbox;
	        _this._ieDebugSandbox = _ieDebugSandbox;
	        _this.BODY_CONTENT_CHANGED_COMMAND = 'hammerhead|command|body-content-changed';
	        _this.ROOT_CLASS = 'root';
	        _this.ROOT_ID = 'root';
	        _this.HIDDEN_CLASS = 'hidden';
	        _this.BLIND_CLASS = 'blind';
	        _this.root = null;
	        _this.lastActiveElement = null;
	        _this.uiStyleSheetsHtmlBackup = null;
	        _this.wrapperCreators = _this._createWrapperCreators();
	        _this._initEventCallbacks();
	        return _this;
	    }
	    ShadowUI.prototype._initEventCallbacks = function () {
	        var _this = this;
	        this.runTaskScriptEventCallback = function (iframe) {
	            var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	            var iframeHead = contentDocument.head;
	            var iframeBody = contentDocument.body;
	            _this._restoreUIStyleSheets(iframeHead, _this._getUIStyleSheetsHtml());
	            _this.markShadowUIContainers(iframeHead, iframeBody);
	        };
	        this.beforeDocumentCleanedEventCallback = function () {
	            _this.uiStyleSheetsHtmlBackup = _this._getUIStyleSheetsHtml();
	        };
	        this.documentCleanedEventCallback = function (e) {
	            _this._restoreUIStyleSheets(e.document.head, _this.uiStyleSheetsHtmlBackup);
	            _this.uiStyleSheetsHtmlBackup = null;
	            _this.markShadowUIContainers(_this.document.head, _this.document.body);
	        };
	        this.documentClosedEventCallback = function (document) {
	            _this._restoreUIStyleSheets(document.head, _this.uiStyleSheetsHtmlBackup);
	            _this.uiStyleSheetsHtmlBackup = null;
	            _this.markShadowUIContainers(document.head, document.body);
	        };
	        this.bodyContentChangedEventCallback = function (body) {
	            var elContextWindow = body[INTERNAL_PROPS.processedContext];
	            if (elContextWindow !== window) {
	                _this._messageSandbox.sendServiceMsg({
	                    cmd: _this.BODY_CONTENT_CHANGED_COMMAND
	                }, elContextWindow);
	            }
	            else
	                _this.onBodyElementMutation();
	        };
	        this.serviceMsgReceivedEventCallback = function (e) {
	            if (e.message.cmd === _this.BODY_CONTENT_CHANGED_COMMAND)
	                _this.onBodyElementMutation();
	        };
	        this.bodyCreatedEventCallback = function (body) { return _this.markShadowUIContainers(_this.document.head, body); };
	    };
	    ShadowUI._filterElement = function (el) {
	        return el && isShadowUIElement(el) ? null : el;
	    };
	    ShadowUI.prototype._filterList = function (list, listLength, predicate) {
	        var filteredList = [];
	        for (var i = 0; i < listLength; i++) {
	            var el = predicate(list[i]);
	            if (el)
	                filteredList.push(list[i]);
	        }
	        nativeMethods.objectDefineProperty(filteredList, 'item', {
	            value: function (index) { return index >= filteredList.length ? null : filteredList[index]; }
	        });
	        if (list.namedItem) {
	            nativeMethods.objectDefineProperty(filteredList, 'namedItem', {
	                value: function (name) { return list.namedItem(name); }
	            });
	        }
	        return filteredList.length === listLength ? list : filteredList;
	    };
	    ShadowUI.prototype._filterNodeList = function (nodeList, originLength) {
	        return this._filterList(nodeList, originLength, function (item) { return ShadowUI._filterElement(item); });
	    };
	    ShadowUI.prototype._filterStyleSheetList = function (styleSheetList, originLength) {
	        return this._filterList(styleSheetList, originLength, function (item) { return ShadowUI._filterElement(item.ownerNode); });
	    };
	    ShadowUI._getFirstNonShadowElement = function (nodeList) {
	        var length = nativeMethods.nodeListLengthGetter.call(nodeList);
	        for (var i = 0; i < length; i++) {
	            if (ShadowUI._filterElement(nodeList[i]))
	                return nodeList[i];
	        }
	        return null;
	    };
	    ShadowUI.prototype._createWrapperCreators = function () {
	        var sandbox = this;
	        return {
	            getElementsByClassName: function (nativeGetElementsByClassNameFnName) {
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    var elements = nativeMethods[nativeGetElementsByClassNameFnName].apply(this, args);
	                    var length = nativeMethods.htmlCollectionLengthGetter.call(elements);
	                    return sandbox._filterNodeList(elements, length);
	                };
	            },
	            getElementsByTagName: function (nativeGetElementsByTagNameFnName) {
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    var nativeCollection = nativeMethods[nativeGetElementsByTagNameFnName].apply(this, args);
	                    var tagName = args[0];
	                    if (typeof tagName !== 'string' || !isHeadOrBodyOrHtmlElement(this) && !isFormElement(this) &&
	                        tagName.toLowerCase() !== 'input' && nativeGetElementsByTagNameFnName !== 'getElementsByTagName')
	                        return nativeCollection;
	                    if (!nativeCollection[HTML_COLLECTION_WRAPPER])
	                        // NOTE: This changes how the native method behaves. The returned collection will have this wrapper attached
	                        // if the method was called with the same tagName parameter.
	                        // This allows skipping the search if the DOM tree has not changed since the last call.
	                        nativeCollection[HTML_COLLECTION_WRAPPER] = new HTMLCollectionWrapper(nativeCollection, tagName);
	                    else
	                        nativeCollection[HTML_COLLECTION_WRAPPER]._refreshCollection();
	                    return nativeCollection[HTML_COLLECTION_WRAPPER];
	                };
	            },
	            querySelector: function (nativeQuerySelectorFnName, nativeQuerySelectorAllFnName) {
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    if (typeof args[0] === 'string')
	                        args[0] = NodeSandbox.processSelector(args[0]);
	                    var element = nativeMethods[nativeQuerySelectorFnName].apply(this, args);
	                    var filteredElement = ShadowUI._filterElement(element);
	                    if (!element || filteredElement)
	                        return filteredElement;
	                    return ShadowUI._getFirstNonShadowElement(nativeMethods[nativeQuerySelectorAllFnName].apply(this, args));
	                };
	            },
	            querySelectorAll: function (nativeQuerySelectorAllFnName) {
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    if (typeof args[0] === 'string')
	                        args[0] = NodeSandbox.processSelector(args[0]);
	                    var list = nativeMethods[nativeQuerySelectorAllFnName].apply(this, args);
	                    var length = nativeMethods.nodeListLengthGetter.call(list);
	                    return sandbox._filterNodeList(list, length);
	                };
	            }
	        };
	    };
	    ShadowUI.prototype._markShadowUIContainerAndCollections = function (containerEl) {
	        var children = nativeMethods.elementChildrenGetter.call(containerEl);
	        ShadowUI._markAsShadowContainer(containerEl);
	        ShadowUI.markAsShadowContainerCollection(children);
	        ShadowUI.markAsShadowContainerCollection(nativeMethods.nodeChildNodesGetter.call(containerEl));
	    };
	    ShadowUI.prototype.markShadowUIContainers = function (head, body) {
	        if (head)
	            this._markShadowUIContainerAndCollections(head);
	        if (body)
	            this._markShadowUIContainerAndCollections(body);
	    };
	    ShadowUI.prototype._bringRootToWindowTopLeft = function () {
	        var rootHasParentWithNonStaticPosition = false;
	        var parent = nativeMethods.nodeParentNodeGetter.call(this.root);
	        while (parent) {
	            var elementPosition = get$1(parent, 'position');
	            if (IS_NON_STATIC_POSITION_RE.test(elementPosition))
	                rootHasParentWithNonStaticPosition = true;
	            parent = nativeMethods.nodeParentNodeGetter.call(parent);
	        }
	        if (rootHasParentWithNonStaticPosition) {
	            var rootOffset = getOffsetPosition(this.root);
	            if (rootOffset.left !== 0 || rootOffset.top !== 0) {
	                var currentRootLeft = parseFloat(get$1(this.root, 'left')) || 0;
	                var currentRootTop = parseFloat(get$1(this.root, 'top')) || 0;
	                var newRootLeft = currentRootLeft - rootOffset.left + 'px';
	                var newRootTop = currentRootTop - rootOffset.top + 'px';
	                set(this.root, 'left', newRootLeft);
	                set(this.root, 'top', newRootTop);
	            }
	        }
	    };
	    ShadowUI.prototype._overrideDocumentMethods = function (window, document) {
	        var shadowUI = this;
	        var docProto = window.Document.prototype;
	        overrideFunction(docProto, 'elementFromPoint', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            // NOTE: T212974
	            shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	            var res = ShadowUI._filterElement(nativeMethods.elementFromPoint.apply(this, args));
	            shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	            return res;
	        });
	        if (document.caretRangeFromPoint) {
	            overrideFunction(docProto, 'caretRangeFromPoint', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	                var res = nativeMethods.caretRangeFromPoint.apply(this, args);
	                if (res && res.startContainer && !ShadowUI._filterElement(res.startContainer))
	                    res = null;
	                shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	                return res;
	            });
	        }
	        if (document.caretPositionFromPoint) {
	            overrideFunction(docProto, 'caretPositionFromPoint', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	                var res = nativeMethods.caretPositionFromPoint.apply(this, args);
	                if (res && res.offsetNode && !ShadowUI._filterElement(res.offsetNode))
	                    res = null;
	                shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
	                return res;
	            });
	        }
	        overrideFunction(docProto, 'getElementById', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            return ShadowUI._filterElement(nativeMethods.getElementById.apply(this, args));
	        });
	        overrideFunction(docProto, 'getElementsByName', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var elements = nativeMethods.getElementsByName.apply(this, args);
	            var length = getElementsByNameReturnsHTMLCollection
	                ? nativeMethods.htmlCollectionLengthGetter.call(elements)
	                : nativeMethods.nodeListLengthGetter.call(elements);
	            return shadowUI._filterNodeList(elements, length);
	        });
	        overrideFunction(docProto, 'getElementsByClassName', this.wrapperCreators.getElementsByClassName('getElementsByClassName'));
	        overrideFunction(docProto, 'getElementsByTagName', this.wrapperCreators.getElementsByTagName('getElementsByTagName'));
	        overrideFunction(docProto, 'querySelector', this.wrapperCreators.querySelector('querySelector', 'querySelectorAll'));
	        overrideFunction(docProto, 'querySelectorAll', this.wrapperCreators.querySelectorAll('querySelectorAll'));
	    };
	    ShadowUI.prototype._overrideElementMethods = function (window) {
	        var elementProto = window.Element.prototype;
	        var bodyProto = window.HTMLBodyElement.prototype;
	        var headProto = window.HTMLHeadElement.prototype;
	        overrideFunction(elementProto, 'getElementsByTagName', this.wrapperCreators.getElementsByTagName('elementGetElementsByTagName'));
	        overrideFunction(bodyProto, 'getElementsByClassName', this.wrapperCreators.getElementsByClassName('elementGetElementsByClassName'));
	        overrideFunction(bodyProto, 'querySelector', this.wrapperCreators.querySelector('elementQuerySelector', 'elementQuerySelectorAll'));
	        overrideFunction(bodyProto, 'querySelectorAll', this.wrapperCreators.querySelectorAll('elementQuerySelectorAll'));
	        overrideFunction(headProto, 'getElementsByClassName', bodyProto.getElementsByClassName);
	        overrideFunction(headProto, 'querySelector', bodyProto.querySelector);
	        overrideFunction(headProto, 'querySelectorAll', bodyProto.querySelectorAll);
	    };
	    ShadowUI.prototype._getUIStyleSheetsHtml = function () {
	        var stylesSelector = 'link.' + SHADOW_UI_CLASS_NAME.uiStylesheet;
	        var stylesheets = this.nativeMethods.querySelectorAll.call(this.document, stylesSelector);
	        var length = this.nativeMethods.nodeListLengthGetter.call(stylesheets);
	        var result = '';
	        for (var i = 0; i < length; i++)
	            result += nativeMethods.elementOuterHTMLGetter.call(stylesheets[i]);
	        return result;
	    };
	    ShadowUI.prototype._restoreUIStyleSheets = function (head, uiStyleSheetsHtml) {
	        if (!head || !uiStyleSheetsHtml)
	            return;
	        var parser = this.nativeMethods.createElement.call(this.document, 'div');
	        nativeMethods.elementInnerHTMLSetter.call(parser, uiStyleSheetsHtml);
	        var children = nativeMethods.elementChildrenGetter.call(parser);
	        var length = nativeMethods.htmlCollectionLengthGetter.call(children);
	        for (var i = 0; i < length; i++) {
	            var child = children[0];
	            ShadowUI.markElementAsShadow(child);
	            this.nativeMethods.removeChild.call(parser, child);
	            this.nativeMethods.appendChild.call(head, child);
	        }
	    };
	    ShadowUI.prototype._markElementsAsShadowInHead = function (head) {
	        var children = nativeMethods.elementChildrenGetter.call(head);
	        var length = nativeMethods.htmlCollectionLengthGetter.call(children);
	        for (var i = 0; i < length; i++) {
	            var child = children[i];
	            if (ShadowUI.containsShadowUIClassPostfix(child))
	                ShadowUI.markElementAsShadow(child);
	        }
	    };
	    ShadowUI.prototype.getRoot = function () {
	        var _this = this;
	        // GH-2418
	        if (isChrome && !ShadowUI.isShadowContainer(this.document.body))
	            this._markShadowUIContainerAndCollections(this.document.body);
	        if (!this.root || /* NOTE: T225944 */ !this.document.body.contains(this.root)) {
	            if (!this.root) {
	                // NOTE: B254893
	                this.root = nativeMethods.createElement.call(this.document, 'div');
	                nativeMethods.setAttribute.call(this.root, 'id', ShadowUI.patchId(this.ROOT_ID));
	                nativeMethods.setAttribute.call(this.root, 'contenteditable', 'false');
	                this.addClass(this.root, this.ROOT_CLASS);
	                ShadowUI.markElementAsShadow(this.root);
	                nativeMethods.appendChild.call(this.document.body, this.root);
	                var nativeDocumentAddEventListener = nativeMethods.documentAddEventListener || nativeMethods.addEventListener;
	                for (var _i = 0, _a = domProcessor.EVENTS; _i < _a.length; _i++) {
	                    var event_1 = _a[_i];
	                    nativeMethods.addEventListener.call(this.root, event_1, stopPropagation);
	                }
	                this._bringRootToWindowTopLeft();
	                nativeDocumentAddEventListener.call(this.document, 'DOMContentLoaded', function () {
	                    _this.onBodyElementMutation();
	                    _this._bringRootToWindowTopLeft();
	                });
	            }
	            else
	                nativeMethods.appendChild.call(this.document.body, this.root);
	        }
	        return this.root;
	    };
	    ShadowUI.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window, window.document);
	        this.markShadowUIContainers(this.document.head, this.document.body);
	        this._overrideDocumentMethods(window, window.document);
	        this._overrideElementMethods(window);
	        // NOTE: document.head equals null after call 'document.open' function
	        if (window.document.head)
	            this._markElementsAsShadowInHead(window.document.head);
	        this._initEvents();
	    };
	    ShadowUI.prototype._initEvents = function () {
	        this._iframeSandbox.on(this._iframeSandbox.RUN_TASK_SCRIPT_EVENT, this.runTaskScriptEventCallback);
	        this._nodeMutation.on(this._nodeMutation.BEFORE_DOCUMENT_CLEANED_EVENT, this.beforeDocumentCleanedEventCallback);
	        this._nodeMutation.on(this._nodeMutation.DOCUMENT_CLEANED_EVENT, this.documentCleanedEventCallback);
	        this._nodeMutation.on(this._nodeMutation.DOCUMENT_CLOSED_EVENT, this.documentClosedEventCallback);
	        this._nodeMutation.on(this._nodeMutation.BODY_CONTENT_CHANGED_EVENT, this.bodyContentChangedEventCallback);
	        this._messageSandbox.on(this._messageSandbox.SERVICE_MSG_RECEIVED_EVENT, this.serviceMsgReceivedEventCallback);
	        this._nodeMutation.on(this._nodeMutation.BODY_CREATED_EVENT, this.bodyCreatedEventCallback);
	    };
	    ShadowUI.prototype.onBodyElementMutation = function () {
	        if (!this.root || !this.document.body)
	            return;
	        var isRootInDom = closest(this.root, 'html');
	        var isRootLastChild = !this.nativeMethods.elementNextElementSiblingGetter.call(this.root);
	        // NOTE: Fix for B239138 - The 'Cannot read property 'document' of null' error
	        // is thrown on recording on the unroll.me site. There was an issue when
	        // document.body was replaced, so we need to reattach a UI to a new body manually.
	        var isRootInBody = nativeMethods.nodeParentNodeGetter.call(this.root) === this.document.body;
	        if (!(isRootInDom && isRootLastChild && isRootInBody))
	            this.nativeMethods.appendChild.call(this.document.body, this.root);
	        this.markShadowUIContainers(this.document.head, this.document.body);
	    };
	    // Accessors
	    ShadowUI.prototype.getFirstChild = function (el) {
	        var childNodes = nativeMethods.nodeChildNodesGetter.call(el);
	        var length = nativeMethods.nodeListLengthGetter.call(childNodes);
	        var filteredNodes = this._filterNodeList(childNodes, length);
	        return filteredNodes[0] || null;
	    };
	    ShadowUI.prototype.getFirstElementChild = function (el) {
	        var children = nativeMethods.elementChildrenGetter.call(el);
	        var length = nativeMethods.htmlCollectionLengthGetter.call(children);
	        return this._filterNodeList(children, length)[0] || null;
	    };
	    ShadowUI.prototype.getLastChild = function (el) {
	        var childNodes = nativeMethods.nodeChildNodesGetter.call(el);
	        var length = nativeMethods.nodeListLengthGetter.call(childNodes);
	        var filteredNodes = this._filterNodeList(childNodes, length);
	        var index = childNodes === filteredNodes ? length - 1 : filteredNodes.length - 1;
	        return index >= 0 ? filteredNodes[index] : null;
	    };
	    ShadowUI.prototype.getLastElementChild = function (el) {
	        var children = nativeMethods.elementChildrenGetter.call(el);
	        var length = nativeMethods.htmlCollectionLengthGetter.call(children);
	        var filteredNodes = this._filterNodeList(children, length);
	        var index = children === filteredNodes ? length - 1 : filteredNodes.length - 1;
	        return index >= 0 ? filteredNodes[index] : null;
	    };
	    ShadowUI.prototype.getNextSibling = function (el) {
	        if (!el)
	            return el;
	        do
	            el = nativeMethods.nodeNextSiblingGetter.call(el);
	        while (el && isShadowUIElement(el));
	        return el;
	    };
	    ShadowUI.prototype.getPrevSibling = function (el) {
	        if (!el)
	            return el;
	        do
	            el = nativeMethods.nodePrevSiblingGetter.call(el);
	        while (el && isShadowUIElement(el));
	        return el;
	    };
	    ShadowUI.prototype.getMutationRecordNextSibling = function (el) {
	        if (!el)
	            return el;
	        while (el && isShadowUIElement(el))
	            el = nativeMethods.nodeNextSiblingGetter.call(el);
	        return el;
	    };
	    ShadowUI.prototype.getMutationRecordPrevSibling = function (el) {
	        if (!el)
	            return el;
	        while (el && isShadowUIElement(el))
	            el = nativeMethods.nodePrevSiblingGetter.call(el);
	        return el;
	    };
	    ShadowUI.prototype.getNextElementSibling = function (el) {
	        do
	            el = nativeMethods.elementNextElementSiblingGetter.call(el);
	        while (el && isShadowUIElement(el));
	        return el;
	    };
	    ShadowUI.prototype.getPrevElementSibling = function (el) {
	        do
	            el = nativeMethods.elementPrevElementSiblingGetter.call(el);
	        while (el && isShadowUIElement(el));
	        return el;
	    };
	    // Utils
	    ShadowUI._checkElementsPosition = function (collection, length) {
	        if (!length)
	            return;
	        var shadowUIElements = [];
	        for (var i = 0; i < length; i++) {
	            var item = collection[i];
	            if (isShadowUIElement(item))
	                shadowUIElements.push(item);
	        }
	        var collectionOwner = shadowUIElements.length && nativeMethods.nodeParentNodeGetter.call(shadowUIElements[0]);
	        for (var _i = 0, shadowUIElements_1 = shadowUIElements; _i < shadowUIElements_1.length; _i++) {
	            var shadowUIElement = shadowUIElements_1[_i];
	            nativeMethods.appendChild.call(collectionOwner, shadowUIElement);
	        }
	    };
	    ShadowUI._hasFlag = function (obj, flag) {
	        try {
	            return !!obj[flag];
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    // IE11 and Edge have a strange behavior: shadow container collection flag may be lost (GH-1763 and GH-2034)
	    ShadowUI._hasCollectionFlagForIE = function (obj, flag) {
	        try {
	            if (flag in obj)
	                return obj[flag];
	            var parent_1 = nativeMethods.nodeParentNodeGetter.call(obj[0]);
	            var result = isHeadOrBodyElement(parent_1) || isFormElement(parent_1);
	            nativeMethods.objectDefineProperty(obj, IS_SHADOW_CONTAINER_COLLECTION_FLAG, { value: result, configurable: true });
	            return result;
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    ShadowUI.isShadowContainer = function (el) {
	        return ShadowUI._hasFlag(el, IS_SHADOW_CONTAINER_FLAG);
	    };
	    ShadowUI.isShadowContainerCollection = function (collection, length) {
	        return isIE && length
	            ? ShadowUI._hasCollectionFlagForIE(collection, IS_SHADOW_CONTAINER_COLLECTION_FLAG)
	            : ShadowUI._hasFlag(collection, IS_SHADOW_CONTAINER_COLLECTION_FLAG);
	    };
	    ShadowUI._isShadowUIChildListMutation = function (mutation) {
	        if (isShadowUIElement(mutation.target))
	            return true;
	        var removedNodesLength = nativeMethods.nodeListLengthGetter.call(mutation.removedNodes);
	        for (var i = 0; i < removedNodesLength; i++) {
	            if (isShadowUIElement(mutation.removedNodes[i]))
	                return true;
	        }
	        var addedNodesLength = nativeMethods.nodeListLengthGetter.call(mutation.addedNodes);
	        for (var i = 0; i < addedNodesLength; i++) {
	            if (isShadowUIElement(mutation.addedNodes[i]))
	                return true;
	        }
	        return false;
	    };
	    ShadowUI._isShadowUIAttributeMutation = function (mutation) {
	        return isShadowUIElement(mutation.target) || isHammerheadAttr(mutation.attributeName);
	    };
	    ShadowUI._isShadowUICharacterDataMutation = function (mutation) {
	        return isShadowUIElement(mutation.target);
	    };
	    ShadowUI.isShadowUIMutation = function (mutation) {
	        switch (mutation.type) {
	            case 'childList':
	                return ShadowUI._isShadowUIChildListMutation(mutation);
	            case 'attributes':
	                return ShadowUI._isShadowUIAttributeMutation(mutation);
	            case 'characterData':
	                return ShadowUI._isShadowUICharacterDataMutation(mutation);
	            default:
	                return false;
	        }
	    };
	    ShadowUI.removeSelfRemovingScripts = function (document) {
	        var selfRemovingScripts = nativeMethods.querySelectorAll.call(document, '.' + SHADOW_UI_CLASS_NAME.selfRemovingScript);
	        var length = nativeMethods.nodeListLengthGetter.call(selfRemovingScripts);
	        for (var i = 0; i < length; i++)
	            removeElement(selfRemovingScripts[i]);
	    };
	    // API
	    ShadowUI.prototype.getShadowUICollectionLength = function (collection, length) {
	        var shadowUIElementCount = 0;
	        for (var i = 0; i < length; i++) {
	            if (isShadowUIElement(collection[i]))
	                shadowUIElementCount++;
	        }
	        if (shadowUIElementCount && !this._ieDebugSandbox.isDebuggerInitiator())
	            ShadowUI._checkElementsPosition(collection, length);
	        return length - shadowUIElementCount;
	    };
	    // NOTE: this method cannot be static because it is a part of the public API
	    ShadowUI.prototype.addClass = function (el, value) {
	        var patchedClass = ShadowUI.patchClassNames(value);
	        addClass(el, patchedClass);
	    };
	    // NOTE: this method cannot be static because it is a part of the public API
	    ShadowUI.prototype.removeClass = function (elem, value) {
	        var patchedClass = ShadowUI.patchClassNames(value);
	        removeClass(elem, patchedClass);
	    };
	    ShadowUI.hasClass = function (el, value) {
	        var patchedClass = ShadowUI.patchClassNames(value);
	        return hasClass(el, patchedClass);
	    };
	    ShadowUI.patchId = function (value) {
	        return value + SHADOW_UI_CLASS_NAME.postfix;
	    };
	    ShadowUI.patchClassNames = function (value) {
	        var names = value.split(/\s+/);
	        for (var i = 0; i < names.length; i++)
	            names[i] += SHADOW_UI_CLASS_NAME.postfix;
	        return names.join(' ');
	    };
	    ShadowUI.prototype.select = function (selector, context) {
	        var patchedSelector = selector.replace(CLASSNAME_RE, function (className) { return className + SHADOW_UI_CLASS_NAME.postfix; });
	        return context
	            ? nativeMethods.elementQuerySelectorAll.call(context, patchedSelector)
	            : nativeMethods.querySelectorAll.call(this.document, patchedSelector);
	    };
	    ShadowUI.prototype.setBlind = function (value) {
	        if (value)
	            this.addClass(this.getRoot(), this.BLIND_CLASS);
	        else
	            this.removeClass(this.getRoot(), this.BLIND_CLASS);
	    };
	    ShadowUI.prototype.getLastActiveElement = function () {
	        return this.lastActiveElement;
	    };
	    ShadowUI.prototype.setLastActiveElement = function (el) {
	        this.lastActiveElement = el;
	    };
	    ShadowUI.prototype.insertBeforeRoot = function (newNodes) {
	        var rootEl = this.getRoot();
	        var rootParent = this.nativeMethods.nodeParentNodeGetter.call(rootEl);
	        var lastParentChild = this.nativeMethods.nodeLastChildGetter.call(rootParent);
	        // GH-2418
	        if (lastParentChild != rootEl)
	            nativeMethods.appendChild.call(rootParent, rootEl);
	        if (newNodes.length > 1 || typeof newNodes[0] !== 'object') {
	            var fragment = document.createDocumentFragment.call(this.document);
	            for (var _i = 0, newNodes_1 = newNodes; _i < newNodes_1.length; _i++) {
	                var node = newNodes_1[_i];
	                if (typeof node === 'string')
	                    node = nativeMethods.createTextNode.call(this.document, node);
	                nativeMethods.appendChild.call(fragment, node);
	            }
	            return nativeMethods.insertBefore.call(rootParent, fragment, rootEl);
	        }
	        return nativeMethods.insertBefore.call(rootParent, newNodes[0], rootEl);
	    };
	    ShadowUI.markElementAsShadow = function (el) {
	        el[INTERNAL_PROPS.shadowUIElement] = true;
	    };
	    // GH-2009
	    ShadowUI.markFormAsShadow = function (form) {
	        var formChildren = nativeMethods.elementChildrenGetter.call(form);
	        ShadowUI._markAsShadowContainer(form);
	        ShadowUI.markAsShadowContainerCollection(form.elements);
	        ShadowUI.markAsShadowContainerCollection(formChildren);
	        var childNodes = nativeMethods.nodeChildNodesGetter.call(form);
	        ShadowUI.markAsShadowContainerCollection(childNodes);
	    };
	    ShadowUI.markElementAndChildrenAsShadow = function (el) {
	        ShadowUI.markElementAsShadow(el);
	        // NOTE: For Text, Comment and ProcessingInstruction nodes
	        if (!el.querySelectorAll)
	            return;
	        var childElements = getNativeQuerySelectorAll(el).call(el, '*');
	        var length = nativeMethods.nodeListLengthGetter.call(childElements);
	        for (var i = 0; i < length; i++)
	            ShadowUI.markElementAsShadow(childElements[i]);
	    };
	    ShadowUI._markAsShadowContainer = function (container) {
	        nativeMethods.objectDefineProperty(container, IS_SHADOW_CONTAINER_FLAG, { value: true });
	    };
	    ShadowUI.markAsShadowContainerCollection = function (collection) {
	        nativeMethods.objectDefineProperty(collection, IS_SHADOW_CONTAINER_COLLECTION_FLAG, { value: true, configurable: true });
	    };
	    ShadowUI.containsShadowUIClassPostfix = function (element) {
	        return typeof element.className === 'string' &&
	            element.className.indexOf(SHADOW_UI_CLASS_NAME.postfix) !== -1;
	    };
	    return ShadowUI;
	}(SandboxBase));

	function constructorIsCalledWithoutNewKeyword (callingContext, constructor) {
	    return callingContext instanceof constructor === false;
	}

	// https://mimesniff.spec.whatwg.org/
	var IMAGE_TYPE_PATTERNS = [
	    {
	        mime: 'image/x-icon',
	        pattern: [0x00, 0x00, 0x01, 0x00],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/x-icon',
	        pattern: [0x00, 0x00, 0x02, 0x00],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/bmp',
	        pattern: [0x42, 0x4D],
	        mask: [0xFF, 0xFF]
	    },
	    {
	        mime: 'image/gif',
	        pattern: [0x47, 0x49, 0x46, 0x38, 0x37, 0x61],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/gif',
	        pattern: [0x47, 0x49, 0x46, 0x38, 0x39, 0x61],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/webp',
	        pattern: [0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00, 0x57, 0x45, 0x42, 0x50, 0x56, 0x50],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/png',
	        pattern: [0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'image/jpeg',
	        pattern: [0xFF, 0xD8, 0xFF],
	        mask: [0xFF, 0xFF, 0xFF]
	    }
	];
	var AUDIO_VIDEO_TYPE_PATTERNS = [
	    {
	        mime: 'audio/basic',
	        pattern: [0x2E, 0x73, 0x6E, 0x64],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'audio/aiff',
	        pattern: [0x46, 0x4F, 0x52, 0x4D, 0x00, 0x00, 0x00, 0x00, 0x41, 0x49, 0x46, 0x46],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'audio/mpeg',
	        pattern: [0x49, 0x44, 0x33],
	        mask: [0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/ogg',
	        pattern: [0x4F, 0x67, 0x67, 0x53, 0x00],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'audio/midi',
	        pattern: [0x4D, 0x54, 0x68, 0x64, 0x00, 0x00, 0x00, 0x06],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'video/avi',
	        pattern: [0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00, 0x41, 0x56, 0x49, 0x20],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'audio/wave',
	        pattern: [0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00, 0x57, 0x41, 0x56, 0x45],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF]
	    }
	];
	var FONT_TYPE_PATTERNS = [
	    {
	        mime: 'application/vnd.ms-fontobject',
	        pattern: [0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4C, 0x50],
	        mask: [0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/octet-stream',
	        pattern: [0x00, 0x01, 0x00, 0x00],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/octet-stream',
	        pattern: [0x4F, 0x54, 0x54, 0x4F],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/octet-stream',
	        pattern: [0x74, 0x74, 0x63, 0x66],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/font-woff',
	        pattern: [0x77, 0x4F, 0x46, 0x46],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    }
	];
	var ARCHIVE_TYPE_PATTERNS = [
	    {
	        mime: 'application/x-gzip',
	        pattern: [0x1F, 0x8B, 0x08],
	        mask: [0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/zip',
	        pattern: [0x50, 0x4B, 0x03, 0x04],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF]
	    },
	    {
	        mime: 'application/x-rar-compressed',
	        pattern: [0x52, 0x61, 0x72, 0x20, 0x1A, 0x07, 0x00],
	        mask: [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
	    }
	];
	function matchPattern(pattern, data) {
	    if (data.length < pattern.pattern.length)
	        return false;
	    var p = 0;
	    var s = 0;
	    var maskedData = null;
	    while (p < pattern.pattern.length) {
	        maskedData = data[s] & pattern.mask[p];
	        if (maskedData !== pattern.pattern[p])
	            return false;
	        s++;
	        p++;
	    }
	    return true;
	}
	function matchMime(patternGroup, data) {
	    // Now, we don't support the cases when the data is is divided into several parts and its parts have different types
	    if (isArrayBuffer(data[0]))
	        data = data[0];
	    else if (isArrayBufferView(data[0]))
	        data = isDataView(data[0]) ? data[0].buffer : data[0];
	    var byteArray = new nativeMethods.Uint8Array(data);
	    for (var _i = 0, patternGroup_1 = patternGroup; _i < patternGroup_1.length; _i++) {
	        var pattern = patternGroup_1[_i];
	        if (matchPattern(pattern, byteArray))
	            return pattern.mime;
	    }
	    byteArray = null;
	    return '';
	}
	function getMimeType (data) {
	    return matchMime(IMAGE_TYPE_PATTERNS, data) ||
	        matchMime(AUDIO_VIDEO_TYPE_PATTERNS, data) ||
	        matchMime(FONT_TYPE_PATTERNS, data) ||
	        matchMime(ARCHIVE_TYPE_PATTERNS, data);
	}

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	var ServiceCommands = {
	    getUploadedFiles: 'hammerhead|command|get-uploaded-files',
	    setCookie: 'hammerhead|command|set-cookie',
	    uploadFiles: 'hammerhead|command|upload-files'
	};

	var FileListWrapper = /** @class */ (function () {
	    function FileListWrapper(fileList) {
	        var _this = this;
	        nativeMethods.objectDefineProperty(this, 'length', {
	            get: function () { return fileList.length; }
	        });
	        for (var i = 0; i < fileList.length; i++)
	            this[i] = FileListWrapper._createFileWrapper(fileList[i]);
	        this.item = function (index) { return _this[index]; };
	    }
	    FileListWrapper._base64ToBlob = function (base64Data, fileInfo, sliceSize) {
	        var mimeType = fileInfo.info.type || '';
	        sliceSize = sliceSize || 512;
	        var byteCharacters = atob(base64Data);
	        var byteArrays = [];
	        for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
	            var slice = byteCharacters.slice(offset, offset + sliceSize);
	            var byteNumbers = new Array(slice.length);
	            for (var i = 0; i < slice.length; i++)
	                byteNumbers[i] = slice.charCodeAt(i);
	            byteArrays.push(new Uint8Array(byteNumbers));
	        }
	        // NOTE: window.File in IE11 is not constructable.
	        return nativeMethods.File
	            ? new File(byteArrays, fileInfo.info.name, { type: mimeType, lastModified: fileInfo.info.lastModified })
	            : new Blob(byteArrays, { type: mimeType });
	    };
	    FileListWrapper._createFileWrapper = function (fileInfo) {
	        var wrapper = null;
	        if (!window.Blob) {
	            wrapper = {
	                size: fileInfo.info.size,
	                type: fileInfo.info.type
	            };
	        }
	        else if (fileInfo.blob) {
	            // NOTE: window.File in IE11 is not constructable.
	            wrapper = nativeMethods.File
	                ? new File([fileInfo.blob], fileInfo.info.name, { type: fileInfo.info.type, lastModified: fileInfo.info.lastModified })
	                : new Blob([fileInfo.blob], { type: fileInfo.info.type });
	        }
	        else
	            wrapper = FileListWrapper._base64ToBlob(fileInfo.data, fileInfo);
	        wrapper.name = fileInfo.info.name;
	        if (fileInfo.info.lastModifiedDate)
	            wrapper.lastModifiedDate = fileInfo.info.lastModifiedDate;
	        wrapper.base64 = fileInfo.data;
	        return wrapper;
	    };
	    return FileListWrapper;
	}());
	//@ts-ignore
	if (window.FileList)
	    FileListWrapper.prototype = FileList.prototype;

	// NOTE: https://html.spec.whatwg.org/multipage/forms.html#fakepath-srsly.
	var FAKE_PATH_STRING = 'C:\\fakepath\\';
	var UploadInfoManager = /** @class */ (function () {
	    function UploadInfoManager(_transport) {
	        this._transport = _transport;
	        this.uploadInfo = [];
	    }
	    UploadInfoManager._getFileListData = function (fileList) {
	        var data = [];
	        for (var _i = 0, fileList_1 = fileList; _i < fileList_1.length; _i++) {
	            var file = fileList_1[_i];
	            data.push(file.base64);
	        }
	        return data;
	    };
	    UploadInfoManager.formatValue = function (fileNames) {
	        var value = '';
	        fileNames = typeof fileNames === 'string' ? [fileNames] : fileNames;
	        if (fileNames && fileNames.length) {
	            if (isWebKit)
	                value = FAKE_PATH_STRING + fileNames[0].split('/').pop();
	            else
	                return fileNames[0].split('/').pop();
	        }
	        return value;
	    };
	    UploadInfoManager.getFileNames = function (fileList, value) {
	        var result = [];
	        if (fileList) {
	            for (var _i = 0, fileList_2 = fileList; _i < fileList_2.length; _i++) {
	                var file = fileList_2[_i];
	                result.push(file.name);
	            }
	        }
	        else if (value.lastIndexOf('\\') !== -1)
	            result.push(value.substr(value.lastIndexOf('\\') + 1));
	        return result;
	    };
	    UploadInfoManager.prototype.loadFilesInfoFromServer = function (filePaths) {
	        return this._transport.asyncServiceMsg({
	            cmd: ServiceCommands.getUploadedFiles,
	            filePaths: typeof filePaths === 'string' ? [filePaths] : filePaths
	        });
	    };
	    UploadInfoManager.prepareFileListWrapper = function (filesInfo) {
	        var errs = [];
	        var validFilesInfo = [];
	        for (var _i = 0, filesInfo_1 = filesInfo; _i < filesInfo_1.length; _i++) {
	            var fileInfo = filesInfo_1[_i];
	            if (fileInfo.err)
	                errs.push(fileInfo);
	            else
	                validFilesInfo.push(fileInfo);
	        }
	        return {
	            errs: errs,
	            fileList: new FileListWrapper(validFilesInfo)
	        };
	    };
	    UploadInfoManager.prototype.sendFilesInfoToServer = function (fileList, fileNames) {
	        return this._transport.asyncServiceMsg({
	            cmd: ServiceCommands.uploadFiles,
	            data: UploadInfoManager._getFileListData(fileList),
	            fileNames: fileNames
	        });
	    };
	    UploadInfoManager.prototype.clearUploadInfo = function (input) {
	        var inputInfo = this.getUploadInfo(input);
	        if (inputInfo) {
	            /*eslint-disable no-restricted-properties*/
	            inputInfo.files = new FileListWrapper([]);
	            inputInfo.value = '';
	            /*eslint-enable no-restricted-properties*/
	            return removeInputInfo(input);
	        }
	        return null;
	    };
	    UploadInfoManager.prototype.getFiles = function (input) {
	        var inputInfo = this.getUploadInfo(input);
	        // eslint-disable-next-line no-restricted-properties
	        return inputInfo ? inputInfo.files : new FileListWrapper([]);
	    };
	    UploadInfoManager.prototype.getUploadInfo = function (input) {
	        for (var _i = 0, _a = this.uploadInfo; _i < _a.length; _i++) {
	            var uploadInfoItem = _a[_i];
	            if (uploadInfoItem.input === input)
	                return uploadInfoItem;
	        }
	        return null;
	    };
	    UploadInfoManager.prototype.getValue = function (input) {
	        var inputInfo = this.getUploadInfo(input);
	        // eslint-disable-next-line no-restricted-properties
	        return inputInfo ? inputInfo.value : '';
	    };
	    UploadInfoManager.prototype.loadFileListData = function (_input, fileList) {
	        if (!fileList.length)
	            return pinkie.resolve(new FileListWrapper([]));
	        return new pinkie(function (resolve) {
	            var fileReader = new FileReader();
	            var readedFiles = [];
	            var index = 0;
	            var file = fileList[index];
	            fileReader.addEventListener('load', function (e) {
	                var info = {
	                    type: file.type,
	                    name: file.name
	                };
	                if (typeof file.lastModified === 'number')
	                    info.lastModified = file.lastModified;
	                if (file.lastModifiedDate)
	                    info.lastModifiedDate = file.lastModifiedDate;
	                readedFiles.push({
	                    data: e.target.result.substr(e.target.result.indexOf(',') + 1),
	                    blob: file.slice(0, file.size),
	                    info: info
	                });
	                if (fileList[++index]) {
	                    file = fileList[index];
	                    fileReader.readAsDataURL(file);
	                }
	                else
	                    resolve(new FileListWrapper(readedFiles));
	            });
	            fileReader.readAsDataURL(file);
	        });
	    };
	    UploadInfoManager.prototype.setUploadInfo = function (input, fileList, value) {
	        var inputInfo = this.getUploadInfo(input);
	        if (!inputInfo) {
	            inputInfo = { input: input };
	            this.uploadInfo.push(inputInfo);
	        }
	        /*eslint-disable no-restricted-properties*/
	        inputInfo.files = fileList;
	        inputInfo.value = value;
	        /*eslint-enable no-restricted-properties*/
	        addInputInfo(input, fileList, value);
	    };
	    return UploadInfoManager;
	}());

	var SANDBOX_BACKUP = 'hammerhead|sandbox-backup';
	function findRecord(storage, iframe) {
	    for (var i = storage.length - 1; i >= 0; i--) {
	        try {
	            if (storage[i].iframe === iframe)
	                return storage[i];
	        }
	        catch (e) {
	            storage.splice(i, 1);
	        }
	    }
	    return void 0;
	}
	function create(window, sandbox) {
	    var topSameDomainWindow = getTopSameDomainWindow(window);
	    var iframe = window !== topSameDomainWindow ? getFrameElement(window) : null;
	    var storage = topSameDomainWindow[SANDBOX_BACKUP];
	    if (!storage) {
	        storage = [];
	        nativeMethods.objectDefineProperty(topSameDomainWindow, SANDBOX_BACKUP, { value: storage });
	    }
	    var record = findRecord(storage, iframe);
	    if (record)
	        record.sandbox = sandbox;
	    else
	        storage.push({ iframe: iframe, sandbox: sandbox });
	}
	function get(window) {
	    var topSameDomainWindow = getTopSameDomainWindow(window);
	    var storage = topSameDomainWindow[SANDBOX_BACKUP];
	    var iframe = window !== topSameDomainWindow ? window.frameElement : null;
	    if (storage) {
	        var record = findRecord(storage, iframe);
	        return record ? record.sandbox : null;
	    }
	    return null;
	}

	var sandboxBackupUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		create: create,
		get: get
	});

	var UploadSandbox = /** @class */ (function (_super) {
	    __extends(UploadSandbox, _super);
	    function UploadSandbox(_listeners, _eventSimulator, transport) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this._eventSimulator = _eventSimulator;
	        _this.START_FILE_UPLOADING_EVENT = 'hammerhead|event|start-file-uploading';
	        _this.END_FILE_UPLOADING_EVENT = 'hammerhead|event|end-file-uploading';
	        _this.infoManager = new UploadInfoManager(transport);
	        return _this;
	    }
	    UploadSandbox.prototype._riseChangeEvent = function (input) {
	        this._eventSimulator.change(input);
	    };
	    UploadSandbox._getCurrentInfoManager = function (input) {
	        var contextWindow = input[INTERNAL_PROPS.processedContext];
	        return get(contextWindow).upload.infoManager;
	    };
	    /*eslint-disable max-nested-callbacks */
	    UploadSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._listeners.addInternalEventBeforeListener(window, ['change'], function (e, dispatched) {
	            var input = e.target;
	            var currentInfoManager = UploadSandbox._getCurrentInfoManager(input);
	            if (!dispatched && isFileInput(input)) {
	                stopPropagation(e);
	                preventDefault(e);
	                var value_1 = nativeMethods.inputValueGetter.call(input);
	                if (!!value_1 || !!currentInfoManager.getValue(input)) {
	                    var files = nativeMethods.inputFilesGetter.call(input);
	                    var fileNames_1 = UploadInfoManager.getFileNames(files, value_1);
	                    _this.emit(_this.START_FILE_UPLOADING_EVENT, fileNames_1, input);
	                    currentInfoManager.loadFileListData(input, files)
	                        .then(function (fileList) {
	                        currentInfoManager.setUploadInfo(input, fileList, value_1);
	                        return _this.infoManager.sendFilesInfoToServer(fileList, fileNames_1);
	                    })
	                        .then(function (uploadInfo) {
	                        _this._riseChangeEvent(input);
	                        _this.emit(_this.END_FILE_UPLOADING_EVENT, uploadInfo);
	                    });
	                }
	            }
	        });
	        if (!settings.get().isRecordMode && (isIE || isFirefox)) {
	            // NOTE: Google Chrome and Safari don't open the native browser dialog when TestCafe clicks on the input.
	            // 'Click' is a complex emulated action that uses 'dispatchEvent' method internally.
	            // Another browsers open the native browser dialog in this case.
	            // This is why, we are forced to prevent the browser's open file dialog.
	            this._listeners.addInternalEventBeforeListener(window, ['click'], function (e, dispatched) {
	                if (dispatched && isFileInput(e.target))
	                    preventDefault(e, true);
	            });
	        }
	    };
	    /*eslint-enable max-nested-callbacks */
	    UploadSandbox.getFiles = function (input) {
	        var files = nativeMethods.inputFilesGetter.call(input);
	        return files !== void 0 ? UploadSandbox._getCurrentInfoManager(input).getFiles(input) : void 0;
	    };
	    UploadSandbox.getUploadElementValue = function (input) {
	        return UploadSandbox._getCurrentInfoManager(input).getValue(input);
	    };
	    UploadSandbox.prototype.setUploadElementValue = function (input, value) {
	        if (value === '' && UploadSandbox._getCurrentInfoManager(input).clearUploadInfo(input) && isIE11)
	            this._riseChangeEvent(input);
	    };
	    // GH-1844, GH-2007
	    UploadSandbox._shouldRaiseChangeEvent = function (filesToUpload, currentUploadInfo) {
	        if (!currentUploadInfo)
	            return true;
	        // eslint-disable-next-line no-restricted-properties
	        var currentFiles = currentUploadInfo.files;
	        if (filesToUpload.length !== currentFiles.length ||
	            isFirefox || (isMacPlatform && isChrome || isSafari))
	            return true;
	        for (var _i = 0, filesToUpload_1 = filesToUpload; _i < filesToUpload_1.length; _i++) {
	            var file = filesToUpload_1[_i];
	            var found = false;
	            for (var _a = 0, currentFiles_1 = currentFiles; _a < currentFiles_1.length; _a++) {
	                var currentFile = currentFiles_1[_a];
	                if (file.name === currentFile.name) {
	                    found = true;
	                    break;
	                }
	            }
	            if (found === false)
	                return true;
	        }
	        return false;
	    };
	    UploadSandbox.prototype.doUpload = function (input, filePaths) {
	        var _this = this;
	        var currentInfoManager = UploadSandbox._getCurrentInfoManager(input);
	        filePaths = filePaths || [];
	        return this.infoManager.loadFilesInfoFromServer(filePaths)
	            .then(function (filesInfo) { return UploadInfoManager.prepareFileListWrapper(filesInfo); })
	            .then(function (data) {
	            if (!data.errs.length) {
	                var value = UploadInfoManager.formatValue(filePaths);
	                var inputInfo = currentInfoManager.getUploadInfo(input);
	                var shouldRaiseChangeEvent = UploadSandbox._shouldRaiseChangeEvent(data.fileList, inputInfo);
	                currentInfoManager.setUploadInfo(input, data.fileList, value);
	                if (shouldRaiseChangeEvent)
	                    _this._riseChangeEvent(input);
	            }
	            return data.errs;
	        });
	    };
	    return UploadSandbox;
	}(SandboxBase));

	var anchor = nativeMethods.createElement.call(document, 'a');
	var emptyAnchor = nativeMethods.createElement.call(document, 'a');
	function getAnchorProperty(el, nativePropGetter) {
	    var href = nativeMethods.anchorHrefGetter.call(el);
	    if (!anchor)
	        reattach();
	    if (href) {
	        nativeMethods.anchorHrefSetter.call(anchor, getDestinationUrl(href));
	        return nativePropGetter.call(anchor);
	    }
	    return nativePropGetter.call(emptyAnchor);
	}
	function setAnchorProperty(el, nativePropSetter, value) {
	    var href = nativeMethods.anchorHrefGetter.call(el);
	    if (!anchor)
	        reattach();
	    if (href) {
	        nativeMethods.anchorHrefSetter.call(anchor, getDestinationUrl(href));
	        nativePropSetter.call(anchor, value);
	        el.setAttribute('href', nativeMethods.anchorHrefGetter.call(anchor));
	    }
	    return value;
	}
	function reattach() {
	    anchor = nativeMethods.createElement.call(document, 'a');
	    emptyAnchor = nativeMethods.createElement.call(document, 'a');
	}
	function dispose() {
	    anchor = null;
	    emptyAnchor = null;
	}

	var DefaultTarget;
	(function (DefaultTarget) {
	    DefaultTarget["form"] = "_self";
	    DefaultTarget["linkOrArea"] = "_self";
	    DefaultTarget["windowOpen"] = "_blank";
	})(DefaultTarget || (DefaultTarget = {}));
	var defaultTarget = DefaultTarget;

	var SET_SERVICE_WORKER_SETTINGS = 'hammerhead|set-service-worker-settings';
	var SET_BLOB_WORKER_SETTINGS = 'hammerhead|set-blob-worker-settings';

	var INSTRUCTION_VALUES = (function () {
	    var values = [];
	    var keys = nativeMethods.objectKeys(SCRIPT_PROCESSING_INSTRUCTIONS);
	    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
	        var key = keys_1[_i];
	        values.push(SCRIPT_PROCESSING_INSTRUCTIONS[key]);
	    }
	    return values;
	})();
	var HTTP_PROTOCOL_RE = /^http/i;
	var ALLOWED_SERVICE_WORKER_PROTOCOLS = ['https:', 'wss:', 'file:'];
	var ALLOWED_SERVICE_WORKER_HOST_NAMES = ['localhost', '127.0.0.1'];
	var JAVASCRIPT_MIME_TYPES = ['text/javascript', 'application/javascript', 'application/x-javascript'];
	// NOTE: SVGAnimatedString prototype does not have a way to access the appropriate svg element.
	// This is why we use this property to store svg element for which animVal and baseVal properties were set.
	// It allows us to change the href-hammerhead-stored-value when it needs.
	var CONTEXT_SVG_IMAGE_ELEMENT = 'hammerhead|context-svg-image-element';
	var SANDBOX_DOM_TOKEN_LIST = 'hammerhead|sandbox-dom-token-list';
	var SANDBOX_DOM_TOKEN_LIST_OWNER = 'hammerhead|sandbox-dom-token-list-owner';
	var SANDBOX_DOM_TOKEN_LIST_UPDATE_FN = 'hammerhead|sandbox-dom-token-list-update';
	var IS_PROXY_OBJECT_INTERNAL_PROP_NAME = 'hammerhead|is-proxy-object|internal-prop-name';
	var IS_PROXY_OBJECT_INTERNAL_PROP_VALUE = 'hammerhead|is-proxy-object|internal-prop-value';
	var PROXY_HANDLER_FLAG = 'hammerhead|proxy-handler-flag';
	var NO_STACK_TRACE_AVAILABLE_MESSAGE = 'No stack trace available';
	var DEFAULT_UNHANDLED_REJECTION_REASON_NAME = 'Error';
	var TRACKED_EVENTS = ['error', 'unhandledrejection', 'hashchange'];
	var WindowSandbox = /** @class */ (function (_super) {
	    __extends(WindowSandbox, _super);
	    function WindowSandbox(nodeSandbox, eventSandbox, uploadSandbox, nodeMutation, _childWindowSandbox, _documentTitleStorageInitializer) {
	        var _this = _super.call(this) || this;
	        _this._childWindowSandbox = _childWindowSandbox;
	        _this._documentTitleStorageInitializer = _documentTitleStorageInitializer;
	        _this.UNCAUGHT_JS_ERROR_EVENT = 'hammerhead|event|uncaught-js-error';
	        _this.UNHANDLED_REJECTION_EVENT = 'hammerhead|event|unhandled-rejection';
	        _this.HASH_CHANGE_EVENT = 'hammerhead|event|hashchange-event';
	        _this.nodeSandbox = nodeSandbox;
	        _this.messageSandbox = eventSandbox.message;
	        _this.listenersSandbox = eventSandbox.listeners;
	        _this.elementEditingWatcher = eventSandbox.elementEditingWatcher;
	        _this.eventSimulator = eventSandbox.eventSimulator;
	        _this.uploadSandbox = uploadSandbox;
	        _this.shadowUI = nodeSandbox.shadowUI;
	        _this.nodeMutation = nodeMutation;
	        _this.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN = SANDBOX_DOM_TOKEN_LIST_UPDATE_FN;
	        return _this;
	    }
	    WindowSandbox._prepareStack = function (msg, stack) {
	        // NOTE: Firefox does not include an error message in a stack trace (unlike other browsers)
	        // It is possible to get a stack trace for unhandled Promise rejections only if Promise is rejected with the 'Error' instance value.
	        // This is why we should convert the stack to a common format.
	        if (!stack || stack.indexOf(msg) === -1) {
	            stack = stack || "    " + NO_STACK_TRACE_AVAILABLE_MESSAGE;
	            return msg + "\n" + stack;
	        }
	        return stack;
	    };
	    WindowSandbox._isProcessableBlobParts = function (parts) {
	        var hasStringItem = false;
	        for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
	            var item = parts_1[_i];
	            if (!hasStringItem && typeof item === 'string') {
	                hasStringItem = true;
	                continue;
	            }
	            if (typeof item !== 'string' && typeof item !== 'number' && typeof item !== 'boolean')
	                return false;
	        }
	        return hasStringItem;
	    };
	    WindowSandbox._isProcessableBlob = function (array, opts) {
	        var type = opts && opts.type && opts.type.toString().toLowerCase() || getMimeType(array);
	        // NOTE: If we cannot identify the content type of data, we're trying to process it as a script
	        // (in the case of the "Array<string | number | boolean>" blob parts array: GH-2115).
	        // Unfortunately, we do not have the ability to exactly identify a script. That's why we make such
	        // an assumption. We cannot solve this problem at the Worker level either, because the operation of
	        // creating a new Blob instance is asynchronous. (GH-231)
	        return (!type || JAVASCRIPT_MIME_TYPES.indexOf(type) !== -1) && WindowSandbox._isProcessableBlobParts(array);
	    };
	    WindowSandbox.prototype._getWindowOpenTarget = function (originTarget) {
	        if (originTarget)
	            return this.nodeSandbox.element.getCorrectedTarget(String(originTarget));
	        return settings.get().allowMultipleWindows ? defaultTarget.windowOpen : '_self';
	    };
	    WindowSandbox.prototype._raiseUncaughtJsErrorEvent = function (type, event, window) {
	        if (isCrossDomainWindows(window, window.top))
	            return;
	        var sendToTopWindow = isIframeWindow(window);
	        var pageUrl = get$2();
	        var msg = null;
	        var stack = null;
	        if (type === this.UNHANDLED_REJECTION_EVENT) {
	            msg = WindowSandbox._formatUnhandledRejectionReason(event.reason);
	            stack = event.reason && event.reason.stack;
	        }
	        else if (type === this.UNCAUGHT_JS_ERROR_EVENT) {
	            msg = event.error ? event.error.message : event.message;
	            stack = event.error && event.error.stack;
	        }
	        stack = WindowSandbox._prepareStack(msg, stack);
	        stack = replaceProxiedUrlsInStack(stack);
	        if (sendToTopWindow) {
	            this.emit(type, { msg: msg, pageUrl: pageUrl, stack: stack, inIframe: true });
	            this.messageSandbox.sendServiceMsg({ msg: msg, pageUrl: pageUrl, stack: stack, cmd: type }, window.top);
	        }
	        else
	            this.emit(type, { msg: msg, pageUrl: pageUrl, stack: stack });
	    };
	    WindowSandbox.prototype._reattachHandler = function (window, eventName) {
	        var nativeAddEventListener = nativeMethods.windowAddEventListener || nativeMethods.addEventListener;
	        var nativeRemoveEventListener = nativeMethods.windowRemoveEventListener || nativeMethods.removeEventListener;
	        nativeRemoveEventListener.call(window, eventName, this);
	        nativeAddEventListener.call(window, eventName, this);
	    };
	    WindowSandbox._formatUnhandledRejectionReason = function (reason) {
	        if (!isPrimitiveType(reason)) {
	            if (reason instanceof nativeMethods.Error) {
	                var name_1 = reason.name || DEFAULT_UNHANDLED_REJECTION_REASON_NAME;
	                return name_1 + ": " + reason.message;
	            }
	            return nativeMethods.objectToString.call(reason);
	        }
	        return String(reason);
	    };
	    WindowSandbox._getUrlAttr = function (el, attr) {
	        var attrValue = nativeMethods.getAttribute.call(el, attr);
	        var currentDocument = el.ownerDocument || document;
	        if (attrValue === '' || attrValue === null && attr === 'action' && emptyActionAttrFallbacksToTheLocation)
	            return urlResolver.resolve('', currentDocument);
	        else if (attrValue === null)
	            return '';
	        else if (HASH_RE$1.test(attrValue))
	            return urlResolver.resolve(attrValue, currentDocument);
	        else if (!isValidUrl(attrValue))
	            return urlResolver.resolve(attrValue, currentDocument);
	        return resolveUrlAsDest(attrValue);
	    };
	    WindowSandbox._removeProcessingInstructions = function (text) {
	        if (text) {
	            text = remove$1(text);
	            return styleProcessor.cleanUp(text, parseProxyUrl);
	        }
	        return text;
	    };
	    WindowSandbox._processTextPropValue = function (el, text) {
	        var processedText = text !== null && text !== void 0 ? String(text) : text;
	        if (processedText) {
	            if (isScriptElement(el))
	                return processScript(processedText, true, false, convertToProxyUrl);
	            else if (isStyleElement(el))
	                return styleProcessor.process(processedText, getProxyUrl, true);
	        }
	        return processedText;
	    };
	    WindowSandbox.prototype._overrideUrlAttrDescriptors = function (attr, elementConstructors) {
	        var windowSandbox = this;
	        for (var _i = 0, elementConstructors_1 = elementConstructors; _i < elementConstructors_1.length; _i++) {
	            var constructor = elementConstructors_1[_i];
	            overrideDescriptor(constructor.prototype, attr, {
	                getter: function () {
	                    return WindowSandbox._getUrlAttr(this, attr);
	                },
	                setter: function (value) {
	                    windowSandbox.nodeSandbox.element.setAttributeCore(this, [attr, value]);
	                }
	            });
	        }
	    };
	    WindowSandbox.prototype._overrideAttrDescriptors = function (attr, elementConstructors) {
	        var windowSandbox = this;
	        for (var _i = 0, elementConstructors_2 = elementConstructors; _i < elementConstructors_2.length; _i++) {
	            var constructor = elementConstructors_2[_i];
	            overrideDescriptor(constructor.prototype, attr, {
	                getter: function () {
	                    return windowSandbox.nodeSandbox.element.getAttributeCore(this, [attr]) || '';
	                },
	                setter: function (value) {
	                    windowSandbox.nodeSandbox.element.setAttributeCore(this, [attr, value]);
	                }
	            });
	        }
	    };
	    WindowSandbox.prototype._overrideUrlPropDescriptor = function (prop, nativePropGetter, nativePropSetter) {
	        // @ts-ignore
	        overrideDescriptor(window.HTMLAnchorElement.prototype, prop, {
	            getter: function () {
	                return getAnchorProperty(this, nativePropGetter);
	            },
	            setter: function (value) {
	                setAnchorProperty(this, nativePropSetter, value);
	            }
	        });
	    };
	    WindowSandbox.prototype._overrideEventPropDescriptor = function (window, eventName, nativePropSetter) {
	        var _this = this;
	        // @ts-ignore
	        var eventPropsOwner = nativeMethods.isEventPropsLocatedInProto ? window.Window.prototype : window;
	        overrideDescriptor(eventPropsOwner, 'on' + eventName, {
	            getter: null,
	            setter: function (handler) {
	                nativePropSetter.call(window, handler);
	                _this.listenersSandbox.emit(_this.listenersSandbox.EVENT_LISTENER_ATTACHED_EVENT, {
	                    el: window,
	                    listener: handler,
	                    eventType: eventName
	                });
	            }
	        });
	    };
	    WindowSandbox.prototype._createOverriddenDOMTokenListMethod = function (nativeMethod) {
	        var windowSandbox = this;
	        return function () {
	            var executionResult = nativeMethod.apply(this, arguments);
	            var tokenListOwner = this[SANDBOX_DOM_TOKEN_LIST_OWNER];
	            if (tokenListOwner)
	                // eslint-disable-next-line no-restricted-properties
	                windowSandbox.nodeSandbox.element.setAttributeCore(tokenListOwner, ['sandbox', this.toString()]);
	            return executionResult;
	        };
	    };
	    WindowSandbox._isSecureOrigin = function (url) {
	        // NOTE: https://www.chromium.org/Home/chromium-security/prefer-secure-origins-for-powerful-new-features
	        var parsedUrl = parseUrl(resolveUrlAsDest(url));
	        /*eslint-disable no-restricted-properties*/
	        return ALLOWED_SERVICE_WORKER_PROTOCOLS.indexOf(parsedUrl.protocol) === -1 &&
	            ALLOWED_SERVICE_WORKER_HOST_NAMES.indexOf(parsedUrl.hostname) === -1;
	        /*eslint-enable no-restricted-properties*/
	    };
	    WindowSandbox.prototype._setSandboxedTextForTitleElements = function (el) {
	        if (isIframeWindow(window))
	            return;
	        var titleElements = getNativeQuerySelectorAll(el).call(el, 'title');
	        for (var _i = 0, titleElements_1 = titleElements; _i < titleElements_1.length; _i++) {
	            var titleElement = titleElements_1[_i];
	            // NOTE: SVGTitleElement can be here (GH-2364)
	            if (!isTitleElement(titleElement))
	                continue;
	            var nativeText = nativeMethods.titleElementTextGetter.call(titleElement);
	            this._documentTitleStorageInitializer.storage.setTitleElementPropertyValue(titleElement, nativeText);
	        }
	    };
	    WindowSandbox.isProxyObject = function (obj) {
	        try {
	            return obj[IS_PROXY_OBJECT_INTERNAL_PROP_NAME] === IS_PROXY_OBJECT_INTERNAL_PROP_VALUE;
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    WindowSandbox.prototype.handleEvent = function (event) {
	        if (event.defaultPrevented)
	            return;
	        if (event.type === 'unhandledrejection')
	            this._raiseUncaughtJsErrorEvent(this.UNHANDLED_REJECTION_EVENT, event, this.window);
	        else if (event.type === 'error') {
	            if (event.message.indexOf('NS_ERROR_NOT_INITIALIZED') !== -1)
	                event.preventDefault();
	            else
	                this._raiseUncaughtJsErrorEvent(this.UNCAUGHT_JS_ERROR_EVENT, event, window);
	        }
	        else if (event.type === 'hashchange')
	            this.emit(this.HASH_CHANGE_EVENT);
	    };
	    WindowSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        var messageSandbox = this.messageSandbox;
	        var nodeSandbox = this.nodeSandbox;
	        var windowSandbox = this;
	        nativeMethods.arrayForEach.call(TRACKED_EVENTS, function (event) {
	            _this._reattachHandler(window, event);
	        });
	        this.listenersSandbox.initElementListening(window, TRACKED_EVENTS);
	        this.listenersSandbox.on(this.listenersSandbox.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
	            if (e.el !== window)
	                return;
	            if (TRACKED_EVENTS.indexOf(e.eventType) !== -1)
	                _this._reattachHandler(window, e.eventType);
	        });
	        this._overrideEventPropDescriptor(window, 'error', nativeMethods.winOnErrorSetter);
	        this._overrideEventPropDescriptor(window, 'hashchange', nativeMethods.winOnHashChangeSetter);
	        if (nativeMethods.winOnUnhandledRejectionSetter)
	            this._overrideEventPropDescriptor(window, 'unhandledrejection', nativeMethods.winOnUnhandledRejectionSetter);
	        messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (e) {
	            var _a = e.message, msg = _a.msg, pageUrl = _a.pageUrl, stack = _a.stack, cmd = _a.cmd;
	            if (cmd === _this.UNCAUGHT_JS_ERROR_EVENT || cmd === _this.UNHANDLED_REJECTION_EVENT)
	                windowSandbox.emit(cmd, { msg: msg, pageUrl: pageUrl, stack: stack });
	        });
	        overrideFunction(window.CanvasRenderingContext2D.prototype, 'drawImage', function () {
	            var _this = this;
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var image = args[0];
	            if (isImgElement(image) && !image[INTERNAL_PROPS.forceProxySrcForImage]) {
	                var src = nativeMethods.imageSrcGetter.call(image);
	                if (sameOriginCheck(location.toString(), src)) {
	                    image = nativeMethods.createElement.call(window.document, 'img');
	                    nativeMethods.imageSrcSetter.call(image, getProxyUrl(src));
	                    args[0] = image;
	                    if (!image.complete) {
	                        nativeMethods.addEventListener.call(image, 'load', function () { return nativeMethods.canvasContextDrawImage.apply(_this, args); });
	                    }
	                }
	            }
	            return nativeMethods.canvasContextDrawImage.apply(this, args);
	        });
	        if (nativeMethods.objectAssign) {
	            overrideFunction(window.Object, 'assign', function (target) {
	                var sources = [];
	                for (var _i = 1; _i < arguments.length; _i++) {
	                    sources[_i - 1] = arguments[_i];
	                }
	                var args = [target];
	                var targetType = typeof target;
	                if (target && (targetType === 'object' || targetType === 'function') && sources.length) {
	                    for (var _a = 0, sources_1 = sources; _a < sources_1.length; _a++) {
	                        var source = sources_1[_a];
	                        var sourceType = typeof source;
	                        if (!source || sourceType !== 'object' && sourceType !== 'function') {
	                            nativeMethods.objectAssign.call(this, target, source);
	                            continue;
	                        }
	                        var sourceSymbols = nativeMethods.objectGetOwnPropertySymbols.call(window.Object, source);
	                        var sourceKeys = nativeMethods.arrayConcat.call(nativeMethods.objectKeys.call(window.Object, source), sourceSymbols);
	                        for (var _b = 0, sourceKeys_1 = sourceKeys; _b < sourceKeys_1.length; _b++) {
	                            var key = sourceKeys_1[_b];
	                            window[SCRIPT_PROCESSING_INSTRUCTIONS.setProperty](target, key, source[key]);
	                        }
	                    }
	                }
	                else
	                    args = nativeMethods.arrayConcat.call(args, sources);
	                return nativeMethods.objectAssign.apply(this, args);
	            });
	        }
	        overrideFunction(window, 'open', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            args[0] = getProxyUrl(args[0]);
	            args[1] = windowSandbox._getWindowOpenTarget(args[1]);
	            return windowSandbox._childWindowSandbox.handleWindowOpen(window, args);
	        });
	        if (window.FontFace) {
	            overrideConstructor(window, 'FontFace', function (family, source, descriptors) {
	                source = styleProcessor.process(source, convertToProxyUrl);
	                return new nativeMethods.FontFace(family, source, descriptors);
	            });
	        }
	        if (window.Worker) {
	            overrideConstructor(window, 'Worker', function WorkerWrapper() {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var isCalledWithoutNewKeyword = constructorIsCalledWithoutNewKeyword(this, WorkerWrapper);
	                if (arguments.length === 0)
	                    // @ts-ignore
	                    return isCalledWithoutNewKeyword ? nativeMethods.Worker() : new nativeMethods.Worker();
	                if (isCalledWithoutNewKeyword)
	                    return nativeMethods.Worker.apply(this, args);
	                var scriptURL = args[0];
	                if (typeof scriptURL !== 'string')
	                    scriptURL = String(scriptURL);
	                scriptURL = getProxyUrl(scriptURL, { resourceType: stringifyResourceType({ isScript: true }) });
	                var worker = arguments.length === 1
	                    ? new nativeMethods.Worker(scriptURL)
	                    : new nativeMethods.Worker(scriptURL, args[1]);
	                // eslint-disable-next-line no-restricted-properties
	                if (parseUrl(scriptURL).protocol == 'blob:') {
	                    worker.postMessage({
	                        cmd: SET_BLOB_WORKER_SETTINGS,
	                        sessionId: settings.get().sessionId,
	                        windowId: settings.get().windowId,
	                        origin: getOriginHeader()
	                    });
	                }
	                return worker;
	            }, true);
	        }
	        if (window.Blob) {
	            overrideConstructor(window, 'Blob', function (array, opts) {
	                if (arguments.length === 0)
	                    return new nativeMethods.Blob();
	                if (WindowSandbox._isProcessableBlob(array, opts))
	                    array = [processScript(array.join(''), true, false, convertToProxyUrl)];
	                // NOTE: IE11 throws an error when the second parameter of the Blob function is undefined (GH-44)
	                // If the overridden function is called with one parameter, we need to call the original function
	                // with one parameter as well.
	                return arguments.length === 1 ? new nativeMethods.Blob(array) : new nativeMethods.Blob(array, opts);
	            });
	        }
	        // NOTE: non-IE11 case. window.File in IE11 is not constructable.
	        if (nativeMethods.File) {
	            overrideConstructor(window, 'File', function (array, fileName, opts) {
	                if (arguments.length === 0)
	                    return new nativeMethods.File();
	                if (WindowSandbox._isProcessableBlob(array, opts))
	                    array = [processScript(array.join(''), true, false, convertToProxyUrl)];
	                return new nativeMethods.File(array, fileName, opts);
	            });
	        }
	        if (window.EventSource) {
	            overrideConstructor(window, 'EventSource', function (url, opts) {
	                if (arguments.length) {
	                    var proxyUrl = getProxyUrl(url, { resourceType: stringifyResourceType({ isEventSource: true }) });
	                    if (arguments.length === 1)
	                        return new nativeMethods.EventSource(proxyUrl);
	                    return new nativeMethods.EventSource(proxyUrl, opts);
	                }
	                return new nativeMethods.EventSource();
	            });
	            window.EventSource.CONNECTING = nativeMethods.EventSource.CONNECTING;
	            window.EventSource.OPEN = nativeMethods.EventSource.OPEN;
	            window.EventSource.CLOSED = nativeMethods.EventSource.CLOSED;
	        }
	        if (window.MutationObserver) {
	            overrideConstructor(window, 'MutationObserver', function (callback) {
	                var wrapper = function (mutations) {
	                    var result = [];
	                    for (var _i = 0, mutations_1 = mutations; _i < mutations_1.length; _i++) {
	                        var mutation = mutations_1[_i];
	                        if (!ShadowUI.isShadowUIMutation(mutation))
	                            result.push(mutation);
	                    }
	                    if (result.length)
	                        callback.call(this, result, this);
	                };
	                return new nativeMethods.MutationObserver(wrapper);
	            });
	            if (window.WebKitMutationObserver)
	                window.WebKitMutationObserver = window.MutationObserver;
	        }
	        if (window.Proxy) {
	            overrideConstructor(window, 'Proxy', function (target, handler) {
	                if (handler.get && !handler.get[PROXY_HANDLER_FLAG]) {
	                    var storedGet_1 = handler.get;
	                    handler.get = function (getterTarget, name, receiver) {
	                        if (name === IS_PROXY_OBJECT_INTERNAL_PROP_NAME)
	                            return IS_PROXY_OBJECT_INTERNAL_PROP_VALUE;
	                        else if (INSTRUCTION_VALUES.indexOf(name) > -1)
	                            return window[name];
	                        var result = storedGet_1.call(this, getterTarget, name, receiver);
	                        if (name === 'eval' && result[CodeInstrumentation.WRAPPED_EVAL_FN])
	                            return result[CodeInstrumentation.WRAPPED_EVAL_FN];
	                        return result;
	                    };
	                    nativeMethods.objectDefineProperty(handler.get, PROXY_HANDLER_FLAG, { value: true, enumerable: false });
	                }
	                return new nativeMethods.Proxy(target, handler);
	            });
	            window.Proxy.revocable = nativeMethods.Proxy.revocable;
	        }
	        if (nativeMethods.registerServiceWorker) {
	            overrideFunction(window.navigator.serviceWorker, 'register', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var url = args[0], opts = args[1];
	                if (typeof url === 'string') {
	                    if (WindowSandbox._isSecureOrigin(url)) {
	                        // NOTE: We cannot create an instance of the DOMException in the Android 6.0 and in the Edge 17 browsers.
	                        // The 'TypeError: Illegal constructor' error is raised if we try to call the constructor.
	                        return pinkie.reject(isAndroid || isMSEdge && version >= 17
	                            ? new Error('Only secure origins are allowed.')
	                            : new DOMException('Only secure origins are allowed.', 'SecurityError'));
	                    }
	                    args[0] = getProxyUrl(url, { resourceType: stringifyResourceType({ isServiceWorker: true }) });
	                }
	                args[1] = { scope: '/' };
	                return nativeMethods.registerServiceWorker.apply(window.navigator.serviceWorker, args)
	                    .then(function (reg) { return new pinkie(function (resolve, reject) {
	                    var parsedProxyUrl = parseProxyUrl(args[0]);
	                    var serviceWorker = reg.installing;
	                    if (!serviceWorker) {
	                        resolve(reg);
	                        return;
	                    }
	                    var channel = new nativeMethods.MessageChannel();
	                    serviceWorker.postMessage({
	                        cmd: SET_SERVICE_WORKER_SETTINGS,
	                        currentScope: getScope(url),
	                        optsScope: getScope(opts && opts.scope),
	                        protocol: parsedProxyUrl.destResourceInfo.protocol,
	                        host: parsedProxyUrl.destResourceInfo.host // eslint-disable-line no-restricted-properties
	                    }, [channel.port1]);
	                    channel.port2.onmessage = function (e) {
	                        var data = nativeMethods.messageEventDataGetter.call(e);
	                        if (data.error)
	                            reject(new Error(data.error));
	                        else
	                            resolve(reg);
	                    };
	                }); });
	            });
	        }
	        if (nativeMethods.getRegistrationServiceWorker) {
	            overrideFunction(window.navigator.serviceWorker, 'getRegistration', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                if (typeof args[0] === 'string')
	                    args[0] = '/';
	                return nativeMethods.getRegistrationServiceWorker.apply(window.navigator.serviceWorker, args);
	            });
	        }
	        if (window.Range.prototype.createContextualFragment) {
	            overrideFunction(window.Range.prototype, 'createContextualFragment', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var tagString = args[0];
	                if (typeof tagString === 'string') {
	                    args[0] = processHtml(tagString, {
	                        processedContext: this.startContainer && this.startContainer[INTERNAL_PROPS.processedContext]
	                    });
	                }
	                var fragment = nativeMethods.createContextualFragment.apply(this, args);
	                nodeSandbox.processNodes(fragment);
	                return fragment;
	            });
	        }
	        if (window.EventTarget) {
	            var overriddenMethods = this.listenersSandbox.createOverriddenMethods();
	            overrideFunction(window.EventTarget.prototype, 'addEventListener', overriddenMethods.addEventListener);
	            overrideFunction(window.EventTarget.prototype, 'removeEventListener', overriddenMethods.removeEventListener);
	        }
	        overrideConstructor(window, 'Image', function () {
	            var image = null;
	            if (!arguments.length)
	                image = new nativeMethods.Image();
	            else if (arguments.length === 1)
	                image = new nativeMethods.Image(arguments[0]);
	            else
	                image = new nativeMethods.Image(arguments[0], arguments[1]);
	            image[INTERNAL_PROPS.forceProxySrcForImage] = true;
	            nodeSandbox.processNodes(image);
	            return image;
	        });
	        overrideConstructor(window, 'Function', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var functionBodyArgIndex = args.length - 1;
	            if (typeof args[functionBodyArgIndex] === 'string')
	                args[functionBodyArgIndex] = processScript(args[functionBodyArgIndex], false, false, convertToProxyUrl);
	            return nativeMethods.Function.apply(this, args);
	        }, true);
	        overrideFunction(window.Function.prototype, 'toString', function () {
	            if (nativeMethods.objectHasOwnProperty.call(this, INTERNAL_PROPS.nativeStrRepresentation))
	                return this[INTERNAL_PROPS.nativeStrRepresentation];
	            return nativeMethods.functionToString.call(this);
	        });
	        if (typeof window.history.pushState === 'function' && typeof window.history.replaceState === 'function') {
	            var createWrapperForHistoryStateManipulationFn = function (nativeFn) {
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    var url = args[2];
	                    if (args.length > 2 && (url !== null && (isIE || url !== void 0)))
	                        args[2] = getProxyUrl(url);
	                    return nativeFn.apply(this, args);
	                };
	            };
	            overrideFunction(window.history, 'pushState', createWrapperForHistoryStateManipulationFn(nativeMethods.historyPushState));
	            overrideFunction(window.history, 'replaceState', createWrapperForHistoryStateManipulationFn(nativeMethods.historyReplaceState));
	        }
	        if (window.navigator.sendBeacon) {
	            overrideFunction(window.navigator, 'sendBeacon', function () {
	                if (typeof arguments[0] === 'string')
	                    arguments[0] = getProxyUrl(arguments[0]);
	                return nativeMethods.sendBeacon.apply(this, arguments);
	            });
	        }
	        if (window.navigator.registerProtocolHandler) {
	            overrideFunction(window.navigator, 'registerProtocolHandler', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var urlIndex = 1;
	                if (typeof args[urlIndex] === 'string') {
	                    // eslint-disable-next-line no-restricted-properties
	                    var destHostname = getParsed().hostname;
	                    var isDestUrl = false;
	                    if (isFirefox) {
	                        var parsedUrl = parseUrl(args[urlIndex]);
	                        // eslint-disable-next-line no-restricted-properties
	                        isDestUrl = parsedUrl.hostname && destHostname === parsedUrl.hostname;
	                    }
	                    else
	                        isDestUrl = sameOriginCheck(get$2(), args[urlIndex]);
	                    if (isDestUrl)
	                        args[urlIndex] = getProxyUrl(args[urlIndex]);
	                }
	                return nativeMethods.registerProtocolHandler.apply(navigator, args);
	            });
	        }
	        if (window.FormData) {
	            overrideFunction(window.FormData.prototype, 'append', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var name = args[0], value = args[1];
	                // NOTE: We should not send our hidden input's value along with the file info,
	                // because our input may have incorrect value if the input with the file has been removed from DOM.
	                if (name === INTERNAL_ATTRIBUTES.uploadInfoHiddenInputName)
	                    return;
	                // NOTE: If we append our file wrapper to FormData, we will lose the file name.
	                // This happens because the file wrapper is an instance of Blob
	                // and a browser thinks that Blob does not contain the "name" property.
	                if (args.length === 2 && isBlob(value) && 'name' in value)
	                    args[2] = value['name'];
	                nativeMethods.formDataAppend.apply(this, args);
	            });
	        }
	        if (window.WebSocket) {
	            overrideConstructor(window, 'WebSocket', function (url, protocols) {
	                if (arguments.length === 0)
	                    return new nativeMethods.WebSocket();
	                var proxyUrl = getProxyUrl(url, { resourceType: stringifyResourceType({ isWebSocket: true }) });
	                if (arguments.length === 1)
	                    return new nativeMethods.WebSocket(proxyUrl);
	                else if (arguments.length === 2)
	                    return new nativeMethods.WebSocket(proxyUrl, protocols);
	                return new nativeMethods.WebSocket(proxyUrl, protocols, arguments[2]);
	            });
	            window.WebSocket.CONNECTING = nativeMethods.WebSocket.CONNECTING;
	            window.WebSocket.OPEN = nativeMethods.WebSocket.OPEN;
	            window.WebSocket.CLOSING = nativeMethods.WebSocket.CLOSING;
	            window.WebSocket.CLOSED = nativeMethods.WebSocket.CLOSED;
	            if (nativeMethods.webSocketUrlGetter) {
	                overrideDescriptor(window.WebSocket.prototype, 'url', {
	                    getter: function () {
	                        var url = nativeMethods.webSocketUrlGetter.call(this);
	                        var parsedUrl = parseProxyUrl(url);
	                        if (parsedUrl && parsedUrl.destUrl)
	                            return parsedUrl.destUrl.replace(HTTP_PROTOCOL_RE, 'ws');
	                        return url;
	                    }
	                });
	            }
	        }
	        overrideDescriptor(window.MessageEvent.prototype, 'origin', {
	            getter: function () {
	                var target = this.target;
	                var origin = nativeMethods.messageEventOriginGetter.call(this);
	                if (isWebSocket(target)) {
	                    var parsedUrl = parseUrl(target.url);
	                    if (parsedUrl)
	                        // eslint-disable-next-line no-restricted-properties
	                        return parsedUrl.protocol + '//' + parsedUrl.host;
	                }
	                else if (isWindow(target)) {
	                    var data = nativeMethods.messageEventDataGetter.call(this);
	                    if (data)
	                        return data.originUrl;
	                }
	                return origin;
	            }
	        });
	        overrideDescriptor(window.HTMLCollection.prototype, 'length', {
	            getter: function () {
	                var length = nativeMethods.htmlCollectionLengthGetter.call(this);
	                if (ShadowUI.isShadowContainerCollection(this, length))
	                    return windowSandbox.shadowUI.getShadowUICollectionLength(this, length);
	                return length;
	            }
	        });
	        overrideDescriptor(window.NodeList.prototype, 'length', {
	            getter: function () {
	                var length = nativeMethods.nodeListLengthGetter.call(this);
	                if (ShadowUI.isShadowContainerCollection(this))
	                    return windowSandbox.shadowUI.getShadowUICollectionLength(this, length);
	                return length;
	            }
	        });
	        overrideDescriptor(window.Element.prototype, 'childElementCount', {
	            getter: function () {
	                if (ShadowUI.isShadowContainer(this)) {
	                    var children = nativeMethods.elementChildrenGetter.call(this);
	                    var length_1 = nativeMethods.htmlCollectionLengthGetter.call(children);
	                    return windowSandbox.shadowUI.getShadowUICollectionLength(children, length_1);
	                }
	                return nativeMethods.elementChildElementCountGetter.call(this);
	            }
	        });
	        if (nativeMethods.performanceEntryNameGetter) {
	            overrideDescriptor(window.PerformanceEntry.prototype, 'name', {
	                getter: function () {
	                    var name = nativeMethods.performanceEntryNameGetter.call(this);
	                    if (isPerformanceNavigationTiming(this)) {
	                        var parsedProxyUrl = parseProxyUrl(name);
	                        if (parsedProxyUrl)
	                            return parsedProxyUrl.destUrl;
	                    }
	                    return name;
	                }
	            });
	        }
	        overrideDescriptor(window.HTMLInputElement.prototype, 'files', {
	            getter: function () {
	                if (this.type.toLowerCase() === 'file')
	                    return UploadSandbox.getFiles(this);
	                return nativeMethods.inputFilesGetter.call(this);
	            }
	        });
	        overrideDescriptor(window.HTMLInputElement.prototype, 'value', {
	            getter: function () {
	                if (this.type.toLowerCase() === 'file')
	                    return UploadSandbox.getUploadElementValue(this);
	                return nativeMethods.inputValueGetter.call(this);
	            },
	            setter: function (value) {
	                if (this.type.toLowerCase() === 'file')
	                    return windowSandbox.uploadSandbox.setUploadElementValue(this, value);
	                nativeMethods.inputValueSetter.call(this, value);
	                var valueChanged = value !== nativeMethods.inputValueGetter.call(this);
	                if (valueChanged && !isShadowUIElement(this) && isTextEditableElementAndEditingAllowed(this))
	                    windowSandbox.elementEditingWatcher.restartWatchingElementEditing(this);
	            }
	        });
	        // NOTE: HTMLInputElement raises the `change` event on `disabled` only in Chrome
	        if (isChrome) {
	            overrideDescriptor(window.HTMLInputElement.prototype, 'disabled', {
	                getter: null,
	                setter: function (value) {
	                    if (nativeMethods.documentActiveElementGetter.call(document) === this) {
	                        var savedValue = windowSandbox.elementEditingWatcher.getElementSavedValue(this);
	                        var currentValue = nativeMethods.inputValueGetter.call(this);
	                        if (windowSandbox.elementEditingWatcher.isEditingObserved(this) && currentValue !== savedValue)
	                            windowSandbox.eventSimulator.change(this);
	                        windowSandbox.elementEditingWatcher.stopWatching(this);
	                    }
	                    nativeMethods.inputDisabledSetter.call(this, value);
	                }
	            });
	        }
	        overrideDescriptor(window.HTMLInputElement.prototype, 'required', {
	            getter: function () {
	                return windowSandbox.nodeSandbox.element.getAttributeCore(this, ['required']) !== null;
	            },
	            setter: function (value) {
	                if (this.type.toLowerCase() !== 'file')
	                    nativeMethods.inputRequiredSetter.call(this, value);
	                else if (value)
	                    windowSandbox.nodeSandbox.element.setAttributeCore(this, ['required', '']);
	                else
	                    windowSandbox.nodeSandbox.element.removeAttributeCore(this, ['required']);
	            }
	        });
	        overrideDescriptor(window.HTMLTextAreaElement.prototype, 'value', {
	            getter: null,
	            setter: function (value) {
	                nativeMethods.textAreaValueSetter.call(this, value);
	                if (!isShadowUIElement(this) && isTextEditableElementAndEditingAllowed(this))
	                    windowSandbox.elementEditingWatcher.restartWatchingElementEditing(this);
	            }
	        });
	        this._overrideUrlAttrDescriptors('data', [window.HTMLObjectElement]);
	        this._overrideUrlAttrDescriptors('src', [
	            window.HTMLImageElement,
	            window.HTMLScriptElement,
	            window.HTMLEmbedElement,
	            window.HTMLSourceElement,
	            window.HTMLMediaElement,
	            window.HTMLInputElement,
	            window.HTMLFrameElement,
	            window.HTMLIFrameElement
	        ]);
	        this._overrideUrlAttrDescriptors('action', [window.HTMLFormElement]);
	        this._overrideUrlAttrDescriptors('formAction', [
	            window.HTMLInputElement,
	            window.HTMLButtonElement
	        ]);
	        this._overrideUrlAttrDescriptors('href', [
	            window.HTMLAnchorElement,
	            window.HTMLLinkElement,
	            window.HTMLAreaElement,
	            window.HTMLBaseElement
	        ]);
	        if (nativeMethods.htmlManifestGetter)
	            this._overrideUrlAttrDescriptors('manifest', [window.HTMLHtmlElement]);
	        this._overrideAttrDescriptors('target', [
	            window.HTMLAnchorElement,
	            window.HTMLFormElement,
	            window.HTMLAreaElement,
	            window.HTMLBaseElement
	        ]);
	        this._overrideAttrDescriptors('formTarget', [
	            window.HTMLInputElement,
	            window.HTMLButtonElement
	        ]);
	        this._overrideAttrDescriptors('autocomplete', [window.HTMLInputElement]);
	        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
	        if (nativeMethods.scriptIntegrityGetter && nativeMethods.linkIntegrityGetter) {
	            this._overrideAttrDescriptors('integrity', [window.HTMLScriptElement]);
	            this._overrideAttrDescriptors('integrity', [window.HTMLLinkElement]);
	        }
	        this._overrideAttrDescriptors('rel', [window.HTMLLinkElement]);
	        if (nativeMethods.linkAsSetter)
	            this._overrideAttrDescriptors('as', [window.HTMLLinkElement]);
	        overrideDescriptor(window.HTMLInputElement.prototype, 'type', {
	            getter: null,
	            setter: function (value) {
	                windowSandbox.nodeSandbox.element.setAttributeCore(this, ['type', value]);
	            }
	        });
	        overrideDescriptor(window.HTMLIFrameElement.prototype, 'sandbox', {
	            getter: function () {
	                var domTokenList = this[SANDBOX_DOM_TOKEN_LIST];
	                if (!domTokenList) {
	                    var span_1 = nativeMethods.createElement.call(document, 'span');
	                    domTokenList = nativeMethods.elementClassListGetter.call(span_1);
	                    span_1.className = windowSandbox.nodeSandbox.element.getAttributeCore(this, ['sandbox']) || '';
	                    nativeMethods.objectDefineProperty(domTokenList, SANDBOX_DOM_TOKEN_LIST_OWNER, { value: this });
	                    nativeMethods.objectDefineProperty(this, SANDBOX_DOM_TOKEN_LIST, { value: domTokenList });
	                    nativeMethods.objectDefineProperty(this, SANDBOX_DOM_TOKEN_LIST_UPDATE_FN, {
	                        value: function (value) {
	                            span_1.className = value;
	                        }
	                    });
	                }
	                return domTokenList;
	            },
	            setter: function (value) {
	                windowSandbox.nodeSandbox.element.setAttributeCore(this, ['sandbox', value]);
	                if (this[SANDBOX_DOM_TOKEN_LIST_UPDATE_FN])
	                    this[SANDBOX_DOM_TOKEN_LIST_UPDATE_FN](windowSandbox.nodeSandbox.element.getAttributeCore(this, ['sandbox']) || '');
	            }
	        });
	        if (nativeMethods.iframeSrcdocGetter) {
	            overrideDescriptor(window.HTMLIFrameElement.prototype, 'srcdoc', {
	                getter: function () {
	                    return windowSandbox.nodeSandbox.element.getAttributeCore(this, ['srcdoc']) || '';
	                },
	                setter: function (value) {
	                    windowSandbox.nodeSandbox.element.setAttributeCore(this, ['srcdoc', value]);
	                }
	            });
	        }
	        this._overrideUrlPropDescriptor('port', nativeMethods.anchorPortGetter, nativeMethods.anchorPortSetter);
	        this._overrideUrlPropDescriptor('host', nativeMethods.anchorHostGetter, nativeMethods.anchorHostSetter);
	        this._overrideUrlPropDescriptor('hostname', nativeMethods.anchorHostnameGetter, nativeMethods.anchorHostnameSetter);
	        this._overrideUrlPropDescriptor('pathname', nativeMethods.anchorPathnameGetter, nativeMethods.anchorPathnameSetter);
	        this._overrideUrlPropDescriptor('protocol', nativeMethods.anchorProtocolGetter, nativeMethods.anchorProtocolSetter);
	        this._overrideUrlPropDescriptor('search', nativeMethods.anchorSearchGetter, nativeMethods.anchorSearchSetter);
	        overrideDescriptor(window.SVGImageElement.prototype, 'href', {
	            getter: function () {
	                var imageHref = nativeMethods.svgImageHrefGetter.call(this);
	                if (!imageHref[CONTEXT_SVG_IMAGE_ELEMENT]) {
	                    nativeMethods.objectDefineProperty(imageHref, CONTEXT_SVG_IMAGE_ELEMENT, {
	                        value: this,
	                        configurable: true
	                    });
	                }
	                return imageHref;
	            }
	        });
	        overrideDescriptor(window.SVGAnimatedString.prototype, 'baseVal', {
	            getter: function () {
	                var baseVal = nativeMethods.svgAnimStrBaseValGetter.call(this);
	                if (this[CONTEXT_SVG_IMAGE_ELEMENT])
	                    baseVal = getDestinationUrl(baseVal);
	                return baseVal;
	            },
	            setter: function (value) {
	                var contextSVGImageElement = this[CONTEXT_SVG_IMAGE_ELEMENT];
	                if (contextSVGImageElement) {
	                    var hasXlinkHrefAttr = nativeMethods.hasAttributeNS.call(contextSVGImageElement, XLINK_NAMESPACE, 'href');
	                    windowSandbox.nodeSandbox.element.setAttributeCore(contextSVGImageElement, [hasXlinkHrefAttr ? 'xlink:href' : 'href', value]);
	                    value = getProxyUrl(value);
	                }
	                nativeMethods.svgAnimStrBaseValSetter.call(this, value);
	            }
	        });
	        overrideDescriptor(window.SVGAnimatedString.prototype, 'animVal', {
	            getter: function () {
	                var animVal = nativeMethods.svgAnimStrAnimValGetter.call(this);
	                if (this[CONTEXT_SVG_IMAGE_ELEMENT])
	                    return getDestinationUrl(animVal);
	                return animVal;
	            }
	        });
	        if (nativeMethods.anchorOriginGetter) {
	            overrideDescriptor(window.HTMLAnchorElement.prototype, 'origin', {
	                getter: function () {
	                    return getAnchorProperty(this, nativeMethods.anchorOriginGetter);
	                }
	            });
	        }
	        overrideDescriptor(window.StyleSheet.prototype, 'href', {
	            getter: function () {
	                return getDestinationUrl(nativeMethods.styleSheetHrefGetter.call(this));
	            }
	        });
	        if (nativeMethods.cssStyleSheetHrefGetter) {
	            overrideDescriptor(window.CSSStyleSheet.prototype, 'href', {
	                getter: function () {
	                    return getDestinationUrl(nativeMethods.cssStyleSheetHrefGetter.call(this));
	                }
	            });
	        }
	        if (nativeMethods.nodeBaseURIGetter) {
	            overrideDescriptor(window.Node.prototype, 'baseURI', {
	                getter: function () {
	                    return getDestinationUrl(nativeMethods.nodeBaseURIGetter.call(this));
	                }
	            });
	        }
	        if (window.DOMParser) {
	            overrideFunction(window.DOMParser.prototype, 'parseFromString', function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var str = args[0];
	                var type = args[1];
	                var processedHtml;
	                if (args.length > 1 && typeof str === 'string' && type === 'text/html') {
	                    processedHtml = processHtml(str);
	                    args[0] = processedHtml;
	                }
	                var document = nativeMethods.DOMParserParseFromString.apply(this, args);
	                if (processedHtml)
	                    ShadowUI.removeSelfRemovingScripts(document);
	                return document;
	            });
	        }
	        overrideDescriptor(window.Node.prototype, 'firstChild', {
	            getter: function () {
	                if (ShadowUI.isShadowContainer(this))
	                    return windowSandbox.shadowUI.getFirstChild(this);
	                return nativeMethods.nodeFirstChildGetter.call(this);
	            }
	        });
	        overrideDescriptor(window.Element.prototype, 'firstElementChild', {
	            getter: function () {
	                if (ShadowUI.isShadowContainer(this))
	                    return windowSandbox.shadowUI.getFirstElementChild(this);
	                return nativeMethods.elementFirstElementChildGetter.call(this);
	            }
	        });
	        overrideDescriptor(window.Node.prototype, 'lastChild', {
	            getter: function () {
	                if (ShadowUI.isShadowContainer(this))
	                    return windowSandbox.shadowUI.getLastChild(this);
	                return nativeMethods.nodeLastChildGetter.call(this);
	            }
	        });
	        overrideDescriptor(window.Element.prototype, 'lastElementChild', {
	            getter: function () {
	                if (ShadowUI.isShadowContainer(this))
	                    return windowSandbox.shadowUI.getLastElementChild(this);
	                return nativeMethods.elementLastElementChildGetter.call(this);
	            }
	        });
	        overrideDescriptor(window.Node.prototype, 'nextSibling', {
	            getter: function () {
	                return windowSandbox.shadowUI.getNextSibling(this);
	            }
	        });
	        overrideDescriptor(window.Node.prototype, 'previousSibling', {
	            getter: function () {
	                return windowSandbox.shadowUI.getPrevSibling(this);
	            }
	        });
	        overrideDescriptor(window.Element.prototype, 'nextElementSibling', {
	            getter: function () {
	                return windowSandbox.shadowUI.getNextElementSibling(this);
	            }
	        });
	        overrideDescriptor(window.Element.prototype, 'previousElementSibling', {
	            getter: function () {
	                return windowSandbox.shadowUI.getPrevElementSibling(this);
	            }
	        });
	        overrideDescriptor(window[nativeMethods.elementHTMLPropOwnerName].prototype, 'innerHTML', {
	            getter: function () {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this))
	                    return windowSandbox._documentTitleStorageInitializer.storage.getTitleElementPropertyValue(this);
	                var innerHTML = nativeMethods.elementInnerHTMLGetter.call(this);
	                if (isScriptElement(this))
	                    return remove$1(innerHTML);
	                else if (isStyleElement(this))
	                    return styleProcessor.cleanUp(innerHTML, parseProxyUrl);
	                return cleanUpHtml(innerHTML);
	            },
	            setter: function (value) {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this)) {
	                    windowSandbox._documentTitleStorageInitializer.storage.setTitleElementPropertyValue(this, value);
	                    return;
	                }
	                var el = this;
	                var isStyleEl = isStyleElement(el);
	                var isScriptEl = isScriptElement(el);
	                var processedValue = value !== null && value !== void 0 ? String(value) : value;
	                if (processedValue) {
	                    if (isStyleEl)
	                        processedValue = styleProcessor.process(processedValue, getProxyUrl, true);
	                    else if (isScriptEl)
	                        processedValue = processScript(processedValue, true, false, convertToProxyUrl);
	                    else {
	                        processedValue = processHtml(processedValue, {
	                            parentTag: el.tagName,
	                            processedContext: el[INTERNAL_PROPS.processedContext]
	                        });
	                    }
	                }
	                if (!isStyleEl && !isScriptEl)
	                    domMutationTracker.onChildrenChanged(el);
	                nativeMethods.elementInnerHTMLSetter.call(el, processedValue);
	                windowSandbox._setSandboxedTextForTitleElements(el);
	                if (isStyleEl || isScriptEl)
	                    return;
	                domMutationTracker.onChildrenChanged(el);
	                if (windowSandbox.document.body === el) {
	                    var shadowUIRoot = windowSandbox.shadowUI.getRoot();
	                    windowSandbox.shadowUI.markShadowUIContainers(windowSandbox.document.head, el);
	                    ShadowUI.markElementAndChildrenAsShadow(shadowUIRoot);
	                }
	                else if (isShadowUIElement(el))
	                    ShadowUI.markElementAndChildrenAsShadow(el);
	                var parentDocument = findDocument(el);
	                var parentWindow = parentDocument ? parentDocument.defaultView : null;
	                // NOTE: For the iframe with an empty src.
	                if (parentWindow && parentWindow !== window &&
	                    parentWindow[INTERNAL_PROPS.processDomMethodName])
	                    parentWindow[INTERNAL_PROPS.processDomMethodName](el, parentDocument);
	                else if (window[INTERNAL_PROPS.processDomMethodName])
	                    window[INTERNAL_PROPS.processDomMethodName](el);
	                // NOTE: Fix for B239138 - unroll.me 'Cannot read property 'document' of null' error raised
	                // during recording. There was an issue when the document.body was replaced, so we need to
	                // reattach UI to a new body manually.
	                // NOTE: This check is required because jQuery calls the set innerHTML method for an element
	                // in an unavailable window.
	                if (window.self) {
	                    // NOTE: Use timeout, so that changes take effect.
	                    if (isHtmlElement(el) || isBodyElement(el))
	                        nativeMethods.setTimeout.call(window, function () { return windowSandbox.nodeMutation.onBodyContentChanged(el); }, 0);
	                }
	            }
	        });
	        overrideDescriptor(window[nativeMethods.elementHTMLPropOwnerName].prototype, 'outerHTML', {
	            getter: function () {
	                var outerHTML = nativeMethods.elementOuterHTMLGetter.call(this);
	                return cleanUpHtml(outerHTML);
	            },
	            setter: function (value) {
	                var el = this;
	                var parentEl = nativeMethods.nodeParentNodeGetter.call(el);
	                domMutationTracker.onElementChanged(el);
	                if (parentEl && value !== null && value !== void 0) {
	                    var parentDocument = findDocument(parentEl);
	                    var parentWindow = parentDocument ? parentDocument.defaultView : null;
	                    nativeMethods.elementOuterHTMLSetter.call(el, processHtml(String(value), {
	                        parentTag: parentEl && parentEl['tagName'],
	                        processedContext: el[INTERNAL_PROPS.processedContext]
	                    }));
	                    windowSandbox._setSandboxedTextForTitleElements(parentEl);
	                    domMutationTracker.onChildrenChanged(parentEl);
	                    // NOTE: For the iframe with an empty src.
	                    if (parentWindow && parentWindow !== window &&
	                        parentWindow[INTERNAL_PROPS.processDomMethodName])
	                        parentWindow[INTERNAL_PROPS.processDomMethodName](parentEl, parentDocument);
	                    else if (window[INTERNAL_PROPS.processDomMethodName])
	                        window[INTERNAL_PROPS.processDomMethodName](parentEl);
	                    // NOTE: This check is required for an element in an unavailable window.
	                    // NOTE: Use timeout, so that changes take effect.
	                    if (window.self && isBodyElement(el))
	                        nativeMethods.setTimeout.call(window, function () { return windowSandbox.shadowUI.onBodyElementMutation(); }, 0);
	                }
	                else
	                    nativeMethods.elementOuterHTMLSetter.call(el, value);
	            }
	        });
	        overrideDescriptor(window.HTMLElement.prototype, 'innerText', {
	            getter: function () {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this))
	                    return windowSandbox._documentTitleStorageInitializer.storage.getTitleElementPropertyValue(this);
	                var textContent = nativeMethods.htmlElementInnerTextGetter.call(this);
	                return WindowSandbox._removeProcessingInstructions(textContent);
	            },
	            setter: function (value) {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this)) {
	                    windowSandbox._documentTitleStorageInitializer.storage.setTitleElementPropertyValue(this, value);
	                    return;
	                }
	                var processedValue = WindowSandbox._processTextPropValue(this, value);
	                domMutationTracker.onChildrenChanged(this);
	                nativeMethods.htmlElementInnerTextSetter.call(this, processedValue);
	            }
	        });
	        overrideDescriptor(window.HTMLScriptElement.prototype, 'text', {
	            getter: function () {
	                var text = nativeMethods.scriptTextGetter.call(this);
	                return remove$1(text);
	            },
	            setter: function (value) {
	                var processedValue = value ? processScript(String(value), true, false, convertToProxyUrl) : value;
	                nativeMethods.scriptTextSetter.call(this, processedValue);
	            }
	        });
	        overrideDescriptor(window.HTMLAnchorElement.prototype, 'text', {
	            getter: function () {
	                var textContent = nativeMethods.anchorTextGetter.call(this);
	                return WindowSandbox._removeProcessingInstructions(textContent);
	            },
	            setter: function (value) {
	                var processedValue = WindowSandbox._processTextPropValue(this, value);
	                domMutationTracker.onChildrenChanged(this);
	                nativeMethods.anchorTextSetter.call(this, processedValue);
	            }
	        });
	        overrideDescriptor(window.Node.prototype, 'textContent', {
	            getter: function () {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this))
	                    return windowSandbox._documentTitleStorageInitializer.storage.getTitleElementPropertyValue(this);
	                var textContent = nativeMethods.nodeTextContentGetter.call(this);
	                return WindowSandbox._removeProcessingInstructions(textContent);
	            },
	            setter: function (value) {
	                if (windowSandbox._documentTitleStorageInitializer && isTitleElement(this)) {
	                    windowSandbox._documentTitleStorageInitializer.storage.setTitleElementPropertyValue(this, value);
	                    return;
	                }
	                var processedValue = WindowSandbox._processTextPropValue(this, value);
	                domMutationTracker.onChildrenChanged(this);
	                nativeMethods.nodeTextContentSetter.call(this, processedValue);
	            }
	        });
	        overrideDescriptor(window[nativeMethods.elementAttributesPropOwnerName].prototype, 'attributes', {
	            getter: function () {
	                return getAttributes(this);
	            }
	        });
	        overrideFunction(window.DOMTokenList.prototype, 'add', this._createOverriddenDOMTokenListMethod(nativeMethods.tokenListAdd));
	        overrideFunction(window.DOMTokenList.prototype, 'remove', this._createOverriddenDOMTokenListMethod(nativeMethods.tokenListRemove));
	        overrideFunction(window.DOMTokenList.prototype, 'toggle', this._createOverriddenDOMTokenListMethod(nativeMethods.tokenListToggle));
	        if (nativeMethods.tokenListReplace)
	            overrideFunction(window.DOMTokenList.prototype, 'replace', this._createOverriddenDOMTokenListMethod(nativeMethods.tokenListReplace));
	        if (nativeMethods.tokenListSupports) {
	            overrideFunction(window.DOMTokenList.prototype, 'supports', function () {
	                if (this[SANDBOX_DOM_TOKEN_LIST_OWNER]) {
	                    var nativeTokenList = nativeMethods.iframeSandboxGetter.call(this[SANDBOX_DOM_TOKEN_LIST_OWNER]);
	                    return nativeMethods.tokenListSupports.apply(nativeTokenList, arguments);
	                }
	                return nativeMethods.tokenListSupports.apply(this, arguments);
	            });
	        }
	        if (nativeMethods.tokenListValueSetter) {
	            overrideDescriptor(window.DOMTokenList.prototype, 'value', {
	                getter: null,
	                setter: function (value) {
	                    var tokenListOwner = this[SANDBOX_DOM_TOKEN_LIST_OWNER];
	                    nativeMethods.tokenListValueSetter.call(this, value);
	                    if (tokenListOwner)
	                        // eslint-disable-next-line no-restricted-properties
	                        windowSandbox.nodeSandbox.element.setAttributeCore(tokenListOwner, ['sandbox', this.value]);
	                }
	            });
	        }
	        overrideFunction(window.DOMImplementation.prototype, 'createHTMLDocument', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var doc = nativeMethods.createHTMLDocument.apply(this, args);
	            urlResolver.init(doc);
	            return doc;
	        });
	        overrideDescriptor(window.MutationRecord.prototype, 'nextSibling', {
	            getter: function () {
	                var originNextSibling = nativeMethods.mutationRecordNextSiblingGetter.call(this);
	                return windowSandbox.shadowUI.getMutationRecordNextSibling(originNextSibling);
	            }
	        });
	        overrideDescriptor(window.MutationRecord.prototype, 'previousSibling', {
	            getter: function () {
	                var originPrevSibling = nativeMethods.mutationRecordPrevSiblingGetter.call(this);
	                return windowSandbox.shadowUI.getMutationRecordPrevSibling(originPrevSibling);
	            }
	        });
	        if (nativeMethods.windowOriginGetter) {
	            overrideDescriptor(window, 'origin', {
	                getter: function () {
	                    var proxyOrigin = nativeMethods.windowOriginGetter.call(this);
	                    if (!proxyOrigin || proxyOrigin === 'null')
	                        return proxyOrigin;
	                    var frame = getFrameElement(windowSandbox.window);
	                    if (frame) {
	                        var sandbox = windowSandbox.nodeSandbox.element.getAttributeCore(frame, ['sandbox']);
	                        if (typeof sandbox === 'string' && sandbox.indexOf('allow-same-origin') === -1)
	                            return 'null';
	                    }
	                    var parsedDestLocation = getParsed();
	                    // eslint-disable-next-line no-restricted-properties
	                    if (parsedDestLocation && parsedDestLocation.protocol === 'file:')
	                        return 'null';
	                    return getOriginHeader();
	                },
	                setter: function (value) {
	                    return nativeMethods.windowOriginSetter.call(this, value);
	                }
	            });
	        }
	        if (this._documentTitleStorageInitializer) {
	            overrideDescriptor(window.HTMLTitleElement.prototype, 'text', {
	                getter: function () {
	                    return windowSandbox._documentTitleStorageInitializer.storage.getTitleElementPropertyValue(this);
	                },
	                setter: function (value) {
	                    windowSandbox._documentTitleStorageInitializer.storage.setTitleElementPropertyValue(this, value);
	                }
	            });
	        }
	    };
	    return WindowSandbox;
	}(SandboxBase));

	function noop () {
	    // NOTE: empty function
	}

	var PropertyAccessorsInstrumentation = /** @class */ (function (_super) {
	    __extends(PropertyAccessorsInstrumentation, _super);
	    function PropertyAccessorsInstrumentation() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    // NOTE: Isolate throw statements into a separate function because the
	    // JS engine doesn't optimize such functions.
	    PropertyAccessorsInstrumentation._error = function (msg) {
	        throw new Error(msg);
	    };
	    PropertyAccessorsInstrumentation._safeIsShadowUIElement = function (owner, propName) {
	        var el = owner[propName];
	        if (!el || !ShadowUI.isShadowContainerCollection(owner))
	            return false;
	        try {
	            return !WindowSandbox.isProxyObject(el) && isShadowUIElement(el);
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    PropertyAccessorsInstrumentation._setCrossDomainLocation = function (location, value) {
	        var proxyUrl = '';
	        if (typeof value !== 'string')
	            value = String(value);
	        if (!DomProcessor.isJsProtocol(value)) {
	            var resourceType = stringifyResourceType({ isIframe: true });
	            value = prepareUrl(value);
	            proxyUrl = location !== window.top.location
	                ? getProxyUrl(value, { resourceType: resourceType })
	                : getProxyUrl(value, { proxyPort: settings.get().crossDomainProxyPort });
	        }
	        else
	            proxyUrl = DomProcessor.processJsAttrValue(value, { isJsProtocol: true, isEventAttr: false });
	        location.href = proxyUrl; // eslint-disable-line no-restricted-properties
	        return value;
	    };
	    PropertyAccessorsInstrumentation.prototype._createPropertyAccessors = function () {
	        return {
	            href: {
	                condition: isLocation,
	                // eslint-disable-next-line no-restricted-properties
	                get: function (crossDomainLocation) { return crossDomainLocation.href; },
	                set: PropertyAccessorsInstrumentation._setCrossDomainLocation
	            },
	            location: {
	                condition: function (owner) { return isDocument(owner) || isWindow(owner); },
	                get: function (owner) {
	                    var locationWrapper = LocationAccessorsInstrumentation.getLocationWrapper(owner);
	                    if (locationWrapper)
	                        return locationWrapper;
	                    else if (!owner.location)
	                        return owner.location;
	                    var wnd = isWindow(owner) ? owner : owner.defaultView;
	                    return new LocationWrapper(wnd, null, noop);
	                },
	                set: function (owner, location) {
	                    //@ts-ignore
	                    var ownerWindow = isWindow(owner) ? owner : owner.defaultView;
	                    var locationWrapper = LocationAccessorsInstrumentation.getLocationWrapper(ownerWindow);
	                    if (!locationWrapper || locationWrapper === owner.location ||
	                        isIE && isCrossDomainWindows(window, ownerWindow))
	                        PropertyAccessorsInstrumentation._setCrossDomainLocation(owner.location, location);
	                    else if (locationWrapper)
	                        locationWrapper.href = location; // eslint-disable-line no-restricted-properties
	                    return location;
	                }
	            }
	        };
	    };
	    PropertyAccessorsInstrumentation._getSetPropertyInstructionByOwner = function (owner, window) {
	        try {
	            return owner && owner[INTERNAL_PROPS.processedContext] &&
	                owner[INTERNAL_PROPS.processedContext] !== window &&
	                owner[INTERNAL_PROPS.processedContext][SCRIPT_PROCESSING_INSTRUCTIONS.setProperty];
	        }
	        catch (e) {
	            return null;
	        }
	    };
	    PropertyAccessorsInstrumentation.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        var accessors = this._createPropertyAccessors();
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.getProperty, {
	            value: function (owner, propName) {
	                if (isNullOrUndefined(owner))
	                    PropertyAccessorsInstrumentation._error("Cannot read property '" + propName + "' of " + inaccessibleTypeToStr(owner));
	                if (WindowSandbox.isProxyObject(owner))
	                    return owner[propName];
	                if (typeof propName === 'string' && shouldInstrumentProperty(propName) &&
	                    accessors[propName].condition(owner))
	                    return accessors[propName].get(owner);
	                if (PropertyAccessorsInstrumentation._safeIsShadowUIElement(owner, propName))
	                    return void 0;
	                return owner[propName];
	            },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.setProperty, {
	            value: function (owner, propName, value) {
	                if (isNullOrUndefined(owner))
	                    PropertyAccessorsInstrumentation._error("Cannot set property '" + propName + "' of " + inaccessibleTypeToStr(owner));
	                if (WindowSandbox.isProxyObject(owner))
	                    return owner[propName] = value; // eslint-disable-line no-return-assign
	                var ownerSetPropertyInstruction = PropertyAccessorsInstrumentation._getSetPropertyInstructionByOwner(owner, window);
	                if (ownerSetPropertyInstruction)
	                    return ownerSetPropertyInstruction(owner, propName, value);
	                if (typeof propName === 'string' && shouldInstrumentProperty(propName) &&
	                    accessors[propName].condition(owner))
	                    return accessors[propName].set(owner, value);
	                return owner[propName] = value; // eslint-disable-line no-return-assign
	            },
	            configurable: true
	        });
	        return accessors;
	    };
	    return PropertyAccessorsInstrumentation;
	}(SandboxBase));

	// OPTIMIZATION: http://jsperf.com/call-apply-optimization
	function fastApply(owner, methName, args) {
	    var meth = owner[methName];
	    switch (args.length) {
	        case 1:
	            return meth.call(owner, args[0]);
	        case 2:
	            return meth.call(owner, args[0], args[1]);
	        case 3:
	            return meth.call(owner, args[0], args[1], args[2]);
	        case 4:
	            return meth.call(owner, args[0], args[1], args[2], args[3]);
	        case 5:
	            return meth.call(owner, args[0], args[1], args[2], args[3], args[4]);
	        default:
	            return meth.apply(owner, args);
	    }
	}

	var MethodCallInstrumentation = /** @class */ (function (_super) {
	    __extends(MethodCallInstrumentation, _super);
	    function MethodCallInstrumentation(_messageSandbox) {
	        var _this = _super.call(this) || this;
	        _this._messageSandbox = _messageSandbox;
	        _this.methodWrappers = {
	            postMessage: {
	                condition: isWindow,
	                method: function (contentWindow, args) { return _messageSandbox.postMessage(contentWindow, args); }
	            },
	            // NOTE: We cannot get the location wrapper for a cross-domain window. Therefore, we need to
	            // intercept calls to the native 'replace' method.
	            replace: {
	                condition: isLocation,
	                method: function (location, args) { return location.replace(getProxyUrl(args[0], {
	                    resourceType: MethodCallInstrumentation._getLocationResourceType(location)
	                })); }
	            },
	            // NOTE: We cannot get the location wrapper for a cross-domain window. Therefore, we need to
	            // intercept calls to the native 'assign' method.
	            assign: {
	                condition: isLocation,
	                method: function (location, args) { return location.assign(getProxyUrl(args[0], {
	                    resourceType: MethodCallInstrumentation._getLocationResourceType(location)
	                })); }
	            }
	        };
	        return _this;
	    }
	    // NOTE: Isolate throw statement into a separate function because JS engine doesn't optimize such functions.
	    MethodCallInstrumentation._error = function (msg) {
	        throw new Error(msg);
	    };
	    MethodCallInstrumentation._getLocationResourceType = function (location) {
	        return window.top.location === location ? null : stringifyResourceType({ isIframe: true });
	    };
	    MethodCallInstrumentation._isPostMessageFn = function (win, fn) {
	        // NOTE: in iOS Safari 9.3 win.postMessage === win.postMessage equals false
	        if (win.postMessage === win.postMessage)
	            return win.postMessage === fn;
	        return fn && typeof fn.toString === 'function' && fn.toString() === win.postMessage.toString();
	    };
	    MethodCallInstrumentation.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.callMethod, {
	            value: function (owner, methName, args) {
	                if (isNullOrUndefined(owner))
	                    MethodCallInstrumentation._error("Cannot call method '" + methName + "' of " + inaccessibleTypeToStr(owner));
	                if (typeof owner[methName] !== 'function')
	                    MethodCallInstrumentation._error("'" + methName + "' is not a function");
	                // OPTIMIZATION: previously we've performed the
	                // `this.methodWrappers.hasOwnProperty(methName)`
	                // check which is quite slow. Now we use the
	                // fast RegExp check instead.
	                if (typeof methName === 'string' && shouldInstrumentMethod(methName) &&
	                    _this.methodWrappers[methName].condition(owner))
	                    return _this.methodWrappers[methName].method(owner, args);
	                return fastApply(owner, methName, args);
	            },
	            configurable: true
	        });
	        var methodCallInstrumentation = this;
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.getPostMessage, {
	            value: function (win, postMessageFn) {
	                if (arguments.length === 1 && !isWindow(win)) {
	                    // @ts-ignore
	                    return win.postMessage;
	                }
	                if (arguments.length === 2 && !MethodCallInstrumentation._isPostMessageFn(this, postMessageFn))
	                    return postMessageFn;
	                return function () {
	                    var args = [];
	                    for (var _i = 0; _i < arguments.length; _i++) {
	                        args[_i] = arguments[_i];
	                    }
	                    //@ts-ignore
	                    return methodCallInstrumentation._messageSandbox.postMessage(this, args);
	                };
	            },
	            configurable: true
	        });
	    };
	    return MethodCallInstrumentation;
	}(SandboxBase));

	var CodeInstrumentation = /** @class */ (function (_super) {
	    __extends(CodeInstrumentation, _super);
	    function CodeInstrumentation(eventSandbox, messageSandbox) {
	        var _this = _super.call(this) || this;
	        _this._methodCallInstrumentation = new MethodCallInstrumentation(eventSandbox.message);
	        _this._locationAccessorsInstrumentation = new LocationAccessorsInstrumentation(messageSandbox);
	        _this._propertyAccessorsInstrumentation = new PropertyAccessorsInstrumentation();
	        return _this;
	    }
	    CodeInstrumentation.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._methodCallInstrumentation.attach(window);
	        this._locationAccessorsInstrumentation.attach(window);
	        this.elementPropertyAccessors = this._propertyAccessorsInstrumentation.attach(window);
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        // NOTE: GH-260
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.getEval, {
	            value: function (evalFn) {
	                if (evalFn !== window.eval)
	                    return evalFn;
	                var evalWrapper = function (script) {
	                    if (typeof script === 'string')
	                        script = processScript(script);
	                    return evalFn(script);
	                };
	                nativeMethods.objectDefineProperty(evalWrapper, CodeInstrumentation.WRAPPED_EVAL_FN, { value: evalFn });
	                return evalWrapper;
	            },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.processScript, {
	            value: function (script, isApply) {
	                if (isApply) {
	                    if (script && script.length && typeof script[0] === 'string') {
	                        var args = [processScript(script[0], false)];
	                        // NOTE: shallow-copy the remaining args. Don't use arr.slice(),
	                        // since it may leak the arguments object.
	                        // See: https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Functions/arguments
	                        for (var i = 1; i < script.length; i++)
	                            args.push(script[i]);
	                        return args;
	                    }
	                }
	                else if (typeof script === 'string')
	                    return processScript(script, false);
	                return script;
	            },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.processHtml, {
	            value: function (win, html) {
	                if (typeof html === 'string')
	                    html = processHtml("<html><body>" + html + "</body></html>", { processedContext: win });
	                return html;
	            },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.getProxyUrl, {
	            value: function (url, baseUrl) {
	                var storedBaseUrl = urlResolver.getBaseUrl(_this.document);
	                var shouldChangeBase = baseUrl && baseUrl !== storedBaseUrl;
	                if (shouldChangeBase)
	                    urlResolver.updateBase(baseUrl, _this.document);
	                var proxyUrl = getProxyUrl(url, { resourceType: stringifyResourceType({ isScript: true }) });
	                if (shouldChangeBase)
	                    urlResolver.updateBase(storedBaseUrl, _this.document);
	                return proxyUrl;
	            },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.restArray, {
	            value: function (array, startIndex) { return nativeMethods.arraySlice.call(array, startIndex); },
	            configurable: true
	        });
	        nativeMethods.objectDefineProperty(window, SCRIPT_PROCESSING_INSTRUCTIONS.restObject, {
	            value: function (obj, excludeProps) {
	                var rest = {};
	                var keys = nativeMethods.objectKeys(obj);
	                for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
	                    var key = keys_1[_i];
	                    if (excludeProps.indexOf(key) < 0)
	                        rest[key] = obj[key];
	                }
	                return rest;
	            },
	            configurable: true
	        });
	    };
	    CodeInstrumentation.WRAPPED_EVAL_FN = 'hammerhead|code-instrumentation|wrapped-eval-fn';
	    return CodeInstrumentation;
	}(SandboxBase));

	// -------------------------------------------------------------
	var TIME_RADIX = 36;
	var CLEAR_COOKIE_VALUE_STR = '=;path=/;expires=Thu, 01 Jan 1970 00:00:01 GMT';
	var CLIENT_COOKIE_SYNC_KEY_FRAGMENT_COUNT = 7;
	var KEY_VALUE_REGEX = /(?:^([^=]+)=([\s\S]*))?/;
	var SYNCHRONIZATION_TYPE = {
	    server: 's',
	    client: 'c',
	    window: 'w'
	};
	var SYNCHRONIZATION_TYPE_RE = new RegExp("^[" + SYNCHRONIZATION_TYPE.server + SYNCHRONIZATION_TYPE.client + SYNCHRONIZATION_TYPE.window + "]+");
	function isSameCookies(cookie1, cookie2) {
	    return cookie1.sid === cookie2.sid &&
	        cookie1.key === cookie2.key &&
	        cookie1.domain === cookie2.domain &&
	        cookie1.path === cookie2.path;
	}
	function sortByOutdatedAndActual(parsedCookies) {
	    var outdated = [];
	    var actual = [];
	    for (var current = 0; current < parsedCookies.length; current++) {
	        var other = current + 1;
	        for (; other < parsedCookies.length; other++) {
	            if (isSameCookies(parsedCookies[current], parsedCookies[other])) {
	                if (parsedCookies[current].lastAccessed > parsedCookies[other].lastAccessed) {
	                    var temp = parsedCookies[current];
	                    parsedCookies[current] = parsedCookies[other];
	                    parsedCookies[other] = temp;
	                }
	                outdated.push(parsedCookies[current]);
	                break;
	            }
	        }
	        if (other === parsedCookies.length)
	            actual.push(parsedCookies[current]);
	    }
	    return { outdated: outdated, actual: actual };
	}
	function stringifySyncType(cookie) {
	    return (cookie.isServerSync ? SYNCHRONIZATION_TYPE.server : '') +
	        (cookie.isClientSync ? SYNCHRONIZATION_TYPE.client : '') +
	        (cookie.isWindowSync ? SYNCHRONIZATION_TYPE.window : '');
	}
	function formatSyncCookieKey(cookie) {
	    var syncType = stringifySyncType(cookie);
	    var key = encodeURIComponent(cookie.key);
	    var domain = encodeURIComponent(cookie.domain);
	    var path = encodeURIComponent(cookie.path);
	    var expires = cookie.expires !== 'Infinity' ? cookie.expires.getTime().toString(TIME_RADIX) : '';
	    var lastAccessed = cookie.lastAccessed.getTime().toString(TIME_RADIX);
	    return syncType + "|" + cookie.sid + "|" + key + "|" + domain + "|" + path + "|" + expires + "|" + lastAccessed;
	}
	function parseClientSyncCookieStr(cookieStr) {
	    var cookies = cookieStr ? cookieStr.split(';') : '';
	    var parsedCookies = [];
	    for (var _i = 0, cookies_1 = cookies; _i < cookies_1.length; _i++) {
	        var cookie = cookies_1[_i];
	        var parsedCookie = parseSyncCookie(trim(cookie));
	        if (parsedCookie)
	            parsedCookies.push(parsedCookie);
	    }
	    return sortByOutdatedAndActual(parsedCookies);
	}
	function prepareSyncCookieProperties(cookie) {
	    cookie.syncKey = cookie.syncKey || formatSyncCookieKey(cookie);
	    cookie.cookieStr = cookie.cookieStr || cookie.syncKey + "=" + cookie.value;
	}
	function formatSyncCookie(cookie) {
	    if (cookie.cookieStr)
	        return cookie.cookieStr + ";path=/";
	    return formatSyncCookieKey(cookie) + "=" + cookie.value + ";path=/";
	}
	function parseSyncCookie(cookieStr) {
	    var _a = KEY_VALUE_REGEX.exec(cookieStr), key = _a[1], value = _a[2];
	    var parsedKey = key !== void 0 && value !== void 0 && key.split('|');
	    if (parsedKey && parsedKey.length !== CLIENT_COOKIE_SYNC_KEY_FRAGMENT_COUNT)
	        return null;
	    return {
	        isServerSync: parsedKey[0].indexOf(SYNCHRONIZATION_TYPE.server) > -1,
	        isClientSync: parsedKey[0].indexOf(SYNCHRONIZATION_TYPE.client) > -1,
	        isWindowSync: parsedKey[0].indexOf(SYNCHRONIZATION_TYPE.window) > -1,
	        sid: parsedKey[1],
	        key: decodeURIComponent(parsedKey[2]),
	        domain: decodeURIComponent(parsedKey[3]),
	        path: decodeURIComponent(parsedKey[4]),
	        expires: parsedKey[5] ? new Date(parseInt(parsedKey[5], TIME_RADIX)) : 'Infinity',
	        lastAccessed: new Date(parseInt(parsedKey[6], TIME_RADIX)),
	        syncKey: key,
	        value: value,
	        cookieStr: cookieStr
	    };
	}
	function changeSyncType(parsedCookie, flags) {
	    if ('server' in flags)
	        parsedCookie.isServerSync = flags.server;
	    if ('client' in flags)
	        parsedCookie.isClientSync = flags.client;
	    if ('window' in flags)
	        parsedCookie.isWindowSync = flags.window;
	    var newSyncTypeStr = stringifySyncType(parsedCookie);
	    parsedCookie.syncKey = parsedCookie.syncKey.replace(SYNCHRONIZATION_TYPE_RE, newSyncTypeStr);
	    parsedCookie.cookieStr = parsedCookie.cookieStr.replace(SYNCHRONIZATION_TYPE_RE, newSyncTypeStr);
	}
	function isOutdatedSyncCookie(currentCookie, newCookie) {
	    return newCookie.isServerSync === currentCookie.isServerSync &&
	        newCookie.sid === currentCookie.sid &&
	        newCookie.key === currentCookie.key &&
	        newCookie.domain === currentCookie.domain &&
	        newCookie.path === currentCookie.path &&
	        newCookie.lastAccessed > currentCookie.lastAccessed;
	}
	function generateDeleteSyncCookieStr(cookie) {
	    return cookie.syncKey + CLEAR_COOKIE_VALUE_STR;
	}

	var sharedCookieUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		SYNCHRONIZATION_TYPE: SYNCHRONIZATION_TYPE,
		parseClientSyncCookieStr: parseClientSyncCookieStr,
		prepareSyncCookieProperties: prepareSyncCookieProperties,
		formatSyncCookie: formatSyncCookie,
		parseSyncCookie: parseSyncCookie,
		changeSyncType: changeSyncType,
		isOutdatedSyncCookie: isOutdatedSyncCookie,
		generateDeleteSyncCookieStr: generateDeleteSyncCookieStr
	});

	var topOpenerWindow = null;
	function getTopOpenerWindow() {
	    if (!topOpenerWindow) {
	        topOpenerWindow = window.top;
	        while (topOpenerWindow.opener && topOpenerWindow !== topOpenerWindow.opener)
	            topOpenerWindow = topOpenerWindow.opener.top;
	    }
	    return topOpenerWindow;
	}

	var SYNC_COOKIE_START_CMD = 'hammerhead|command|sync-cookie-start';
	var SYNC_COOKIE_DONE_CMD = 'hammerhead|command|sync-cookie-done';
	var SYNC_MESSAGE_TIMEOUT = 500;
	var SYNC_MESSAGE_ATTEMPT_COUNT = 5;
	var WindowSync = /** @class */ (function () {
	    function WindowSync(_cookieSandbox, _messageSandbox, _childWindowSandbox) {
	        this._cookieSandbox = _cookieSandbox;
	        this._messageSandbox = _messageSandbox;
	        this._childWindowSandbox = _childWindowSandbox;
	        this._win = null;
	        this._messageIdGenerator = null;
	        this._resolversMap = new Map();
	    }
	    WindowSync._getCookieSandbox = function (win) {
	        try {
	            // eslint-disable-next-line no-restricted-properties
	            var cookieSandbox = win[INTERNAL_PROPS.hammerhead].sandbox.cookie;
	            return cookieSandbox.document && cookieSandbox;
	        }
	        catch (e) {
	            return null;
	        }
	    };
	    WindowSync.prototype._onMsgReceived = function (_a) {
	        var _this = this;
	        var message = _a.message, source = _a.source;
	        if (message.cmd === SYNC_COOKIE_START_CMD) {
	            this._cookieSandbox.syncWindowCookie(message.cookies);
	            if (this._win !== this._win.top)
	                this._messageSandbox.sendServiceMsg({ id: message.id, cmd: SYNC_COOKIE_DONE_CMD }, source);
	            else if (this._win !== getTopOpenerWindow()) {
	                this.syncBetweenWindows(message.cookies, source)
	                    .then(function () { return _this._messageSandbox.sendServiceMsg({
	                    id: message.id,
	                    cmd: SYNC_COOKIE_DONE_CMD
	                }, source); });
	            }
	            else
	                this.syncBetweenWindows(message.cookies, source);
	        }
	        else if (message.cmd === SYNC_COOKIE_DONE_CMD) {
	            var resolver = this._resolversMap.get(message.id);
	            if (resolver)
	                resolver();
	        }
	    };
	    WindowSync.prototype._getWindowsForSync = function (initiator, currentWindow, windows) {
	        if (windows === void 0) { windows = []; }
	        if (currentWindow !== initiator && currentWindow !== this._win.top)
	            windows.push(currentWindow);
	        // @ts-ignore
	        for (var _i = 0, _a = currentWindow.frames; _i < _a.length; _i++) {
	            var frameWindow = _a[_i];
	            this._getWindowsForSync(initiator, frameWindow, windows);
	        }
	        return windows;
	    };
	    WindowSync.prototype._sendSyncMessage = function (win, cmd, cookies) {
	        var _this = this;
	        var id = this._messageIdGenerator.increment();
	        var attempts = 0;
	        return new pinkie(function (resolve) {
	            var timeoutId = null;
	            var resolveWrapper = function () {
	                nativeMethods.clearTimeout.call(_this._win, timeoutId);
	                _this._resolversMap.delete(id);
	                resolve();
	            };
	            var sendMsg = function () {
	                // NOTE: The window was removed if the parent property is null.
	                if (attempts++ < SYNC_MESSAGE_ATTEMPT_COUNT || !win.parent) {
	                    _this._messageSandbox.sendServiceMsg({ id: id, cmd: cmd, cookies: cookies }, win);
	                    timeoutId = nativeMethods.setTimeout.call(_this._win, sendMsg, SYNC_MESSAGE_TIMEOUT * attempts);
	                }
	                else
	                    resolveWrapper();
	            };
	            _this._resolversMap.set(id, resolveWrapper);
	            sendMsg();
	        });
	    };
	    WindowSync.prototype._delegateSyncBetweenWindowsToMainTopWindow = function (cookies) {
	        var topOpenerWindow = getTopOpenerWindow();
	        var cookieSandboxTop = WindowSync._getCookieSandbox(topOpenerWindow);
	        if (cookieSandboxTop) {
	            cookieSandboxTop.syncWindowCookie(cookies);
	            cookieSandboxTop.getWindowSync().syncBetweenWindows(cookies, this._win);
	        }
	        else
	            this._messageSandbox.sendServiceMsg({ cmd: SYNC_COOKIE_START_CMD, cookies: cookies }, topOpenerWindow);
	    };
	    WindowSync.prototype._removeSyncCookie = function (cookies) {
	        var doc = this._win.document;
	        var clientCookieStr = cookies[0].isClientSync && nativeMethods.documentCookieGetter.call(doc);
	        for (var _i = 0, cookies_1 = cookies; _i < cookies_1.length; _i++) {
	            var parsedCookie_1 = cookies_1[_i];
	            nativeMethods.documentCookieSetter.call(doc, generateDeleteSyncCookieStr(parsedCookie_1));
	        }
	        // NOTE: client cookie is passed one at a time
	        var parsedCookie = cookies[0];
	        if (clientCookieStr && CookieSandbox.isSyncCookieExists(parsedCookie, clientCookieStr)) {
	            changeSyncType(parsedCookie, { window: false });
	            nativeMethods.documentCookieSetter.call(doc, formatSyncCookie(parsedCookie));
	        }
	    };
	    WindowSync.prototype.syncBetweenWindows = function (cookies, initiator) {
	        var _this = this;
	        var topOpenerWindow = getTopOpenerWindow();
	        if (this._win !== this._win.top || this._win !== topOpenerWindow && !initiator) {
	            this._delegateSyncBetweenWindowsToMainTopWindow(cookies);
	            return pinkie.resolve();
	        }
	        var windowsForSync = this._getWindowsForSync(initiator, this._win);
	        var syncMessages = [];
	        if (this._win === topOpenerWindow) {
	            for (var _i = 0, _a = this._childWindowSandbox.getChildWindows(); _i < _a.length; _i++) {
	                var win = _a[_i];
	                var cookieSandbox = WindowSync._getCookieSandbox(win);
	                if (cookieSandbox)
	                    syncMessages.push(cookieSandbox.getWindowSync().syncBetweenWindows(cookies, this._win));
	                else
	                    syncMessages.push(this._sendSyncMessage(win, SYNC_COOKIE_START_CMD, cookies));
	            }
	        }
	        for (var _b = 0, windowsForSync_1 = windowsForSync; _b < windowsForSync_1.length; _b++) {
	            var win = windowsForSync_1[_b];
	            var cookieSandbox = WindowSync._getCookieSandbox(win);
	            if (cookieSandbox)
	                cookieSandbox.syncWindowCookie(cookies);
	            else
	                syncMessages.push(this._sendSyncMessage(win, SYNC_COOKIE_START_CMD, cookies));
	        }
	        if (syncMessages.length) {
	            var syncMessagesPromise = pinkie.all(syncMessages);
	            if (this._win === topOpenerWindow)
	                return syncMessagesPromise.then(function () { return _this._removeSyncCookie(cookies); });
	            return syncMessagesPromise.then();
	        }
	        else {
	            this._removeSyncCookie(cookies);
	            return pinkie.resolve();
	        }
	    };
	    WindowSync.prototype.attach = function (win) {
	        var _this = this;
	        this._win = win;
	        this._messageSandbox.on(this._messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (e) { return _this._onMsgReceived(e); });
	        if (win === win.top) {
	            this._messageIdGenerator = this._messageIdGenerator || new IntegerIdGenerator();
	            this._resolversMap = this._resolversMap || new Map();
	        }
	    };
	    return WindowSync;
	}());

	var COOKIE_PAIR_REGEX = /^((?:=)?([^=;]*)\s*=\s*)?([^\n\r\0]*)/;
	var TRAILING_SEMICOLON_REGEX = /;+$/;
	var FIX_COOKIE_DATE = /((?:\s|,)[0-9]{1,2})(?:\s|-)([A-Za-z]{3})(?:\s|-)([0-9]{4}\s)/;
	function parse(str) {
	    str = trim(str);
	    var trailingSemicolonCheck = TRAILING_SEMICOLON_REGEX.exec(str);
	    if (trailingSemicolonCheck)
	        str = str.slice(0, trailingSemicolonCheck.index);
	    var firstSemicolonIdx = str.indexOf(';');
	    var keyValueString = firstSemicolonIdx > -1 ? str.substr(0, firstSemicolonIdx) : str;
	    var keyValueParsingResult = COOKIE_PAIR_REGEX.exec(keyValueString);
	    if (!keyValueParsingResult)
	        return null;
	    var parsedCookie = {
	        key: keyValueParsingResult[1] ? trim(keyValueParsingResult[2]) : '',
	        value: trim(keyValueParsingResult[3])
	    };
	    if (firstSemicolonIdx === -1)
	        return parsedCookie;
	    var attributesString = trim(str.slice(firstSemicolonIdx).replace(/^\s*;\s*/, ''));
	    if (attributesString.length === 0)
	        return parsedCookie;
	    var attrValStrings = attributesString.split(/\s*;\s*/);
	    while (attrValStrings.length) {
	        var attrValueStr = attrValStrings.shift();
	        var separatorIdx = attrValueStr.indexOf('=');
	        var key = null;
	        var value = null;
	        var date = null;
	        if (separatorIdx === -1)
	            key = attrValueStr;
	        else {
	            key = attrValueStr.substr(0, separatorIdx);
	            value = trim(attrValueStr.substr(separatorIdx + 1));
	        }
	        key = trim(key.toLowerCase());
	        switch (key) {
	            case 'expires':
	                value = value.replace(FIX_COOKIE_DATE, '$1 $2 $3');
	                date = getUTCDate(Date.parse(value));
	                if (date)
	                    parsedCookie.expires = date;
	                break;
	            case 'max-age':
	                parsedCookie.maxAge = value;
	                break;
	            case 'path':
	                parsedCookie.path = value;
	                break;
	            case 'secure':
	                parsedCookie.secure = true;
	                break;
	            case 'httponly':
	                parsedCookie.httpOnly = true;
	                break;
	            case 'domain':
	                // NOTE: Remove leading '.'.
	                parsedCookie.domain = trim(value.replace(/^\./, ''));
	                break;
	        }
	    }
	    return parsedCookie;
	}
	function formatClientString(parsedCookie) {
	    // eslint-disable-next-line no-restricted-properties
	    var cookieStr = parsedCookie.value || '';
	    if (parsedCookie.key !== '')
	        cookieStr = parsedCookie.key + '=' + cookieStr;
	    return cookieStr;
	}
	function setDefaultValues(parsedCookie, _a) {
	    var hostname = _a.hostname, pathname = _a.pathname;
	    if (!parsedCookie.domain)
	        parsedCookie.domain = hostname; // eslint-disable-line no-restricted-properties
	    if (!parsedCookie.path || parsedCookie.path.charAt(0) !== '/') {
	        var path = pathname; // eslint-disable-line no-restricted-properties
	        var defaultPath = path.slice(0, path.lastIndexOf('/'));
	        parsedCookie.path = defaultPath || '/';
	    }
	    if (!parsedCookie.expires)
	        parsedCookie.expires = 'Infinity';
	}
	function domainMatch(currentDomain, cookieDomain) {
	    if (!cookieDomain)
	        return true;
	    currentDomain = currentDomain.toLowerCase();
	    cookieDomain = cookieDomain.toLowerCase();
	    if (currentDomain === cookieDomain)
	        return true;
	    var cookieDomainIdx = currentDomain.indexOf(cookieDomain);
	    return cookieDomainIdx > 0 &&
	        currentDomain.length === cookieDomain.length + cookieDomainIdx &&
	        currentDomain.charAt(cookieDomainIdx - 1) === '.';
	}
	function pathMatch(currentPath, cookiePath) {
	    if (!cookiePath || cookiePath.charAt(0) !== '/' || currentPath === cookiePath)
	        return true;
	    return currentPath.length > cookiePath.length && currentPath.indexOf(cookiePath) === 0 &&
	        (cookiePath.charAt(cookiePath.length - 1) === '/' || currentPath.charAt(cookiePath.length) === '/');
	}
	function getUTCDate(timestamp) {
	    if (!arguments.length)
	        timestamp = nativeMethods.dateNow();
	    else if (isNaN(timestamp))
	        return null;
	    // NOTE: remove milliseconds
	    timestamp = Math.floor(timestamp / 1000) * 1000;
	    return new nativeMethods.date(timestamp);
	}

	var cookieUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		parse: parse,
		formatClientString: formatClientString,
		setDefaultValues: setDefaultValues,
		domainMatch: domainMatch,
		pathMatch: pathMatch,
		getUTCDate: getUTCDate
	});

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	// NOTE: At least 4096 bytes per cookie (as measured by the sum of the length of the cookie's name,
	// value, and attributes). Specification https://tools.ietf.org/html/rfc6265#page-27 (GH-767)
	var BYTES_PER_COOKIE_LIMIT = 4096;

	var MIN_DATE_VALUE = new nativeMethods.date(0).toUTCString();
	var CookieSandbox = /** @class */ (function (_super) {
	    __extends(CookieSandbox, _super);
	    function CookieSandbox(messageSandbox, _unloadSandbox, childWindowSandbox) {
	        var _this = _super.call(this) || this;
	        _this._unloadSandbox = _unloadSandbox;
	        _this._windowSync = new WindowSync(_this, messageSandbox, childWindowSandbox);
	        return _this;
	    }
	    CookieSandbox._removeAllSyncCookie = function () {
	        var cookies = nativeMethods.documentCookieGetter.call(document);
	        var parsedCookies = parseClientSyncCookieStr(cookies);
	        var sessionId = settings.get().sessionId;
	        for (var _i = 0, _a = parsedCookies.outdated; _i < _a.length; _i++) {
	            var outdatedCookie = _a[_i];
	            nativeMethods.documentCookieSetter.call(document, generateDeleteSyncCookieStr(outdatedCookie));
	        }
	        for (var _b = 0, _c = parsedCookies.actual; _b < _c.length; _b++) {
	            var parsedCookie = _c[_b];
	            if (parsedCookie.sid === sessionId && (parsedCookie.isWindowSync || parsedCookie.isServerSync)) {
	                nativeMethods.documentCookieSetter.call(document, generateDeleteSyncCookieStr(parsedCookie));
	                if (parsedCookie.isClientSync) {
	                    changeSyncType(parsedCookie, { window: false });
	                    nativeMethods.documentCookieSetter.call(document, formatSyncCookie(parsedCookie));
	                }
	            }
	        }
	    };
	    CookieSandbox.prototype._canSetCookie = function (cookie, setByClient) {
	        // eslint-disable-next-line no-restricted-properties
	        if (setByClient && (cookie.length > BYTES_PER_COOKIE_LIMIT || getParsed().protocol === 'file:'))
	            return false;
	        var clientCookie = "key" + nativeMethods.mathRandom.call(nativeMethods.math) + "=value";
	        nativeMethods.documentCookieSetter.call(this.document, clientCookie);
	        var documentCookieIsEmpty = !nativeMethods.documentCookieGetter.call(this.document);
	        if (!documentCookieIsEmpty)
	            nativeMethods.documentCookieSetter.call(this.document, clientCookie + ";expires=" + MIN_DATE_VALUE);
	        return !documentCookieIsEmpty;
	    };
	    CookieSandbox._updateClientCookieStr = function (cookieKey, newCookieStr) {
	        var cookieStr = settings.get().cookie; // eslint-disable-line no-restricted-properties
	        var cookies = cookieStr ? cookieStr.split(';') : [];
	        var changedCookies = [];
	        var replaced = false;
	        var searchStr = cookieKey === '' ? null : cookieKey + '=';
	        // NOTE: Replace a cookie if it already exists.
	        for (var _i = 0, cookies_1 = cookies; _i < cookies_1.length; _i++) {
	            var cookie = cookies_1[_i];
	            cookie = trim(cookie);
	            var isCookieExists = searchStr ? cookie.indexOf(searchStr) === 0 : cookie.indexOf('=') === -1;
	            if (!isCookieExists)
	                changedCookies.push(cookie);
	            else if (newCookieStr !== null) {
	                changedCookies.push(newCookieStr);
	                replaced = true;
	            }
	        }
	        if (!replaced && newCookieStr !== null)
	            changedCookies.push(newCookieStr);
	        settings.get().cookie = changedCookies.join('; '); // eslint-disable-line no-restricted-properties
	    };
	    CookieSandbox.prototype.getCookie = function () {
	        this.syncCookie();
	        // eslint-disable-next-line no-restricted-properties
	        return settings.get().cookie || '';
	    };
	    CookieSandbox.prototype.setCookie = function (cookie) {
	        var setByClient = typeof cookie === 'string';
	        // NOTE: Cookie cannot be set in iframe without src in IE
	        // Also cookie cannot be set on a page with 'file:' protocol
	        // or if the length of cookie higher than limit
	        if (!this._canSetCookie(cookie, setByClient))
	            return;
	        var parsedCookie = setByClient ? parse(cookie) : cookie;
	        if (!parsedCookie || parsedCookie.httpOnly)
	            return;
	        var parsedDestLocation = getParsed();
	        // NOTE: All Hammerhad sessions have the same domain, so we need to validate the Domain attribute manually
	        // according to a test url.
	        // eslint-disable-next-line no-restricted-properties
	        if (!domainMatch(parsedDestLocation.hostname, parsedCookie.domain))
	            return;
	        // eslint-disable-next-line no-restricted-properties
	        if ((!parsedCookie.secure || parsedDestLocation.protocol === 'https:') &&
	            // eslint-disable-next-line no-restricted-properties
	            pathMatch(parsedDestLocation.pathname, parsedCookie.path)) {
	            var currentDate = getUTCDate();
	            var clientCookieStr = null;
	            if (!parsedCookie.expires || parsedCookie.expires === 'Infinity' || parsedCookie.expires > currentDate)
	                clientCookieStr = formatClientString(parsedCookie);
	            CookieSandbox._updateClientCookieStr(parsedCookie.key, clientCookieStr);
	        }
	        if (setByClient) {
	            setDefaultValues(parsedCookie, parsedDestLocation);
	            this._syncClientCookie(parsedCookie);
	            this.syncCookie();
	        }
	    };
	    CookieSandbox.prototype.syncCookie = function () {
	        var cookies = nativeMethods.documentCookieGetter.call(this.document);
	        var parsedCookies = parseClientSyncCookieStr(cookies);
	        var sessionId = settings.get().sessionId;
	        var serverSyncCookies = [];
	        for (var _i = 0, _a = parsedCookies.outdated; _i < _a.length; _i++) {
	            var outdatedCookie = _a[_i];
	            nativeMethods.documentCookieSetter.call(this.document, generateDeleteSyncCookieStr(outdatedCookie));
	        }
	        for (var _b = 0, _c = parsedCookies.actual; _b < _c.length; _b++) {
	            var parsedCookie = _c[_b];
	            if (parsedCookie.sid !== sessionId)
	                continue;
	            if (parsedCookie.isServerSync)
	                serverSyncCookies.push(parsedCookie);
	            else if (parsedCookie.isWindowSync)
	                this.setCookie(parsedCookie);
	        }
	        if (serverSyncCookies.length)
	            this._syncServerCookie(serverSyncCookies);
	    };
	    CookieSandbox.prototype._syncServerCookie = function (parsedCookies) {
	        for (var _i = 0, parsedCookies_1 = parsedCookies; _i < parsedCookies_1.length; _i++) {
	            var parsedCookie = parsedCookies_1[_i];
	            this.setCookie(parsedCookie);
	            nativeMethods.documentCookieSetter.call(this.document, generateDeleteSyncCookieStr(parsedCookie));
	            changeSyncType(parsedCookie, { server: false, window: true });
	            nativeMethods.documentCookieSetter.call(this.document, formatSyncCookie(parsedCookie));
	        }
	        this._windowSync.syncBetweenWindows(parsedCookies);
	    };
	    CookieSandbox.prototype._syncClientCookie = function (parsedCookie) {
	        parsedCookie.isClientSync = true;
	        parsedCookie.isWindowSync = true;
	        parsedCookie.sid = settings.get().sessionId;
	        parsedCookie.lastAccessed = new nativeMethods.date();
	        prepareSyncCookieProperties(parsedCookie);
	        nativeMethods.documentCookieSetter.call(this.document, formatSyncCookie(parsedCookie));
	        this._windowSync.syncBetweenWindows([parsedCookie]);
	    };
	    CookieSandbox.isSyncCookieExists = function (parsedCookie, clientCookieStr) {
	        var startIndex = clientCookieStr.indexOf(parsedCookie.cookieStr);
	        var endIndex = startIndex + parsedCookie.cookieStr.length;
	        return startIndex > -1 && (clientCookieStr.length === endIndex || clientCookieStr.charAt(endIndex) === ';');
	    };
	    CookieSandbox.prototype.syncWindowCookie = function (parsedCookies) {
	        var clientCookie = nativeMethods.documentCookieGetter.call(this.document);
	        for (var _i = 0, parsedCookies_2 = parsedCookies; _i < parsedCookies_2.length; _i++) {
	            var parsedCookie = parsedCookies_2[_i];
	            if (CookieSandbox.isSyncCookieExists(parsedCookie, clientCookie))
	                this.setCookie(parsedCookie);
	        }
	    };
	    CookieSandbox.prototype.getWindowSync = function () {
	        return this._windowSync;
	    };
	    CookieSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        this._windowSync.attach(window);
	        if (window === getTopOpenerWindow())
	            this._unloadSandbox.on(this._unloadSandbox.UNLOAD_EVENT, CookieSandbox._removeAllSyncCookie);
	    };
	    return CookieSandbox;
	}(SandboxBase));

	var EDITING_OBSERVED_FLAG = 'hammerhead|editing-observed';
	var PREVIOUS_VALUE_PROPERTY = 'hammerhead|previous-value';
	var ElementEditingWatcher = /** @class */ (function () {
	    function ElementEditingWatcher(_eventSimulator) {
	        this._eventSimulator = _eventSimulator;
	    }
	    ElementEditingWatcher.prototype._onBlur = function (e) {
	        if (!this.processElementChanging(e.target))
	            this.stopWatching(e.target);
	    };
	    ElementEditingWatcher.prototype._onChange = function (e) {
	        this.stopWatching(e.target);
	    };
	    ElementEditingWatcher._getValue = function (el) {
	        if (isInputElement(el))
	            return nativeMethods.inputValueGetter.call(el);
	        else if (isTextAreaElement(el))
	            return nativeMethods.textAreaValueGetter.call(el);
	        // eslint-disable-next-line no-restricted-properties
	        return el.value;
	    };
	    ElementEditingWatcher.prototype.stopWatching = function (el) {
	        var _this = this;
	        if (!el)
	            return;
	        nativeMethods.removeEventListener.call(el, 'blur', function (e) { return _this._onBlur(e); });
	        nativeMethods.removeEventListener.call(el, 'change', function (e) { return _this._onChange(e); });
	        if (el[EDITING_OBSERVED_FLAG] !== void 0)
	            delete el[EDITING_OBSERVED_FLAG];
	        if (el[PREVIOUS_VALUE_PROPERTY] !== void 0)
	            delete el[PREVIOUS_VALUE_PROPERTY];
	    };
	    ElementEditingWatcher.prototype.watchElementEditing = function (el) {
	        var _a;
	        var _this = this;
	        if (!el || el[EDITING_OBSERVED_FLAG] || !isTextEditableElementAndEditingAllowed(el) || isShadowUIElement(el))
	            return;
	        nativeMethods.objectDefineProperties(el, (_a = {},
	            _a[EDITING_OBSERVED_FLAG] = { value: true, configurable: true, writable: true },
	            _a[PREVIOUS_VALUE_PROPERTY] = { value: ElementEditingWatcher._getValue(el), configurable: true, writable: true },
	            _a));
	        nativeMethods.addEventListener.call(el, 'blur', function (e) { return _this._onBlur(e); });
	        nativeMethods.addEventListener.call(el, 'change', function (e) { return _this._onChange(e); });
	    };
	    ElementEditingWatcher.prototype.restartWatchingElementEditing = function (el) {
	        if (el && el[EDITING_OBSERVED_FLAG])
	            el[PREVIOUS_VALUE_PROPERTY] = ElementEditingWatcher._getValue(el);
	    };
	    ElementEditingWatcher.prototype.processElementChanging = function (el) {
	        if (el && el[EDITING_OBSERVED_FLAG] && ElementEditingWatcher._getValue(el) !== el[PREVIOUS_VALUE_PROPERTY]) {
	            this._eventSimulator.change(el);
	            this.restartWatchingElementEditing(el);
	            return true;
	        }
	        return false;
	    };
	    ElementEditingWatcher.prototype.getElementSavedValue = function (el) {
	        return el[PREVIOUS_VALUE_PROPERTY];
	    };
	    ElementEditingWatcher.prototype.isEditingObserved = function (el) {
	        return el[EDITING_OBSERVED_FLAG];
	    };
	    return ElementEditingWatcher;
	}());

	var WINDOW_ACTIVATED_EVENT = 'hammerhead|event|window-activated';
	var WINDOW_DEACTIVATED_EVENT = 'hammerhead|event|window-deactivated';
	var ActiveWindowTracker = /** @class */ (function (_super) {
	    __extends(ActiveWindowTracker, _super);
	    function ActiveWindowTracker(_messageSandbox) {
	        var _this = _super.call(this) || this;
	        _this._messageSandbox = _messageSandbox;
	        _this._isIframeWindow = false;
	        _this._activeWindow = null;
	        _this._isActive = false;
	        return _this;
	    }
	    ActiveWindowTracker.prototype._notifyPrevActiveWindow = function () {
	        try {
	            if (this._activeWindow.top && this._activeWindow !== this._activeWindow.top) {
	                this._messageSandbox.sendServiceMsg({
	                    cmd: WINDOW_DEACTIVATED_EVENT
	                }, this._activeWindow);
	            }
	        }
	        catch (err) {
	            // NOTE: The error appears in IE when the corresponding iframe is removed.
	        }
	    };
	    ActiveWindowTracker.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._isIframeWindow = isIframeWindow(window);
	        this._activeWindow = !this._isIframeWindow ? window.top : null;
	        this._isActive = !this._isIframeWindow;
	        this._messageSandbox.on(this._messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (e) {
	            if (e.message.cmd === WINDOW_ACTIVATED_EVENT) {
	                _this._notifyPrevActiveWindow();
	                _this._isActive = false;
	                _this._activeWindow = e.source;
	            }
	            else if (e.message.cmd === WINDOW_DEACTIVATED_EVENT)
	                _this._isActive = false;
	        });
	    };
	    ActiveWindowTracker.prototype.isCurrentWindowActive = function () {
	        return this._isActive;
	    };
	    ActiveWindowTracker.prototype.makeCurrentWindowActive = function () {
	        this._isActive = true;
	        if (!this._isIframeWindow) {
	            this._notifyPrevActiveWindow();
	            this._activeWindow = this.window;
	        }
	        else {
	            this._messageSandbox.sendServiceMsg({
	                cmd: WINDOW_ACTIVATED_EVENT
	            }, this.window.top);
	        }
	    };
	    return ActiveWindowTracker;
	}(SandboxBase));

	var INTERNAL_FOCUS_BLUR_FLAG_PREFIX = 'hammerhead|event|internal-';
	var PREVENT_FOCUS_ON_CHANGE = isChrome;
	var eventsMap = {
	    bubbles: {
	        'focus': 'focusin',
	        'blur': 'focusout'
	    },
	    nonBubbles: {
	        'focusin': 'focus',
	        'focusout': 'blur'
	    }
	};
	var FocusBlurSandbox = /** @class */ (function (_super) {
	    __extends(FocusBlurSandbox, _super);
	    function FocusBlurSandbox(_listeners, _eventSimulator, messageSandbox, _timersSandbox, elementEditingWatcher) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this._eventSimulator = _eventSimulator;
	        _this._timersSandbox = _timersSandbox;
	        _this._topWindow = null;
	        _this._lastFocusedElement = null;
	        _this._scrollState = {};
	        _this._activeWindowTracker = new ActiveWindowTracker(messageSandbox);
	        _this._elementEditingWatcher = elementEditingWatcher;
	        return _this;
	    }
	    FocusBlurSandbox._getNativeMeth = function (el, event) {
	        if (isSVGElement(el)) {
	            if (event === 'focus')
	                return nativeMethods.svgFocus;
	            else if (event === 'blur')
	                return nativeMethods.svgBlur;
	        }
	        //@ts-ignore
	        return nativeMethods[event];
	    };
	    FocusBlurSandbox._restoreElementScroll = function (el, scroll) {
	        var newScroll = getElementScroll(el);
	        if (newScroll.left !== scroll.left)
	            setScrollLeft(el, scroll.left);
	        if (newScroll.top !== scroll.top)
	            setScrollTop(el, scroll.top);
	    };
	    FocusBlurSandbox.prototype._onChangeActiveElement = function (activeElement) {
	        if (this._lastFocusedElement === activeElement)
	            return;
	        if (this._lastFocusedElement &&
	            nativeMethods.getAttribute.call(this._lastFocusedElement, INTERNAL_ATTRIBUTES.focusPseudoClass))
	            nativeMethods.removeAttribute.call(this._lastFocusedElement, INTERNAL_ATTRIBUTES.focusPseudoClass);
	        if (isElementFocusable(activeElement) && !(isBodyElement(activeElement) &&
	            getTabIndex(activeElement) === null)) {
	            this._lastFocusedElement = activeElement;
	            nativeMethods.setAttribute.call(activeElement, INTERNAL_ATTRIBUTES.focusPseudoClass, true);
	        }
	        else
	            this._lastFocusedElement = null;
	    };
	    FocusBlurSandbox.prototype._shouldUseLabelHtmlForElement = function (el, type) {
	        return type === 'focus' && !!el.htmlFor && isLabelElement(el) && !isElementFocusable(el);
	    };
	    FocusBlurSandbox.prototype._getElementNonScrollableParentsScrollState = function (el) {
	        var scrollState = [];
	        var elementParents = getParents(el);
	        for (var _i = 0, elementParents_1 = elementParents; _i < elementParents_1.length; _i++) {
	            var elementParent = elementParents_1[_i];
	            if (get$1(elementParent, 'overflow') === 'hidden') {
	                scrollState.push({
	                    element: elementParent,
	                    state: getElementScroll(elementParent)
	                });
	            }
	        }
	        return scrollState;
	    };
	    FocusBlurSandbox.prototype._restoreElementNonScrollableParentsScrollState = function (scrollState) {
	        for (var _i = 0, scrollState_1 = scrollState; _i < scrollState_1.length; _i++) {
	            var scrollStateEntry = scrollState_1[_i];
	            FocusBlurSandbox._restoreElementScroll(scrollStateEntry.element, scrollStateEntry.state);
	        }
	    };
	    FocusBlurSandbox.prototype._saveScrollStateIfNecessary = function (el, preventScrolling) {
	        if (preventScrolling)
	            this._scrollState.windowScroll = getElementScroll(this.window);
	        if (isIE)
	            this._scrollState.elementNonScrollableParentsScrollState = this._getElementNonScrollableParentsScrollState(el);
	    };
	    FocusBlurSandbox.prototype._restoreScrollStateIfNecessary = function (preventScrolling) {
	        if (preventScrolling)
	            FocusBlurSandbox._restoreElementScroll(this.window, this._scrollState.windowScroll);
	        if (isIE)
	            this._restoreElementNonScrollableParentsScrollState(this._scrollState.elementNonScrollableParentsScrollState);
	    };
	    FocusBlurSandbox.prototype._raiseEvent = function (el, type, callback, _a) {
	        // NOTE: We cannot use Promise because 'resolve' will be called async, but we need to resolve
	        // immediately in IE9 and IE10.
	        var _this = this;
	        var withoutHandlers = _a.withoutHandlers, isAsync = _a.isAsync, forMouseEvent = _a.forMouseEvent, preventScrolling = _a.preventScrolling, relatedTarget = _a.relatedTarget, focusedOnChange = _a.focusedOnChange;
	        var simulateEvent = function () {
	            // NOTE: The focus and blur events should be raised after activeElement is changed (B237489)
	            // in MSEdge, the focus/blur events are executed  synchronously.
	            if (isIE && version < 12) {
	                _this.window.setTimeout(function () {
	                    _this.window.setTimeout(function () {
	                        delete el[FocusBlurSandbox.getInternalEventFlag(type)];
	                    }, 0);
	                }, 0);
	            }
	            else
	                delete el[FocusBlurSandbox.getInternalEventFlag(type)];
	            if (!withoutHandlers) {
	                var bubblesEventType = eventsMap.bubbles[type];
	                var bubblesEventShouldRaiseFirstly = isIE11 || isMSEdge && version < 17 && type === 'blur';
	                if (isAsync) {
	                    // NOTE: focusin, focusout events are synchronously
	                    _this._eventSimulator[bubblesEventType](el, relatedTarget);
	                    _this._timersSandbox.deferFunction(function () { return _this._eventSimulator[type](el, relatedTarget); });
	                }
	                else if (bubblesEventShouldRaiseFirstly) {
	                    _this._eventSimulator[bubblesEventType](el, relatedTarget);
	                    _this._eventSimulator[type](el, relatedTarget);
	                }
	                else {
	                    _this._eventSimulator[type](el, relatedTarget);
	                    _this._eventSimulator[bubblesEventType](el, relatedTarget);
	                }
	            }
	            else if (type === 'focus' && PREVENT_FOCUS_ON_CHANGE) {
	                var preventFocus = function (_e, _dispatched, _preventEvent, cancelHandlers, stopEventPropagation) {
	                    cancelHandlers();
	                    stopEventPropagation();
	                };
	                _this._listeners.addInternalEventBeforeListener(window, ['focus'], preventFocus);
	                _this._eventSimulator['focus'](el, relatedTarget);
	                _this._listeners.removeInternalEventBeforeListener(window, ['focus'], preventFocus);
	            }
	            callback();
	        };
	        if (el[type]) {
	            // NOTE: To guarantee that all focus/blur events are raised, we need to raise them manually.
	            this._saveScrollStateIfNecessary(el, preventScrolling);
	            if (this._shouldUseLabelHtmlForElement(el, type)) {
	                var htmlForElement = nativeMethods.getElementById.call(findDocument(el), el.htmlFor);
	                if (htmlForElement)
	                    el = htmlForElement;
	                else {
	                    callback();
	                    return;
	                }
	            }
	            el[FocusBlurSandbox.getInternalEventFlag(type)] = true;
	            // NOTE: We should guarantee that activeElement will be changed, therefore we need to call the native
	            // focus/blur event.
	            if (!focusedOnChange)
	                FocusBlurSandbox._getNativeMeth(el, type).call(el);
	            this._restoreScrollStateIfNecessary(preventScrolling);
	            var curDocument = findDocument(el);
	            var activeElement = getActiveElement(curDocument);
	            // NOTE: If the element was not focused and has a parent with tabindex, we focus this parent.
	            var parent_1 = nativeMethods.nodeParentNodeGetter.call(el);
	            var parentWithTabIndex = parent_1 === document ? null : closest(parent_1, '[tabindex]');
	            if (type === 'focus' && activeElement !== el && parentWithTabIndex && forMouseEvent) {
	                // NOTE: In WebKit, Safari and MSEdge, calling the native focus event for a parent element
	                // raises page scrolling. We can't prevent it. Therefore, we need to restore a page scrolling value.
	                var needPreventScrolling = isWebKit || isSafari || isIE;
	                this._raiseEvent(parentWithTabIndex, 'focus', simulateEvent, {
	                    preventScrolling: needPreventScrolling,
	                    forMouseEvent: forMouseEvent
	                });
	            }
	            // NOTE: Some browsers don't change document.activeElement after calling element.blur() if a browser
	            // window is in the background. That's why we call body.focus() without handlers. It should be called
	            // synchronously because client scripts may expect that document.activeElement will be changed immediately
	            // after element.blur() is called.
	            else if (type === 'blur' && activeElement === el && el !== curDocument.body)
	                this._raiseEvent(curDocument.body, 'focus', simulateEvent, { withoutHandlers: true });
	            else if (!el.disabled)
	                simulateEvent();
	            else
	                callback();
	        }
	        else
	            simulateEvent();
	    };
	    FocusBlurSandbox.getInternalEventFlag = function (type) {
	        return INTERNAL_FOCUS_BLUR_FLAG_PREFIX + type;
	    };
	    FocusBlurSandbox.getNonBubblesEventType = function (bubblesEventType) {
	        return eventsMap.nonBubbles[bubblesEventType];
	    };
	    FocusBlurSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._activeWindowTracker.attach(window);
	        this._topWindow = isCrossDomainWindows(window, window.top) ? window : window.top;
	        this._listeners.addInternalEventBeforeListener(window, ['focus', 'blur'], function () {
	            var activeElement = getActiveElement(_this.document);
	            _this._onChangeActiveElement(activeElement);
	        });
	    };
	    FocusBlurSandbox.prototype._raiseSelectionChange = function (callback, el) {
	        // NOTE: In MSEdge, the 'selectionchange' event doesn't occur immediately (it occurs with a some delay)
	        // so we should raise it right after the 'focus' event is raised.
	        if (isMSEdge && el && isTextEditableElement(el))
	            this._eventSimulator.selectionchange(el);
	        if (typeof callback === 'function')
	            callback();
	    };
	    FocusBlurSandbox.prototype.focus = function (el, callback, silent, forMouseEvent, isNativeFocus, preventScrolling) {
	        var _this = this;
	        // NOTE: el.focus() does not raise the event if the element is invisible. If the element is located
	        // within an invisible iframe, all browsers except Chrome do not raise the event (GH-442)
	        var raiseEventInIframe = !isNativeFocus || isWebKit ||
	            !isElementInInvisibleIframe(el);
	        var elDocument = (el[INTERNAL_PROPS.processedContext] || this.window).document;
	        if (!raiseEventInIframe || isNativeFocus && !isElementVisible(el, elDocument))
	            return null;
	        var isElementInIframe$1 = isElementInIframe(el);
	        var iframeElement = isElementInIframe$1 ? getIframeByElement(el) : null;
	        var curDocument = findDocument(el);
	        var isBodyElement$1 = isBodyElement(el);
	        var activeElement = getActiveElement();
	        var activeElementDocument = findDocument(activeElement);
	        var withoutHandlers = false;
	        var needBlur = false;
	        var needBlurIframe = false;
	        var isContentEditable = isContentEditableElement(el);
	        var isCurrentWindowActive = this._activeWindowTracker.isCurrentWindowActive();
	        if (activeElement === el)
	            withoutHandlers = !(isBodyElement$1 && isContentEditable && !isCurrentWindowActive);
	        else
	            withoutHandlers = isBodyElement$1 && !(isContentEditable || isIE);
	        // NOTE: In IE, if you call focus() or blur() methods from script, an active element is changed immediately,
	        // but events are raised asynchronously after some timeout.
	        var isAsync = false;
	        var raiseFocusEvent = function () {
	            if (!isCurrentWindowActive && !isShadowUIElement(el))
	                _this._activeWindowTracker.makeCurrentWindowActive();
	            var raiseEventArgs = {
	                withoutHandlers: withoutHandlers || silent,
	                isAsync: isAsync,
	                forMouseEvent: forMouseEvent,
	                preventScrolling: preventScrolling,
	                relatedTarget: activeElement
	            };
	            _this._raiseEvent(el, 'focus', function () {
	                if (!silent)
	                    _this._elementEditingWatcher.watchElementEditing(el);
	                // NOTE: If we call focus for an unfocusable element (like 'div' or 'image') in iframe, we should
	                // specify document.active for this iframe manually, so we call focus without handlers.
	                if (isElementInIframe$1 && iframeElement &&
	                    getActiveElement(_this._topWindow.document) !== iframeElement)
	                    _this._raiseEvent(iframeElement, 'focus', function () { return _this._raiseSelectionChange(callback, el); }, { withoutHandlers: true, isAsync: isAsync });
	                else
	                    _this._raiseSelectionChange(callback, el);
	            }, raiseEventArgs);
	        };
	        if (isNativeFocus && isIE) {
	            // NOTE: In IE, the focus() method does not have any effect if it is called in the focus event handler
	            // during the  second event phase.
	            if ((this._eventSimulator.isSavedWindowsEventsExists() || version > 10) &&
	                this.window.event &&
	                this.window.event.type === 'focus' && this.window.event.srcElement === el) {
	                this._raiseSelectionChange(callback, el);
	                return null;
	            }
	            // NOTE: In MSEdge, the focus/blur events are executed synchronously.
	            if (version < 12)
	                isAsync = true;
	        }
	        if (activeElement && activeElement.tagName) {
	            if (activeElement !== el) {
	                // NOTE: B253685
	                if (curDocument !== activeElementDocument && activeElement === activeElementDocument.body)
	                    needBlur = false;
	                else if (activeElement === curDocument.body) {
	                    // NOTE: The Blur event is raised for the body only in IE. In addition, we must not call the
	                    // blur function for the body because this moves the browser window into the background.
	                    if (!silent && isIE) {
	                        if (isAsync)
	                            this._timersSandbox.setTimeout.call(this.window, function () { return _this._eventSimulator.blur(activeElement); }, 0);
	                        else
	                            this._eventSimulator.blur(activeElement);
	                    }
	                }
	                else if (!el.disabled)
	                    needBlur = true;
	            }
	            // NOTE: B254260
	            needBlurIframe = curDocument !== activeElementDocument &&
	                isElementInIframe(activeElement, activeElementDocument);
	        }
	        // NOTE: We always call blur for iframe manually without handlers (B254260).
	        if (needBlurIframe && !needBlur) {
	            if (isIE) {
	                // NOTE: We should call blur for iframe with handlers in IE but we can't call the method 'blur'
	                // because activeElement !== element and handlers will not be called.
	                this._eventSimulator.blur(getIframeByElement(activeElement));
	                raiseFocusEvent();
	            }
	            else
	                this.blur(getIframeByElement(activeElement), raiseFocusEvent, true, isNativeFocus);
	        }
	        else if (needBlur) {
	            this.blur(activeElement, function (focusOnChange) {
	                if (needBlurIframe)
	                    _this.blur(getIframeByElement(activeElement), raiseFocusEvent, true, isNativeFocus);
	                else if (!focusOnChange)
	                    raiseFocusEvent();
	                else if (typeof callback === 'function')
	                    callback();
	            }, silent, isNativeFocus, el);
	        }
	        else
	            raiseFocusEvent();
	        return null;
	    };
	    FocusBlurSandbox.prototype.blur = function (el, callback, withoutHandlers, isNativeBlur, relatedTarget) {
	        var curDocument = findDocument(el);
	        var activeElement = getActiveElement(curDocument);
	        // NOTE: In IE, if you call the focus() or blur() method from script, an active element is changed
	        // immediately but events are raised asynchronously after some timeout (in MSEdgethe focus/blur methods
	        // are executed synchronously).
	        var isAsync = isNativeBlur && isIE && version < 12;
	        if (activeElement !== el)
	            withoutHandlers = true;
	        var focusedOnChange = false;
	        if (!withoutHandlers) {
	            var focusOnChangeHandler = function (e) {
	                focusedOnChange = e.target === el;
	            };
	            if (PREVENT_FOCUS_ON_CHANGE)
	                this._listeners.addInternalEventBeforeListener(window, ['focus'], focusOnChangeHandler);
	            this._elementEditingWatcher.processElementChanging(el);
	            if (PREVENT_FOCUS_ON_CHANGE)
	                this._listeners.removeInternalEventBeforeListener(window, ['focus'], focusOnChangeHandler);
	            this._elementEditingWatcher.stopWatching(el);
	        }
	        var raiseEventParameters = {
	            withoutHandlers: withoutHandlers,
	            isAsync: isAsync,
	            relatedTarget: relatedTarget,
	            focusedOnChange: focusedOnChange
	        };
	        this._raiseEvent(el, 'blur', function () {
	            if (typeof callback === 'function')
	                callback(focusedOnChange);
	        }, raiseEventParameters);
	    };
	    FocusBlurSandbox.prototype.dispose = function () {
	        this._lastFocusedElement = null;
	    };
	    return FocusBlurSandbox;
	}(SandboxBase));

	var HoverSandbox = /** @class */ (function (_super) {
	    __extends(HoverSandbox, _super);
	    function HoverSandbox(_listeners) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this._hoverElementFixed = false;
	        _this._lastHoveredElement = null;
	        return _this;
	    }
	    HoverSandbox._setHoverMarker = function (newHoveredElement, jointParent) {
	        if (jointParent)
	            nativeMethods.setAttribute.call(jointParent, INTERNAL_ATTRIBUTES.hoverPseudoClass, '');
	        while (newHoveredElement && newHoveredElement.tagName) {
	            // NOTE: Assign a pseudo-class marker to the elements until the joint parent is found.
	            if (newHoveredElement !== jointParent) {
	                nativeMethods.setAttribute.call(newHoveredElement, INTERNAL_ATTRIBUTES.hoverPseudoClass, '');
	                var associatedElement = getAssociatedElement(newHoveredElement);
	                if (associatedElement)
	                    nativeMethods.setAttribute.call(associatedElement, INTERNAL_ATTRIBUTES.hoverPseudoClass, '');
	                newHoveredElement = nativeMethods.nodeParentNodeGetter.call(newHoveredElement);
	            }
	            else
	                break;
	        }
	    };
	    // NOTE: In this method, we go up to the tree of elements and look for a joint parent for the
	    // previous and new hovered elements. Processing is needed only until  that parent is found.
	    // In this case, we'll reduce the number of dom calls.
	    HoverSandbox.prototype._clearHoverMarkerUntilJointParent = function (newHoveredElement) {
	        var jointParent = null;
	        if (this._lastHoveredElement) {
	            var el = this._lastHoveredElement;
	            while (el && el.tagName && el.contains) {
	                var associatedElement = getAssociatedElement(el);
	                if (associatedElement)
	                    nativeMethods.removeAttribute.call(associatedElement, INTERNAL_ATTRIBUTES.hoverPseudoClass);
	                // NOTE: Check that the current element is a joint parent for the hovered elements.
	                if (!el.contains(newHoveredElement)) {
	                    nativeMethods.removeAttribute.call(el, INTERNAL_ATTRIBUTES.hoverPseudoClass);
	                    el = nativeMethods.nodeParentNodeGetter.call(el);
	                }
	                else {
	                    jointParent = el;
	                    break;
	                }
	            }
	            if (jointParent)
	                nativeMethods.removeAttribute.call(jointParent, INTERNAL_ATTRIBUTES.hoverPseudoClass);
	        }
	        return jointParent;
	    };
	    HoverSandbox.prototype._onHover = function (_a) {
	        var target = _a.target, clientX = _a.clientX, clientY = _a.clientY;
	        var hoverIsDisabled = isIE && shouldIgnoreMouseEventInsideIframe(target, clientX, clientY);
	        if (!hoverIsDisabled)
	            this._hover(target);
	    };
	    HoverSandbox.prototype._hover = function (el) {
	        if (!this._hoverElementFixed && !isShadowUIElement(el)) {
	            var jointParent = this._clearHoverMarkerUntilJointParent(el);
	            HoverSandbox._setHoverMarker(el, jointParent);
	            this._lastHoveredElement = el;
	        }
	    };
	    HoverSandbox.prototype.fixHoveredElement = function () {
	        this._hoverElementFixed = true;
	    };
	    HoverSandbox.prototype.freeHoveredElement = function () {
	        this._hoverElementFixed = false;
	    };
	    HoverSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._listeners.addInternalEventBeforeListener(window, ['mouseover', 'touchstart'], function (e) { return _this._onHover(e); });
	    };
	    HoverSandbox.prototype.dispose = function () {
	        this._lastHoveredElement = null;
	    };
	    return HoverSandbox;
	}(SandboxBase));

	// NOTE: For internal usage of Listeners.
	var ELEMENT_LISTENING_EVENTS_STORAGE_PROP = 'hammerhead|element-listening-events-storage-prop';
	function getElementCtx(el) {
	    return el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
	}
	function getEventCtx(el, event) {
	    event = isIE && version > 10 && /MSPointer/.test(event) ? event.replace('MS', '').toLowerCase() : event;
	    var elementCtx = getElementCtx(el);
	    return elementCtx && elementCtx[event];
	}
	function isElementListening(el) {
	    return !!el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
	}
	function addListeningElement(el, events) {
	    var elementCtx = getElementCtx(el) || {};
	    for (var i = 0; i < events.length; i++) {
	        if (!elementCtx[events[i]]) {
	            elementCtx[events[i]] = {
	                internalBeforeHandlers: [],
	                internalAfterHandlers: [],
	                outerHandlers: [],
	                outerHandlersWrapper: null,
	                wrappers: [],
	                cancelOuterHandlers: false
	            };
	        }
	    }
	    if (!isElementListening(el)) {
	        nativeMethods.objectDefineProperty(el, ELEMENT_LISTENING_EVENTS_STORAGE_PROP, {
	            value: elementCtx,
	            writable: true
	        });
	    }
	}
	function removeListeningElement(el) {
	    delete el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
	}
	function addInternalAfterHandler(el, events, handler) {
	    var elementCtx = getElementCtx(el);
	    for (var _i = 0, events_1 = events; _i < events_1.length; _i++) {
	        var event_1 = events_1[_i];
	        elementCtx[event_1].internalAfterHandlers.unshift(handler);
	        nativeMethods.addEventListener.call(el, event_1, handler);
	    }
	}
	function addFirstInternalBeforeHandler(el, events, handler) {
	    var elementCtx = getElementCtx(el);
	    for (var _i = 0, events_2 = events; _i < events_2.length; _i++) {
	        var event_2 = events_2[_i];
	        elementCtx[event_2].internalBeforeHandlers.unshift(handler);
	    }
	}
	function addInternalBeforeHandler(el, events, handler) {
	    var elementCtx = getElementCtx(el);
	    for (var _i = 0, events_3 = events; _i < events_3.length; _i++) {
	        var event_3 = events_3[_i];
	        elementCtx[event_3].internalBeforeHandlers.push(handler);
	    }
	}
	function removeInternalBeforeHandler(el, events, handler) {
	    var elementCtx = getElementCtx(el);
	    for (var _i = 0, events_4 = events; _i < events_4.length; _i++) {
	        var event_4 = events_4[_i];
	        var internalBeforeHandlers = elementCtx[event_4].internalBeforeHandlers;
	        var handlerIndex = internalBeforeHandlers.indexOf(handler);
	        if (handlerIndex > -1)
	            internalBeforeHandlers.splice(handlerIndex, 1);
	    }
	}
	function wrapEventListener(eventCtx, listener, wrapper, useCapture) {
	    eventCtx.outerHandlers.push({
	        fn: listener,
	        useCapture: useCapture || false
	    });
	    eventCtx.wrappers.push(wrapper);
	}
	function getWrapper(eventCtx, listener, useCapture) {
	    var originListeners = eventCtx.outerHandlers;
	    var wrappers = eventCtx.wrappers;
	    var wrapper = null;
	    for (var i = 0; i < originListeners.length; i++) {
	        var curListener = originListeners[i];
	        if (curListener.fn === listener && (curListener.useCapture || false) === (useCapture || false)) {
	            wrapper = wrappers[i];
	            wrappers.splice(i, 1);
	            originListeners.splice(i, 1);
	            return wrapper;
	        }
	    }
	    return null;
	}
	function updateInternalAfterHandlers(el, eventType) {
	    var elementCtx = getElementCtx(el);
	    for (var _i = 0, _a = elementCtx[eventType].internalAfterHandlers; _i < _a.length; _i++) {
	        var handler = _a[_i];
	        nativeMethods.removeEventListener.call(el, eventType, handler);
	        nativeMethods.addEventListener.call(el, eventType, handler);
	    }
	}

	var listeningContextUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		getElementCtx: getElementCtx,
		getEventCtx: getEventCtx,
		isElementListening: isElementListening,
		addListeningElement: addListeningElement,
		removeListeningElement: removeListeningElement,
		addInternalAfterHandler: addInternalAfterHandler,
		addFirstInternalBeforeHandler: addFirstInternalBeforeHandler,
		addInternalBeforeHandler: addInternalBeforeHandler,
		removeInternalBeforeHandler: removeInternalBeforeHandler,
		wrapEventListener: wrapEventListener,
		getWrapper: getWrapper,
		updateInternalAfterHandlers: updateInternalAfterHandlers
	});

	var LISTENED_EVENTS = [
	    'click', 'mousedown', 'mouseup', 'dblclick', 'contextmenu', 'mousemove', 'mouseover', 'mouseout',
	    'pointerdown', 'pointermove', 'pointerover', 'pointerout', 'pointerup',
	    'MSPointerDown', 'MSPointerMove', 'MSPointerOver', 'MSPointerOut', 'MSPointerUp',
	    'touchstart', 'touchmove', 'touchend',
	    'keydown', 'keypress', 'keyup',
	    'change', 'focus', 'blur', 'focusin', 'focusout'
	];
	var EVENT_SANDBOX_DISPATCH_EVENT_FLAG = 'hammerhead|event-sandbox-dispatch-event-flag';
	var Listeners = /** @class */ (function (_super) {
	    __extends(Listeners, _super);
	    function Listeners() {
	        var _this = _super.call(this) || this;
	        _this.EVENT_LISTENER_ATTACHED_EVENT = 'hammerhead|event|event-listener-attached';
	        _this.EVENT_LISTENER_DETACHED_EVENT = 'hammerhead|event|event-listener-detached';
	        _this.listeningCtx = listeningContextUtils;
	        _this.addInternalEventBeforeListener = _this.listeningCtx.addInternalBeforeHandler;
	        _this.addFirstInternalEventBeforeListener = _this.listeningCtx.addFirstInternalBeforeHandler;
	        _this.addInternalEventAfterListener = _this.listeningCtx.addInternalAfterHandler;
	        _this.removeInternalEventBeforeListener = _this.listeningCtx.removeInternalBeforeHandler;
	        return _this;
	    }
	    Listeners._getNativeAddEventListener = function (el) {
	        if (isIE11) {
	            if (isWindow(el))
	                return nativeMethods.windowAddEventListener;
	            return el.body !== void 0 ? nativeMethods.documentAddEventListener : nativeMethods.addEventListener;
	        }
	        return nativeMethods.addEventListener;
	    };
	    Listeners._getNativeRemoveEventListener = function (el) {
	        if (isIE11) {
	            if (isWindow(el))
	                return nativeMethods.windowRemoveEventListener;
	            return el.body !== void 0 ? nativeMethods.documentRemoveEventListener : nativeMethods.removeEventListener;
	        }
	        return nativeMethods.removeEventListener;
	    };
	    Listeners._isIEServiceHandler = function (listener) {
	        return listener.toString() === '[object FunctionWrapper]';
	    };
	    Listeners._getEventListenerWrapper = function (eventCtx, listener) {
	        return function (e) {
	            // NOTE: Ignore IE11's and Edge's service handlers (GH-379)
	            if (Listeners._isIEServiceHandler(listener) || eventCtx.cancelOuterHandlers)
	                return null;
	            if (typeof eventCtx.outerHandlersWrapper === 'function')
	                return eventCtx.outerHandlersWrapper.call(this, e, listener);
	            return callEventListener(this, listener, e);
	        };
	    };
	    Listeners._isDifferentHandler = function (outerHandlers, listener, useCapture) {
	        for (var _i = 0, outerHandlers_1 = outerHandlers; _i < outerHandlers_1.length; _i++) {
	            var outerHandler = outerHandlers_1[_i];
	            if (outerHandler.fn === listener && outerHandler.useCapture === useCapture)
	                return false;
	        }
	        return true;
	    };
	    Listeners._getUseCaptureParam = function (optionalParam) {
	        if (optionalParam && typeof optionalParam === 'boolean')
	            return optionalParam;
	        else if (optionalParam && typeof optionalParam === 'object')
	            return !!optionalParam.capture;
	        return false;
	    };
	    Listeners.prototype._createEventHandler = function () {
	        var listeners = this;
	        return function (e) {
	            var el = this;
	            var elWindow = el[INTERNAL_PROPS.processedContext] || window;
	            var eventPrevented = false;
	            var handlersCancelled = false;
	            var stopPropagationCalled = false;
	            var eventCtx = listeners.listeningCtx.getEventCtx(el, e.type);
	            if (!eventCtx)
	                return;
	            var internalHandlers = eventCtx.internalBeforeHandlers;
	            eventCtx.cancelOuterHandlers = false;
	            var preventEvent = function (allowBubbling) {
	                eventPrevented = true;
	                preventDefault(e, allowBubbling);
	            };
	            var cancelHandlers = function () {
	                if (!handlersCancelled)
	                    eventCtx.cancelOuterHandlers = true;
	                handlersCancelled = true;
	            };
	            var stopEventPropagation = function () {
	                stopPropagationCalled = true;
	                stopPropagation(e);
	            };
	            // NOTE: Some listeners can remove itself when executed, so we need to copy the list of listeners here
	            var currentInternalHandlers = nativeMethods.arraySlice.call(internalHandlers);
	            for (var _i = 0, currentInternalHandlers_1 = currentInternalHandlers; _i < currentInternalHandlers_1.length; _i++) {
	                var internalHandler = currentInternalHandlers_1[_i];
	                internalHandler.call(el, e, elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG], preventEvent, cancelHandlers, stopEventPropagation);
	                if (eventPrevented || stopPropagationCalled)
	                    break;
	            }
	        };
	    };
	    Listeners.prototype.createOverriddenMethods = function () {
	        var listeners = this;
	        return {
	            addEventListener: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var eventType = args[0], listener = args[1];
	                var el = this;
	                var useCapture = Listeners._getUseCaptureParam(args[2]);
	                var eventCtx = getEventCtx(el, eventType);
	                var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
	                if (!eventCtx || !isValidEventListener(listener))
	                    return nativeAddEventListener.apply(el, args);
	                // NOTE: T233158
	                var isDifferentHandler = Listeners._isDifferentHandler(eventCtx.outerHandlers, listener, useCapture);
	                if (!isDifferentHandler)
	                    return null;
	                var wrapper = Listeners._getEventListenerWrapper(eventCtx, listener);
	                args[1] = wrapper;
	                wrapEventListener(eventCtx, listener, wrapper, useCapture);
	                var res = nativeAddEventListener.apply(el, args);
	                updateInternalAfterHandlers(el, eventType);
	                listeners.emit(listeners.EVENT_LISTENER_ATTACHED_EVENT, { el: el, eventType: eventType, listener: listener });
	                return res;
	            },
	            removeEventListener: function () {
	                var args = [];
	                for (var _i = 0; _i < arguments.length; _i++) {
	                    args[_i] = arguments[_i];
	                }
	                var eventType = args[0], listener = args[1];
	                var el = this;
	                var useCapture = Listeners._getUseCaptureParam(args[2]);
	                var nativeRemoveEventListener = Listeners._getNativeRemoveEventListener(el);
	                var eventCtx = getEventCtx(el, eventType);
	                if (!eventCtx || !isValidEventListener(listener))
	                    return nativeRemoveEventListener.apply(el, args);
	                var wrapper = getWrapper(eventCtx, listener, useCapture);
	                args[1] = wrapper;
	                var res = nativeRemoveEventListener.apply(el, args);
	                listeners.emit(listeners.EVENT_LISTENER_DETACHED_EVENT, { el: el, listener: listener, eventType: eventType });
	                return res;
	            }
	        };
	    };
	    Listeners.prototype.initElementListening = function (el, events) {
	        if (events === void 0) { events = LISTENED_EVENTS; }
	        var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
	        for (var _i = 0, events_1 = events; _i < events_1.length; _i++) {
	            var event_1 = events_1[_i];
	            if (!this.listeningCtx.getEventCtx(el, event_1))
	                nativeAddEventListener.call(el, event_1, this._createEventHandler(), true);
	        }
	        this.listeningCtx.addListeningElement(el, events);
	        if (isIE11) {
	            var overriddenMethods = this.createOverriddenMethods();
	            if (!el.addEventListener) {
	                // NOTE: we cannot use 'overrideFunction' here since the functions may not exist
	                el.addEventListener = overriddenMethods.addEventListener;
	                el.removeEventListener = overriddenMethods.removeEventListener;
	                overrideStringRepresentation(el.addEventListener, nativeMethods.addEventListener);
	                overrideStringRepresentation(el.removeEventListener, nativeMethods.removeEventListener);
	            }
	            else if (isNativeFunction(el.addEventListener)) {
	                overrideFunction(el, 'addEventListener', overriddenMethods.addEventListener);
	                overrideFunction(el, 'removeEventListener', overriddenMethods.removeEventListener);
	            }
	        }
	    };
	    Listeners.prototype.initDocumentBodyListening = function (doc) {
	        addListeningElement(doc.body, DOM_EVENTS);
	        if (isIE11) {
	            var overriddenMethods = this.createOverriddenMethods();
	            overrideFunction(doc.body, 'addEventListener', overriddenMethods.addEventListener);
	            overrideFunction(doc.body, 'removeEventListener', overriddenMethods.removeEventListener);
	        }
	    };
	    Listeners.prototype.restartElementListening = function (el) {
	        var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
	        var elementCtx = this.listeningCtx.getElementCtx(el);
	        if (elementCtx) {
	            var eventNames = nativeMethods.objectKeys(elementCtx);
	            for (var _i = 0, eventNames_1 = eventNames; _i < eventNames_1.length; _i++) {
	                var eventName = eventNames_1[_i];
	                nativeAddEventListener.call(el, eventName, this._createEventHandler(), true);
	            }
	        }
	    };
	    Listeners.prototype.cancelElementListening = function (el) {
	        this.listeningCtx.removeListeningElement(el);
	        if (el.body)
	            this.listeningCtx.removeListeningElement(el.body);
	    };
	    Listeners.beforeDispatchEvent = function (el) {
	        var elWindow = el[INTERNAL_PROPS.processedContext] || window;
	        elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG] = (elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG] || 0) + 1;
	    };
	    Listeners.afterDispatchEvent = function (el) {
	        var elWindow = el[INTERNAL_PROPS.processedContext] || window;
	        elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG]--;
	        if (!elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG])
	            delete elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG];
	    };
	    Listeners.prototype.setEventListenerWrapper = function (el, events, wrapper) {
	        if (!this.listeningCtx.isElementListening(el))
	            this.initElementListening(el, events);
	        for (var _i = 0, events_2 = events; _i < events_2.length; _i++) {
	            var event_2 = events_2[_i];
	            var eventCtx = this.listeningCtx.getEventCtx(el, event_2);
	            eventCtx.outerHandlersWrapper = wrapper;
	        }
	    };
	    Listeners.prototype.getEventListeners = function (el, event) {
	        var eventCtx = this.listeningCtx.getEventCtx(el, event);
	        if (!eventCtx)
	            return null;
	        return nativeMethods.arrayMap.call(eventCtx.outerHandlers, function (handler) { return handler.fn; });
	    };
	    return Listeners;
	}(EventEmitter));

	var Selection = /** @class */ (function () {
	    function Selection(eventSandbox) {
	        this.focusBlurSandbox = eventSandbox.focusBlur;
	        this.timersSandbox = eventSandbox.timers;
	        this.listeners = eventSandbox.listeners;
	        this.eventSimulator = eventSandbox.eventSimulator;
	        var selection = this;
	        var eventSimulator = this.eventSimulator;
	        var listeners = this.listeners;
	        var timersSandbox = this.timersSandbox;
	        this.setSelectionRangeWrapper = function () {
	            var selectionStart = arguments[0];
	            var selectionEnd = arguments[1];
	            var selectionDirection = arguments[2] || 'none';
	            var el = this;
	            var fn = isTextAreaElement(el) ? nativeMethods.textAreaSetSelectionRange : nativeMethods.setSelectionRange;
	            var activeElement = getActiveElement(findDocument(el));
	            var curDocument = findDocument(el);
	            var isElementActive = false;
	            var selectionSetter = function () {
	                // NOTE: These browsers cannot restore the `selectionStart` and `selectionEnd` properties when we change the `type` attribute.
	                // So we need to use our own mechanism to store the `selectionStart` and `selectionEnd` properties.
	                var useInternalSelection = isInputWithoutSelectionProperties(el);
	                var savedType = el.type;
	                var res;
	                if (useInternalSelection)
	                    el.setAttribute('type', 'text');
	                // NOTE: In MSEdge, an error occurs when the setSelectionRange method is called for an input with
	                // 'display = none' and selectionStart !== selectionEnd in other IEs, the error doesn't occur, but
	                // as a result selectionStart === selectionEnd === 0.
	                try {
	                    res = fn.call(el, selectionStart, selectionEnd, selectionDirection);
	                }
	                catch (e) {
	                    res = fn.call(el, 0, 0, selectionDirection);
	                }
	                if (useInternalSelection) {
	                    el[INTERNAL_PROPS.selection] = {
	                        selectionStart: el.selectionStart,
	                        selectionEnd: el.selectionEnd,
	                        selectionDirection: el.selectionDirection
	                    };
	                    el.setAttribute('type', savedType);
	                    // HACK: (A problem with input type = 'number' after Chrome is updated to v.33.0.1750.117 and
	                    // in Firefox 29.0.  T101195) To set right selection: if the input type is 'number' or 'email',
	                    // we need to change the type to text, and then restore it after setting selection.(B254340).
	                    // However, the type is changed asynchronously in this case. To force type changing,we need to
	                    // call blur, Then raise the focus event to make the element active.
	                    if (isElementActive) {
	                        selection.focusBlurSandbox.blur(el, null, true);
	                        selection.focusBlurSandbox.focus(el, null, true);
	                        // HACK: we should call focus for previous active element again because
	                        // in Firefox 55.0.3 after first focus active element isn't changed
	                        if (getActiveElement(findDocument(el)) !== el)
	                            selection.focusBlurSandbox.focus(el, null, true);
	                    }
	                }
	                // NOTE: In MSEdge, the 'selectionchange' event doesn't occur immediately (it occurs with a delay)
	                // So, we should raise it right after the 'setSelectionRange' method.
	                if (isMSEdge)
	                    eventSimulator.selectionchange(el);
	                return res;
	            };
	            if (activeElement === el) {
	                isElementActive = true;
	                return selectionSetter();
	            }
	            var needFocus = isIE11 || isMSEdge &&
	                (version === 17 && !curDocument.hasFocus() || version > 17);
	            return selection.wrapSetterSelection(el, selectionSetter, needFocus);
	        };
	        this.selectWrapper = function () {
	            // NOTE: Non-standard IE Only class TextRange
	            // @ts-ignore
	            var element = this.parentElement();
	            if (!element || getActiveElement(findDocument(element)) === element)
	                return nativeMethods.select.call(this);
	            var result = null;
	            var focusRaised = false;
	            var focusHandler = function (e) {
	                if (e.target === element || element.style.display === 'none')
	                    focusRaised = true;
	            };
	            listeners.addInternalEventBeforeListener(document, ['focus'], focusHandler);
	            result = nativeMethods.select.call(this);
	            timersSandbox.setTimeout.call(window, function () {
	                timersSandbox.setTimeout.call(window, function () {
	                    listeners.removeInternalEventBeforeListener(document, ['focus'], focusHandler);
	                    if (!focusRaised)
	                        eventSimulator.focus(element);
	                }, 0);
	            }, 0);
	            return result;
	        };
	    }
	    Selection.prototype.setSelection = function (el, start, end, direction) {
	        if (el.setSelectionRange)
	            el.setSelectionRange(start, end, direction);
	        else {
	            el.selectionStart = start;
	            el.selectionEnd = end;
	        }
	    };
	    Selection.prototype.getSelection = function (el) {
	        var internalSelection = el[INTERNAL_PROPS.selection];
	        return {
	            start: internalSelection ? internalSelection.selectionStart : el.selectionStart,
	            end: internalSelection ? internalSelection.selectionEnd : el.selectionEnd,
	            direction: internalSelection ? internalSelection.selectionDirection : el.selectionDirection
	        };
	    };
	    Selection.prototype.wrapSetterSelection = function (el, selectionSetter, needFocus, isContentEditable) {
	        var _this = this;
	        var curDocument = findDocument(el);
	        var activeElement = getActiveElement(curDocument);
	        var result = null;
	        var focusRaised = false;
	        // NOTE: we should not call focus during selection setting
	        // if element has been focused already (TestCafe GH-2301)
	        needFocus = needFocus && activeElement !== el;
	        var focusHandler = function (e) {
	            if (e.target === el || el.style.display === 'none')
	                focusRaised = true;
	        };
	        if (needFocus)
	            this.listeners.addInternalEventBeforeListener(document, ['focus'], focusHandler);
	        // The focus and blur events
	        Listeners.beforeDispatchEvent(el);
	        Listeners.beforeDispatchEvent(el);
	        result = selectionSetter();
	        // The focus and blur events
	        Listeners.afterDispatchEvent(el);
	        Listeners.afterDispatchEvent(el);
	        if (needFocus) {
	            activeElement = getActiveElement(curDocument);
	            if (activeElement !== el && (isWebKit || isMSEdge && version > 17)) {
	                if (focusRaised)
	                    el[FocusBlurSandbox.getInternalEventFlag('focus')] = true;
	                el.focus();
	            }
	            // NOTE: In MSEdge, focus and blur are sync.
	            if (isIE11) {
	                this.timersSandbox.setTimeout.call(window, function () {
	                    _this.timersSandbox.setTimeout.call(window, function () {
	                        _this.listeners.removeInternalEventBeforeListener(document, ['focus'], focusHandler);
	                        if (!focusRaised)
	                            _this.eventSimulator.focus(el);
	                    }, 0);
	                }, 0);
	            }
	            else {
	                this.listeners.removeInternalEventBeforeListener(document, ['focus'], focusHandler);
	                if (!focusRaised) {
	                    // NOTE: In Firefox, raising the dispatchEvent 'focus' doesn’t activate an element.
	                    // We should call the native focus method.
	                    if (isContentEditable && isFirefox)
	                        this.focusBlurSandbox.focus(el, null, true, false, true);
	                    else
	                        this.eventSimulator.focus(el);
	                }
	            }
	        }
	        return result;
	    };
	    return Selection;
	}());

	// https://html.spec.whatwg.org/multipage/interaction.html#the-drag-data-item-kind
	var DATA_TRANSFER_ITEM_KIND = {
	    string: 'string',
	    file: 'file'
	};

	// https://html.spec.whatwg.org/multipage/interaction.html#datatransferitem
	var DataTransferItem = /** @class */ (function () {
	    function DataTransferItem(kind, type, data) {
	        nativeMethods.objectDefineProperty(this, 'kind', {
	            enumerable: true,
	            get: function () { return kind; }
	        });
	        nativeMethods.objectDefineProperty(this, 'type', {
	            enumerable: true,
	            get: function () { return type; }
	        });
	        nativeMethods.objectDefineProperty(this, 'getAsString', {
	            configurable: true,
	            enumerable: true,
	            get: function () {
	                return function (callback) {
	                    if (!arguments.length)
	                        throw new Error("Failed to execute 'getAsString' on 'DataTransferItem': 1 argument required, but only 0 present.");
	                    if (typeof callback !== 'function')
	                        return;
	                    if (kind !== DATA_TRANSFER_ITEM_KIND.string)
	                        return;
	                    nativeMethods.setTimeout.call(window, function () { return callback(data); }, 0);
	                };
	            }
	        });
	        nativeMethods.objectDefineProperty(this, 'getAsFile', {
	            configurable: true,
	            enumerable: true,
	            get: function () {
	                return function () {
	                    if (kind !== DATA_TRANSFER_ITEM_KIND.file)
	                        return null;
	                    return data;
	                };
	            }
	        });
	    }
	    return DataTransferItem;
	}());
	if (nativeMethods.DataTransferItem)
	    DataTransferItem.prototype = nativeMethods.DataTransferItem.prototype;

	// https://html.spec.whatwg.org/multipage/interaction.html#drag-data-store-mode
	var DATA_STORE_MODE = {
	    readwrite: 'readwrite',
	    readonly: 'readonly',
	    protected: 'protected'
	};

	function parseTextUriList(textUriList) {
	    textUriList = textUriList.replace(/\r\n$/, '');
	    var res = [];
	    if (textUriList === '')
	        return res;
	    textUriList = textUriList.split(/\r\n/);
	    for (var _i = 0, textUriList_1 = textUriList; _i < textUriList_1.length; _i++) {
	        var textUri = textUriList_1[_i];
	        if (textUri !== '#')
	            res.push(textUri);
	    }
	    return res;
	}
	function processFormat(format) {
	    if (format === 'text')
	        return 'text/plain';
	    if (format === 'url')
	        return 'text/uri-list';
	    return format;
	}
	// https://html.spec.whatwg.org/multipage/interaction.html#datatransferitemlist
	var DataTransferItemList = /** @class */ (function () {
	    function DataTransferItemList(dataStore) {
	        var _this = this;
	        // Internals
	        var items = [];
	        var itemsData = [];
	        var getTypes = function () {
	            var res = [];
	            for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
	                var item = items_1[_i];
	                res.push(item.type);
	            }
	            return res;
	        };
	        var updateIndexes = function () {
	            var idx = 0;
	            while (items[idx] !== void 0 || _this[idx] !== void 0) {
	                var item = items[idx];
	                nativeMethods.objectDefineProperty(_this, idx, {
	                    enumerable: item !== void 0,
	                    configurable: true,
	                    value: item
	                });
	                idx++;
	            }
	        };
	        var getItemData = function (format) {
	            var convertToUrl = false;
	            format = processFormat(format);
	            if (format === 'url')
	                convertToUrl = true;
	            var item = '';
	            for (var i = 0; i < items.length; i++) {
	                if (items[i].type === format)
	                    item = itemsData[i];
	            }
	            if (convertToUrl && item)
	                item = parseTextUriList(item)[0];
	            return item;
	        };
	        var removeItem = function (format) {
	            format = processFormat(format);
	            for (var i = 0; i < items.length; i++) {
	                if (items[i].type === format) {
	                    items.splice(i, 1);
	                    itemsData.splice(i, 1);
	                    break;
	                }
	            }
	            updateIndexes();
	        };
	        var addItem = function (data, type, allowReplace) {
	            var newItem = null;
	            if (typeof data === 'string') {
	                var typeLowerCase = type.toString().toLowerCase();
	                var itemData = getItemData(typeLowerCase);
	                if (!allowReplace && itemData)
	                    throw new Error("Failed to execute 'add' on 'DataTransferItemList': An item already exists for type '" + typeLowerCase + "'.");
	                if (itemData)
	                    removeItem(typeLowerCase);
	                newItem = new DataTransferItem(DATA_TRANSFER_ITEM_KIND.string, processFormat(type), data);
	            }
	            else
	                newItem = new DataTransferItem(DATA_TRANSFER_ITEM_KIND.file, null, data);
	            items.push(newItem);
	            itemsData.push(data);
	            updateIndexes();
	            return newItem;
	        };
	        // Internal API
	        this.getAndHideInternalMethods = function () {
	            var res = { getTypes: getTypes, getItemData: getItemData, removeItem: removeItem, addItem: addItem };
	            delete _this.getAndHideInternalMethods;
	            return res;
	        };
	        // API
	        nativeMethods.objectDefineProperty(this, 'length', {
	            enumerable: true,
	            get: function () { return items.length; }
	        });
	        nativeMethods.objectDefineProperty(this, 'remove', {
	            configurable: true,
	            enumerable: true,
	            get: function () {
	                return function (idx) {
	                    if (dataStore.mode !== DATA_STORE_MODE.readwrite)
	                        return;
	                    items.splice(idx, 1);
	                    itemsData.splice(idx, 1);
	                    updateIndexes();
	                };
	            }
	        });
	        nativeMethods.objectDefineProperty(this, 'clear', {
	            configurable: true,
	            enumerable: true,
	            get: function () {
	                return function () {
	                    if (dataStore.mode !== DATA_STORE_MODE.readwrite)
	                        return;
	                    items = [];
	                    itemsData = [];
	                    updateIndexes();
	                };
	            }
	        });
	        nativeMethods.objectDefineProperty(this, 'add', {
	            configurable: true,
	            enumerable: true,
	            get: function () {
	                return function (data, type) {
	                    if (!arguments.length)
	                        throw new Error("Failed to execute 'add' on 'DataTransferItemList': 1 argument required, but only 0 present.");
	                    if (arguments.length === 1 && typeof data === 'string')
	                        throw new Error("Failed to execute 'add' on 'DataTransferItemList': parameter 1 is not of type 'File'.");
	                    if (dataStore.mode !== DATA_STORE_MODE.readwrite)
	                        return void 0;
	                    return addItem(data, type, false);
	                };
	            }
	        });
	    }
	    return DataTransferItemList;
	}());
	if (nativeMethods.DataTransferItemList)
	    DataTransferItemList.prototype = nativeMethods.DataTransferItemList.prototype;

	// https://w3c.github.io/FileAPI/#filelist-section
	var FileList$1 = /** @class */ (function () {
	    function FileList() {
	        nativeMethods.objectDefineProperty(this, 'length', {
	            enumerable: true,
	            get: function () { return 0; }
	        });
	        nativeMethods.objectDefineProperty(this, 'item', {
	            enumerable: true,
	            get: function () {
	                return function () {
	                    return void 0;
	                };
	            }
	        });
	    }
	    return FileList;
	}());
	if (nativeMethods.FileList)
	    FileList$1.prototype = nativeMethods.FileList.prototype;

	// https://html.spec.whatwg.org/multipage/interaction.html#dom-datatransfer-dropeffect
	var DROP_EFFECT = {
	    none: 'none',
	    copy: 'copy',
	    link: 'link',
	    move: 'move'
	};

	// https://html.spec.whatwg.org/multipage/interaction.html#dom-datatransfer-effectallowed
	var EFFECT_ALLOWED = {
	    uninitialized: 'uninitialized',
	    none: 'none',
	    copy: 'copy',
	    copyLink: 'copyLink',
	    copyMove: 'copyMove',
	    link: 'link',
	    linkMove: 'linkMove',
	    move: 'move',
	    all: 'all'
	};

	// NOTE: Although DataTransfer interface has a constructor, it is not possible to
	// https://html.spec.whatwg.org/multipage/interaction.html#datatransfer
	var DataTransfer = /** @class */ (function () {
	    function DataTransfer(dataStore) {
	        var dropEffect = DROP_EFFECT.none;
	        var effectAllowed = EFFECT_ALLOWED.uninitialized;
	        var itemList = new DataTransferItemList(dataStore);
	        var itemListInternals = itemList.getAndHideInternalMethods();
	        var fileList = new FileList$1();
	        var emptyItemList = new DataTransferItemList(dataStore);
	        var emptyListInternals = emptyItemList.getAndHideInternalMethods();
	        var getActualItemList = function () {
	            return dataStore.mode === DATA_STORE_MODE.protected ? emptyItemList : itemList;
	        };
	        var getActualItemListInternals = function () {
	            return dataStore.mode === DATA_STORE_MODE.protected ? emptyListInternals : itemListInternals;
	        };
	        nativeMethods.objectDefineProperty(this, 'dropEffect', {
	            configurable: true,
	            enumerable: true,
	            get: function () { return dropEffect; },
	            set: function (value) {
	                if (DROP_EFFECT[value])
	                    dropEffect = DROP_EFFECT[value];
	                return value;
	            }
	        });
	        nativeMethods.objectDefineProperty(this, 'effectAllowed', {
	            configurable: true,
	            enumerable: true,
	            get: function () { return effectAllowed; },
	            set: function (value) {
	                if (EFFECT_ALLOWED[value])
	                    effectAllowed = EFFECT_ALLOWED[value];
	                return value;
	            }
	        });
	        if (!isIE11) {
	            nativeMethods.objectDefineProperty(this, 'items', {
	                configurable: true,
	                enumerable: true,
	                get: getActualItemList
	            });
	        }
	        nativeMethods.objectDefineProperty(this, 'types', {
	            configurable: true,
	            enumerable: true,
	            get: function () { return getActualItemListInternals().getTypes(); }
	        });
	        nativeMethods.objectDefineProperty(this, 'files', {
	            configurable: true,
	            enumerable: true,
	            get: function () { return fileList; }
	        });
	        if (!isIE11) {
	            this.setDragImage = function () {
	                // do nothing
	            };
	        }
	        this.getData = function (format) {
	            if (!arguments.length)
	                throw new Error("Failed to execute 'getData' on 'DataTransfer': 1 argument required, but only 0 present.");
	            format = format.toString().toLowerCase();
	            return getActualItemListInternals().getItemData(format);
	        };
	        this.setData = function (format, data) {
	            if (arguments.length < 2)
	                throw new Error("Failed to execute 'setData' on 'DataTransfer': 2 argument required, but only " + arguments.length + " present.");
	            if (dataStore.mode !== DATA_STORE_MODE.readwrite)
	                return;
	            format = format.toString().toLowerCase();
	            itemListInternals.addItem(data, format, true);
	        };
	        this.clearData = function (format) {
	            if (dataStore.mode !== DATA_STORE_MODE.readwrite)
	                return;
	            if (format === void 0)
	                // @ts-ignore
	                itemList.clear();
	            else
	                itemListInternals.removeItem(format);
	        };
	    }
	    return DataTransfer;
	}());
	if (hasDataTransfer)
	    DataTransfer.prototype = nativeMethods.DataTransfer.prototype;

	// https://html.spec.whatwg.org/multipage/interaction.html#the-drag-data-store
	var DragDataStore = /** @class */ (function () {
	    function DragDataStore() {
	        this.mode = DATA_STORE_MODE.readwrite;
	    }
	    DragDataStore.prototype.setReadOnlyMode = function () {
	        this.mode = DATA_STORE_MODE.readonly;
	    };
	    DragDataStore.prototype.setProtectedMode = function () {
	        this.mode = DATA_STORE_MODE.protected;
	    };
	    return DragDataStore;
	}());

	var EventSandbox = /** @class */ (function (_super) {
	    __extends(EventSandbox, _super);
	    function EventSandbox(listeners, eventSimulator, elementEditingWatcher, unloadSandbox, messageSandbox, _shadowUI, timerSandbox) {
	        var _this = _super.call(this) || this;
	        _this._shadowUI = _shadowUI;
	        _this.EVENT_PREVENTED_EVENT = 'hammerhead|event|event-prevented';
	        _this.listeners = listeners;
	        _this.elementEditingWatcher = elementEditingWatcher;
	        _this.unload = unloadSandbox;
	        _this.timers = timerSandbox;
	        _this.eventSimulator = eventSimulator;
	        _this.focusBlur = new FocusBlurSandbox(listeners, eventSimulator, messageSandbox, timerSandbox, elementEditingWatcher);
	        _this.selection = new Selection(_this);
	        _this.hover = new HoverSandbox(listeners);
	        _this._shadowUI = _shadowUI;
	        _this.message = messageSandbox;
	        _this.DataTransfer = DataTransfer;
	        _this.DragDataStore = DragDataStore;
	        _this._overriddenMethods = null;
	        _this._onFocus = null;
	        _this._cancelInternalEvents = null;
	        _this._createOverriddenMethods();
	        _this._createInternalHandlers();
	        return _this;
	    }
	    EventSandbox.prototype._createOverriddenMethods = function () {
	        var selection = this.selection;
	        var focusBlurSandbox = this.focusBlur;
	        var eventSimulator = this.eventSimulator;
	        var sandbox = this;
	        this._overriddenMethods = {
	            dispatchEvent: function () {
	                Listeners.beforeDispatchEvent(this);
	                var res = isIE11 && isWindow(this)
	                    ? nativeMethods.windowDispatchEvent.apply(this, arguments)
	                    : nativeMethods.dispatchEvent.apply(this, arguments);
	                Listeners.afterDispatchEvent(this);
	                return res;
	            },
	            click: function () {
	                Listeners.beforeDispatchEvent(this);
	                var res = eventSimulator.nativeClick(this, nativeMethods.click);
	                Listeners.afterDispatchEvent(this);
	                return res;
	            },
	            setSelectionRange: function () {
	                return selection.setSelectionRangeWrapper.apply(this, arguments);
	            },
	            select: function () {
	                return selection.selectWrapper.call(this);
	            },
	            focus: function () {
	                return focusBlurSandbox.focus(this, null, false, false, true);
	            },
	            blur: function () {
	                return focusBlurSandbox.blur(this, null, false, true);
	            },
	            preventDefault: function () {
	                sandbox.emit(sandbox.EVENT_PREVENTED_EVENT, this);
	                return nativeMethods.preventDefault.call(this);
	            }
	        };
	    };
	    EventSandbox.prototype._createInternalHandlers = function () {
	        var shadowUI = this._shadowUI;
	        var document = this.document;
	        var eventSimulator = this.eventSimulator;
	        this._onFocus = function (e) {
	            var focusedEl = e.target;
	            var activeEl = getActiveElement(document);
	            if (!isShadowUIElement(focusedEl) && !isShadowUIElement(activeEl))
	                shadowUI.setLastActiveElement(activeEl);
	        };
	        this._cancelInternalEvents = function (e, _dispatched, _preventEvent, _cancelHandlers, stopPropagation) {
	            // NOTE: We should cancel events raised by calling the native function (focus, blur) only if the
	            // element has a flag. If an event is dispatched, we shouldn't cancel it.
	            // After calling a native function two events were raised
	            // .focus() -> focus, focusin
	            // .blur() -> blur, focusout
	            // So we should prevent both events
	            var eventType = FocusBlurSandbox.getNonBubblesEventType(e.type) || e.type;
	            var internalEventFlag = FocusBlurSandbox.getInternalEventFlag(eventType);
	            //@ts-ignore
	            if (e.target[internalEventFlag] && !e[eventSimulator.DISPATCHED_EVENT_FLAG])
	                stopPropagation();
	        };
	    };
	    EventSandbox.prototype._preventInputNativeDialogs = function (window) {
	        var shouldPreventClickEvents = isFirefox || isIE;
	        if (!shouldPreventClickEvents)
	            return;
	        // NOTE: Google Chrome and Safari don't open the native browser dialog when TestCafe clicks on the input.
	        // 'Click' is a complex emulated action that uses 'dispatchEvent' method internally.
	        // Another browsers open the native browser dialog in this case.
	        // This is why, we are forced to prevent the browser's open file dialog.
	        this.listeners.addInternalEventBeforeListener(window, ['click'], function (e, dispatched) {
	            if (dispatched && isInputWithNativeDialog(e.target))
	                preventDefault(e, true);
	        });
	    };
	    EventSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        overrideFunction(window.HTMLInputElement.prototype, 'setSelectionRange', this._overriddenMethods.setSelectionRange);
	        overrideFunction(window.HTMLTextAreaElement.prototype, 'setSelectionRange', this._overriddenMethods.setSelectionRange);
	        if (isIE11) {
	            overrideFunction(window.Window.prototype, 'dispatchEvent', this._overriddenMethods.dispatchEvent);
	            overrideFunction(window.Document.prototype, 'dispatchEvent', this._overriddenMethods.dispatchEvent);
	            overrideFunction(window.HTMLElement.prototype, 'dispatchEvent', this._overriddenMethods.dispatchEvent);
	            overrideFunction(window.SVGElement.prototype, 'dispatchEvent', this._overriddenMethods.dispatchEvent);
	        }
	        else {
	            overrideFunction(window.EventTarget.prototype, 'dispatchEvent', this._overriddenMethods.dispatchEvent);
	        }
	        overrideFunction(window.HTMLElement.prototype, 'focus', this._overriddenMethods.focus);
	        overrideFunction(window.HTMLElement.prototype, 'blur', this._overriddenMethods.blur);
	        overrideFunction(window.HTMLElement.prototype, 'click', this._overriddenMethods.click);
	        overrideFunction(window.Event.prototype, 'preventDefault', this._overriddenMethods.preventDefault);
	        // @ts-ignore Window constructor has no the focus method
	        window.Window.focus = this._overriddenMethods.focus;
	        // NOTE: we cannot use 'overrideFunction' here since the focus method may not exist
	        // @ts-ignore Window constructor has no the focus method
	        overrideStringRepresentation(window.Window.focus, nativeMethods.focus);
	        // @ts-ignore Window constructor has no the blur method
	        window.Window.blur = this._overriddenMethods.blur;
	        // NOTE: we cannot use 'overrideFunction' here since the blur method may not exist
	        // @ts-ignore Window constructor has no the blur method
	        overrideStringRepresentation(window.Window.blur, nativeMethods.blur);
	        // @ts-ignore TextRange exists only in IE
	        if (window.TextRange && window.TextRange.prototype.select)
	            // @ts-ignore TextRange exists only in IE
	            overrideFunction(window.TextRange.prototype, 'select', this._overriddenMethods.select);
	        this.listeners.initElementListening(document, DOM_EVENTS);
	        this.listeners.initElementListening(window, DOM_EVENTS.concat(['load', 'beforeunload', 'pagehide', 'unload', 'message']));
	        this.listeners.addInternalEventBeforeListener(window, ['focus'], this._onFocus);
	        this.listeners.addInternalEventBeforeListener(window, ['focus', 'blur', 'change', 'focusin', 'focusout'], this._cancelInternalEvents);
	        this._preventInputNativeDialogs(window);
	        this.unload.attach(window);
	        this.message.attach(window);
	        this.timers.attach(window);
	        this.focusBlur.attach(window);
	        this.hover.attach(window);
	    };
	    EventSandbox.prototype.reattach = function (window) {
	        this.listeners.restartElementListening(window.document);
	        this.listeners.restartElementListening(window);
	    };
	    return EventSandbox;
	}(SandboxBase));

	function extend(target) {
	    var args = [];
	    for (var _i = 1; _i < arguments.length; _i++) {
	        args[_i - 1] = arguments[_i];
	    }
	    target = target || {};
	    var currentObj = null;
	    var copy = null;
	    if (typeof target !== 'object' && target.toString() !== '[object Function]')
	        target = {};
	    for (var i = 0; i < args.length; i++) {
	        currentObj = args[i];
	        if (currentObj !== null) {
	            for (var name_1 in currentObj) {
	                copy = currentObj[name_1];
	                if (target !== copy && copy !== void 0)
	                    target[name_1] = copy;
	            }
	        }
	    }
	    return target;
	}

	var TOUCH_EVENT_RADIUS = 25;
	var TOUCH_EVENT_FORCE = 0.5;
	var POINTER_EVENT_BUTTON = {
	    noButton: -1,
	    leftButton: 0,
	    rightButton: 2
	};
	// NOTE: (IE11 only) 'MouseEvent.detail' value always equals 0 for 'click' and 'dblclick' events.
	// Otherwise, MouseEvent.detail behave as described in specification - https://www.w3.org/TR/uievents/#events-mouseevents
	var DEFAULT_MOUSE_EVENT_DETAIL_PROP_VALUE = {
	    click: isIE ? 0 : 1,
	    dblclick: isIE ? 0 : 2,
	    mousedown: 1,
	    mouseup: 1
	};
	var KEY_EVENT_NAME_RE = /^key\w+$/;
	var MOUSE_EVENT_NAME_RE = /^((mouse\w+)|((dbl)?click)|(contextmenu)|(drag\w*)|(drop))$/;
	var TOUCH_EVENT_NAME_RE = /^touch\w+$/;
	var FOCUS_IN_OUT_EVENT_NAME_RE = /^focus(in|out)$/;
	// NOTE: initTextEvent method required INPUT_METHOD param in IE
	var DOM_INPUT_METHOD_KEYBOARD = 1;
	var MOUSE_TO_POINTER_EVENT_TYPE_MAP = {
	    mousedown: 'pointerdown',
	    mouseup: 'pointerup',
	    mousemove: 'pointermove',
	    mouseover: 'pointerover',
	    mouseenter: 'pointerenter',
	    mouseout: 'pointerout',
	};
	var TOUCH_TO_POINTER_EVENT_TYPE_MAP = {
	    touchstart: 'pointerdown',
	    touchend: 'pointerup',
	    touchmove: 'pointermove'
	};
	var DISABLEABLE_HTML_ELEMENT_TYPE_CHECKERS = [
	    isButtonElement,
	    isFieldSetElement,
	    isInputElement,
	    isOptGroupElement,
	    isOptionElement,
	    isSelectElement,
	    isTextAreaElement
	];
	var EventSimulator = /** @class */ (function () {
	    function EventSimulator() {
	        this.DISPATCHED_EVENT_FLAG = 'hammerhead|dispatched-event';
	        this.touchIdentifier = nativeMethods.dateNow();
	        // NOTE: (IE only) If event dispatching calls a native click function, we should clear the window.event
	        // property (which was set in the raiseDispatchEvent function). Otherwise, the window.event property will
	        // contain the dispatched event, not the native click event. We should restore the window.event value after
	        // the click is handled. (B237144)
	        this.savedWindowEvents = [];
	        this.savedNativeClickCount = 0;
	        this.browserWithNewEventsStyle = !isIE11;
	    }
	    EventSimulator._dispatchStorageEvent = function (el, args) {
	        var ev = nativeMethods.documentCreateEvent.call(document, 'StorageEvent');
	        ev.initStorageEvent('storage', args.canBubble, args.cancelable, args.key, args.oldValue, args.newValue, args.url, null);
	        nativeMethods.objectDefineProperty(ev, 'storageArea', {
	            get: function () { return args.storageArea; },
	            configurable: true
	        });
	        if (args.key === null) {
	            nativeMethods.objectDefineProperty(ev, 'key', {
	                get: function () { return null; },
	                configurable: true
	            });
	        }
	        return el.dispatchEvent(ev);
	    };
	    EventSimulator.prototype._dispatchTouchEvent = function (el, args) {
	        var ev = nativeMethods.documentCreateEvent.call(document, 'TouchEvent');
	        // HACK: A test for iOS by using initTouchEvent arguments.
	        // TODO: Replace it with a user agent analysis later.
	        if (ev.initTouchEvent) {
	            if (isIOS) {
	                ev.initTouchEvent(args.type, args.canBubble, args.cancelable, args.view, args.detail, args.screenX, args.screenY, args.pageX, args.pageY, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.touches, args.targetTouches, args.changedTouches, args.scale === void 0 ? 1.0 : args.scale, 
	                // NOTE: B237995
	                args.rotation === void 0 ? 0.0 : args.rotation);
	            }
	            else if (ev.initTouchEvent.length === 12) {
	                // NOTE: The Firefox.
	                ev.initTouchEvent(args.type, args.canBubble, args.cancelable, args.view, args.detail, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.touches, args.targetTouches, args.changedTouches);
	            }
	            else {
	                // NOTE: The default Android browser, Dolphin.
	                ev.initTouchEvent(args.touches, args.targetTouches, args.changedTouches, args.type, args.view, args.screenX, args.screenY, args.pageX - args.view.pageXOffset, args.pageY - args.view.pageYOffset, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey);
	            }
	        }
	        else {
	            ev = new nativeMethods.WindowTouchEvent(args.type, {
	                touches: args.touches,
	                targetTouches: args.targetTouches,
	                changedTouches: args.changedTouches,
	                ctrlKey: args.ctrlKey,
	                altKey: args.altKey,
	                shiftKey: args.shiftKey,
	                metaKey: args.metaKey,
	                bubbles: args.canBubble,
	                cancelable: args.cancelable,
	                cancelBubble: false,
	                defaultPrevented: false,
	                detail: args.detail,
	                view: args.view
	            });
	        }
	        if (hasPointerEvents)
	            this._dispatchPointerEvent(el, args);
	        return el.dispatchEvent(ev);
	    };
	    EventSimulator._getUIEventArgs = function (type, options) {
	        if (options === void 0) { options = {}; }
	        var detail = 'detail' in options ? options.detail : DEFAULT_MOUSE_EVENT_DETAIL_PROP_VALUE[type];
	        return {
	            type: type,
	            composed: options.composed,
	            canBubble: options.canBubble !== false,
	            cancelable: options.cancelable !== false,
	            view: options.view || window,
	            detail: detail || 0,
	            ctrlKey: options.ctrlKey || false,
	            altKey: options.altKey || false,
	            shiftKey: options.shiftKey || false,
	            metaKey: options.metaKey || false
	        };
	    };
	    EventSimulator._getStorageEventArgs = function (options) {
	        if (options === void 0) { options = {}; }
	        return extend(options, {
	            canBubble: options.canBubble !== false,
	            cancelable: options.cancelable !== false
	        });
	    };
	    EventSimulator._getMouseEventArgs = function (type, options) {
	        if (options === void 0) { options = {}; }
	        return extend(EventSimulator._getUIEventArgs(type, options), {
	            screenX: options.screenX || 0,
	            screenY: options.screenY || 0,
	            clientX: options.clientX || 0,
	            clientY: options.clientY || 0,
	            button: options.button === void 0 ? BUTTON.left : options.button,
	            buttons: options.buttons === void 0 ? BUTTONS_PARAMETER.leftButton : options.buttons,
	            relatedTarget: options.relatedTarget || null,
	            which: options.which
	        });
	    };
	    EventSimulator._getKeyEventArgs = function (type, options) {
	        var keyOptions = {
	            keyCode: options.keyCode || 0,
	            charCode: options.charCode || 0,
	            which: type === 'press' ? options.charCode : options.keyCode
	        };
	        if ('keyIdentifier' in options)
	            keyOptions.keyIdentifier = options.keyIdentifier;
	        if ('key' in options)
	            keyOptions.key = options.key;
	        return extend(EventSimulator._getUIEventArgs(type, options), keyOptions);
	    };
	    EventSimulator._getModifiersAsString = function (args) {
	        var modifiersString = '';
	        for (var modifier in KEYBOARD_MODIFIERS_PARAMETER) {
	            if (nativeMethods.objectHasOwnProperty.call(KEYBOARD_MODIFIERS_PARAMETER, modifier) && args[modifier])
	                modifiersString += KEYBOARD_MODIFIERS_PARAMETER[modifier] + ' ';
	        }
	        return modifiersString;
	    };
	    EventSimulator._prepareMouseEventOptions = function (options) {
	        if (options === void 0) { options = {}; }
	        var buttons = options.buttons === void 0 ? BUTTONS_PARAMETER.noButton : options.buttons;
	        var button = BUTTON.left;
	        options.buttons = buttons;
	        options.button = options.button || button;
	        if (isWebKit) {
	            options.which = WHICH_PARAMETER.leftButton;
	            if (options.buttons === BUTTONS_PARAMETER.noButton)
	                options.which = WHICH_PARAMETER.noButton;
	            if (options.buttons === BUTTONS_PARAMETER.rightButton)
	                options.which = WHICH_PARAMETER.rightButton;
	        }
	        return options;
	    };
	    EventSimulator.prototype._simulateEvent = function (el, event, userOptions, options) {
	        var _this = this;
	        if (options === void 0) { options = {}; }
	        var args = null;
	        var dispatch = null;
	        // NOTE: We don't simulate a click on links with modifiers (ctrl, shift, ctrl+shift, alt),
	        // because it causes the opening of a browser window or additional tabs in it or loading files.
	        var isClickOnLink = event === 'click' && isAnchorElement(el);
	        var opts = extend(userOptions ? {
	            clientX: userOptions.clientX,
	            clientY: userOptions.clientY,
	            screenX: userOptions.screenX,
	            screenY: userOptions.screenY,
	            altKey: isClickOnLink ? false : userOptions.alt,
	            shiftKey: isClickOnLink ? false : userOptions.shift,
	            ctrlKey: isClickOnLink ? false : userOptions.ctrl,
	            metaKey: userOptions.meta,
	            button: userOptions.button,
	            which: userOptions.which,
	            buttons: userOptions.buttons,
	            relatedTarget: userOptions.relatedTarget
	        } : {}, options);
	        opts.composed = isComposedEvent(event);
	        if (!opts.relatedTarget)
	            opts.relatedTarget = document.body;
	        if (event === 'storage') {
	            opts = extend(opts, userOptions);
	            args = EventSimulator._getStorageEventArgs(opts);
	            dispatch = EventSimulator._dispatchStorageEvent;
	        }
	        else if (MOUSE_EVENT_NAME_RE.test(event)) {
	            if (userOptions && userOptions.button !== void 0)
	                opts = extend(opts, { button: userOptions.button });
	            args = EventSimulator._getMouseEventArgs(event, opts);
	            // eslint-disable-next-line no-shadow
	            dispatch = function (el, args) { return _this._dispatchMouseRelatedEvents(el, args, userOptions); };
	        }
	        else if (KEY_EVENT_NAME_RE.test(event)) {
	            if (userOptions &&
	                (userOptions.keyCode !== void 0 || userOptions.charCode !== void 0)) {
	                opts = extend(opts, {
	                    key: userOptions.key || void 0,
	                    keyCode: userOptions.keyCode || 0,
	                    charCode: userOptions.charCode || 0
	                });
	                if ('keyIdentifier' in userOptions)
	                    opts.keyIdentifier = userOptions.keyIdentifier;
	                if ('key' in userOptions)
	                    opts.key = userOptions.key;
	            }
	            args = EventSimulator._getKeyEventArgs(event, opts);
	            // eslint-disable-next-line no-shadow
	            dispatch = function (el, args) { return _this._dispatchKeyEvent(el, args); };
	        }
	        else if (TOUCH_EVENT_NAME_RE.test(event)) {
	            args = this._getTouchEventArgs(event, extend(opts, { target: el }));
	            // eslint-disable-next-line no-shadow
	            dispatch = function (el, args) { return _this._dispatchTouchEvent(el, args); };
	        }
	        return dispatch(el, args);
	    };
	    EventSimulator.prototype._getTouchEventArgs = function (type, options) {
	        var _a, _b;
	        if (options === void 0) { options = {}; }
	        var args = extend(EventSimulator._getUIEventArgs(type, options), {
	            screenX: options.screenX || 0,
	            screenY: options.screenY || 0,
	            clientX: options.clientX || 0,
	            clientY: options.clientY || 0,
	            pageX: options.clientX || 0,
	            pageY: options.clientY || 0,
	            identifier: this._getTouchIdentifier(type)
	        });
	        if (nativeMethods.documentCreateTouch) {
	            if (isIOS)
	                args.touch = nativeMethods.documentCreateTouch.call(document, args.view, options.target, args.identifier, args.clientX, args.clientY, 0, 0);
	            else {
	                // NOTE: B237995
	                args.touch = nativeMethods.documentCreateTouch.call(document, args.view, options.target, args.identifier, args.pageX, args.pageY, args.screenX, args.screenY, args.clientX, args.clientY, null, null, args.rotation === void 0 ? 0 : args.rotation);
	            }
	        }
	        else {
	            args.touch = new nativeMethods.WindowTouch({
	                identifier: args.identifier,
	                target: options.target,
	                clientX: args.clientX,
	                clientY: args.clientY,
	                pageX: args.pageX,
	                pageY: args.pageY,
	                screenX: args.screenX,
	                screenY: args.screenY,
	                rotationAngle: 0,
	                radiusX: TOUCH_EVENT_RADIUS,
	                radiusY: TOUCH_EVENT_RADIUS,
	                force: TOUCH_EVENT_FORCE
	            });
	        }
	        args.changedTouches = [args.touch];
	        // NOTE: T170088
	        args.touches = args.type === 'touchend' ? [] : args.changedTouches;
	        if (nativeMethods.documentCreateTouchList) {
	            args.changedTouches = (_a = nativeMethods.documentCreateTouchList).call.apply(_a, __spreadArrays([document], args.changedTouches));
	            args.touches = (_b = nativeMethods.documentCreateTouchList).call.apply(_b, __spreadArrays([document], args.touches));
	        }
	        args.targetTouches = args.touches;
	        return args;
	    };
	    EventSimulator.prototype._getTouchIdentifier = function (type) {
	        // NOTE: A touch point is created when the 'touchstart' event occurs. When the point' is moved,
	        // its id must not be changed (T112153).
	        if (type === 'touchstart')
	            this.touchIdentifier++;
	        return this.touchIdentifier;
	    };
	    EventSimulator.prototype._raiseNativeClick = function (el, originClick) {
	        // NOTE: B254199
	        var iframe = isElementInIframe(el) && getIframeByElement(el);
	        var curWindow = iframe ? nativeMethods.contentWindowGetter.call(iframe) : window;
	        var prevWindowEvent = curWindow.event;
	        if (isIE11)
	            delete curWindow.event;
	        originClick.call(el);
	        // NOTE: Window.event becomes empty when the click event handler
	        // triggers the click event for a different element in IE11.(GH-226).
	        if (isIE11 && prevWindowEvent) {
	            nativeMethods.objectDefineProperty(curWindow, 'event', {
	                get: function () { return prevWindowEvent; },
	                configurable: true
	            });
	        }
	    };
	    EventSimulator.prototype._dispatchKeyEvent = function (el, args) {
	        var ev = null;
	        if (this.browserWithNewEventsStyle && nativeMethods.WindowKeyboardEvent) {
	            var eventArgs = {
	                bubbles: args.canBubble,
	                composed: args.composed,
	                cancelable: args.cancelable,
	                cancelBubble: false,
	                defaultPrevented: false,
	                view: args.view,
	                detail: args.detail,
	                ctrlKey: args.ctrlKey,
	                altKey: args.altKey,
	                shiftKey: args.shiftKey,
	                metaKey: args.metaKey,
	                keyCode: args.keyCode,
	                charCode: args.charCode,
	                which: args.which
	            };
	            if ('keyIdentifier' in args)
	                eventArgs.keyIdentifier = args.keyIdentifier;
	            if ('key' in args)
	                eventArgs.key = args.key;
	            ev = new nativeMethods.WindowKeyboardEvent(args.type, eventArgs);
	        }
	        else if (nativeMethods.documentCreateEvent) {
	            ev = nativeMethods.documentCreateEvent.call(document, 'KeyboardEvent');
	            ev.initKeyboardEvent(args.type, args.canBubble, args.cancelable, args.view, '', 0, EventSimulator._getModifiersAsString(args), false, '');
	        }
	        if (ev) {
	            // NOTE: the window.event.keyCode, window.event.charCode, window.event.which and
	            // window.event.key properties are not assigned after KeyboardEvent is created
	            nativeMethods.objectDefineProperty(ev, 'keyCode', {
	                configurable: true,
	                enumerable: true,
	                get: function () { return args.keyCode; }
	            });
	            nativeMethods.objectDefineProperty(ev, 'charCode', {
	                configurable: true,
	                enumerable: true,
	                get: function () { return args.charCode; }
	            });
	            nativeMethods.objectDefineProperty(ev, 'which', {
	                configurable: true,
	                enumerable: true,
	                get: function () { return args.which; }
	            });
	            if ('key' in args) {
	                nativeMethods.objectDefineProperty(ev, 'key', {
	                    configurable: true,
	                    enumerable: true,
	                    get: function () { return args.key; }
	                });
	            }
	            if ('keyIdentifier' in args) {
	                nativeMethods.objectDefineProperty(ev, 'keyIdentifier', {
	                    configurable: true,
	                    enumerable: true,
	                    get: function () { return args.keyIdentifier; }
	                });
	            }
	            var defaultPrevented_1 = false;
	            var returnValue_1 = true;
	            // NOTE: the dispatchEvent method does not return false in the case when preventDefault method
	            // was called for events that were created with the KeyboardEvent constructor
	            if (this.browserWithNewEventsStyle) {
	                ev.preventDefault = function () {
	                    defaultPrevented_1 = true;
	                    nativeMethods.preventDefault.call(ev);
	                    return false;
	                };
	            }
	            // NOTE: the dispatchEvent method does not return false when returnValue was set to false (only in MSEdge)
	            if (isMSEdge) {
	                nativeMethods.objectDefineProperty(ev, 'returnValue', {
	                    get: function () { return returnValue_1; },
	                    set: function (value) {
	                        if (value === false)
	                            ev.preventDefault();
	                        returnValue_1 = value;
	                    }
	                });
	            }
	            var res = this._raiseDispatchEvent(el, ev);
	            if (isMSEdge)
	                return returnValue_1 && !defaultPrevented_1;
	            else if (isIE)
	                return res;
	            return !defaultPrevented_1;
	        }
	        return null;
	    };
	    EventSimulator.prototype._getPointerEventTypeInfo = function (type) {
	        if (MOUSE_TO_POINTER_EVENT_TYPE_MAP[type]) {
	            return {
	                eventType: MOUSE_TO_POINTER_EVENT_TYPE_MAP[type],
	                pointerType: 'mouse'
	            };
	        }
	        if (isChrome && TOUCH_TO_POINTER_EVENT_TYPE_MAP[type]) {
	            return {
	                eventType: TOUCH_TO_POINTER_EVENT_TYPE_MAP[type],
	                pointerType: 'touch'
	            };
	        }
	        return null;
	    };
	    EventSimulator.prototype._dispatchPointerEvent = function (el, args) {
	        var pointerEventTypeInfo = this._getPointerEventTypeInfo(args.type);
	        if (!pointerEventTypeInfo)
	            return;
	        var eventType = pointerEventTypeInfo.eventType, pointerType = pointerEventTypeInfo.pointerType;
	        var pointEvent = null;
	        var elPosition = getOffsetPosition(el);
	        var elBorders = getBordersWidth(el);
	        var elClientPosition = offsetToClientCoords({
	            x: elPosition.left + elBorders.left,
	            y: elPosition.top + elBorders.top
	        });
	        var pointerArgs = extend({
	            width: 1,
	            height: 1,
	            pressure: 0,
	            tiltX: 0,
	            tiltY: 0,
	            // NOTE: This parameter must be "1" for “mouse”.
	            pointerId: 1,
	            pointerType: pointerType,
	            timeStamp: nativeMethods.dateNow(),
	            isPrimary: true
	        }, args);
	        pointerArgs.type = eventType;
	        pointerArgs.offsetX = args.clientX - elClientPosition.x;
	        pointerArgs.offsetY = args.clientY - elClientPosition.y;
	        if (args.type === 'mousemove' || args.type === 'mouseover' || args.type === 'mouseout')
	            pointerArgs.button = args.buttons === BUTTONS_PARAMETER.noButton ? POINTER_EVENT_BUTTON.noButton : pointerArgs.button;
	        if (isIE11) {
	            pointerArgs.rotation = 0;
	            pointEvent = nativeMethods.documentCreateEvent.call(document, 'PointerEvent');
	            // NOTE: We set the relatedTarget argument to null because IE has a memory leak.
	            pointEvent.initPointerEvent(pointerArgs.type, pointerArgs.canBubble, pointerArgs.cancelable, window, pointerArgs.detail, pointerArgs.screenX, pointerArgs.screenY, pointerArgs.clientX, pointerArgs.clientY, pointerArgs.ctrlKey, pointerArgs.altKey, pointerArgs.shiftKey, pointerArgs.metaKey, pointerArgs.button, null, pointerArgs.offsetX, pointerArgs.offsetY, pointerArgs.width, pointerArgs.height, pointerArgs.pressure, pointerArgs.rotation, pointerArgs.tiltX, pointerArgs.tiltY, pointerArgs.pointerId, pointerArgs.pointerType, pointerArgs.timeStamp, pointerArgs.isPrimary);
	            // NOTE: After dispatching the pointer event, it doesn't contain the 'target' and 'relatedTarget' properties.
	            nativeMethods.objectDefineProperty(pointEvent, 'target', {
	                get: function () { return el; },
	                configurable: true
	            });
	            nativeMethods.objectDefineProperty(pointEvent, 'relatedTarget', {
	                get: function () { return args.relatedTarget; },
	                configurable: true
	            });
	            nativeMethods.objectDefineProperty(pointEvent, 'buttons', {
	                get: function () { return args.buttons; }
	            });
	        }
	        else {
	            pointerArgs.bubbles = true;
	            pointerArgs.cancelable = true;
	            pointEvent = new nativeMethods.WindowPointerEvent(eventType, pointerArgs);
	        }
	        this._raiseDispatchEvent(el, pointEvent);
	    };
	    EventSimulator.prototype._elementCanBeDisabled = function (el) {
	        for (var _i = 0, DISABLEABLE_HTML_ELEMENT_TYPE_CHECKERS_1 = DISABLEABLE_HTML_ELEMENT_TYPE_CHECKERS; _i < DISABLEABLE_HTML_ELEMENT_TYPE_CHECKERS_1.length; _i++) {
	            var elementCanBeDisabled = DISABLEABLE_HTML_ELEMENT_TYPE_CHECKERS_1[_i];
	            if (elementCanBeDisabled(el)) {
	                return true;
	            }
	        }
	        return false;
	    };
	    EventSimulator.prototype._dispatchMouseRelatedEvents = function (el, args, userOptions) {
	        if (userOptions === void 0) { userOptions = {}; }
	        if (args.type !== 'mouseover' && args.type !== 'mouseenter' && shouldIgnoreMouseEventInsideIframe(el, args.clientX, args.clientY))
	            return true;
	        // NOTE: In IE, submit doesn't work if a click is simulated for some submit button's children (for example,
	        // img, B236676). In addition, if a test is being recorded in IE, the target of a click event is always a
	        // button, not a child, so the child does not receive the click event.
	        if (isIE) {
	            if (args.type === 'click' || args.type === 'mouseup' || args.type === 'mousedown') {
	                var elParent = nativeMethods.nodeParentNodeGetter.call(el);
	                var closestButton = closest(elParent, 'button');
	                if (elParent && closestButton) {
	                    if (nativeMethods.getAttribute.call(closestButton, 'type') === 'submit')
	                        el = closestButton;
	                }
	            }
	        }
	        if (hasPointerEvents)
	            this._dispatchPointerEvent(el, args);
	        return this._dispatchMouseEvent(el, args, userOptions);
	    };
	    EventSimulator.prototype._dispatchMouseEvent = function (el, args, _a) {
	        var _this = this;
	        var dataTransfer = _a.dataTransfer, timeStamp = _a.timeStamp;
	        var disabledParent = findParent(el, true, function (node) {
	            return _this._elementCanBeDisabled(node) && node.hasAttribute && nativeMethods.hasAttribute.call(node, 'disabled');
	        });
	        if (disabledParent)
	            return null;
	        var event = null;
	        if (this.browserWithNewEventsStyle && nativeMethods.WindowMouseEvent) {
	            event = new nativeMethods.WindowMouseEvent(args.type, {
	                bubbles: args.canBubble,
	                composed: args.composed,
	                cancelable: args.cancelable,
	                view: window,
	                detail: args.detail,
	                screenX: args.screenX,
	                screenY: args.screenY,
	                clientX: args.clientX,
	                clientY: args.clientY,
	                ctrlKey: args.ctrlKey,
	                altKey: args.altKey,
	                shiftKey: args.shiftKey,
	                metaKey: args.metaKey,
	                button: args.button,
	                buttons: args.buttons,
	                relatedTarget: args.relatedTarget
	            });
	        }
	        else {
	            event = nativeMethods.documentCreateEvent.call(document, 'MouseEvents');
	            event.initMouseEvent(args.type, args.canBubble, args.cancelable, window, args.detail, args.screenX, args.screenY, args.clientX, args.clientY, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.button, args.relatedTarget);
	            nativeMethods.objectDefineProperty(event, 'buttons', {
	                get: function () { return args.buttons; }
	            });
	        }
	        // NOTE: T188166 (act.hover triggers the mouseenter event with the "which" parameter set to 1).
	        if (args.which !== void 0 && isWebKit) {
	            nativeMethods.objectDefineProperty(event, 'which', {
	                get: function () { return args.which; }
	            });
	        }
	        if (timeStamp && !isIE) {
	            nativeMethods.objectDefineProperty(event, 'timeStamp', {
	                get: function () { return timeStamp; }
	            });
	        }
	        if (dataTransfer) {
	            nativeMethods.objectDefineProperty(event, 'dataTransfer', {
	                configurable: true,
	                enumerable: true,
	                get: function () { return dataTransfer; }
	            });
	        }
	        return this._raiseDispatchEvent(el, event);
	    };
	    EventSimulator.prototype._dispatchFocusEvent = function (el, name, relatedTarget) {
	        if (relatedTarget === void 0) { relatedTarget = null; }
	        var event = null;
	        var bubbles = FOCUS_IN_OUT_EVENT_NAME_RE.test(name);
	        if (this.browserWithNewEventsStyle && nativeMethods.WindowFocusEvent) {
	            event = new nativeMethods.WindowFocusEvent(name, {
	                bubbles: bubbles,
	                composed: isComposedEvent(name),
	                cancelable: false,
	                cancelBubble: false,
	                relatedTarget: relatedTarget,
	                defaultPrevented: false
	            });
	        }
	        else if (nativeMethods.documentCreateEvent) {
	            event = nativeMethods.documentCreateEvent.call(document, 'FocusEvent');
	            event.initFocusEvent(name, bubbles, true, null, 0, bubbles ? relatedTarget : null);
	        }
	        if (event) {
	            event[this.DISPATCHED_EVENT_FLAG] = true;
	            return this._raiseDispatchEvent(el, event);
	        }
	        return null;
	    };
	    EventSimulator.prototype._dispatchTextEvent = function (el, text) {
	        if (nativeMethods.WindowTextEvent && nativeMethods.documentCreateEvent) {
	            var event_1 = nativeMethods.documentCreateEvent.call(document, 'TextEvent');
	            var args = {
	                eventType: isIE11 ? 'textinput' : 'textInput',
	                bubbles: true,
	                cancelable: true,
	                view: window,
	                data: text,
	                inputMethod: DOM_INPUT_METHOD_KEYBOARD,
	                locale: navigator.language
	            };
	            event_1.initTextEvent(args.eventType, args.bubbles, args.cancelable, args.view, args.data, args.inputMethod, args.locale);
	            return this._raiseDispatchEvent(el, event_1);
	        }
	        return null;
	    };
	    EventSimulator.prototype._dispatchInputEvent = function (el, type, data) {
	        if (!nativeMethods.WindowInputEvent)
	            return this._dispatchEvent(el, type, true);
	        var args = {
	            bubbles: true,
	            composed: isComposedEvent(type),
	            cancelable: true,
	            view: window,
	            inputType: 'insertText'
	        };
	        if (data !== void 0)
	            args.data = data;
	        var event = new nativeMethods.WindowInputEvent(type, args);
	        return this._raiseDispatchEvent(el, event);
	    };
	    EventSimulator.prototype._dispatchEvent = function (el, name, shouldBubble, flag) {
	        var ev = null;
	        if (nativeMethods.documentCreateEvent) {
	            ev = nativeMethods.documentCreateEvent.call(document, 'Events');
	            ev.initEvent(name, shouldBubble, true);
	        }
	        if (ev) {
	            if (flag)
	                ev[flag] = true;
	            return this._raiseDispatchEvent(el, ev);
	        }
	        return null;
	    };
	    EventSimulator.prototype._raiseDispatchEvent = function (el, ev) {
	        var iframe = isElementInIframe(el) && getIframeByElement(el);
	        var curWindow = iframe ? nativeMethods.contentWindowGetter.call(iframe) : window;
	        if (isIE11 && iframe && curWindow) {
	            // NOTE: In IE, when we raise an event by using the dispatchEvent function, the window.event object is null.
	            // If a real event happens, there is a window.event object, but it is not identical with the first argument
	            // of the event handler. The window.Event object is identical with the object that is created when we raise
	            // the event by using  the fireEvent function. So, when we raise the event by using the dispatchEvent function,
	            // we need to set the window.event object manually. An exception for IE11: The window.event object is not null
	            // and it’s the same as in the event handler (only in window.top.event). Also, in iE11, window.event doesn’t
	            // have the returnValue property, so it’s impossible to prevent the event by assigning window.event.returnValue
	            // to false.
	            // NOTE: In IE11, iframe's window.event object is null. We need to set
	            // iframe's window.event object manually by using window.event (B254199).
	            nativeMethods.objectDefineProperty(curWindow, 'event', {
	                get: function () { return window.event; },
	                configurable: true
	            });
	        }
	        var res = el.dispatchEvent(ev);
	        // NOTE: GH-226
	        if (isIE11 && curWindow)
	            delete curWindow.event;
	        return res;
	    };
	    /* NOTE: options = {
	     [clientX: integer,]
	     [clientY: integer,]
	     [alt: true|false,]
	     [ctrl: true|false,]
	     [shift: true|false,]
	     [meta: true|false,]
	     [button: Util.BUTTON]
	    } */
	    EventSimulator.prototype.click = function (el, options) {
	        return this._simulateEvent(el, 'click', options, {
	            button: BUTTON.left,
	            buttons: BUTTONS_PARAMETER.noButton
	        });
	    };
	    EventSimulator.prototype.nativeClick = function (el, originClick) {
	        this._raiseNativeClick(el, originClick);
	    };
	    EventSimulator.prototype.dblclick = function (el, options) {
	        return this._simulateEvent(el, 'dblclick', options, {
	            button: BUTTON.left,
	            buttons: BUTTONS_PARAMETER.noButton
	        });
	    };
	    EventSimulator.prototype.rightclick = function (el, options) {
	        return this._simulateEvent(el, 'click', options, {
	            button: BUTTON.right,
	            buttons: BUTTONS_PARAMETER.rightButton
	        });
	    };
	    EventSimulator.prototype.contextmenu = function (el, options) {
	        return this._simulateEvent(el, 'contextmenu', options, {
	            button: BUTTON.right,
	            buttons: BUTTONS_PARAMETER.noButton
	        });
	    };
	    EventSimulator.prototype.mousedown = function (el, options) {
	        if (options === void 0) { options = {}; }
	        var button = options.button === void 0 ? BUTTON.left : options.button;
	        var buttons = button === BUTTON.left ? BUTTONS_PARAMETER.leftButton : BUTTONS_PARAMETER.rightButton;
	        options.button = button;
	        options.buttons = options.buttons === void 0 ? buttons : options.buttons;
	        return this._simulateEvent(el, 'mousedown', options);
	    };
	    EventSimulator.prototype.mouseup = function (el, options) {
	        if (options === void 0) { options = {}; }
	        var button = options.button === void 0 ? BUTTON.left : options.button;
	        return this._simulateEvent(el, 'mouseup', options, {
	            button: button,
	            buttons: BUTTONS_PARAMETER.noButton
	        });
	    };
	    EventSimulator.prototype.mouseover = function (el, options) {
	        options = EventSimulator._prepareMouseEventOptions(options);
	        return this._simulateEvent(el, 'mouseover', options);
	    };
	    EventSimulator.prototype.mousemove = function (el, options) {
	        options = EventSimulator._prepareMouseEventOptions(options);
	        return this._simulateEvent(el, 'mousemove', options, { cancelable: false });
	    };
	    EventSimulator.prototype.mouseout = function (el, options) {
	        options = EventSimulator._prepareMouseEventOptions(options);
	        return this._simulateEvent(el, 'mouseout', options);
	    };
	    EventSimulator.prototype.mouseenter = function (el, options) {
	        options = EventSimulator._prepareMouseEventOptions(options);
	        return this._simulateEvent(el, 'mouseenter', options, { canBubble: false });
	    };
	    EventSimulator.prototype.mouseleave = function (el, options) {
	        options = EventSimulator._prepareMouseEventOptions(options);
	        return this._simulateEvent(el, 'mouseleave', options, { canBubble: false });
	    };
	    // NOTE: Keyboard events.
	    EventSimulator.prototype.keypress = function (el, options) {
	        return this._simulateEvent(el, 'keypress', options);
	    };
	    EventSimulator.prototype.keyup = function (el, options) {
	        return this._simulateEvent(el, 'keyup', options);
	    };
	    EventSimulator.prototype.keydown = function (el, options) {
	        return this._simulateEvent(el, 'keydown', options);
	    };
	    // NOTE: Control events.
	    // NOTE: "focus", "blur" and "selectionchange" shouldn't bubble (T229732),
	    // but "input", "change" and "submit" should do it (GH-318).
	    EventSimulator.prototype.blur = function (el, relatedTarget) {
	        return this._dispatchFocusEvent(el, 'blur', relatedTarget);
	    };
	    EventSimulator.prototype.focus = function (el, relatedTarget) {
	        return this._dispatchFocusEvent(el, 'focus', relatedTarget);
	    };
	    EventSimulator.prototype.focusin = function (el, relatedTarget) {
	        return this._dispatchFocusEvent(el, 'focusin', relatedTarget);
	    };
	    EventSimulator.prototype.focusout = function (el, relatedTarget) {
	        return this._dispatchFocusEvent(el, 'focusout', relatedTarget);
	    };
	    EventSimulator.prototype.storage = function (window, options) {
	        return this._simulateEvent(window, 'storage', options);
	    };
	    EventSimulator.prototype.change = function (el) {
	        return this._dispatchEvent(el, 'change', true, this.DISPATCHED_EVENT_FLAG);
	    };
	    EventSimulator.prototype.textInput = function (el, data) {
	        return this._dispatchTextEvent(el, data);
	    };
	    EventSimulator.prototype.beforeInput = function (el, data) {
	        return this._dispatchInputEvent(el, 'beforeinput', data);
	    };
	    EventSimulator.prototype.input = function (el, data) {
	        return this._dispatchInputEvent(el, 'input', data);
	    };
	    EventSimulator.prototype.submit = function (el) {
	        return this._dispatchEvent(el, 'submit', true);
	    };
	    EventSimulator.prototype.selectionchange = function (el) {
	        return this._dispatchEvent(el, 'selectionchange', false);
	    };
	    // NOTE: Touch events.
	    EventSimulator.prototype.touchstart = function (el, options) {
	        return this._simulateEvent(el, 'touchstart', options);
	    };
	    EventSimulator.prototype.touchend = function (el, options) {
	        return this._simulateEvent(el, 'touchend', options);
	    };
	    EventSimulator.prototype.touchmove = function (el, options) {
	        return this._simulateEvent(el, 'touchmove', options);
	    };
	    // NOTE: drag and drop
	    EventSimulator.prototype.dragstart = function (el, options) {
	        return this._simulateEvent(el, 'dragstart', options);
	    };
	    EventSimulator.prototype.drag = function (el, options) {
	        return this._simulateEvent(el, 'drag', options);
	    };
	    EventSimulator.prototype.dragenter = function (el, options) {
	        return this._simulateEvent(el, 'dragenter', options);
	    };
	    EventSimulator.prototype.dragover = function (el, options) {
	        return this._simulateEvent(el, 'dragover', options);
	    };
	    EventSimulator.prototype.dragleave = function (el, options) {
	        return this._simulateEvent(el, 'dragleave', options);
	    };
	    EventSimulator.prototype.drop = function (el, options) {
	        return this._simulateEvent(el, 'drop', options);
	    };
	    EventSimulator.prototype.dragend = function (el, options) {
	        return this._simulateEvent(el, 'dragend', options);
	    };
	    EventSimulator.prototype.isSavedWindowsEventsExists = function () {
	        return this.savedWindowEvents && this.savedWindowEvents.length;
	    };
	    return EventSimulator;
	}());

	var MessageType;
	(function (MessageType) {
	    MessageType["Service"] = "hammerhead|service-msg";
	    MessageType["User"] = "hammerhead|user-msg";
	})(MessageType || (MessageType = {}));
	var MessageSandbox = /** @class */ (function (_super) {
	    __extends(MessageSandbox, _super);
	    function MessageSandbox(_listeners, _unloadSandbox) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this._unloadSandbox = _unloadSandbox;
	        _this.PING_DELAY = 200;
	        _this.PING_IFRAME_TIMEOUT = 7000;
	        _this.PING_IFRAME_MIN_TIMEOUT = 100;
	        _this.SERVICE_MSG_RECEIVED_EVENT = 'hammerhead|event|service-msg-received';
	        _this.RECEIVE_MSG_FN = 'hammerhead|receive-msg-function';
	        // NOTE: The window.top property may be changed after an iframe is removed from DOM in IE, so we save it.
	        _this.topWindow = null;
	        _this.window = null;
	        _this.storedOnMessageHandler = null;
	        _this.isWindowUnloaded = false;
	        _this.iframeInternalMsgQueue = [];
	        return _this;
	    }
	    MessageSandbox._getMessageData = function (e) {
	        var rawData = isMessageEvent(e) ? nativeMethods.messageEventDataGetter.call(e) : e.data;
	        return typeof rawData === 'string' ? parse$1(rawData) : rawData;
	    };
	    // NOTE: some window may be unavailable for the sending message, for example, if it was removed.
	    // In some browsers, window.postMessage is equal null, but other throw exception by property access.
	    MessageSandbox._isWindowAvailable = function (window) {
	        try {
	            return !!window.postMessage;
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    // @ts-ignore
	    MessageSandbox.prototype._onMessage = function (e) {
	        var data = MessageSandbox._getMessageData(e);
	        if (data.type === MessageType.Service && e.source) {
	            if (this.pingCmd && data.message.cmd === this.pingCmd && data.message.isPingResponse) {
	                this.pingCallback();
	                this.pingCallback = null;
	                this.pingCmd = null;
	            }
	            else
	                this.emit(this.SERVICE_MSG_RECEIVED_EVENT, { message: data.message, source: e.source, ports: e.ports });
	        }
	    };
	    MessageSandbox.prototype._onWindowMessage = function (e, originListener) {
	        var data = MessageSandbox._getMessageData(e);
	        if (data.type !== MessageType.Service) {
	            var originUrl = get$2();
	            if (data.targetUrl === '*' || sameOriginCheck(originUrl, data.targetUrl))
	                return callEventListener(this.window, originListener, e);
	        }
	        return null;
	    };
	    MessageSandbox._wrapMessage = function (type, message, targetUrl) {
	        var parsedDest = getParsed();
	        var originUrl = formatUrl({
	            /*eslint-disable no-restricted-properties*/
	            protocol: parsedDest.protocol,
	            host: parsedDest.host
	            /*eslint-enable no-restricted-properties*/
	        });
	        return { message: message, originUrl: originUrl, targetUrl: targetUrl, type: type };
	    };
	    MessageSandbox.prototype._removeInternalMsgFromQueue = function (sendFunc) {
	        for (var index = 0, length_1 = this.iframeInternalMsgQueue.length; index < length_1; index++) {
	            if (this.iframeInternalMsgQueue[index].sendFunc === sendFunc) {
	                this.iframeInternalMsgQueue.splice(index, 1);
	                return true;
	            }
	        }
	        return false;
	    };
	    MessageSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        // NOTE: The window.top property may be changed after an iframe is removed from DOM in IE, so we save it.
	        this.topWindow = window.top;
	        this.isWindowUnloaded = false;
	        this._unloadSandbox.on(this._unloadSandbox.UNLOAD_EVENT, function () {
	            _this.isWindowUnloaded = true;
	            while (_this.iframeInternalMsgQueue.length) {
	                var msgInfo = _this.iframeInternalMsgQueue[0];
	                nativeMethods.clearTimeout.call(_this.window, msgInfo.timeoutId);
	                msgInfo.sendFunc();
	            }
	        });
	        var onMessageHandler = function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            return fastApply(_this, '_onMessage', args);
	        };
	        var onWindowMessageHandler = function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            return fastApply(_this, '_onWindowMessage', args);
	        };
	        this._listeners.addInternalEventBeforeListener(window, ['message'], onMessageHandler);
	        this._listeners.setEventListenerWrapper(window, ['message'], onWindowMessageHandler);
	        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
	        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
	        nativeMethods.objectDefineProperty(window, this.RECEIVE_MSG_FN, {
	            value: onMessageHandler,
	            configurable: true
	        });
	        // @ts-ignore
	        overrideDescriptor(window.MessageEvent.prototype, 'data', {
	            getter: function () {
	                var target = this.target;
	                var data = nativeMethods.messageEventDataGetter.call(this);
	                if (data && data.type !== MessageType.Service && isWindow(target))
	                    return data.message;
	                return data;
	            }
	        });
	        // @ts-ignore
	        var eventPropsOwner = nativeMethods.isEventPropsLocatedInProto ? window.Window.prototype : window;
	        overrideDescriptor(eventPropsOwner, 'onmessage', {
	            getter: function () { return _this.storedOnMessageHandler; },
	            setter: function (handler) {
	                _this.storedOnMessageHandler = typeof handler === 'function' ? handler : null;
	                nativeMethods.winOnMessageSetter.call(window, _this.storedOnMessageHandler
	                    ? function (e) { return _this._onWindowMessage(e, handler); }
	                    : null);
	            }
	        });
	    };
	    MessageSandbox.prototype.postMessage = function (contentWindow, args) {
	        var targetUrl = args[1] || getOriginHeader();
	        // NOTE: Here, we pass all messages as "no preference" ("*").
	        // We do an origin check in "_onWindowMessage" to access the target origin.
	        args[1] = '*';
	        args[0] = MessageSandbox._wrapMessage(MessageType.User, args[0], targetUrl);
	        return fastApply(contentWindow, 'postMessage', args);
	    };
	    MessageSandbox.prototype.sendServiceMsg = function (msg, targetWindow, ports) {
	        var _this = this;
	        var message = MessageSandbox._wrapMessage(MessageType.Service, msg);
	        var canSendDirectly = !isCrossDomainWindows(targetWindow, this.window) && !!targetWindow[this.RECEIVE_MSG_FN];
	        if (!canSendDirectly)
	            return MessageSandbox._isWindowAvailable(targetWindow) && targetWindow.postMessage(message, '*', ports);
	        var sendFunc = function (force) {
	            // NOTE: In IE, this function is called on the timeout despite the fact that the timer has been cleared
	            // in the unload event handler, so we check whether the function is in the queue
	            if (force || _this._removeInternalMsgFromQueue(sendFunc)) {
	                // NOTE: The 'sendFunc' function may be called on timeout, so we must call 'canSendDirectly' again,
	                // because the iframe could become cross-domain in the meantime. Unfortunately, Chrome hangs when
	                // trying to call the 'isCrossDomainWindows' function, so we have to wrap it in 'try/catch'.
	                try {
	                    targetWindow[_this.RECEIVE_MSG_FN]({
	                        // NOTE: Cloning a message to prevent this modification.
	                        data: parse$1(stringify(message)),
	                        source: _this.window,
	                        ports: ports
	                    });
	                }
	                // eslint-disable-next-line no-empty
	                catch (e) {
	                }
	            }
	        };
	        if (!this.isWindowUnloaded) {
	            // NOTE: Imitation of a delay for the postMessage method.
	            // We use the same-domain top window
	            // so that the function called by setTimeout is executed after removing the iframe
	            var topSameDomainWindow = getTopSameDomainWindow(this.window);
	            var timeoutId = nativeMethods.setTimeout.call(topSameDomainWindow, sendFunc, 10);
	            this.iframeInternalMsgQueue.push({ timeoutId: timeoutId, sendFunc: sendFunc });
	        }
	        else
	            sendFunc(true);
	        return null;
	    };
	    // NOTE: This code is used only in legacy API.
	    MessageSandbox.prototype.pingIframe = function (targetIframe, pingMessageCommand, shortWaiting) {
	        var _this = this;
	        return new pinkie(function (resolve, reject) {
	            var pingInterval = null;
	            var pingTimeout = null;
	            var targetWindow = null;
	            var sendPingRequest = function () {
	                targetWindow = nativeMethods.contentWindowGetter.call(targetIframe);
	                if (targetWindow) {
	                    _this.sendServiceMsg({
	                        cmd: _this.pingCmd,
	                        isPingRequest: true
	                    }, targetWindow);
	                }
	            };
	            var cleanTimeouts = function () {
	                nativeMethods.clearInterval.call(_this.window, pingInterval);
	                nativeMethods.clearTimeout.call(_this.window, pingTimeout);
	                _this.pingCallback = null;
	                _this.pingCmd = null;
	                pingInterval = null;
	                pingTimeout = null;
	            };
	            pingTimeout = nativeMethods.setTimeout.call(_this.window, function () {
	                cleanTimeouts();
	                reject();
	            }, shortWaiting ? _this.PING_IFRAME_MIN_TIMEOUT : _this.PING_IFRAME_TIMEOUT);
	            _this.pingCallback = function () {
	                cleanTimeouts();
	                resolve();
	            };
	            _this.pingCmd = pingMessageCommand;
	            sendPingRequest();
	            pingInterval = nativeMethods.setInterval.call(_this.window, sendPingRequest, _this.PING_DELAY);
	        });
	    };
	    return MessageSandbox;
	}(SandboxBase));

	var NodeMutation = /** @class */ (function (_super) {
	    __extends(NodeMutation, _super);
	    function NodeMutation() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this.BEFORE_DOCUMENT_CLEANED_EVENT = 'hammerhead|event|before-document-cleaned';
	        _this.DOCUMENT_CLEANED_EVENT = 'hammerhead|event|document-cleaned';
	        _this.DOCUMENT_CLOSED_EVENT = 'hammerhead|event|document-closed';
	        _this.BODY_CONTENT_CHANGED_EVENT = 'hammerhead|event|body-content-changed';
	        _this.BODY_CREATED_EVENT = 'hammerhead|event|body-created';
	        _this.IFRAME_ADDED_TO_DOM_EVENT = 'hammerhead|event|iframe-added-to-dom';
	        return _this;
	    }
	    NodeMutation.prototype.onIframeAddedToDOM = function (iframe) {
	        this.emit(this.IFRAME_ADDED_TO_DOM_EVENT, iframe);
	    };
	    NodeMutation.prototype.onBeforeDocumentCleaned = function (document) {
	        this.emit(this.BEFORE_DOCUMENT_CLEANED_EVENT, document);
	    };
	    NodeMutation.prototype.onDocumentCleaned = function (window, document) {
	        this.emit(this.DOCUMENT_CLEANED_EVENT, { window: window, document: document });
	    };
	    NodeMutation.prototype.onDocumentClosed = function (document) {
	        this.emit(this.DOCUMENT_CLOSED_EVENT, document);
	    };
	    NodeMutation.prototype.onBodyContentChanged = function (element) {
	        this.emit(this.BODY_CONTENT_CHANGED_EVENT, element);
	    };
	    NodeMutation.prototype.onBodyCreated = function (body) {
	        this.emit(this.BODY_CREATED_EVENT, body);
	    };
	    return NodeMutation;
	}(EventEmitter));

	// NOTE: When you call the focus and blur function for some elements in IE, the event handlers  must be raised
	// asynchronously, but before executing functions that are called by using the window.setTimeout function. So,
	// we need to raise the handlers with a timeout, but do it before calling other asynchronous functions.
	var TimersSandbox = /** @class */ (function (_super) {
	    __extends(TimersSandbox, _super);
	    function TimersSandbox() {
	        var _this = _super.call(this) || this;
	        _this.timeouts = [];
	        _this.deferredFunctions = [];
	        _this.setTimeout = nativeMethods.setTimeout;
	        return _this;
	    }
	    TimersSandbox.prototype._wrapTimeoutFunctionsArguments = function (args) {
	        var _this = this;
	        var isScriptFirstArg = typeof args[0] === 'string';
	        var func = !isScriptFirstArg ? args[0] : null;
	        var script = isScriptFirstArg ? processScript(args[0], false) : null;
	        if (isIE && version < 12) {
	            var timersSandbox_1 = this;
	            var fnToRun_1 = isScriptFirstArg ? function () {
	                // NOTE: We are switching eval to the global context with this assignment.
	                // Unlike eval, the setTimeout/setInterval functions always work in the global context.
	                var ev = _this.window.eval;
	                return ev(script);
	            } : func;
	            args[0] = function () {
	                return timersSandbox_1._callDeferredFunction(fnToRun_1, arguments);
	            };
	        }
	        else if (isScriptFirstArg)
	            args[0] = script;
	        return args;
	    };
	    TimersSandbox.prototype._callDeferredFunction = function (fn, args) {
	        if (this.timeouts.length) {
	            var curTimeouts = [];
	            var curHandlers = [];
	            for (var i = 0; i < this.timeouts.length; i++) {
	                curTimeouts.push(this.timeouts[i]);
	                curHandlers.push(this.deferredFunctions[i]);
	            }
	            this.timeouts = [];
	            this.deferredFunctions = [];
	            for (var j = 0; j < curTimeouts.length; j++) {
	                nativeMethods.clearInterval.call(this.window, curTimeouts[j]);
	                curHandlers[j]();
	            }
	            // NOTE: Handlers can create new deferred functions.
	            return this._callDeferredFunction(fn, args);
	        }
	        return fn.apply(this.window, args);
	    };
	    TimersSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        var timersSandbox = this;
	        overrideFunction(window, 'setTimeout', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            return nativeMethods.setTimeout.apply(window, timersSandbox._wrapTimeoutFunctionsArguments(args));
	        });
	        overrideFunction(window, 'setInterval', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            return nativeMethods.setInterval.apply(window, timersSandbox._wrapTimeoutFunctionsArguments(args));
	        });
	        // NOTE: We are saving the setTimeout wrapper for internal use in case the page-script replaces
	        // it with an invalid value.
	        this.setTimeout = window.setTimeout;
	    };
	    TimersSandbox.prototype.deferFunction = function (fn) {
	        var _this = this;
	        var deferredFunction = function () {
	            fn();
	            for (var i = 0; i < _this.deferredFunctions.length; i++) {
	                if (_this.deferredFunctions[i] === deferredFunction) {
	                    _this.deferredFunctions.splice(i, 1);
	                    _this.timeouts.splice(i, 1);
	                    break;
	                }
	            }
	        };
	        this.deferredFunctions.push(deferredFunction);
	        this.timeouts.push(nativeMethods.setTimeout.call(window, deferredFunction, 0));
	    };
	    return TimersSandbox;
	}(SandboxBase));

	function createPropertyDesc(descBase) {
	    descBase.configurable = true;
	    descBase.enumerable = true;
	    return descBase;
	}

	var UnloadSandbox = /** @class */ (function (_super) {
	    __extends(UnloadSandbox, _super);
	    function UnloadSandbox(_listeners) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this.BEFORE_UNLOAD_EVENT = 'hammerhead|event|before-unload';
	        _this.BEFORE_BEFORE_UNLOAD_EVENT = 'hammerhead|event|before-before-unload';
	        _this.UNLOAD_EVENT = 'hammerhead|event|unload';
	        _this.beforeUnloadProperties = {
	            storedReturnValue: '',
	            prevented: false,
	            storedHandler: null,
	            nativeEventName: UnloadSandbox._getBeforeUnloadEventName(),
	            eventName: _this.BEFORE_UNLOAD_EVENT,
	            eventPropSetter: UnloadSandbox._getBeforeUnloadPropSetter()
	        };
	        _this.unloadProperties = {
	            storedReturnValue: '',
	            prevented: false,
	            storedHandler: null,
	            nativeEventName: 'unload',
	            eventName: _this.UNLOAD_EVENT,
	            eventPropSetter: nativeMethods.winOnUnloadSetter
	        };
	        return _this;
	    }
	    UnloadSandbox._getBeforeUnloadEventName = function () {
	        // NOTE: the ios devices do not support beforeunload event
	        // https://developer.apple.com/library/ios/documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html#//apple_ref/doc/uid/TP40006511-SW5
	        return isIOS ? 'pagehide' : 'beforeunload';
	    };
	    UnloadSandbox._getBeforeUnloadPropSetter = function () {
	        // NOTE: the ios devices do not support beforeunload event
	        // https://developer.apple.com/library/ios/documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html#//apple_ref/doc/uid/TP40006511-SW5
	        return isIOS ? nativeMethods.winOnPageHideSetter : nativeMethods.winOnBeforeUnloadSetter;
	    };
	    // NOTE: This handler has to be called after others.
	    UnloadSandbox.prototype._emitEvent = function (eventProperties) {
	        this.emit(eventProperties.eventName, {
	            returnValue: eventProperties.storedReturnValue,
	            prevented: eventProperties.prevented
	        });
	    };
	    UnloadSandbox.prototype._createEventHandler = function (eventProperties) {
	        return function (e, originListener) {
	            // NOTE: Overriding the returnValue property to prevent a native dialog.
	            nativeMethods.objectDefineProperty(e, 'returnValue', createPropertyDesc({
	                get: function () { return eventProperties.storedReturnValue; },
	                set: function (value) {
	                    // NOTE: In all browsers, if the property is set to any value, unload is prevented. In FireFox,
	                    // only if a value is set to an empty string, the unload operation is prevented.
	                    eventProperties.storedReturnValue = value;
	                    eventProperties.prevented = isFirefox ? value !== '' : true;
	                }
	            }));
	            nativeMethods.objectDefineProperty(e, 'preventDefault', createPropertyDesc({
	                get: function () { return function () {
	                    eventProperties.prevented = true;
	                    return true;
	                }; },
	                set: function () { return void 0; }
	            }));
	            var res = originListener(e);
	            if (res !== void 0) {
	                eventProperties.storedReturnValue = res;
	                eventProperties.prevented = true;
	            }
	        };
	    };
	    UnloadSandbox.prototype._reattachListener = function (eventProperties) {
	        var nativeAddEventListener = nativeMethods.windowAddEventListener || nativeMethods.addEventListener;
	        var nativeRemoveEventListener = nativeMethods.windowRemoveEventListener || nativeMethods.removeEventListener;
	        // NOTE: reattach the Listener, it'll be the last in the queue.
	        nativeRemoveEventListener.call(this.window, eventProperties.nativeEventName, this);
	        nativeAddEventListener.call(this.window, eventProperties.nativeEventName, this);
	    };
	    UnloadSandbox.prototype._setEventListenerWrapper = function (eventProperties) {
	        this._listeners.setEventListenerWrapper(window, [eventProperties.nativeEventName], this._createEventHandler(eventProperties));
	    };
	    UnloadSandbox.prototype._addEventListener = function (eventProperties) {
	        var _this = this;
	        var nativeAddEventListener = nativeMethods.windowAddEventListener || nativeMethods.addEventListener;
	        nativeAddEventListener.call(window, eventProperties.nativeEventName, this);
	        this._listeners.on(this._listeners.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
	            if (e.el === window && e.eventType === eventProperties.nativeEventName)
	                _this._reattachListener(eventProperties);
	        });
	    };
	    UnloadSandbox.prototype._overrideEventDescriptor = function (eventProperties) {
	        var _this = this;
	        // @ts-ignore
	        var eventPropsOwner = nativeMethods.isEventPropsLocatedInProto ? window.Window.prototype : window;
	        // @ts-ignore
	        overrideDescriptor(eventPropsOwner, 'on' + eventProperties.nativeEventName, {
	            getter: function () { return eventProperties.storedHandler; },
	            setter: function (handler) { return _this.setOnEvent(eventProperties, window, handler); }
	        });
	    };
	    UnloadSandbox.prototype._attachEvent = function (eventProperties) {
	        this._setEventListenerWrapper(eventProperties);
	        this._addEventListener(eventProperties);
	        this._overrideEventDescriptor(eventProperties);
	    };
	    UnloadSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._attachEvent(this.beforeUnloadProperties);
	        this._attachEvent(this.unloadProperties);
	        this._listeners.addInternalEventBeforeListener(window, [this.beforeUnloadProperties.nativeEventName], function () { return _this.emit(_this.BEFORE_BEFORE_UNLOAD_EVENT); });
	    };
	    UnloadSandbox.prototype.setOnEvent = function (eventProperties, window, handler) {
	        var _this = this;
	        if (typeof handler === 'function') {
	            eventProperties.storedHandler = handler;
	            eventProperties.eventPropSetter.call(window, function (e) { return _this._createEventHandler(eventProperties)(e, handler); });
	            this._reattachListener(eventProperties);
	        }
	        else {
	            eventProperties.storedHandler = null;
	            eventProperties.eventPropSetter.call(window, null);
	        }
	    };
	    UnloadSandbox.prototype.handleEvent = function (e) {
	        if (e.type === this.beforeUnloadProperties.nativeEventName)
	            this._emitEvent(this.beforeUnloadProperties);
	        else if (e.type === this.unloadProperties.nativeEventName)
	            this._emitEvent(this.unloadProperties);
	    };
	    return UnloadSandbox;
	}(SandboxBase));

	var SandboxBaseWithDelayedSettings = /** @class */ (function (_super) {
	    __extends(SandboxBaseWithDelayedSettings, _super);
	    function SandboxBaseWithDelayedSettings(_waitHammerheadSettings) {
	        var _this = _super.call(this) || this;
	        _this._waitHammerheadSettings = _waitHammerheadSettings;
	        if (_waitHammerheadSettings) {
	            _waitHammerheadSettings.then(function () {
	                _this._waitHammerheadSettings = null;
	            });
	        }
	        return _this;
	    }
	    SandboxBaseWithDelayedSettings.prototype.gettingSettingInProgress = function () {
	        return !!this._waitHammerheadSettings;
	    };
	    SandboxBaseWithDelayedSettings.prototype.delayUntilGetSettings = function (action) {
	        return this._waitHammerheadSettings.then(action);
	    };
	    return SandboxBaseWithDelayedSettings;
	}(SandboxBase));

	// -------------------------------------------------------------
	var AUTHENTICATE_PREFIX = '~~~TestCafe added this prefix to hide the authentication dialog box~~~';
	var AUTHORIZATION_PREFIX = '~~~TestCafe added this prefix to control the authorization flow~~~';
	function addAuthenticatePrefix(value) {
	    return AUTHENTICATE_PREFIX + value;
	}
	function hasAuthenticatePrefix(value) {
	    return value.indexOf(AUTHENTICATE_PREFIX) > -1;
	}
	function removeAuthenticatePrefix(value) {
	    return value.replace(AUTHENTICATE_PREFIX, '');
	}
	function isAuthenticateHeader(headerName) {
	    var headerNameStr = String(headerName).toLowerCase();
	    return headerNameStr === BUILTIN_HEADERS.wwwAuthenticate || headerNameStr === BUILTIN_HEADERS.proxyAuthenticate;
	}
	function addAuthorizationPrefix(value) {
	    return AUTHORIZATION_PREFIX + value;
	}
	function hasAuthorizationPrefix(value) {
	    return value.indexOf(AUTHORIZATION_PREFIX) > -1;
	}
	function removeAuthorizationPrefix(value) {
	    return value.replace(AUTHORIZATION_PREFIX, '');
	}
	function isAuthorizationHeader(headerName) {
	    var headerNameStr = String(headerName).toLowerCase();
	    return headerNameStr === BUILTIN_HEADERS.authorization || headerNameStr === BUILTIN_HEADERS.proxyAuthorization;
	}

	var sharedHeadersUtils = /*#__PURE__*/Object.freeze({
		__proto__: null,
		addAuthenticatePrefix: addAuthenticatePrefix,
		hasAuthenticatePrefix: hasAuthenticatePrefix,
		removeAuthenticatePrefix: removeAuthenticatePrefix,
		isAuthenticateHeader: isAuthenticateHeader,
		addAuthorizationPrefix: addAuthorizationPrefix,
		hasAuthorizationPrefix: hasAuthorizationPrefix,
		removeAuthorizationPrefix: removeAuthorizationPrefix,
		isAuthorizationHeader: isAuthorizationHeader
	});

	var XHR_READY_STATES = ['UNSENT', 'OPENED', 'HEADERS_RECEIVED', 'LOADING', 'DONE'];
	var XhrSandbox = /** @class */ (function (_super) {
	    __extends(XhrSandbox, _super);
	    function XhrSandbox(_cookieSandbox, waitHammerheadSettings) {
	        var _this = _super.call(this, waitHammerheadSettings) || this;
	        _this._cookieSandbox = _cookieSandbox;
	        _this.XHR_COMPLETED_EVENT = 'hammerhead|event|xhr-completed';
	        _this.XHR_ERROR_EVENT = 'hammerhead|event|xhr-error';
	        _this.BEFORE_XHR_SEND_EVENT = 'hammerhead|event|before-xhr-send';
	        return _this;
	    }
	    XhrSandbox.createNativeXHR = function () {
	        var xhr = new nativeMethods.XMLHttpRequest();
	        xhr.open = nativeMethods.xhrOpen;
	        xhr.abort = nativeMethods.xhrAbort;
	        xhr.send = nativeMethods.xhrSend;
	        xhr.addEventListener = nativeMethods.xhrAddEventListener || nativeMethods.addEventListener;
	        xhr.removeEventListener = nativeMethods.xhrRemoveEventListener || nativeMethods.removeEventListener;
	        xhr.setRequestHeader = nativeMethods.xhrSetRequestHeader;
	        xhr.getResponseHeader = nativeMethods.xhrGetResponseHeader;
	        xhr.getAllResponseHeaders = nativeMethods.xhrGetAllResponseHeaders;
	        xhr.overrideMimeType = nativeMethods.xhrOverrideMimeType;
	        xhr.dispatchEvent = nativeMethods.xhrDispatchEvent || nativeMethods.dispatchEvent;
	        return xhr;
	    };
	    XhrSandbox.openNativeXhr = function (xhr, url, isAsync) {
	        xhr.open('POST', url, isAsync);
	        xhr.setRequestHeader(BUILTIN_HEADERS.cacheControl, 'no-cache, no-store, must-revalidate');
	    };
	    XhrSandbox._reopenXhr = function (xhr, reqOpts) {
	        var url = reqOpts.openArgs[1];
	        var withCredentials = xhr.withCredentials;
	        reqOpts.withCredentials = withCredentials;
	        reqOpts.openArgs[1] = getAjaxProxyUrl(url, withCredentials ? Credentials.include : Credentials.sameOrigin);
	        nativeMethods.xhrOpen.apply(xhr, reqOpts.openArgs);
	        reqOpts.openArgs[1] = url;
	        for (var _i = 0, _a = reqOpts.headers; _i < _a.length; _i++) {
	            var header = _a[_i];
	            nativeMethods.xhrSetRequestHeader.apply(xhr, header);
	        }
	    };
	    XhrSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        var xhrSandbox = this;
	        var xmlHttpRequestProto = window.XMLHttpRequest.prototype;
	        var emitXhrCompletedEvent = function () {
	            var nativeRemoveEventListener = nativeMethods.xhrRemoveEventListener || nativeMethods.removeEventListener;
	            xhrSandbox.emit(xhrSandbox.XHR_COMPLETED_EVENT, { xhr: this });
	            nativeRemoveEventListener.call(this, 'loadend', emitXhrCompletedEvent);
	        };
	        var syncCookieWithClientIfNecessary = function () {
	            if (this.readyState < this.HEADERS_RECEIVED)
	                return;
	            var nativeRemoveEventListener = nativeMethods.xhrRemoveEventListener || nativeMethods.removeEventListener;
	            xhrSandbox._cookieSandbox.syncCookie();
	            nativeRemoveEventListener.call(this, 'readystatechange', syncCookieWithClientIfNecessary);
	        };
	        var xmlHttpRequestWrapper = function () {
	            var nativeAddEventListener = nativeMethods.xhrAddEventListener || nativeMethods.addEventListener;
	            var xhr = new nativeMethods.XMLHttpRequest();
	            nativeAddEventListener.call(xhr, 'loadend', emitXhrCompletedEvent);
	            nativeAddEventListener.call(xhr, 'readystatechange', syncCookieWithClientIfNecessary);
	            return xhr;
	        };
	        for (var _i = 0, XHR_READY_STATES_1 = XHR_READY_STATES; _i < XHR_READY_STATES_1.length; _i++) {
	            var readyState = XHR_READY_STATES_1[_i];
	            nativeMethods.objectDefineProperty(xmlHttpRequestWrapper, readyState, nativeMethods.objectGetOwnPropertyDescriptor(nativeMethods.XMLHttpRequest, readyState));
	        }
	        // NOTE: We cannot just assign constructor property of the prototype of XMLHttpRequest starts from safari 9.0
	        overrideConstructor(window, 'XMLHttpRequest', xmlHttpRequestWrapper);
	        nativeMethods.objectDefineProperty(xmlHttpRequestProto, 'constructor', {
	            value: xmlHttpRequestWrapper
	        });
	        overrideFunction(xmlHttpRequestProto, 'abort', function () {
	            var _this = this;
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (xhrSandbox.gettingSettingInProgress())
	                return void xhrSandbox.delayUntilGetSettings(function () { return _this.abort.apply(_this, args); });
	            nativeMethods.xhrAbort.apply(this, args);
	            xhrSandbox.emit(xhrSandbox.XHR_ERROR_EVENT, {
	                err: new Error('XHR aborted'),
	                xhr: this
	            });
	        });
	        // NOTE: Redirect all requests to the Hammerhead proxy and ensure that requests don't
	        // violate Same Origin Policy.
	        overrideFunction(xmlHttpRequestProto, 'open', function () {
	            var _this = this;
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var url = args[1];
	            if (getProxyUrl(url) === url)
	                return void nativeMethods.xhrOpen.apply(this, args);
	            if (xhrSandbox.gettingSettingInProgress())
	                return void xhrSandbox.delayUntilGetSettings(function () { return _this.open.apply(_this, args); });
	            url = typeof url === 'string' ? url : String(url);
	            args[1] = getAjaxProxyUrl(url, this.withCredentials ? Credentials.include : Credentials.sameOrigin);
	            nativeMethods.xhrOpen.apply(this, args);
	            args[1] = url;
	            XhrSandbox.REQUESTS_OPTIONS.set(this, {
	                withCredentials: this.withCredentials,
	                openArgs: args,
	                headers: []
	            });
	        });
	        overrideFunction(xmlHttpRequestProto, 'send', function () {
	            var _this = this;
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (xhrSandbox.gettingSettingInProgress())
	                return void xhrSandbox.delayUntilGetSettings(function () { return _this.send.apply(_this, args); });
	            var reqOpts = XhrSandbox.REQUESTS_OPTIONS.get(this);
	            if (reqOpts.withCredentials !== this.withCredentials)
	                XhrSandbox._reopenXhr(this, reqOpts);
	            xhrSandbox.emit(xhrSandbox.BEFORE_XHR_SEND_EVENT, { xhr: this });
	            nativeMethods.xhrSend.apply(this, args);
	            // NOTE: For xhr with the sync mode
	            if (this.readyState === this.DONE)
	                emitXhrCompletedEvent.call(this);
	            syncCookieWithClientIfNecessary.call(this);
	        });
	        overrideFunction(xmlHttpRequestProto, 'setRequestHeader', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (isAuthorizationHeader(args[0]))
	                args[1] = addAuthorizationPrefix(args[1]);
	            nativeMethods.xhrSetRequestHeader.apply(this, args);
	            var reqOpts = XhrSandbox.REQUESTS_OPTIONS.get(this);
	            if (reqOpts)
	                reqOpts.headers.push([String(args[0]), String(args[1])]);
	        });
	        if (nativeMethods.xhrResponseURLGetter) {
	            overrideDescriptor(window.XMLHttpRequest.prototype, 'responseURL', {
	                getter: function () {
	                    return getDestinationUrl(nativeMethods.xhrResponseURLGetter.call(this));
	                }
	            });
	        }
	        overrideFunction(xmlHttpRequestProto, 'getResponseHeader', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var value = nativeMethods.xhrGetResponseHeader.apply(this, args);
	            if (value && isAuthenticateHeader(args[0]))
	                value = removeAuthenticatePrefix(value);
	            return value;
	        });
	        overrideFunction(xmlHttpRequestProto, 'getAllResponseHeaders', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var allHeaders = nativeMethods.xhrGetAllResponseHeaders.apply(this, args);
	            while (hasAuthenticatePrefix(allHeaders))
	                allHeaders = removeAuthenticatePrefix(allHeaders);
	            return allHeaders;
	        });
	    };
	    XhrSandbox.REQUESTS_OPTIONS = new WeakMap();
	    return XhrSandbox;
	}(SandboxBaseWithDelayedSettings));

	function getCredentialsMode(credentialsOpt) {
	    credentialsOpt = String(credentialsOpt).toLowerCase();
	    switch (credentialsOpt) {
	        case 'omit': return Credentials.omit;
	        case 'same-origin': return Credentials.sameOrigin;
	        case 'include': return Credentials.include;
	        default: return Credentials.unknown;
	    }
	}
	var DEFAULT_REQUEST_CREDENTIALS = getCredentialsMode(nativeMethods.Request && new nativeMethods.Request(location.toString()).credentials);
	var FetchSandbox = /** @class */ (function (_super) {
	    __extends(FetchSandbox, _super);
	    function FetchSandbox(cookieSandbox, waitHammerheadSettings) {
	        var _this = _super.call(this, waitHammerheadSettings) || this;
	        _this.cookieSandbox = cookieSandbox;
	        _this.FETCH_REQUEST_SENT_EVENT = 'hammerhead|event|fetch-request-sent-event';
	        return _this;
	    }
	    FetchSandbox._removeAuthHeadersPrefix = function (name, value) {
	        if (isAuthorizationHeader(name))
	            return removeAuthorizationPrefix(value);
	        else if (isAuthenticateHeader(name))
	            return removeAuthenticatePrefix(value);
	        return value;
	    };
	    FetchSandbox._processInit = function (init) {
	        var headers = init.headers;
	        if (!headers)
	            return init;
	        if (!isFetchHeaders(headers)) {
	            headers = headers ? new nativeMethods.Headers(headers) : new nativeMethods.Headers();
	            init.headers = headers;
	        }
	        var authorizationValue = nativeMethods.headersGet.call(headers, BUILTIN_HEADERS.authorization);
	        var proxyAuthorizationValue = nativeMethods.headersGet.call(headers, BUILTIN_HEADERS.proxyAuthorization);
	        if (authorizationValue !== null && !hasAuthorizationPrefix(authorizationValue))
	            nativeMethods.headersSet.call(headers, BUILTIN_HEADERS.authorization, addAuthorizationPrefix(authorizationValue));
	        if (proxyAuthorizationValue !== null && !hasAuthorizationPrefix(proxyAuthorizationValue))
	            nativeMethods.headersSet.call(headers, BUILTIN_HEADERS.proxyAuthorization, addAuthorizationPrefix(proxyAuthorizationValue));
	        return init;
	    };
	    FetchSandbox._processArguments = function (args) {
	        var input = args[0], init = args[1];
	        var inputIsString = typeof input === 'string';
	        var optsCredentials = getCredentialsMode(init && init.credentials);
	        if (!isFetchRequest(input)) {
	            var url = inputIsString ? input : String(input);
	            var credentials = optsCredentials === Credentials.unknown ? DEFAULT_REQUEST_CREDENTIALS : optsCredentials;
	            args[0] = getAjaxProxyUrl(url, credentials);
	            args[1] = FetchSandbox._processInit(init || {});
	        }
	        else {
	            if (optsCredentials !== Credentials.unknown)
	                args[0] = getAjaxProxyUrl(input.url, optsCredentials);
	            if (init && init.headers && input.destination !== 'worker')
	                args[1] = FetchSandbox._processInit(init);
	        }
	    };
	    FetchSandbox._processHeaderEntry = function (entry, isOnlyValue) {
	        if (isOnlyValue === void 0) { isOnlyValue = false; }
	        if (entry.done)
	            return entry;
	        /* eslint-disable no-restricted-properties */
	        var processedValue = FetchSandbox._removeAuthHeadersPrefix(entry.value[0], entry.value[1]);
	        if (isOnlyValue)
	            entry.value = processedValue;
	        else
	            entry.value[1] = processedValue;
	        /* eslint-enable no-restricted-properties */
	        return entry;
	    };
	    FetchSandbox._entriesWrapper = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var iterator = nativeMethods.headersEntries.apply(this, args);
	        var nativeNext = iterator.next;
	        iterator.next = function () { return FetchSandbox._processHeaderEntry(nativeNext.call(iterator)); };
	        return iterator;
	    };
	    FetchSandbox._valuesWrapper = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var iterator = nativeMethods.headersEntries.apply(this, args);
	        var nativeNext = iterator.next;
	        iterator.next = function () { return FetchSandbox._processHeaderEntry(nativeNext.call(iterator), true); };
	        return iterator;
	    };
	    FetchSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window, window.document);
	        if (!nativeMethods.fetch)
	            return;
	        var sandbox = this;
	        overrideConstructor(window, 'Request', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            FetchSandbox._processArguments(args);
	            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = nativeMethods.headersEntries;
	            var request = args.length === 1
	                ? new nativeMethods.Request(args[0])
	                : new nativeMethods.Request(args[0], args[1]);
	            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = FetchSandbox._entriesWrapper;
	            return request;
	        });
	        overrideDescriptor(window.Request.prototype, 'url', {
	            getter: function () {
	                return getDestinationUrl(nativeMethods.requestUrlGetter.call(this));
	            }
	        });
	        overrideDescriptor(window.Request.prototype, 'referrer', {
	            getter: function () {
	                return getDestinationUrl(nativeMethods.requestReferrerGetter.call(this));
	            }
	        });
	        overrideFunction(window, 'fetch', function () {
	            var _this = this;
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (sandbox.gettingSettingInProgress())
	                return sandbox.delayUntilGetSettings(function () { return _this.fetch.apply(_this, args); });
	            // NOTE: Safari processed the empty `fetch()` request without `Promise` rejection (GH-1613)
	            if (!args.length && !isSafari)
	                return nativeMethods.fetch.apply(this, args);
	            try {
	                FetchSandbox._processArguments(args);
	            }
	            catch (e) {
	                return nativeMethods.promiseReject.call(sandbox.window.Promise, e);
	            }
	            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = nativeMethods.headersEntries;
	            var fetchPromise = nativeMethods.fetch.apply(this, args);
	            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = FetchSandbox._entriesWrapper;
	            sandbox.emit(sandbox.FETCH_REQUEST_SENT_EVENT, fetchPromise);
	            return nativeMethods.promiseThen.call(fetchPromise, function (response) {
	                sandbox.cookieSandbox.syncCookie();
	                return response;
	            });
	        });
	        overrideDescriptor(window.Response.prototype, 'url', {
	            getter: function () {
	                return getDestinationUrl(nativeMethods.responseUrlGetter.call(this));
	            }
	        });
	        overrideFunction(window.Headers.prototype, 'entries', FetchSandbox._entriesWrapper);
	        overrideFunction(window.Headers.prototype, Symbol.iterator, FetchSandbox._entriesWrapper);
	        overrideFunction(window.Headers.prototype, 'values', FetchSandbox._valuesWrapper);
	        overrideFunction(window.Headers.prototype, 'forEach', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var callback = args[0];
	            if (typeof callback === 'function') {
	                args[0] = function (value, name, headers) {
	                    value = FetchSandbox._removeAuthHeadersPrefix(name, value);
	                    callback.call(this, value, name, headers);
	                };
	            }
	            return nativeMethods.headersForEach.apply(this, args);
	        });
	        overrideFunction(window.Headers.prototype, 'get', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var value = nativeMethods.headersGet.apply(this, args);
	            return value && FetchSandbox._removeAuthHeadersPrefix(args[0], value);
	        });
	        overrideFunction(window.Headers.prototype, 'set', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (isAuthorizationHeader(args[0]))
	                args[1] = addAuthorizationPrefix(args[1]);
	            return nativeMethods.headersSet.apply(this, args);
	        });
	    };
	    return FetchSandbox;
	}(SandboxBaseWithDelayedSettings));

	var STORAGE_WRAPPER_KEY = 'hammerhead|get-storage-wrapper';
	var EMPTY_OLD_VALUE_ARG = isIE ? '' : null;
	var storageWrappersInternalInfo = new WeakMap();
	var KEY = 0;
	var VALUE = 1;
	var StorageInheritor = /** @class */ (function () {
	    function StorageInheritor() {
	    }
	    return StorageInheritor;
	}());
	StorageInheritor.prototype = Storage.prototype;
	var StorageWrapper = /** @class */ (function (_super) {
	    __extends(StorageWrapper, _super);
	    function StorageWrapper(window, nativeStorage, nativeStorageKey) {
	        var _this = _super.call(this) || this;
	        storageWrappersInternalInfo.set(_this, {
	            win: window,
	            ctx: window,
	            lastState: null,
	            eventEmitter: new EventEmitter(),
	            nativeStorage: nativeStorage, nativeStorageKey: nativeStorageKey
	        });
	        _this.loadStorage();
	        return _this;
	    }
	    Object.defineProperty(StorageWrapper.prototype, "internal", {
	        get: function () {
	            return storageWrappersInternalInfo.get(this);
	        },
	        enumerable: true,
	        configurable: true
	    });
	    StorageWrapper.create = function (window, nativeStorage, nativeStorageKey) {
	        var storageWrapper = new StorageWrapper(window, nativeStorage, nativeStorageKey);
	        if (!window.Proxy)
	            // @ts-ignore
	            return storageWrapper;
	        return new nativeMethods.Proxy(storageWrapper, {
	            get: function (target, property) {
	                if (property === STORAGE_WRAPPER_KEY)
	                    return target;
	                return target[property];
	            },
	            set: function (target, property, value) {
	                target['setItem'](property, value);
	                return true;
	            },
	            deleteProperty: function (target, key) {
	                target['removeItem'](key);
	                return true;
	            }
	        });
	    };
	    StorageWrapper.prototype.setContext = function (context) {
	        this.internal.ctx = context;
	    };
	    StorageWrapper.prototype.getContext = function () {
	        return this.internal.ctx;
	    };
	    StorageWrapper.prototype.saveToNativeStorage = function () {
	        var state = stringify(this.getCurrentState());
	        if (this.internal.nativeStorage[this.internal.nativeStorageKey] !== state)
	            this.internal.nativeStorage[this.internal.nativeStorageKey] = state;
	    };
	    StorageWrapper.prototype.getCurrentState = function () {
	        var addedProperties = nativeMethods.objectKeys(this);
	        var state = [[], []];
	        for (var _i = 0, addedProperties_1 = addedProperties; _i < addedProperties_1.length; _i++) {
	            var addedProperty = addedProperties_1[_i];
	            state[KEY].push(addedProperty);
	            state[VALUE].push(this[addedProperty]);
	        }
	        return state;
	    };
	    StorageWrapper.prototype.restore = function (storageStr) {
	        this.clearStorage();
	        this.loadStorage(storageStr);
	    };
	    StorageWrapper.prototype.clearStorage = function () {
	        var addedProperties = nativeMethods.objectKeys(this);
	        var changed = false;
	        for (var _i = 0, addedProperties_2 = addedProperties; _i < addedProperties_2.length; _i++) {
	            var addedProperty = addedProperties_2[_i];
	            delete this[addedProperty];
	            changed = true;
	        }
	        if (changed)
	            this.internal.lastState = this.getCurrentState();
	        return changed;
	    };
	    StorageWrapper.prototype.loadStorage = function (storageStateStr) {
	        if (!storageStateStr)
	            storageStateStr = this.internal.nativeStorage[this.internal.nativeStorageKey];
	        var storageState = parse$1(storageStateStr || '[[],[]]');
	        var storageStateLength = storageState[KEY].length;
	        for (var i = 0; i < storageStateLength; i++)
	            this[storageState[KEY][i]] = storageState[VALUE][i];
	        this.internal.lastState = storageState;
	    };
	    StorageWrapper.prototype.raiseStorageChanged = function (key, oldValue, newValue) {
	        var url = null;
	        try {
	            var parsedContextUrl = parseProxyUrl(this.internal.ctx.location.toString());
	            url = parsedContextUrl ? parsedContextUrl.destUrl : get$2();
	        }
	        catch (e) {
	            this.internal.ctx = this.internal.win;
	            url = get$2();
	        }
	        this.internal.eventEmitter.emit('change', { key: key, oldValue: oldValue, newValue: newValue, url: url });
	    };
	    StorageWrapper.prototype.checkStorageChanged = function () {
	        var lastState = this.internal.lastState;
	        var currentState = this.getCurrentState();
	        for (var i = 0; i < lastState[KEY].length; i++) {
	            var lastStateKey = lastState[KEY][i];
	            var lastStateValue = lastState[VALUE][i];
	            var keyIndex = currentState[KEY].indexOf(lastStateKey);
	            if (keyIndex !== -1) {
	                if (currentState[VALUE][keyIndex] !== lastStateValue)
	                    this.raiseStorageChanged(currentState[KEY][keyIndex], lastStateValue, currentState[VALUE][keyIndex]);
	                currentState[KEY].splice(keyIndex, 1);
	                currentState[VALUE].splice(keyIndex, 1);
	            }
	            else
	                this.raiseStorageChanged(lastStateKey, lastStateValue, null);
	        }
	        for (var j = 0; j < currentState[KEY].length; j++)
	            this.raiseStorageChanged(currentState[KEY][j], EMPTY_OLD_VALUE_ARG, currentState[VALUE][j]);
	        this.internal.lastState = this.getCurrentState();
	    };
	    StorageWrapper.prototype.addChangeEventListener = function (fn) {
	        this.internal.eventEmitter.on('change', fn);
	    };
	    StorageWrapper.prototype.removeChangeEventListener = function (fn) {
	        this.internal.eventEmitter.off('change', fn);
	    };
	    StorageWrapper.prototype.unwrapProxy = function () {
	        if (nativeMethods.Proxy) {
	            var wrapper = this[STORAGE_WRAPPER_KEY];
	            return wrapper || this;
	        }
	        return this;
	    };
	    return StorageWrapper;
	}(StorageInheritor));
	var ourMethods = nativeMethods.objectKeys(StorageWrapper.prototype);
	if (ourMethods.indexOf('internal') === -1)
	    ourMethods.push('internal');
	for (var _i = 0, ourMethods_1 = ourMethods; _i < ourMethods_1.length; _i++) {
	    var method = ourMethods_1[_i];
	    if (method === 'constructor' || method === 'internal')
	        continue;
	    nativeMethods.objectDefineProperty(StorageWrapper.prototype, method, {
	        value: StorageWrapper.prototype[method],
	        configurable: false,
	        enumerable: false,
	        writable: false
	    });
	}
	var internalDescriptor = nativeMethods.objectGetOwnPropertyDescriptor(StorageWrapper.prototype, 'internal');
	internalDescriptor.configurable = false;
	internalDescriptor.enumerable = false;
	nativeMethods.objectDefineProperty(StorageWrapper.prototype, 'internal', internalDescriptor);
	nativeMethods.objectDefineProperty(StorageWrapper.prototype, 'constructor', nativeMethods.objectGetOwnPropertyDescriptor(Storage.prototype, 'constructor'));
	nativeMethods.objectDefineProperty(StorageWrapper, 'INTERNAL_METHODS', { value: ourMethods });

	// -------------------------------------------------------------
	// WARNING: this file is used by both the client and the server.
	// Do not use any browser or node-specific API!
	// -------------------------------------------------------------
	/* eslint hammerhead/proto-methods: 2 */
	var STORAGE_WRAPPER_PREFIX = 'hammerhead|storage-wrapper|';
	function getStorageKey(sessionId, host) {
	    return STORAGE_WRAPPER_PREFIX + sessionId + '|' + host;
	}

	var API_KEY_PREFIX = 'hammerhead|api-key-prefix|';
	var STORAGE_PROPS = nativeMethods.arrayConcat.call(nativeMethods.objectKeys(Storage.prototype), StorageWrapper.INTERNAL_METHODS);
	var StorageSandbox = /** @class */ (function (_super) {
	    __extends(StorageSandbox, _super);
	    function StorageSandbox(_listeners, _unloadSandbox, _eventSimulator) {
	        var _this = _super.call(this) || this;
	        _this._listeners = _listeners;
	        _this._unloadSandbox = _unloadSandbox;
	        _this._eventSimulator = _eventSimulator;
	        _this.localStorageProxy = null;
	        _this.sessionStorageProxy = null;
	        _this.intervalId = null;
	        _this.isLocked = false;
	        return _this;
	    }
	    StorageSandbox.prototype._simulateStorageEvent = function (storageArea, e) {
	        if (this.isDeactivated() || storageArea.unwrapProxy().getContext() === this.window)
	            return;
	        var event = e;
	        event.storageArea = storageArea;
	        this._eventSimulator.storage(this.window, event);
	    };
	    // NOTE: We are using a single storage wrapper instance for all same-domain windows.
	    // This wrapper is saved in the top same-domain window's sandbox.
	    StorageSandbox.prototype._createStorageWrappers = function () {
	        var _this = this;
	        // eslint-disable-next-line no-restricted-properties
	        var host = getParsed().host;
	        var sessionId = settings.get().sessionId;
	        var storageKey = getStorageKey(sessionId, host);
	        var topSameDomainWindow = getTopSameDomainWindow(this.window);
	        var topSameDomainHammerhead = topSameDomainWindow[INTERNAL_PROPS.hammerhead];
	        var topSameDomainStorageSandbox = topSameDomainHammerhead.sandbox.storageSandbox;
	        // NOTE: Use the already created wrappers.
	        if (topSameDomainStorageSandbox !== this) {
	            this.localStorageProxy = topSameDomainStorageSandbox.localStorageProxy;
	            this.sessionStorageProxy = topSameDomainStorageSandbox.sessionStorageProxy;
	        }
	        // NOTE: Or create new.
	        else {
	            var nativeLocalStorage = this.nativeMethods.winLocalStorageGetter.call(this.window);
	            var nativeSessionStorage = this.nativeMethods.winSessionStorageGetter.call(this.window);
	            this.localStorageProxy = StorageWrapper.create(this.window, nativeLocalStorage, storageKey);
	            this.sessionStorageProxy = StorageWrapper.create(this.window, nativeSessionStorage, storageKey);
	            var saveToNativeStorages = function () {
	                if (_this.isLocked)
	                    return;
	                _this.localStorageProxy.unwrapProxy().saveToNativeStorage();
	                _this.sessionStorageProxy.unwrapProxy().saveToNativeStorage();
	            };
	            this._unloadSandbox.on(this._unloadSandbox.UNLOAD_EVENT, saveToNativeStorages);
	            // NOTE: In some case, a browser does not emit the onBeforeUnload event and we need manually watch navigation (GH-1999).
	            // Also, on iOS devices, we realize the BEFORE_UNLOAD_EVENT through the onPageHide event that browser emits too late
	            // and we do not have time to save the localStorage wrapper to the native localStorage (GH-1507).
	            hammerhead.pageNavigationWatch.on(hammerhead.pageNavigationWatch.PAGE_NAVIGATION_TRIGGERED_EVENT, saveToNativeStorages);
	        }
	    };
	    StorageSandbox.prototype._overrideStorageEvent = function () {
	        // NOTE: IE11 has the StorageEvent property, but it is not a constructor
	        if (typeof StorageEvent === 'object')
	            return;
	        overrideConstructor(this.window, 'StorageEvent', function (type, opts) {
	            var _a;
	            var storedArea = (_a = opts) === null || _a === void 0 ? void 0 : _a.storageArea;
	            if (storedArea)
	                delete opts.storageArea;
	            var event;
	            if (arguments.length === 0)
	                event = new nativeMethods.StorageEvent();
	            else if (arguments.length === 1)
	                event = new nativeMethods.StorageEvent(type);
	            else
	                event = new nativeMethods.StorageEvent(type, opts);
	            if (storedArea) {
	                nativeMethods.objectDefineProperty(event, 'storageArea', {
	                    get: function () { return storedArea; },
	                    set: function () { return void 0; }
	                });
	            }
	            return event;
	        });
	    };
	    StorageSandbox.prototype.clear = function () {
	        var localStorageWrapper = this.localStorageProxy.unwrapProxy();
	        var sessionStorageWrapper = this.sessionStorageProxy.unwrapProxy();
	        nativeMethods.storageRemoveItem.call(localStorageWrapper.internal.nativeStorage, localStorageWrapper.internal.nativeStorageKey);
	        nativeMethods.storageRemoveItem.call(sessionStorageWrapper.internal.nativeStorage, sessionStorageWrapper.internal.nativeStorageKey);
	    };
	    StorageSandbox.prototype.lock = function () {
	        this.isLocked = true;
	    };
	    StorageSandbox.prototype.backup = function () {
	        return {
	            localStorage: stringify(this.localStorageProxy.unwrapProxy().getCurrentState()),
	            sessionStorage: stringify(this.sessionStorageProxy.unwrapProxy().getCurrentState())
	        };
	    };
	    StorageSandbox.prototype.restore = function (_a) {
	        var localStorage = _a.localStorage, sessionStorage = _a.sessionStorage;
	        this.localStorageProxy.unwrapProxy().restore(localStorage);
	        this.sessionStorageProxy.unwrapProxy().restore(sessionStorage);
	    };
	    StorageSandbox.prototype._overrideStorageProps = function () {
	        overrideFunction(window.Storage.prototype, 'clear', function () {
	            var storage = this.unwrapProxy();
	            if (!storage.clearStorage())
	                return;
	            storage.raiseStorageChanged(null, null, null);
	        });
	        overrideFunction(window.Storage.prototype, 'getItem', function (key) {
	            if (arguments.length === 0)
	                throw new TypeError("Failed to execute 'getItem' on 'Storage': 1 argument required, but only 0 present.");
	            var storage = this.unwrapProxy();
	            var validKey = StorageSandbox._wrapKey(key);
	            return nativeMethods.objectHasOwnProperty.call(storage, validKey) ? storage[validKey] : null;
	        });
	        overrideFunction(window.Storage.prototype, 'key', function (keyNum) {
	            if (arguments.length === 0)
	                throw new TypeError("TypeError: Failed to execute 'key' on 'Storage': 1 argument required, but only 0 present.");
	            // NOTE: http://w3c-test.org/webstorage/storage_key.html
	            keyNum %= 0x100000000;
	            if (isNaN(keyNum))
	                keyNum = 0;
	            var storage = this.unwrapProxy();
	            var addedProperties = nativeMethods.objectKeys(storage);
	            var isValidNum = keyNum >= 0 && keyNum < addedProperties.length;
	            return isValidNum ? StorageSandbox._unwrapKey(addedProperties[keyNum]) : null;
	        });
	        overrideFunction(window.Storage.prototype, 'removeItem', function (key) {
	            if (arguments.length === 0)
	                throw new TypeError("Failed to execute 'removeItem' on 'Storage': 1 argument required, but only 0 present.");
	            var storage = this.unwrapProxy();
	            var validKey = StorageSandbox._wrapKey(key);
	            delete storage[validKey];
	            storage.checkStorageChanged();
	        });
	        overrideFunction(window.Storage.prototype, 'setItem', function (key, value) {
	            if (arguments.length < 2)
	                throw new TypeError("Failed to execute 'setItem' on 'Storage': 2 arguments required, but only " + arguments.length + " present.");
	            var storage = this.unwrapProxy();
	            var validKey = StorageSandbox._wrapKey(key);
	            storage[validKey] = String(value);
	            storage.checkStorageChanged();
	        });
	        overrideDescriptor(window.Storage.prototype, 'length', {
	            getter: function () {
	                return nativeMethods.objectKeys(this).length;
	            },
	            setter: null
	        });
	    };
	    StorageSandbox.prototype._overrideStoragesGetters = function () {
	        var _this = this;
	        var storagesPropsOwner = this.nativeMethods.getStoragesPropsOwner(window);
	        // NOTE: Storage properties is located in Window.prototype in the IE11 and these are non configurable.
	        // We define descriptors from a prototype with an overridden getter on a window instance.
	        // We don't need define descriptors again if these was overridden.
	        var shouldDefineStorageProps = !this.nativeMethods.isStoragePropsLocatedInProto ||
	            !this.nativeMethods.objectHasOwnProperty.call(window, 'localStorage');
	        if (!shouldDefineStorageProps)
	            return;
	        this.nativeMethods.objectDefineProperties(window, {
	            'localStorage': createOverriddenDescriptor(storagesPropsOwner, 'localStorage', {
	                // @ts-ignore
	                getter: function () {
	                    _this.localStorageProxy.unwrapProxy().setContext(window);
	                    return _this.localStorageProxy;
	                }
	            }),
	            'sessionStorage': createOverriddenDescriptor(storagesPropsOwner, 'sessionStorage', {
	                // @ts-ignore
	                getter: function () {
	                    _this.sessionStorageProxy.unwrapProxy().setContext(window);
	                    return _this.sessionStorageProxy;
	                }
	            })
	        });
	    };
	    StorageSandbox._wrapKey = function (key) {
	        var keyStr = String(key);
	        return STORAGE_PROPS.indexOf(keyStr) !== -1 ? API_KEY_PREFIX + keyStr : keyStr;
	    };
	    StorageSandbox._unwrapKey = function (key) {
	        return key.replace(API_KEY_PREFIX, '');
	    };
	    StorageSandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        this._overrideStorageProps();
	        this._createStorageWrappers();
	        var localStorageWrapper = this.localStorageProxy.unwrapProxy();
	        var sessionStorageWrapper = this.sessionStorageProxy.unwrapProxy();
	        this.intervalId = nativeMethods.setInterval.call(this.window, function () {
	            localStorageWrapper.checkStorageChanged();
	            sessionStorageWrapper.checkStorageChanged();
	        }, 10);
	        this.localStorageChangeHandler = function (e) { return _this._simulateStorageEvent(_this.localStorageProxy, e); };
	        this.sessionStorageChangeHandler = function (e) { return _this._simulateStorageEvent(_this.sessionStorageProxy, e); };
	        localStorageWrapper.addChangeEventListener(this.localStorageChangeHandler);
	        sessionStorageWrapper.addChangeEventListener(this.sessionStorageChangeHandler);
	        this._listeners.initElementListening(window, ['storage']);
	        this._listeners.addInternalEventBeforeListener(window, ['storage'], function (_, dispatched, preventEvent) { return !dispatched && preventEvent(); });
	        this._overrideStorageEvent();
	        this._overrideStoragesGetters();
	    };
	    StorageSandbox.prototype.dispose = function () {
	        this.localStorageProxy.unwrapProxy().removeChangeEventListener(this.localStorageChangeHandler);
	        this.sessionStorageProxy.unwrapProxy().removeChangeEventListener(this.sessionStorageChangeHandler);
	        var topSameDomainWindow = getTopSameDomainWindow(this.window);
	        // NOTE: For removed iframe without src in IE11 window.top equals iframe's window
	        if (this.window === topSameDomainWindow && !topSameDomainWindow.frameElement)
	            nativeMethods.clearInterval.call(this.window, this.intervalId);
	    };
	    return StorageSandbox;
	}(SandboxBase));

	var ElectronSandbox = /** @class */ (function (_super) {
	    __extends(ElectronSandbox, _super);
	    function ElectronSandbox() {
	        return _super !== null && _super.apply(this, arguments) || this;
	    }
	    ElectronSandbox._createFnWrapper = function (vm, nativeFn) {
	        return function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (typeof args[0] === 'string')
	                args[0] = processScript(args[0]);
	            return nativeFn.apply(vm, args);
	        };
	    };
	    ElectronSandbox._overrideElectronModulePaths = function (window) {
	        var electronRequire = window.require;
	        var path = electronRequire('path');
	        var destination = getParsed();
	        /*eslint-disable no-restricted-properties*/
	        if (destination.protocol !== 'file:')
	            return;
	        var pathname = window.process.platform === 'win32' && destination.pathname[0] === '/'
	            ? destination.pathname.substr(1)
	            : destination.pathname;
	        /*eslint-enable no-restricted-properties*/
	        window.__filename = path.normalize(decodeURIComponent(pathname));
	        window.__dirname = path.dirname(window.__filename);
	        window.module.filename = window.__filename;
	        window.module.paths = window.module.paths.concat(electronRequire('module')._nodeModulePaths(window.__dirname));
	    };
	    ElectronSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        if (!window.require)
	            return;
	        var vm = window.require('vm');
	        if (!vm)
	            return;
	        var nativeMethods = this.nativeMethods;
	        if (nativeMethods.refreshElectronMeths(vm)) {
	            overrideFunction(vm, 'createScript', ElectronSandbox._createFnWrapper(vm, nativeMethods.createScript));
	            // NOTE: DebugContext has been removed in V8 and is not available in Node.js 10+
	            if (vm.runInDebugContext)
	                overrideFunction(vm, 'runInDebugContext', ElectronSandbox._createFnWrapper(vm, nativeMethods.runInDebugContext));
	            overrideFunction(vm, 'runInContext', ElectronSandbox._createFnWrapper(vm, nativeMethods.runInContext));
	            overrideFunction(vm, 'runInNewContext', ElectronSandbox._createFnWrapper(vm, nativeMethods.runInNewContext));
	            overrideFunction(vm, 'runInThisContext', ElectronSandbox._createFnWrapper(vm, nativeMethods.runInThisContext));
	            ElectronSandbox._overrideElectronModulePaths(window);
	        }
	    };
	    return ElectronSandbox;
	}(SandboxBase));

	var ConsoleSandbox = /** @class */ (function (_super) {
	    __extends(ConsoleSandbox, _super);
	    function ConsoleSandbox(_messageSandbox) {
	        var _this = _super.call(this) || this;
	        _this._messageSandbox = _messageSandbox;
	        _this.CONSOLE_METH_CALLED_EVENT = 'hammerhead|event|console-meth-called';
	        _this._serviceMsgReceivedEventCallback = function (_a) {
	            var message = _a.message;
	            if (message.cmd === _this.CONSOLE_METH_CALLED_EVENT)
	                _this.emit(_this.CONSOLE_METH_CALLED_EVENT, { meth: message.meth, line: message.line });
	        };
	        return _this;
	    }
	    ConsoleSandbox.prototype._toString = function (obj) {
	        try {
	            return String(obj);
	        }
	        catch (e) {
	            return 'object';
	        }
	    };
	    ConsoleSandbox.prototype._proxyConsoleMeth = function (meth) {
	        var _this = this;
	        overrideFunction(this.window.console, meth, function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            if (!isCrossDomainWindows(window, window.top)) {
	                var sendToTopWindow = isIframeWindow(window);
	                var line = nativeMethods.arrayMap.call(args, _this._toString).join(' ');
	                if (sendToTopWindow) {
	                    _this.emit(_this.CONSOLE_METH_CALLED_EVENT, { meth: meth, line: line, inIframe: true });
	                    _this._messageSandbox.sendServiceMsg({ meth: meth, line: line, cmd: _this.CONSOLE_METH_CALLED_EVENT }, window.top);
	                }
	                else
	                    _this.emit(_this.CONSOLE_METH_CALLED_EVENT, { meth: meth, line: line });
	            }
	            _this.nativeMethods.consoleMeths[meth].apply(_this.nativeMethods.console, args);
	        });
	    };
	    ConsoleSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        this._proxyConsoleMeth('log');
	        this._proxyConsoleMeth('info');
	        this._proxyConsoleMeth('error');
	        this._proxyConsoleMeth('warn');
	        this._messageSandbox.on(this._messageSandbox.SERVICE_MSG_RECEIVED_EVENT, this._serviceMsgReceivedEventCallback);
	    };
	    return ConsoleSandbox;
	}(SandboxBase));

	var CSS_STYLE_IS_PROCESSED = 'hammerhead|style|is-processed';
	var CSS_STYLE_PROXY_OBJECT = 'hammerhead|style|proxy-object';
	var CSS_STYLE_PROXY_TARGET = 'hammerhead|style|proxy-target';
	var StyleSandbox = /** @class */ (function (_super) {
	    __extends(StyleSandbox, _super);
	    function StyleSandbox() {
	        var _this = _super.call(this) || this;
	        _this.URL_PROPS = ['background', 'backgroundImage', 'borderImage',
	            'borderImageSource', 'listStyle', 'listStyleImage', 'cursor'];
	        _this.DASHED_URL_PROPS = StyleSandbox._generateDashedProps(_this.URL_PROPS);
	        _this.FEATURES = _this._detectBrowserFeatures();
	        return _this;
	    }
	    StyleSandbox._convertToDashed = function (prop) {
	        return prop.replace(/[A-Z]/g, '-$&').toLowerCase();
	    };
	    StyleSandbox._generateDashedProps = function (props) {
	        var dashedProps = [];
	        for (var _i = 0, props_1 = props; _i < props_1.length; _i++) {
	            var prop = props_1[_i];
	            var dashedProp = StyleSandbox._convertToDashed(prop);
	            if (prop !== dashedProp)
	                dashedProps.push(dashedProp);
	        }
	        return dashedProps;
	    };
	    StyleSandbox.prototype._detectBrowserFeatures = function () {
	        var features = {};
	        // NOTE: The CSS2Properties class is supported only in the Firefox
	        // and its prototype contains all property descriptors
	        // @ts-ignore
	        features.protoContainsAllProps = !!window.CSS2Properties;
	        // NOTE: The CSSStyleDeclaration class contains not dashed url properties only in the IE
	        features.protoContainsUrlProps = this.nativeMethods.objectHasOwnProperty
	            // @ts-ignore
	            .call(window.CSSStyleDeclaration.prototype, 'background');
	        if (!features.protoContainsAllProps && !features.protoContainsUrlProps) {
	            var testDiv = this.nativeMethods.createElement.call(document, 'div');
	            var propertySetterIsCalled_1 = false;
	            var testDivDescriptor = this.nativeMethods.objectGetOwnPropertyDescriptor
	                // @ts-ignore
	                .call(window.Object, testDiv.style, 'background');
	            if (testDivDescriptor.configurable) {
	                // eslint-disable-next-line no-restricted-properties
	                delete testDivDescriptor.value;
	                delete testDivDescriptor.writable;
	                testDivDescriptor.set = function () {
	                    propertySetterIsCalled_1 = true;
	                };
	                this.nativeMethods.objectDefineProperty(testDiv.style, 'background', testDivDescriptor);
	                testDiv.style.background = 'url';
	            }
	            // NOTE: A style instance contains all url properties.
	            // They are non-configurable in Safari less than 11.1.
	            // Their setter cannot be called in Safari 11.1.
	            features.propsCannotBeOverridden = !testDivDescriptor.configurable || !propertySetterIsCalled_1;
	        }
	        return features;
	    };
	    StyleSandbox.prototype._overrideStyleProp = function (proto, prop) {
	        var nativeMethods = this.nativeMethods;
	        var dashedProp = StyleSandbox._convertToDashed(prop);
	        overrideDescriptor(proto, prop, {
	            getter: function () {
	                var value = nativeMethods.styleGetPropertyValue.call(this, dashedProp);
	                return styleProcessor.cleanUp(value, parseProxyUrl);
	            },
	            setter: function (value) {
	                if (typeof value === 'string')
	                    value = styleProcessor.process(value, getProxyUrl);
	                nativeMethods.styleSetProperty.call(this, dashedProp, value);
	            }
	        });
	    };
	    StyleSandbox.prototype._overrideStyleInstanceProp = function (style, prop) {
	        var nativeMethods = this.nativeMethods;
	        var dashedProp = StyleSandbox._convertToDashed(prop);
	        overrideDescriptor(style, prop, {
	            getter: function () {
	                var value = nativeMethods.styleGetPropertyValue.call(this, dashedProp);
	                return styleProcessor.cleanUp(value, parseProxyUrl);
	            },
	            setter: function (value) {
	                if (typeof value === 'string')
	                    value = styleProcessor.process(value, getProxyUrl);
	                nativeMethods.styleSetProperty.call(this, dashedProp, value);
	            }
	        });
	    };
	    StyleSandbox.prototype._processStyleInstance = function (style) {
	        var isProcessed = style[CSS_STYLE_IS_PROCESSED];
	        if (!isProcessed) {
	            for (var _i = 0, _a = this.DASHED_URL_PROPS; _i < _a.length; _i++) {
	                var prop = _a[_i];
	                this._overrideStyleInstanceProp(style, prop);
	            }
	            if (!this.FEATURES.protoContainsUrlProps) {
	                for (var _b = 0, _c = this.URL_PROPS; _b < _c.length; _b++) {
	                    var prop = _c[_b];
	                    this._overrideStyleInstanceProp(style, prop);
	                }
	            }
	            this.nativeMethods.objectDefineProperty(style, CSS_STYLE_IS_PROCESSED, { value: true });
	        }
	        return style;
	    };
	    StyleSandbox.prototype._getStyleProxy = function (style) {
	        var _this = this;
	        var proxyObject = style[CSS_STYLE_PROXY_OBJECT];
	        if (!proxyObject) {
	            proxyObject = new this.nativeMethods.Proxy(style, {
	                get: function (target, prop) {
	                    if (_this.URL_PROPS.indexOf(prop) !== -1 || _this.DASHED_URL_PROPS.indexOf(prop) !== -1)
	                        return styleProcessor.cleanUp(target[prop], parseProxyUrl);
	                    if (prop === CSS_STYLE_PROXY_TARGET)
	                        return target;
	                    return target[prop];
	                },
	                set: function (target, prop, value) {
	                    if (_this.URL_PROPS.indexOf(prop) !== -1 || _this.DASHED_URL_PROPS.indexOf(prop) !== -1) {
	                        if (typeof value === 'string')
	                            value = styleProcessor.process(value, getProxyUrl);
	                    }
	                    target[prop] = value;
	                    return true;
	                }
	            });
	            this.nativeMethods.objectDefineProperty(style, CSS_STYLE_PROXY_OBJECT, { value: proxyObject });
	        }
	        return proxyObject;
	    };
	    StyleSandbox.prototype._overrideCSSStyleDeclarationFunctionsCtx = function (window) {
	        var styleDeclarationProto = window.CSSStyleDeclaration.prototype;
	        var _loop_1 = function (prop) {
	            var nativeFn = this_1.nativeMethods.objectGetOwnPropertyDescriptor.call(window.Object, styleDeclarationProto, prop).value; // eslint-disable-line no-restricted-properties
	            if (this_1.nativeMethods.objectHasOwnProperty.call(styleDeclarationProto, prop) &&
	                typeof nativeFn === 'function') {
	                styleDeclarationProto[prop] = function () {
	                    return nativeFn.apply(this[CSS_STYLE_PROXY_TARGET] || this, arguments);
	                };
	                // NOTE: we cannot use 'overrideFunction' here since the function may not exist
	                overrideStringRepresentation(styleDeclarationProto[prop], nativeFn);
	            }
	        };
	        var this_1 = this;
	        for (var prop in styleDeclarationProto) {
	            _loop_1(prop);
	        }
	    };
	    StyleSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window);
	        var nativeMethods = this.nativeMethods;
	        var styleSandbox = this;
	        overrideDescriptor(window[nativeMethods.htmlElementStylePropOwnerName].prototype, 'style', {
	            getter: this.FEATURES.protoContainsAllProps ? null : function () {
	                var style = nativeMethods.htmlElementStyleGetter.call(this);
	                if (styleSandbox.FEATURES.propsCannotBeOverridden)
	                    return styleSandbox._getStyleProxy(style);
	                return styleSandbox._processStyleInstance(style);
	            },
	            setter: nativeMethods.htmlElementStyleSetter ? function (value) {
	                var processedCss = styleProcessor.process(value, getProxyUrl);
	                nativeMethods.htmlElementStyleSetter.call(this, processedCss);
	            } : null
	        });
	        if (this.FEATURES.protoContainsAllProps) {
	            for (var _i = 0, _a = this.URL_PROPS; _i < _a.length; _i++) {
	                var prop = _a[_i];
	                // @ts-ignore
	                this._overrideStyleProp(window.CSS2Properties.prototype, prop);
	            }
	            for (var _b = 0, _c = this.DASHED_URL_PROPS; _b < _c.length; _b++) {
	                var prop = _c[_b];
	                // @ts-ignore
	                this._overrideStyleProp(window.CSS2Properties.prototype, prop);
	            }
	        }
	        else if (this.FEATURES.protoContainsUrlProps) {
	            for (var _d = 0, _e = this.URL_PROPS; _d < _e.length; _d++) {
	                var prop = _e[_d];
	                this._overrideStyleProp(window.CSSStyleDeclaration.prototype, prop);
	            }
	        }
	        overrideDescriptor(window.CSSStyleDeclaration.prototype, 'cssText', {
	            getter: function () {
	                var cssText = nativeMethods.styleCssTextGetter.call(this);
	                return styleProcessor.cleanUp(cssText, parseProxyUrl);
	            },
	            setter: function (value) {
	                if (typeof value === 'string')
	                    value = styleProcessor.process(value, getProxyUrl);
	                nativeMethods.styleCssTextSetter.call(this, value);
	            }
	        });
	        overrideFunction(window.CSSStyleSheet.prototype, 'insertRule', function (rule, index) {
	            var newRule = styleProcessor.process(rule, getProxyUrl);
	            return nativeMethods.styleInsertRule.call(this, newRule, index);
	        });
	        overrideFunction(window.CSSStyleDeclaration.prototype, 'getPropertyValue', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var value = nativeMethods.styleGetPropertyValue.apply(this, args);
	            return styleProcessor.cleanUp(value, parseProxyUrl);
	        });
	        overrideFunction(window.CSSStyleDeclaration.prototype, 'setProperty', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var value = args[1];
	            if (typeof value === 'string')
	                args[1] = styleProcessor.process(value, getProxyUrl);
	            return nativeMethods.styleSetProperty.apply(this, args);
	        });
	        overrideFunction(window.CSSStyleDeclaration.prototype, 'removeProperty', function () {
	            var args = [];
	            for (var _i = 0; _i < arguments.length; _i++) {
	                args[_i] = arguments[_i];
	            }
	            var oldValue = nativeMethods.styleRemoveProperty.apply(this, args);
	            return styleProcessor.cleanUp(oldValue, parseProxyUrl);
	        });
	        // NOTE: We need to override context of all functions from the CSSStyleDeclaration prototype if we use the Proxy feature.
	        // Can only call CSSStyleDeclaration.<function name> on instances of CSSStyleDeclaration
	        // The error above occurs if functions will be called on a proxy instance.
	        if (this.FEATURES.propsCannotBeOverridden)
	            this._overrideCSSStyleDeclarationFunctionsCtx(window);
	    };
	    return StyleSandbox;
	}(SandboxBase));

	var BROWSERTOOLS_CONSOLE_SAFEFUNC = '__BROWSERTOOLS_CONSOLE_SAFEFUNC';
	var IEDebugSandbox = /** @class */ (function (_super) {
	    __extends(IEDebugSandbox, _super);
	    function IEDebugSandbox() {
	        var _this = _super.call(this) || this;
	        _this._isDebuggerInitiator = false;
	        return _this;
	    }
	    IEDebugSandbox.prototype._createFuncWrapper = function (func) {
	        var _this = this;
	        if (typeof func === 'function') {
	            return function (fn, safeAssert) {
	                var ieDebugSandbox = _this;
	                return function () {
	                    ieDebugSandbox._isDebuggerInitiator = true;
	                    try {
	                        var result = fn(arguments);
	                        ieDebugSandbox._isDebuggerInitiator = false;
	                        return result;
	                    }
	                    catch (e) {
	                        safeAssert(e);
	                    }
	                    ieDebugSandbox._isDebuggerInitiator = false;
	                };
	            };
	        }
	        return func;
	    };
	    IEDebugSandbox.prototype.isDebuggerInitiator = function () {
	        return this._isDebuggerInitiator;
	    };
	    IEDebugSandbox.prototype.attach = function (window) {
	        var _this = this;
	        if (!isIE)
	            return;
	        var descriptor = this.nativeMethods.objectGetOwnPropertyDescriptor(window, BROWSERTOOLS_CONSOLE_SAFEFUNC);
	        var wrapper = void 0;
	        if (!descriptor || descriptor.value) { // eslint-disable-line no-restricted-properties
	            if (descriptor)
	                wrapper = this._createFuncWrapper(descriptor.value); // eslint-disable-line no-restricted-properties
	            this.nativeMethods.objectDefineProperty(window, BROWSERTOOLS_CONSOLE_SAFEFUNC, {
	                set: function (fn) {
	                    wrapper = _this._createFuncWrapper(fn);
	                },
	                get: function () { return wrapper; }
	            });
	        }
	    };
	    return IEDebugSandbox;
	}(SandboxBase));

	function getRandomInt16Value () {
	    var array = new nativeMethods.Uint16Array(1);
	    nativeMethods.cryptoGetRandomValues.call(nativeMethods.crypto, array);
	    return array[0];
	}

	var DEFAULT_WINDOW_PARAMETERS = 'width=500px, height=500px';
	var STORE_CHILD_WINDOW_CMD = 'hammerhead|command|store-child-window';
	var ChildWindowSandbox = /** @class */ (function (_super) {
	    __extends(ChildWindowSandbox, _super);
	    function ChildWindowSandbox(_messageSandbox, _listeners) {
	        var _this = _super.call(this) || this;
	        _this._messageSandbox = _messageSandbox;
	        _this._listeners = _listeners;
	        _this.WINDOW_OPENED_EVENT = 'hammerhead|event|window-opened';
	        return _this;
	    }
	    ChildWindowSandbox._shouldOpenInNewWindowOnElementAction = function (el, defaultTarget) {
	        var hasDownloadAttribute = nativeMethods.getAttribute.call(el, 'download');
	        if (hasDownloadAttribute)
	            return false;
	        var target = this._calculateTargetForElement(el);
	        return this._shouldOpenInNewWindow(target, defaultTarget);
	    };
	    ChildWindowSandbox._shouldOpenInNewWindow = function (target, defaultTarget) {
	        target = target || defaultTarget;
	        target = target.toLowerCase();
	        if (isKeywordTarget(target))
	            return target === '_blank';
	        return !findByName(target);
	    };
	    ChildWindowSandbox.prototype._openUrlInNewWindow = function (url, windowName, windowParams, window) {
	        var windowId = getRandomInt16Value().toString();
	        windowParams = windowParams || DEFAULT_WINDOW_PARAMETERS;
	        windowName = windowName || windowId;
	        var newPageUrl = getPageProxyUrl(url, windowId);
	        var targetWindow = window || this.window;
	        var openedWindow = nativeMethods.windowOpen.call(targetWindow, newPageUrl, windowName, windowParams);
	        this._tryToStoreChildWindow(openedWindow, getTopOpenerWindow());
	        this.emit(this.WINDOW_OPENED_EVENT, { windowId: windowId, window: openedWindow });
	        return { windowId: windowId, wnd: openedWindow };
	    };
	    ChildWindowSandbox._calculateTargetForElement = function (el) {
	        var _a;
	        var base = nativeMethods.querySelector.call(findDocument(el), 'base');
	        return el.target || ((_a = base) === null || _a === void 0 ? void 0 : _a.target);
	    };
	    ChildWindowSandbox.prototype.handleClickOnLinkOrArea = function (el) {
	        var _this = this;
	        if (!settings.get().allowMultipleWindows)
	            return;
	        this._listeners.initElementListening(el, ['click']);
	        this._listeners.addInternalEventAfterListener(el, ['click'], function (e) {
	            if (e.defaultPrevented)
	                return;
	            if (!ChildWindowSandbox._shouldOpenInNewWindowOnElementAction(el, defaultTarget.linkOrArea))
	                return;
	            // TODO: need to check that specified 'area' are clickable (initiated new page opening)
	            var url = nativeMethods.anchorHrefGetter.call(el);
	            e.preventDefault();
	            _this._openUrlInNewWindow(url);
	        });
	    };
	    ChildWindowSandbox.prototype.handleWindowOpen = function (window, args) {
	        var url = args[0], target = args[1], parameters = args[2];
	        if (settings.get().allowMultipleWindows && ChildWindowSandbox._shouldOpenInNewWindow(target, defaultTarget.windowOpen)) {
	            var openedWindowInfo = this._openUrlInNewWindow(url, target, parameters, window);
	            return openedWindowInfo.wnd;
	        }
	        return nativeMethods.windowOpen.apply(window, args);
	    };
	    ChildWindowSandbox.prototype._handleFormSubmitting = function (window) {
	        var _this = this;
	        if (!settings.get().allowMultipleWindows)
	            return;
	        this._listeners.initElementListening(window, ['submit']);
	        this._listeners.addInternalEventBeforeListener(window, ['submit'], function (e) {
	            if (!isFormElement(e.target))
	                return;
	            var form = e.target;
	            if (!ChildWindowSandbox._shouldOpenInNewWindowOnElementAction(form, defaultTarget.form))
	                return;
	            var aboutBlankUrl = getProxyUrl(SPECIAL_BLANK_PAGE);
	            var openedInfo = _this._openUrlInNewWindow(aboutBlankUrl);
	            var formAction = nativeMethods.formActionGetter.call(form);
	            var newWindowUrl = getPageProxyUrl(formAction, openedInfo.windowId);
	            nativeMethods.formActionSetter.call(form, newWindowUrl);
	            nativeMethods.formTargetSetter.call(form, openedInfo.windowId);
	            // TODO: On hammerhead start we need to clean up the window.name
	            // It's necessary for form submit.
	            // Also we need clean up the form target to the original value.
	        });
	    };
	    ChildWindowSandbox.prototype._tryToStoreChildWindow = function (win, topOpenerWindow) {
	        try {
	            topOpenerWindow[INTERNAL_PROPS.hammerhead].sandbox.childWindow.addWindow(win);
	            return true;
	        }
	        catch (e) {
	            return false;
	        }
	    };
	    ChildWindowSandbox.prototype._setupChildWindowCollecting = function (window) {
	        var _this = this;
	        if (isIframeWindow(window))
	            return;
	        var topOpenerWindow = getTopOpenerWindow();
	        if (window !== topOpenerWindow) {
	            if (!this._tryToStoreChildWindow(window, topOpenerWindow))
	                this._messageSandbox.sendServiceMsg({ cmd: STORE_CHILD_WINDOW_CMD }, topOpenerWindow);
	        }
	        else {
	            this._childWindows = new Set();
	            this._messageSandbox.on(this._messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (_a) {
	                var message = _a.message, source = _a.source;
	                if (message.cmd === STORE_CHILD_WINDOW_CMD)
	                    _this._childWindows.add(source);
	            });
	        }
	    };
	    ChildWindowSandbox.prototype.addWindow = function (win) {
	        this._childWindows.add(win);
	    };
	    ChildWindowSandbox.prototype.getChildWindows = function () {
	        var _this = this;
	        var childWindows = [];
	        // eslint-disable-next-line hammerhead/proto-methods
	        this._childWindows.forEach(function (win) {
	            // NOTE: sort windows that can be closed
	            if (win.parent)
	                childWindows.push(win);
	            else
	                _this._childWindows.delete(win);
	        });
	        return childWindows;
	    };
	    ChildWindowSandbox.prototype.attach = function (window) {
	        _super.prototype.attach.call(this, window, window.document);
	        this._handleFormSubmitting(window);
	        this._setupChildWindowCollecting(window);
	    };
	    return ChildWindowSandbox;
	}(SandboxBase));

	var Sandbox = /** @class */ (function (_super) {
	    __extends(Sandbox, _super);
	    function Sandbox(transport) {
	        var _this = _super.call(this) || this;
	        create(window, _this);
	        add(window);
	        var ieDebugSandbox = new IEDebugSandbox();
	        var listeners = new Listeners();
	        var nodeMutation = new NodeMutation();
	        var unloadSandbox = new UnloadSandbox(listeners);
	        var messageSandbox = new MessageSandbox(listeners, unloadSandbox);
	        var eventSimulator = new EventSimulator();
	        var elementEditingWatcher = new ElementEditingWatcher(eventSimulator);
	        var timersSandbox = new TimersSandbox();
	        var childWindowSandbox = new ChildWindowSandbox(messageSandbox, listeners);
	        var cookieSandbox = new CookieSandbox(messageSandbox, unloadSandbox, childWindowSandbox);
	        // API
	        _this.ieDebug = ieDebugSandbox;
	        _this.cookie = cookieSandbox; // eslint-disable-line no-restricted-properties
	        _this.childWindow = childWindowSandbox;
	        _this.storageSandbox = new StorageSandbox(listeners, unloadSandbox, eventSimulator);
	        _this.xhr = new XhrSandbox(cookieSandbox);
	        _this.fetch = new FetchSandbox(cookieSandbox);
	        _this.iframe = new IframeSandbox(nodeMutation, cookieSandbox);
	        _this.shadowUI = new ShadowUI(nodeMutation, messageSandbox, _this.iframe, ieDebugSandbox);
	        _this.upload = new UploadSandbox(listeners, eventSimulator, transport);
	        _this.event = new EventSandbox(listeners, eventSimulator, elementEditingWatcher, unloadSandbox, messageSandbox, _this.shadowUI, timersSandbox);
	        _this.node = new NodeSandbox(nodeMutation, _this.iframe, _this.event, _this.upload, _this.shadowUI, cookieSandbox, _this.childWindow);
	        _this.codeInstrumentation = new CodeInstrumentation(_this.event, messageSandbox);
	        _this.console = new ConsoleSandbox(messageSandbox);
	        _this.style = new StyleSandbox();
	        _this.unload = unloadSandbox;
	        if (isElectron)
	            _this.electron = new ElectronSandbox();
	        _this.windowStorage = windowStorage;
	        return _this;
	    }
	    // NOTE: In some cases, IE raises the "Can't execute code from a freed script" exception,
	    // so that we cannot use a sandbox created earlier and we have to create a new one.
	    Sandbox._canUseSandbox = function (sandbox) {
	        try {
	            //@ts-ignore
	            sandbox.off();
	        }
	        catch (e) {
	            return false;
	        }
	        return true;
	    };
	    Sandbox.prototype.onIframeDocumentRecreated = function (iframe) {
	        if (iframe) {
	            var contentWindow = nativeMethods.contentWindowGetter.call(iframe);
	            var contentDocument = nativeMethods.contentDocumentGetter.call(iframe);
	            // NOTE: Try to find an existing iframe sandbox.
	            var sandbox = get(contentWindow);
	            if (sandbox && Sandbox._canUseSandbox(sandbox)) {
	                if (!contentWindow[INTERNAL_PROPS.sandboxIsReattached] || sandbox.document !== contentDocument) {
	                    // NOTE: Inform the sandbox so that it restores communication with the recreated document.
	                    sandbox.reattach(contentWindow, contentDocument);
	                }
	            }
	            else {
	                // NOTE: Remove saved native methods for iframe
	                if (contentWindow[INTERNAL_PROPS.iframeNativeMethods])
	                    delete contentWindow[INTERNAL_PROPS.iframeNativeMethods];
	                // NOTE: If the iframe sandbox is not found, this means that iframe is not initialized.
	                // In this case, we need to inject Hammerhead.
	                // HACK: IE10 cleans up overridden methods after the document.write method call.
	                this.nativeMethods.restoreDocumentMeths(contentWindow, contentDocument);
	                // NOTE: A sandbox for this iframe is not found (iframe is not yet initialized).
	                // Inform IFrameSandbox about this, and it injects Hammerhead.
	                this.iframe.onIframeBeganToRun(iframe);
	            }
	        }
	    };
	    Sandbox.prototype.reattach = function (window, document) {
	        nativeMethods.objectDefineProperty(window, INTERNAL_PROPS.sandboxIsReattached, { value: true, configurable: false });
	        // NOTE: Assign the existing sandbox to the cleared document.
	        if (isIE)
	            this.nativeMethods.refreshIfNecessary(document, window);
	        urlResolver.init(document);
	        this.event.reattach(window);
	        this.shadowUI.attach(window);
	        // NOTE: T182337
	        this.codeInstrumentation.attach(window);
	        this.node.doc.attach(window, document);
	        this.console.attach(window);
	        this.childWindow.attach(window);
	    };
	    Sandbox.prototype.attach = function (window) {
	        var _this = this;
	        _super.prototype.attach.call(this, window);
	        nativeMethods.objectDefineProperty(window, INTERNAL_PROPS.sandboxIsReattached, { value: true, configurable: false });
	        urlResolver.init(this.document);
	        // NOTE: Eval Hammerhead code script.
	        this.iframe.on(this.iframe.EVAL_HAMMERHEAD_SCRIPT_EVENT, function (e) {
	            // @ts-ignore
	            nativeMethods.contentWindowGetter.call(e.iframe).eval("(" + initHammerheadClient.toString() + ")();//# sourceURL=hammerhead.js");
	        });
	        // NOTE: We need to reattach a sandbox to the recreated iframe document.
	        this.node.mutation.on(this.node.mutation.DOCUMENT_CLEANED_EVENT, function (e) { return _this.reattach(e.window, e.document); });
	        this.ieDebug.attach(window);
	        this.iframe.attach(window);
	        this.xhr.attach(window);
	        this.fetch.attach(window);
	        this.storageSandbox.attach(window);
	        this.codeInstrumentation.attach(window);
	        this.shadowUI.attach(window);
	        this.event.attach(window);
	        this.node.attach(window);
	        this.upload.attach(window);
	        this.cookie.attach(window); // eslint-disable-line no-restricted-properties
	        this.console.attach(window);
	        this.style.attach(window);
	        this.childWindow.attach(window);
	        if (this.electron)
	            this.electron.attach(window);
	        this.unload.on(this.unload.UNLOAD_EVENT, function () { return _this.dispose(); });
	    };
	    Sandbox.prototype._removeInternalProperties = function () {
	        var removeListeningElement = this.event.listeners.listeningCtx.removeListeningElement;
	        removeListeningElement(this.window);
	        removeListeningElement(this.document);
	        var childNodes = nativeMethods.querySelectorAll.call(this.document, '*');
	        var length = nativeMethods.nodeListLengthGetter.call(childNodes);
	        for (var i = 0; i < length; i++) {
	            var childNode = childNodes[i];
	            delete childNode[INTERNAL_PROPS.processedContext];
	            removeListeningElement(childNode);
	        }
	    };
	    Sandbox.prototype.dispose = function () {
	        this.event.hover.dispose();
	        this.event.focusBlur.dispose();
	        dispose$1();
	        dispose();
	        urlResolver.dispose(this.document);
	        this.storageSandbox.dispose();
	        this._removeInternalProperties();
	    };
	    return Sandbox;
	}(SandboxBase));

	var SERVICE_MESSAGES_WAITING_INTERVAL = 50;
	var TransportLegacy = /** @class */ (function () {
	    function TransportLegacy() {
	        this._activeServiceMsgCount = 0;
	    }
	    TransportLegacy._getStoredMessages = function () {
	        var nativeLocalStorage = nativeMethods.winLocalStorageGetter.call(window);
	        var storedMessagesStr = nativeMethods.storageGetItem.call(nativeLocalStorage, settings.get().sessionId);
	        return storedMessagesStr ? parse$1(storedMessagesStr) : [];
	    };
	    TransportLegacy._storeMessage = function (msg) {
	        var storedMessages = TransportLegacy._getStoredMessages();
	        var nativeLocalStorage = nativeMethods.winLocalStorageGetter.call(window);
	        storedMessages.push(msg);
	        nativeMethods.storageSetItem.call(nativeLocalStorage, settings.get().sessionId, stringify(storedMessages));
	    };
	    TransportLegacy._removeMessageFromStore = function (cmd) {
	        var messages = TransportLegacy._getStoredMessages();
	        var nativeLocalStorage = nativeMethods.winLocalStorageGetter.call(window);
	        for (var i = 0; i < messages.length; i++) {
	            if (messages[i].cmd === cmd) {
	                messages.splice(i, 1);
	                break;
	            }
	        }
	        nativeMethods.storageSetItem.call(nativeLocalStorage, settings.get().sessionId, stringify(messages));
	    };
	    TransportLegacy.prototype.batchUpdate = function () {
	        var storedMessages = TransportLegacy._getStoredMessages();
	        if (!storedMessages.length)
	            return pinkie.resolve();
	        var tasks = [];
	        var nativeLocalStorage = nativeMethods.winLocalStorageGetter.call(window);
	        nativeMethods.storageRemoveItem.call(nativeLocalStorage, settings.get().sessionId);
	        for (var _i = 0, storedMessages_1 = storedMessages; _i < storedMessages_1.length; _i++) {
	            var storedMessage = storedMessages_1[_i];
	            tasks.push(this.queuedAsyncServiceMsg(storedMessage));
	        }
	        return pinkie.all(tasks);
	    };
	    TransportLegacy.prototype.waitForServiceMessagesCompleted = function (timeout) {
	        var _this = this;
	        return new pinkie(function (resolve) {
	            if (!_this._activeServiceMsgCount) {
	                resolve();
	                return;
	            }
	            var intervalId = null;
	            var timeoutId = nativeMethods.setTimeout.call(window, function () {
	                nativeMethods.clearInterval.call(window, intervalId);
	                resolve();
	            }, timeout);
	            intervalId = nativeMethods.setInterval.call(window, function () {
	                if (_this._activeServiceMsgCount)
	                    return;
	                nativeMethods.clearInterval.call(window, intervalId);
	                nativeMethods.clearTimeout.call(window, timeoutId);
	                resolve();
	            }, SERVICE_MESSAGES_WAITING_INTERVAL);
	        });
	    };
	    return TransportLegacy;
	}());

	var SET_INITIAL_WORKER_SETTINGS_CMD = 'hammerhead|transport|set-initial-worker-settings';
	var HANDLE_PORT_CMD = 'hammerhead|transport|handle-port';

	var GET_MESSAGE_PORT = 'hammerhead|command|get-message-port';
	var SET_MESSAGE_PORT = 'hammerhead|command|set-message-port';
	var Transport = /** @class */ (function (_super) {
	    __extends(Transport, _super);
	    function Transport() {
	        var _this = _super !== null && _super.apply(this, arguments) || this;
	        _this._transportWorker = null;
	        _this._idGenerator = new IntegerIdGenerator();
	        _this._messageCallbacks = new Map();
	        _this._queue = [];
	        _this._shouldAddReferer = Transport._shouldAddReferer();
	        return _this;
	    }
	    Transport.prototype.start = function (messageSandbox) {
	        var _this = this;
	        if (window === window.top) {
	            // @ts-ignore
	            this._transportWorker = new nativeMethods.Worker(settings.get().transportWorkerUrl, { name: 'Transport' });
	            this._transportWorker.postMessage({
	                cmd: SET_INITIAL_WORKER_SETTINGS_CMD,
	                sessionId: settings.get().sessionId,
	                serviceMsgUrl: settings.get().serviceMsgUrl
	            });
	            this._transportWorker.addEventListener('message', function (e) { return _this._onWorkerMessage(e); });
	            this._processQueue();
	        }
	        else
	            messageSandbox.sendServiceMsg({ cmd: GET_MESSAGE_PORT }, window.top);
	        messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (_a) {
	            var message = _a.message, source = _a.source, ports = _a.ports;
	            if (message.cmd === GET_MESSAGE_PORT) {
	                var channel = new nativeMethods.MessageChannel();
	                messageSandbox.sendServiceMsg({ cmd: SET_MESSAGE_PORT }, source, [channel.port1]);
	                _this._transportWorker.postMessage({ cmd: HANDLE_PORT_CMD }, [channel.port2]);
	            }
	            else if (message.cmd === SET_MESSAGE_PORT) {
	                _this._transportWorker = ports[0];
	                _this._transportWorker.onmessage = function (e) { return _this._onWorkerMessage(e); };
	                _this._processQueue();
	            }
	        });
	    };
	    Transport.prototype._processQueue = function () {
	        for (var _i = 0, _a = this._queue; _i < _a.length; _i++) {
	            var msgWrapper = _a[_i];
	            this._transportWorker.postMessage(msgWrapper);
	        }
	        this._queue.length = 0;
	    };
	    Transport._shouldAddReferer = function () {
	        var frameElement = getFrameElement(window);
	        return frameElement && isIframeWithoutSrc(frameElement);
	    };
	    Transport.prototype._onWorkerMessage = function (e) {
	        var _a = e.data, id = _a.id, result = _a.result;
	        if (!this._messageCallbacks.has(id))
	            return;
	        this._messageCallbacks.get(id)(result.err, result.data);
	        this._messageCallbacks.delete(id);
	    };
	    Transport.prototype.asyncServiceMsg = function (msg, queued) {
	        var _this = this;
	        if (queued === void 0) { queued = false; }
	        return new pinkie(function (resolve, reject) {
	            var id = _this._idGenerator.increment();
	            ++_this._activeServiceMsgCount;
	            if (_this._shouldAddReferer)
	                msg.referer = settings.get().referer;
	            _this._messageCallbacks.set(id, function (err, data) {
	                --_this._activeServiceMsgCount;
	                if (!err)
	                    resolve(data);
	                else {
	                    if (!msg.disableResending && (isWebKit || isFirefox)) {
	                        TransportLegacy._removeMessageFromStore(msg.cmd);
	                        TransportLegacy._storeMessage(msg);
	                        resolve();
	                    }
	                    if (msg.allowRejecting)
	                        reject(new Error(err));
	                }
	            });
	            if (_this._transportWorker)
	                _this._transportWorker.postMessage({ id: id, queued: queued, msg: msg });
	            else
	                _this._queue.push({ id: id, queued: queued, msg: msg });
	        });
	    };
	    Transport.prototype.queuedAsyncServiceMsg = function (msg) {
	        return this.asyncServiceMsg(msg, true);
	    };
	    return Transport;
	}(TransportLegacy));

	function nextTick () {
	    return new pinkie(function (resolve) { return nativeMethods.setTimeout.call(window, resolve, 0); });
	}

	var PageNavigationWatch = /** @class */ (function (_super) {
	    __extends(PageNavigationWatch, _super);
	    function PageNavigationWatch(_eventSandbox, _codeInstrumentation, _elementSandbox) {
	        var _this = _super.call(this) || this;
	        _this._eventSandbox = _eventSandbox;
	        _this._codeInstrumentation = _codeInstrumentation;
	        _this._elementSandbox = _elementSandbox;
	        _this.PAGE_NAVIGATION_TRIGGERED_EVENT = 'hammerhead|event|page-navigation-triggered';
	        _this._lastLocationValue = window.location.toString();
	        return _this;
	    }
	    PageNavigationWatch.prototype._formWatch = function (elementSandbox, eventSandbox) {
	        var onFormSubmit = function (form) {
	            var targetWindow = PageNavigationWatch._getTargetWindow(form);
	            PageNavigationWatch._onNavigationTriggeredInWindow(targetWindow, nativeMethods.formActionGetter.call(form));
	        };
	        // NOTE: fires when form.submit() is called
	        elementSandbox.on(elementSandbox.BEFORE_FORM_SUBMIT_EVENT, function (e) { return onFormSubmit(e.form); });
	        // NOTE: fires when the form is submitted by clicking the submit button
	        eventSandbox.listeners.initElementListening(window, ['submit']);
	        eventSandbox.listeners.addInternalEventBeforeListener(window, ['submit'], function (e) {
	            var prevented = false;
	            if (!isFormElement(e.target))
	                return;
	            var onPreventDefault = function (preventedEvent) {
	                prevented = prevented || preventedEvent === e;
	            };
	            eventSandbox.on(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
	            nextTick()
	                .then(function () {
	                eventSandbox.off(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
	                // NOTE: the defaultPrevented flag is saved between event raises in all browsers
	                // except IE. In IE, it is reset to false before the next handler is executed.
	                if (!e.defaultPrevented && !prevented)
	                    onFormSubmit(e.target);
	            });
	        });
	    };
	    PageNavigationWatch._getTargetWindow = function (el) {
	        var target = nativeMethods.getAttribute.call(el, DomProcessor.getStoredAttrName('target')) ||
	            nativeMethods.getAttribute.call(el, 'target') ||
	            '_self';
	        switch (target) {
	            case '_top':
	                return window.top;
	            case '_parent':
	                return window.parent;
	            case '_self':
	                return window;
	            default:
	                return findByName(target);
	        }
	    };
	    PageNavigationWatch.prototype._linkWatch = function (eventSandbox) {
	        eventSandbox.listeners.initElementListening(window, ['click']);
	        eventSandbox.listeners.addInternalEventBeforeListener(window, ['click'], function (e) {
	            var link = isAnchorElement(e.target) ? e.target : closest(e.target, 'a');
	            if (link && !isShadowUIElement(link)) {
	                var prevented_1 = false;
	                var targetWindow_1 = PageNavigationWatch._getTargetWindow(link);
	                var href_1 = nativeMethods.anchorHrefGetter.call(link);
	                var onPreventDefault_1 = function (preventedEvent) {
	                    prevented_1 = prevented_1 || preventedEvent === e;
	                };
	                eventSandbox.on(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault_1);
	                nextTick()
	                    .then(function () {
	                    eventSandbox.off(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault_1);
	                    // NOTE: the defaultPrevented flag is saved between event raises in all browsers
	                    // except IE. In IE, it is reset to false before the next handler is executed.
	                    if (!e.defaultPrevented && !prevented_1)
	                        PageNavigationWatch._onNavigationTriggeredInWindow(targetWindow_1, href_1);
	                });
	            }
	        });
	    };
	    PageNavigationWatch.prototype._locationWatch = function (codeInstrumentation) {
	        var _this = this;
	        var locationAccessorsInstrumentation = codeInstrumentation._locationAccessorsInstrumentation;
	        var locationChangedHandler = function (newLocation) { return _this.onNavigationTriggered(newLocation); };
	        locationAccessorsInstrumentation.on(locationAccessorsInstrumentation.LOCATION_CHANGED_EVENT, locationChangedHandler);
	    };
	    PageNavigationWatch._onNavigationTriggeredInWindow = function (win, url) {
	        try {
	            win[INTERNAL_PROPS.hammerhead].pageNavigationWatch.onNavigationTriggered(url);
	        }
	        // eslint-disable-next-line no-empty
	        catch (e) {
	        }
	    };
	    PageNavigationWatch.prototype.onNavigationTriggered = function (url) {
	        var currentLocation = this._lastLocationValue;
	        this._lastLocationValue = window.location.toString();
	        if (url !== currentLocation && (url.charAt(0) === '#' || isChangedOnlyHash(currentLocation, url)) ||
	            DomProcessor.isJsProtocol(url))
	            return;
	        var parsedProxyUrl = parseProxyUrl$1(url);
	        if (!parsedProxyUrl)
	            return;
	        this.emit(this.PAGE_NAVIGATION_TRIGGERED_EVENT, parsedProxyUrl.destUrl);
	    };
	    PageNavigationWatch.prototype.start = function () {
	        this._locationWatch(this._codeInstrumentation);
	        this._linkWatch(this._eventSandbox);
	        this._formWatch(this._elementSandbox, this._eventSandbox);
	    };
	    return PageNavigationWatch;
	}(EventEmitter));

	var SCRIPT_SELECTOR = "." + SHADOW_UI_CLASS_NAME.script;
	function removeInjectedScript () {
	    var injectedScript = nativeMethods.querySelector.call(document, SCRIPT_SELECTOR);
	    if (injectedScript)
	        removeElement(injectedScript);
	}

	var Hammerhead = /** @class */ (function () {
	    function Hammerhead() {
	        var _this = this;
	        this.win = null;
	        this.transport = new Transport();
	        this.sandbox = new Sandbox(this.transport);
	        this.pageNavigationWatch = new PageNavigationWatch(this.sandbox.event, this.sandbox.codeInstrumentation, this.sandbox.node.element);
	        this.EVENTS = {
	            beforeFormSubmit: this.sandbox.node.element.BEFORE_FORM_SUBMIT_EVENT,
	            beforeBeforeUnload: this.sandbox.event.unload.BEFORE_BEFORE_UNLOAD_EVENT,
	            beforeUnload: this.sandbox.event.unload.BEFORE_UNLOAD_EVENT,
	            unload: this.sandbox.event.unload.UNLOAD_EVENT,
	            bodyCreated: this.sandbox.node.mutation.BODY_CREATED_EVENT,
	            documentCleaned: this.sandbox.node.mutation.DOCUMENT_CLEANED_EVENT,
	            uncaughtJsError: this.sandbox.node.win.UNCAUGHT_JS_ERROR_EVENT,
	            unhandledRejection: this.sandbox.node.win.UNHANDLED_REJECTION_EVENT,
	            startFileUploading: this.sandbox.upload.START_FILE_UPLOADING_EVENT,
	            endFileUploading: this.sandbox.upload.END_FILE_UPLOADING_EVENT,
	            evalIframeScript: this.sandbox.iframe.EVAL_EXTERNAL_SCRIPT_EVENT,
	            xhrCompleted: this.sandbox.xhr.XHR_COMPLETED_EVENT,
	            xhrError: this.sandbox.xhr.XHR_ERROR_EVENT,
	            beforeXhrSend: this.sandbox.xhr.BEFORE_XHR_SEND_EVENT,
	            fetchSent: this.sandbox.fetch.FETCH_REQUEST_SENT_EVENT,
	            pageNavigationTriggered: this.pageNavigationWatch.PAGE_NAVIGATION_TRIGGERED_EVENT,
	            scriptElementAdded: this.sandbox.node.element.SCRIPT_ELEMENT_ADDED_EVENT,
	            consoleMethCalled: this.sandbox.console.CONSOLE_METH_CALLED_EVENT,
	            windowOpened: this.sandbox.childWindow.WINDOW_OPENED_EVENT
	        };
	        this.PROCESSING_INSTRUCTIONS = {
	            dom: {
	                script: SCRIPT_PROCESSING_INSTRUCTIONS,
	                internal_attributes: INTERNAL_ATTRIBUTES,
	                internal_props: INTERNAL_PROPS
	            }
	        };
	        this.SHADOW_UI_CLASS_NAME = SHADOW_UI_CLASS_NAME;
	        this.SESSION_COMMAND = ServiceCommands;
	        this.EventEmitter = EventEmitter;
	        // Methods
	        this.doUpload = function (input, filePaths) { return _this.sandbox.upload.doUpload(input, filePaths); };
	        this.createNativeXHR = XhrSandbox.createNativeXHR;
	        this.processScript = processScript;
	        // Modules
	        this.Promise = pinkie;
	        this.json = json;
	        this.nativeMethods = this.sandbox.nativeMethods;
	        this.shadowUI = this.sandbox.shadowUI;
	        this.storages = this.sandbox.storageSandbox;
	        this.eventSandbox = {
	            listeners: this.sandbox.event.listeners,
	            hover: this.sandbox.event.hover,
	            focusBlur: this.sandbox.event.focusBlur,
	            elementEditingWatcher: this.sandbox.event.elementEditingWatcher,
	            eventSimulator: this.sandbox.event.eventSimulator,
	            selection: this.sandbox.event.selection,
	            message: this.sandbox.event.message,
	            timers: this.sandbox.event.timers,
	            DataTransfer: this.sandbox.event.DataTransfer,
	            DragDataStore: this.sandbox.event.DragDataStore
	        };
	        var processingUtils = {
	            script: scriptProcessingUtils,
	            header: headerProcessingUtils,
	            instrumentation: instrumentationProcessingUtils
	        };
	        this.utils = {
	            browser: browserUtils,
	            dom: domUtils,
	            event: eventUtils,
	            position: positionUtils,
	            style: styleUtils,
	            types: typeUtils,
	            trim: trim,
	            extend: extend,
	            html: htmlUtils,
	            url: urlUtils,
	            featureDetection: featureDetection,
	            destLocation: destLocationUtils,
	            overriding: overridingUtils,
	            cookie: cookieUtils,
	            getMimeType: getMimeType,
	            urlResolver: urlResolver,
	            processing: processingUtils,
	            removeInjectedScript: removeInjectedScript
	        };
	        this.sharedUtils = {
	            cookie: sharedCookieUtils,
	            url: sharedUrlUtils,
	            headers: sharedHeadersUtils,
	            stackProcessing: sharedStackProcessingUtils,
	            selfRemovingScripts: sharedSelfRemovingScripts
	        };
	        this.settings = settings;
	        this.sandboxes = {
	            XhrSandbox: XhrSandbox,
	            StyleSandbox: StyleSandbox,
	            ShadowUISandbox: ShadowUI,
	            ElectronSandbox: ElectronSandbox,
	            UploadSandbox: UploadSandbox,
	            ChildWindowSandbox: ChildWindowSandbox
	        };
	        this.sandboxUtils = {
	            hiddenInfo: hiddenInfoUtils,
	            listeningContext: listeningContextUtils,
	            backup: sandboxBackupUtils,
	            domMutationTracker: domMutationTracker,
	            defaultTarget: defaultTarget,
	            UploadInfoManager: UploadInfoManager,
	            FileListWrapper: FileListWrapper,
	            EventListeners: Listeners,
	            StorageWrapper: StorageWrapper,
	            CodeInstrumentation: CodeInstrumentation,
	            LocationInstrumentation: LocationAccessorsInstrumentation,
	            LocationWrapper: LocationWrapper
	        };
	        this.processors = {
	            styleProcessor: styleProcessor,
	            domProcessor: domProcessor,
	            DomProcessor: DomProcessor
	        };
	    }
	    Hammerhead.prototype._getEventOwner = function (evtName) {
	        switch (evtName) {
	            case this.EVENTS.pageNavigationTriggered:
	                return this.pageNavigationWatch;
	            case this.EVENTS.beforeUnload:
	            case this.EVENTS.beforeBeforeUnload:
	            case this.EVENTS.unload:
	                return this.sandbox.event.unload;
	            case this.EVENTS.bodyCreated:
	            case this.EVENTS.documentCleaned:
	                return this.sandbox.node.mutation;
	            case this.EVENTS.uncaughtJsError:
	            case this.EVENTS.unhandledRejection:
	                return this.sandbox.node.win;
	            case this.EVENTS.startFileUploading:
	            case this.EVENTS.endFileUploading:
	                return this.sandbox.upload;
	            case this.EVENTS.evalIframeScript:
	                return this.sandbox.iframe;
	            case this.EVENTS.xhrCompleted:
	            case this.EVENTS.xhrError:
	            case this.EVENTS.beforeXhrSend:
	                return this.sandbox.xhr;
	            case this.EVENTS.beforeFormSubmit:
	            case this.EVENTS.scriptElementAdded:
	                return this.sandbox.node.element;
	            case this.EVENTS.fetchSent:
	                return this.sandbox.fetch;
	            case this.EVENTS.consoleMethCalled:
	                return this.sandbox.console;
	            case this.EVENTS.windowOpened:
	                return this.sandbox.childWindow;
	            default:
	                return null;
	        }
	    };
	    Hammerhead._cleanLocalStorageServiceData = function (sessionId, window) {
	        var nativeLocalStorage = nativeMethods.winLocalStorageGetter.call(window);
	        nativeMethods.storageRemoveItem.call(nativeLocalStorage, sessionId);
	    };
	    Hammerhead.prototype.on = function (evtName, handler) {
	        var eventOwner = this._getEventOwner(evtName);
	        if (eventOwner)
	            eventOwner.on(evtName, handler);
	    };
	    Hammerhead.prototype.off = function (evtName, handler) {
	        var eventOwner = this._getEventOwner(evtName);
	        if (eventOwner)
	            eventOwner.off(evtName, handler);
	    };
	    Hammerhead.prototype.navigateTo = function (url, forceReload) {
	        var _this = this;
	        var navigationUrl = getNavigationUrl(url, this.win);
	        if (!navigationUrl)
	            return;
	        // @ts-ignore
	        this.win.location = navigationUrl;
	        if (forceReload) {
	            this.sandbox.node.win.on(this.sandbox.node.win.HASH_CHANGE_EVENT, function () {
	                _this.win.location.reload(true);
	            });
	        }
	    };
	    Hammerhead.prototype.start = function (initSettings, win) {
	        this.win = win || window;
	        settings.set(initSettings);
	        if (initSettings.isFirstPageLoad)
	            Hammerhead._cleanLocalStorageServiceData(initSettings.sessionId, this.win);
	        domProcessor.forceProxySrcForImage = initSettings.forceProxySrcForImage;
	        domProcessor.allowMultipleWindows = initSettings.allowMultipleWindows;
	        this.transport.start(this.eventSandbox.message);
	        this.sandbox.attach(this.win);
	        this.pageNavigationWatch.start();
	    };
	    return Hammerhead;
	}());
	var hammerhead = new Hammerhead();
	// NOTE: The 'load' event is raised after calling document.close for a same-domain iframe
	// So, we need to define the '%hammerhead%' variable as 'configurable' so that it can be redefined.
	nativeMethods.objectDefineProperty(window, INTERNAL_PROPS.hammerhead, {
	    value: hammerhead,
	    configurable: true
	});

	return hammerhead;

}());

})();

window['%hammerhead%'].utils.removeInjectedScript();
